/************************************************************************
 *									*
 *	Denna modul hanterar all in och utmatning till och fraan	*
 *	bildskaermen							*
 *	Rutinen scrini maaste anropas foere nagonting annat		*
 *									*
 *	rev.hist.							*
 *	840903	AN	orginal vers.					*
 *	850220	AN	rev. 1						*
 *									*
 ************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/tty.h>

#include "ble.h"
#include "edit.h"

/*	diverse konstanter: */

#define	STDIN	0	/* standard input file descriptor */
#define	STDOUT	1	/* standard output file descriptor */

#define	NUMESC	((sizeof esctab) / (sizeof(struct tesc))) /* antal escapesekvenser */

/*	terminal beroende variabler */

#define	tcbsiz	128	/* laengd paa terminal control string buffert */

typedef	struct {
	int	co;	/* antal kolumner */
	int	li;	/* antal rader */
	char	*bs;	/* backspace string */
	char	*up;	/* cursor up */
	char	*cl;	/* clear screen */
	char	*cm;	/* cursor motion */
	char	*ce;	/* clear to end of line */
	char	*in;	/* terminal inittialization file name */
	char	*is;	/* terminal inittialization string */
	char	*pc;	/* pad character */
	char	*NR;	/* inverse video */
	char	*NM;	/* normal video */
	} ttyrec;

static struct tesc {	/* escapesekvenser och deras motsvarande kommandon */
	char	*sekv;	/* initieras i scrini */
	int	kommando;
	} esctab [] = {
	{0,k_pf1},	/* function key 1 */
	{0,k_pf2},	/* function key 2 */
	{0,k_pf3},	/* function key 3 */
	{0,k_pf4},	/* function key 4 */
	{0,k_prevli},	/* up arrow key */
	{0,k_nextli},	/* down arrow key */
	{0,k_backch},	/* left arrow key */
	{0,k_forwch},	/* right arrow key */
	{0,k_top},	/* home key */
	};

	/* variabeldeklarationer */

static	struct termio	ttystatus;	/* terminalstatus */
static	struct termio	savestatus;	/*      -"-       */
static	ttyrec	term;			/* terminalbeskrivnings-recordet */
static	int	currow,curcol;		/* current rad och kolumn */
static	char	tstrings[tcbsiz];	/* terminal control string buffert */
static	int	videomode;		/* normal = 0, inverse = 1 */
char	PC,*BC,*UP;		/* help variables for termlib routines */
int	ospeed;			/* terminal speed for padding delays */

/* exerna rutiner */

char	*ttyname();	/* get terminal name */
char	*getenv();	/* get environment variable */
char	*tgetstr();	/* get a terminal dependent char string */
char	*tgoto();	/* get cursorpositioneringsstraeng */
	putch();	/* forward */
/**/
/******************************************************************
initierar terminalhanteraren
******************************************************************/
scrini()
{
	char	*tname,ch,tbuff[1024];
	int	fd;

	if ((tname=ttyname(STDIN)) == NULL)
		return(e_noterm);

	/* read the terminal data base */

	if ((tname=getenv("TERM")) == NULL)
		return(e_notenv);

	if (tgetent(tbuff,tname) != 1)
		return(e_notety);

	tname = tstrings;
	term.co = tgetnum ("co");	/* antal kolumner */
	term.li = tgetnum ("li");	/* antal rader */
	if (tgetflag ("bs"))	/* backspace */
		term.bs="\b";
	else
		term.bs = tgetstr ("bc",&tname);
	term.up = tgetstr ("up",&tname);	/* cursor up */
	term.cl = tgetstr ("cl",&tname);	/* clear screen */
	term.cm = tgetstr ("cm",&tname);	/* cursor motion */
	term.ce = tgetstr ("ce",&tname);	/* clear to end of line */
	term.in = tgetstr ("if",&tname);	/* init file name */
	term.is = tgetstr ("is",&tname);	/* init string */
	term.pc = tgetstr ("pc",&tname);	/* pad character */
	term.NR = tgetstr ("NR",&tname);	/* inverse video */
	term.NM = tgetstr ("NM",&tname);	/* normal video */

	esctab[0].sekv = tgetstr ("k1",&tname);	/* function key 1 */
	esctab[1].sekv = tgetstr ("k2",&tname);	/* function key 2 */
	esctab[2].sekv = tgetstr ("k3",&tname);	/* function key 3 */
	esctab[3].sekv = tgetstr ("k4",&tname);	/* function key 4 */
	esctab[4].sekv = tgetstr ("ku",&tname);	/* up arrow key */
	esctab[5].sekv = tgetstr ("kd",&tname);	/* down arrow key */
	esctab[6].sekv = tgetstr ("kl",&tname);	/* left arrow key */
	esctab[7].sekv = tgetstr ("kr",&tname);	/* right arrow key */
	esctab[8].sekv = tgetstr ("kh",&tname);	/* home key */

	PC = term.pc ? *(term.pc) : '\0';
	BC = term.bs;
	UP = term.up;

	/* check for buffer owerflow */
	if (tname >= &tstrings[tcbsiz])
		return(e_tcbovf);

	/* check screen size */
	if (term.co<0 || term.co>arkvidd+1 || term.li<0 || term.li>arkhojd+1)
		return(e_scrsiz);

	/* kontrollera att CL och CM finns */
	if (term.cl == NULL)
		return(e_nocl);
	if (term.cm == NULL)
		return(e_nocm);


	/* saett upp terminalen i raett mode */

	if (ioctl(STDIN,TCGETA,&savestatus) < 0)
		return(e_ioctl);
	ioctl(STDIN,TCGETA,&ttystatus);
	ospeed = ttystatus.c_cflag & CBAUD;
	ttystatus.c_iflag = 0;
	ttystatus.c_oflag = 0;
	ttystatus.c_lflag = 0;
	ttystatus.c_tcc[VMIN] = 1;	/* ett tecken aat gaangen */
	ioctl(STDIN,TCSETAF,&ttystatus);

	/* initiera terminalen */

	if (term.is != NULL)
		tputs(term.is,1,putch);
	if (term.in != NULL)
		if ((fd=open(term.in,0)) > 0) {
			while (read(fd,&ch,1)>0)
				putch(ch);
			close(fd);
			}

	currow=0;
	curcol=0;

	videomode = 0;		/* normal = 0, inverse = 1 */

	clscr();

	return(e_ok);

}
/******************************************************************
release terminalen
******************************************************************/
scrrel()
{
	cpos(23,0);
	ioctl(STDIN,TCSETAF,&savestatus);
}
/******************************************************************
skriv ett tecken utan att uppdatera curcol
******************************************************************/
putch(tkn)
char	tkn;
{
	write(STDOUT,&tkn,1);
}
/******************************************************************
positionera cursor
******************************************************************/
cpos(row,col)
{
	if ( row != currow || col != curcol ) {
		tputs(tgoto(term.cm,col,row),1,putch);
		currow = row;
		curcol = col;
	}
}
/******************************************************************
skriv en straeng
******************************************************************/
outs(string)
char	*string;
{
	while (*string)
		outc(*string++);
}
/******************************************************************
skriv ett tecken
******************************************************************/
outc(tkn)
char	tkn;
{
	if (curcol < arkvidd) {
		write(STDOUT,&tkn,1);
		++curcol;
	}
}
/******************************************************************
posistionera och skriv en straeng
******************************************************************/
cpouts(row,col,string)
char	*string;
{
	cpos(row,col);
	outs(string);
}
/******************************************************************
posistionera och skriv ett tecken
******************************************************************/
cpoutc(row,col,tkn)
char	tkn;
{
	cpos(row,col);
	outc(tkn);
}
/******************************************************************
laes tecken fraan keyboard och oeversaett esc-sekvenser
till kontrolltecken enl filen termcap
******************************************************************/
char ink()
{

	char	ch;
	char	string[10];
	int	escseq;
	int	halvtraeff;
	int	len;
	int	i;

	read(STDIN,&ch,1);
	ch &= 0177;
	if (ch == delete)
		ch = k_backsp;
	if (ch == '\033') {
		escseq = TRUE;
		string[0] = '\033';
		len = 1;
		while (escseq) {
			read(STDIN,&ch,1);
			ch &= 0177;
			string[len] = ch;
			if (len++ >= 10)
				escseq = FALSE;
			halvtraeff = FALSE;

			for ( i = 0 ; i < NUMESC ; ++i ) {
				if ( strncmp(string,esctab[i].sekv,len) == 0 ) {
					halvtraeff = TRUE;
					if ( strlen(esctab[i].sekv) == len ) {
						ch = esctab[i].kommando;
						escseq = FALSE;
					}
				}
			}

			if (halvtraeff == FALSE)
				escseq = FALSE;

			len = len; /* dummystatement to satisfy compiler */

		}
	}

	return(ch);

}
/******************************************************************
positionera cursor och laes ett kommando
******************************************************************/
char cpink(row,col)
{
	cpos(row,col);
	return(ink());
}
/******************************************************************
clear screen
******************************************************************/
clscr()
{
	tputs(term.cl,term.li,putch);
}
/******************************************************************
clear line
******************************************************************/
clline(row)
{
	int	i;
	int	vmold;	/* spara videomode */

	cpos(row,0);
	if (term.ce)
		tputs(term.ce,1,putch);
	else {
		vmold = videomode;
		if ( vmold == 1)
			norvid();
		for ( i = 0 ; i < arkvidd ; ++i )
			outc(' ');
		if ( vmold == 1)
			invvid();
		}
}
/******************************************************************
inverse video
******************************************************************/
invvid()
{
	if (videomode == 0) {
		videomode = 1;
		tputs(term.NR,1,putch);
	}
}
/******************************************************************
normal video
******************************************************************/
norvid()
{
	if (videomode == 1) {
		videomode = 0;
		tputs(term.NM,1,putch);
	}
}
