{$nomain}
{***************************************************************}
{	This module contains routines performing the batch	}
{	commands.						}
{***************************************************************}

%Include 'global.h';
%Include 'turbodef.h';
%Include 'main.h';
%Include 'screen.h';
%Include 'kbd.h';
%Include 'opipret.h';
%Include 'batch.h';

VAR	{$own='batch'}

    TimeoutEnd	: real;

{***************************************************************}
{	BatchInit:						}
{	Initiation of the batch test.				}
{***************************************************************}

Procedure BatchInit;				{ Global }

BEGIN	{ BatchInit }

    BatchFileName := 'BATCH.MTS';
    BatchList := NIL;

END;	{ BatchInit }

{***************************************************************}
{	CF_Batch_p:						}
{	The BATCH command. 					}
{***************************************************************}

Procedure CF_Batch_p;				{ Global }

BEGIN	{ CF_Batch_p }

    WriteWindow ( BatchIdle );
    Mode := BatchIdle;
    DisplayFKeys;
    Menu := BatchMenu;
    Display_BatchParms;

END;	{ CF_Batch_p }

{***************************************************************}
{	CF_BatNam_p:						}
{	The BatNam command. 					}
{***************************************************************}

Procedure CF_BatNam_p;				{ Global }

BEGIN	{ CF_BatNam_p }

    CposWrite ( 20, 35, 'cf_BatNam' );

END;	{ CF_BatNam_p }

{***************************************************************}
{	CF_Start_p:						}
{	The Start command. 					}
{***************************************************************}

Procedure CF_Start_p;				{ Global }

BEGIN	{ CF_Start_p }

    CposWrite ( 20, 35, 'cf_Start' );

END;	{ CF_Start_p }

{***************************************************************}
{	CF_Wait_p:						}
{	The Wait command. 					}
{***************************************************************}

Procedure CF_Wait_p;				{ Global }

BEGIN	{ CF_Wait_p }

    CposWrite ( 20, 35, 'cf_Wait' );

END;	{ CF_Wait_p }

{***************************************************************}
{	CF_Go_p:						}
{	The Go command. 					}
{***************************************************************}

Procedure CF_Go_p;				{ Global }

BEGIN	{ CF_Go_p }

    CposWrite ( 20, 35, 'cf_Go' );

END;	{ CF_Go_p }

{***************************************************************}
{	CF_Step_p:						}
{	The Step command. 					}
{***************************************************************}

Procedure CF_Step_p;				{ Global }

BEGIN	{ CF_Step_p }

    CposWrite ( 20, 35, 'cf_Step' );

END;	{ CF_Step_p }

{***************************************************************}
{	CF_BatchStop_p:						}
{	The Stop command in batch mode.				}
{***************************************************************}

Procedure CF_BatchStop_p;			{ Global }

BEGIN	{ CF_BatchStop_p }

    CposWrite ( 20, 35, 'cf_BatchStop' );

END;	{ CF_BatchStop_p }

{***************************************************************}
{	BatchLoop:						}
{***************************************************************}

Procedure BatchLoop;				{ Global }

VAR
    BatLineType	: BatLineTyp_t;		{ Batch command or command to remote }
    AnswersExp	: int2;			{ Number of answers expected }
    TimeoutEnd	: int4;			{ All answers should have come now }
    ChFetched	: boolean;		{ Was there a character fetched? }
    Keytype	: Keytype_t;		{ What's in ch }
    ch		: char;			{ The character }


{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{	ReceiveMsg:						}
{	Checks that the number of answers expected is received.	}
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

Procedure ReceiveMsg;

VAR
    Present_time	: real;
    CompleteMsg		: boolean;
    MsgType		: MsgType_t;		{ Type of message received }
    CommErr		: boolean;		{ Communication error }

BEGIN	{ ReceiveMsg }

    Scan_Comm( CompleteMsg, MsgType, CommErr );	{ Scan communication }
    IF (CompleteMsg OR CommErr) AND (Msgtype = AnswRec) THEN BEGIN
	AnswersExp := AnswersExp - 1;
    END; { IF }

    Present_time := time_a1;			{ Check if timeout }
    IF Present_time < SendTime THEN BEGIN
	Present_time := Present_time + 86400;	{ 24 hours in seconds }
    END; { IF }
    IF (Present_time > TimeoutEnd) AND (AnswersExp > 0) THEN BEGIN
    	CommErrHandler( Timeout );		{ Display and log timeout }
	AnswersExp := 0;
    END; { IF }

END;	{ ReceiveMsg }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{	ExecOneLine:						}
{	Executes a line in the batch file.			}
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

Procedure ExecOneLine;

VAR
    SendTime	: real;

BEGIN	{ ExecOneLine }

    CASE BatLineType OF
    	RemoteCmd : BEGIN
	    SendOneLine( RemoteLine );		{ Send one line to remote }
	    SendTime := Time_a1;
	    TimeoutEnd := SendTime + ( TimeoutTime * AnswersExp );
	END; { RemoteCmd }
	BatCmd	  : BEGIN
	    CASE BatCmd OF
	    	For	: Do_For;
		Next	: Do_Next;
		Pause	: Make_Pause;
		Echo	: Echo;
		Log	: Log;
		Stop	: Stop;
	    END; { CASE }
	END; { BatCmd }
    END; { CASE }
    BatLineType := n{sta radtyp;

END;	{ ExecOneLine }

{***************************************************************}

BEGIN	{ BatchLoop }

    REPEAT
	Display_TimeLeft;
	Scan_Kbd( ChFetched, Keytype, ch );
	IF ChFetched THEN BEGIN
	    Get_OpCmd( ChFetched, Keytype, ch );
	END; { IF }
	IF Mode = BatchSend THEN BEGIN
	    Receive_RemoteMsg;		{ Fetch messages from remote }
	    IF BatLineType = BatCmd
	    OR ( (BatLineType = RemoteCmd) AND (AnswersExp = 0) ) THEN BEGIN
	    	Exec_One_Line;		{ Execute one line in the batch file }
	    END; { IF }
	END; { IF }
    UNTIL Menu <> BatchMenu;

END;	{ BatchLoop }
