{$nomain}
{***************************************************************}
{	This module contains general communication routines.	}
{***************************************************************}

%Include 'global.h';
%Include 'turbodef.h';
%Include 'error.h';
%Include 'screen.h';
%Include 'setup.h';
%Include 'sthform.h';
%Include 'gpib.h';
%Include 'rs232.h';
%include 'l1string.h';
%Include 'log.h';
%Include 'comm.h';
%Include 'comm.dat';

{***************************************************************}
{	IniComm:						}
{	Initiates communication.				}
{***************************************************************}

Procedure IniComm;				{ Global }

BEGIN	{ IniComm }

    InbufLen := 0;		{ Clear the input buffer }
    OutbufLen := 0;		{ Clear the output buffer }

END;	{ IniComm }

{***************************************************************}
{	Show_CommStr:						}
{	Shows a string sent or received.			}
{***************************************************************}

Procedure Show_CommStr;				{ Global }

VAR
    Row		: byte;		{ Row in show buffer }
    ExtShowStr	: lstr132;	{ Extended string for printer and file }

BEGIN	{ Show_CommStr }

    FOR Row := 1 TO NumShowRows DO BEGIN
    	ScrollWriteLn( ShowBuf[Row] ); 
	IF (FileLog_On) OR (PrnLog_On) THEN BEGIN
	    ExtShowStr := ShowBuf[Row];
	    AddToExtShowStr;

	    IF FileLog_On THEN FileLog( ExtShowStr );
	    IF PrnLog_On THEN PrnLog( ExtShowStr );
	END; { IF }
    END; { FOR }

END;	{ Show_CommStr }

{*******************************************************}
{	Send_Cmd.					}
{	Sends commands to remote.			}
{*******************************************************}

Procedure Send_Cmd;

BEGIN	{ Send_Cmd }

    Pack_Cmd( CmdLen );
    CASE CommInFace OF
    	Gpib	: GpibSend;
	RS232	: RS232Send;
    END;
    Format_Cmd;
    Show_CommStr;

END;	{ Send_Cmd }

{***************************************************************}
{	Scan_Comm:						}
{	Reads characters until no characters are available	}
{	or a complete message is received.			}
{***************************************************************}

Procedure Scan_Comm;				{ Global }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{	CharsAvail:						}
{	Check if there is a character available.		}
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

Function CharsAvail	: boolean;

BEGIN	{ CharsAvail }

    CASE CommInFace OF
    	Gpib	: CharsAvail := GpibCharsAvail;
	RS232	: CharsAvail := RS232CharsAvail;
    END;

END;	{ CharsAvail }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{	Get_CommChars:						}
{	Reads character received.				}
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

Procedure Get_CommChars( VAR CompleteMsg : boolean );

BEGIN	{ Get_CommChars }

    CASE CommInFace OF
    	Gpib	: GpibRead;
	RS232	: RS232Read( CompleteMsg );
    END;

END;	{ Get_CommChars }

{***************************************************************}

BEGIN	{ Scan_Comm }

    CompleteMsg := false;
    WHILE (CharsAvail) AND (NOT CompleteMsg) DO BEGIN
    	Get_CommChars( CompleteMsg );		{ Fetch characters }
	IF InBufLen > MaxMsgLen THEN BEGIN	{ No end of message? }
	    CommErrHandler( MsgTooLong );
	    InBufLen := MaxMsgLen - InBufLen;	{ Truncate message }
	    CompleteMsg := true;		{ Consider message complete }
	END; { IF }
	IF CompleteMsg THEN BEGIN
	    Check_Msg( MsgType );
	    Format_Msg;
	    Show_CommStr;
	    InBufLen := 0;			{ Clear input buffer }
	END; { IF }
    END; { WHILE }

END;	{ Scan_Comm }
