{$nomain}
{***************************************************************}
{	This module contains the editor routines.		}
{***************************************************************}

%Include 'global.h';
%Include 'turbodef.h';
%Include 'l1string.h';
%Include 'main.h';
%Include 'screen.h';
%Include 'kbd.h';
%Include 'opipret.h';
%Include 'error.h';
%Include 'edit.h';

CONST
    FirstRow	= 1;
    LastRow	= 20;
    FirstCol	= 1;
    LastCol	= 80;
    
TYPE
    EBufPtr_t = RECORD		{ Current position in the editbuffer }
	Row	: byte;
	Col	: byte; END;

VAR	{$own="edit"}

    EBuf	: array [FirstRow..LastRow] of lstr80; { The edit buffer }
    EBufPtr	: EBufPtr_t;		{ Current position in the editbuffer }
    InsertMode	: boolean;
    Buf_saved	: boolean;		{ Is the buffer saved? }
    Save_buf	: boolean;		{ Should the buffer be saved? }
    DelLine	: lstr80;		{ The deleted line }

{***************************************************************}
{	Init_Edit:						}
{	Initiation of the editor.				}
{***************************************************************}

Procedure Init_Edit;				{ Global }

VAR
    RowI	: byte;

BEGIN	{ Init_Edit }

    OpInput_Exp := Cmd;	{ No operator command parameters expected }
    FOR RowI := FirstRow TO LastRow DO
	EBuf[EBufPtr.Row] := '';	{ No characters yet }
    EBufPtr.Row := FirstRow;		{ First row in buffer }
    EBufPtr.Col := FirstCol;		{ First column in buffer }
    InsertMode := false;
    DelLine := '';			{ No line deleted yet }
    Buf_saved := false;
    Save_buf := true;

END;	{ Init_Edit }

{***************************************************************}
{	Display_EBuf:						}
{	Displays the editbuffer.				}
{***************************************************************}

Procedure Display_EBuf;

VAR
    RowI		: byte;

BEGIN	{ Display_EBuf }

    GotoXY( FirstCol, FirstRow + 2);
    FOR RowI := FirstRow TO LastRow DO
        WriteLn( EBuf[RowI] );
    Break( Output );
    GotoXY( EBufPtr.Col, EBufPtr.Row + 2);

END;	{ Display_EBuf }

{***************************************************************}
{	CF_Edit_p:						}
{	The EDIT command. 					}
{***************************************************************}

Procedure CF_Edit_p;				{ Global }

BEGIN	{ CF_Edit_p }

    WriteWindow( Edit );
    Mode := Edit;
    DisplayFKeys;
    Menu := EditMenu;
    Display_EBuf;				{ Display editbuffer }

END;	{ CF_Edit_p }

{***************************************************************}
{	F_CursUp_p:						}
{	The up arrow in edit mode.				}
{***************************************************************}

Procedure F_CursUp_p;				{ Global }

BEGIN	{ F_CursUp_p }

    IF EBufPtr.Row > FirstRow THEN BEGIN		{ Not first row? }
        EBufPtr.Row := EBufPtr.Row - 1;				{ Change row }
	IF Length( EBuf[EBufPtr.Row] ) + 1 < EBufPtr.Col THEN { NL shorter? }
            EBufPtr.Col := Length( EBuf[EBufPtr.Row] ) + 1;	{ Pos at eol }
        GotoXY( EBufPtr.Col, EBufPtr.Row + 2);			{ Pos cursor }
    END; { IF }

END;	{ F_CursUp_p }

{***************************************************************}
{	F_CursDown_p:						}
{	The Down arrow in edit mode.				}
{***************************************************************}

Procedure F_CursDown_p;				{ Global }

BEGIN	{ F_CursDown_p }

    IF EBufPtr.Row < LastRow THEN BEGIN			{ Not last row? }
        EBufPtr.Row := EBufPtr.Row + 1;				{ Change row }
	IF Length( EBuf[EBufPtr.Row] ) + 1 < EBufPtr.Col THEN { NL shorter? }
            EBufPtr.Col := Length( EBuf[EBufPtr.Row] ) + 1;	{ Pos at eol }
        GotoXY( EBufPtr.Col, EBufPtr.Row + 2);			{ Pos cursor }
    END; { IF }

END;	{ F_CursDown_p }

{***************************************************************}
{	F_CursRight_p:						}
{	The Right arrow.					}
{***************************************************************}

Procedure F_CursRight_p;			{ Global }

BEGIN	{ F_CursRight_p }

    IF EBufPtr.Col < LastCol THEN BEGIN			{ Not last column? }
        IF EBufPtr.Col < Length( EBuf[EBufPtr.Row] ) + 1 THEN BEGIN { eol? }
            EBufPtr.Col := EBufPtr.Col + 1;		{ Next column }
            GotoXY( EBufPtr.Col, EBufPtr.Row + 2);		{ Pos cursor }
        END; { IF }
    END; { IF }

END;	{ F_CursRight_p }

{***************************************************************}
{	F_CursLeft_p:						}
{	The Left arrow.						}
{***************************************************************}

Procedure F_CursLeft_p;				{ Global }

BEGIN	{ F_CursLeft_p }

    IF EBufPtr.Col > FirstCol THEN BEGIN		{ Not first column? }
        EBufPtr.Col := EBufPtr.Col - 1;			{ Previous column }
        GotoXY( EBufPtr.Col, EBufPtr.Row + 2);		{ Pos cursor }
    END; { IF }

END;	{ F_CursLeft_p }

{***************************************************************}
{	F_Home_p:						}
{	The Home key.	 					}
{***************************************************************}

Procedure F_Home_p;				{ Global }

BEGIN	{ F_Home_p }

    EBufPtr.Row := FirstRow;
    EBufPtr.Col := FirstCol;
    GotoXY( EBufPtr.Col, EBufPtr.Row + 2);		{ Pos cursor }

END;	{ F_Home_p }

{***************************************************************}
{	F_End_p:						}
{	The End key.	 					}
{***************************************************************}

Procedure F_End_p;				{ Global }

BEGIN	{ F_End_p }

    EBufPtr.Row := LastRow;
    EBufPtr.Col := FirstCol;
    GotoXY( EBufPtr.Col, EBufPtr.Row + 2);		{ Pos cursor }

END;	{ F_End_p }

{***************************************************************}
{	F_Ins_p:						}
{	The Ins key.	 					}
{***************************************************************}

Procedure F_Ins_p;				{ Global }

BEGIN	{ F_Ins_p }

    InsertMode := NOT InsertMode;

END;	{ F_Ins_p }

{***************************************************************}
{	Remove_Char:						}
{	Removes a character at current position, in buffer and	}
{	on screen.	 					}
{***************************************************************}

Procedure Remove_Char;

VAR
    StrPart	: lstr80;
    Range	: byte;

BEGIN	{ Remove_Char }
					{ Remove character from buffer }
    Delstring( EBuf[EBufPtr.Row], EBufPtr.Col, 1 );

					{ Remove character from screen }
    IF  EBufPtr.Col <= Length( EBuf[EBufPtr.Row] ) THEN BEGIN
				{ Characters to the right of the cursor }
        Range := Length( EBuf[EBufPtr.Row] ) - EBufPtr.Col + 1;
        Substring( StrPart, EBuf[EBufPtr.Row], EBufPtr.Col, Range );
        CursOff;
        Write( StrPart ); Break( Output );
        CursOn;
    END; { IF }
    ClrEol;	
    GotoXY( EBufPtr.Col, EBufPtr.Row + 2);

END;	{ Remove_Char }

{***************************************************************}
{	F_Del_p:						}
{	The Del key.	 					}
{***************************************************************}

Procedure F_Del_p;				{ Global }

BEGIN	{ F_Del_p }

    IF EBufPtr.Col < Length( EBuf[EBufPtr.Row] ) + 1 THEN BEGIN { eol? }
	Remove_Char;
    END; { IF }

END;	{ F_Del_p }

{*******************************************************}
{	Check_CF_GetBuf_parm:				}
{	Tries to open a file with the name of the	}
{	parameter of GetBuf command.			}
{*******************************************************}

Procedure Check_CF_GetBuf_parm;

VAR
    FileErrCode	: int4;

BEGIN	{ Check_CF_GetBuf_parm }

{XXX    IF NOT Buf_saved THEN
    	SaveQ;				{ Ask if buffer should be saved }
    IF (Buf_saved) OR (NOT Save_buf) THEN BEGIN
    	IF OpLineParm.Txt[1] = '' THEN BEGIN
            ParmQ := 'Filename';
	    DefVal := 'EDIT.MTS';
        END ELSE { IF OpLineParm.Txt[1] <> '' THEN } BEGIN
	    Reset( EditFile, OpLineParm.Txt[1] );
	    IF IoError( EditFile ) THEN BEGIN
	    	FileErrCode := IoStatus( EditFile );
	    	FileIoErrHandler( EditFile, EditFileName, FileErrCode );
		Parm_ok := false;
	    END ELSE BEGIN
	    	NoIoError( EditFile );		{ Turn on error trapping }
		EditFileName := OpLineParm.Txt[1];
		All_parms := true;
	    END;
	END;
    END;

END;	{ Check_CF_GetBuf_parm }

{***************************************************************}
{	CF_GetBuf_p:						}
{	The GetBuf command. 					}
{***************************************************************}

Procedure CF_GetBuf_p;				{ Global }

BEGIN	{ CF_GetBuf_p }

    Scroll (  FirstRow, FirstCol, LastRow, LastCol, 0, Up );{ Rm old lines }
    EBufPtr.Row := 1;
    EBufPtr.Col := 1;
    WHILE (NOT Eof) OR (EBufPtr.Row > LastRow) DO BEGIN	{ Read into buffer }
	ReadLn( EditFile, EBuf[EBufPtr.Row] );
	EBufPtr.Row := EBufPtr.Row + 1;
	EBufPtr.Col := EBufPtr.Col + 1;
    END; { WHILE }
    Close( Editfile );
    Display_EBuf;				{ Display editbuffer }
    Save_Buf := true;

END;	{ CF_GetBuf_p }

{*******************************************************}
{	Check_CF_SavBuf_parm:				}
{	Tries to open a file with the name of the	}
{	parameter of SavBuf command.			}
{*******************************************************}

Procedure Check_CF_SavBuf_parm;

VAR
    FileErrCode	: int4;

BEGIN	{ Check_CF_SavBuf_parm }

    IF OpLineParm.Txt[1] = '' THEN BEGIN
        ParmQ := 'Filename';
	DefVal := LogFileName;
    END ELSE { IF OpLineParm.Txt[1] <> '' THEN } BEGIN
        ReWrite( EditFile, OpLineParm.Txt[1] );
	IF IoError( EditFile ) THEN BEGIN
	    FileErrCode := IoStatus( EditFile );
	    FileIoErrHandler( EditFile, EditFileName, FileErrCode );
	    Parm_ok := false;
	END ELSE BEGIN
	    NoIoError( EditFile );		{ Turn on error trapping }
	    EditFileName := OpLineParm.Txt[1];
	    All_parms := true;
	END;
    END;

END;	{ Check_CF_SavBuf_parm }

{***************************************************************}
{	CF_SavBuf_p:						}
{	The SavBuf command. 					}
{***************************************************************}

Procedure CF_SavBuf_p;				{ Global }

VAR
    RowI	: byte;
BEGIN	{ CF_SavBuf_p }

    FOR RowI := FirstRow TO LastRow DO
        WriteLn( EBuf[RowI] );
    Close( Editfile );

END;	{ CF_SavBuf_p }

{***************************************************************}
{	F_DelLin_p:						}
{	The DelLin command. 					}
{***************************************************************}

Procedure F_DelLin_p;				{ Global }

VAR
    RowI	: byte;

BEGIN	{ F_DelLin_p }

    Scroll( EBufPtr.Row, FirstCol, LastRow, LastCol, 0, Up );

    DelLine := EBuf[EBufPtr.Row];		{ Save deleted line }

    FOR RowI := (EBufPtr.Row) TO (LastRow - 1) DO
        EBuf[RowI] := EBuf[RowI + 1];		{ Move lines in buffer }

    IF EBufPtr.Row < LastRow THEN BEGIN		{ If not last line }
	IF Length( EBuf[EBufPtr.Row] ) + 1 < EBufPtr.Col THEN { NL shorter? }
            EBufPtr.Col := Length( EBuf[EBufPtr.Row] ) + 1;	{ Pos at eol }
    END ELSE
    	EBufPtr.Col := FirstCol;		{ Pos at beginning of line }

    EBuf[LastRow] := '';			{ Empty last line in buffer }
    GotoXY( EBufPtr.Col, EBufPtr.Row + 2);			{ Pos cursor }

END;	{ F_DelLin_p }

{***************************************************************}
{	F_UnDel_p:						}
{	The UnDel command. 					}
{***************************************************************}

Procedure F_UnDel_p;				{ Global }

VAR
    RowI	: byte;

BEGIN	{ F_UnDel_p }

    IF DelLine <> '' THEN BEGIN
    	Scroll( EBufPtr.Row, FirstCol, LastRow, LastCol, 0, Down );
	GotoXY( FirstCol, EBufPtr.Row + 2);			{ Pos cursor }
	WriteLn( DelLine );				{ Write deleted line }

	FOR RowI := (LastRow) DOWNTO (EBufPtr.Row - 1) DO
    	    EBuf[RowI] := EBuf[RowI - 1];	{ Move lines in buffer }

	IF EBufPtr.Row < LastRow THEN BEGIN		{ If not last line }
	    IF Length( EBuf[EBufPtr.Row] ) + 1 < EBufPtr.Col THEN
            	EBufPtr.Col := Length( EBuf[EBufPtr.Row] ) + 1;	{ Pos at eol }
	END ELSE
    	    EBufPtr.Col := FirstCol;
	EBuf[EBufPtr.Row] := DelLine;

	GotoXY( EBufPtr.Col, EBufPtr.Row + 2);			{ Pos cursor }
    END;

END;	{ F_UnDel_p }

{***************************************************************}
{	Do_BS:							}
{	Performs a backspace in the editbuffer.			}
{***************************************************************}

Procedure Do_BS;

BEGIN	{ Do_BS }

    IF EBufPtr.Col > 1 THEN BEGIN	{ Is there a character to remove? }
	EBufPtr.Col := EBufPtr.Col - 1;
        GotoXY( EBufPtr.Col, EBufPtr.Row + 2);
	Remove_Char;
    END; { IF }

END;	{ Do_BS }

{***************************************************************}
{	Do_CR:							}
{	Moves the buffer pointer and cursor one line down.	}
{***************************************************************}

Procedure Do_CR;

BEGIN	{ Do_CR }

    IF EBufPtr.Row < LastRow THEN BEGIN		{ If not last row }
        EBufPtr.Row := EBufPtr.Row + 1;
        EBufPtr.Col := FirstCol;
        GotoXY( EBufPtr.Col, EBufPtr.Row + 2);		{ Pos cursor }
    END; { IF }

END;	{ Do_CR }

{***************************************************************}
{	Put_Ch_In_Buf:						}
{	Puts a character in the edit buffer.			}
{***************************************************************}

Procedure Put_Ch_In_Buf ( Ch	: char );

VAR
    ChStr	: str1;

BEGIN	{ Put_Ch_In_Buf }

    IF InsertMode THEN BEGIN
    	AssChar( ChStr, Ch );
	Insert( EBuf[EBufPtr.Row], ChStr, EBufPtr.Col );
    END ELSE BEGIN
	EBuf[EBufPtr.Row, EBufPtr.Col] := ch;
    END; { ELSE }

END;	{ Put_Ch_In_Buf }

{***************************************************************}
{	Put_Ch_On_Screen:					}
{	Puts a character in the edit window.			}
{***************************************************************}

Procedure Put_Ch_On_Screen ( Ch	: char );

VAR
    StrPart	: lstr80;
    Range	: byte;

BEGIN	{ Put_Ch_On_Screen }

    IF  EBufPtr.Col > Length( EBuf[EBufPtr.Row] ) THEN BEGIN
        Write( Ch ); Break( Output );
    END ELSE BEGIN		{ Characters to the right of the cursor }
        Range := Length( EBuf[EBufPtr.Row] ) - EBufPtr.Col + 1;
        Substring( StrPart, EBuf[EBufPtr.Row], EBufPtr.Col, Range );
	CursOff;
        Write( StrPart ); Break( Output );
	CursOn;
    END; { ELSE }

END;	{ Put_Ch_On_Screen }

{***************************************************************}
{	Put_Ch:							}
{	Puts a character in the edit buffer and window.		}
{	Adjusts the pointer.					}
{***************************************************************}

Procedure Put_Ch( Ch	: char );

VAR
    StrPart	: lstr80;
    Range	: byte;
    ChStr	: str1;

BEGIN	{ Put_Ch }

    IF NOT ((EBuf[EBufPtr.Row, EBufPtr.Col] = ch) AND NOT InsertMode) THEN
    BEGIN
							{ Put char in buffer }
        IF InsertMode THEN BEGIN
	    AssChar( ChStr, Ch );
    	    Insert( EBuf[EBufPtr.Row], ChStr, EBufPtr.Col );
        END ELSE BEGIN
    	    EBuf[EBufPtr.Row, EBufPtr.Col] := ch;
        END; { ELSE }
							{ Put char on screen }
        IF  EBufPtr.Col > Length( EBuf[EBufPtr.Row] ) THEN BEGIN
            Write( Ch ); Break( Output );
        END ELSE BEGIN		{ Characters to the right of the cursor }
            Range := Length( EBuf[EBufPtr.Row] ) - EBufPtr.Col + 1;
            Substring( StrPart, EBuf[EBufPtr.Row], EBufPtr.Col, Range );
	    CursOff;
            Write( StrPart ); Break( Output );
	    CursOn;
        END; { ELSE }
    END;
    IF EBufPtr.Col < LastCol THEN BEGIN		{ If not last column }
        EBufPtr.Col := EBufPtr.Col + 1;		{ Adjust pointer }
        GotoXY( EBufPtr.Col, EBufPtr.Row + 2);
    END; { IF }

END;	{ Put_Ch }

{***************************************************************}
{	EditLoop:						}
{	The editor.						}
{***************************************************************}

Procedure EditLoop;				{ Global }

VAR
    ChFetched	: boolean;		{ Was a character fetched? }
    Keytype	: Keytype_t;		{ What's in ch }
    ch		: char;			{ The character }

BEGIN	{ EditLoop }

    REPEAT
	Scan_Kbd( ChFetched, Keytype, ch );
        IF ChFetched THEN BEGIN
            IF ( Keytype = functionkey ) OR ( OpInput_exp = Parm ) THEN
		Get_OpCmd( ChFetched, Keytype, ch )	{ Command key }
	    ELSE BEGIN
		CASE ch OF
		    BS : Do_BS;
		    CR : Do_CR;
		    OTHERWISE { IF PrintableCh THEN } Put_Ch( Ch );
	    	END; { CASE }
	    END; { ELSE }
	END; { IF }
    UNTIL Menu <> EditMenu;

END;	{ EditLoop }
