{$nomain}
{***************************************************************}
{	This module contains error handling routines.		}
{***************************************************************}

%Include 'global.h';
%Include 'screen.h';
%Include 'error.h';

TYPE
    Err_rec = RECORD
	ErrType	: ErrType_t;		{ New errortype }
	Msg	: lstr78;		{ Error message }
    END;

CONST
    ErrorRow	= 23;
    ErrCodeStrWidth = 4;	{ Width of string to put error code in }
    FirstErrMsg	= 1
    LastErrMsg	= 6;

VAR	{$own='error'}

    PcErrCnt	: int4;		{ PC error counter }
    ErrText	: array [FirstErrMsg..LastErrMsg] of lstr78;

{***************************************************************}
{	ErrInit:						}
{	Initializes the error handling.				}
{***************************************************************}

Procedure ErrInit;			{ Global }

VAR
    i	: byte;

BEGIN	{ ErrInit }

    PcErrCnt := 0;
    FOR i := FirstErrMsg TO LastErrMsg DO
	Clear( ErrText[i] );

END;	{ ErrInit }

{***************************************************************}
{	Clr_ErrMsg						}
{	Clears error message on line 23.			}
{***************************************************************}

Procedure Clr_ErrMsg;				{ Global }

CONST
    Line = '--------------------------------------------------------------------------------';

BEGIN	{ Clr_ErrMsg }

    IF mode = edit THEN
    	CposWrite( ErrorRow, 1, Line )
    ELSE
    	ClrLine( ErrorRow );

END;	{ Clr_ErrMsg }

{***************************************************************}
{	Inc_cnt_p:						}
{	Increments the error counter.				}
{***************************************************************}

Procedure Inc_cnt_p;			{ Global }

BEGIN	{ Inc_cnt_p }

    PcErrCnt := PcErrCnt + 1;

END;	{ Inc_cnt_p }

{***************************************************************}
{	ScrollWrite_err_p:					}
{	Writes an error message in the scroll window.		}
{***************************************************************}

Procedure ScrollWrite_err_p;			{ Global }

VAR
    i		: byte;
    ErrStr	: lstr80;

BEGIN	{ ScrollWrite_err_p }

    InvVid;						{ Inverse video }
    i := 1;
    REPEAT
        ErrStr := ' ' + ErrText[i] + ' ';
        ScrollWriteLn( ErrStr );
        i := i + 1;
    UNTIL Length( ErrText[i] ) = 0 OR i > LastErrMsg;
    NorVid;						{ Normal video }

END;	{ ScrollWrite_err_p }

{***************************************************************}
{	ErrLineWrite_p:						}
{	Writes an error message on the error message line.	}
{***************************************************************}

Procedure ErrLineWrite_p;			{ Global }

VAR
    ErrStr	: lstr80;

BEGIN	{ ErrLineWrite_p }

    InvVid;						{ Inverse video }
    ErrStr := ' ' + ErrText[1] + ' ';
    CposWrite( ErrRow, 1, ErrStr );
    Break (Output);
    NorVid;						{ Normal video }

END;	{ ErrLineWrite_p }

{***************************************************************}
{	OpConfirm_p:						}
{	Waits for operator to hit a key.			}
{***************************************************************}

Procedure OpConfirm_p;				{ Global }

VAR
    ch	: char;

BEGIN	{ OpConfirm_p }

    WHILE Keypressed DO
    	ReadKbd( ch );			{ Clear the type-ahead buffer }
    Write( chr( 7 ) );			{ Beep }
    ScrollWrite_err( ' Hit any key to continue ' );
    REPEAT

    UNTIL KeyPressed;
    WHILE KeyPressed DO
    	ReadKbd( ch );			{ Wait until key pressed }

END;	{ OpConfirm_p }

{***************************************************************}
{	ForceOpen_LogFile:					}
{***************************************************************}

Procedure ForceOpen_LogFile;			{ Global }

BEGIN	{ ForceOpen_LogFile }

    IF NOT FileLogOn THEN Open_LogFile( 'PRINTERR.LOG' );

END;	{ ForceOpen_LogFile }

{***************************************************************}
{	FileLog_err:						}
{***************************************************************}

Procedure FileLog_err;			{ Global }

VAR
    i	: byte;

BEGIN	{ FileLog_err }

    i := 1;
    REPEAT
	Insert( ErrText[i], '&&& E  &&& ', 1 );
        IF FileLog_on THEN FileLog( ErrText[i] );
        i := i + 1;
    UNTIL (Length( ErrText[i] ) = 0) OR (i > LastErrMsg);

END;	{ FileLog_err }

{***************************************************************}
{	Terminate_FileLog:					}
{***************************************************************}

Procedure Terminate_FileLog;			{ Global }

BEGIN	{ Terminate_FileLog }

   FileLog_on := false;
   Close( LogFile );

END;	{ Terminate_FileLog }

{***************************************************************}
{	Act_PrnLog:						}
{	Activates printer log if printer is OK.			}
{***************************************************************}

Procedure Act_PrnLog;			{ Global }

BEGIN	{ Act_PrnLog }

    IF NOT PrintLog_on THEN Check_Printer;

END;	{ Act_PrnLog }

{***************************************************************}
{	Print_err:						}
{***************************************************************}

Procedure Print_err;			{ Global }

VAR
    i	: byte;

BEGIN	{ Print_err }

    i := 1;
    REPEAT
	Insert ( ErrText[i], '&&& E  &&& ', 1 );
        IF Print_on THEN PrintLog( ErrText[i] );
        i := i + 1;
    UNTIL Length( ErrText[i] ) = 0 OR i > LastErrMsg;

END;	{ Print_err }

{***************************************************************}
{	Terminate_PrnLog:					}
{***************************************************************}

Procedure Terminate_PrnLog;			{ Global }

BEGIN	{ Terminate_PrnLog }

    Proff;

END;	{ Terminate_PrnLog }

{***************************************************************}
{	Terminate_SyntCheck:					}
{***************************************************************}

Procedure Terminate_SyntCheck;			{ Global }

BEGIN	{ Terminate_SyntCheck }

    Cont_SyntCheck := false;

END;	{ Terminate_SyntCheck }

{***************************************************************}
{	Terminate_batch:					}
{***************************************************************}

Procedure Terminate_batch;			{ Global }

BEGIN	{ Terminate_batch }

    Cont_Batch := false;

END;	{ Terminate_batch }

{***************************************************************}
{	Terminate_IntactSend:					}
{***************************************************************}

Procedure Terminate_IntactSend;			{ Global }

BEGIN	{ Terminate_IntactSend }

    Mode := InteractiveIdle;

END;	{ Terminate_IntactSend }

{***************************************************************}
{	Terminate_EditFileAcc:					}
{***************************************************************}

Procedure Terminate_EditFileAcc;			{ Global }

BEGIN	{ Terminate_EditFileAcc }

    Cont_EditFileAcc := false;

END;	{ Terminate_EditFileAcc }

{***************************************************************}
{	Terminate_CommScan:					}
{***************************************************************}

Procedure Terminate_CommScan;			{ Global }

BEGIN	{ Terminate_CommScan }

    Cont_CommScan := false;

END;	{ Terminate_CommScan }

{***************************************************************}
{	Terminate_pgm:						}
{***************************************************************}

Procedure Terminate_pgm;			{ Global }

BEGIN	{ Terminate_pgm }

    Cont_Pgm := false;

END;	{ Terminate_pgm }

{***************************************************************}
{	ErrAction:						}
{	Checks which actions the error requires.		}
{***************************************************************}

Procedure ErrAction( ErrType : ErrType_t );		{ Global }

TYPE
    { Different actions }

    Action_t = ( Inc_cnt, ScrollWrite_err, ErrLineWrite, OpConfirm,
                 Open_LogFile, FileLog_err, Term_FileLog, Act_PrintLog,
                 Print_err, Term_PrintLog, Term_SyntCheck, Term_Batch,
                 Term_IntactSend, Term_EditFileAcc, Term_CommScan,
                 Term_Pgm ); 

    { Table telling what actions different error types requires }
    Action_arr = array [Inc_cnt..Term_pgm, BatSyntax..Prn_busy] of boolean;

CONST
    T = true;
    f = false;
    ActionTab	= Action_arr

       {    ISy   EFR   LFW   MAl   PgI   PIO   PrP    }
       { BSy   BFR   EFW   Com   OpI   PrT   PDs   PrB }

      ( ( T, f, T, T, T, T, T, T, f, f, T, T, f, T, f ),  { Inc_cnt }
	( T, T, T, f, f, f, T, T, f, f, T, f, T, T, f ),  { ScrollWrite_err }
	( f, f, f, T, T, T, f, f, T, T, f, T, f, f, f ),  { ErrLineWrite }
	( f, f, T, f, f, f, f, f, f, T, f, f, f, f, f ),  { OpConfirm }
	( f, f, f, f, f, f, f, f, f, f, T, T, f, T, f ),  { Open_LogFile }
	( T, f, T, f, f, f, T, T, f, T, T, T, f, T, f ),  { FileLog_err }
	( f, f, f, f, f, T, f, f, f, f, f, f, f, f, f ),  { Term_FileLog }
	( f, f, f, f, f, T, f, f, f, f, f, f, f, f, f ),  { Act_PrintLog }
	( T, f, T, f, f, T, T, T, f, T, f, f, f, f, f ),  { Print_err }
	( f, f, f, f, f, f, f, f, f, f, f, T, T, T, f ),  { Term_PrintLog }
	( f, f, T, f, f, f, f, T, f, f, f, f, f, f, f ),  { Term_SyntCheck }
	( f, f, T, f, f, f, f, f, f, f, f, f, f, f, f ),  { Term_Batch }
	( f, T, f, f, f, f, f, f, f, f, f, f, f, f, f ),  { Term_IntactSend }
	( f, f, f, T, T, f, f, f, f, f, f, f, f, f, f ),  { Term_EditFileAcc }
	( f, f, f, f, f, f, f, f, f, f, f, f, f, f, f ),  { Term_CommScan }
	( f, f, f, f, f, f, f, f, f, f, f, f, f, f, f ) );{ Term_Pgm }

VAR
    i	: byte;

BEGIN	{ ErrAction }

    IF ActionTab[Inc_cnt,	   ErrType] THEN Inc_cnt_p;
    IF ActionTab[ScrollWrite_err,  ErrType] THEN ScrollWrite_err_p;
    IF ActionTab[ErrLineWrite,	   ErrType] THEN ErrLineWrite_p;
    IF ActionTab[OpConfirm,	   ErrType] THEN OpConfirm_p;
    IF ActionTab[Open_LogFile,	   ErrType] THEN Open_LogFile_p;
    IF ActionTab[FileLog_err,	   ErrType] THEN FileLog_err_p;
    IF ActionTab[Term_FileLog,	   ErrType] THEN Term_FileLog_p;
    IF ActionTab[Act_PrintLog,	   ErrType] THEN Act_PrintLog_p;
    IF ActionTab[Print_err,	   ErrType] THEN Print_err_p;
    IF ActionTab[Term_PrintLog,	   ErrType] THEN Term_PrintLog_p;
    IF ActionTab[Term_SyntCheck,   ErrType] THEN Term_SyntCheck_p;
    IF ActionTab[Term_Batch,	   ErrType] THEN Term_Batch_p;
    IF ActionTab[Term_IntactSend,  ErrType] THEN Term_IntactSend_p;
    IF ActionTab[Term_EditFileAcc, ErrType] THEN Term_EditFileAcc_p;
    IF ActionTab[Term_CommScan,	   ErrType] THEN Term_CommScan_p;
    IF ActionTab[Term_Pgm,	   ErrType] THEN Term_Pgm_p;

    i := 1;
    REPEAT
	Clear( ErrText[i] );
        i := i + 1;
    UNTIL Length( ErrText[i] ) = 0 OR i > LastErrMsg;

END;	{ ErrAction }

{***************************************************************}
{	BatSyntErrHandler:					}
{	Takes care of the batch syntax errors.			}
{***************************************************************}

Procedure BatSyntErrHandler( BatSyntErr	 : BatSyntErr_t;
			     BatFileLine : BatFileLine_t;
			     LineLen	 : int2		);	{ Global }

TYPE
    BatSyntErrTab_arr = array [Illeg_char..No_EndOfString_found] of lstr78;

CONST
    BatSyntErrTab = BatSyntErrTab_arr		{ Syntax error messages }
	      ( 'Illegal character',
		'Line too long',
		'Unknown command',
		'Parameter missing',
		'Too many parameters',
		'Not a value',
		'Parameter out of range',
		'Value out of range',
		'For-Next level',
		'No corresponding FOR statement',
		'Missing NEXT statement',
		'Too many answers expected',
		'No end of string found (")' );

BEGIN	{ BatSyntErrHandler }

    ErrText[1] := 'Syntax error: ';
    ErrText[1] := ErrText[1] + BatSyntErrTab[BatSyntErr];
    ErrText[2] := BatFileLine;
    Head2 := false;			{ Skriv inte ut rubrik2 }
    ErrAction( BatSyntax );
    Head2 := true;

END;	{ BatSyntErrHandler }

{***************************************************************}
{	IntactSyntErrHandler:					}
{***************************************************************}

Procedure IntactSyntErrHandler;			{ Global }

BEGIN	{ IntactSyntErrHandler }

    ErrText[1] := 'No end of string found (")' );
    ErrAction( IntactSynt );

END;	{ IntactSyntErrHandler }

{***************************************************************}
{	CommErrHandler:						}
{***************************************************************}

Procedure CommErrHandler( CommErr : CommErr_t );	{ Global }

TYPE
    CommErrTab_arr = array [Invalid_msg..Timeout] of lstr78;

CONST
    CommErrTab = CommErrTab_arr
	      ( 'Invalid message from remote',
	        'Remote message too long',
		'Timeout. No characters received' );

BEGIN	{ CommErrHandler }

    ErrText[1] := CommErrTab[CommErr];
    ErrAction( Comm );

END;	{ CommErrHandler }

{***************************************************************}
{	Rs232RetCodeHandler:					}
{***************************************************************}

Procedure Rs232RetCodeHandler( RetCode : int2 );		{ Global }

TYPE 
    RetCodeErr_t = ( Invalid_COM_port, COM_port_is_not_open,
                     Invalid_option_spec_or_parm, No_serial_port_found,
                     Output_queue_is_full, COM_port_already_open,
                     Input_queue_is_empty, Asynch_Manager_internal,
                     Unexp_RetCode ); 

    Rs232RetCodeTab_arr = array [Invalid_COM_port..Asynch_Manager_internal]
				of Err_rec;

CONST
    Rs232RetCodeTab = Rs232RetCodeTab_arr
	      ( ( PgmInternal, 'Invalid COM port' ),
		( PgmInternal, 'COM port is not open' ),
		( PgmInternal, 'Invalid option specification or parameter' ),
		( PgmInternal, 'No serial port found' ),
		( PgmInternal, 'Output queue is full' ),
		( PgmInternal, 'COM port already open' ),
		( ???????????, 'Input queue is empty' ),
		( PgmInternal, 'Asynch Manager internal' ),
		( PgmInternal, 'Unexpected Asynch Manager return code : ' ) );

VAR
    RetCodeErr	: RetCodeErr_t;
    ErrVal_str	: lstr4;

BEGIN	{ Rs232RetCodeHandler }

    CASE RetCode OF
	  2 : RetCodeErr := Invalid_COM_port;
	  3 : RetCodeErr := COM_port_is_not_open;
	  4 : RetCodeErr := Invalid_option_spec_or_parm;
	  6 : RetCodeErr := No_serial_port_found;
	  7 : RetCodeErr := Output_queue_is_full;
	  9 : RetCodeErr := COM_port_already_open;
	 10 : RetCodeErr := Input_queue_is_empty;
	255 : RetCodeErr := Asynch_Manager_internal; 
	OTHERWISE RetCodeErr := Unexp_RetCode;
    END; { CASE }
    ErrText[1] := Rs232RetCodeTab[RetCodeErr].Msg;
    IF RetCodeErr = Unexp_RetCode THEN BEGIN
	Str( RetCode: 1, ErrValStr );
	ErrText := ErrText + ErrVal_str;
    END;
    ErrAction( Rs232RetCodeTab[RetCodeErr].ErrType );

END;	{ Rs232RetCodeHandler }

{***************************************************************}
{	Rs232IqErrHandler:					}
{***************************************************************}

Procedure Rs232IqErrHandler( IqStatus : word );			{ Global }

TYPE
    Rs232IqErrTab_rec = RECORD
		Value	: int2;
		Msg	: lstr78;
    END;

    Rs232IqErrTab_arr = array [Input_lost..Break] of Rs232IqErrTab_rec;

CONST
    Rs232IqErrTab = Rs232IqErrTab_arr
	      ( ( 16#0001, 'Input lost' ),
		( 16#0002, 'Asynch Manager internal, input queue' ),
		( 16#0010, 'Parity' ),
		( 16#0020, 'Overrun' ),
		( 16#0040, 'Framing' ),
		( 16#0080, 'Break' ) );

VAR
    IqErr	: word;
    i		: byte;

BEGIN	{ Rs232IqErrHandler }

    i := 1;
    FOR IqErr := Input_lost TO Break DO BEGIN
	IF IqStatus AND Rs232IqErrTab[IqErr] = Rs232IqErrTab[IqErr] THEN BEGIN
	    ErrText[i] := Rs232IqErrTab[IqErr];
	END;
	i := i + 1;
    END;
    ErrAction( Comm );

END;	{ Rs232IqErrHandler }

{***************************************************************}
{	GpibErrHandler:						}
{***************************************************************}

Procedure GpibErrHandler( GpibErr : int2 );		{ Global }

TYPE
    GpibErrTab_arr = array [EUnknown..EAddr] of Err_rec;

CONST
    GpibErrTab = GpibErrTab_arr
	      ( ( PgmInternal, 'Unknown' ),
		( PgmInternal, 'Invalid select code or device address' ),
		( PgmInternal, 'Command parameter outside range' ),
		( Comm,        'Timeout (but character(s) received)' ),
		( PgmInternal, 'HP-IB must be system controller' ),
		( PgmInternal, 'Pass control is not supported' ),
		( PgmInternal, 'No number' ),
		( PgmInternal, 'Improper addressing' ) );

VAR
    ErrVal_str	: lstr4;

BEGIN	{ GpibErrHandler }

    IF GpibErr > 0 AND GpibErr < 9 THEN BEGIN	{ Correct GPIB error code }
	ErrText[1] := GpibErrTab[GpibErr].Msg;
	ErrAction( GpibErrTab[GpibErr].ErrType );
    END	ELSE BEGIN				{ Undefined GPIB error code }
	Str( GpibErr: 1, ErrValStr );
	ErrText := "Unexpected GPIB error code: " + ErrVal_str;
	ErrAction( PgmInternal );
    END; { ELSE }
    
END;	{ GpibErrHandler }

{***************************************************************}
{	FileIoErrHandler:					}
{***************************************************************}

Procedure FileIoErrHandler( File     : text;
			    FileName : string[FileNameSize];
			    IoErr    : int4 );			{ Global }

TYPE
    File_t = ( BatchFile, EditFile, LogFile );

    FileErrType_t = ( Cannot_open_file, Default_file_name_syntax_error,
                      Default_file_switch_error, File_name_syntax_error,
                      File_already_open, Too_many_files_open,
                      Reading_past_end_of_file, BatchFileRead,
                      BatchFileWrite, EditFileRead, EditFileWrite,
                      LogFileRead, LogFileWrite, Invalid_file_reference,
                      Not_a_random_access_file, File_not_open,
                      Incomplete_record_at_end_of_file,
                      Cannot_delete_file, Cannot_rename_file,
                      Cannot_lock_unlock_file, No_read_access,
                      No_write_access, Not_enough_file_buffer_memory,
                      Put_not_at_end_of_file, Unexp_RunTimeErr,
                      SystemErr ); 

    FileRWErrTab_arr = packed array [BatchFile..LogFile, -20..-21] 
				    of FileErrType_t;

    FileIoErrTab_arr = array [Cannot_open_file..SystemErr] of Err_rec;

CONST
    FileRWErrTab = FileRWErrTab_arr
	      ( ( BatchFileRead, BatchFileWrite ),
		( EditFileRead,  EditFileWrite  ),
		( LogFileRead,   LogFileWrite   ) );

    FileIoErrTab = FileIoErrTab_arr
	      ( ( OpInput,       'Cannot open file' ),
		( OpInput,       'Default file name syntax error' ),
		( OpInput,       'Default file switch error' ),
		( OpInput,       'File name syntax error' ),
		( PgmInternal,   'File already open' ),
		( PgmInternal,   'Too many files open' ),
		( PgmInternal,   'Reading past end of file' ),
		( BatchFileRead, 'Error reading batch file' ),
		( PgmInternal,   'Error writing batch file' ),
		( EditFileRead,  'Error reading edit file' ),
		( EditFileWrite, 'Error writing edit file' ),
		( PgmInternal,   'Error reading log file' ),
		( LogFileWrite,  'Error writing log file' ),
		( PgmInternal,   'Invalid file reference' ),
		( PgmInternal,   'Not a random access file' ),
		( PgmInternal,   'File not open' ),
		( PgmInternal,   'Incomplete record at end of file' ),
		( PgmInternal,   'Cannot delete file' ),
		( PgmInternal,   'Cannot rename file' ),
		( PgmInternal,   'Cannot lock/unlock file' ),
		( PgmInternal,   'No read access' ),
		( PgmInternal,   'No write access' ),
		( PgmInternal,   'Not enough file buffer memory' ),
		( PgmInternal,   'Put() not at end of file' ),
		( PgmInternal,   'Unexpected run-time error' ),
		( PgmInternal,   'System error' ) );

VAR
    FileErrType	: FileErrType_t;
    ErrVal_str	: lstr4;

BEGIN	{ FileIoErrHandler }

    IF IoErr < 0 THEN BEGIN		{ Pascal run-time error code }
	IF IoErr = -20 OR IoErr = -21 THEN
	    FileErrType := FileRWErrTab[File, IoErr]	{ Read/write error }
	ELSE BEGIN
	    CASE IoErr OF				{ Other I/O errors }
		-13 : FileErrType := Cannot_open_file;
		-14 : FileErrType := Default_file_name_syntax_error;
		-15 : FileErrType := Default_file_switch_error;
		-16 : FileErrType := File_name_syntax_error;
		-17 : FileErrType := File_already_open;
		-18 : FileErrType := Too_many_files_open;
		-19 : FileErrType := Reading_past_end_of_file;
		-28 : FileErrType := Invalid_file_reference;
		-31 : FileErrType := Not_a_random_access_file;
		-33 : FileErrType := File_not_open;
		-41 : FileErrType := Incomplete_record_at_end_of_file;
		-42 : FileErrType := Cannot_delete_file;
		-43 : FileErrType := Cannot_rename_file;
		-45 : FileErrType := Cannot_lock_unlock_file;
		-46 : FileErrType := No_read_access;
		-47 : FileErrType := No_write_access;
		-49 : FileErrType := Not_enough_file_buffer_memory;
		-50 : FileErrType := Put_not_at_end_of_file;
		OTHERWISE FileErrType := Unexp_RunTimeErr;
	    END; { CASE }
	END; { ELSE }
    END	ELSE BEGIN
	FileErrtype := SystemErr;
    END; { ELSE }
    ErrText[1] := FileIoErrTab[FileErrType].Msg + " " + FileName;
    IF FileErrType = Unexp_RunTimeErr OR FileErrType = SystemErr THEN BEGIN
	Str( IoErr: 1, ErrValStr );
	ErrText := ErrText + ErrVal_str;
    END;
    ErrAction( FileIoErrTab[FileErrType].ErrType );

END;	{ FileIoErrHandler }

{***************************************************************}
{	MemAllocErrHandler:					}
{***************************************************************}

Procedure MemAllocErrHandler;			{ Global }

BEGIN	{ MemAllocErrHandler }

    ErrText[1] := 'Batch file to big to fit in RAM';
    ErrAction( MemAlloc );

END;	{ MemAllocErrHandler }

{***************************************************************}
{	OpCmdErrHandler:					}
{***************************************************************}

Procedure OpCmdErrHandler( OpCmdErr );			{ Global }

TYPE
    OpCmdErrTab_arr = array [InvalidCmd..InvalidParm] of lstr78;

CONST
    OpCmdErrTab = OpCmdErrTab_arr
	      ( 'Unknown command',
		'Command not allowed here',
		'Invalid function key',
		'Too few parameters',
		'Too many parameters',
		'Invalid parameter' );

BEGIN	{ OpCmdErrHandler }

    ErrText[1] := OpCmdErrTab[OpCmdErr].Msg;
    ErrAction( OpInput );
    OpInput_Ok := false;

END;	{ OpCmdErrHandler }

{***************************************************************}
{	PrnErrHandler:						}
{***************************************************************}

Procedure PrnErrHandler( PrnStatus );			{ Global }

TYPE
    PrnErrTab_rec = RECORD
	MaskVal	: byte;
	ErrVal	: byte;
	ErrType	: ErrType_t;		{ New errortype }
	Msg	: lstr78;		{ Error message }
    END;

    PrnErrTab_arr = array [Timeout..Busy] of PrnErrTab_rec;

CONST
    PrnErrTab = PrnErrTab_arr
	      ( ( 2#00000001, 2#00000001, PrnTimeout,     'Timeout' ),
		( 2#00001000, 2#00001000, PrnIo,          'I/O error' ),
		( 2#00010000, 2#00000000, Prn_desel,      'Deselected' ),
		( 2#00100000, 2#00100000, Prn_OutOfPaper, 'Out of paper' ),
		( 2#10000000, 2#00000000, Prn_busy,       'Busy' ) );

VAR
    PrnErr	: byte;
    i		: byte;

BEGIN	{ PrnErrHandler }

    i := 1;
    FOR PrnErr := PrintTimeout TO Busy DO BEGIN
	IF PrnStatus AND PrnErrTab[PrnErr].Mask = PrnErrTab[PrnErr].ErrVal
	THEN BEGIN
	    ErrText[i] := PrnErrTab[PrnErr].Msg;
	    ErrAction( PrnErrTab[PrnErr].ErrType );
	END;
	i := i + 1;
    END;

END;	{ PrnErrHandler }

{***************************************************************}
{	InternalErrHandler:					}
{***************************************************************}

Procedure InternalErrHandler;			{ Global }

TYPE
    InternErrTab_arr = array [One..The_other] of Err_rec;

CONST
    InternErrTab = InternErrTab_arr
	      ( 'Internfel 1',
		'Internfel 2' );

BEGIN	{ InternalErrHandler }

    ErrText[1] := InterErrTab[InternErr].Msg;
    ErrAction( PgmInternal );

END;	{ InternalErrHandler }
