PROGRAM hpib (input, output);

{ This program is a modified version of the Op Amp program described
  in Section 2B of the HP-IB Command Library for MS-DOS manual.           }

%include 'global.h';
%include 'gpib.h';

procedure Halt; EXTERNAL;

VAR
  isc            : int2;        { Interface select code }
  sth            : int2;        { Address of STH }

{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}

PROCEDURE Error_handler (error: int2; routine: lstr10);

     { Checks for error return from I/O Library call }

VAR
  estring: lstr40;

BEGIN
    IF error <> noerr THEN BEGIN
    	Errstr (error, estring);	{ Get corresponding error string }
	writeln;
	writeln ('Error in call to ', routine);
	writeln (error:6, ' ', estring);
	writeln;
	write ('Press <RETURN> to continue program: ');
	readln;
    END; { if }
END;

{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}

PROCEDURE Getaddr;

BEGIN
    write ('Enter interface select code (usually 7) <RET> ');
    readln (isc);
    REPEAT
	write ('Enter bus address of STH: ');
	readln (sth);
	IF (sth > 29) OR (sth < 0) THEN
	    writeln('      Bus address must be between 0 and 29');
    UNTIL (sth >= 0) AND (sth <= 29);
    sth := isc * 100 + sth;
end;

{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}

PROCEDURE Initialize;

     { Performs bus initialization.  Establishes a timeout value in the
       event there is an error in a device, sends a universal device
       clear placing all devices in a known state and sets REN so devices
       will go to remote mode when addressed to listen.                   }

VAR
    return_status: int2;

BEGIN
    return_status := IORESET (isc);
    Error_handler (return_status, 'IORESET   ');
    return_status := IOTIMEOUT (isc, 5.0);    
    Error_handler (return_status, 'IOTIMEOUT ');
    return_status := IOCLEAR (isc);
    Error_handler (return_status, 'IOCLEAR   ');
    return_status := IOREMOTE (isc);
    Error_handler (return_status, 'IOREMOTE  ');
END;

{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}

PROCEDURE sth_send;

     { Sends a command to STH;	}

 
VAR
    commands       : str4;
    return_status  : int2;

BEGIN
    commands := 'M2 2';		{ send '2' to unit 2 too }
    return_status := IOOUTPUTS (sth, commands, 4);
    Error_handler (return_status, 'IOOUTPUTS ');
END;

{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}

PROCEDURE Wait_for_SRQ;

     { Wait for STH to request service indicating it will send data.
       No need of serial poll since only STH is connected.	}

CONST
    srqline = 1;

VAR
    return_status  : int2;
    response       : int2;
    i		   : int2;

BEGIN	{ Wait_for_SRQ }

    i := 0;
    REPEAT
    	return_status := IOSTATUS (isc, srqline, response);
	Error_handler (return_status, 'IOSTATUS  ');
	i := i + 1;
	IF i >= 10000 THEN BEGIN
	    WriteLn ( 'No SRQ' );
	    Halt;
	END;
    UNTIL response <> 0;

END;	{ Wait_for_SRQ }

{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}

PROCEDURE Readout;

VAR
  i		: int2;
  length	: int2;
  result	: str255;
  return_status	: int2;

BEGIN
    length := 255;
    return_status := IOENTERS (sth, result, length);
    Error_handler (return_status, 'IOENTERS  ');
    Write ('The returned answer is: ');
    FOR i := 1 TO length DO
    	write (result[i]);
    WriteLn;
    Break (output);
END;

{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}

BEGIN { main }

    Getaddr;
    Initialize;
    sth_send;
    Wait_for_SRQ;
    Readout;

END. { main }
