{$nomain}
{***************************************************************}
{	This module contains keyboard handling routines.	}
{***************************************************************}

%Include 'global.h';
%Include 'turbodef.h';
%Include 'screen.h';
%Include 'kbd.h';
%Include 'opipret.h';

TYPE
    fk_char_t	= array[fk_1..fk_end] of char;

CONST

    { Connection between fkey_t and characters after escape }

    fk_char = fk_char_t (
	';','<','=','>',	{ fk_1 - fk_4 }
	'?','@','A','B',	{ fk_5 - fk_8 }
	'C','D','H','P',	{ fk_9,fk_10,fk_up,fk_down }
	'K','M','R','S',	{ fk_left,fk_right,fk_ins,fk_del }
	'G','O');		{ fk_home,fk_end }

VAR	{$own='Kbd'}

    OpInput_OK	: boolean;	{ If command / parameter is OK }
    OpLine	: lstr80;	{ Operator command line }

{***************************************************************}
{	Init_Kbd:						}
{	Initiates keyboard routines.				}
{***************************************************************}

Procedure Init_Kbd;				{ Global }

BEGIN	{ Init_Kbd }

    OpInput_OK := true;		{ No error message on screen }
    OpLine := '';		{ Operator command line }

END;	{ Init_Kbd }

{***************************************************************}
{	Scan_Kbd:						}
{	Reads one character or a function key from kbd, if 	}
{	there is any available.					}
{***************************************************************}

Procedure Scan_Kbd;

BEGIN	{ Scan_Kbd }

    IF KeyPressed THEN BEGIN
	ReadKbd(ch);
	ChFetched := true;
{D}	IF ch = chr(3) THEN		{ Ctrl-Break or Ctrl-C }
{D}	    Halt;
	IF ( ch = ESC ) AND KeyPressed THEN BEGIN
	    ReadKbd(ch);
	    Keytype := functionkey;
	END ELSE BEGIN
	    Keytype := character;
	END;
    END ELSE BEGIN
	ChFetched := false;
    END;

END;	{ Scan_Kbd }

{***************************************************************}
{	EraseOpLine:						}
{	Clears Operator input line, buffer and screen.		}
{***************************************************************}

Procedure EraseOpLine;

BEGIN	{ EraseOpLine }

    ClrOpLine;			{ Operator input line on screen }
    OpLine := '';		{ Operator command line }
    
END;	{ EraseOpLine }

{***************************************************************}
{	Get_OpCmd:						}
{	Reads characters from keyboard, until no characters are	}
{	available or a complete command is received.		}
{***************************************************************}

Procedure Get_OpCmd;			{ Global }

VAR
    CompleteCmd	: boolean;		{ True when CR or func. key }
    FKey	: FKey_t;		{ The function key }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{	Get_FKey:						}
{	Finds out which function key has been pressed.		}
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

Procedure Get_Fkey;

VAR
    fi		: fkey_t;		{ Index when searching function key }
    Valid_FKey	: boolean;		{ If valid fkey }

BEGIN	{ Get_Fkey }

    Valid_FKey := false;
    FOR fi := fk_1 TO fk_end DO BEGIN
    	IF ch = fk_char[fi] THEN BEGIN	{ Function key used in this program? }
	    Valid_FKey := true;
	    fkey := fi;
	END;
    END;
    IF NOT Valid_FKey THEN BEGIN
    	ch := NULL;			{ Non existing fkey }
	key := nokey;
    END;

END;	{ Get_Fkey }

{***************************************************************}

BEGIN	{ Get_OpCmd }

    CompleteCmd := false;
    IF NOT OpInput_OK THEN BEGIN	{ Error message on screen? }
	Clr_ErrorMsg;
	OpInput_OK := true;
    END;

    WHILE ChFetched AND NOT CompleteCmd DO BEGIN
    	IF Keytype = functionkey THEN
    	    Get_Fkey
	ELSE
	    fkey := fk_other;
	IF (Keytype = functionkey) OR ((ch = CR) AND (OpLine <> '')) THEN BEGIN
	    CompleteCmd := true;
	    UnpackOpCmd (fkey, OpLine, OpInput_OK);
	    IF OpInput_OK THEN BEGIN
		DisplayPrompt;
		OpLine := '';
	    END ELSE BEGIN
	    	WHILE keypressed DO 
		    ReadKbd(ch);	{ Clear input buffer }
		{ beep; }		{ Beep and a short pause }
		{ pause; }
	    END;
	END ELSE IF ( ch = ESC ) THEN BEGIN
	    IF OpLine = '' THEN OpInput_exp := Cmd
	    ELSE EraseOpLine;		{ Screen after prompter, and buffer }
	END ELSE IF ( ch = BS ) THEN BEGIN
	    IF OpLine <> '' THEN BEGIN
	    	Write(BS, ' ', BS); Break (Output );
		OpLine[0] := chr(ord (OpLine[0]) - 1);
	    END;
	END ELSE IF ( (ch >= #32) AND (ch <= #254) ) AND (ch <> #127) AND
		    ( WhereX < 80 ) THEN BEGIN
	    Write(ch); Break (Output );
	    OpLine := OpLine + UpCase(ch);
	END;
	IF NOT CompleteCmd THEN Scan_Kbd( ChFetched, Keytype, ch );
    END; { WHILE }

END;	{ Get_OpCmd }
