{$nomain}
{***************************************************************}
{	This module contains log routines.	 		}
{***************************************************************}

%Include 'global.h';
%Include 'turbodef.h';
%Include 'main.h';
%Include 'screen.h';
%Include 'time.h';
%Include 'log.h';

Procedure Time  ( VAR	S  : string ); EXTERNAL;
Procedure Log; FORWARD;

TYPE
    LogTypeCnt_arr	= array [S..E] OF int4;

CONST
    LF			= chr(10);	{ Linefeed }
    FF			= chr(12);	{ Formfeed }
    CR			= chr(13);	{ Carrige Return }
    LastLine		= 42;		{ Antal rader pa papperet }
    Line = '------------------------------------------------------------------------------------------------------------------';

VAR
    LogFileName	: lstr13);
    HeadStr_1	: lstr132);	{ LogHead pa  varje Page }
    HeadStr_2	: lstr132);	{ LogHead, rubriker }
    TextStr	: lstr132);	{ Stren som Logs }
    Cnt		: LogTypeCnt_arr;	{ Teststatistk }
    TmpDate	: str9;		{ L{gg Date h{r medans Logning sker s} att }
    				{ man har samma Date hela Timeen }
    OldDate	: str9;		{ Date senast Logde Row }
    Page	: int2;		{ Page i Logningen }
    LineNr	: int2;		{ Rowraknare }
    CommHead	: boolean;	{ om alla rubrikrows skall ut }
    DiskErr	: boolean;	{ true om det blir Errkod fr}n disken }

{*******************************************************}
{	Make_LogFileName.				}
{	Skapar ett LogFile-Name.			}
{*******************************************************}

Procedure Make_LogFileName;

VAR
    TmpStr	: str9;
    StartTime	: lstr8;

BEGIN	{ Make_LogFileName }

    TmpStr := Date;
    StartTime.len := 8;
    Time ( StartTime );
    LogFileName[1] := TmpStr[2];			{ Ar }
    IF TmpStr[4] = '1' THEN
	TmpStr[5] := Chr ( Ord(TmpStr[5]) + 17 );	{ ( HEX ) }
    LogFileName[2] := TmpStr[5];			{ Manad }
    LogFileName[3] := TmpStr[7];			{ Dag }
    LogFileName[4] := TmpStr[8];			{ Dag }
    LogFileName[5] := StartTime[1];			{ Timme }
    LogFileName[6] := StartTime[2];			{ Timme }
    LogFileName[7] := StartTime[4];			{ Minut }
    LogFileName[8] := StartTime[5];			{ Minut }
    LogFileName.LEN := 8;
    Concat ( LogFileName, '.LOG ' );

    CposWrite ( 23, 1, 'LogFile: ' );
    CposWrite ( 23, 10, LogFileName );

END;	{ Make_LogFileName }

{*******************************************************}
{	Open_LogFile.					}
{	Oppnar LogFilen.				}
{*******************************************************}

Procedure Open_LogFile; EXTERNAL;

BEGIN	{ Open_LogFile }

    Assign ( LogFile, LogFileName );
    LogFile.TRAP := true;		{ Error trapping paaslagen }
    LogFile.ERRS := 0;			{ For sakerhets skull }
    ReWrite ( LogFile );
    IF LogFile.ERRS <> 0 THEN BEGIN
	ShowFileErr( 'OPEN','LogFile', LogFileName, Ord( LogFile.ERRS ) );
	Close( LogFile );
	RETURN;
    END;

END;	{ Open_LogFile }

{*******************************************************}
{	PrOff						}
{	Slar av printer					}
{*******************************************************}

Procedure PrOff; EXTERNAL;

BEGIN	{ PrOff }

	PrintOn := false;
	InvVid;
	CposWrite(25,73,' PrOff ');
	NorVid;

END;	{ Proff }

{*******************************************************}
{	Check_Printer.					}
{	Kollar att printern ar OK.			}
{*******************************************************}

Procedure Check_Printer ( status:byte ); EXTERNAL;

VAR
    PrintOn_Old : boolean;

BEGIN	{ Check_Printer }

    PrintOn_Old := PrintOn;
    PrintOn := ( status AND 2#00111000 ) = 2#00010000;

    IF PrintOn <> PrintOn_Old THEN BEGIN
	IF PrintOn THEN BEGIN
	    InvVid;
	    CposWrite(25,73,' PrOn  ');
	    NorVid;
	END ELSE
	    PrOff;
    END;

END;	{ Check_Printer }

{*******************************************************}
{	HexConv Convertera int2 till en str{ng i	}
{	hexformat. ex: #00				}
{*******************************************************}

function HexConv ( NumOfNumCh : int2;
		   Value     : int2 ): lstr5; EXTERNAL;


VAR
    i   : int2;
    str : lstr5;

BEGIN	{ HexConv }

    
    Eval ( Encode ( str, Value: NumOfNumCh+1 :16 ) );
    str[1] := '#';
    FOR i := 2 TO NumOfNumCh DO
	IF Str[i] = ' ' THEN
	    Str[i] := '0';
    HexConv := str;

END;	{ HexConv }

{*******************************************************}
{	PStatConv Converterar PStatus till hex		}
{	ex: #XX01					}
{*******************************************************}

function PStatConv ( Pstat : int2 ): lstr5; EXTERNAL;


BEGIN	{ PStatConv }

    PStatConv := HexConv ( 4, Pstat );
    PStatConv[2] := 'X';
    PStatConv[3] := 'X';

END;	{ PStatConv }

{*******************************************************}
{	Add_Space:					}
{	Lagg till blanka till angiven Col.		}
{*******************************************************}

Procedure Add_Space ( Col : byte );

VAR
    i	: byte;

BEGIN	{ Add_Space }

    FOR i := (TextStr.LEN + 2)  TO Col DO
	Concat ( TextStr, ' ' );

END;	{ Add_Space }

{*******************************************************}
{	Make_LogHead					}
{*******************************************************}

Procedure Make_LogHead;

VAR
    PageStr	: lstr4;
    i		: byte;

BEGIN	{ Make_LogHead }
 
    HeadStr_1 := FF;
    Concat ( HeadStr_1, 'Date: ' );
    Concat ( HeadStr_1, TmpDate );
    Concat ( HeadStr_1, ' LogFile: ' );
    Concat ( HeadStr_1, LogFileName );
    Concat ( HeadStr_1, ' BatchFile: ' );
    Concat ( HeadStr_1, BatFileName );
    FOR i := (HeadStr_1.LEN + 2)  TO 110 DO
	Concat (HeadStr_1, ' ' );
    Eval ( Encode (PageStr, Page :4 ) );
    Concat ( HeadStr_1, PageStr );
    Concat ( HeadStr_1, CR );
    Concat ( HeadStr_1, LF );
    HeadStr_2 := '';

END;	{ Make_LogHead }

{*******************************************************}
{	Log_Msg Logs ett valfritt meddelade p}	}
{	Logemedierna					}
{*******************************************************}

Procedure Log_Msg ( CONST msg : string ); EXTERNAL;

VAR
    i		: int2;

BEGIN	{ Log_Msg }

    Put_Times;			{ Time + forbrukad testTime till TextStr }
    Add_Space (30);
    Concat( TextStr, msg);
    Log;

END;	{ Log_Msg }

{*******************************************************}
{	DiskLog:					}
{	skriver en Log-Row p} disk.			}
{*******************************************************}

Procedure DiskLog ( CONST msg : string );

VAR
    str		: lstr80;

BEGIN	{ DiskLog }

    If NOT DiskErr THEN BEGIN
    	WriteLn ( LogFile, msg );		{ Skriv ut pa LogFile }
	IF LogFile.ERRS <> 0 THEN BEGIN
	    DiskErr := true;
	    IF LogFile.ERRS = 8 THEN
		str := 'Disken full, Logning forts{tter endast p} printern'
	    ELSE BEGIN
		EVAL ( ENCODE ( str, LogFile.ERRS : 1 ) );
		INSERT ( 'Err fr}n Disk, Logning forts{tter endast p} printern, Errkod: ', str, 1);
	    END;
	    Log_msg ( str );
	END;
    END; { IF }

END;	{ DiskLog }

{*******************************************************}
{	Log.						}
{	Logs en textStr.				}
{*******************************************************}

Procedure Log;

CONST
    BusyTimeOut		= 5;		{ Sekunder }

VAR
    newpage		: boolean;	{ om det skall bytas Page }

BEGIN	{ Log }

    IF (TmpDate <> OldDate) OR (LineNr > LastLine) THEN BEGIN
    	OldDate := TmpDate;
	Page := Page + 1;
	Make_LogHead;			{ Fyll HeadStr_1 och HeadStr_2 }
	LineNr := 0;
    END; { IF }

    newpage := false;

    IF LineNr = 0 THEN BEGIN
        newpage := true;
        LineNr := 4;
	DiskLog ( HeadStr_1 );
	IF CommHead THEN BEGIN
	    LineNr := 5;
	    DiskLog( HeadStr_2 );
	END;
	DiskLog ( Line );
    END;
    DiskLog ( TextStr );		{ Skriv ut pa LogFile }

    IF newpage THEN BEGIN
    	IF PrintOn THEN
	    Check_Printer ( PrintLn ( HeadStr_1 ) );
	IF CommHead AND PrintOn THEN
	    Check_Printer ( PrintLn ( HeadStr_2 ) );
    	IF PrintOn THEN
	    Check_Printer ( PrintLn ( Line ) );
    END;
    IF PrintOn THEN
	Check_Printer ( PrintLn ( TextStr ) );	{ Skriv ut pa printer }

    LineNr := LineNr + 1;

END;	{ Log }

{*******************************************************}
{	Log-command.					}
{*******************************************************}

Procedure Log ( Rowparts : linepart_t;
		 number	  : int2	); EXTERNAL;

VAR
    i		: int2;

BEGIN	{ Log }

    Put_Times;			{ Time + forbrukad testTime till TextStr }
    Add_Space (30);
    FOR i := 2 TO number DO BEGIN
        CASE Rowparts[i].typ OF
	    variable : Concat_Value ( Get_VarValue (Rowparts[i].txt), FALSE );
	    value   : Concat_Value ( Rowparts[i].value, FALSE );
	    Strng    : Concat ( TextStr, Rowparts[i].txt );
	END; { CASE }
    END; { FOR }
    Log;

END;	{ Log }

{*******************************************************}
{	Init_List.					}
{	Initiera Listn f|re syntaxkontroll		}
{*******************************************************}

Procedure Init_List; EXTERNAL;

BEGIN	{ Init_List }

    TmpDate := Date;
    OldDate := TmpDate;		{ s} Date inte hinner {ndra p} sig }
    Page := 1;
    Make_LogHead;			{ Fyll HeadStr_1 och HeadStr_2 }
    Delete(Headstr_1,1,1);		{ ta bort ff f|re f|rsta utskrift }
    LineNr := 0;
    CommHead := true;			{ skriv alla rubrikrows vid sidbyte }

END;	{ Init_List }

{*******************************************************}
{	Screen_Log.					}
{	Logs p} bildsk{rmen den commandRow		}
{	som intepreteras.				}
{*******************************************************}

Procedure Screen_Log ( Rowparts : linepart_t;
			number    : int2;
			Row    : Cmd_ptr); EXTERNAL;

VAR
	i, x	: int2;

BEGIN	{ Screen_Log }

    TextStr := Rowparts[1].txt;		{ command }
    Add_Space (13);			{ parametrar i kol 13 }
    FOR i := 2 TO number DO BEGIN
	IF i > 2 THEN Concat ( TextStr, ', ' );
	CASE Rowparts[i].typ OF
	    variable :
	    	IF ( i = 2 ) AND ( Row^.Cmd = kfor ) THEN
		    Concat ( TextStr, Rowparts[i].txt )
		ELSE
		    Concat_Value( Get_VarValue(Rowparts[i].txt), false );
	    strng : BEGIN
	    	Concat ( TextStr, '"' );
	    	Concat ( TextStr, Rowparts[i].txt );
	    	Concat ( TextStr, '"' );
	    END;
	    value : Concat ( TextStr, Rowparts[i].txt );
	END; { CASE }
    END; { FOR }

    x := Positn('''', Row^.txt, 1);	   { kolla om det finns kommentar }
    IF x <> 0 THEN BEGIN
    	Concat ( TextStr, ' ' );	   { minst en blank f|re kommentar }
	Add_Space (40);			   { kommentarer i kol 40 }
	FOR i := x TO ord(Row^.txt.LEN) DO
	    IF TextStr.LEN < 79 THEN	   { max 79 Ch/Row }
	    	CONCAT (TextStr, Row^.txt[i]);
    END;

    ScrollWriteLn ( TextStr );		{ skriv ut Row }

END;	{ Screen_Log }

{*******************************************************}
{	Statistic.					}
{	Logs the statistics.				}
{*******************************************************}

Procedure Statistic; EXTERNAL;

BEGIN	{ Statistic }

    TmpDate := Date;		{ Eftersom Log anv{nder sig av TmpDate }
    IF LineNr <> 0 THEN
	LineNr := 100;		{ forcera sidbyte }
    CommHead := false;		{ inget CommHead }
    LineNr := 100;
    CommHead := true;		{ }terst{ll }

END;	{ Statistic }
