{$nomain}
{***************************************************************}
{	This module contains operator command handling routines.}
{***************************************************************}

%Include 'global.h';
%Include 'turbodef.h';
%Include 'main.h';
%Include 'screen.h';
%Include 'kbd.h';
%Include 'setup.h';
%Include 'batch.h'; 
%Include 'inter.h';
%Include 'edit.h';
%Include 'opipret.h';

CONST
    NumLineOpCmds	= 29;		{ Number of command-strings }
    MaxOpCmdLength	= 6;		{ Longest command string }

TYPE
    OpCmd_t	= ( cf_setup, cf_batch, cf_intact, cf_exit,	{ main }
    		    cf_main, cf_return,
		    cf_inface, c_gpib, f_gpib, c_rs232, f_rs232,{ setup }
		    cf_timout, cf_battim, c_print, f_print,
		    c_fillog, f_fillog, cf_lognam, c_logtyp,
		    f_logtyp,
		    f_baud, f_parity, f_datbit, f_stpbit,	{ RS232 }
		    f_gpibadr,					{ GPIB }
		    f_logtyp_s, f_logtyp_ar, f_logtyp_sr,	{ logtyp }
		    f_logtyp_ur, f_logtyp_er, f_logtyp_e,
		    cf_batnam, cf_start, cf_wait, cf_go,	{ batch }
		    cf_step,
		    cf_edit, cf_sndone, cf_sndbuf, cf_send,	{ intact }
		    f_prevline, f_nextline,
		    cf_stop, 					{ bat+int }
		    f_cursup, f_cursdown, f_cursright,		{ edit }
		    f_cursleft, f_home, f_end, f_ins, f_del,
		    cf_getbuf, cf_savbuf, f_dellin, f_undel,
		    cf_invalid );				{ no command }

	{ In the type "OpCmd_t" the meaning of the prefix is:
	  c_  : The command is allowed only on the command line
	  f_  : The command is allowed only as a function key
	  cf_ : The command is allowed in both ways		}

    FKeyTransTab_t	= array [fk_1..fk_end, Main..Edit] of OpCmd_t;

    OpCmdTrans_t = packed RECORD
    	OpCmdString	: packed array [1..MaxOpCmdLength] of char;
    	OpCmd		: OpCmd_t;
    END;
    LineOpCmdTransTab_t	= packed array [1..NumLineOpCmds] of OpCmdTrans_t;

    ComRec_t = packed RECORD
    	mode		: packed array [Main..Edit] of boolean;
	MaxNumParm	: byte;
    END;
    ComTab_t	= array [cf_setup..f_undel] of ComRec_t;

CONST
	{ Table for translating function keys to commands }

    FKeyTransTab	= FKeyTransTab_t (
{fk_1}	( cf_setup,   cf_main,    cf_main,    cf_main,    cf_main,
	  cf_main,    cf_invalid, cf_main,    cf_invalid, cf_main     ),
{fk_2}	( cf_batch,   cf_inface,   f_gpibadr,  f_baud,     f_logtyp_s,
	  cf_batnam,  cf_invalid, cf_edit,    cf_invalid, cf_getbuf   ),
{fk_3}	( cf_intact,   f_gpib,    cf_invalid,  f_parity,   f_logtyp_ar,
	  cf_start,   cf_invalid, cf_sndone,  cf_invalid, cf_savbuf   ),
{fk_4}	( cf_invalid,  f_rs232,   cf_invalid,  f_datbit,   f_logtyp_sr,
	  cf_go,      cf_wait,    cf_sndbuf,  cf_invalid,  f_dellin   ),
{fk_5}	( cf_invalid, cf_timout,  cf_invalid,  f_stpbit,   f_logtyp_ur,
	  cf_step,    cf_step,    cf_send,    cf_invalid,  f_undel    ),
{fk_6}	( cf_invalid, cf_battim,  cf_invalid, cf_invalid,  f_logtyp_er,
	  cf_invalid, cf_stop,    cf_invalid, cf_stop,    cf_invalid  ),
{fk_7}	( cf_invalid,  f_print,   cf_invalid, cf_invalid,  f_logtyp_e,
	  cf_invalid, cf_invalid, cf_invalid, cf_invalid, cf_invalid  ),
{fk_8}	( cf_invalid,  f_fillog,  cf_invalid, cf_invalid, cf_invalid,
	  cf_invalid, cf_invalid, cf_invalid, cf_invalid, cf_invalid  ),
{fk_9}	( cf_invalid, cf_lognam,  cf_invalid, cf_invalid, cf_invalid,
	  cf_invalid, cf_invalid, cf_invalid, cf_invalid, cf_invalid  ),
{fk_10}	( cf_exit,     f_logtyp,   cf_return,  cf_return,  cf_return,
	  cf_invalid, cf_invalid, cf_invalid, cf_invalid, cf_return   ),
{fk_up}	( cf_invalid, cf_invalid, cf_invalid, cf_invalid, cf_invalid,
	  cf_invalid, cf_invalid,  f_prevline, cf_invalid, f_cursup    ),
{fk_down}( cf_invalid, cf_invalid, cf_invalid, cf_invalid, cf_invalid,
	  cf_invalid, cf_invalid,  f_nextline, cf_invalid, f_cursdown ),
{fk_left}( cf_invalid, cf_invalid, cf_invalid, cf_invalid, cf_invalid,
	  cf_invalid, cf_invalid, cf_invalid, cf_invalid,  f_cursleft ),
{fk_right}(cf_invalid, cf_invalid, cf_invalid, cf_invalid, cf_invalid,
	  cf_invalid, cf_invalid, cf_invalid, cf_invalid,  f_cursright ),
{fk_ins}( cf_invalid, cf_invalid, cf_invalid, cf_invalid, cf_invalid,
	  cf_invalid, cf_invalid, cf_invalid, cf_invalid,  f_ins ),
{fk_del}(cf_invalid, cf_invalid, cf_invalid, cf_invalid, cf_invalid,
	  cf_invalid, cf_invalid, cf_invalid, cf_invalid,  f_del ),
{fk_home}( cf_invalid, cf_invalid, cf_invalid, cf_invalid, cf_invalid,
	  cf_invalid, cf_invalid, cf_invalid, cf_invalid,  f_home ),
{fk_end}( cf_invalid, cf_invalid, cf_invalid, cf_invalid, cf_invalid,
	  cf_invalid, cf_invalid, cf_invalid, cf_invalid,  f_end ) );


	  { Table for translating commands given on the command line }

    LineOpCmdTransTab	= LineOpCmdTransTab_t (
    	('BATCH ', cf_batch  ),
    	('BATNAM', cf_batnam ),
    	('BATTIM', cf_battim ),
    	('DELLIN', f_dellin  ),
    	('EDIT  ', cf_edit   ),
    	('EXIT  ', cf_exit   ),
    	('FILLOG', c_fillog  ),
    	('GETBUF', cf_getbuf ),
    	('GO    ', cf_go     ),
    	('GPIB  ', c_gpib    ),
    	('INFACE', cf_inface ),
    	('INTACT', cf_intact ),
    	('LOGNAM', cf_lognam ),
    	('LOGTYP', c_logtyp  ),
    	('MAIN  ', cf_main   ),
    	('PRINT ', c_print   ),
    	('RETURN', cf_return ),
    	('RS232 ', c_rs232   ),
    	('SAVBUF', cf_savbuf ),
    	('SEND  ', cf_send   ),
    	('SETUP ', cf_setup  ),
    	('SNDBUF', cf_sndbuf ),
    	('SNDONE', cf_sndone ),
    	('START ', cf_start  ),
    	('STEP  ', cf_step   ),
    	('STOP  ', cf_stop   ),
    	('TIMOUT', cf_timout ),
    	('UNDEL ', f_undel   ),
    	('WAIT  ', cf_wait   ) );


	{ Table for checking if the command is valid }

    TR = true;
    fa = false;

    ComTab	= ComTab_t (

    	  { Mai Set GPI RS2 Log BId BSe IId ISe Edi }

	( ( TR, fa, TR, TR, TR, TR, fa, TR, fa, fa ), 0 ),  { cf_setup }
	( ( TR, TR, TR, TR, TR, fa, fa, TR, fa, fa ), 0 ),  { cf_batch }
	( ( TR, TR, TR, TR, TR, TR, fa, fa, fa, fa ), 0 ),  { cf_intact }
	( ( TR, TR, TR, TR, TR, TR, TR, TR, TR, fa ), 0 ),  { cf_exit }
	( ( fa, TR, TR, TR, TR, TR, fa, TR, fa, TR ), 0 ),  { cf_main }
	( ( fa, fa, TR, TR, TR, fa, fa, fa, fa, TR ), 0 ),  { cf_return }
	( ( TR, TR, TR, TR, TR, TR, fa, TR, fa, fa ), 0 ),  { cf_inface }
	( ( TR, TR, TR, TR, TR, TR, fa, TR, fa, fa ), 2 ),  { c_gpib }
	( ( fa, TR, fa, fa, fa, fa, fa, fa, fa, fa ), 0 ),  { f_gpib }
	( ( TR, TR, TR, TR, TR, TR, fa, TR, fa, fa ), 2 ),  { c_rs232 }
	( ( fa, TR, fa, fa, fa, fa, fa, fa, fa, fa ), 0 ),  { f_rs232 }
	( ( TR, TR, TR, TR, TR, TR, TR, TR, TR, fa ), 1 ),  { cf_timout }
	( ( TR, TR, TR, TR, TR, TR, TR, TR, TR, fa ), 1 ),  { cf_battim }
	( ( TR, TR, TR, TR, TR, TR, TR, TR, TR, fa ), 1 ),  { c_print }
	( ( fa, TR, fa, fa, fa, fa, fa, fa, fa, fa ), 0 ),  { f_print }
	( ( TR, TR, TR, TR, TR, TR, TR, TR, TR, fa ), 1 ),  { c_fillog }
	( ( fa, TR, fa, fa, fa, fa, fa, fa, fa, fa ), 0 ),  { f_fillog }
	( ( TR, TR, TR, TR, TR, TR, fa, TR, fa, fa ), 1 ),  { cf_lognam }
	( ( TR, TR, TR, TR, TR, TR, TR, TR, TR, fa ), 6 ),  { c_logtyp }
	( ( fa, TR, fa, fa, fa, fa, fa, fa, fa, fa ), 0 ),  { f_logtyp }
	( ( fa, fa, fa, TR, fa, fa, fa, fa, fa, fa ), 0 ),  { f_baud }
	( ( fa, fa, fa, TR, fa, fa, fa, fa, fa, fa ), 0 ),  { f_parity }
	( ( fa, fa, fa, TR, fa, fa, fa, fa, fa, fa ), 0 ),  { f_datbit }
	( ( fa, fa, fa, TR, fa, fa, fa, fa, fa, fa ), 0 ),  { f_stpbit }
	( ( fa, fa, TR, fa, fa, fa, fa, fa, fa, fa ), 0 ),  { f_gpibadr }
	( ( fa, fa, fa, fa, TR, fa, fa, fa, fa, fa ), 0 ),  { f_logtyp_s }
	( ( fa, fa, fa, fa, TR, fa, fa, fa, fa, fa ), 0 ),  { f_logtyp_ar }
	( ( fa, fa, fa, fa, TR, fa, fa, fa, fa, fa ), 0 ),  { f_logtyp_sr }
	( ( fa, fa, fa, fa, TR, fa, fa, fa, fa, fa ), 0 ),  { f_logtyp_ur }
	( ( fa, fa, fa, fa, TR, fa, fa, fa, fa, fa ), 0 ),  { f_logtyp_er }
	( ( fa, fa, fa, fa, TR, fa, fa, fa, fa, fa ), 0 ),  { f_logtyp_e }
	( ( TR, TR, TR, TR, TR, TR, fa, TR, fa, fa ), 1 ),  { cf_batnam }
	( ( fa, fa, fa, fa, fa, TR, fa, fa, fa, fa ), 0 ),  { cf_start }
	( ( fa, fa, fa, fa, fa, fa, TR, fa, fa, fa ), 0 ),  { cf_wait }
	( ( fa, fa, fa, fa, fa, TR, fa, fa, fa, fa ), 0 ),  { cf_go }
	( ( fa, fa, fa, fa, fa, TR, TR, fa, fa, fa ), 0 ),  { cf_step }
	( ( TR, TR, TR, TR, TR, TR, fa, TR, fa, fa ), 0 ),  { cf_edit }
	( ( fa, fa, fa, fa, fa, fa, fa, TR, fa, fa ), 0 ),  { cf_sndone }
	( ( fa, fa, fa, fa, fa, fa, fa, TR, fa, fa ), 0 ),  { cf_sndbuf }
	( ( fa, fa, fa, fa, fa, fa, fa, TR, fa, fa ), 0 ),  { cf_send }
	( ( fa, fa, fa, fa, fa, fa, fa, TR, fa, fa ), 0 ),  { f_prevline }
	( ( fa, fa, fa, fa, fa, fa, fa, TR, fa, fa ), 0 ),  { f_nextline }
	( ( fa, fa, fa, fa, fa, fa, TR, fa, TR, fa ), 0 ),  { cf_stop }
	( ( fa, fa, fa, fa, fa, fa, fa, fa, fa, TR ), 0 ),  { f_cursup }
	( ( fa, fa, fa, fa, fa, fa, fa, fa, fa, TR ), 0 ),  { f_cursdown }
	( ( fa, fa, fa, fa, fa, fa, fa, fa, fa, TR ), 0 ),  { f_cursright }
	( ( fa, fa, fa, fa, fa, fa, fa, fa, fa, TR ), 0 ),  { f_cursleft }
	( ( fa, fa, fa, fa, fa, fa, fa, fa, fa, TR ), 0 ),  { f_home }
	( ( fa, fa, fa, fa, fa, fa, fa, fa, fa, TR ), 0 ),  { f_end }
	( ( fa, fa, fa, fa, fa, fa, fa, fa, fa, TR ), 0 ),  { f_ins }
	( ( fa, fa, fa, fa, fa, fa, fa, fa, fa, TR ), 0 ),  { f_del }
	( ( TR, TR, TR, TR, TR, TR, fa, TR, fa, TR ), 1 ),  { cf_getbuf }
	( ( TR, TR, TR, TR, TR, TR, fa, TR, fa, TR ), 1 ),  { cf_savbuf }
	( ( fa, fa, fa, fa, fa, fa, fa, fa, fa, TR ), 0 ),  { f_dellin }
	( ( fa, fa, fa, fa, fa, fa, fa, fa, fa, TR ), 0 ) );{ f_undel }


VAR	{$own='opipret'}

    Parm_asked_for	: boolean;	{ Has the parameter been asked for? }
    OpCmd		: OpCmd_t;	{ Last command }

{***************************************************************}
{	Init_OpIpret:						}
{	Initiates OpIpret routines.				}
{***************************************************************}

Procedure Init_OpIpret;				{ Global }

BEGIN	{ Init_OpIpret }

    OpInput_exp := Cmd;			{ Command expected }
    OpLineParm.Num := 0;		{ No parameters so far }
    Parm_asked_for := false;

END;	{ Init_OpIpret }

{***************************************************************}
{	GetLinePart:						}
{	Picks a part of operator input line.			}
{***************************************************************}

Function GetLinePart(VAR line: lstr80 ) : lstr80;

VAR
    p : int4;

BEGIN	{ GetLinePart }

    IF Length(line) = 0 THEN BEGIN
    	GetLinePart := '';
    END ELSE BEGIN
    	WHILE pos(' ',line) = 1 DO		{ delete leading spaces }
	    DeleteStr(line,1,1);
	p := pos(' ',line);
	IF p <> 0 THEN BEGIN
	    GetLinePart := Copy(line,1,p-1);
	    DeleteStr(line,1,p);
	END ELSE BEGIN
	    GetLinePart := line;
	    line := '';
	END;
    END;
END;	{ GetLinePart }

{***************************************************************}
{	TranslateElement:					}
{	Converts a string to a command.				}
{***************************************************************}

Function TranslateElement (LinePart : lstr6) : OpCmd_t;

VAR
    i		: byte;
    Found	: boolean;
    part	: packed array [1..MaxOpCmdLength] of char;

BEGIN	{ TranslateElement }

    part := '      ';
    FOR i := 1 to length (linepart) DO
    	part[i] := linepart[i];
    i := 0;
    Found := false;
    REPEAT
	i := i + 1;
    	IF LineOpCmdTransTab[i].OpCmdString = part THEN
	    Found := true;
    UNTIL Found or ( i >= NumLineOpCmds );
    IF Found THEN
    	TranslateElement := LineOpCmdTransTab[i].OpCmd
    ELSE
    	TranslateElement := cf_invalid;

END;	{ TranslateElement }

{***************************************************************}
{	Check_parm:						}
{	Calls the correct parameter checking routine.		}
{***************************************************************}

Procedure Check_parm(	    OpCmd	: OpCmd_t;
			    ParmI	: byte;
			VAR Parm_ok	: boolean;
			VAR All_parms	: boolean;
			VAR ParmQ	: lstr20;
			VAR DefVal	: lstr20   );

BEGIN	{  Check_parm }

    Parm_ok := true;
    All_parms := false;
    ParmQ := '';
    DefVal := '';
    CASE OpCmd of
	c_gpib	  : Check_c_gpib_parm	( ParmI, Parm_ok, 
 				          All_parms, ParmQ, DefVal );
	c_rs232	  : Check_c_rs232_parm	( ParmI, Parm_ok, 
					  All_parms, ParmQ, DefVal );
	cf_timout : Check_cf_timout_parm( ParmI, Parm_ok, 
					  All_parms, ParmQ, DefVal );
	cf_battim : Check_cf_battim_parm(        Parm_ok, 
					  All_parms, ParmQ, DefVal );
	c_print	  : Check_c_print_parm	( ParmI, Parm_ok, 
					  All_parms, ParmQ, DefVal );
	c_fillog  : Check_c_fillog_parm	( ParmI, Parm_ok, 
					  All_parms, ParmQ, DefVal );
	cf_lognam : Check_cf_lognam_parm( ParmI, Parm_ok, 
					  All_parms, ParmQ, DefVal );
	c_logtyp  : Check_c_logtyp_parm	( ParmI, Parm_ok, 
					  All_parms, ParmQ, DefVal );
	cf_batnam : Check_cf_batnam_parm( ParmI, Parm_ok, 
					  All_parms, ParmQ, DefVal );
	cf_getbuf : Check_CF_GetBuf_parm( Parm_ok, All_parms, ParmQ,
					  DefVal );
	cf_savbuf : Check_CF_SavBuf_parm( Parm_ok, All_parms, ParmQ,
					  DefVal );
	OTHERWISE
{D}	    Write('Command not supposed to have parameters: ',ord(OpCmd));
    END;

END;	{ Check_parm }

{***************************************************************}
{	Perform_OpCmd:						}
{	Calls the correct operator command routine.		}
{***************************************************************}

Procedure Perform_OpCmd (	    OpCmd	: OpCmd_t;
				VAR OpErr	: OpErr_t );

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{	Cf_Return_p:						}
{	The return command.					}
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

Procedure CF_Return_p;

BEGIN	{ CF_Return_p }

    CASE Menu OF 
    	SetupMenu	: CF_Setup_p;
	EditMenu	: CF_IntAct_p;
    END; { CASE }

END;	{ CF_Return_p }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{	SelStop:						}
{	Batch or interactive-Stop?				}
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

Procedure SelStop;

BEGIN	{ SelStop }

    IF Mode = BatchSend THEN CF_BatchStop_p
    ELSE CF_IntActStop_p;	 	{ Mode = InteractiveSend }

END;	{ SelStop }

BEGIN	{ Perform_OpCmd }

{D}    CposWrite(20, 35,'                        ');

    OpErr := OpCmdOk;
    CASE OpCmd of
    	cf_setup	: CF_Setup_p;
	cf_batch	: CF_Batch_p;
	cf_intact	: CF_Intact_p;
	cf_exit		: Menu := DOS;
	cf_main		: CF_Main_p;
	cf_return	: CF_Return_p;
	cf_inface	: cf_inface_p;
	c_gpib		: c_gpib_p;
	f_gpib		: f_gpib_p;
	c_rs232		: c_rs232_p;
	f_rs232		: f_rs232_p;
	cf_timout	: cf_timout_p;
	cf_battim	: cf_battim_p;
	c_print		: c_print_p;
	f_print		: f_print_p;
	c_fillog	: c_fillog_p;
	f_fillog	: f_fillog_p;
	cf_lognam	: cf_lognam_p;
	c_logtyp	: c_logtyp_p;
	f_logtyp	: f_logtyp_p;
	f_baud		: f_baud_p;
	f_parity	: f_parity_p;
	f_datbit	: f_datbit_p;
	f_stpbit	: f_stpbit_p;
	f_gpibadr	: f_gpibadr_p;
	f_logtyp_s	: f_logtyp_s_p;
	f_logtyp_ar	: f_logtyp_ar_p;
	f_logtyp_sr	: f_logtyp_sr_p;
	f_logtyp_ur	: f_logtyp_ur_p;
	f_logtyp_er	: f_logtyp_er_p;
	f_logtyp_e	: f_logtyp_e_p;
	cf_batnam	: cf_batnam_p;
	cf_start	: cf_start_p;
	cf_wait		: cf_wait_p;
	cf_go		: cf_go_p;
	cf_step		: cf_step_p;
	cf_edit		: cf_edit_p;
	cf_sndone	: cf_sndone_p;
	cf_sndbuf	: cf_sndbuf_p;
	cf_send		: cf_send_p;
	f_prevline	: f_prevline_p;
	f_nextline	: f_nextline_p;
	cf_stop		: SelStop;
	f_cursup	: f_cursup_p;
	f_cursdown	: f_cursdown_p;
	f_cursright	: f_cursright_p;
	f_cursleft	: f_cursleft_p;
	f_home		: f_home_p;
	f_end		: f_end_p;
	f_ins		: f_ins_p;
	f_del		: f_del_p;
	cf_getbuf	: cf_getbuf_p;
	cf_savbuf	: cf_savbuf_p;
	f_dellin	: f_dellin_p;
	f_undel		: f_undel_p;
{D}	OTHERWISE	  Write('Unknown Operator command: ',ord(OpCmd));
    END;

END;	{ Perform_OpCmd }

{***************************************************************}
{	Unpack_OpCmd:						}
{	Unpacks a command or function key entered by the 	}
{	operator.						}
{***************************************************************}

Procedure Unpack_OpCmd;			{ Global }

VAR
    LinePart		: lstr80;	{ A part of the OpLine }
    OpErr		: OpErr_t;
    Perform_is_ok	: boolean;	{ Is it ok to perform the command? }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{	Get_Cmd:						}
{	Feches the command from the command line.		}
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

Procedure Get_Cmd;

BEGIN	{ Get_Cmd }

    LinePart := GetLinePart(OpLine);
    IF length(LinePart) > MaxOpCmdLength THEN BEGIN
    	OpCmdErrHandler( InvalidCmd);
    END ELSE BEGIN
	OpCmd := TranslateElement (LinePart);
	IF OpCmd <> cf_invalid THEN BEGIN
    	    IF ComTab[OpCmd].mode[Mode] THEN BEGIN
		OpLineParm.Num := 0;
	    END ELSE BEGIN
	    	OpCmdErrHandler( InvalidMode);
	    END;
	END ELSE BEGIN
	    OpCmdErrHandler( InvalidCmd);
	END;
    END;

END;	{ Get_Cmd }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{	Get_OpCmdParm:						}
{	Feches the parameters (from the command line).		}
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

Procedure Get_OpCmdParm;

BEGIN	{ Get_OpCmdParm }

    LinePart := GetLinePart( OpLine );	{ Get a parameter from command line }
    IF OpLineParm.Num = 6 THEN BEGIN
    	OpCmdErrHandler( TooManyParms );
    END ELSE BEGIN
    	OpLineParm.Txt[OpLineParm.Num + 1] := LinePart;	{ Put it in parm-area}
	OpLineParm.Num := OpLineParm.Num + 1;
    END;

END;	{ Get_OpCmdParm }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{	ParmInput:						}
{	Takes care of operator command parameters.		}
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

Procedure ParmInput;				{ Global }

VAR
    ParmI	   : byte;	{ Index in parameter area }
    Parm_ok	   : boolean;	{ Parameter is correct? }
    All_parms	   : boolean;	{ All parameters needed are received? }
    ParmQ	   : lstr20;	{ The parameter question }
    DefVal	   : lstr20;	{ In case there is a default value }
    Parm_exist	   : boolean;	{ Another parameter to check? }


Function Parm_exist : boolean;	{ Was there a parm? }

BEGIN	{ Parm_exist }

    Parm_exist := OpLineParm[ParmI] <> '';

END;	{ Parm_exist }

BEGIN	{ Parminput }

    Parm_asked_for := 0;				{ Every new command }

    IF Parm_asked_for = 0 THEN ParmI := 1;	{ Start parmcheck with last }
    ELSE ParmI := Parm_asked_for;		{ requested parameter	    }
    REPEAT
        Check_parm( OpCmd, ParmI, Parm_ok, All_parms, ParmQ, DefVal );
        IF Parm_ok THEN BEGIN
            IF All_parms THEN BEGIN
                Perform_is_ok := true;
            END ELSE { IF NOT All_parms THEN } BEGIN
                IF Parm_exist THEN BEGIN
		    ParmI := ParmI + 1;			{ Look for next parm }
                END ELSE { IF NOT Parm_exist } THEN BEGIN
                    IF Parm_asked_for < ParmI THEN BEGIN
		    	Prompter := ParmQ + DefVal + ':'; { Ask for parm }
			DisplayPrompt;
                        Parm_asked_for := ParmI;
                    END ELSE { IF Parm_asked_for >= ParmI THEN } BEGIN
                        IF DefVal <> '' THEN BEGIN
                            OpLineParm[ParmI] := DefVal;{ Assign default val }
                        END ELSE { IF DefVal = '' THEN } BEGIN
                            OpCmdErrHandler( Too_few_parms );
                        END;
                    END;
		END;
            END;
        END ELSE { IF NOT Parm_ok THEN } BEGIN
            OpCmdErrHandler( Invalid_parm );
	END;
    UNTIL (Perform_is_ok) OR (NOT Parm_exist) OR (NOT Parm_ok)

END;	{ Parminput }

BEGIN	{ Unpack_OpCmd }

    OpInput_ok := true;
    Perform_is_ok  := false;
    IF fkey <> fk_other THEN BEGIN			{ Function key }
    	IF FKeyTransTab[fkey,mode] <> cf_invalid THEN
	    OpCmd := FKeyTransTab[fkey,mode];
	ELSE OpCmdErrHandler( InvalidFkey );
    END ELSE BEGIN					{ CR }
	IF OpInput_exp = Cmd THEN Get_Cmd;
	WHILE OpInput_ok AND ( length( OpLine ) <> 0 ) DO
	    Get_OpCmdParm;
	IF OpInput_ok THEN IF (OpLineParm.Num > ComTab[OpCmd].MaxNumParm) THEN
	    OpCmdErrHandler( TooManyParms );	{ Too many parameters }
    END; { ELSE }

    IF OpInput_ok THEN BEGIN
    	IF ComTab[Cmd].MaxParmNum > 0 THEN Parminput	{ Command uses parms }
	ELSE Perform_is_ok := true;
    END;

    IF Perform_is_ok THEN BEGIN
    	Perform_OpCmd( OpCmd );			{ Perform the command }
	OpInput_exp := Cmd;
    END ELSE { IF NOT Perform_is_ok THEN } BEGIN
	OpInput_exp := Parm;
    END;

END;	{ Unpack_OpCmd }
