{$nomain}
{***************************************************************}
{	This module contains screen handling routines.		}
{***************************************************************}

%Include 'global.h';
%Include 'turbodef.h';
%Include 'dosbios.h';
%Include 'screen.h';
%Include 'kbd.h';

CONST

    Line = '--------------------------------------------------------------------------------';
    LineRow1	= 3;
    LineRow2	= 22;
    ErrRow	= 23;
    OpCmdRow	= 24;
    FKeyRow	= 25;

{***************************************************************}
{	PrintableCh:						}
{	Checks if a character is printable.			}
{***************************************************************}

Function PrintableCh;

BEGIN	{ PrintableCh }

    PrintableCh := (ch >= #32) AND (ch <= #254) ) AND (ch <> #127);

END;	{ PrintableCh }

{***************************************************************}
{	ClrWindow:						}
{	Clears the screen from row 4 to 21			}
{***************************************************************}

Procedure ClrWindow;

VAR
    regs	: regs_t;

BEGIN	{ ClrWindow }

    regs.b.dh := 3;			{ Row 4 }
    regs.b.dl := 0;			{ Col 1 }
    regs.b.bh := 0;			{ Page 0 }
    regs.b.ah := 2;			{ Set cursor position }
    p_int86( 16, regs, regs );		{ Interrupt Video_Io }

    regs.b.bh := 0;			{ Page 0 }
    regs.b.bl := 7;			{ Attribute }
    regs.w.cx := 1440;			{ 18 rows in characters }
    regs.b.al := ord( ' ' );		{ Blank }
    regs.b.ah := 9;			{ Write char/attr at current pos }
    p_int86( 16, regs, regs );		{ Interrupt Video_Io }

END;	{ ClrWindow }

{***************************************************************}
{	CursOn:							}
{	Turns on the cursor					}
{***************************************************************}

Procedure CursOn;

VAR
    regs	: regs_t;

BEGIN	{ CursOn }

    regs.b.ch := 7;			{ Start line }
    regs.b.cl := 8;			{ End line }
    regs.b.ah := 1;			{ Set cursor type }
    p_int86( 16, regs, regs );		{ Interrupt Video_Io }

END;	{ CursOn }


{***************************************************************}
{	CursOff:						}
{	Turns off the cursor					}
{***************************************************************}

Procedure CursOff;

VAR
    regs	: regs_t;

BEGIN	{ CursOff }

    regs.b.ch := 32;			{ Start line }
    regs.b.cl := 0;			{ End line }
    regs.b.ah := 1;			{ Set cursor type }
    p_int86( 16, regs, regs );		{ Interrupt Video_Io }

END;	{ CursOff }

{***************************************************************}
{	Scroll:							}
{***************************************************************}

Procedure Scroll;				{ Global }

VAR
    regs	: regs_t;

BEGIN	{ Scroll }

    regs.b.bh := 7;			{ Attribute }
    regs.b.ch := UpperRow - 1;		{ Start pos row active page }
    regs.b.cl := UpperCol - 1;		{ Start pos col active page }
    regs.b.dh := LowerRow - 1;		{ End pos row active page }
    regs.b.dl := LowerCol - 1;		{ End pos col active page }
    regs.b.al := NumLines;		{ Number of lines to scroll }
    regs.b.ah := Direction;		{ Scroll active page up/down }
    p_int86( 16, regs, regs );		{ Interrupt Video_Io }

END;	{ Scroll }

{***************************************************************}
{	WriteStdPic:						}
{	Writes what is always on the screen.			}
{***************************************************************}

Procedure WriteStdPic;				{ Global }

VAR
    Row	: byte;
    Col	: byte;

BEGIN	{ WriteStdPic }

    GotoXY( 1, 1 );
    Write('LOG:               BATCH:               EDIT:'); Break( Output );

    GotoXY( 1, LineRow1 );
    Write( Line ); Break( Output );

    GotoXY( 1, LineRow2 );
    Write( Line ); Break( Output );

    GotoXY( 1, FKeyRow );
    Write( '1       2       3       4       5       6       7       8       9      10' );
    Break( Output );

END;	{ WriteStdPic }

{***************************************************************}
{	WriteWindow:						}
{	Writes a window including a headline showing current	}
{	menu.		 					}
{***************************************************************}

Procedure WriteWindow;				{ Global }

BEGIN	{ WriteWindow }

    IF (Menu <> SetupMenu) OR NOT
    ( NewMode in [Setup,Setup_GPIB,Setup_RS232,Setup_LogType] ) THEN
    	ClrWindow;
    IF Mode = Edit THEN BEGIN
    	Scroll(  2, 1,  3, 80, 1, Down );
    	Scroll( 22, 1, 23, 80, 1, Up   );
    END;
    CposWrite( 3, 1, Line ); Break( Output );
    InvVid;
    CASE NewMode OF
    	Main		:     CposWrite( 3, 38, ' MAIN ' );
    	Setup		:     CposWrite( 3, 37, ' SETUP ' );
    	Setup_GPIB	:     CposWrite( 3, 34, ' SETUP / GPIB ' );
    	Setup_RS232	:     CposWrite( 3, 33, ' SETUP / RS232 ' );
    	Setup_LogType	:     CposWrite( 3, 32, ' SETUP / LOGTYPE ' );
    	BatchIdle,
	BatchSend	:     CposWrite( 3, 37, ' BATCH ' );
    	InteractiveIdle,
	InteractiveSend	:     CposWrite( 3, 34, ' INTERACTIVE ' );
    	Edit		:     CposWrite( 3, 38, ' EDIT ' );
    END; { CASE }
    NorVid;
    IF NewMode = Edit THEN BEGIN
    	Scroll(  2, 1,  3, 80, 1, Up   );
    	Scroll( 22, 1, 23, 80, 1, Down );
    END;

END;	{ WriteWindow }

{***************************************************************}
{	Clr_ErrLine						}
{	Clears error message on line 23.			}
{***************************************************************}

Procedure Clr_ErrLine;				{ Global }

BEGIN	{ Clr_ErrLine }

    IF mode = edit THEN
    	CposWrite( ErrRow, 1, Line)
    ELSE
    	ClrLine( ErrRow );

END;	{ Clr_ErrLine }

{***************************************************************}
{	ClrOpLine						}
{	Clears operator input line after prompter.		}
{***************************************************************}

Procedure ClrOpLine;				{ Global }

BEGIN	{ ClrOpLine }

    GotoXY( length(prompter) + 2, OpCmdRow );
    ClrEol;

END;	{ ClrOpLine }


{***************************************************************}
{	DisplayPrompt						}
{	Displays the prompter on the operator command line.	}
{***************************************************************}

Procedure DisplayPrompt;			{ Global }

BEGIN	{ DisplayPrompt }

    GotoXY(1, OpCmdRow);
    Write(Prompter, ' ');
    ClrEol;

END;	{ DisplayPrompt }

{***************************************************************}
{	ClrLine							}
{	Clears a line.						}
{***************************************************************}

Procedure ClrLine;				{ Global }

VAR
    OldRow	: byte;
    OldCol	: byte;

BEGIN	{ ClrLine }

    OldRow := WhereY;
    OldCol := WhereX;
    GotoXY( 1, row );
    ClrEol;
    GotoXY( OldCol, OldRow );

END;	{ ClrLine }

{***************************************************************}
{	InvVid							}
{	Inverse Video						}
{***************************************************************}

Procedure InvVid;				{ Global }

BEGIN	{ InvVid }

    Write(CHR(27),'[7m');

END;	{ InvVid }

{***************************************************************}
{	NorVid							}
{	Normal Video						}
{***************************************************************}

Procedure NorVid;				{ Global }

BEGIN	{ NorVid }

    Write(CHR(27),'[0m');

END;	{ NorVid }

{***************************************************************}
{	CposWrite:						}
{	Positions the cursor, writes a string and positions	}
{	the cursor back.					}
{***************************************************************}

Procedure CposWrite;				{ Global }

VAR
    OldRow	: byte;
    OldCol	: byte;

BEGIN	{ CposWrite }

    OldRow := WhereY;
    OldCol := WhereX;
    GotoXY( col, row );
    Write( Str );
    GotoXY( OldCol, OldRow);

END;	{ CposWrite }

{***************************************************************}
{	ScrollWrite:						}
{	Writes a string in the scroll region.			}
{***************************************************************}

Procedure ScrollWrite;				{ Global }

VAR
    TmpStr	: lstr255;
    Col		: byte;
    i		: byte;

BEGIN	{ ScrollWrite }

    Col := WhereX;
    TmpStr := Str;
    IF length(TmpStr) <> 0 THEN BEGIN		{ Replace tabs with space }
    	i := Pos( chr(9), TmpStr ); 
    	WHILE i <> 0 DO BEGIN
	    TmpStr[i] := ' ';
	    i := Pos( chr(9), TmpStr );
	END;
    END;
    IF length(TmpStr) > ( 80 - Col ) THEN	{ Too long to display? }
	TmpStr[0] := chr( 80 - Col );
    Write(TmpStr);

END;	{ ScrollWrite }

{***************************************************************}
{	DebugWrite:						}
{	Writes a string in the scroll region.			}
{	(Only for debugging.)					}
{***************************************************************}

Procedure DebugWrite;				{ Global }

BEGIN	{ DebugWrite }

    ScrollWrite( Str );

END;	{ DebugWrite }

{***************************************************************}
{	ScrollWriteLn:						}
{	Writes a string in the scroll region and scrolls	}
{	when needed.						}
{***************************************************************}

Procedure ScrollWriteLn;			{ Global }

VAR
    Row	: byte;
    Col	: byte;

BEGIN	{ ScrollWriteLn }

    Row := WhereY;
    Col := WhereX;
    ScrollWrite( Str );
    IF Row >= 21 THEN BEGIN
	Scroll( 4, 1, 21, 80, 1, Up );
	GotoXY(1, 21);
    END ELSE
	WriteLn;

END;	{ ScrollWriteLn }

{***************************************************************}
{	DebugWriteLn:						}
{	Writes a string in the scroll region and scrolls	}
{	when needed. (Only for debugging.)			}
{***************************************************************}

Procedure DebugWriteLn;				{ Global }

BEGIN	{ DebugWriteLn }

    ScrollWriteLn( Str );

END;	{ DebugWriteLn }

{***************************************************************}
{	DisplayFKeys;						}
{	Displays on line 25 which function keys can be used	}
{	in different situations.				}
{***************************************************************}

Procedure DisplayFKeys;				{ Global }

TYPE
    FkeyStr_t =array [Main..Edit, fk_1..fk_10] of packed array [1..6] of char;

CONST
    FkeyStr	= FkeyStr_t (

 	(' Setup', ' Batch', 'IntAct', '      ', '      ',	{ Main }
	 '      ', '      ', '      ', '      ', ' Exit ' ),
 	(' Main ', 'InFace', ' GPIB ', ' RS232', 'TimOut',	{ Setup }
	 'BatTim', ' Print', 'FilLog', 'LogNam', 'LogTyp' ),
	(' Main ', ' Addr ', '      ', '      ', '      ',	{ SetGPIB }
	 '      ', '      ', '      ', '      ', 'Return' ),
	(' Main ', ' Baud ', 'Parity', 'DatBit', 'StpBit',	{ SetRS232 }
	 '      ', '      ', '      ', '      ', 'Return' ),
	(' Main ', ' Send ', 'AnsRec', 'SpoRec', 'UexRec',	{ SetLogType }
	 'ErrRec', ' Error', '      ', '      ', 'Return' ),
	(' Main ', 'BatNam', ' Start', '  Go  ', ' Step ',	{ Batch Idle }
	 '      ', '      ', '      ', '      ', '      ' ),
	('      ', '      ', '      ', ' Wait ', ' Step ',	{ Batch Send }
	 ' Stop ', '      ', '      ', '      ', '      ' ),
	(' Main ', ' Edit ', 'SndOne', 'SndBuf', ' Send ',	{ IntAct Idle}
	 '      ', '      ', '      ', '      ', '      ' ),
	('      ', '      ', '      ', '      ', '      ',	{ IntAct Send}
	 ' Stop ', '      ', '      ', '      ', '      ' ),
	(' Main ', 'GetBuf', 'SavBuf', 'DelLin', ' UnDel',	{ Edit }
	 '      ', '      ', '      ', '      ', 'Return' ) );

VAR
    fkey	: fkey_t;		{ Function key }
    Col		: byte;			{ Column on screen }

BEGIN	{ DisplayFKeys }

    InvVid;						{ Inverse video }
    Col := 2;
    FOR fkey := fk_1 TO fk_9 DO BEGIN
    	CposWrite( FKeyRow, Col, fkeyStr[Mode,fkey] );	{ GotoXY ? }
	Col := Col + 8;
    END; { FOR }
    CposWrite( FKeyRow, Col, fkeyStr[Mode,fk_10] );	{ GotoXY ? }
    NorVid;						{ Normal video }

END;	{ DisplayFKeys }
