{$nomain}
{***************************************************************}
{	This module contains routines performing the setup	}
{	commands.						}
{***************************************************************}

%Include 'global.h';
%Include 'turbodef.h';
%Include 'main.h';
%Include 'error.h';
%Include 'screen.h';
%Include 'kbd.h';
%Include 'opipret.h';
%Include 'asynch.h';
%Include 'setup.h';

TYPE
    OffOnStr_t		= array [false..true] of str3;

CONST
    OffOnStr		= OffOnStr_t ( 'OFF', 'ON ' );


TYPE
    Logtype_t		= ( S, AR, SR, UR, ER, E );
    LogtypeRec_t	= RECORD
    			  Col	: byte;
			  Row	: byte;
			  str	: str10;
			  END;
    DispLtTab_t		= array [S..E] of LogtypeRec_t;

CONST
    DispLtTab	= DispLtTab_t		{ For displaying logtypes selected }
    	( ( 40, 11, '*** S  ***' ),
	  ( 40, 12, '=== AR ===' ),
    	  ( 40, 13, '+++ SR +++' ),
    	  ( 40, 14, '??? UR ???' ),
    	  ( 40, 15, ';;; ER ;;;' ),
    	  ( 40, 16, '&&& E  &&&' ) );


CONST
    aBaud	= 1;		{ Asynch Manager values }
    aParity	= 2;
    aDatabits	= 3;
    aStopbits	= 4;

TYPE
    Rs232Parm1_rec = packed RECORD
    	OpCmdParmStr	: lstr6;
    	OpCmdParmQ	: lstr20;
    	DefVal		: lstr20;
    END;
    Rs232Parm1Tab_t	= packed array [aBaud..aStopbits] of Rs232Parm1_rec;

CONST
    Rs232Parm1Tab	= Rs232Parm1Tab_t
      (	('BAUD',   'Baudrate', '9600' ),
      	('PARITY', 'Parity',   'No'   ),
    	('DATBIT', 'Databits', '8'    ),
    	('STPBIT', 'Stopbits', '1'    ) );


CONST
    a110	= 0;		{ Asynch Manager values }
    a150	= 1;
    a300	= 2;
    a600	= 3;
    a1200	= 4;
    a2400	= 5;
    a4800	= 6;
    a9600	= 7;

TYPE
    Baud_rec	= RECORD
		  Str		: str4;
		  Baud_Next	: int2;
		  END;
    Baud_tab	= array [a110..a9600] of Baud_rec;

CONST
    BaudTab	= Baud_tab
    	( ( '110 ', a150  ),
	  ( '150 ', a300  ),
	  ( '300 ', a600  ),
	  ( '600 ', a1200 ),
	  ( '1200', a2400 ),
	  ( '2400', a4800 ),
	  ( '4800', a9600 ),
	  ( '9600', a110  ) );


CONST
    aNo		= 0;		{ Asynch Manager values }
    aOdd	= 1;
    aEven	= 2;

TYPE
    ParityRec_t		= RECORD
			  Str		: str4;
			  Parity_next	: int2;	
			  END;
    ParityTab_t		= array [aNo..aEven] of ParityRec_t;

CONST
    ParityTab		= ParityTab_t	( ( 'NO  ', aOdd  ),
    					  ( 'ODD ', aEven ),
					  ( 'EVEN', aNo   ) );


CONST
    aFive	= 0;		{ Asynch Manager values }
    aSix	= 1;
    aSeven	= 2;
    aEight	= 3;

TYPE
    DatabitRec_t	= RECORD
			  Str		: str1;
			  Databit_next	: int2;	
			  END;
    DatabitTab_t	= array [aFive..aEight] of DatabitRec_t;

CONST
    DatabitTab		= DatabitTab_t	( ( '5', aSix   ),
    					  ( '6', aSeven ),
    					  ( '7', aEight ),
					  ( '8', aFive  ) );


CONST
    aOne	= 0;		{ Asynch Manager values }
    aTwo	= 1;

TYPE
    StopbitRec_t	= RECORD
			  Str		: str1;
			  Stopbit_next	: int2;	
			  END;
    StopbitTab_t	= array [aOne..aTwo] of StopbitRec_t;

CONST
    StopbitTab		= StopbitTab_t	( ( '1', 2 ),
					  ( '2', 1 ) );

TYPE
    CommInFace_t	= ( Gpib, Rs232 );
    CommInfaceStr_tab	= array [Gpib..Rs232] of str5;

CONST
    CommInfaceStrTab	= CommInfaceStr_tab ( 'GPIB ', 'RS232' );

VAR	{$own='setup'}

    GpibRet		: boolean;
    RS232Ret		: boolean;
    LogtypeRet		: boolean;
    LtTab		: array [S..E] of boolean;
    FileLog		: boolean;
    PrintLog		: boolean;
    BatTestTime		: real;
    BatTestTimeStr	: lstr8;
    AnswTimeout		: int4;
    Rs232CommParm	: int2;
    Baudrate		: int2;
    Parity		: int2;
    Databits		: int2;
    StopBits		: int2;
    GpibAdr		: int2;
    CommInFace		: CommInFace_t;

{***************************************************************}
{	Ini_Setup:						}
{	Initiation of setup variables.				}
{***************************************************************}

Procedure Ini_Setup;				{ Global }

VAR
    Logtype	: Logtype_t;

BEGIN	{ Ini_Setup }

    GpibRet		:= false;
    RS232Ret		:= false;
    LogtypeRet		:= false;
    FOR Logtype := S to E DO
    	LtTab[Logtype] := true;

    LogFileName		:= 'MTS.LOG';
    FileLog		:= true;
    PrintLog		:= true;

    BatTestTime		:= 0;
    BatTestTimeStr	:= 'Forever';

    AnswTimeout		:= 10;

    Baudrate		:= a9600;
    Parity		:= aNo;
    Databits		:= aEight;
    StopBits		:= aOne;

    GpibAdr		:= 1;

    CommInFace		:= Gpib;

END;	{ Ini_Setup }

{***************************************************************}
{	Display_Logtype:					}
{	Displays if a logtype is selected or not.		}
{***************************************************************}

Procedure Display_Logtype	( Logtype : Logtype_t );

BEGIN	{ Display_Logtype }

    GotoXY ( DispLtTab[Logtype].Col, DispLtTab[Logtype].Row );
    IF LtTab[Logtype] THEN BEGIN
    	InvVid;
	Write  ( DispLtTab[Logtype].str );
	NorVid;
    END ELSE Write  ( DispLtTab[Logtype].str );
    Break( Output );

END;	{ Display_Logtype }

{***************************************************************}
{	Display_SetupParms:					}
{	Displays the setup parameters.				}
{***************************************************************}

Procedure Display_SetupParms;				{ Global }

VAR
    Logtype	: Logtype_t;

BEGIN	{ Display_SetupParms }

    GotoXY ( 1, 5); Write( 'Communication interface : ' );

    IF NOT GpibRet THEN BEGIN		{ Not Returned from Setup/GPIB? }
    	GotoXY ( 1, 7); Write ( 'GPIB  : Address         : ' );
    END;

    IF NOT RS232Ret THEN BEGIN		{ Not Returned from Setup/RS232? }
    	GotoXY ( 1, 9); Write ( 'RS232 : Baudrate        : ' );
	GotoXY ( 1,10); Write ( '        Parity          : ' );
	GotoXY ( 1,11); Write ( '        Databits        : ' );
	GotoXY ( 1,12); Write ( '        Stopbits        : ' );
    END; { IF }

    GotoXY ( 1,14); Write ( 'Timeout (ms/answ. exp.) : ' );
    GotoXY ( 1,16); Write ( 'Batch test time         : ' );

    GotoXY (40, 5); Write ( 'Printer log  : ' );
    GotoXY (40, 6); Write ( 'File log     : ' );
    GotoXY (40, 7); Write ( 'Logfilename  : ' );

    IF NOT LogtypeRet THEN BEGIN	{ Not Returned from Setup/Logtype? }
    	GotoXY (40, 9); Write ( 'Logtype : ( ' );
	InvVid; Write ( ' ON '); NorVid; Write ( ' OFF  )' );

	FOR Logtype := S to E DO
	    Display_Logtype ( Logtype );

	NorVid;		Write ( ' Error' );
	GotoXY (51,15); Write ( 'Error Received' );
	GotoXY (51,14); Write ( 'Unexpected Receive' );
	GotoXY (51,13); Write ( 'Spontanous Receive' );
	GotoXY (51,12); Write ( 'Answer Received' );
	GotoXY (51,11); Write ( 'Send' );
    END; { IF }

    HighVideo;

    GotoXY (55, 7); Write ( LogFileName );
    GotoXY (55, 6); Write ( OffOnStr[FileLog]  );
    GotoXY (55, 5); Write ( OffOnStr[PrintLog] );

    GotoXY (27,16); Write ( BatTestTimeStr );
    GotoXY (27,14); Write ( AnswTimeout : 1 );

    IF NOT RS232Ret THEN BEGIN		{ Not Returned from Setup/RS232? }
    	GotoXY (27,12); Write ( StopBits : 1 );
	GotoXY (27,11); Write ( Databits : 1 );
	GotoXY (27,10); Write ( ParityTab[Parity].Str );
	GotoXY (27, 9); Write ( BaudTab[Baudrate].Str );
    END; { IF }

    IF NOT GpibRet THEN	BEGIN		{ Not Returned from Setup/GPIB? }
    	GotoXY (27, 7); Write ( GpibAdr : 1 );
    END;

    GotoXY (27, 5); Write ( CommInFaceStrTab[CommInFace] );
    Break( Output );
    LowVideo;

    GotoXY ( 3, OpCmdRow );

END;	{ Display_SetupParms }

{***************************************************************}
{	CF_Setup_p:						}
{	The Setup command. 					}
{***************************************************************}

Procedure CF_Setup_p;				{ Global }

BEGIN	{ CF_Setup_p }

    WriteWindow( Setup );
    GpibRet := ( Mode = Setup_GPIB );
    RS232Ret := ( Mode = Setup_RS232 );
    LogtypeRet := ( Mode = Setup_Logtype );
    Mode := Setup;
    DisplayFKeys;
    Menu := SetupMenu;
    Display_SetupParms;

END;	{ CF_Setup_p }

{***************************************************************}
{	CF_Inface_p:						}
{	The InFace command. 					}
{***************************************************************}

Procedure CF_Inface_p;				{ Global }

BEGIN	{ CF_Inface_p }

    IF CommInface = Gpib THEN BEGIN
    	CommInface := Rs232;
    ELSE BEGIN
    	CommInface := Gpib;
    END;
{ Skriv ut i |vre v{nstra h|rnet }
    IF Mode = Setup THEN BEGIN
    	HighVideo;
	CposWrite( 5, 27, CommInFaceStrTab[CommInFace] );
	LowVideo;
    END;

END;	{ CF_Inface_p }

{***************************************************************}
{	Set_GpibAdr:						}
{	Sets the address on GPIB.				}
{***************************************************************}

Procedure Set_GpibAdr;

CONST
    ISC		= 7;
    UnTalk	= '-';
    UnLsn	= '?';
    TalkBase	= 64;
    LsnBase	= 32;
VAR
    RetCode	: int2;
    Cmds	: str4;
    TalkAdr	: str1;
    LsnAdr	: str1;
    GpibAdrStr	: str2;

BEGIN	{ Set_GpibAdr }

    TalkAdr := Chr( TalkBase + GpibAdr );
    LsnAdr := Chr( LsnBase + GpibAdr );
    Cmds := UnTalk + UnLsn + TalkAdr + LsnAdr;
    RetCode := IoSend( ISC, Cmds, 4 );
    IF RetCode = OK THEN BEGIN
    	Str( GpibAdr : 2, GpibAdrStr );
    	CposWrite( 2, 6, GpibAdrStr );
	IF (Mode = Setup) OR (Mode = Setup_Gpib) THEN BEGIN
	    HighVideo;
	    CposWrite( 7, 27, GpibAdrStr );
	    LowVideo;
	END;
    END ELSE BEGIN
    	GpibErrHandler( RetCode );
    END;

END;	{ Set_GpibAdr }

{***************************************************************}
{	Check_Gpib_parm:					}
{	The parameter check for the GPIB command given on the	}
{	command line.						}
{***************************************************************}

Procedure Check_Gpib_parm;				{ Global }

VAR
    Value	: int2;
    ErrCode	: int2;

BEGIN	{ Check_Gpib_parm }

    CASE ParmI OF
    	1 : BEGIN
	    IF OpLineParm.Txt[1] = '' THEN BEGIN
		ParmQ := 'Communication parameter';
	    END ELSE BEGIN 
		Parm_ok := (OpLineParm.Txt[1] = 'ADDRESS');
	    END;
	END; { 1 }
    	2 : BEGIN
	    IF OpLineParm.Txt[2] = '' THEN BEGIN
	    	ParmQ := 'Address';
		DefVal := '1';
	    END ELSE { IF OpLineParm.Txt[2] <> '' THEN } BEGIN
		Val( OpLineParm.Txt[2], Value, ErrCode );
		IF ErrCode <> 0 THEN 
		    Parm_ok := false
		ELSE BEGIN 
		    IF (Value >= 0) AND (Value <= 30) THEN BEGIN
			GpibAdr := Value;
			All_parms := true;
		    END ELSE BEGIN
		    	Parm_ok := false;
		    END; { ELSE }
		END; { ELSE }
	    END; { ELSE }
	END; { 2 }
    END; { CASE }

END;	{ Check_Gpib_parm }

{***************************************************************}
{	C_Gpib_p:						}
{	The GPIB command given on the command line.		}
{***************************************************************}

Procedure C_Gpib_p;				{ Global }

BEGIN	{ C_Gpib_p }

    IF CommInFace = Gpib THEN Set_GpibAdr;

END;	{ C_Gpib_p }

{***************************************************************}
{	F_Gpib_p:						}
{	The GPIB command given through function key.		}
{***************************************************************}

Procedure F_Gpib_p;				{ Global }

BEGIN	{ F_Gpib_p }

    WriteWindow ( Setup_GPIB );
    Mode := Setup_GPIB;
    Scroll (  5,  1,  5, 39, 0, Up );
    Scroll (  9,  1, 16, 39, 0, Up );
    Scroll (  5, 40, 16, 79, 0, Up );
    DisplayFKeys;

END;	{ F_Gpib_p }

{***************************************************************}
{	Set_Rs232TrmOp:						}
{	Sets a transmission option on COM1.			}
{***************************************************************}

Procedure Set_Rs232TrmOp;

VAR
    RetCode	: int2;
    Value	: int2;
    Col		: byte;
    Row		: byte;
    ValStr	: lstr4;

BEGIN	{ Set_Rs232TrmOp }

    CASE Rs232CommParm OF
    	aBaud : BEGIN
	    Value := Baudrate;
	    Col := 14;
	    Row := 9;
	    ValStr := BaudTab[Baudrate].Str;
	END; { Baud }
	aParity : BEGIN
	    Value := Parity;
	    Col := 19;
	    Row := 10;
	    ValStr := ParityTab[Parity].Str;
	END; { Parity }
	aDatabits : BEGIN
	    Value := Databits;
	    Col := 24;
	    Row := 11;
	    ValStr := DatabitTab[Databits].Ch;
	END; { Databits }	
	aStopbits : BEGIN
	    Value := Stopbits;
	    Col := 26;
	    Row := 12;
	    ValStr := StopbitTab[Stopbits].Ch;
	END; { Stopbits }
    END; { CASE }

    RetCode := Setop_a1( COM1, Rs232CommParm, Value );
    IF RetCode = OK THEN BEGIN
    	CposWrite( 2, Col, ValStr );
	IF (Mode = Setup) OR (Mode = Setup_Rs232) THEN BEGIN
	    HighVideo;
	    CposWrite( Row, 27, ValStr );
	    LowVideo;
	END;
    END ELSE
    	Rs232RetCodeHandler( RetCode );

END;	{ Set_Rs232TrmOp }

{***************************************************************}
{	Check_Rs232_parm:					}
{	The parameter check for the Rs232 command given on the	}
{	command line.						}
{***************************************************************}

Procedure Check_Rs232_parm;				{ Global }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{	Get_Rs232CommParm:					}
{	Converts parameter 1 from string to internal type.	}
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

Function Get_Rs232CommParm : boolean;

VAR
    Parm1I	: int2;
    Found	: boolean;

BEGIN	{ Get_Rs232CommParm }

    Parm1I := aBaud;
    Found := false;
    REPEAT
    	IF Rs232Parm1Tab[Parm1I].OpCmdParmStr = OpLineParm.Txt[1] THEN BEGIN
	    Rs232CommParm := Parm1I;
	    Found := true;
	END;
	Parm1I := Parm1I + 1;
    UNTIL Found OR (Parm1I > aStopbits);
    Get_Rs232CommParm := Found;

END;	{ Get_Rs232CommParm }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{	Get_ParmQ:						}
{	Finds the right parameter question and default value.	}
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

Procedure Get_ParmQ;

VAR
    ParmI	: int2;
    Found	: boolean;

BEGIN	{ Get_ParmQ }

    ParmI := 0;
    Found := false;
    REPEAT
	ParmI := ParmI + 1;
    	IF Rs232Parm1Tab[ParmI].OpCmdParmStr = OpLineParm.Txt[1] THEN BEGIN
	    ParmQ := Rs232Parm1Tab[ParmI].OpCmdParmQ;
	    DefVal := Rs232Parm1Tab[ParmI].DefVal;
	    Found := true;
	END;
    UNTIL Found;

END;	{ Get_ParmQ }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{	Get_speed:						}
{	Translates speed string to program internal value.	}
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

Function Get_speed : boolean;

VAR
    BaudI	: int2;
    Found	: boolean;

BEGIN	{ Get_speed }

    BaudI := a110;
    Found := false;
    REPEAT
    	IF BaudTab[BaudI].Str = OpLineParm.Txt[2] THEN BEGIN
	    Baudrate := BaudI;
	    Found := true;
	END;
	BaudI := BaudI + 1;
    UNTIL Found OR (BaudI > a9600);
    Get_speed := Found;

END;	{ Get_speed }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{	Get_parity:						}
{	Translates parity string to program internal value.	}
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

Function Get_parity : boolean;

VAR
    ParityI	: int2;
    Found	: boolean;

BEGIN	{ Get_parity }

    ParityI := aNo;
    Found := false;
    REPEAT
    	IF ParityTab[ParityI].Str = OpLineParm.Txt[2] THEN BEGIN
	    Parity := ParityI;
	    Found := true;
	END;
	ParityI := ParityI + 1;
    UNTIL Found OR (ParityI > aEven);
    Get_parity := Found;

END;	{ Get_parity }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{	Get_Databits:						}
{	Translates databits string to program internal value.	}
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

Function Get_Databits : boolean;

VAR
    DatabitI	: int2;
    Found	: boolean;

BEGIN	{ Get_Databits }

    DatabitI := aFive;
    Found := false;
    REPEAT
    	IF DatabitTab[DatabitI].Str = OpLineParm.Txt[2] THEN BEGIN
	    Databits := DatabitI;
	    Found := true;
	END;
	DatabitI := DatabitI + 1;
    UNTIL Found OR (DatabitI > aEight);
    Get_Databits := Found;

END;	{ Get_Databits }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{	Get_Stopbits:						}
{	Translates stopbits string to program internal value.	}
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

Function Get_Stopbits : boolean;

VAR
    StopbitI	: int2;
    Found	: boolean;

BEGIN	{ Get_Stopbits }

    StopbitI := aOne;
    Found := false;
    REPEAT
    	IF StopbitTab[StopbitI].Str = OpLineParm.Txt[2] THEN BEGIN
	    Stopbits := StopbitI;
	    Found := true;
	END;
	StopbitI := StopbitI + 1;
    UNTIL Found OR (StopbitI > aTwo);
    Get_Stopbits := Found;

END;	{ Get_Stopbits }

BEGIN	{ Check_Rs232_parm }

    CASE ParmI OF
    	1 : BEGIN
	    IF OpLineParm.Txt[1] = '' THEN BEGIN
		ParmQ := 'Communication parameter';
	    END ELSE BEGIN 
		Parm_ok := Get_Rs232CommParm;
	    END;
	END; { 1 }
    	2 : BEGIN
	    IF OpLineParm.Txt[2] = '' THEN BEGIN
	    	Get_ParmQ;
	    END ELSE { IF OpLineParm.Txt[2] <> '' THEN } BEGIN
		CASE Rs232CommParm OF
		    aBaud	: Parm_ok := Get_speed;
		    aParity	: Parm_ok := Get_parity;
		    aDatabits	: Parm_ok := Get_databits;
		    aStopbits	: Parm_ok := Get_stopbits;
	        END; { CASE }
		All_parms := Parm_ok;
	    END; { ELSE }
	END; { 2 }
    END; { CASE }

END;	{ Check_Rs232_parm }

{***************************************************************}
{	C_Rs232_p:						}
{	The RS232 command given on the command line.		}
{***************************************************************}

Procedure C_Rs232_p;				{ Global }

VAR
    Value	: int2;
    Col		: byte;
    Str		: lstr4;

BEGIN	{ C_Rs232_p }

    IF CommInFace = RS232 THEN Set_Rs232TrmOp;

END;	{ C_Rs232_p }

{***************************************************************}
{	F_Rs232_p:						}
{	The RS232 command given through function key.		}
{***************************************************************}

Procedure F_Rs232_p;				{ Global }

BEGIN	{ F_Rs232_p }

    WriteWindow ( Setup_RS232 );
    Scroll (  5,  1,  7, 39, 0, Up );
    Scroll ( 14,  1, 16, 39, 0, Up );
    Scroll (  5, 40, 16, 79, 0, Up );
    Mode := Setup_RS232;
    DisplayFKeys;

END;	{ F_Rs232_p }

{***************************************************************}
{	Check_TimOut_parm:					}
{	The parameter check of the TimOut command.		}
{***************************************************************}

Procedure Check_TimOut_parm;			{ Global }

VAR
    Value	: int2;
    ErrCode	: int2;

BEGIN	{ Check_TimOut_parm }

    IF OpLineParm.Txt[1] = '' THEN BEGIN
    	ParmQ := 'Time in ms';
	DefVal := '10';
    END ELSE { IF OpLineParm.Txt[1] <> '' THEN } BEGIN
	Val( OpLineParm.Txt[1], Value, ErrCode );
	IF ErrCode <> 0 THEN 
	    Parm_ok := false
	ELSE BEGIN 
	    IF (Value >= 1) AND (Value <= 999) THEN BEGIN
		AnswTimeout := Value;
		All_parms := true;
	    END ELSE BEGIN 
	    	Parm_ok := false;
	    END; { ELSE }
	END; { ELSE }
    END; { ELSE }

END;	{ Check_TimOut_parm }

{***************************************************************}
{	CF_Timout_p:						}
{	The TimOut command.					}
{***************************************************************}

Procedure CF_Timout_p;				{ Global }

VAR
    TimeoutStr	: str3;

BEGIN	{ CF_Timout_p }

    Str( AnswTimeout : 3, TimeoutStr );
    CposWrite( 2, 29, TimeoutStr );
    IF Mode = Setup THEN BEGIN
    	HighVideo;
	CposWrite( 14, 27, TimeoutStr );
	LowVideo;
    END;

END;	{ CF_Timout_p }

{***************************************************************}
{	Check_BatTim_parm:					}
{	The parameter check of the BatTim command.		}
{***************************************************************}

Procedure Check_BatTim_parm;			{ Global }

TYPE
    TimePart_t	= ( Hours, Minutes, Seconds );

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{	TimePart_ok:						}
{	Checks the values in the time parameter.		}
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

Function TimePart_ok( TimePart : TimePart_t ) : boolean;

VAR
    PartStr	: lstr2;
    Value	: int2;
    ErrCode	: int2;
    UpVal	: byte;

BEGIN	{ TimePart_ok }

    Val( PartStr, Value, ErrCode );
    IF ErrCode <> 0 THEN 
    	TimePart_ok := false
    ELSE BEGIN 
    	IF TimePart = Hours THEN UpVal := 99
	ELSE UpVal := 59;
    	TimePart_ok := (Value >= 0) AND (Value <= UpVal) THEN BEGIN
	END; { IF }
    END; { ELSE }

END;	{ TimePart_ok }

BEGIN	{ Check_BatTim_parm }

    IF OpLineParm.Txt[1] = '' THEN BEGIN
    	ParmQ := 'Time to run batch test';
	DefVal := 'Forever';
    END ELSE { IF OpLineParm.Txt[1] <> '' THEN } BEGIN
        IF OpLineParm.Txt[2] <> 'FOREVER' THEN BEGIN
	    IF Len( OpLineParm.Txt[2] ) IN [ 2, 5, 8 ] THEN BEGIN
	    	IF TimePart_ok( Hours ) THEN BEGIN
		    IF Len( OpLineParm.Txt[2] ) IN [ 5, 8 ] THEN BEGIN
		        IF OpLineParm.Txt[2, 3] = ':' THEN BEGIN
		    	    IF TimePart_ok( Minutes ) THEN BEGIN
			    	IF Len( OpLineParm.Txt[2] ) = 8 THEN BEGIN
				    IF OpLineParm.Txt[2, 6] = ':' THEN BEGIN
					Parm_ok := TimePart_ok( Seconds );
				    END ELSE Parm_ok := false;
		                END;
		            END ELSE Parm_ok := false;
		        END ELSE Parm_ok := false;
		    END;
		END ELSE Parm_ok := false;
	    END ELSE Parm_ok := false;
	END; { ELSE }
	All_parms := Parm_ok;
    END; { ELSE }

END;	{ Check_BatTim_parm }

{***************************************************************}
{	CF_BatTim_p:						}
{	The BatTim command.					}
{***************************************************************}

Procedure CF_BatTim_p;				{ Global }

BEGIN	{ CF_BatTim_p }


END;	{ CF_BatTim_p }

{***************************************************************}
{	C_Print_p:						}
{	The Print command given on command line.		}
{***************************************************************}

Procedure C_Print_p;				{ Global }

BEGIN	{ C_Print_p }


{ {ndra enligt parameter 1 ??? }

{ toggla tillsvidare... }

    PrintLog := NOT PrintLog;			{ Toggle }

    IF mode = setup THEN BEGIN
    	HighVideo;
	CposWrite ( 5, 55, OffOnStr[PrintLog] );
	LowVideo;
    END;

    CposWrite ( 2, 68, 'PR' + OffOnStr[PrintLog] );

END;	{ C_Print_p }

{***************************************************************}
{	F_Print_p:						}
{	The Print command given through the functionkey.	}
{***************************************************************}

Procedure F_Print_p;				{ Global }

BEGIN	{ F_Print_p }

    PrintLog := NOT PrintLog;			{ Toggle }

    IF mode = setup THEN BEGIN
    	HighVideo;
	CposWrite ( 5, 55, OffOnStr[PrintLog] );
	LowVideo;
    END;

    CposWrite ( 2, 68, 'PR' + OffOnStr[PrintLog] );

END;	{ F_Print_p }

{***************************************************************}
{	C_FilLog_p:						}
{	The FilLog command given on command line.		}
{***************************************************************}

Procedure C_FilLog_p;				{ Global }

BEGIN	{ C_FilLog_p }

{ {ndra enligt parameter 1 ??? }

{ toggla tillsvidare... }

    FileLog := NOT FileLog;			{ Toggle }

    IF mode = setup THEN BEGIN
    	HighVideo;
	CposWrite ( 6, 55, OffOnStr[FileLog] );
	LowVideo;
    END;

    CposWrite ( 2, 74, 'LOG' + OffOnStr[FileLog] );

END;	{ C_FilLog_p }

{***************************************************************}
{	F_FilLog_p:						}
{	The FilLog command given through functionkey		}
{***************************************************************}

Procedure F_FilLog_p;				{ Global }

BEGIN	{ F_FilLog_p }

    FileLog := NOT FileLog;			{ Toggle }

    IF mode = setup THEN BEGIN
    	HighVideo;
	CposWrite ( 6, 55, OffOnStr[FileLog] );
	LowVideo;
    END;

    CposWrite ( 2, 74, 'LOG' + OffOnStr[FileLog] );

END;	{ F_FilLog_p }

{***************************************************************}
{	CF_LogNam_p:						}
{	The LogNam command.					}
{***************************************************************}

Procedure CF_LogNam_p;				{ Global }

BEGIN	{ CF_LogNam_p }

    CposWrite ( 20, 35, 'CF_LogNam' );

END;	{ CF_LogNam_p }

{***************************************************************}
{	C_LogTyp_p:						}
{	The LogTyp command given on the command line.		}
{***************************************************************}

Procedure C_LogTyp_p;				{ Global }

BEGIN	{ C_LogTyp_p }

    CposWrite ( 20, 35, 'C_LogTyp' );

END;	{ C_LogTyp_p }

{***************************************************************}
{	F_Logtyp_p:						}
{	The LOGTYP command given through function key.		}
{***************************************************************}

Procedure F_Logtyp_p;				{ Global }

BEGIN	{ F_Logtyp_p }

    WriteWindow ( Setup_Logtype );
    Scroll (  5,  1, 16, 39, 0, Up );
    Scroll (  5, 40,  8, 79, 0, Up );
    Mode := Setup_Logtype;
    DisplayFKeys;

END;	{ F_Logtyp_p }

{***************************************************************}
{	F_Baud_p:						}
{	The Baud command given through function key.		}
{***************************************************************}

Procedure F_Baud_p;				{ Global }

BEGIN	{ F_Baud_p }

    Rs232CommParm := aBaud;
    Baudrate := BaudTab[Baudrate].Baud_Next;
    Set_Rs232TrmOp;

END;	{ F_Baud_p }

{***************************************************************}
{	F_Parity_p:						}
{	The Parity command given through function key.		}
{***************************************************************}

Procedure F_Parity_p;				{ Global }

BEGIN	{ F_Parity_p }

    Rs232CommParm := aParity;
    Parity := ParityTab[Parity].Parity_Next;
    Set_Rs232TrmOp;

END;	{ F_Parity_p }

{***************************************************************}
{	F_DatBit_p:						}
{	The DatBit command given through function key.		}
{***************************************************************}

Procedure F_DatBit_p;				{ Global }

BEGIN	{ F_DatBit_p }

    Rs232CommParm := aDatabits;
    Databits := DatabitTab[Databits].Databit_Next;
    Set_Rs232TrmOp;

END;	{ F_DatBit_p }

{***************************************************************}
{	F_StpBit_p:						}
{	The StpBit command given through function key.		}
{***************************************************************}

Procedure F_StpBit_p;				{ Global }

BEGIN	{ F_StpBit_p }

    Rs232CommParm := aStopbits;
    Stopbits := StopbitTab[Stopbits].Stopbit_Next;
    Set_Rs232TrmOp;

END;	{ F_StpBit_p }

{***************************************************************}
{	F_GpibAdr_p:						}
{	The GpibAdr command given through function key.		}
{***************************************************************}

Procedure F_GpibAdr_p;				{ Global }

BEGIN	{ F_GpibAdr_p }

    IF Gpibadr < 30 THEN GpibAdr := GpibAdr + 1
    ELSE GpibAdr := 0;
    Set_GpibAdr;

END;	{ F_GpibAdr_p }

{***************************************************************}
{	F_LogTyp_S_p:						}
{	The LogTyp_S command given through function key.	}
{***************************************************************}

Procedure F_LogTyp_S_p;				{ Global }

BEGIN	{ F_LogTyp_S_p }

    LtTab[S] := NOT LtTab[S];			{ Toggle }
    Display_Logtype ( S );

END;	{ F_LogTyp_S_p }

{***************************************************************}
{	F_LogTyp_AR_p:						}
{	The LogTyp_AR command given through function key.	}
{***************************************************************}

Procedure F_LogTyp_AR_p;			{ Global }

BEGIN	{ F_LogTyp_AR_p }

    LtTab[AR] := NOT LtTab[AR];			{ Toggle }
    Display_Logtype ( AR );

END;	{ F_LogTyp_AR_p }

{***************************************************************}
{	F_LogTyp_SR_p:						}
{	The LogTyp_SR command given through function key.	}
{***************************************************************}

Procedure F_LogTyp_SR_p;			{ Global }

BEGIN	{ F_LogTyp_SR_p }

    LtTab[SR] := NOT LtTab[SR];			{ Toggle }
    Display_Logtype ( SR );

END;	{ F_LogTyp_SR_p }

{***************************************************************}
{	F_LogTyp_UR_p:						}
{	The LogTyp_UR command given through function key.	}
{***************************************************************}

Procedure F_LogTyp_UR_p;			{ Global }

BEGIN	{ F_LogTyp_UR_p }

    LtTab[UR] := NOT LtTab[UR];			{ Toggle }
    Display_Logtype ( UR );

END;	{ F_LogTyp_UR_p }

{***************************************************************}
{	F_LogTyp_ER_p:						}
{	The LogTyp_ER command given through function key.	}
{***************************************************************}

Procedure F_LogTyp_ER_p;			{ Global }

BEGIN	{ F_LogTyp_ER_p }

    LtTab[ER] := NOT LtTab[ER];			{ Toggle }
    Display_Logtype ( ER );

END;	{ F_LogTyp_ER_p }

{***************************************************************}
{	F_LogTyp_E_p:						}
{	The LogTyp_E command given through function key.	}
{***************************************************************}

Procedure F_LogTyp_E_p;				{ Global }

BEGIN	{ F_LogTyp_E_p }

    LtTab[E] := NOT LtTab[E];			{ Toggle }
    Display_Logtype ( E );

END;	{ F_LogTyp_E_p }

{***************************************************************}
{	SetupLoop:						}
{	Waits for something to setup.				}
{***************************************************************}

Procedure SetupLoop;				{ Global }

VAR
    ChFetched	: boolean;		{ Was there a character fetched? }
    Keytype	: Keytype_t;		{ What's in ch }
    ch		: char;			{ The character }

BEGIN	{ SetupLoop }

    REPEAT
	Scan_Kbd( ChFetched, Keytype, ch );
	IF ChFetched THEN
	    Get_OpCmd( ChFetched, Keytype, ch );
    UNTIL Menu <> SetupMenu;

END;	{ SetupLoop }
