{$nomain}
{*******************************************************}
{	This module contains the syntaxchecker.		}
{*******************************************************}

%Include 'global.h';
%Include 'turbodef.h';
%Include 'batch.h';
%Include 'screen.h';
%Include 'log.h';

Function p_inew ( size	: Int2 )
			: lstrX_ptr;		EXTERNAL;

Procedure p_dispos ( var p	: lstrX_ptr;
		       size	: int2);	EXTERNAL;

CONST
    MaxAnswExp = 32767;

TYPE
    SrcLine_t = RECORD
    	len : int2;		{ Length }
	pos : int2;		{ Until where in txt syntaxcheck is performed }
	txt : BatFileLine_t;	{ The line }
    END;

TYPE
    Comlist_t	= ARRAY [bc_for..bc_next] OF lstr5;

CONST
    comlist	= comlist_t
    	( 'FOR', 'PAUSE', 'STOP', 'ECHO', 'LOG', 'NEXT' );


CONST
    FNMax	= 19;		{ Max number of levels in for-next-loops }

TYPE
    ForNext_t = RECORD
	stack	: ARRAY [1..FNMax] OF BaList_Ptr;	{ For-Next stack }
	index	: int2;					{ For-Next index }
    END;

{*******************************************************}
{	TextAlloc:					}
{	Allocates memory for text and puts it in the	}
{	linked list.					}
{*******************************************************}
Function TextAlloc(	curr	: BaList_ptr;
		    VAR line	: SrcLine_t)
		    		: BatErrCode_t;
VAR
    i	: int2;
    
BEGIN	{ TextAlloc }

    curr^.Text := p_inew(line.len-line.pos);
    IF curr^.Text = NIL THEN BEGIN		{ Memory allocation failure }
	TextAlloc := OutOfMemory;
	Log_SyntErr(OutOfMemory, line.txt, line.len);
    END ELSE BEGIN
	FOR i := 0 TO line.len-line.pos DO   { move line to allocated memory }
	    curr^.Text^[i+1] := line.txt[line.pos+i];
	curr^.TextSize := Line.len-line.pos+1;
	TextAlloc := NoError;
    END;

END;	{ TextAlloc }

{*******************************************************}
{	SkipDelim:					}
{	Skips delimiters in line. I.e. adjusts Line.pos	}
{	to first 'non-delimiter' after line.pos.	}
{*******************************************************}
Procedure SkipDelim( VAR line	: SrcLine_t);

VAR
    delim	: set of char;

BEGIN	{ SkipDelim }

    delim := [' ',TAB,','];

    WHILE (line.txt[line.pos] in delim) AND (line.pos < line.len) DO
	line.pos := line.pos + 1;

END;	{ SkipDelim }

{*******************************************************}
{	GetFirstElement:				}
{	Returns index to first and last position of	}
{	the first element (after pos) in a line		}
{*******************************************************}
Procedure GetFirstElement ( VAR line	: SrcLine_t;
			    VAR first	: int2;
			    VAR last	: int2);

VAR
    delim	: set of char;

BEGIN	{ GetFirstElement }

    { find first position of element }
    SkipDelim(line);

    first := line.pos;

    { find last position of command }
    delim := [' ',TAB,','];
    last := line.pos;
    WHILE ( NOT (line.txt[last] in delim) ) AND ( last < line.len ) DO
    	last := last + 1;

END;	{ GetFirstElement }

{*******************************************************}
{	GetBatchCmd:					}
{	Find out the command (the first element) of a	}
{	batch line, separated by space, tab or comma.	}
{	If the line starts with a number it is returned	}
{	and the command is set to remote. Line.pos is	}
{	adjusted.					}
{*******************************************************}
Function GetBatchCmd (	VAR line	: SrcLine_t;
			VAR AnswExp	: int2)
					: BaCmd_t;

VAR
    fp		: int2;		{ First position }
    lp		: int2;		{ Last position }
    cmd		: BaCmd_t;
    cmd_i	: BaCmd_t;
    cmdstr	: lstr5;
    status	: integer;
    i		: int2;

BEGIN	{ GetBatchCmd }

    cmd := bc_missing;

    line.pos := 1;
    GetFirstElement ( line, fp, lp );

    IF line.txt[fp] = '''' THEN
	cmd := bc_comment
    ELSE IF (lp - fp) <= 5 THEN BEGIN    { No command more than 5 char's }
	cmdstr := '';
	FOR i := fp TO lp DO
	    cmdstr := cmdstr + line.txt[i];

	cmd := bc_missing;			{ find matching command }
	FOR cmd_i := bc_For TO bc_Next DO
	    IF cmdstr <> ComList[cmd_i] THEN
	    	cmd := cmd_i;

	IF cmd = bc_missing THEN BEGIN		{ check for remote command }
	    Val( cmdstr, AnswExp, status );
	    IF status = 0 THEN
		IF AnswExp > MaxAnswExp THEN
		    cmd := bc_TooManyAnswExp
		ELSE
		    cmd := bc_Remote;
	END;

    END;	{ ELSE IF }

    { Adjust line.pos }
    line.pos := lp;
    SkipDelim(line);

    GetBatchCmd := cmd;

END;	{ GetBatchCmd }

{*******************************************************}
{	ValOut:						}
{	returs true if value is lower than lo		}
{	or higher than hi				}
{*******************************************************}
function ValOut(	value	: int2;
			lo	: int2;
			hi	: int2)
				: boolean;

BEGIN	{ ValOut }

    ValOut := false;

    IF ( value < lo ) OR ( value > hi ) THEN
	ValOut := true;

END;	{ ValOut }

{*******************************************************}
{	CheckFor.					}
{	syntax check of FOR command			}
{*******************************************************}
Function CheckFor(    curr	: BaList_ptr;
		  VAR fn	: fornext_t;
		  VAR line	: SrcLine_t)
				: BatErrCode_t;

VAR
    i		: int2;
    fp,lp	: int2;
    param	: lstr5;
    status	: integer;

BEGIN	{ CheckFor }

    GetFirstElement ( line, fp, lp );
    IF lp - fp > 5 THEN BEGIN
	CheckFor := ParameterOutOfRange;
	Log_SyntErr(ParameterOutOfRange, line.txt, line.len);
    END ELSE BEGIN
    	param := '';
    	FOR i := fp to lp-1 DO
	    param := param + line.txt[i];
	Val( param, curr^.arg1, status );
	IF (status <> 0) OR Valout(curr^.arg1, 0, 32767) THEN BEGIN
	    CheckFor := ParameterOutOfRange;
	    Log_SyntErr(ParameterOutOfRange, line.txt, line.len);
	END ELSE BEGIN
	    fn.index := fn.index + 1;
	    IF fn.index > fnmax THEN BEGIN
		CheckFor := ForLevel;
		Log_SyntErr(ForLevel, line.txt, line.len);
	    END ELSE BEGIN
		fn.stack[fn.index] := curr;
		CheckFor := NoError;
		Line.pos := lp;
		SkipDelim(line);
		IF line.pos <> line.len THEN		{ Comments ? }
		    CheckFor := TextAlloc(curr, line);	{ Comments ! }
	    END;
	END;
    END;

END;	{ CheckFor }

{*******************************************************}
{	CheckPause.					}
{	syntax check of PAUSE command			}
{*******************************************************}
Function CheckPause(	curr	: BaList_ptr;
		    VAR line	: SrcLine_t)
		    		: BatErrCode_t;

VAR
    i		: int2;

BEGIN	{ CheckPause }

{???}

    CheckPause := NoError;

END;	{ CheckPause }

{*******************************************************}
{	CheckStop.					}
{	syntax check of STOP command			}
{*******************************************************}
Function CheckStop(	curr	: BaList_ptr;
		    VAR line	: SrcLine_t)
		    		: BatErrCode_t;

VAR
    i		: int2;

BEGIN	{ CheckStop }

{???}

    CheckStop := NoError;

END;	{ CheckStop }

{*******************************************************}
{	CheckRemote.					}
{	syntax check of remote command			}
{*******************************************************}
Function CheckRemote(	curr	: BaList_ptr;
		    VAR line	: SrcLine_t)
		    		: BatErrCode_t;

VAR
    i		: int2;

BEGIN	{ CheckRemote }

{???}

    CheckRemote := NoError;

END;	{ CheckRemote }

{*******************************************************}
{	CheckEchoLog.					}
{	Syntax check of ECHO and LOG command		}
{*******************************************************}
Function CheckEchoLog(	    curr	: BaList_ptr;
			VAR line	: SrcLine_t)
					: BatErrCode_t;

BEGIN	{ CheckEchoLog }

    IF line.pos = line.len THEN BEGIN		{ Empty log command }
    	line.len := line.len + 1;
        line.txt[line.len] := ' ';		{ log a space }
    END;

    CheckEchoLog := TextAlloc(curr, line);

END;	{ CheckEchoLog }

{*******************************************************}
{	CheckNext.					}
{	syntax check of NEXT command			}
{*******************************************************}
Function CheckNext(   curr	: BaList_ptr;
		  VAR fn	: fornext_t;
		  VAR line	: SrcLine_t)
		    		: BatErrCode_t;

BEGIN	{ CheckNext }

    IF fn.index = 0 THEN BEGIN		{ any FOR on the stack }
	CheckNext := NextLevel;
	Log_SyntErr(NextLevel, line.txt, line.len);
    END ELSE BEGIN
	CheckNext := NoError;
	curr^.ForNextPtr := fn.stack[fn.index];
	fn.stack[fn.index]^.ForNextPtr := curr;
	fn.index := fn.index - 1;
    END;

END;	{ CheckNext }

{*******************************************************}
{	SyntaxCheck:					}
{	syntax check of one line			}
{*******************************************************}
Procedure SyntaxCheck (	    curr	: BaList_ptr;
		        VAR fn		: fornext_t;
			VAR line	: SrcLine_t;
			VAR status	: BatErrCode_t);

BEGIN	{ SyntaxCheck }

    curr^.BaCmd := GetBatchCmd ( Line, curr^.AnswExp );

    CASE curr^.BaCmd OF

    	bc_For : status := CheckFor( curr, fn, line);

	bc_Pause : status := CheckPause( curr, line);

	bc_Stop : status := CheckStop( curr, line);

	bc_Echo,
	bc_Log : status := CheckEchoLog( curr, line);

	bc_Next : status := CheckNext( curr, fn, line);

	bc_Remote : status := CheckRemote( curr, line);

	bc_missing : BEGIN
	    status := CommandMissing;
	    Log_SyntErr(status, line.txt, line.len);
	END;

	bc_TooManyAnswExp : BEGIN
	    status := TooManyAnswExp;
	    Log_SyntErr(status, line.txt, line.len);
	END;

	bc_comment :
	    status := CommentLine;

    END;	{ CASE }

END;	{ SyntaxCheck }

{*******************************************************}
{	Open_BatchFile:					}
{*******************************************************}
procedure Open_BatchFile (VAR errcode:BatErrCode_t);

VAR
    status	: integer;

BEGIN	{ Open_BatchFile }

    errcode := NoError;

    Reset(BatchFile, BatchFileName, '.MTS', status);

    IF status = -1 THEN BEGIN
	ShowFileErr('OPEN', 'BatchFile', BatchFileName, iostatus(BatchFile));
	errcode := OpenBatchFile;
    END;

    noioerror(BatchFile);	{ turn on error trapping }

END;	{ Open_BatchFile }

{*******************************************************}
{	RemoveBatchList:				}
{	releasar minnet som allokerats till		}
{	commandolistan					}
{*******************************************************}
procedure RemoveBatchList;

VAR
    curr	: BaList_ptr;
    pnext	: BaList_ptr;

BEGIN	{ RemoveBatchList }

    curr := BatchList;

    WHILE curr <> NIL DO BEGIN
	pnext := curr^.pnext;
	IF curr^.Text <> NIL THEN
	    p_dispos( curr^.Text, curr^.TextSize);
	dispose(curr);
	curr := pnext;
    END;

    BatchList := NIL;

END;	{ RemoveBatchList }

{*******************************************************}
{	CreateCommandList:				}
{	Builts the linked batch command list and	}
{	syntax check it					}
{*******************************************************}
procedure CreateCommandList ( VAR error:BatErrCode_t ); { global }

VAR
    curr	: BaList_ptr;
    currold	: BaList_ptr;
    line	: SrcLine_t;
    i		: int2;
    continue	: boolean;	{ false when serious error found }
    status	: BatErrCode_t;
    linenr	: int2;
    fn		: fornext_t;	{ The FOR-NEXT stack }

BEGIN	{ CreateCommandList }

    error := NoError;

    Open_BatchFile (error);
    IF error <> NoError THEN
    	continue := false;

    IF BatchList <> NIL THEN
    	RemoveBatchList;

    New(curr);			{ first line }
    curr^.Text := NIL;

    BatchList := curr;
    linenr := 0;		{ linenumber in batch file }
    fn.index := 0;			{ index in fnstack }

    DebugWriteln ('Syntax check of command file');

    WHILE NOT eof(BatchFile) AND continue DO BEGIN

	{ Check if the memory is enough, else stop syntax check }
	{ This assumes that at least 2 NEW's has been done OK }

	IF Memavail < 100 THEN BEGIN	{ save some for the stack }
	    error := OutOfMemory;
	    Log_SyntErr(error, line.txt, line.len);
	    continue := false;
	END;

	ReadLn(BatchFile, line.txt);

	IF ioerror(BatchFile) THEN BEGIN
	    ShowFileErr('READ', 'BatchFile', BatchFileName, iostatus(BatchFile));
	    error := ReadBatchFile;
	    continue := false;
	END;

	linenr := linenr + 1;

	Line.len := MaxBatLineSize;
	WHILE ( Line.len > 1 ) AND ( line.txt[Line.len] = ' ' ) DO
	    Line.len := Line.len - 1;		{ search for last non-blank }
	IF line.txt[Line.len] <> ' ' THEN	{ empty line ? }
	    Line.len := 0;

	IF continue AND ( Line.len <> 0 ) THEN BEGIN

	    IF NOT eoln(BatchFile) THEN BEGIN
	    	error := TooLongLine;
		Log_SyntErr(error, line.txt, line.len);
	    END;

	    SyntaxCheck ( curr, fn, line, status );
	    IF status = NoError THEN BEGIN
	    	currold := curr;
		New(curr^.pnext);
		curr := curr^.pnext;
		curr^.Text := NIL;
	    END ELSE IF status <> CommentLine THEN
	    	error := status;

	END;	{ if }
    END;	{ while }

    IF continue THEN BEGIN
	IF fn.index <> 0 THEN BEGIN	{ to few next }
	    error := MissingNextStatement;
	    Log_SyntErr(error, line.txt, line.len);
	END;
    END;

    dispose(curr);		{ This was not needed }
    currold^.pnext := NIL;	{ End of the linked list }

    IF NOT continue THEN
	RemoveBatchList;

    Close ( BatchFile );

END;	{ CreateCommandList }
