{$title:'MODULE: Varhandler' $pagesize:41 $linesize:106}

Module Varhandler;

{*******************************************************}
{	Modulen innehaller variabelhanteringen.		}
{*******************************************************}

{$Include:'decl.h'}
{$Include:'global.h'}
{$Include:'screen.h'}

TYPE

  Fdef_Var	= RECORD	{ Fordefinierad variabel }
    namn	: lstr10;	{ variabelns namn }
    pvar	: adr of real;	{ pekare till variabeln }
  END;

  var_ptr	= ^var_t;

  var_t		= RECORD	{ Egendefinierade variabler }
    txt		: lstr10;	{ variabelns namn }
    val		: real;		{ vaerde }
    pnext	: var_ptr;	{ pekare till naesta record }
  END;

VAR [PUBLIC]
  FdefVarTab	: ARRAY [1..10] OF Fdef_Var;
				{ Tabell over fordefinierade variabler }
  Ant_Fdef_Var	: integer;	{ Antal fordefinierade variabler }
  vlist		: var_ptr;	{ pekare till den laenkade variabellistan }

{$page+}
{*******************************************************}
{	ClrVlist.					}
{	Nollstaeller variabellistan			}
{*******************************************************}

procedure ClrVlist [PUBLIC];

BEGIN	{ ClrVlist }

  vlist := NIL;

END;	{ ClrVlist }

{$page+}
{*******************************************************}
{	Make_FdefVarTab.				}
{	Skapar en variabelnamntabell over de 		}
{	fordefinierade variablerna.			}
{*******************************************************}

procedure Make_FdefVarTab [PUBLIC];

BEGIN	{ Make_FdefVarTab }

  FdefVarTab[1].namn := 'ANT_OBJ';	FdefVarTab[1].pvar := ADR ant_obj;
  Ant_Fdef_Var := 1;

END;	{ Make_FdefVarTab }

{$page+}
{*******************************************************}
{	CheckVar.					}
{	Kontrollerar om en variabel ar fordefinierad	}
{	eller inte.					}
{*******************************************************}

function CheckVar ( varnamn : lstr10 ) : vartyp_t [PUBLIC];

VAR
  i		: integer;

BEGIN	{ CheckVar }

  FOR i := 1 TO Ant_Fdef_Var DO BEGIN
    IF FdefVarTab[i].namn = varnamn THEN BEGIN
      CheckVar := Fordefinierad;		{ Fordefinierad }
      RETURN; END;
  END;
  CheckVar := Egendefinierad;

END;	{ CheckVar }

{$page+}
{*******************************************************}
{	MakeVar.					}
{	Skapar en variabel i variabellistan om 		}
{	variabeln inte redan finns.			}
{*******************************************************}

procedure MakeVar( varnamn : lstr10 ) [PUBLIC];

VAR
  vptr		: var_ptr;
  vptro		: var_ptr;

BEGIN	{ MakeVar }

  vptr := vlist;		{ boerja fraan boerjan }
  WHILE vptr <> NIL DO BEGIN
    IF vptr^.txt = varnamn THEN	{ kolla foerst om den redan finns }
      RETURN;			{ variabeln finns redan }
    vptro := vptr;
    vptr := vptr^.pnext;
  END;

  New(vptr);			{ skapa ny variabel }
  vptr^.txt := varnamn;
  vptr^.val := 0;
  vptr^.pnext := NIL;

  IF vlist = NIL THEN		{ laenka in i listan }
    vlist := vptr
  ELSE
    vptro^.pnext := vptr;

END;	{ MakeVar }

{$page+}
{*******************************************************}
{	Get_VarValue.					}
{	Hamtar vardet pa en variabel.			}
{*******************************************************}

function Get_VarValue ( varnamn : lstr10 ) : real [PUBLIC];

VAR
  i		: integer;
  vptr		: var_ptr;

BEGIN	{ Get_VarValue }

  FOR i := 1 TO Ant_Fdef_Var DO BEGIN	{ Kolla forst om fordefinierad }
    IF FdefVarTab[i].namn = varnamn THEN BEGIN
      Get_VarValue := FdefVarTab[i].pvar^;	{ Fordefinierad }
      RETURN; END;
  END;

  vptr := vlist;			{ Egendefinierad }
  WHILE vptr <> NIL DO BEGIN
    IF vptr^.txt = varnamn THEN BEGIN
      Get_VarValue := vptr^.val;		{ Hamta vardet }
      RETURN; END;
    vptr := vptr^.pnext;
  END;

  DebugWriteLn ('Get_VarValue hittade ingen variabel');	{ D }

END;	{ Get_VarValue }

{$page+}
{*******************************************************}
{	Put_VarValue.					}
{	Andrar vardet pa en variabel.			}
{*******************************************************}

procedure Put_VarValue ( varnamn : lstr10;  varvarde : real )
			 [PUBLIC];

VAR
  vptr		: var_ptr;

BEGIN	{ Put_VarValue }

  vptr := vlist;
  WHILE vptr <> NIL DO BEGIN
    IF vptr^.txt = varnamn THEN BEGIN
      vptr^.val := varvarde;		{ Skriv vardet }
      RETURN; END;
    vptr := vptr^.pnext;
  END;

  DebugWriteLn ('Put_VarValue hittade ingen variabel')	{ D }

END;	{ Put_VarValue }

{$page+}
{*******************************************************}
{	TaBortVariabelLista:				}
{	releasar minnet som allokerats till		}
{	VariabelListan					}
{*******************************************************}
procedure TaBortVariabelLista [PUBLIC];

VAR
  pnext		: var_ptr;

BEGIN	{ TaBortVariabelLista }

  WHILE vlist <> NIL DO BEGIN
    pnext := vlist^.pnext;
    dispose(vlist);
    vlist := pnext;
  END;

END;	{ TaBortVariabelLista }

END.	{ Module }
