/*
 * mpstart.  program to invoke another program, after
 *	trimming the envioronment down to the minimum
 *	number of elements needed by multiplan.  the
 *	purpose of this procedure is to maximize the
 *	available stack space in the interpreter, and
 *	to also thereby maximize available data space
 *	for multiplan.
 */

/* list of the environment variables to be kept around */

char *keeper[] =
	{
	"TERMCAP",		/* must be FIRST, handled specially */
	"PATH",
	"TERM",
	"LPR",
	"TZ",
	"MPLIB",
	(char *)0		/* terminator */
	};

#define MAXENV (sizeof(keeper)/sizeof(char *))

main(argc,argv,envp)
	int argc;
	char *argv[];
	char *envp[];
	{
	int ii, jj, kk;
	char *newenv[MAXENV];
	char *name;
	char *pp;
	extern char *getenv();

	if (argc < 2)
		{
		printf("Usage: %s program_path [arg]...\n",argv[0]);
		exit(1);
		}
	/*
	 * construct list of variables to be preserved
	 */
		/* if TERMCAP is a pathname, leave it in, else zap it.
		 * the rationale is that the capability description is often
		 * quite large, and therefore takes up valuable stack
		 * space, when multiplan can easily just read the description
		 * into a throwaway area of the interpreter heap.
		 */
	pp = getenv("TERMCAP");
	if ( !(pp && *pp == '/') ) /* TERMCAP is not a pathname */
		*keeper[0] = '\0';	/* so don't retain it */

	kk = 0;		/* index into newenv[] */

	for (ii=0; envp[ii] != (char *)0; ii++)
		for (jj=0; keeper[jj] != (char *)0; jj++)
			if (match(keeper[jj],envp[ii]))
				{
				newenv[kk++] = envp[ii];
				*keeper[jj] = '\0';	/* remove */
				}
	newenv[kk] = 0; /* terminate new environment vector */

	/*
	 * set up to call desired program
	 */

	name = argv[1];

	/*
	 * reconstruct argument vector
	 */

	for (ii=0; ii<argc; ii++)
		argv[ii] = argv[ii+1];

	execve(name,argv,newenv);
	printf("mpstart program failure\n");
	}

/*
 * match.  tests a name from keeper[] against a single
 *	environmental variable, to see if they match.
 *	returns 1 on match, 0 on nomatch.
 */

int match(name,var)
	char *name, *var;
	{
	int ii;

	while (*name++ == *var++)
		;

	if  ( (*--name == '\0') && (*--var == '=' ) )
		return(1);
	else
		return(0);
	}
