PAGE	60,132
INCLUDE B:BIG.INC
;
;	ALOC.ASM:	ALLOCATE/DEALLOCATE POOL MEMORY SVC PROCESSORS.
;	F. W. PORRETTO, 12/15/82
;
;		HERE ARE THE "FRONT END" ROUTINES WHICH ACCEPT AN SDB TO
;	ALLOCATE OR DEALLOCATE MEMORY AND TURN IT INTO A SUBROUTINE CALL 
;	TO ONE OF THE LOWER-LEVEL POOL HANDLERS IN POOL.ASM.
;
		NAME	POOL_SVC_SERVICES
;
MTOS_CS		SEGMENT PARA PUBLIC
		ASSUME	CS:MTOS_CS,DS:MTOS_DS,SS:MTOS_DS
		PUBLIC	X_ALOC,X_DALOC
;
;	LOCAL EQUATES
;
WT		EQU	01	;SDB OFFSET TO REQUEST OPTION (0=CONTINUE)
POOL		EQU	01	;SDB OFFSET TO POOL ID
SZE		EQU	02	;SDB OFFSET TO THE NUMBER OF BYTES
BUFR		EQU	04	;SDB OFFSET TO THE BUFFER POINTER
WT_MASK		EQU	80H	;MASK TO ISOLATE WT
POOL_MASK	EQU	1FH	;MASK TO ISOLATE POOL ID
;
		EXTRN	X_TASK_STATE	:NEAR	;SET TASK STATE
		EXTRN	X_SET_READY	:NEAR
		EXTRN	X_SFLAGS	:NEAR	;SET TASK CONDITION CODES
		EXTRN	X_GET_BLOCK	:NEAR	;ALLOCATE BLOCK
		EXTRN	X_FREE_BLOCK	:NEAR	;RELEASE BLOCK
		EXTRN	X_WAIT_FOR_BLOCK:NEAR	;WAIT UNTIL BLOCK IS AVAILABLE
		EXTRN	X_POOL_DATA	:NEAR	;GET MEMORY POOL INFO.
PAGE
;*************************************************************
;*      X_ALOC:  PROCESS ALLOCATE MEMORY SERVICE REQUEST     *
;*************************************************************
;
;	INPUT:	ES:SI=	SDB ADDRESS.
;		BX=	TCB REQUESTING SERVICE
;	OUTPUT:	ZERO FLAG= 1 IF MEMORY NOT AVAILABLE AND REQUEST
;               WAS FOR A 'CONTINUE' FUNCTION.
;
;		THIS ROUTINE EXTRACTS AND VALIDATES THE SDB PARAMETERS, IN-
;	SURING THE GROSS LEGALITY OF THE REQUEST, AND INVOKES THE LOWER LEVEL
;	ROUTINES X_GET_BLOCK AND, OPTIONALLY, X_WAIT_FOR_BLOCK TO PROVIDE THE
;	MOST DIRECT LEVELS OF POOL REQUEST PROCESSING.
;
X_ALOC		PROC	FAR
		MOV	DL,ES:[SI+POOL]		;BX= TCB   DL= POOL ID
		AND	DX,POOL_MASK		;ISOLATE POOL ID
		CMP	DX,X_MEM_POOL_NUM	;VALID ID ?
		JGE	ALOCERR			;--NO--
		PUSH	BX			;SAVE TCB ADDRESS
		CALL	X_POOL_DATA		;GET POOL TABLES
						;DI= MPD  BX= MPB
		POP	BX			;RESTORE TCB ADDRESS
		MOV	CX,ES:[SI+SZE]		;NUMBER OF BYTES REQUESTED
		CMP	CX,4			;MUST BE AT LEAST 4 ?
		JB	ALOCERR			;NO (TOO SMALL)
		PUSH	DS
		MOV	AX,MTOS_GROM		;SET UP TO BE ABLE TO REACH THE
		MOV	DS,AX			;GLOBAL ROM SEGMENT.
		MOV	AX,[DI+MPD_SZE]		;TOTAL NUMBER OF BYTES
		POP	DS			;RESTORE MTOS_DS.
		SUB	AX,CX			;REMOVE REQUESTED BYTES
		JB	ALOCERR			;TOO MANY REQUESTED!
		JE	ALOC15			;EXACTLY MATCHES POOL SIZE.
		CMP	AX,4			;ARE THERE AT LEAST FOUR LEFT ?
		JAE	ALOC15			;YES.
ALOCERR:
		MOV	AX,SVC_FAILED+PARM_ERR	;NO:
		CALL	X_SFLAGS		;TELL USER CALL FAILED DUE TO
		RET				;PARAMETER ERROR.
ALOC15:						;ATTEMPT ACTUAL ALLOCATION:
		MOV	BP,ES			;SAVE SDB SEGMENT
		CALL	X_GET_BLOCK		;TRY FOR THE MEMORY BLOCK...
		JC	NO_BLOCK		;NO CAN DO JUST NOW.
		PUSH	ES			;YES:
		MOV	ES,BP			;RESTORE SDB SEGMENT,
		LES	DI,DWORD PTR ES:[SI+BUFR]	;GET BUFFER POINTER.
		POP	WORD PTR ES:[DI+2]	;INSERT BLOCK SEGMENT,
		MOV	ES:[DI],BX		;THEN OFFSET.
		RET				;DONE.
NO_BLOCK:
		MOV	ES,BP			;SAVE SDB SEGMENT
		AND	WORD PTR ES:[SI+WT],WT_MASK	;GET WAIT OPTION...
		JNE	WAIT_TASK		;HE ASKED FOR WAIT.
		MOV	AX,SVC_FAILED		;NO WAIT: ANNOUNCE FAILURE.
		CALL	X_SFLAGS
		RET
WAIT_TASK:
		LES	DI,DWORD PTR ES:[SI+BUFR]	;GET BUFFER POINTER
		CALL	X_WAIT_FOR_BLOCK	;CAUSE TASK TO WAIT FOR MEMORY
		RET				;DONE.
X_ALOC		ENDP
PAGE
;
;	UTILITY ROUTINE TO CHECK FOR FREE BLOCK INTERSECTIONS:
;
;	INPUT:	ES= POOL SEGMENT,
;		DI= ADDRESS OF CURRENTLY FREE BLOCK,
;		SI= ADDRESS OF BLOCK TO BE FREED,
;		CX= SIZE OF BLOCK TO BE FREED.
;	OUTPUT:	IF CARRY=1, THE BLOCKS INTERSECT. OTHERWISE, THERE IS NO
;		CONFLICT BETWEEN THEM.
;
INTERSECT	PROC	NEAR
		MOV	AX,ES:[DI+2]		;GET FREE BLOCK SIZE
		ADD	AX,DI			;FIND DORSAL EDGE.
		CMP	AX,SI			;FREE BLOCK < NEW BLOCK?
		JBE	IOK			;YES.
		MOV	AX,SI			;NO: FIND DORSAL EDGE
		ADD	AX,CX			;OF NEW BLOCK.
		CMP	AX,DI			;NEW BLOCK < FREE BLOCK?
		JBE	IOK			;YES.
		STC				;NO: THE TWO CLASH.
		RET
IOK:		CLC
		RET
INTERSECT	ENDP
PAGE
;***************************************************************
;*      X_DALOC:  PROCESS SVC FOR FREEING MEMORY POOL BLOCK    *
;***************************************************************
;
;	INPUT:	ES:SI=	SDB ADDRESS.
;		BX=	REQUESTOR'S TCB ADDRESS.
;	OUTPUT:	SVC ERROR IF ANY PARAMETER IS NOT VALID OR
;		BLOCK IS ALREADY FREE.
;
;	THIS ROUTINE EXTRACTS AND VALIDATES THE SDB PARAMETERS FOR THE REQUEST,
;	AND INSURES THAT THE INDICATED BLOCK IS DEALLOCATABLE: DOES NOT INTER-
;	SECT WITH ANY OTHER FREE BLOCK. ALSO, X_DALOC DETERMINES THE POOL TO
;	WHICH THE BLOCK BELONGS BEFORE INVOKING X_FREE_BLOCK FOR DIRECT POOL
;	DEALLOCATION CONTROL.
;
X_DALOC		PROC	FAR
		MOV	CX,ES:[SI+SZE]		;GET BLOCK SIZE TO CX
		CMP	CX,3			;LEGAL SIZE?
		JBE	DALERR			;NO, TOO SMALL.
		LES	SI,DWORD PTR ES:[SI+BUFR]	;POINTER TO BLK POINTER
		LES	SI,DWORD PTR ES:[SI]	;GET THE BLOCK ADDRESS.
		XOR	DX,DX			;SET POOL ID = 0.
;
;	WE WILL CHECK THE BLOCK'S ADDRESS AGAINST THE SYSTEM'S POOLS IN ORDER
;	OF THEIR POOL IDENTIFIERS UNTIL WE FIND THE ONE FROM WHICH IT WAS 
;	ALLOCATED OR RUN OUT OF POOLS.
;
DAL_0:
		CMP	DX,X_MEM_POOL_NUM	;RUN OUT OF POOLS YET?
		JAE	DALERR			;YES.
		CALL	X_POOL_DATA		;NO: GET MPB AND MPD ADDRESSES
		PUSH	DS			;SAVE MTOS_DS WHILE WE GO
		MOV	AX,MTOS_GROM		;BRIEFLY TO THE GLOBAL ROM
		MOV	DS,AX			;SEGMENT FOR THE MPD.
		MOV	AX,ES			;BLOCK SEGMENT
		CMP	AX,[DI+MPD_DS]		;MATCHES POOL SEGMENT?
		JNE	DAL_3			;NO.
		MOV	AX,SI			;YES:
		ADD	AX,CX			;CALCULATE BLOCK DORSAL EDGE.
		CMP	AX,[DI+MPD_SZE]		;BLOCK WITHIN POOL?
		JBE	DAL_4			;YES!
DAL_3:
		POP	DS			;RESTORE MTOS_DS.
		INC	DX			;NO: BUMP POOL NUMBER
		JMP	DAL_0			;AND TRY NEXT POOL.
DAL_4:
		POP	DS			;RESTORE MTOS_DS,
		MOV	DI,[BX+MPB_FREEQ]	;GET HEAD OF POOL FREE LIST
DAL_5:
		CMP	DI,LINK_END		;END OF FREE LIST?
		JE	DAL_6			;YES: BLOCK MAY BE FREED.
		CALL	INTERSECT		;NO: CHECK FOR INTERSECTION
		JC	DALERR2			;PART OR ALL OF BLOCK FREE.
		MOV	DI,ES:[DI]		;NO INTERSECTION: GET FLINK
		JMP	DAL_5			;AND KEEP CHECKING.
DAL_6:
		MOV	BX,SI			;ADDR OF BLOCK TO ES:BX
		CALL	X_FREE_BLOCK		;FREE IT.
		JMP	SHORT DALRET
DALERR2:					;BLOCK INTERSECTION ERROR:
DALERR:						;PARAMETER ERROR:
		MOV	AX,SVC_FAILED+PARM_ERR
		CALL	X_SFLAGS
DALRET:						;DEALLOCATION SUCCEEDED:
		RET
X_DALOC		ENDP
;
MTOS_CS		ENDS
		END
