PAGE	60,132
INCLUDE B:BIG.INC
;
;	BUGS.ASM:	EXTENSION SUPPORT SERVICES AND MISCELLANY FOR DEBUGGER.
;	F. W. PORRETTO, 7/7/82
;
;	THIS MODULE CONTAINS THE FOLLOWING:
;	1) THE CODE FOR THE DEBUGGER SUPPORT SERVICES:
;		-SET BREAKPOINT,
;		-KILL BREAKPOINT,
;		-MIGRATE TASK TO CPU (INEFFECTIVE IN UNIPROCESSING),
;		-HALT TASK,
;		-CONTINUE TASK,
;		-RETURN TCB ADDRESS FOR SPECIFIED TASK,
;		-RETURN TASK NUMBER FOR SPECIFIED TCB ADDRESS,
;		-SET TASK'S LOCAL EVENT FLAGS,
;		-CLEAR TASK'S LOCAL EVENT FLAGS,
;	...IMPLEMENTED AS ONE MTOS SERVICE.
;
;	2) THE INTERRUPT HANDLERS FOR BREAKPOINT AND SINGLE-STEP INTERRUPTS,
;	   TO BE USED ONLY WHEN THE DEBUGGER IS INSTALLED.
;
;	THE READER HAS PROBABLY ALREADY NOTICED THE BREAKPOINT AND SINGLE-STEP
;	HANDLERS IN MISCIN.ASM, WHICH IN THE DISTRIBUTION VERSION OF THE SYSTEM
;	ARE NOT DECLARED PUBLIC. THE USE OF THOSE MODULES IS RECOMMENDED FOR
;	NON-USERS OF THE MTOS-86 TASK DEBUGGER; DECLARE THEM PUBLIC IF YOU WILL
;	NOT BE USING THESE.
;
;	SYMBOLS FOR USE HERE:
;
;	1) THE SBREAK FUNCTION:
;
SBRK_CPU	EQU	1		;CPU ID.
SBRK_ID		EQU	SBRK_CPU+1	;BREAKPOINT ID.
SBRK_FLAGS	EQU	SBRK_ID+1	;BREAKPOINT FLAGS: SEE BELOW.
SBRK_OFF	EQU	SBRK_FLAGS+1	;BREAKPOINT OFFSET,
SBRK_SEG	EQU	SBRK_OFF+2	;           SEGMENT.
;
;	FLAGS FOR BREAKPOINT:
;
HARD_POINT	EQU	00000000B	;"HARD" BREAKPOINT: SURVIVES EXECUTION
SOFT_POINT	EQU	00000001B	;"SOFT" BREAKPOINT: DIES ON EXECUTION.
HALT_ALL	EQU	00000010B	;HALT ALL OTHER TASKS.
;
;	2) THE KBREAK FUNCTION:
;
KBRK_CPU	EQU	1		;CPU NUMBER
KBRK_ID		EQU	KBRK_CPU+1	;BREAKPOINT ID.
;
;	3) THE BREAKPOINT BLOCK (BPB): BUILT IN AN SMR BLOCK.
;
BPB_LINK	EQU	0		;FORWARD LINK OR 0.
BPB_ID		EQU	BPB_LINK+2	;BREAKPOINT ID.
BPB_FLAGS	EQU	BPB_ID+1	;BREAKPOINT FLAGS: AS IN SBREAK FCTN.
BPB_OFF		EQU	BPB_FLAGS+1	;BREAKPOINT OFFSET,
BPB_SEG		EQU	BPB_OFF+2	;           SEGMENT.
BPB_TCB		EQU	BPB_SEG+2	;TCB OF DEBUGGER WHICH SET THIS BKPT.
BPB_OPCODE	EQU	BPB_TCB+2	;OPCODE REPLACED BY INT 3.
BPB_SIZE	EQU	BPB_OPCODE+1	;SIZE OF BPB IN BYTES.
;
;	4) THE QSTRT PARAMETERS TO THE DEBUGGER ON BREAKPOINT EXECUTION:
;
BRK_ID		EQU	RCB_PARAM	;BREAKPOINT ID.
BRK_FLAGS	EQU	BRK_ID+1	;BREAKPOINT FLAGS.
BRK_CPU		EQU	BRK_FLAGS+1	;BKPT OCCURRED ON THIS CPU (1 BYTE)
BRK_TASK	EQU	BRK_CPU+2	;TASK ID OF ACTIVE TASK AT BREAKPOINT.
BRK_TCB		EQU	BRK_TASK+2	;TCB ADDRESS OF TASK.
BRK_POINT	EQU	BRK_TCB+4	;ADDRESS OF BREAKPOINT.
BRK_SIZE	EQU	BRK_POINT+4	;SIZE OF PARAMETER STRING IN BYTES
;
;	5) MISCELLANY
;
INT_ON		EQU	0200H		;(IF) INTERRUPTS-ON FLAG.
TRAP_ON		EQU	0100H		;(TF) TRAP FLAG FOR SINGLE-STEPPING
BREAK_INT	EQU	0CCH		;INT 3 FOR BREAKPOINT PLACEMENT.
;
		NAME	DEBUG_SUPPORT
;
;
MTOS_CS		SEGMENT PARA PUBLIC
		ASSUME	CS:MTOS_CS,DS:MTOS_DS,SS:MTOS_DS
		PUBLIC	X_DBGSVC,X_BPT,X_SS
		EXTRN	X_SCHDLR:NEAR
		EXTRN	X_GET_SMR:NEAR
		EXTRN	X_PUT_SMR:NEAR
		EXTRN	X_QUEUE:NEAR
		EXTRN	X_SFLAGS:NEAR
		EXTRN	X_GET_TCB_ADR:NEAR
		EXTRN	X_GET_TASK_ID:NEAR
		EXTRN	X_SET_READY:NEAR
		EXTRN	X_TASK_STATE:NEAR
		EXTRN	X_ELIPSE:FAR
		EXTRN	X_SYSTATE:FAR
		EXTRN	X_START_TASK:NEAR
		EXTRN	X_DQUEUE:NEAR
		EXTRN	X_SET_EF:NEAR
		EXTRN	X_RESET_EF:NEAR
;
PAGE
;
;	JUMP TABLE FOR SUBROUTINES:
;
DEB_JUMPS	DD	X_SBREAK	;SUBSERVICE 0: SET BREAKPOINT.
		DD	X_KBREAK	;    "      1: KILL BREAKPOINT.
		DD	X_MIGRATE	;    "      2: MIGRATE TASK TO CPU.
		DD	X_HALT		;    "      3: HALT SPECIFIED TASK.
		DD	X_CONT		;    "      4: UN-HALT SPECIFIED TASK.
		DD	X_TCBADR	;    "      5: TASK NR.-TO-TCB ADDRESS.
		DD	X_TASKID	;    "      6: TCB ADDRESS-TO-TASK NR.
		DD	X_SLOCEF	;    "      7: SET SPECIFIED TASK'S
					;              LOCAL EVENT FLAGS.
		DD	X_CLOCEF	;    "      8: RESET SPECIFIED TASK'S
					;              LOCAL EVENT FLAGS.
MAX_DBG_CODE	EQU	($-DEB_JUMPS)/4
PAGE
;********************************************************************
;*     X_DBGSVC:        SUBDISPATCHER FOR EXTENSION SUPPORT.        *
;********************************************************************
;
;	INPUT:	ES:SI=	SDB ADDRESS,
;		BX=	REQUESTOR'S TCB ADDRESS.
;	OUTPUT:	NONE.
;
;		THIS ROUTINE ACTS AS A SUB-ROUTER FOR THE DEBUGGER SUPPORT
;	CLUSTER, ALLOWING THE SET OF DEBUGGER SUPPORT SERVICES TO OCCUPY ONLY
;	ONE SVC POSITION.
;
X_DBGSVC	PROC	FAR
		MOV	AL,ES:[SI+1]		;GET SUPPORT SERVICE CODE
		CMP	AL,MAX_DBG_CODE		;LEGAL?
		JL	DB0			;YES.
		MOV	AX,SVC_FAILED+PARM_ERR	;NO.
		CALL	X_SFLAGS
		RET
DB0:
		XOR	AH,AH			;CLEAR AH
		SHL	AX,1			;MAKE INDEX INTO
		SHL	AX,1			;THE DOUBLEWORD-ENTRY
		MOV	DI,AX			;JUMP TABLE,
		INC	SI			;BYPASS SVC CODE AND
		PUSH	BX			;REWRITE THE ADVANCED OFFSET
		MOV	BX,XCSRB		;OVER THE VALUE CURRENTLY
		MOV	[BX+SRB_SI],SI		;STORED IN THE SRB, IN CASE THE
		POP	BX			;ROUTINE CALLED MIGRATES.
		JMP	DWORD PTR CS:DEB_JUMPS[DI]	;ENTER NEEDED ROUTINE.
X_DBGSVC	ENDP
PAGE
;*************************************************************
;*   KILL_BREAK:        KILL BREAKPOINT WHOSE BPB IS IN SI   *
;*************************************************************
;
;	INPUT:	DI=	BPB QUEUE HEAD,
;		SI=	BPB OF BREAKPOINT TO KILL.
;	OUTPUT:	NONE.
;
KILL_BREAK	PROC	NEAR
		PUSH	ES
		PUSH	BP
		XCHG	BX,SI		;GET BPB TO BX
		CALL	X_DQUEUE	;REMOVE FROM BPB QUEUE
		MOV	AL,[BX+BPB_OPCODE]	;GET OPCODE TO REPLACE
		LES	BP,DWORD PTR [BX+BPB_OFF]	;GET ADDRESS...
		MOV	ES:[BP],AL	;REPLACE OPCODE IN INSTRUCTION.
		CALL	X_PUT_SMR	;DEALLOCATE THE BPB
		XCHG	BX,SI
		POP	BP
		POP	ES
		RET
KILL_BREAK	ENDP
;
PAGE
;***********************************************************************
;*     MATCH_BREAK:     LOCATE THE BPB FOR A SPECIFIED BREAKPOINT.     *
;***********************************************************************
;
;	INPUT:	BX=	REQUESTING TCB ADDRESS
;		DI=	BPB QUEUE HEAD
;		DL=	BREAKPOINT ID.
;	OUTPUT:	SI RETURNS 0 FOR NOT FOUND, OR ADDRESS OF BPB IF FOUND.
;
MATCH_BREAK	PROC	NEAR
		PUSH	DI
MB0:
		MOV	SI,[DI]		;GET NEXT BPB
		TEST	SI,SI		;ANY?
		JNE	MB1		;YES.
		POP	DI		;NO.
		RET
MB1:
		CMP	BX,[SI+BPB_TCB]	;RIGHT DEBUGGER?
		JNE	MB2		;NO.
		CMP	DL,[SI+BPB_ID]	;YES: RIGHT BREAKPOINT?
		JNE	MB2		;NO.
		POP	DI		;YES: RETURN THIS ADDRESS.
		RET
MB2:
		MOV	DI,SI		;CURRENT BECOMES PREVIOUS
		JMP	SHORT MB0	;AND KEEP LOOKING.
MATCH_BREAK	ENDP
PAGE
;****************************************************************
;*     X_SBREAK:        SET BREAKPOINT ON SPECIFIED CPU.        *
;****************************************************************
;
;		THIS ROUTINE SETS A BREAKPOINT, IF POSSIBLE, IN THE MEMORY
;	OF THE SPECIFIED CPU. IF THE ID SPECIFIED IS ALREADY IN USE, THE 
;	REQUEST IS DISHONORED.
;
;	INPUT:	ES:SI=	ADJUSTED SDB ADDRESS,
;		BX=	REQUESTOR'S TCB ADDRESS.
;	OUTPUT:	NONE.
;
X_SBREAK	PROC	FAR
		MOV	DI,OFFSET X_BREAK_Q	;LOCATE BREAKPOINT QUEUE.
		MOV	BX,X_CTCB		;GET REQUESTOR'S TCB ADDRESS
		MOV	DL,ES:[SI+SBRK_ID]	;GET DESIRED BREAK ID.
		PUSH	SI			;SAVE SDB OFFSET A MOMENT.
		CALL	MATCH_BREAK		;IS BREAKPOINT IN USE?
		TEST	SI,SI			;HMM?
		POP	SI			;RETRIEVE SDB OFFSET AFTER TEST
		JE	XSB2			;NO.
		MOV	AX,SVC_FAILED+PARM_ERR	;YES:
XSB4:
		CALL	X_SFLAGS		;NO DICE.
		JMP	SHORT XSB5		;TELL USER HE FAILED.
XSB2:
		PUSH	ES
		LES	BP,DWORD PTR ES:[SI+SBRK_OFF]
		CMP	BYTE PTR ES:[BP],BREAK_INT	;ALREADY A BREAK?
		POP	ES
		JNE	XSB3			;NO, PROCEED.
		MOV	AX,SVC_FAILED+FLAG_CF	;YES:
		JMP	SHORT XSB4		;TELL USER HE BLEW IT.
XSB3:
		CALL	X_GET_SMR		;ALLOCATE A BPB
		MOV	[BX+BPB_ID],DL		;INSERT BREAKPOINT ID
		MOV	AL,ES:[SI+SBRK_FLAGS]
		MOV	[BX+BPB_FLAGS],AL	;INSERT HARD/SOFT FLAGS
		MOV	AX,X_CTCB
		MOV	[BX+BPB_TCB],AX		;INSERT DEBUGGER TCB ADDRESS
		LES	BP,DWORD PTR ES:[SI+SBRK_OFF]
		MOV	[BX+BPB_OFF],BP		;INSERT BREAKPOINT OFFSET,
		MOV	[BX+BPB_SEG],ES		;THEN   BREAKPOINT SEGMENT.
		CALL	X_QUEUE			;ATTACH BPB TO LIST.
		MOV	AL,ES:[BP]		;GET OPCODE AGAIN.
		MOV	[BX+BPB_OPCODE],AL	;SAVE IN BPB
		MOV	BYTE PTR ES:[BP],BREAK_INT	;AND DEPOSIT INTERRUPT
XSB5:
		MOV	BX,X_CTCB		;GET REQUESTOR'S TCB,
		MOV	AX,BLOCK		;MOVE HIM OUT OF SAFETY STATE
		CALL	X_TASK_STATE		;SO WE CAN RETURN HIM TO...
		CALL	X_SET_READY		;READY STATE.
		RET				;DONE.
X_SBREAK	ENDP
PAGE
;****************************************************************************
;*     X_KBREAK:        KILL BREAKPOINT IN THE MEMORY OF SPECIFIED CPU.     *
;****************************************************************************
;
;		THIS ROUTINE IS THE CONVERSE OF X_SBREAK.
;
;	INPUT:	ES:SI=	ADJUSTED SDB ADDRESS,
;		BX=	REQUESTOR'S TCB ADDRESS.
;	OUTPUT:	NONE.
;
X_KBREAK	PROC	FAR
		MOV	DL,ES:[SI+KBRK_ID]	;GET BREAKPOINT ID.
		MOV	DI,OFFSET X_BREAK_Q	;ADDRESS BREAKPOINT QUEUE.
		MOV	BX,X_CTCB		;GET REQUESTOR'S TCB.
		CALL	MATCH_BREAK		;DOES THIS ONE EXIST?
		TEST	SI,SI			;HMM?
		JNE	XKB2			;YES.
		MOV	AX,SVC_FAILED+PARM_ERR	;NO:
		CALL	X_SFLAGS		;TELL USER.
		JMP	SHORT XKB3
XKB2:
		CALL	KILL_BREAK		;KILL THE BREAKPOINT
XKB3:
		MOV	AX,BLOCK		;MOVE REQUESTOR OUT OF SAFETY
		CALL	X_TASK_STATE		;STATE SO THAT HE CAN BE
		CALL	X_SET_READY		;RETURNED TO READY STATE.
		RET
X_KBREAK	ENDP
;
PAGE
;**************************************************************************
;*    X_MIGRATE:        CAUSE REQUESTOR TO MIGRATE TO SPECIFIED CPU.      *
;**************************************************************************
;
;	INPUT:	ES:SI=	ADJUSTED SDB ADDRESS,
;		BX=	REQUESTOR'S TCB ADDRESS.
;	OUTPUT:	NONE.
;
;		THE CALLER TASK IS EXPECTED TO BE GLOBAL: NO ATTEMPT IS MADE TO
;	VERIFY THIS. HE IS BOUND TO THE CPU OF HIS CHOICE BY THE USE OF THE
;	TCB_PID FIELD.
;
;	** NOTE!! **
;		FOR OBVIOUS REASONS, THIS SERVICE HAS NO EFFECT IN A
;		UNIPROCESSING ENVIRONMENT.
;
X_MIGRATE	PROC	FAR
		MOV	AL,ES:[SI+1]		;GET CPU NUMBER.
		MOV	[BX+TCB_PID],AL		;PUT INTO TCB.
		RET				;WHEN THE SVC HANDLER GETS CON-
						;TROL ON RETURN, IT WILL CALL
						;X_SET_READY, WHICH WILL PUT
						;THE TCB ONTO THE CORRECT Q.
X_MIGRATE	ENDP
PAGE
;****************************************************
;*      X_HALT:         HALT THE SPECIFIED TASK.    *
;****************************************************
;
;	INPUT:	ES:SI=	ADJUSTED SDB ADDRESS,
;		BX=	REQUESTOR'S TCB ADDRESS.
;	OUTPUT:	NONE
;
;		THIS ROUTINE HALTS THE SPECIFIED TASK, IF THAT TASK IS:
;	1)NOT THE CALLER, AND
;	2)NOT DESIGNATED UN-HALT-ABLE (SEE TCD_FLAGS IN EQUS.EQU).
;
X_HALT		PROC	FAR
		MOV	AX,ES:[SI+1]		;GET TASK ID.
		CALL	X_GET_TCB_ADR		;MAKE TCB ADDRESS
		JNC	XH0			;GOOD.
XH1:
		MOV	AL,SVC_FAILED+PARM_ERR	;NO GOOD.
		CALL	X_SFLAGS
		RET
XH0:
		CMP	X_CTCB,BX		;IS TASK THE CALLER?
		JE	XH1			;YES, AGAIN REJECT REQUEST.
		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_NOHLT	;MAY HALT??
		JNZ	XC1			;NO, DESIGNATED UN-HALT-ABLE.
		CMP	BYTE PTR [BX+TCB_STATE],READY	;NO: IS TASK READY?
		JE	XH2			;YES, MAY BE BLOCKED.
		MOV	AX,SVC_FAILED+FLAG_CF	;NO:
		CALL	X_SFLAGS		;FAIL THE REQUEST.
		RET
XH2:
		MOV	AX,BLOCK+STA_HLT	;SET TASK'S STATE
		CALL	X_TASK_STATE		;TO DEBUGGER HALT.
		RET
X_HALT		ENDP
PAGE
;*************************************************************************
;*      X_CONT:         CONTINUE A TASK FROM EITHER BLOCK+STA_HLT OR     *
;*                      BLOCK+STA_BREAK STATUS.                          *
;*************************************************************************
;
;	INPUT:	ES:SI=	ADJUSTED SDB ADDRESS
;		BX=	REQUESTOR'S TCB ADDRESS
;	OUTPUT:	NONE.
;
;		THIS IS THE METHOD FOR CONTINUING TASKS HALTED BY THE X_HALT
;	SERVICE OR BY THE EXECUTION OF A BREAKPOINT.
;
X_CONT		PROC	FAR
		MOV	AX,ES:[SI+1]		;GET TASK ID.
		CALL	X_GET_TCB_ADR		;MAKE TCB ADDRESS
		JNC	XC0			;GOOD.
XC1:
		MOV	AX,SVC_FAILED+PARM_ERR	;NOT GOOD.
		CALL	X_SFLAGS
		RET
XC0:
		CMP	BYTE PTR [BX+TCB_STATE],BLOCK+STA_HLT
		JE	XC3			;ONLY READY THE TASK IF STATUS
		CMP	BYTE PTR [BX+TCB_STATE],BLOCK+STA_BREAK
		JE	XC3			;IS ONE OF THE TWO ALLOWED ONES
		MOV	AX,SVC_FAILED+FLAG_CF	;IT ISN'T
		CALL	X_SFLAGS		;SO FAIL THE REQUEST.
		RET
XC3:
		CALL	X_SET_READY		;IT IS.
		RET
X_CONT		ENDP
PAGE
;******************************************************************
;*     X_TCBADR:        RETURN TCB ADDRESS OF SPECIFIED TASK.     *
;******************************************************************
;
;	INPUT:	ES:SI=	ADJUSTED SDB ADDRESS,
;		BX=	REQUESTOR'S TCB ADDRESS.
;	OUTPUT:	NONE.
;
X_TCBADR	PROC	FAR
		MOV	AL,ES:[SI+1]		;GET TASK NUMBER
		CALL	X_GET_TCB_ADR		;CONVERT TO TCB ADDRESS
		JNC	XTCB0			;IT'S A TASK, ALRIGHT.
		MOV	AX,SVC_FAILED+PARM_ERR	;NO, IT ISN'T
		CALL	X_SFLAGS
		RET
XTCB0:
		LES	BP,DWORD PTR ES:[SI+3]	;GET ADDRESS OF DELIVERY BUFFER
		MOV	ES:[BP],BX		;INSERT TCB OFFSET,
		MOV	ES:[BP+2],DS		;           SEGMENT.
		RET
X_TCBADR	ENDP
PAGE
;********************************************************************
;*         X_TASKID:  RETURN TASK NUMBER FOR TCB ADDRESS GIVEN      *
;********************************************************************
;
;	INPUT:	ES:SI=	ADJUSTED SDB ADDRESS,
;		BX=	REQUESTOR'S TCB.
;	OUTPUT:	NONE.
;
;		THIS ROUTINE PERFORMS THE INVERSE FUNCTION FROM X_TCBADR.
;
X_TASKID	PROC	FAR
		MOV	BX,ES:[SI+1]		;GET TCB ADDRESS.
		CALL	X_GET_TASK_ID		;ATTEMPT CONVERSION.
		JNC	XTID0			;SUCCESS.
		MOV	AX,SVC_FAILED+PARM_ERR	;BAD TASK ID:
		CALL	X_SFLAGS		;RETURN PARAMETER ERROR.
		RET
XTID0:
		LES	DI,DWORD PTR ES:[SI+3]	;GET ADDRESS OF DELIVERY BUFFER
		MOV	ES:[DI],AX		;RETURN TASK NUMBER THERE.
		RET
X_TASKID	ENDP
PAGE
;**********************************************************************
;*          X_SLOCEF:	SET LOCAL EVENT FLAGS OF ANOTHER TASK         *
;**********************************************************************
;
;	INPUT:	ES:SI=	ADJUSTED SDB ADDRESS,
;		BX=	REQUESTING TASK.
;	OUTPUT:	NONE.
;
;		THIS ROUTINE PROVIDES LOCAL EVENT FLAG ACCESS TO THE FLAG
;	GROUPS OF OTHER TASKS, WHICH IS USEFUL WHEN THE REQUESTOR IS A
;	"UTILITY TASK" USED TO EXTEND THE POWERS OF THE SYSTEM; A FILE
;	SYSTEM IS A GOOD EXAMPLE.
;
X_SLOCEF	PROC	FAR
		MOV	AX,ES:[SI+1]		;GET TASK NUMBER.
		CALL	X_GET_TCB_ADR		;MAKE TCB ADDRESS FROM IT.
		JNC	SLOC0			;GOT IT.
		MOV	AX,SVC_FAILED+PARM_ERR	;NO GOOD:
		CALL	X_SFLAGS		;RETURN PARAMETER ERROR.
		RET
SLOC0:
		MOV	CX,ES:[SI+3]		;GET MASK VALUE,
		XOR	DL,DL			;SET GROUP NUMBER TO 0,
		CALL	X_SET_EF		;SET THE FLAGS.
		RET
X_SLOCEF	ENDP
PAGE
;*********************************************************************
;*         X_CLOCEF:	CLEAR ANOTHER TASK'S LOCAL EVENT FLAGS       *
;*********************************************************************
;
;	INPUT:	ES:SI=	ADJUSTED SDB ADDRESS,
;		BX=	REQUESTOR TCB.
;	OUTPUT:	NONE.
;
;		THIS ROUTINE IS THE FUNCTIONAL INVERSE OF X_SLOCEF.
;
X_CLOCEF	PROC	FAR
		MOV	AX,ES:[SI+1]		;GET TARGET TASK NUMBER.
		CALL	X_GET_TCB_ADR		;MAKE TCB ADDRESS...
		JNC	CLOC0			;GOT IT.
		MOV	AX,SVC_FAILED+PARM_ERR	;NOT A VALID TASK ID.
		CALL	X_SFLAGS
		RET
CLOC0:
		MOV	CX,ES:[SI+3]		;GET MASK WORD,
		XOR	DL,DL			;SPECIFY GROUP 0,
		CALL	X_RESET_EF		;PERFORM RESET OPERATION.
		RET
X_CLOCEF	ENDP
PAGE
;***********************************************************
;*      X_BPT:          BREAKPOINT INTERRUPT HANDLER.      *
;***********************************************************
;
;	ALL EXECUTION-TIME BREAKPOINT PROCESSING IS PERFORMED HERE.
;	MOST IMPORTANT IS THE QSTRTING OF THE SPECIFIED DEBUGGER TASK.
;
X_BPT		PROC	FAR
		CALL	X_ELIPSE
		MOV	BX,X_ATCB		;GET TCB ADDRESS.
		MOV	X_ATCB,0		;LIE TO THE SCHEDULER.
;
;	THE FOLLOWING FOUR INSTRUCTIONS SUPPORT THE USE OF THE 8087 NUMERIC
;	DATA COPROCESSOR. IF THAT DEVICE IS NOT USED, THEY SHOULD BE LEFT
;	COMMENTED OUT. ELSE, REMOVE THE LEADING SEMICOLONS.
;
;		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_8087	;8087 USER?
;		JE	XB10			;NO.
;		MOV	BP,[BX+TCB_8087]	;YES: GET SAVE AREA ADDRESS
;		FSAVE	DS:[BP]			;AND SAVE NDP'S REGISTERS.
;
XB10:
 		CALL	X_SYSTATE		;GO TO S STATE.
		MOV	X_CTCB,BX		;SAVE THE TCB WHERE IT
		MOV	SI,XCSRB		;IS EXPECTED TO BE.
		MOV	[SI+SRB_TCB],BX
		PUSH	DS			;SAVE MTOS_DS.
		LDS	DI,DWORD PTR [BX+TCB_SP]	;GET TASK STACK POINTER
		LES	BP,DWORD PTR [DI+STK_IP]	;GET ADDRESS OF BKPT.
		DEC	BP			;BACK UP IP ONE BYTE (INT 3)
		MOV	[DI+STK_IP],BP		;REINSERT IP INTO TASK STACK.
		POP	DS			;RETRIEVE MTOS_DS
		MOV	DI,OFFSET X_BREAK_Q	;ADDRESS BREAKPOINT LIST.
		PUSH	DI			;SAVE THAT ADDRESS.
		MOV	AX,ES			;SEGMENT TO AX FOR CONVENIENCE
XB0:
		MOV	SI,[DI]			;FORWARD TO NEXT BPB:
		TEST	SI,SI			;IS THERE ONE?
		JE	XB2A			;NO: SPURIOUS BREAKPOINT.
		CMP	[SI+BPB_OFF],BP		;YES: OFFSETS MATCH?
		JNE	XB1			;NO.
		CMP	[SI+BPB_SEG],AX		;YES: SEGMENTS MATCH?
		JE	XB3			;YES, FOUND IT!
XB1:
		MOV	DI,SI			;NO: CURRENT BPB --> PREVIOUS
		JMP	SHORT XB0		;AND GO AROUND AGAIN.
XB2A:
		MOV	AX,BLOCK+STA_ERR2	;BLOCK TASK FOR RUNTIME ERROR
		CALL	X_TASK_STATE		;(SPURIOUS BREAKPOINT)
		POP	DI			;CLEAN THE STACK,
		RET
XB3:
		MOV	AX,BLOCK+STA_BREAK	;SET THE TASK TO
		CALL	X_TASK_STATE		;THE BREAKPOINTED BLOCK STATE.
		MOV	DH,[SI+BPB_ID]		;GET BKPT ID TO DH,
		MOV	DL,[SI+BPB_FLAGS]	;BKPT FLAGS TO DL.
		MOV	CX,[SI+BPB_TCB]		;SAVE DEBUGGER TCB ADDR IN CX.
		TEST	DL,SOFT_POINT		;SOFT BREAKPOINT?
		JE	XB4			;NO, HARD: ENTER TRAP LOGIC.
		POP	DI			;SOFT: RESTORE QUEUE HEAD
		CALL	KILL_BREAK		;AND KILL THE BREAKPOINT.
		JMP	SHORT XB5		;BYPASS TRAP LOGIC.
XB4:
		MOV	AL,[SI+BPB_OPCODE]	;GET OPCODE FROM BPB
		MOV	ES:[BP],AL		;PUT IT BACK INTO INSTRUCTION.
		PUSH	ES
		LES	DI,DWORD PTR [BX+TCB_SP]	;GO TO TASK STACK AGAIN
		OR	WORD PTR ES:[DI+STK_FL],TRAP_ON	;TO SET SINGLESTEP FLAG
		POP	ES			;RESTORE BKPT SEGMENT ADDRESS
		POP	DI			;AND BP Q HEAD.
XB5:
		TEST	DL,HALT_ALL		;HALT ALL TASKS ON EXECUTION?
		JE	XB6			;NO.
		PUSH	BX			;YES: SAVE TASK'S TCB ADDRESS
		XOR	AX,AX			;AND START FROM TASK 0.
XB7:
		PUSH	AX			;SAVE TASK ID.
		CALL	X_GET_TCB_ADR		;GET TCB ADDRESS OF TASK.
		JC	XB8			;** NOT A TASK **
		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_NOHLT	;HALTABLE?
		JNE	XB8			;NO: PROBABLY A DEBUGGER.
		CMP	BYTE PTR [BX+TCB_STATE],READY	;YES: TASK READY?
		JNE	XB8			;NO, MUSTN'T MEDDLE.
		MOV	AX,BLOCK+STA_HLT	;YES:
		CALL	X_TASK_STATE		;HALT IT.
XB8:
		POP	AX			;RETRIEVE TASK ID.
		INC	AX			;ADVANCE IT.
		CMP	AX,X_TASK_NUM		;DONE?
		JB	XB7			;NO.
		POP	BX			;YES: RESTORE TCB ADDRESS
XB6:
		CALL	X_GET_TASK_ID		;GET BREAKPOINTED TASK'S ID.
		PUSH	AX			;SAVE THAT AND
		PUSH	BX			;TCB ADDRESS.
		CALL	X_GET_SMR		;GET SMR FOR RCB.
		POP	WORD PTR [BX+BRK_TCB]	;INSERT TCB ADDRESS,
		MOV	[BX+BRK_TCB+2],DS	;DWORD FORM.
		POP	WORD PTR [BX+BRK_TASK]	;INSERT TASK ID.
		MOV	[BX+BRK_POINT],BP	;INSERT BREAKPOINT ADDRESS:
		MOV	[BX+BRK_POINT+2],ES	;ANOTHER DWORD.
		MOV	[BX+BRK_FLAGS],DL	;INSERT BKPT FLAGS.
		MOV	AX,X_PID
		MOV	[BX+BRK_CPU],AL		;INSERT CPU NUMBER.
		MOV	[BX+BRK_ID],DH		;INSERT BREAKPOINT ID!
		MOV	DI,CX			;DEBUGGER TCB ADDR TO DI.
		XOR	SI,SI			;NO REQUESTOR.
		MOV	CX,255			;HIGH PRIORITY, NO COORDINATION
		MOV	BP,BX			;USE OUR RCB,
		CALL	X_START_TASK		;GO!!
		RET
X_BPT		ENDP
PAGE
;******************************************************************************
;*      X_SS:           SINGLE-STEP HANDLER WHICH DOES POST-EXECUTION PROCES- *
;*                      ING FOR HARD BREAKPOINTS.                             *
;******************************************************************************
;
;		THIS ROUTINE COLLABORATES WITH THE SCHEDULER FOR THIS CPU TO
;	HANDLE THE HARD BREAKPOINT SINGLE-STEP TRAP. THE SCHEDULER WILL HAVE
;	SAVED, IN X_TASK_IPCS, THE ADDRESS AT WHICH THE CURRENT TASK WAS SCHE-
;	DULED, WHICH IS OF COURSE WHERE THE SINGLE-STEP TRAP OCCURRED. THE
;	INT 3 INSTRUCTION IS RETURNED TO THAT LOCATION, THE TASK'S T-BIT IS
;	RETURNED TO 0, AND EXECUTION RESUMES.
;
;		TO UNDERSTAND WHY THIS HANDLER DOES WHAT IT DOES, IT IS NECES-
;	SARY TO UNDERSTAND THOROUGHLY THE NATURE OF THE 8086 SINGLESTEP TRAP
;	AND ITS RELATIONSHIP TO OTHER INTERRUPT POSSIBILITIES, HARD AND SOFT.
;	IF, AFTER THE EXECUTION OF ANY 8086 INSTRUCTION, BOTH THE T-BIT AND I-
;	BIT ARE ON, AND ANY INTERRUPT IS PENDING, THE STACK WILL BE THUS:
;
;		TOP:	IP OF INTERRUPT HANDLER,
;			CS OF INTERRUPT HANDLER,
;			FL AT TIME OF INTERRUPT, WITH T-BIT=I-BIT=0.
;			IP OF INTERRUPTED PROGRAM,
;			CS OF INTERRUPTED PROGRAM,
;			FL AT TIME OF INTERRUPT, WITH T-BIT=I-BIT=1.
;		(STACK TOP BEFORE INTERRUPT.)
;
;		IF NO INTERRUPT IS PENDING BUT THE T-BIT IS SET, THE STACK WILL
;	BE THUS:
;
;		TOP:	IP OF TRAPPED PROGRAM,
;			CS OF TRAPPED PROGRAM,
;			FL AT TIME OF TRAP, WITH T-BIT=1.
;		(STACK TOP BEFORE TRAP.)
;
;		IN ORDER TO MAKE HARD BREAKPOINTS POSSIBLE, THE T-BIT FUNCTION
;	IS NEEDED TO SUPPORT "BREAKPOINT TRANSPARENCY." SINCE A VALID AND VERY
;	COMMON 8086 INSTRUCTION IS THE SVC INTERRUPT, THE SINGLESTEP HANDLER
;	MUST COPE WITH BOTH POSSIBILITIES ABOVE FLUIDLY.
;
X_SS		PROC	FAR
		PUSH	BP		;SAVE THE BP REGISTER, WHICH WE NEED.
		PUSH	DI		;MAKE ONE POINTER'S WORTH OF WORKSPACE.
		PUSH	ES
		MOV	BP,SP		;MARK CURRENT STACK POSITION.
		MOV	DI,MTOS_DS	;SET UP TO REACH THE
		MOV	ES,DI		;MTOS RAM DATA SEGMENT.
		LES	DI,ES:X_TASK_IPCS	;GET TRAP POINT ADDRESS.
		TEST	WORD PTR [BP+10],TRAP_ON	;INTERVENING INTERRUPT?
		JNE	XSS0		;NOT IF T-BIT IS SET.
		ADD	BP,6		;YES: SKIP THREE STACK WORDS, TO POINT
					;TO IP OF TRAPPED INTERRUPT HANDLER.
		CMP	DI,[BP+6]	;TRAP OFFSET=SAVED OFFSET?
		JNE	XSS0		;NO.
		MOV	DI,ES
		CMP	DI,[BP+8]	;YES: TRAP SEGMENT=SAVED SEGMENT?
		JNE	XSS0		;NO.
		POP	ES		;YES: INTERRUPT LATCHED BEFORE TRAPPED
		POP	DI		;INSTRUCTION COULD BE EXECUTED: JUST
		POP	BP		;RETURN AND WAIT FOR NEXT TRAP!
		IRET
XSS0:
		AND	WORD PTR [BP+10],NOT TRAP_ON	;QUENCH THE T-BIT.
		MOV	BYTE PTR ES:[DI],BREAK_INT	;INSTALL INT 3 AGAIN,
		POP	ES
		POP	DI
		POP	BP
		IRET			;ALL DONE!
X_SS		ENDP
MTOS_CS		ENDS
		END
