PAGE	60,132
;
;	C.ASM:	C-86 -TO- MTOS-86 INTERFACE FOR LARGE-MODEL C-86 TASKS.
;	F. W. PORRETTO,	8/10/82
;
;	THIS MODULE CONTAINS ALL THE CODE REQUIRED TO PROVIDE A SERVICE
;	INTERFACE USABLE BY INTEL C-86 PROGRAMS. ALL THE BASIC SERVICES
;	ARE SUPPORTED. ANY HIGH-LEVEL LANGUAGE WHOSE PARAMETER-PASSING CON-
;	VENTIONS ARE A CLOSE MATCH TO THOSE OF C-86 MAY BE FITTED TO THIS
;	INTERFACE, THOUGH SOME CUSTOMIZATION OF THESE ROUTINES MAY BE REQUIRED.
;
;	THE REASON FOR THE REFERENCE TO INTEL C-86 SPECIFICALLY IS THAT THIS
;	INTERFACE WAS TESTED WITH THAT COMPILER. HOWEVER, THE INTERFACE STAN-
;	DARD FOR LINKAGE BETWEEN C FUNCTIONS SHOULD MAKE THIS INTERFACE USABLE
;	WITH OTHER LARGE-MODEL COMPILERS AS WELL. NOTE: EVERY INTERFACE ROUTINE
;	PROVIDED HERE HAS A NAME SUFFIXED BY AN UNDERSCORE, BECAUSE THE INTEL
;	C-86 COMPILER AUTOMATICALLY SUFFIXES **ALL** FUNCTION NAMES, WHETHER
;	INTERNAL OR EXTERNAL TO THE MODULE BEING COMPILED, WITH THE UNDERSCORE
;	CHARACTER. THESE UNDERSCORES MAY BE REMOVED IF THEY OBSTRUCT THE USE OF
;	THE INTERFACE WITH ANOTHER COMPILER; THEY ARE NOT SEMANTICALLY IMPOR-
;	TANT.
;
;	THE ROUTINES WHICH PROVIDE THE INTERFACE ARE MULTIPROCESSOR-CO-ENTRANT,
;	A REQUIREMENT FOR THIS PARTICULAR APPLICATION. THE REQUIRED SDBS ARE
;	BUILT ON THE REQUESTING TASK'S STACK, AND SUBMITTED TO MTOS VIA THE
;	TYPE 1 SVC (SDB ADDRESS PUSHED BEFORE INTERRUPT INSTRUCTION). THERE IS
;	A REQUIREMENT FOR A RATHER UNUSUAL KIND OF STACK MAINTENANCE, OWING
;	TO THE ORDER OF EVENTS AND THE WAY IN WHICH VARIOUS REQUIRED ITEMS ARE
;	SAVED:
;
;	1)AT THE "BOTTOM" OF THE STACK WILL BE THE USER'S PARAMETERS. C-86
;	  MAINTAINS A CALL-BY-VALUE CONVENTION, AND ARRANGES ITS ARGUMENTS ON
;	  THE STACK IN ORDER OF POSITION IN THE CALLING SEQUENCE.
;
;	2)ABOVE THAT IS THE RETURN ADDRESS TO THE TASK, WHICH IS ASSUMED TO BE
;	  A C FUNCTION COMPILED IN THE LARGE MODEL.
;
;	3)ABOVE THAT, THE INTERFACE ROUTINES ALLOCATE SPACE IN WHICH TO BUILD
;	  THE SDB FOR THE REQUESTED FUNCTION. IT SHOULD ALSO BE OBSERVED HERE
;	  THAT THE INTERFACE DOES ALL NECESSARY SHIFTING AND PARAMETER COMBI-
;	  NING, BUT NO VALIDATION.
;
;	4)ABOVE THAT, AT THE TOP, ARE THE REGISTERS OF THE CALLING TASK, SAVED
;	  AS NECESSARY, AND FOR A SHORT TIME THE ADDRESS OF THE SDB AS IT IS
;	  SUBMITTED TO MTOS.
;
;	THE READER WILL NOTE THAT THE FUNDAMENTAL OPERATIONS OF ANY SELECTED
;	ROUTINE ARE MUCH THE SAME AS THOSE OF ANY OTHER. TO CONSERVE MEMORY,
;	SEVERAL ROUTINES SHARE PROCESSING WITH ONE ANOTHER. THE ACTUAL GOINGS-
;	ON SHOULD BE FAIRLY OBVIOUS, GIVEN A KNOWLEDGE OF THE MTOS-86 SDB FOR-
;	MATS AND BEARING IN MIND THE STACK CONTROL SCHEME DESCRIBED ABOVE.
;
		NAME	C_TO_MTOS
;
C_CS		SEGMENT PARA PUBLIC
		ASSUME	CS:C_CS
;
		PUBLIC	ALOC_,CNTRL_,CPRI_,DALOC_,DSCRTE_,ENDT_,ENDR_,PAUSE_
		PUBLIC	QSTRT_,STRT_,RECV_,REF_,RESUME_,RIO_,RSF_,SEF_
		PUBLIC	SEFET_,SEND_,STIME_,TEF_,TIME_,TSF_,WEF_,QCOPY_
		PUBLIC	CREATE_,DELETE_,DELTSK_,LOADED_,HSFS_
                PUBLIC	GETPARM_,SVCOK_
;
;	SPECIAL NOTE!!
;
;		IN THIS, THE IBM PC MACROASSEMBLER VERSION OF MTOS-86UP V3.0,
;	IT HAS BEEN NECESSARY TO MAKE A SLIGHT CHANGE TO ACCOMMODATE A FAULT
;	IN THE ASSEMBLER: THE VECTOR NUMBERS FOR THE TWO SVC INTERRUPT IN-
;	STRUCTIONS ARE DECLARED LOCALLY TO THIS MODULE, THUS:
;
X_SVC_VECTOR	EQU	186
;
PAGE
;
;	TYPE 1 SVC GENERATOR:
;
;	THIS ROUTINE PRESUMES THAT AT SS:BP IS THE SDB, AND THAT AT 
;	SS:[BP][DI] IS THE C-86 CALLING PROCEDURE'S STATUS VARIABLE,
;	INTO WHICH THE FLAGS REGISTER IS FORCED ON RETURN FROM MTOS.
;
MTOS		PROC	NEAR
		PUSH	SS			;STACK THE ADDRESS OF THE
		PUSH	BP			;SDB JUST BUILT.
		INT	LOW X_SVC_VECTOR	;INVOKE MTOS.
		POP	BP			;REMOVE THE SDB ADDRESS
		POP	SS			;FROM THE STACK.
		PUSHF				;SAVE RETURNED STATUS.
		LDS	DI,DWORD PTR [BP][DI]	;GET ADDRESS OF STATUS WORD
		POP	WORD PTR [DI]		;RETURN STATUS WORD FILLED.
		RET
MTOS		ENDP
PAGE
;
;	SYMBOLS FOR ALOC:
;
ALOCS_WTID	EQU	1	;SDB:	WAIT PARAMETER, POOL ID.
ALOCS_SZE	EQU	2	;	SIZE OF REQUIRED BLOCK.
ALOCS_BUFR	EQU	4	;	ADDRESS OF BLOCK POINTER BUFFER.
ALOCS_SIZE	EQU	8	;ALLOCATED FOR SDB.
ALOC_WT		EQU	12	;C-86:	WAIT PARAMETER,
ALOC_ID		EQU	14	;	POOL ID,
ALOC_SZE	EQU	16	;	SIZE OF REQUIRED BLOCK,
ALOC_BUFR	EQU	18	;	ADDRESS OF BLOCK POINTER BUFFER,
ALOC_STS	EQU	22	;	ADDRESS OF STATUS WORD.
ALOC_SIZE	EQU	ALOC_STS-ALOCS_SIZE	;USED BY C-86.
;
;
ALOC_		PROC	FAR
		SUB	SP,ALOCS_SIZE		;ALLOW FOR SDB
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],0FH	;ALOC SERVICE CODE.
		MOV	AL,[BP+ALOC_WT]
		ROR	AL,1
		OR	AL,[BP+ALOC_ID]
		MOV	[BP+ALOCS_WTID],AL	;WAIT PARAM AND POOL ID.
		MOV	AX,[BP+ALOC_SZE]
		MOV	[BP+ALOCS_SZE],AX	;SIZE OF BLOCK NEEDED.
		LDS	DI,DWORD PTR [BP+ALOC_BUFR]
		MOV	[BP+ALOCS_BUFR],DI	;BUFFER OFFSET
		MOV	[BP+ALOCS_BUFR+2],DS	;       SEGMENT
		MOV	DI,ALOC_STS
		CALL	MTOS			;TYPE 1 SVC
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,ALOCS_SIZE		;DESTROY SDB
		RET				;DONE.
ALOC_		ENDP
PAGE
;
;	SYMBOLS FOR CNTRL:
;
CNTRLS_SE	EQU	1	;SDB:	ENTER/EXIT INDICATOR.
CNTRLS_CID	EQU	2	;	CONTROLLED PROGRAM ID.
CNTRLS_SIZE	EQU	4	;ALLOCATED FOR SDB.
CNTRL_SE	EQU	8	;C-86:	ENTER/EXIT INDICATOR.
CNTRL_CID	EQU	10	;	CONTROLLED PROGRAM ID.
CNTRL_STS	EQU	12	;	ADDRESS OF STATUS WORD.
CNTRL_SIZE	EQU	CNTRL_STS-CNTRLS_SIZE	;USED BY C-86
;
;
CNTRL_		PROC	FAR
		SUB	SP,CNTRLS_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	AL,[BP+CNTRL_SE]
		ROR	AL,1
		MOV	[BP+CNTRLS_SE],AL	;ENTER/EXIT FLAG.
		MOV	AL,[BP+CNTRL_CID]
		MOV	[BP+CNTRLS_CID],AL	;CONTROLLED PROGRAM ID.
		MOV	BYTE PTR [BP],011H	;CNTRL CODE.
		MOV	DI,CNTRL_STS
		CALL	MTOS			;TYPE 1 SVC.
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,CNTRLS_SIZE		;DESTROY SDB
		RET				;DONE.
CNTRL_		ENDP
PAGE
;
;	SYMBOLS FOR CPRI:
;
CPRIS_NPS	EQU	1	;SDB:	NEW-PRIORITY-SOURCE SELECTOR,
CPRIS_PRIV	EQU	2	;	ABSOLUTE PRIORITY IF USED,
CPRIS_SIZE	EQU	4	;ALLOCATED FOR SDB.
CPRI_NPS	EQU	8	;C-86:	NEW-PRIORITY-SOURCE SELECTOR,
CPRI_PRIV	EQU	10	;	ABSOLUTE PRIORITY IF USED,
CPRI_STS	EQU	12	;	ADDRESS OF STATUS WORD.
CPRI_SIZE	EQU	CPRI_STS-CPRIS_SIZE	;USED BY C-86.
;
;
CPRI_		PROC	FAR
		SUB	SP,CPRIS_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	AL,[BP+CPRI_NPS]
		MOV	[BP+CPRIS_NPS],AL	;PRIORITY SELECTOR.
		MOV	AL,[BP+CPRI_PRIV]
		MOV	[BP+CPRIS_PRIV],AL	;(MAYBE) PRIORITY VALUE.
		MOV	BYTE PTR [BP],05H	;CPRI CODE.
		MOV	DI,CPRI_STS
		CALL	MTOS			;TYPE 1 SVC.
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,CPRIS_SIZE		;DESTROY SDB.
		RET				;DONE.
CPRI_		ENDP
PAGE
;
;	SYMBOLS FOR DALOC:
;
DALOCS_SZE	EQU	2	;SDB:	SIZE OF DEALLOCATED BLOCK.
DALOCS_BUFR	EQU	4	;	ADDRESS OF BLOCK POINTER BUFFER.
DALOCS_SIZE	EQU	8	;ALLOCATED FOR SDB.
DALOC_SZE	EQU	12	;C-86:	SIZE OF DEALLOCATED BLOCK.
DALOC_BUFR	EQU	14	;	ADDRESS OF BLOCK POINTER BUFFER.
DALOC_STS	EQU	18	;	ADDRESS OF STATUS WORD.
DALOC_SIZE	EQU	DALOC_STS-DALOCS_SIZE	;USED BY C-86.
;
;
DALOC_		PROC	FAR
		SUB	SP,DALOCS_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	AX,[BP+DALOC_SZE]
		MOV	[BP+DALOCS_SZE],AX	;SIZE OF BLOCK.
		LDS	DI,DWORD PTR [BP+DALOC_BUFR]
		MOV	[BP+DALOCS_BUFR],DI
		MOV	[BP+DALOCS_BUFR+2],DS	;BUFFER POINTER
		MOV	BYTE PTR [BP],010H	;DALOC CODE.
		MOV	DI,DALOC_STS
		CALL	MTOS			;TYPE 1 SVC.
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,DALOCS_SIZE		;DESTROY SDB.
		RET				;DONE.
DALOC_		ENDP
PAGE
;
;	SYMBOLS FOR DSCRTE:
;
DSCRTES_IOTYP	EQU	1	;SDB:	TYPE OF I/O REQUIRED, 'AND' OR 'OR',
DSCRTES_GRP	EQU	2	;	DISCRETE GROUP ID,
DSCRTES_BITS	EQU	4	;	DISCRETE BITS TO AFFECT.
DSCRTES_SIZE	EQU	6	;ALLOCATED FOR SDB.
DSCRTE_IO	EQU	10	;C-86:	INPUT OR OUTPUT,
DSCRTE_TYP	EQU	12	;	'AND' OR 'OR',
DSCRTE_GRP	EQU	14	;	DISCRETE GROUP ID,
DSCRTE_BITS	EQU	16	;	DISCRETE BITS TO AFFECT.
DSCRTE_STS	EQU	18	;	ADDRESS OF STATUS WORD.
DSCRTE_SIZE	EQU	DSCRTE_STS-DSCRTES_SIZE	;USED BY C-86.
;
;
DSCRTE_		PROC	FAR
		SUB	SP,DSCRTES_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],014H	;DSCRTE CODE.
		MOV	AL,[BP+DSCRTE_TYP]
		ROR	AL,1
		OR	AL,[BP+DSCRTE_IO]
		MOV	[BP+DSCRTES_IOTYP],AL	;IO AND TYPE COMBINED.
		MOV	AX,[BP+DSCRTE_GRP]
		MOV	[BP+DSCRTES_GRP],AX	;GROUP NUMBER
		MOV	AL,[BP+DSCRTE_BITS]
		MOV	[BP+DSCRTES_BITS],AX	;BIT PATTERN.
		MOV	DI,DSCRTE_STS
		CALL	MTOS			;TYPE 1 SVC
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,DSCRTES_SIZE		;DESTROY SDB.
		RET				;DONE.
DSCRTE_		ENDP
PAGE
;
;	SYMBOLS FOR ENDR:
;
ENDRS_RSBTME	EQU	1	;SDB:	RESTART BASE AND TIME-UNIT CODE,
ENDRS_LEN	EQU	2	;	NUMBER OF TIME-UNITS.
ENDRS_SIZE	EQU	4	;ALLOCATED FOR SDB.
ENDR_RSB	EQU	8	;C-86:	RESTART BASE,
ENDR_TME	EQU	10	;	TIME-UNIT CODE,
ENDR_LEN	EQU	12	;	NUMBER OF TIME-UNITS.
;
;
ENDR_		PROC	FAR
		SUB	SP,ENDRS_SIZE		;ALLOW FOR SDB.
		MOV	BP,SP			;MARK THE SDB.
		MOV	BYTE PTR [BP],04H	;ENDR CODE.
		MOV	AL,[BP+ENDR_RSB]
		ROR	AL,1
		MOV	AH,[BP+ENDR_TME]
		SHL	AH,1
		SHL	AH,1
		OR	AL,AH
		MOV	[BP+ENDRS_RSBTME],AL	;RESTART BASE AND UNIT COMBINED
		MOV	AL,[BP+ENDR_LEN]
		MOV	[BP+ENDRS_LEN],AL	;NR OF TIME UNITS
		CALL	MTOS			;NO RETURN FROM ENDR.
ENDR_		ENDP
;
;	SPECIAL VERSION OF MTOS'S ENDT SERVICE:
;
;	UNLIKE THE STANDARD VERSION CALLED FROM ASSEMBLY-LANGUAGE TASKS, THIS
;	VERSION ACCEPTS A PARAMETER: THE "COMPLETION CODE." THAT WORD VALUE IS
;	PUT IN AX BEFORE ISSUING THE SVC, SO THAT IF THE TERMINATING TASK WAS
;	STARTED WITH WAIT-TILL-TERMINATION COORDINATION, THE COMPLETION CODE
;	WILL BE MADE AVAILABALE TO THE STARTING TASK. THE STARTER CAN RECEIVE
;	THE COMPLETION CODE BY ASSIGNING THE INTEGER RETURNED BY THE STRT OR
;	QSTRT FUNCTION (WHICHEVER WAS USED) TO AN INTEGER VARIABLE OF ITS OWN.
;
ENDT_		PROC	FAR
		ADD	SP,4			;BYPASS USELESS RETURN ADDRESS.
		POP	AX			;RETURN CODE IS UNDERNEATH IT.
		INT	LOW X_SVC_VECTOR+1
		DD	ENDT_SDB
ENDT_		ENDP
ENDT_SDB	DB	03H		;ENDT CODE.
PAGE
;
;	SYMBOLS FOR PAUSE:
;
PAUSES_TME	EQU	1	;SDB:	TIME-UNIT CODE,
PAUSES_LEN	EQU	2	;	NUMBER OF TIME-UNITS,
PAUSES_SIZE	EQU	4	;ALLOCATED FOR SDB.
PAUSE_TME	EQU	8	;C-86:	TIME-UNIT CODE,
PAUSE_LEN	EQU	10	;	NUMBER OF TIME-UNITS,
PAUSE_STS	EQU	12	;	ADDRESS OF STATUS WORD.
PAUSE_SIZE	EQU	PAUSE_STS-PAUSES_SIZE
;
;
PAUSE_		PROC	FAR
		SUB	SP,PAUSES_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],06H	;PAUSE CODE.
		MOV	AL,[BP+PAUSE_TME]
		SHL	AL,1
		SHL	AL,1
		MOV	[BP+PAUSES_TME],AL	;TIME UNIT
		MOV	AL,[BP+PAUSE_LEN]
		MOV	[BP+PAUSES_LEN],AL	;NUMBER OF UNITS.
		MOV	DI,PAUSE_STS
		CALL	MTOS			;TYPE 1 SVC
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,PAUSES_SIZE		;DESTROY SDB
		RET				;DONE.
PAUSE_		ENDP
PAGE
;
;	SYMBOLS FOR STRT AND QSTRT:
;
STRTS_TP	EQU	1	;SDB:	COORDINATION AND PRIORITY SELECTOR,
STRTS_TASK	EQU	2	;	TASK ID,
STRTS_PRIV	EQU	4	;	ABSOLUTE PRIORITY IF USED,
STRTS_P1	EQU	5	;	PARAMETER WORD 1,
STRTS_P2	EQU	7	;	PARAMETER WORD 2.
STRTS_SIZE	EQU	10	;ALLOCATED FOR SDB.
STRT_TCORD	EQU	14	;C-86:	COORDINATION SELECTOR,
STRT_TASK	EQU	16	;	TASK ID,
STRT_PRIS	EQU	18	;	PRIORITY SELECTOR,
STRT_PRIV	EQU	20	;	ABSOLUTE PRIORITY IS USED,
STRT_P1		EQU	22	;	PARAMETER WORD 1,
STRT_P2		EQU	24	;	PARAMETER WORD 2.
STRT_STS	EQU	26	;	ADDRESS OF STATUS WORD.
STRT_SIZE	EQU	STRT_STS-STRTS_SIZE	;USED BY C-86.
;
;
QSTRT_		PROC	FAR
		SUB	SP,STRTS_SIZE		;ALLOW FOR SDB.
		PUSH	CX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],02H	;QSTRT CODE.
		JMP	STRT_IN			;TRANSFER TO STRT ROUTINE.
QSTRT_		ENDP
;
STRT_		PROC	FAR
		SUB	SP,STRTS_SIZE
		PUSH	CX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB AS ABOVE.
		MOV	BYTE PTR [BP],01H	;STRT CODE.
STRT_IN		LABEL	NEAR
		MOV	AL,[BP+STRT_TCORD]
		MOV	CL,5
		SHL	AL,CL
		OR	AL,[BP+STRT_PRIS]
		MOV	[BP+STRTS_TP],AL	;COORDINATION AND PRIORITY SEL.
		MOV	AX,[BP+STRT_TASK]
		MOV	[BP+STRTS_TASK],AX	;TASK NUMBER.
		MOV	AL,[BP+STRT_PRIV]
		MOV	[BP+STRTS_PRIV],AL	;(MAYBE) PRIORITY VALUE.
		MOV	AX,[BP+STRT_P1]
		MOV	[BP+STRTS_P1],AX	;PARAMETER 1
		MOV	AX,[BP+STRT_P2]
		MOV	[BP+STRTS_P2],AX	;PARAMETER 2
		MOV	DI,STRT_STS
		CALL	MTOS			;TYPE 1 SVC
;
;	NOTE THAT AX IS UNDISTURBED AFTER THE RETURN FROM MTOS, SO THAT STRT
;	AND QTSRT REQUESTS WITH COORDINATION MODE 2 (WAIT FOR TERMINATION) CAN
;	RECEIVE A COMPLETION CODE FROM THE TERMINATING TASK.
;
		POP	DS
		POP	DI
		POP	BP
		POP	CX
		ADD	SP,STRTS_SIZE		;DESTROY SDB.
		RET				;DONE.
STRT_		ENDP
PAGE
;
;	SYMBOLS FOR RECV:
;
RECVS_WT	EQU	1	;SDB:	WAIT SELECTOR,
RECVS_0		EQU	2	;	RESERVED BYTE,
RECVS_BOX	EQU	4	;	MAILBOX ID,
RECVS_BUFR	EQU	5	;	ADDRESS OF 6-BYTE BUFFER FOR MESSAGE.
RECVS_SIZE	EQU	10	;ALLOCATED FOR SDB.
RECV_WT		EQU	14	;C-86:	WAIT SELECTOR,
RECV_BOX	EQU	16	;	MAILBOX ID,
RECV_BUFR	EQU	18	;	ADDRESS OF 6-BYTE BUFFER FOR MESSAGE,
RECV_STS	EQU	22	;	ADDRESS OF STATUS WORD.
RECV_SIZE	EQU	RECV_STS-RECVS_SIZE	;USED BY C-86
;
;
RECV_		PROC	FAR
		SUB	SP,RECVS_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	AL,[BP+RECV_WT]
		ROR	AL,1
		MOV	[BP+RECVS_WT],AL	;WAIT PARAMETER.
		MOV	WORD PTR [BP+RECVS_0],0	;ZAP THE RESERVED FIELDS.
		MOV	AL,[BP+RECV_BOX]
		MOV	[BP+RECVS_BOX],AL	;BOX NUMBER
		LDS	DI,DWORD PTR [BP+RECV_BUFR]
		MOV	[BP+RECVS_BUFR],DI	;BUFFER OFFSET,
		MOV	[BP+RECVS_BUFR+2],DS	;SEGMENT.
		MOV	BYTE PTR [BP],0EH	;RECV CODE.
		MOV	DI,RECV_STS
		CALL	MTOS			;TYPE 1 SVC.
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,RECVS_SIZE		;DESTROY SDB.
		RET				;DONE.
RECV_		ENDP
PAGE
;
;	SYMBOLS FOR REF, SEF
;
REFS_GRP	EQU	1	;SDB:	EVENT FLAG GROUP ID,
REFS_FLAGS	EQU	2	;	FLAG BITS TO AFFECT.
REFS_SIZE	EQU	4	;ALLOCATED FOR SDB.
REF_GRP		EQU	8	;C-86:	EVENT FLAG GROUP ID,
REF_FLAGS	EQU	10	;	FLAG BITS TO AFFECT,
REF_STS		EQU	12	;	ADDRESS OF STATUS WORD.
REF_SIZE	EQU	REF_STS-REFS_SIZE	;USED BY C-86.
;
;
REF_		PROC	FAR
		SUB	SP,REFS_SIZE
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8		;MARK THE SDB.
		MOV	BYTE PTR [BP],09H	;REF CODE.
REF_IN		LABEL	NEAR
		MOV	AL,[BP+REF_GRP]
		MOV	[BP+REFS_GRP],AL	;GROUP NUMBER.
		MOV	AX,[BP+REF_FLAGS]
		MOV	[BP+REFS_FLAGS],AX	;FLAG PATTERN.
		MOV	DI,REF_STS
		CALL	MTOS			;TYPE 1 SVC
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,REFS_SIZE		;DESTROY THE SDB
		RET				;DONE.
REF_		ENDP
;
SEF_		PROC	FAR
		SUB	SP,REFS_SIZE
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],08H	;SEF CODE.
		JMP	REF_IN
SEF_		ENDP
PAGE
;
;	SYMBOLS FOR RESUME:
;
RESUMES_TASK	EQU	1	;SDB:	TASK ID.
RESUMES_SIZE	EQU	4	;ALLOCATED FOR SDB.
RESUME_TASK	EQU	8	;C-86:	TASK ID,
RESUME_STS	EQU	10	;	ADDRESS OF STATUS WORD.
RESUME_SIZE	EQU	RESUME_STS-RESUMES_SIZE	;USED BY C-86
;
;
RESUME_		PROC	FAR
		SUB	SP,RESUMES_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],07H	;RESUME CODE.
		MOV	AX,[BP+RESUME_TASK]
		MOV	[BP+RESUMES_TASK],AX	;TASK NUMBER.
		MOV	DI,RESUME_STS
		CALL	MTOS			;TYPE 1 SVC.
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,RESUMES_SIZE		;DESTROY THE SDB.
		RET				;DONE.
RESUME_		ENDP
PAGE
;
;	SYMBOLS FOR RIO:
;
RIOS_FUNC	EQU	1	;SDB:	FUNCTION CODE OF DRIVER SERVICE,
RIOS_DEV	EQU	2	;	DEVICE ID,
RIOS_UNIT	EQU	3	;	UNIT ID,
RIOS_GRP	EQU	4	;	EVENT FLAG GROUP ID,
RIOS_FLAGS	EQU	5	;	FLAG BITS TO SET AT COMPLETION,
RIOS_STA	EQU	7	;	ADDRESS OF I/O STATUS DOUBLEWORD,
RIOS_PARAM	EQU	11	;	ADDRESS OF I/O PARAMETER LIST,
RIOS_SIZE	EQU	16	;ALLOCATED FOR SDB.
RIO_FUNC	EQU	20	;C-86:	FUNCTION CODE OF DRIVER SERVICE,
RIO_DEV		EQU	22	;	DEVICE ID,
RIO_UNIT	EQU	24	;	UNIT ID,
RIO_GRP		EQU	26	;	EVENT FLAG GROUP ID,
RIO_FLAGS	EQU	28	;	FLAG BITS TO SET AT COMPLETION,
RIO_STA		EQU	30	;	ADDRESS OF I/O STATUS DOUBLEWORD,
RIO_PARAM	EQU	34	;	ADDRESS OF I/O PARAMETER LIST,
RIO_STS		EQU	38	;	ADDRESS OF MTOS STATUS WORD.
RIO_SIZE	EQU	RIO_STS-RIOS_SIZE	;USED BY C-86.
;
;
RIO_		PROC	FAR
		SUB	SP,RIOS_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],015H	;RIO CODE.
		MOV	AL,[BP+RIO_FUNC]
		MOV	[BP+RIOS_FUNC],AL	;I/O FUNCTION CODE.
		MOV	AL,[BP+RIO_DEV]
		MOV	[BP+RIOS_DEV],AL	;DEVICE
		MOV	AL,[BP+RIO_UNIT]
		MOV	[BP+RIOS_UNIT],AL	;UNIT
		MOV	AL,[BP+RIO_GRP]
		MOV	[BP+RIOS_GRP],AL	;FLAG GROUP
		MOV	AX,[BP+RIO_FLAGS]
		MOV	[BP+RIOS_FLAGS],AX	;FLAG PATTERN
		LDS	DI,DWORD PTR [BP+RIO_STA]
		MOV	[BP+RIOS_STA],DI
		MOV	[BP+RIOS_STA+2],DS	;STATUS BUFFER POINTER
		LDS	DI,DWORD PTR [BP+RIO_PARAM]
		MOV	[BP+RIOS_PARAM],DI
		MOV	[BP+RIOS_PARAM+2],DS	;PARAMETER STRING POINTER.
		MOV	DI,RIO_STS
		CALL	MTOS			;TYPE 1 SVC
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,RIOS_SIZE		;DESTROY THE SDB
		RET				;DONE.
RIO_		ENDP
PAGE
;
;	SYMBOLS FOR RSF, TSF:
;
RSFS_SID	EQU	1	;SDB:	SEMAPHORE ID.
RSFS_SIZE	EQU	4	;ALLOCATED FOR SDB.
RSF_SID		EQU	8	;C-86:	SEMAPHORE ID.
RSF_STS		EQU	10	;	ADDRESS OF STATUS WORD.
RSF_SIZE	EQU	RSF_STS-RSFS_SIZE	;USED BY C-86.
;
;
RSF_		PROC	FAR
		SUB	SP,RSFS_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],16H	;RSF CODE.
RSF_IN		LABEL	NEAR
		MOV	AX,[BP+RSF_SID]
		MOV	[BP+RSFS_SID],AX	;SEMAPHORE NUMBER.
		MOV	DI,RSF_STS
		CALL	MTOS			;TYPE 1 SVC.
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,RSFS_SIZE		;DESTROY THE SDB.
		RET				;DONE.
RSF_		ENDP
;
TSF_		PROC	FAR
		SUB	SP,RSFS_SIZE
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],017H	;TSF CODE.
		JMP	RSF_IN
TSF_		ENDP
PAGE
;
;	SYMBOLS FOR SEFET:
;
SEFETS_GRP	EQU	1	;SDB:	EVENT FLAG GROUP ID,
SEFETS_FLAGS	EQU	2	;	FLAG BITS TO SET AT TIMER EXPIRATION,
SEFETS_TME	EQU	4	;	TIME-UNIT CODE,
SEFETS_LEN	EQU	5	;	NUMBER OF TIME-UNITS.
SEFETS_SIZE	EQU	6	;ALLOCATED FOR SDB.
SEFET_GRP	EQU	10	;C-86:	EVENT FLAG GROUP ID,
SEFET_FLAGS	EQU	12	;	FLAG BITS TO SET AT TIMER EXPIRATION,
SEFET_TME	EQU	14	;	TIME-UNIT CODE,
SEFET_LEN	EQU	16	;	NUMBER OF TIME-UNITS.
SEFET_STS	EQU	18	;	ADDRESS OF STATUS WORD.
SEFET_SIZE	EQU	SEFET_STS-SEFETS_SIZE	;USED BY C-86.
;
;
SEFET_		PROC	FAR
		SUB	SP,SEFETS_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],0BH	;SEFET CODE.
		MOV	AL,[BP+SEFET_LEN]
		MOV	[BP+SEFETS_LEN],AL	;NUMBER OF TIME UNITS.
		MOV	AL,[BP+SEFET_TME]
		SHL	AL,1
		SHL	AL,1
		MOV	[BP+SEFETS_TME],AL	;TIME UNIT CODE.
		MOV	AX,[BP+SEFET_FLAGS]
		MOV	[BP+SEFETS_FLAGS],AX	;FLAG PATTERN
		MOV	AL,[BP+SEFET_GRP]
		MOV	[BP+SEFETS_GRP],AL	;FLAG GROUP NUMBER.
		MOV	DI,SEFET_STS
		CALL	MTOS			;TYPE 1 SVC.
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,SEFETS_SIZE		;DESTROY SDB.
		RET				;DONE.
SEFET_		ENDP
PAGE
;
;	SYMBOLS FOR SEND:
;
SENDS_WPS	EQU	1	;SDB:	WAIT AND PRIORITY SELECTOR,
SENDS_PRIV	EQU	2	;	ABSOLUTE PRIORITY IF USED,
SENDS_NODE	EQU	3	;	NODE NUMBER (MACHINE ID),
SENDS_BOX	EQU	4	;	MAILBOX ID,
SENDS_BUFR	EQU	5	;	ADDRESS OF 6-BYTE MESSAGE.
SENDS_SIZE	EQU	10	;ALLOCATED FOR SDB.
SEND_WT		EQU	14	;C-86:	WAIT SELECTOR,
SEND_PRIS	EQU	16	;	PRIORITY SELECTOR,
SEND_PRIV	EQU	18	;	ABSOLUTE PRIORITY IF USED,
SEND_BOX	EQU	20	;	MAILBOX ID,
SEND_BUFR	EQU	22	;	ADDRESS OF 6-BYTE MESSAGE.
SEND_STS	EQU	26	;	ADDRESS OF STATUS WORD.
SEND_SIZE	EQU	SEND_STS-SENDS_SIZE	;USED BY C-86.
;
;
SEND_		PROC	FAR
		SUB	SP,SENDS_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],0DH	;SEND CODE.
		MOV	AL,[BP+SEND_WT]
		ROR	AL,1
		OR	AL,[BP+SEND_PRIS]
		MOV	[BP+SENDS_WPS],AL	;WAIT MODE AND PRIORITY SELECT.
		MOV	AL,[BP+SEND_PRIV]
		MOV	[BP+SENDS_PRIV],AL	;(MAYBE) PRIORITY VALUE.
		MOV	BYTE PTR [BP+SENDS_NODE],0	;ZAP NODE NUMBER.
		MOV	AL,[BP+SEND_BOX]
		MOV	[BP+SENDS_BOX],AL	;MAILBOX NUMBER.
		LDS	DI,DWORD PTR [BP+SEND_BUFR]
		MOV	[BP+SENDS_BUFR],DI	;BUFFER OFFSET
		MOV	[BP+SENDS_BUFR+2],DS	;       SEGMENT.
		MOV	DI,SEND_STS
		CALL	MTOS			;TYPE 1 SVC
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,SENDS_SIZE		;DESTROY THE SDB.
		RET 				;DONE.
SEND_		ENDP
PAGE
;
;	SYMBOLS FOR TIME, STIME:
;
TIMES_FMT	EQU	1	;SDB:	RESERVED "FORMAT" BYTE (0),
TIMES_BUFR	EQU	2	;	ADDRESS OF 17-BYTE BUFFER.
TIMES_SIZE	EQU	6	;ALLOCATED FOR SDB.
TIME_FMT	EQU	10	;C-86:	RESERVED "FORMAT" BYTE (0),
TIME_BUFR	EQU	12	;	ADDRESS OF 17-BYTE BUFFER.
TIME_STS	EQU	16	;	ADDRESS OF STATUS WORD.
TIME_SIZE	EQU	TIME_STS-TIMES_SIZE	;USED BY C-86.
;
;
STIME_		PROC	FAR
		SUB	SP,TIMES_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],013H	;STIME CODE.
		JMP	TIME_IN
STIME_		ENDP
;
TIME_		PROC	FAR
		SUB	SP,TIMES_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],012H	;TIME CODE.
TIME_IN		LABEL	NEAR
		MOV	BYTE PTR [BP+TIMES_FMT],0	;ZAP FORMAT CODE.
		LDS	DI,DWORD PTR [BP+TIME_BUFR]
		MOV	[BP+TIMES_BUFR],DI	;BUFFER OFFSET,
		MOV	[BP+TIMES_BUFR+2],DS	;       SEGMENT.
		MOV	DI,TIME_STS
		CALL	MTOS			;TYPE 1 SVC
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,TIMES_SIZE		;DESTROY THE SDB.
		RET				;DONE.
TIME_		ENDP
PAGE
;
;	SYMBOLS FOR TEF, WEF:
;
TEFS_TGRP	EQU	1	;SDB:	TYPE OF TEST/WAIT AND FLAG GROUP ID,
TEFS_FLAGS	EQU	2	;	FLAG BITS TO TEST OR AWAIT,
TEFS_SIZE	EQU	4	;ALLOCATED FOR SDB.
TEF_TYP		EQU	8	;C-86:	TYPE OF TEST/WAIT,
TEF_GRP		EQU	10	;	FLAG GROUP ID,
TEF_FLAGS	EQU	12	;	FLAG BITS TO TEST OR AWAIT.
TEF_STS		EQU	14	;	ADDRESS OF STATUS WORD.
TEF_SIZE	EQU	TEF_STS-TEFS_SIZE	;USED BY C-86.
;
;
TEF_		PROC	FAR
		SUB	SP,TEFS_SIZE
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],0AH	;TEF CODE.
TEF_IN		LABEL	NEAR
		MOV	AL,[BP+TEF_TYP]
		ROR	AL,1
		OR	AL,[BP+TEF_GRP]
		MOV	[BP+TEFS_TGRP],AL	;TEST TYPE AND GROUP NUMBER
		MOV	AX,[BP+TEF_FLAGS]
		MOV	[BP+TEFS_FLAGS],AX	;FLAG PATTERN.
		MOV	DI,TEF_STS
		CALL	MTOS			;TYPE 1 SVC
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,TEFS_SIZE		;DESTROY THE SDB.
		RET				;DONE.
TEF_		ENDP
;
WEF_		PROC	FAR
		SUB	SP,TEFS_SIZE
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],0CH	;WEF CODE.
		JMP	TEF_IN
WEF_		ENDP
PAGE
;
;	SYMBOLS FOR QCOPY:
;
QCOPYS_BUFR	EQU	1	;SDB:	ADDRESS OF 32-BYTE BUFFER.
QCOPYS_SIZE	EQU	6	;ALLOCATED FOR SDB
QCOPY_BUFR	EQU	10	;C-86:	ADDRESS OF 32-BYTE BUFFER.
QCOPY_STS	EQU	14	;	ADDRESS OF STATUS WORD.
QCOPY_SIZE	EQU	QCOPY_STS-QCOPYS_SIZE	;USED BY C-86.
;
;
QCOPY_		PROC	FAR
		SUB	SP,QCOPYS_SIZE
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],018H	;QCOPY CODE.
		LDS	DI,DWORD PTR [BP+QCOPY_BUFR]
		MOV	[BP+QCOPYS_BUFR],DI	;BUFFER OFFSET,
		MOV	[BP+QCOPYS_BUFR+2],DS	;       SEGMENT.
		MOV	DI,QCOPY_STS
		CALL	MTOS
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,QCOPYS_SIZE		;DESTROY THE SDB.
		RET				;DONE.
QCOPY_		ENDP
PAGE
;
;	SYMBOLS FOR CREATE:
;
CREATES_BUFR	EQU	1	;SDB:	ADDRESS OF BUFFER FOR ASSIGNED TASK ID
CREATES_TCD	EQU	5	;	ADDRESS OF TCD IMAGE.
CREATES_SIZE	EQU	10	;ALLOCATED FOR SDB.
CREATE_BUFR	EQU	14	;C-86:	ADDRESS OF BUFFER FOR ASSIGNED TASK ID,
CREATE_TCD	EQU	18	;	ADDRESS OF TCD IMAGE,
CREATE_STS	EQU	22	;	ADDRESS OF STATUS WORD.
CREATE_SIZE	EQU	CREATE_STS-CREATES_SIZE	;USED BY C-86.
CREATE_		PROC	FAR
		SUB	SP,CREATES_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],1AH	;CREATE CODE.
		LDS	DI,DWORD PTR [BP+CREATE_TCD]	;ADDRESS OF TCD IMAGE
		MOV	[BP+CREATES_TCD],DI
		MOV	[BP+CREATES_TCD+2],DS	;...TO SDB.
		LDS	DI,DWORD PTR [BP+CREATE_BUFR]	;TASK ID BUFFER ADDR.
		MOV	[BP+CREATES_BUFR],DI
		MOV	[BP+CREATES_BUFR+2],DS		;... TO SDB.
		MOV	DI,CREATE_STS
		CALL	MTOS			;TYPE 1 SVC.
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,CREATES_SIZE		;DESTROY SDB.
		RET				;DONE.
CREATE_		ENDP
PAGE
;
;	DELETE SERVICE NEEDS NO SYMBOLS, SINCE THE SDB IS ONLY A FUNCTION CODE
;
DELETE_		PROC	FAR
		INT	LOW X_SVC_VECTOR+1	;TYPE 2 SVC
		DD	DELETE_SDB
DELETE_		ENDP
DELETE_SDB	DB	01BH			;DELETE CODE.
;
;	SYMBOLS FOR DELTSK:
;
DELTSKS_ID	EQU	1	;SDB:	TASK ID TO BE DELETED,
DELTSKS_SIZE	EQU	4	;ALLOCATED FOR SDB.
DELTSK_ID	EQU	8	;C-86:	TASK ID TO BE DELETED,
DELTSK_STS	EQU	10	;	ADDRESS OF STATUS WORD.
DELTSK_SIZE	EQU	DELTSK_STS-DELTSKS_SIZE	;USED BY C-86.
;
;
DELTSK_		PROC	FAR
		SUB	SP,DELTSKS_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],01CH	;DELTSK CODE.
		MOV	AX,[BP+DELTSK_ID]	;TASK ID...
		MOV	[BP+DELTSKS_ID],AX	;...TO SDB.
		MOV	DI,DELTSK_STS
		CALL	MTOS			;TYPE 1 SVC
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,DELTSKS_SIZE		;DESTROY SDB.
		RET				;DONE.
DELTSK_		ENDP
PAGE
;
;	SYMBOLS FOR LOADED:
;
LOADEDS_STATUS	EQU	1	;SDB:	LOAD COMPLETION STATUS,
LOADEDS_ORD	EQU	2	;	OVERLAY REGION DESCRIPTOR ADDRESS,
LOADEDS_SIZE	EQU	6	;ALLOCATED FOR THE SDB.
LOADED_STATUS	EQU	10	;C-86:	LOAD COMPLETION STATUS,
LOADED_ORD	EQU	12	;	OVERLAY REGION DESCRIPTOR ADDRESS,
LOADED_STS	EQU	16	;	ADDRESS OF STATUS WORD.
LOADED_SIZE	EQU	LOADED_STS-LOADEDS_SIZE	;USED BY C-86
;
;
LOADED_		PROC	FAR
		SUB	SP,LOADEDS_SIZE		;ALLOW FOR THE SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],01DH	;LOADED SERVICE CODE.
		MOV	AL,[BP+LOADED_STATUS]	;GET LOAD COMPLETION STATUS
		MOV	[BP+LOADEDS_STATUS],AL	;TO SDB.
		LDS	DI,DWORD PTR [BP+LOADED_ORD]	;ADDR OF ORD
		MOV	[BP+LOADEDS_ORD],DI	;TO THE SDB.
		MOV	[BP+LOADEDS_ORD+2],DS	;(BOTH PARTS.)
		MOV	DI,LOADED_STS		;ADDRESS THE STATUS VARIABLE.
		CALL	MTOS			;TYPE 1 SVC.
		POP	DS
		POP	DI
		POP	BP
		POP	AX			;RESTORE REGISTERS,
		ADD	SP,LOADEDS_SIZE		;DESTROY STACKED SDB,
		RET				;THAT'S IT.
LOADED_		ENDP
;
;	SYMBOLS FOR HSFS SERVICE:
;
HSFSS_OPCODE	EQU	1	;SDB:	HSFS OPCODE.
HSFSS_FMASK	EQU	2	;	EVENT FLAG MASK WORD.
HSFSS_STS	EQU	4	;	ADDRESS OF STATUS WORD.
HSFSS_RPB	EQU	8	;	ADDRESS OF REQUEST RPB.
HSFSS_SIZE	EQU	12	;BYTES NEEDED TO CONSTRUCT SDB.
HSFS_OPCODE	EQU	16	;C:	HSFS OPCODE.
HSFS_FMASK	EQU	18	;	EVENT FLAG MASK WORD.
HSFS_STS	EQU	20	;	REQUEST STATUS WORD ADDRESS.
HSFS_RPB	EQU	24	;	ADDRESS OF RPB.
HSFS_SIZE	EQU	HSFS_RPB-HSFSS_SIZE	;BYTES USED BY C-86 TO PASS PARAMETERS.
;
;	THE INTERFACE PROCEDURE.
;
HSFS_		PROC	FAR
		SUB	SP,HSFSS_SIZE		;MAKE ROOM FOR GENERATED SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP			;MARK THE SDB FRAME,
		ADD	BP,8			;BELOW THE SAVE AREA.
		MOV	BYTE PTR [BP],01EH	;SDB SERVICE CODE.
		MOV	AX,[BP+HSFS_OPCODE]	;HSFS OPCODE.
		MOV	[BP+HSFSS_OPCODE],AL	;(BECOMES A BYTE.)
		MOV	AX,[BP+HSFS_FMASK]	;EVENT FLAG MASK WORD
		MOV	[BP+HSFSS_FMASK],AX
		LDS	DI,DWORD PTR [BP+HSFS_STS]	;ADDRESS OF STATUS
		MOV	[BP+HSFSS_STS],DI	;WORD BUFFER.
		MOV	[BP+HSFSS_STS+2],DS
		LDS	DI,DWORD PTR [BP+HSFS_RPB]	;ADDRESS OF REQUEST'S
		MOV	[BP+HSFSS_RPB],DI	;RPB.
		MOV	[BP+HSFSS_RPB+2],DS
		PUSH	SS			;ADDRESS OF THE SDB ONTO
		PUSH	BP			;THE TASK'S STACK.
		INT	LOW X_SVC_VECTOR	;TYPE 1 SERVICE CALL.
		ADD	SP,4			;DESTROY SDB ADDRESS.
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,HSFSS_SIZE		;DESTROY SDB CONSTRUCTION.
		RET				;RETURN TO CALLER.
HSFS_		ENDP
PAGE
;
;	UTILITY PROCEDURES SVCOK AND GETPARM:
;
SVCOK_		PROC	FAR
		PUSH	BP
		MOV	BP,SP
		XOR	AL,AL		;ASSUME FALSITY.
		TEST	WORD PTR [BP+6],0040H	;SVC FAILED?
		JNE	@1		;YES.
		NOT	AL		;NO.
@1:		POP	BP
		RET	
SVCOK_		ENDP
;
;
GETPARM_	PROC	FAR
		MOV	DX,BX		;BX PARAMETER BECOMES POINTER SEGMENT
		MOV	AX,AX		;AX PARAMETER BECOMES POINTER OFFSET.
		RET
GETPARM_	ENDP
;
C_CS		ENDS
		END
