PAGE	60,132
INCLUDE B:BIG.INC
;
;		** MTOS-86 CONSOLE DRIVER:     9/3/82 **
;		** TYPE 2 DRIVER FOR LEVEL-II NUCLEUS **
;		F. W. PORRETTO, FOR INDUSTRIAL PROGRAMMING, INC.
;
;
;		THIS COLLECTION OF ROUTINES AND INTERRUPT HANDLERS CONSTITUTES
;	THE MTOS-86 CONSOLE DRIVER. IN THE DELIVERED CONFIGURATION, IT HANDLES
;	A TERMINAL INTERFACED TO AN iSBC 86/12A BOARD THROUGH THE ONBOARD 8251A
;	USART. THROUGH THE CODE OF THIS DRIVER, THE READER WILL FIND NOTATIONS
;	CONCERNING THE MODIFICATION OF THE DRIVER FOR THE SUPPORT OF MULTIPLE
;	TERMINALS, SHOULD YOUR PRODUCT REQUIRE IT. THE DRIVER WOULD REQUIRE EX-
;	TENSIVE MODIFICATION TO SUPPORT ANY OTHER KIND OF INTERFACING DEVICE.
;
		NAME	TDRIVER
;
;	LINKAGE:
;		1. ENTRY POINTS TO THESE ROUTINES.
;
		PUBLIC	X_INIT_CON		;INITIALIZE USARTS, UCBS.
		PUBLIC	X_CON_BR		;BRANCH TABLE ADDRESS.
;
;		2. REFERENCES TO MTOS-86 CONVENIENCE ROUTINES.
;
		EXTRN	X_DEVDATA:FAR
		EXTRN	XOVER:FAR
		EXTRN	XDELTMB:FAR
		EXTRN	XTIMEOUT:FAR
		EXTRN	XSETUNS:FAR
		EXTRN	X_ELIPSE:FAR
		EXTRN	X_XEL:FAR
		EXTRN	X_SYSTATE:FAR
PAGE
;
;	CONVENIENCE DEFINITIONS:
;		1. THE FOLLOWING DEFINITIONS WITH NAMES BEGINNING 'TD' ARE
;		   USED TO REFER TO THE APPLICABLE UCD WHEN ES IS SET TO
;		   THE MTOS_GROM AND DI CONTAINS THE OFFSET OF THE UCD:
;
TD_INTNO	EQU	BYTE PTR ES:[DI+UCD_INTNO]
TD_PITCMP	EQU	BYTE PTR ES:[DI+UCD_PORT]
TD_PITCMD	EQU	BYTE PTR ES:[DI+UCD_PORT+1]
TD_PITCTP	EQU	BYTE PTR ES:[DI+UCD_PORT+2]
TD_PITCT	EQU	BYTE PTR ES:[DI+UCD_PORT+3]
TD_UCMDP	EQU	BYTE PTR ES:[DI+UCD_PORT+4]
TD_UMOD		EQU	BYTE PTR ES:[DI+UCD_PORT+5]
TD_UCMD		EQU	BYTE PTR ES:[DI+UCD_PORT+6]
TD_UDATP	EQU	BYTE PTR ES:[DI+UCD_PORT+7]
TD_TIME		EQU	WORD PTR ES:[DI+UCD_PORT+8]
;
;		NOTE THE USAGE OF THE GENERAL PURPOSE CELLS IN THE UCD
;		TO HOLD THE FOLLOWING:
;
;	0:	TD_PITCMP:	PROGRAMMABLE INTERVAL TIMER COMMAND PORT
;	1:	TD_PITCMD:	PIT COMMAND TO PRODUCE BAUD RATE GENERATOR
;	2:	TD_PITCTP:	PIT COUNTER PORT FOR BAUD RATE
;	3:	TD_PITCT:	COUNT BYTE TO PRODUCE DESIRED BAUD RATE.
;	4:	TD_UCMDP:	USART COMMAND/STATUS PORT.
;	5:	TD_UMOD:	OPERATING MODE (FRAME CHARACTERISTICS) OF
;				USART.
;	6:	TD_UCMD:	COMMAND STATE (SIGNAL CHARACTERISTICS) OF
;				USART.
;	7:	TD_UDATP:	USART TRANSMIT/RECEIVE DATA PORT.
;	8,9:	TD_TIME:	TIMEOUT PARAMETERS FOR ALL CONSOLE REQUESTS.
;	10-15:			UNUSED.
;
;		2. THE FOLLOWING DEFINITIONS WITH NAMES BEGINNING 'TU' ARE
;		   USED TO REFER TO THE APPLICABLE UCB WHEN BX CONTAINS THE
;		   OFFSET OF THE UCB:
;
TU_UNIT		EQU	BYTE PTR [BX+UCB_UNIT]
TU_STS		EQU	BYTE PTR [BX+UCB_STS]
TU_QIRB		EQU	WORD PTR [BX+UCB_QIRB]
TU_CIRB		EQU	WORD PTR [BX+UCB_CIRB]
TU_DCB		EQU	WORD PTR [BX+UCB_DCB]
TU_UCD		EQU	WORD PTR [BX+UCB_UCD]
TU_TCB		EQU	WORD PTR [BX+UCB_TCB]
TU_TIME		EQU	WORD PTR [BX+UCB_GPR]
TU_UCMD		EQU	BYTE PTR [BX+UCB_GPR+2]
TU_XBUF		EQU	DWORD PTR [BX+UCB_GPR+4]
TU_XBF0		EQU	WORD PTR [BX+UCB_GPR+4]
TU_XBF1		EQU	WORD PTR [BX+UCB_GPR+6]
TU_XCNT		EQU	WORD PTR [BX+UCB_GPR+8]
TU_RBUF		EQU	DWORD PTR [BX+UCB_GPR+10]
TU_RBF0		EQU	WORD PTR [BX+UCB_GPR+10]
TU_RBF1		EQU	WORD PTR [BX+UCB_GPR+12]
TU_RCNT		EQU	WORD PTR [BX+UCB_GPR+14]
TU_RMAX		EQU	WORD PTR [BX+UCB_GPR+16]
TU_DELC		EQU	WORD PTR [BX+UCB_GPR+18]
TU_BYTE		EQU	BYTE PTR [BX+UCB_GPR+20]
TU_BYTE2	EQU	BYTE PTR [BX+UCB_GPR+21]
TU_CMDP		EQU	BYTE PTR [BX+UCB_GPR+22]
TU_DATP		EQU	BYTE PTR [BX+UCB_GPR+23]
;
;		NOTE THAT THE 32 BYTES OF GENERAL PURPOSE VARIABLE SPACE IN
;		THE UCB HAS BEEN ALLOCATED AS FOLLOWS:
;
;	0-1:	TU_TIME:	TIMEOUT SPECIFIER FOR CONSOLE REQUESTS.
;	2:	TU_UCMD:	COMMAND BYTE FOR UNIT OPERATION, FROM TD_UCMD.
;	3:			RESERVED, MUST BE 0.
;	4-7:	TU_XBUF:	ADDRESS OF CURRENT TRANSMIT BUFFER
;	8-9:	TU_XCNT:	LENGTH OF CURRENT TRANSMIT BUFFER
;	10-13:	TU_RBUF:	ADDRESS OF NEXT FREE CELL IN RECEIVE BUFFER
;	14-15:	TU_RCNT:	CURRENT COUNT OF CHARACTERS IN RECEIVE BUFFER
;	16-17:	TU_RMAX:	MAXIMUM COUNT OF CHARACTERS IN RECEIVE BUFFER
;	18-19:	TU_DELC:	COUNT OF BACKSPACE OPERATIONS LEFT TO DO.
;	20:	TU_BYTE:	HOLDS CARET FOR CONTROL CODE ECHOES.
;	21:	TU_BYTE2:	BUFFER FOR CHARACTER ECHOES.
;	22:	TU_CMDP:	PORT ADDRESS OF USART COMMAND/STATUS PORT.
;	23:	TU_DATP:	PORT ADDRESS OF USART DATA IN/OUT PORT.
;	24-31:			UNUSED.
;
;		3. THE FOLLOWING DEFINITIONS WHOSE NAMES BEGIN WITH 'TR'
;		   ARE FOR USE IN REFERRING TO ANY IRB WHOSE OFFSET IS IN
;		   SI:
;
TR_LINK		EQU	WORD PTR [SI+IRB_LINK]
TR_PRI		EQU	BYTE PTR [SI+IRB_PRI]
TR_TCB		EQU	WORD PTR [SI+IRB_TCB]
TR_UCB		EQU	WORD PTR [SI+IRB_UCB]
TR_TMB		EQU	WORD PTR [SI+IRB_TMB]
TR_FUNCT	EQU	BYTE PTR [SI+IRB_FUNCT]
TR_DEVICE	EQU	BYTE PTR [SI+IRB_DEVICE]
TR_UNIT		EQU	BYTE PTR [SI+IRB_UNIT]
TR_GRP		EQU	BYTE PTR [SI+IRB_GRP]
TR_FLAGS	EQU	WORD PTR [SI+IRB_FLAGS]
TR_STATUS	EQU	DWORD PTR [SI+IRB_STATUS]
TR_BUFR		EQU	DWORD PTR [SI+IRB_BUFR]
TR_BUF0		EQU	WORD PTR [SI+IRB_BUFR]
TR_BUF1		EQU	WORD PTR [SI+IRB_BUFR+2]
;
;		4. A FEW MORE SYMBOLS FOR REFERRING TO READ AND
;		   WRITE REQUEST PARAMETER BLOCKS:
;
RD_BUFR		EQU	DWORD PTR ES:[DI]	;ADDRESS OF READ BUFFER
RD_MAX		EQU	WORD PTR ES:[DI+4]	;MAX NUMBER OF CHARS TO READ
;
WT_BUFR		EQU	DWORD PTR ES:[DI]	;ADDRESS OF WRITE BUFFER
WT_CNT		EQU	WORD PTR ES:[DI+4]	;MAX NUMBER OF CHARS TO WRITE
;
WTRD_WTBUF	EQU	DWORD PTR ES:[DI]	;WRITE BUFFER FOR WRITE/READ
WTRD_WTCNT	EQU	WORD PTR ES:[DI+4]	;WRITE COUNT FOR WRITE/READ
WTRD_RDBUF	EQU	DWORD PTR ES:[DI+6]	;READ BUFFER FOR WRITE/READ
WTRD_RDCNT	EQU	WORD PTR ES:[DI+10]	;READ MAX COUNT FOR WRITE/READ
;
;		5. SOME FINAL USEFUL SYMBOLS:
;
;		-- HOW TO TURN THE TRANSMITTER ON --
;
XMITTER_ON	EQU	00000001B
;
;			-- FUNCTION CODES --
;
RSRV_UNIT	EQU	0	;RESERVE UNRESERVED UNIT
RLSE_UNIT	EQU	1	;RELEASE RESERVED UNIT
QUEUED_READ	EQU	2	;NORMAL READ, INCLUDING PROMPT STRING
QUEUED_WRITE	EQU	3	;NORMAL WRITE
IMMED_WRITE	EQU	4	;VESTIGIAL IMMEDIATE ("EMERGENCY") WRITE
ONECH_READ	EQU	5	;ONE-BYTE TRANSPARENT READ
WRITE_XRD	EQU	6	;WRITE-THEN-TRANSPARENT READ
WRITE_READ	EQU	7	;WRITE-THEN-NORMAL READ.
;
;			-- UNIT STATUSES --
;
UNIT_BUSY	EQU	00000001B	;UNIT BUSY-BIT.
READ_BIT	EQU	00000010B	;READ REQUEST BIT.
WRITE_BIT	EQU	00000100B	;WRITE REQUEST BIT.
UNIT_STOP	EQU	00001000B	;WRITE SUSPENDED BY CONTROL S
READ_CTLX	EQU	00010000B	;CONTROL X DIRECTS READ RESET
XPRN_BIT	EQU	00100000B	;TRANSPARENT READ BIT
RD1_BIT		EQU	01000000B	;ONE-CHARACTER-READ BIT.
;
;			-- XSTATUS REDEFINITIONS --
;
XLASTCHR	EQU	BYTE PTR XSTATUS+1
XPRIMARY	EQU	BYTE PTR XSTATUS
XCOUNT		EQU	WORD PTR XSTATUS+2
;
;			-- RETURNED STATUSES --
;
BAD_PARAMS	EQU	-1
SUCCESS		EQU	2
TIMEOUT_STAT	EQU	0FFH + 100H * 0FCH	;WORD REFERENCE
;
;			-- SOME IMPORTANT ASCII CODES --
;
ASCII_BELL	EQU	07H
ASCII_CR	EQU	0DH
ASCII_LF	EQU	0AH
ASCII_ESC	EQU	1BH
ASCII_CTLQ	EQU	11H
ASCII_CTLS	EQU	13H
ASCII_CTLX	EQU	18H
ASCII_CTLZ	EQU	1AH
ASCII_SPACE	EQU	20H
ASCII_CARET	EQU	5EH
ASCII_RUB	EQU	7FH
PAGE
;
;		OPEN THE CODE SEGMENT:
;		IN THE INTERESTS OF THE SIMPLEST POSSIBLE HANDLING OF THE
;		ADDRESSING PROBLEM, SEVERAL INVARIANT TABLES HAVE BEEN 
;		INCLUDED IN THE CODE SEGMENT OF THIS DRIVER.
;
TDRIVER		SEGMENT	PARA
		ASSUME	CS:TDRIVER,DS:MTOS_DS,SS:MTOS_DS
;
;			-- BRANCH TABLE USED BY X_RIO --
;
;		ENTRY THROUGH THE ADDRESSES IN THIS TABLE PROVIDES FOR THE
;	ACTIVATION OF A NEW IRB BY THE DRIVER.
;
X_CON_BR	EQU	THIS DWORD
		DD	X_READ		;FUNC #2: NORMAL READ.
		DD	X_WRITE		;FUNC #3: NORMAL WRITE.
		DD	X_WRITE		;FUNC #4: VESTIGIAL IMMEDIATE WRITE.
		DD	X_READ_ONE	;FUNC #5: ONE CHARACTER XPARENT READ.
		DD	X_WRT_XRD	;FUNC #6: WRITE/XPARENT READ.
		DD	X_WRT_RD	;FUNC #7: WRITE/READ.
;
;	TABLE OF INTERRUPT HANDLER OFFSETS USED BY X_INIT_CON TO SET UP
;	INTERRUPT VECTORS FOR ALL UNITS.
;
;	** NOTE!! **
;		THIS IS ONE OF THE PLACES IN THE CONSOLE DRIVER WHERE
;	MODIFICATIONS ARE REQUIRED FOR A MULTI-TERMINAL INSTALLATION.
;
I_HANDLERS	DW	RCVR_0,XMTR_0		;HANDLERS FOR UNIT #0
		DW	RCVR_1,XMTR_1		;HANDLERS FOR UNIT #1
;
;			-- BACK SPACE STRING (DESTRUCTIVE) --
;
BACK_SPACER	DB	08,20H,08,08,20H,08
PAGE
;
;		** EXECUTABLE CODE STARTS HERE **
;
;
;	SECTION 1:		INITIALIZATION ROUTINE.
;
;			THIS ROUTINE, CALLED AUTOMATICALLY BY MTOS-86 AT ANY
;			SYSTEM RESET ACTION, HAS THE RESPONSIBILITY FOR PER-
;			FORMING THE COMPLETE INITIALIZATION OF THE USART, 
;			THE BAUD RATE GENERATOR, AND ANY "UNIT PERSONALITY"
;			DATA STRUCTURES (HERE THAT MEANS SETTING UP THE 
;			DEFAULT PARAMETERS). IT IS ENTERED BY A FAR CALL 
;			WITH INTERRUPTS DISABLED.
;
X_INIT_CON	PROC	FAR
;
		MOV	BX,XUCB		;GET UCB ADDRESS.
		MOV	BP,OFFSET I_HANDLERS
		MOV	DI,TU_UCD	;GET UCD ADDRESS
		MOV	AL,TD_UCMDP	;GET COMMAND PORT ADDRESS FROM UCD,
		MOV	AH,TD_UDATP	;ALSO DATA IN/OUT PORT ADDRESS,
		MOV	TU_CMDP,AL	;AND MOVE THOSE TWO PORT ADDRESSES TO
		MOV	TU_DATP,AH	;THE MORE CONVENIENTLY ADDRESSED UCB.
		MOV	AL,TD_UCMD	;ALSO GET THE UNIT COMMAND BYTE TO THE
		MOV	TU_UCMD,AL	;UCB, AND NOT LEAST OF ALL,
		MOV	AX,TD_TIME	;THE TIME SPECIFIER FOR ALL INTERNALLY
		MOV	TU_TIME,AX	;TIMED REQUESTS.
		MOV	TU_BYTE,ASCII_CARET	;READY FOR CONTROL CODE ECHOES
		MOV	AL,TD_INTNO	;GET INTERRUPT VECTOR NUMBER
		XOR	AH,AH		;AND CONSTRUCT FROM IT THE ADDRESS
		ADD	AX,AX		;OF THE RECEIVE INTERRUPT VECTOR
		ADD	AX,AX		;ENTRY, PRESUMED TO BE JUST BEFORE THE
		MOV	SI,AX		;TRANSMIT VECTOR ENTRY IN LOW MEMORY.
		MOV	AL,TU_UNIT	;GET UNIT NUMBER OF UNIT,
		XOR	AH,AH		;MAKE IT 16 BITS WIDE,
		SHL	AX,1		;AND MULTIPLY IT
		SHL	AX,1		;BY FOUR TO REACH THE INTERRUPT ENTRIES
		ADD	BP,AX		;FOR THIS UNIT.
		PUSH	DS		;SAVE MTOS_DS WHILE WE USE DS TO POINT
		XOR	AX,AX		;INTO THE VECTOR PAGE.
		MOV	DS,AX		;SOMEBODY'S GOTTA DO IT.
		MOV	AX,CS:[BP]	;GET THE OFFSET OF RECEIVER HDLR
		MOV	[SI],AX		;PUT IT INTO VECTOR
		MOV	[SI+2],CS	;PUT CS INTO SEGMENT FIELDS
		MOV	[SI+6],CS	;OF BOTH RECEIVE AND TRANSMIT VECTORS
		MOV	AX,CS:[BP+2]	;FINALLY, GET OFFSET OF XMIT HANDLER
		MOV	[SI+4],AX	;AND PUT IT IN XMIT VECTOR.
		POP	DS		;RESTORE DS TO MTOS_DS.
		XOR	DH,DH		;NOW DO THE PROGRAMMING
		MOV	DL,TD_PITCMP	;OF THE PIT-BAUD RATE GENERATOR
		MOV	AL,TD_PITCMD	;FIRST SET UP
		OUT	DX,AL		;THE PULSE PATTERN IN THE PIT.
		MOV	DL,TD_PITCTP	;THEN PUT OUT
		MOV	AL,TD_PITCT	;THE COUNT BYTE USED TO DIVIDE THE
		OUT	DX,AL		;SYSTEM CLOCK INTO THE BAUD RATE.
		XOR	AL,AL		;NOW PUT THREE NULL COMMANDS
		MOV	DL,TD_UCMDP	;TO THE USART COMMAND PORT
		OUT	DX,AL		;TO INSURE THAT, WHEN WE
		CALL	DELAY		;TELL THE USART TO RESET
		OUT	DX,AL		;ITSELF AND ACCEPT ITS MODE BYTE,
		CALL	DELAY		;IT WILL DO SO.
		OUT	DX,AL		;THE DELAYS ARE SO THAT THE PROCES-
		CALL	DELAY 		;SOR DOESN'T OVERRUN THE USART.
		MOV	AL,40H		;NOW PUT OUT THE
		OUT	DX,AL		;RESET COMMAND
		CALL	DELAY
		MOV	AL,TD_UMOD	;AND THEN THE
		OUT	DX,AL		;OPERATING CHARACTERISTICS
		CALL	DELAY		;(FRAME LENGTH, PARITY, ETC.)
		MOV	AL,TD_UCMD	;THEN, FINALLY,
		AND	AL,NOT XMITTER_ON	;SET UP COMMAND FOR IDLE STATE,
		OUT	DX,AL		;AND SET UP STANDARD SIGNALS/CONTROLS.
		RET
X_INIT_CON	ENDP
;
;	DELAY ROUTINE FOR PROCESSOR-USART COORDINATION.
;
;	THE INTEL 8251A USART HAS A WORST-CASE INTERNAL CYCLE TIME OF
;	4 MICROSECONDS. THEREFORE, COMMANDS FED TO THE USART SHOULD BE
;	SEPARATED BY AT LEAST THAT MUCH TIME. A CALL, TWO PUSHES, TWO
;	POPS AND A RETURN SHOULD DO IT FOR PROCESSORS AT 8 MHZ AND BELOW.
;
DELAY:
		PUSH	AX
		POP	AX
		PUSH	AX
		POP	AX
		RET
;
PAGE
;
;	SECTION 2:		SERVICE REQUEST PROCESSORS.
;
;		THESE ROUTINES ARE EXECUTED AT THE S STATE, AND HAVE AS THEIR
;		RESPONSIBILITY THE ACTIVATION OF I/O FROM SUBMITTED IRBS.
;
;	SECTION 2.1:		NORMAL READ REQUEST PROCESSOR.
;
X_READ		PROC	FAR
		MOV	BX,XUCB
		MOV	SI,XCIRB		;GOT UCB AND IRB ADDRESSES.
		OR	TU_STS,READ_BIT		;SET THE STATUS TO READ
		LES	DI,TR_BUFR		;GET READ BUFFER ADDRESS
		MOV	AX,RD_MAX		;GET MAX COUNT FROM IT
		TEST	AX,AX			;NULL REQUEST?
		JNZ	X_READ_1		;NO, O.K.
		INC	AX			;GOT TO READ AT LEAST 1 CHAR
X_READ_1:
		MOV	TU_RMAX,AX		;SET MAX READ LENGTH
		LES	DI,RD_BUFR		;GET BUFFER ADDRESS.
		MOV	TU_RBF0,DI		;PUT ADDRESS INTO
		MOV	TU_RBF1,ES		;READ CONTROL VARIABLES.
		MOV	TU_RCNT,0		;NO CHARS READ YET!
		MOV	TU_DELC,0		;AS ABOVE.
		CALL	MAKE_TMB		;HANDLE TMB CREATION/DELETION
		RET
X_READ		ENDP
;
;	SECTION 2.2:		NORMAL WRITE REQUEST PROCESSOR.
;
X_WRITE		PROC	FAR
		MOV	BX,XUCB
		MOV	SI,XCIRB	;GOT UCB AND IRB ADDRESSES
		OR	TU_STS,WRITE_BIT	;SET CORRECT STATUS
		CALL	MAKE_TMB	;HANDLE TMB CREATION/DELETION.
		LES	DI,TR_BUFR	;GET PARAMETER BUFFER ADDRESS
		MOV	CX,WT_CNT	;GET LENGTH OF WRITE BUFFER
		LES	DI,WT_BUFR	;GET BUFFER ADDRESS
		CALL	START_WRITE	;INITIATE I/O
		RET
X_WRITE		ENDP
;
;	SECTION 2.3:		ONE-BYTE READ REQUEST PROCESSOR.
;
X_READ_ONE	PROC	FAR
		MOV	BX,XUCB		;GET UCB ADDRESS
		MOV	SI,XCIRB	;GOT UCB AND IRB ADDRESSES.
		OR	TU_STS,(RD1_BIT+XPRN_BIT+READ_BIT)	;SET STATUS
		CALL	MAKE_TMB	;HANDLE TMB CREATION/DELETION.
		RET
X_READ_ONE	ENDP
;
;	SECTION 2.4:		WRITE/XPARENT READ REQUEST PROCESSOR.
;
X_WRT_XRD	PROC	FAR
		MOV	BX,XUCB		;INITIALIZE UCB POINTER
		MOV	SI,XCIRB	;GOT UCB AND IRB ADDRESSES.
		OR	TU_STS,XPRN_BIT	;SET THE TRANSPARENCY BIT
		CALL	FAR PTR X_WRT_RD	;AND TREAT THE REQUEST 
		RET			;AS A WRITE/READ.
X_WRT_XRD	ENDP
;
;	SECTION 2.5:		WRITE/READ REQUEST PROCESSOR.
;
X_WRT_RD	PROC	FAR
		MOV	BX,XUCB		;POINT BX AT UCB
		MOV	SI,XCIRB	;GOT UCB AND IRB ADDRESSES
		OR	TU_STS,(WRITE_BIT+READ_BIT)	;SET UNIT STATUS
		CALL	MAKE_TMB	;CREATE OR DELETE TMB
		LES	DI,TR_BUFR	;ADDRESS THE REQUEST PARAMETERS:
		MOV	AX,WTRD_RDCNT	;GET THE READ COUNT MAXIMUM
		TEST	AX,AX		;NULL?
		JNZ	X_WTRD_1	;NO.
		INC	AX		;YES, MAKE IT 1.
X_WTRD_1:
		MOV	TU_RCNT,0	;CLEAR THE READ COUNT
		MOV	TU_DELC,0	;CLEAR THE POSITIONS COUNT.
		MOV	TU_RMAX,AX	;INSERT MAX READ LENGTH
		LES	DI,WTRD_RDBUF	;GET BUFFER ADDRESS
		MOV	TU_RBF0,DI	;AND INSERT IT
		MOV	TU_RBF1,ES	;INTO THE READ CONTROL VARIABLES.
		LES	DI,TR_BUFR	;READDRESS THE PARAMETER LIST
		MOV	CX,WTRD_WTCNT	;GET THE WRITE COUNT
		LES	DI,WTRD_WTBUF	;AND THE POINTER TO THE WRITE BUFFER
		CALL	START_WRITE	;INITIATE THE WRITE.
		RET
X_WRT_RD	ENDP
;
;	SECTION 2.6:		SUPPORT ROUTINES.
;
;		THESE ROUTINES ARE FOR GENERAL SUPPORT JOBS, EITHER BY CALL
;		FROM A REQUEST PROCESSOR, OR BY CALL FROM THE TRAILING EDGE
;		OF AN INTERRUPT HANDLER.
;
;	SECTION 2.6.1:		INITIATE A WRITE REQUEST.
;
;		HERE'S WHERE ALL THE HOUSEKEEPING AND THE I/O INITIATION
;		FOR A WRITE REQUEST ARE PERFORMED.
;
;	REQUIRED PARAMETERS:
;		ES:DI POINTS TO STRING,
;		CX CONTAINS LENGTH (EXCLUSIVE OF NULL TERMINATION)
;
START_WRITE	PROC	NEAR
		MOV	TU_XCNT,CX	;INSTALL COUNT IN UCB
		MOV	TU_XBF0,DI	;INSTALL BUFFER POINTER
		MOV	TU_XBF1,ES	;IN THE UCB
		CALL	START_XMITTER	;START TRANSMIT INTERRUPT TRAIN,
		RET			;LET THE INTERRUPTS HAVE IT.
START_WRITE	ENDP
;
;	SECTION 2.6.2:		CONSTRUCT AND ENQUEUE A WATCHDOG TIMER.
;
MAKE_TMB	PROC	NEAR
		MOV	AX,TU_TIME	;GET TIMEOUT VALUE TO AX.
		TEST	AH,AH		;TIMEOUT FACILITY IN USE?
		JNZ	MAKE_TMB_0	;YES, MAKE A TMB AND ENQUEUE IT.
		CALL	XDELTMB		;NO: FREE THE TMB.
		RET
MAKE_TMB_0:
		MOV	XTNUM,AH	;PUT NUMBER OF UNITS IN XTNUM,
		MOV	XTUNIT,AL	;AND TYPE OF UNITS IN XTUNIT.
		CALL	XTIMEOUT	;ACTIVATE THE TIMER.
		RET
MAKE_TMB	ENDP
;
;	SOME RANDOM UTILITY PROCEDURES FOR HANDLING VARIOUS TRANSMISSION-
;	RELATED NECESSITIES.
;
START_XMITTER	PROC	NEAR		;ACTIVATE UNIT TRANSMITTER:
		PUSH	DI
		PUSH	DX
		MOV	DL,TU_CMDP	;GET USART COMMAND PORT.
		XOR	DH,DH
		MOV	AL,TU_UCMD	;GET STANDARD COMMAND, AND MAKE SURE
		OR	AL,XMITTER_ON	;THE TRANSMIT-ENABLE BIT IS UP.
		OUT	DX,AL		;SEND TO THE USART.
		POP	DX
		POP	DI
		RET
START_XMITTER	ENDP
;
STOP_XMITTER	PROC	NEAR		;DEACTIVATE UNIT TRANSMITTER:
		PUSH	DI
		PUSH	DX
		MOV	DL,TU_CMDP	;GET USART COMMAND PORT,
		XOR	DH,DH
		MOV	AL,TU_UCMD	;GET STANDARD COMMAND AND MAKE SURE
		AND	AL,NOT XMITTER_ON	;TRANSMIT IS DISABLED.
		OUT	DX,AL		;DISABLE TRANSMITTER.
		POP	DX
		POP	DI
		RET
STOP_XMITTER	ENDP
;
ECHO_AL		PROC	NEAR		;INITIATE ECHO OF WHATEVER'S IN AL:
		MOV	TU_BYTE2,AL	;DROP CHARACTER INTO ECHO BUFFER,
		MOV	TU_XCNT,1	;INDICATE 1 CHARACTER WRITE,
		MOV	TU_XBF1,DS	;FROM THE ECHO BUFFER.
		LEA	AX,TU_BYTE2
		MOV	TU_XBF0,AX
		CALL	START_XMITTER	;START THE TRANSMITTER.
		RET
ECHO_AL		ENDP
PAGE
;
;	SECTION 3:		INTERRUPT HANDLERS.
;
;		THE 8251A USART GENERATES SEPARATE RECEIVED CHARACTER READY
;		AND TRANSMITTER BUFFER AVAILABLE INTERRUPTS, WHICH THIS DRI-
;		VER ASSUMES ARE CONFIGURED AS CONSECUTIVE INTERRUPT VECTORS.
;		SEE THE INITIALIZATION ROUTINE FOR DETAILS.
;
;
;	SECTION 3.1		RECEIVED CHARACTER READY HANDLERS.
;
;		THIS INTERRUPT WILL OCCUR WHENEVER A CHARACTER IS ASSEMBLED
;		BY THE USART'S INPUT SECTION. HERE THE DRIVER HANDLES:
;			-- RECEIVED CHARACTER BUFFERING,
;			-- RECEIVED CHARACTER ECHO,
;			-- BUFFER EDITING,
;			-- READ REQUEST COMPLETION.
;
RCVR_0		PROC	FAR			;RECEIVER INTERRUPT, UNIT 0.
		CALL	X_ELIPSE		;GO TO I STATE.
		MOV	AL,LOW X_SYS_DEV	;GET DEVICE NUMBER TO AL,
		MOV	DL,0			;UNIT NUMBER TO DL.
		JMP	FAR PTR RCVR_INTRP	;GO TO COMMON HANDLER.
RCVR_0		ENDP
;
RCVR_1		PROC	FAR			;HANDLER FOR UNIT #1
		CALL	X_ELIPSE		;GO TO I STATE,
		MOV	AL,LOW X_SYS_DEV	;DEVICE NUMBER TO AL,
		MOV	DL,1			;UNIT NUMBER TO DL,
		JMP	FAR PTR RCVR_INTRP	;GO TO COMMON HANDLER.
RCVR_1		ENDP
;
;		** COMMON RECEIVER INTERRUPT HANDLER **
;
;		THIS HANDLER IS ENTERED BY ALL CONTROL PATHS ORIGINATING AT
;	A CONSOLE DRIVER UNIT-SPECIFIC RECEIVE INTERRUPT HANDLER. AT THIS
;	POINT, THE SYSTEM IS IN I STATE, AND AL AND DL INDICATE THE INTER-
;	RUPTING DEVICE UNIT.
;
;		WHILE IN I STATE, THE HANDLER WILL MAINTAIN BX AS THE POINTER
;	TO THE UCB, AND DX WILL CONTAIN THE DATA PORT ADDRESS. AL IS USED TO
;	CARRY THE CHARACTER RECEIVED AROUND. CX, SI, AND DI ARE USED AS GEN-
;	ERAL PURPOSE REGISTERS.
;
RCVR_INTRP	PROC	FAR
		CALL	X_DEVDATA		;CONSTRUCT UCB/DCB ADDRESSES
		MOV	DL,TU_DATP		;GET DATA PORT ADDRESS FROM UCB
		XOR	DH,DH
		IN	AL,DX			;GET THE CHARACTER!
		TEST	TU_STS,WRITE_BIT	;WERE WE WRITING??
		JNZ	X_NOT_READING		;YES, DON'T READ YET.
		TEST	TU_STS,READ_BIT		;NOT WRITING: WERE WE READING?
		JZ	X_NOT_READING		;NO. UNEXPECTED INPUT.
		TEST	TU_STS,XPRN_BIT		;YES: IN NORMAL READ MODE?
		JNZ	X_XPRN_READ		;NO, TRANSPARENT MODE.
;
;	SINCE WE ARE NOT IN TRANSPARENT MODE, BEFORE INVOKING THE STANDARD
;	PROCESSING FOR THE RECEIVED CHARACTER WE MUST CHECK TO SEE WHETHER
;	IT IS ONE OF THE FOUR SPECIAL CHARACTERS WHICH CAUSE SPECIAL EVENTS:
;		-ASCII_CR:	ECHO AND TERMINATE READ REQUEST.
;		-ASCII_CTLZ:	TERMINATE READ REQUEST.
;		-ASCII_RUB:	DELETE LAST CHARACTER READ.
;		-ASCII_CTLX:	"RESTART" READ REQUEST FROM SCRATCH.
;
		CMP	AL,ASCII_CR		;CARRIAGE RETURN?
		JNE	R01			;NO.
		JMP	X_CR			;YES.
R01:
		CMP	AL,ASCII_CTLZ		;CONTROL Z?
		JNE	R02			;NO.
		JMP	X_TRM			;YES.
R02:
		CMP	AL,ASCII_RUB		;RUBOUT?
		JNE	R03			;NO.
		JMP	X_RUB			;YES.
R03:
		CMP	AL,ASCII_CTLX		;CONTROL X?
		JNE	R04			;NO.
		JMP	X_CTX			;YES.
R04:
		JMP	X_REG_CHAR		;REGULAR CHARACTER, IT SEEMS.
;
;		-- IN TRANSPARENT MODE, WE CHECK NEITHER FOR TERMINATOR
;		   CHARACTERS, NOR FOR SPECIAL EDITING CHARACTERS, NOR DO
;		   WE ECHO THE INPUT WE RECEIVE. --
;
X_XPRN_READ:
		TEST	TU_STS,RD1_BIT		;ONE CHARACTER XPARENT READ?
		JNZ	X_XPRN_CHR		;YES, THAT'S A SPECIAL CASE.
		LES	DI,TU_RBUF		;NO:
		MOV	ES:[DI],AL		;CHAR TO BUFFER WITHOUT ADO.
		INC	DI			;ADVANCE BUFFER POINTER.
		MOV	TU_RBF0,DI
		INC	TU_RCNT			;THAT'S ONE MORE.
		MOV	SI,TU_RCNT
		CMP	SI,TU_RMAX		;BUFFER FULL?
		JZ	X_XPRN_RDOVR		;YES (ONLY WAY TO TERMINATE
						;A TRANSPARENT READ).
		JMP	X_RINT_OVR		;NO: DISMISS INTERRUPT.
X_XPRN_RDOVR:
		CALL	X_SYSTATE		;GO TO S STATE.
		MOV	XLASTCHR,AL		;PUT LAST CHARACTER IN STATUS
		JMP	X_READ_OVER		;READ REQUEST TERMINATION IS
						;UNCHANGED BY TRANSPARENCY.
;
;		-- HANDLE RECEPTION OF ONE CHARACTER IN TRANSPARENT MODE --
;
;		SINCE THE FUNCTION IS LIMITED TO THE RECEPTION OF ONE CHAR-
;	ACTER, THE CHARACTER IS RETURNED IN THE TERMINATOR FIELD OF THE STATUS
;	DOUBLEWORD, AND REQUEST COMPLETION IS DECLARED IMMEDIATELY.
;
X_XPRN_CHR:
		MOV	TU_RCNT,1		;SET COUNT TO 1.
		CALL	X_SYSTATE		;GO TO S STATE
		MOV	XLASTCHR,AL		;PUT CHARACTER INTO STATUS
		JMP	X_READ_OVER		;ENTER COMPLETIOON OF REQUEST.
;
;		HERE WE HANDLE CHARACTERS RECEIVED WHILE NO READ REQUEST IS
;	BEING SERVICED. CONTROL-S (ASCII XOFF) STOPS OUTPUT ON THE UNIT,
;	CONTROL-Q (ASCII XON) RESTARTS THE STOPPED OUTPUT, CONTROL-X
;	(ASCII CAN) ABORTS THE CURRENT OUTPUT REQUEST, IF ANY, AND
;	ESCAPE INVOKES THE UNSOLICITED INPUT TASK.
;
X_NOT_READING:
		CMP	AL,ASCII_CTLQ		;CONTROL Q? (X-ON?)
		JZ	X_READ_CTLQ		;YES.
		CMP	AL,ASCII_CTLS		;NO: CONTROL S? (X-OFF?)
		JZ	X_READ_CTLS		;YES.
		CMP	AL,ASCII_CTLX		;NO: CONTROL X? (OUTPUT ABORT)
		JZ	X_READ_CTLX		;YES.
X_MAYBE_ESC:
		CMP	AL,ASCII_ESC		;FINALLY, AN ESCAPE CODE
		JNZ	X_RINT_OVR		;WILL GET US TO START UI TASK.
		CALL	X_SYSTATE		;GO TO S STATE
		MOV	XUCB,BX
		MOV	AH,[BX+UCB_UNIT]	;GET THE UNIT NUMBER,
		MOV	DI,[BX+UCB_DCB]		;AND GO TO THE DCB FOR THE
		MOV	AL,[DI+DCB_ID]		;DEVICE NUMBER.
		MOV	WORD PTR XSTATUS,AX	;PUT 'EM IN XSTATUS.
		CALL	XSETUNS			;START UI TASK.
		RET				;RETURN TO SCHEDULER.
X_READ_CTLX:
		MOV	TU_XCNT,0		;CLEAR THE TRANSMIT COUNT.
		AND	TU_STS,NOT UNIT_STOP	;ALSO REMOVE ANY STOP BIT.
		JMP	X_RINT_OVR
X_READ_CTLS:					;CONTROL S DURING WRITE:
		OR	TU_STS,UNIT_STOP	;STOP THE OUTPUT!
		JMP	X_RINT_OVR		;DONE.
X_READ_CTLQ:					;CONTROL Q DURING WRITE:
		TEST	TU_STS,UNIT_STOP	;WAS OUTPUT HALTED?
		JZ	X_RINT_OVR		;IF NOT, WE DO NOTHING!
		AND	TU_STS,NOT UNIT_STOP	;IT WAS: UN-HALT IT.
		CMP	TU_XCNT,0		;WAS ANYTHING LEFT TO SEND?
		JZ	X_RINT_OVR		;NO, NOTHING MORE TO DO.
		CALL	START_XMITTER		;YES: START TRANSMITTER,
		JMP	X_RINT_OVR		;LET INTERRUPT HANDLE THE REST.
;
;		-- NORMAL CHARACTER HANDLING HERE --
;
;		THE CHARACTER RECEIVED IS DROPPED INTO THE USER'S BUFFER.
;	ECHO CONSIDERATIONS ARE ADDRESSED. THE FULLNESS OF THE BUFFER IS 
;	CHECKED, AND IF APPLICABLE, THE TERMINAL BELL IS RUNG, OR THE REQUEST
;	IS RELEASED AS COMPLETE.
;
X_REG_CHAR:
		LES	DI,TU_RBUF		;GET THE READ BUFFER POINTER
		MOV	ES:[DI],AL		;DROP THE BYTE INTO IT.
		INC	TU_RBF0			;ADVANCE POINTER.
		INC	TU_RCNT			;BUMP RECEIVED CHARACTER COUNT
		MOV	SI,TU_RCNT		;GET FILL COUNT
		CMP	SI,TU_RMAX		;LAST AVAILABLE BUFFER CELL?
		JZ	X_FULL_BUFR		;YES, AND NO TERMINATOR!
		MOV	CX,1			;ASSUME DISPLAYABLE CHARACTER
		MOV	TU_XBF1,DS		;BEING SENT FROM ECHO BUFFER.
		LEA	DI,TU_BYTE2		;START OF ECHO IF DISPLAYABLE.
		CMP	AL,ASCII_SPACE		;TRULY VISIBLE CHARACTER?
		JAE	X_REG_1			;YES, NORMAL ECHO OPERATION.
		ADD	AL,040H			;NO, UPGRADE TO DISPLAYABLE,
		DEC	DI			;BACK UP TO CARET CELL,
		INC	CX			;SET UP FOR CONTROL CODE ECHO.
X_REG_1:
		MOV	TU_BYTE2,AL		;PUT ECHO CHAR INTO BYTE CELL.
		MOV	TU_XCNT,CX		;SET TRANSMISSION LENGTH,
		MOV	TU_XBF0,DI		;OFFSET OF STRING POINTER.
		ADD	TU_DELC,CX		;ADVANCE COUNT OF SCREEN POSI-
						;TIONS CONSUMED.
		CALL	START_XMITTER		;START THE ECHO SEQUENCE.
X_RINT_OVR:
		JMP	X_XEL			;DISMISS THE INTERRUPT.
X_FULL_BUFR:					;SERVICE BUFFER FULL CONDITION.
		DEC	TU_RBF0			;EXCEPT IN TRANSPARENT MODE,
		DEC	TU_RCNT			;BUFFER MAYN'T FILL UP
		MOV	AL,ASCII_BELL		;WITHOUT A TERMINATOR.
		CALL	ECHO_AL			;RING TERMINAL BELL.
		JMP	X_RINT_OVR		;FINISH INTERRUPT PROCESSING.
;
;		-- HANDLE CONTROL-X (ZAP THE WHOLE LINE) --
;
X_CTX:
		MOV	AX,TU_RCNT		;CLEAR THE RECEIVER BUFFER BACK
		SUB	TU_RBF0,AX		;TO ITS INITIAL STATE, AND THEN
		MOV	TU_RCNT,0		;RESET THE RECEIVED COUNT.
		OR	TU_STS,READ_CTLX	;SET THE CONTROL-X FLAG.
		CALL	START_XMITTER		;START THE TRANSMITTER,
		JMP	X_RINT_OVR		;AND LET THE TRANSMITTER
;
;		-- HANDLE RUBOUT (DELETE LAST CHARACTER) --
;
X_RUB:
		CMP	TU_RCNT,0		;ANYTHING ENTERED YET?
		JZ	X_RINT_OVR		;NO. RUBOUT IGNORED.
		DEC	TU_RBF0			;YES: BACK UP BUFFER PTR.
		DEC	TU_RCNT			;ALSO THE READ COUNT.
		LES	DI,TU_RBUF		;GET OLD POINTER AND THEN
		MOV	AL,ES:[DI]		;LAST BYTE RECEIVED.
		MOV	TU_XBF0,OFFSET BACK_SPACER	;ADDRESS DELETE STRING.
		MOV	TU_XBF1,CS		;(THAT'S IN TDRIVER SEGMENT)
		MOV	CX,3			;MOST DELETIONS= 3 BYTES.
		MOV	AH,1			;NORMAL CHARACTER= 1 POSITION.
		CMP	AL,ASCII_SPACE		;LAST CHARACTER DISPLAYABLE?
		JAE	X_RUB_1			;YES.
		MOV	CX,6			;NO: CONTROL CODE DELETION.
		INC	AH			;THAT FREES 2 POSITIONS.
X_RUB_1:
		MOV	AL,AH			;MAKE DELETED POSITIONS COUNT
		CBW				;SIXTEEEN BITS WIDE AND ADJUST
		SUB	TU_DELC,AX		;TOTAL POSITIONS COUNT WITH IT.
		MOV	TU_XCNT,CX		;SET LENGTH OF TRANSMISSION,
		CALL	START_XMITTER		;BEGIN THE OUTPUT,
		JMP	X_RINT_OVR		;DISMISS INTERRUPT.
;
;		-- TERMINATION CHARACTERS (CR, CTL-Z) --
;
;		OF THE TERMINATION CHARACTERS, ONLY THE CARRIAGE RETURN
;	IS ECHOED. AFTER THE CONTROL Z, THE CURRENT CURSOR POSITION
;	IS PRESERVED WHEN THE REQUEST COMPLETION IS DECLARED.
;
;		NOTE THAT THE INTERRUPT HANDLER INVOKES S STATE TO COMPLETE THE
;	REQUEST, THE DETAILS OF WHICH ARE HANDLED BY THE COMPLETION/RESCHED-
;	ULING ROUTINE XOVER.
;
X_CR:
		CALL	ECHO_AL			;ECHO A CARRIAGE RETURN,
		MOV	AL,ASCII_CR		;AND RETURN IT AS TERMINATOR.
X_TRM:
		CALL	X_SYSTATE		;GO TO S STATE
		MOV	XUCB,BX			;SET CRITICAL VARIABLE.
		MOV	XLASTCHR,AL		;ENTER TERMINATOR IN STATUS.
		LES	DI,TU_RBUF		;IT ALSO GOES
		MOV	ES:[DI],AL		;IN THE BUFFER
		INC	TU_RCNT			;AND IN THE COUNT.
;
X_READ_OVER:					;NOW COMPLETE THE REQUEST:
		AND	TU_STS,NOT (READ_BIT+XPRN_BIT+RD1_BIT) ;LOWER THE READ,
						;TRANSPARENCY, READ-ONE BITS.
		MOV	SI,TU_CIRB		;GET CURRENT IRB ADDRESS AND
		MOV	XCIRB,SI		;SET UP CRITICAL VARIABLE
		MOV	XPRIMARY,SUCCESS	;SET SUCCESS STATUS
		MOV	AX,TU_RCNT		;MOVE THE COUNT OF BYTES READ
		MOV	XCOUNT,AX		;TO THE STATUS
		JMP	XOVER			;COMPLETE THE REQUEST.
RCVR_INTRP	ENDP
;
;	SECTION 3.2:		TRANSMITTER BUFFER AVAILABLE HANDLERS.
;
;		THIS HANDLER PROPELS THE OUTPUT OF A STRING OF CHARACTERS
;	FROM EACH TO THE NEXT. UNLIKE EARLIER VERSIONS OF THE CONSOLE DRI-
;	VER, IT DOES NOT ASSUME EDGE-DETECT MODE AT THE INTERRUPT CONTROLLER,
;	BUT WILL FUNCTION IN EITHER EDGE OR LEVEL-DETECT MODES.
;
;		NOTE THAT, AS IN THE RECEIVED CHARACTER INTERRUPT HANDLER,
;	THE BX REGISTER CARRIES THE ADDRESS OF THE UCB, AND THE OTHER REGIS-
;	TERS ARE MORE OR LESS GENERAL PURPOSE.
;
XMTR_0		PROC	FAR			;TRANSMITTER INTERRUPT, UNIT 0.
		CALL	X_ELIPSE		;GO TO I STATE.
		MOV	AL,LOW X_SYS_DEV	;SET DEVICE NUMBER,
		MOV	DL,0			;    UNIT   NUMBER,
		JMP	FAR PTR XMTR_INTRP	;GO TO COMMON TRANSMIT HANDLER.
XMTR_0		ENDP
;
XMTR_1		PROC	FAR			;TRANSMITTER INTERRUPT, UNIT 1.
		CALL	X_ELIPSE		;GO TO I STATE.
		MOV	AL,LOW X_SYS_DEV	;SET DEVICE NUMBER,
		MOV	DL,1			;    UNIT NUMBER,
		JMP	FAR PTR XMTR_INTRP	;GO TO COMMON TRANSMIT HDLR.
XMTR_1		ENDP
;
;		** COMMON TRANSMIT INTERRUPT HANDLER **
;
;		THIS HANDLER IS REACHED FROM ALL CONTROL PATHS ORIGINATING 
;	AT A CONSOLE DRIVER UNIT-SPECIFIC TRANSMITTER INTERRUPT HANDLER.
;
;		WHEN THIS POINT IS REACHED, THE SYSTEM IS IN I STATE, AND
;	AL AND DL INDICATE THE INTERRUPTING DEVICE/UNIT.
;
XMTR_INTRP	PROC	NEAR
		CALL	X_DEVDATA		;GET UCB, DCB ADDRESSES.
		CMP	TU_XCNT,0		;IS THERE MORE TO TRANSMIT?
		JNZ	X_XMIT_0		;YES.
		TEST	TU_STS,WRITE_BIT	;NO: WAS A WRITE REQUESTED?
		JNZ	X_WRITE_OVER		;YES, COMPLETE IT.
		TEST	TU_STS,READ_CTLX	;NO: IS CTL X FLAG UP?
		JNZ	X_PROP_CTLX		;YES: HANDLE LINE DELETION.
		CALL	STOP_XMITTER		;NO: WHATEVER, IT'S OVER,
		JMP	X_XEL			;SO DISMISS THE INTERRUPT.
;
;			-- BACKSPACE WHOLE LINE --
;
;		THIS SEGMENT OF THE TRANSMITTER BUFFER AVAILABLE INTERRUPT
;	HANDLER COOPERATES WITH THE RECEIVED CHARACTER HANDLER IN IMPLE-
;	MENTING THE CONTROL-X FUNCTION. IF THE STATUS IN TU_STS INDICATES
;	THAT SUCH A DELETION IS IN PROGRESS, THE TRANSMITTER HANDLER WILL
;	RE-INITIATE THE WRITING OF THE SPECIAL SEQUENCE IN BACK_SPACER
;	UNTIL EVERY CHARACTER READ UP TO THIS POINT IN THE REQUEST HAS
;	BEEN DESTROYED.
;
X_PROP_CTLX:
		CMP	TU_DELC,0		;MORE TO DELETE?
		JZ	X_CTLX_DONE		;NO.
		DEC	TU_DELC			;YES: BANG DOWN ANOTHER BYTE.
		MOV	TU_XBF0,OFFSET BACK_SPACER	;SET UP FOR ANOTHER
		MOV	TU_XBF1,CS		;OUTPUT OF THE BACKSPACE
		MOV	TU_XCNT,3		;STRING.
		JMP	SHORT X_XMIT_1
X_CTLX_DONE:
		AND	TU_STS,NOT READ_CTLX	;WHEN DONE WITH DELETE,
		CALL	STOP_XMITTER		;TURN OFF THE TRANSMITTER AND
		JMP	SHORT X_WRITE_HALT	;CLEAR CTLX BIT AND DISMISS.
;
;			-- TRANSMIT NEXT CHARACTER IF WE CAN --
;
;		IF THERE IS A CHARACTER LEFT TO SEND, AND IF THE UNIT IS NOT
;	UNDER THE INFLUENCE OF A RECEIVED CONTROL S, THE CHARACTER IS SENT
;	AND THE TRANSMIT POINTER AND COUNTER ARE ADVANCED.
;
X_XMIT_0:
		TEST	TU_STS,UNIT_STOP	;IS UNIT HALTED FOR OUTPUT?
		JZ	X_XMIT_1		;IF NOT, TRANSMIT NEXT BYTE!
		CALL	STOP_XMITTER		;IF SO, QUENCH TRANSMITTER
		JMP	SHORT X_WRITE_HALT	;AND WAIT.
X_XMIT_1:
		LES	DI,TU_XBUF		;GET BUFFER POINTER
		MOV	AL,ES:[DI]		;GET NEXT CHARACTER TO SEND
		INC	TU_XBF0			;ADVANCE BUFFER POINTER
		DEC	TU_XCNT			;DECREMENT COUNT-TO-SEND
		TEST	AL,AL			;NULL CHARACTER?
		JZ	X_WRITE_OVER		;YES: FORCE WRITE TERMINATION
		MOV	DL,TU_DATP		;NO: GET DATA PORT ADDRESS
		XOR	DH,DH
		OUT	DX,AL			;TRANSMIT CHARACTER
X_WRITE_HALT:
		JMP	X_XEL			;DISMISS INTERRUPT.
;
;			-- WRITE REQUEST COMPLETION --
;
X_WRITE_OVER:
		CALL	STOP_XMITTER		;TRANSMISSION COMPLETE:
		MOV	TU_XCNT,0		;JUST IN CASE OF NULL CHARACTER
		AND	TU_STS,NOT WRITE_BIT	;CLEAR UNIT WRITE STATUS
		TEST	TU_STS,READ_BIT		;CONCURRENT READ REQUEST?
		JNZ	X_WRITE_HALT		;IF SO, LEAVE UNIT MARKED BUSY!
		CALL	X_SYSTATE		;NO: GO TO S STATE
		MOV	SI,TU_CIRB		;PUT ADDRESS OF CURRENT IRB
		MOV	XCIRB,SI		;INTO INTERFACE VARIABLE.
		MOV	XSTATUS,SUCCESS		;RETURN STATUS
		JMP	XOVER			;COMPLETE THE REQUEST
XMTR_INTRP	ENDP
TDRIVER		ENDS
		END
