PAGE	60,132
INCLUDE B:BIG.INC
;
;	CREATE.ASM:	TASK CREATION/DELETION/LOAD MANAGEMENT SERVICES.
;	F. W. PORRETTO, 12/15/82
;
;		THIS MODULE CONTAINS THE SERVICE ROUTINES FOR THE TASK CREATE
;	AND DELETE SERVICES. IT ENABLES A CALLING TASK TO SPECIFY THE ALLOCA-
;	TION OF DYNAMICALLY USABLE TCB, TCD AND 8087 SPACE FOR A TASK NOT IN-
;	CLUDED IN THE SYSTEM'S ROM-CONFIGURED TASK LOAD (I.E., WHICH IS NOT
;	REPRESENTED TO MTOS BY AN ENTRY IN X_TCD_TABLE).
;
;		ALSO SUPPORTED BY CODE IN THIS MODULE IS THE SYSTEM'S BUILT-IN
;	FACILITY FOR THE USE OF LOAD REGIONS ("OVERLAY REGIONS"). THE TWO ROU-
;	TINES X_OVIN AND X_OVOUT SUPPLY THE INTERFACE TO THE START/END SERVICE
;	ROUTINES IN START.ASM, WHILE THE X_LOADED SVC PROCESSOR COORDINATES
;	LOADER TASK FUNCTION WITH MTOS CONTINGENT PROCESSING.
;
		NAME	CREATE_DELETE_TASKS
;
MTOS_CS		SEGMENT PARA PUBLIC
		ASSUME	CS:MTOS_CS,DS:MTOS_DS,SS:MTOS_DS
		PUBLIC	X_CREATE,X_DELETE,X_DELTSK,X_LOADED,X_OVIN,X_OVOUT
		EXTRN	X_ALLOC_8087:NEAR
		EXTRN	X_DALLOC_8087:NEAR
		EXTRN	X_NEXT_START:NEAR
		EXTRN	X_SFLAGS:NEAR
		EXTRN	X_ENDT:FAR
		EXTRN	X_GET_TASK_ID:NEAR
		EXTRN	X_GET_TCB_ADR:NEAR
		EXTRN	X_GET_SMR:NEAR
		EXTRN	X_PUT_SMR:NEAR
		EXTRN	X_START_TASK:NEAR
		EXTRN	X_SET_READY:NEAR
		EXTRN	X_TASK_STATE:NEAR
		EXTRN	X_QUEUE:NEAR
		EXTRN	X_DQUEUE_TMB:NEAR
		EXTRN	X_OCB_SIZE_VAR:BYTE
;
;	CONVENIENCE SYMBOLS:
;
WTSTART		EQU	1		;WAIT-FOR-START COORDINATION.
;
LOADED_STS	EQU	1		;SDB OFFSET OF LOADER STATUS.
LOADED_ORD	EQU	2		;SDB OFFSET OF ORD ADDRESS.
;
LRCB_ORD	EQU	RCB_PARAM	;QCOPY PARAMETER OFFSET FOR ORD ADDRESS
LRCB_TLD	EQU	LRCB_ORD+4	;QCOPY PARAMETER OFFSET FOR TLD ADDRESS
PAGE
;*************************************************************************
;*	X_FIND_OCB:	LOCATE THE OCB FOR A GIVEN OVERLAY REGION.       *
;*************************************************************************
;
;	INPUT:		CL= OVERLAY REGION IDENTIFIER NUMBER.
;	OUTPUT:		SI= OCB ADDRESS 
;
X_FIND_OCB	PROC	NEAR
		MOV	AL,CL
		MOV	SI,OFFSET X_OCB_TABLE
		MUL	CS:X_OCB_SIZE_VAR	;COMPUTE OCB OFFSET INTO TABLE
		ADD	SI,AX			;ADD TO BASE OF TABLE.
		RET
X_FIND_OCB	ENDP
PAGE
;*************************************************************
;*      START_LOAD: INVOKE THE LOADER TASK TO LOAD!          *
;*************************************************************
;
;	INPUT:	BP= ADDRESS OF RCB BEING TAKEN OVER,
;		SI= ADDRESS OF OCB, ALREADY LOCKED.
;	OUTPUT:	NONE.
;
START_LOAD	PROC	NEAR
		PUSH	BX
		PUSH	CX
		PUSH	BP
		PUSH	SI
		PUSH	DI
		MOV	DI,DS:[BP+RCB_TCB]	;GET TCB OF TASK TO LOAD.
		PUSH	DI			;SAVE THE ADDRESS OF TCB
		MOV	[SI+OCB_RCB],BP		;REGISTER RCB OF LOAD.
		CALL	X_GET_SMR		;GET LOADER RCB.
		LES	DI,DWORD PTR [SI+OCB_ORD]	;ADDRESS ORD,
		MOV	[BX+LRCB_ORD],DI	;AND PUT THE ADDRESS INTO THE
		MOV	[BX+LRCB_ORD+2],ES	;LOADER RCB.
		POP	DI			;DI = ADDRESS OF TCB
		LES	DI,DWORD PTR [DI+TCB_TCD]	;TCD OF TASK TO LOAD
		LES	DI,DWORD PTR ES:[DI+TCD_LOAD]	;LEADS TO TLD.
		MOV	[BX+LRCB_TLD],DI	;PUT TLD ADDRESS INTO THE
		MOV	[BX+LRCB_TLD+2],ES	;LOADER RCB.
		MOV	CX,00FFH		;HIGH PRI, NO COORD.
		XOR	SI,SI			;NO REQUESTOR.
		MOV	BP,BX			;LOADER RCB TO BP,
		MOV	AX,X_LOADER_TASK	;DETERMINE THE TCB ADDRESS OF
		CALL	X_GET_TCB_ADR		;THE LOADER TASK,
		MOV	DI,BX			;THAT'S WHOM WE'RE STARTING,
		CALL	X_START_TASK		;DO IT, ALREADY!
		POP	DI
		POP	SI
		POP	BP
		POP	CX
		POP	BX
		RET
START_LOAD	ENDP
PAGE
;******************************************************************************
;*	X_OVIN:		REGISTER PERMISSION TO START TASK WITH THE OVERLAY    *
;*			MANAGER, OR WAIT.                                     *
;******************************************************************************
;
;		THIS ROUTINE IS CALLED UPON BY THE START SUBSYSTEM TO PROCURE
;	PERMISSION TO START A LOADABLE TASK. IF THE TASK MAY BE STARTED IMMEDI-
;	ATELY (I.E., IF IT IS CURRENTLY RESIDENT IN THE DESIGNATED OVERLAY
;	REGION), PERMISSION IS GRANTED; OTHERWISE, THE OVERLAY MANAGER ASSUMES
;	CONTROL OF THE START REQUEST BY TAKING THE RCB AWAY FROM THE START
;	SUBSYSTEM AND USING IT TO CONTROL THE ACTION OF THE LOADER.
;
;	INPUT:		BP= ADDRESS OF RCB TO BE EVALUATED,
;	OUTPUT:		CARRY=0 IF PERMISSION GRANTED, OR
;			CARRY=1 IF PERMISSION DENIED.
;
X_OVIN		PROC	NEAR
		MOV	AX,X_LOADER_TASK	;GET LOADER TASK ID.
		CMP	AX,-1			;IS THERE ONE?
		JNE	XOV0			;YES.
		STC				;NO, RETURN PERMISSION DENIED
		RET
XOV0:
		PUSH	BX
		PUSH	CX
		PUSH	SI
		PUSH	DI
		PUSH	BP
		PUSH	ES
		MOV	BX,DS:[BP+RCB_TCB]	;GET TARGET TCB ADDRESS.
		LES	DI,DWORD PTR [BX+TCB_TCD]	;GET TCD ADDRESS,
		LES	DI,DWORD PTR ES:[DI+TCD_LOAD]	;GET LOAD DSCPTR.
		MOV	CL,ES:[DI+TLD_OVID]	;GET ORD NUMBER,
		CALL	X_FIND_OCB		;FIND OCB,
		CMP	BYTE PTR [SI+OCB_STS],0	;REGION BUSY?
		JE	XOV1			;NO.
		CMP	BX,[SI+OCB_TCB]		;YES: RESIDENT= TARGET??
		JE	XOV1			;YES.
		LEA	DI,[SI+OCB_WQ]		;NO: PUT THE RCB ONTO
		MOV	BX,BP			;THE OVERLAY WAIT QUEUE.
		CALL	X_QUEUE
		STC				;DENY PERMISSION.
		JMP	SHORT XOV9
XOV1:
		MOV	BYTE PTR [SI+OCB_STS],1	;BUSY THE OVERLAY REGION.
		CMP	BX,[SI+OCB_TCB]		;TARGET= RESIDENT TASK?
		JNE	XOV2			;NO, LOAD IS NECESSARY.
		CLC				;GIVE PERMISSION,
		JMP	SHORT XOV9		;AND RETURN.
XOV2:
		CALL	START_LOAD		;START THE LOADER TASK.
		STC				;DENY PERMISSION.
XOV9:
		POP	ES
		POP	BP
		POP	DI
		POP	SI
		POP	CX
		POP	BX
		RET
X_OVIN		ENDP
PAGE
;**************************************************************************
;*	X_OVOUT:	REGISTER THE TERMINATION OF A LOADABLE TASK.      *
;**************************************************************************
;
;		THIS ROUTINE IS CALLED BY THE START/END SUBSYSTEM WHENEVER A
;	LOADABLE TASK ISSUES AN ENDT. IT FREES THE OVERLAY REGION FOR USE BY 
;	ANY OTHER LOADABLE TASK.
;
;	INPUT:		BX= TCB ADDRESS.
;	OUTPUT:		NONE.
;
X_OVOUT		PROC	NEAR
		PUSH	SI
		PUSH	BP
		PUSH	CX
		MOV	CX,X_OVERLAYS_NUM	;GET NUMBER OF OVERLAY REGIONS
		MOV	SI,OFFSET X_OCB_TABLE	;GET ZEROTH OCB ADDRESS
XOVO1:
		CMP	BX,[SI+OCB_TCB]		;RIGHT ONE?
		JE	XOVO2			;YES.
		ADD	SI,OCB_SIZE		;NO:
		LOOP	XOVO1			;KEEP LOOKING IF ANY MORE.
		JMP	SHORT XOVO5		;FAILED TO FIND IT.
XOVO2:
		MOV	BYTE PTR [SI+OCB_STS],0	;SET NOT IN USE.
		CMP	WORD PTR [SI+OCB_WQ],0	;ANY WAITING LOAD OPERATIONS?
		JE	XOVO3			;NO.
		MOV	BP,[SI+OCB_WQ]		;YES, GET FIRST ONE WAITING
		MOV	AX,DS:[BP]		;UNLINK IT FROM
		MOV	[SI+OCB_WQ],AX		;THE WAIT QUEUE,
		CALL	START_LOAD		;START THE LOAD OPERATION.
XOVO3:
XOVO5:
		POP	CX
		POP	BP
		POP	SI
		RET
X_OVOUT		ENDP
PAGE
;****************************************************************
;*    ALLOC_TCB:        ALLOCATE TCB AND TCD DYNAMICALLY        *
;****************************************************************
;
;	INPUT:	NONE.
;	OUTPUT:	AX RETURNS TASK NUMBER,
;		BX RETURNS TCB ADDRESS,
;
ALLOC_TCB	PROC	NEAR
		CALL	X_GET_SMR		;ALLOCATE SMR FOR TCD.
		MOV	DI,BX			;SAVE TCD IN DI.
		CALL	X_GET_SMR		;GET SMR BLOCK FOR TCB
		MOV	[BX+TCB_TCD],DI		;LINK THE TCD TO THE TCB
		MOV	[BX+TCB_TCD+2],DS	;THUS.
		CALL	X_GET_TASK_ID		;SEND TASK ID IN AX.
		RET
ALLOC_TCB	ENDP
;****************************************************************
;*   DALLOC_TCB:        DEALLOCATE TCB/TCD COMBINATION.         *
;****************************************************************
;
;	INPUT:	BX= TCB ADDRESS, PROPERLY LINKED TO TCD.
;	OUTPUT:	NONE.
;
DALLOC_TCB	PROC	NEAR
		PUSH	BX			;SAVE TCB OFFSET,
		MOV	BX,[BX+TCB_TCD]		;AND DEALLOCATE TCD FIRST:
		CALL	X_PUT_SMR		;THERE IT GOES.
		POP	BX			;RETRIEVE TCB OFFSET
		CALL	X_PUT_SMR		;AND RETURN IT TO SMR POOL
		RET
DALLOC_TCB	ENDP
PAGE
;****************************************************************
;*	X_CREATE:	CREATE TCB/TCD COMBINATION.             *
;****************************************************************
;
;	INPUT:	ES:SI= SDB ADDRESS, BX= REQUESTOR'S TCB.
;	OUTPUT: NONE.
;
;	ATTEMPT TO ALLOCATE THE TCB, TCD AND IF NECESSARY THE 8087 SAVE AREA
;	FOR A DYNAMIC TASK. IN CASE OF UNAVAILABILITY, RETURN FAILURE STATUS.
;
CREATE_TCD	EQU	5	;POINTER TO TCD IMAGE.
CREATE_BUFR	EQU	1	;POINTER TO WORD BUFFER FOR TASK ID.
;
X_CREATE	PROC	FAR
		CALL	ALLOC_TCB		;ATTEMPT TCB/TCD ALLOCATION
		PUSH	DS			;PUT MTOS_DS INTO ES AND
		PUSH	ES			;SDB SEGMENT INTO DS, SHORTLY.
		LDS	DI,DWORD PTR ES:[SI+CREATE_BUFR] ;GET BFR POINTER
		MOV	[DI],AX			;PUT TASK ID THERE.
		POP	DS			;NOW DO THE DS/ES REVERSAL:
		POP	ES			;THUS.
		CLD				;CLEAR DIRECTION FLAG
		MOV	DI,ES:[BX+TCB_TCD]	;GET TCD ADDRESS
		LDS	SI,DWORD PTR [SI+CREATE_TCD]	;ADDRESS TCD IMAGE
		MOV	CX,TCD_SIZE		;IT'S THIS BIG
	REP	MOVSB				;MOVE IT TO ALLOCATED TCD.
		PUSH	ES
		POP	DS			;RESTORE MTOS_DS TO DS REG.
		MOV	DI,[BX+TCB_TCD]		;DI= TCD ADDRESS IN MTOS_DS,
		MOV	AL,[DI+TCD_PID]		;GET CPU ID,
		MOV	[BX+TCB_PID],AL		;TO TCB,
		MOV	AL,[DI+TCD_FLAGS]	;GET TASK FLAGS,
		OR	AL,TFL_ALLOC		;RAISE ALLOCATION FLAG,
		MOV	[BX+TCB_FLAGS],AL	;PUT FLAGS TO TCB.
		TEST	AL,TFL_8087		;8087 USER, THIS TASK?
		JE	XCRT15			;NO.
		CALL	X_ALLOC_8087		;YES: TRY FOR IT...
		MOV	[BX+TCB_8087],BP	;IF WE GOT IT, LINK IT INTO TCB
		JNC	XCRT15			;GOT IT.
		CALL	DALLOC_TCB		;NO GO! DEALLOCATE TCB/TCD,
		MOV	AL,SVC_FAILED		;RETURN FAILURE STATUS.
		CALL	X_SFLAGS
		RET
XCRT15:
		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_OVPRI	;PRI-OVERRIDE?
		JE	XCRT20			;NO.
		MOV	SI,BX			;YES:
		CALL	X_GET_SMR		;GET AN SMR BLOCK
		MOV	[SI+TCB_TMB],BX		;FOR THE OVERRIDE TMB.
XCRT20:
		RET				;DONE!
X_CREATE	ENDP
PAGE
;*****************************************************************************
;*	X_DELETE:	DESTROY DYNAMIC TASK'S TCB/TCD IF APPROPRIATE.       *
;*****************************************************************************
;
;	INPUT:		ES:SI= SDB ADDRESS (NOTHING IN IT THOUGH)
;			BX= REQUESTOR'S TCB ADDRESS
;	OUTPUT:		NONE.
;
;		THIS ROUTINE FORCES THE REQUESTOR TO DORMANT STATE THROUGH THE
;	STANDARD X_ENDT SERVICE, AND THEN LOOKS AT WHETHER THE TASK STAYED IN
;	DORMANCY OR WAS REACTIVATED BY A PENDING RCB. IF THE FORMER, THE TASK'S
;	TCB, TCD AND 8087 SAVE AREA ARE DELETED. IF THE LATTER, THE REQUEST IS
;	MERELY AN ENDT.
;
X_DELETE	PROC	FAR
		PUSH	BX			;SAVE TCB ADDRESS
		CALL	X_ENDT			;TERMINATE CALLING TASK.
		POP	BX
		CALL	X_GET_TASK_ID		;GET TASK'S ID NUMBER
		CMP	AX,X_STATIC_TASKS	;IS IT IN THE STATIC SET?
		JAE	XDLT12			;NO.
		RET				;A FANCY ENDT.
XDLT12:
		CMP	BYTE PTR [BX+TCB_STATE],DORMANT	;IS TASK DORMANT?
		JNE	XDLT20			;NO, TERMINATE THE SERVICE.
		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_8087	;8087 USER?
		JZ	XDLT10			;NO.
		MOV	BP,[BX+TCB_8087]	;YES: GET ADDRESS OF SAVE AREA
		CALL	X_DALLOC_8087		;FREE IT.
XDLT10:
		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_OVPRI	;PRI-OVERRIDE?
		JE	XDLT15			;NO.
		PUSH	BX			;YES:
		MOV	BX,[BX+TCB_TMB]		;GET TMB FROM TIMER QUEUE,
		CALL	X_DQUEUE_TMB		;IF IT'S RUNNING,
		CALL	X_PUT_SMR		;AND DEALLOCATE IT.
		POP	BX			;RETRIEVE TCB ADDRESS.
XDLT15:
		CALL	DALLOC_TCB		;DEALLOCATE THE TCB AND TCD.
XDLT20:
		RET
X_DELETE	ENDP
PAGE
;*****************************************************************************
;*	X_DELTSK:	DELETE SPECIFIED TARGET TASK IF IT IS DORMANT.       *
;*****************************************************************************
;
;	INPUT:	ES:SI=	SDB ADDRESS,
;		BX=	REQUESTOR'S TCB ADDRESS
;
;		THIS ROUTINE WILL CAUSE THE DELETION OF A TASK'S TCB/TCD IF
;	THAT TASK IS NOT IN THE STATIC SET AND IF IT IS CURRENTLY DORMANT WITH
;	NO RCBS PENDING.
;
DELTSK_ID	EQU	1
;
X_DELTSK	PROC	FAR
		MOV	AX,ES:[SI+DELTSK_ID]	;GET TASK NUMBER OF TARGET
		CMP	AX,X_STATIC_TASKS	;IN THE STATIC SET?
		JAE	XDE0			;NO.
XDE4:
		MOV	AX,SVC_FAILED+PARM_ERR	;YES:
XDE2:
		CALL	X_SFLAGS		;PARAMETER ERROR.
		RET
XDE0:
		CALL	X_GET_TCB_ADR		;GET TASK'S TCB ADDRESS
		JC	XDE4			;NOT A TASK!
		CMP	BYTE PTR [BX+TCB_STATE],DORMANT	;TASK DORMANT??
		JE	XDE1			;YES, PROCEED.
		MOV	AX,SVC_FAILED+FLAG_CF	;TELL USER I CAN'T DO IT.
		JMP	SHORT XDE2
XDE1:
		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_8087	;NDP USER?
		JE	XDE3			;NO.
		MOV	BP,[BX+TCB_8087]	;YES:
		CALL	X_DALLOC_8087		;DEALLOCATE 8087 SAVE AREA.
XDE3:
		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_OVPRI	;PRI-OVERRIDE?
		JE	XDE5			;NO.
		PUSH	BX			;YES: SAVE TCB ADDR,
		MOV	BX,[BX+TCB_TMB]		;GET TMB ADDR,
		CALL	X_DQUEUE_TMB		;DEACTIVATE IT,
		CALL	X_PUT_SMR		;DEALLOCATE IT,
		POP	BX			;RESTORE TCB ADDR.
XDE5:
		CALL	DALLOC_TCB		;TCB/TCD COMBINATION.
		RET
X_DELTSK	ENDP
PAGE
;******************************************************************************
;*	X_LOADED:	SVC ROUTINE TO FORCE THE START OF A TASK WHICH HAS    *
;*			JUST FINISHED BEING LOADED.                           *
;******************************************************************************
;
;	THIS ROUTINE COMPLETES THE LOAD OPERATION'S CONTINGENT PROCESSING BY
;	FINISHING ALL THE DETAILS OF TASK STARTUP.
;
;	INPUT:	ES:SI=	SDB ADDRESS,
;		BX=	LOADER TASK'S TCB ADDRESS.
;
;	OUTPUT:	NONE.
;
X_LOADED	PROC	FAR
		MOV	DL,ES:[SI+LOADED_STS]	;GET STATUS OF LOAD OPERATION.
		LES	DI,DWORD PTR ES:[SI+LOADED_ORD]	;GET THE ORD ADDRESS
		MOV	CL,ES:[DI+ORD_ID]	;GET THE OVERLAY REGION ID.
		CALL	X_FIND_OCB		;LOCATE THE REGION'S OCB.
		MOV	BP,[SI+OCB_RCB]		;GET THE LOADING RCB
		MOV	WORD PTR [SI+OCB_RCB],0	;CLEAR THE CELL.
		MOV	BX,DS:[BP+RCB_WTCB]	;GET REQUESTOR'S TCB ADDR.
		TEST	DL,DL			;GOOD LOAD?
		JE	XLS0			;YES.
		MOV	WORD PTR [SI+OCB_TCB],0	;NO: INDICATE NO TASK RESIDENT!
		TEST	BX,BX			;REQUESTOR= TASK?
		JE	XLS1			;NO.
		LES	DI,DWORD PTR [BX+TCB_SP]	;YES: SET HIS FAIL FLAG
		OR	WORD PTR ES:[DI+STK_FL],SVC_FAILED+FLAG_OF
		CALL	X_SET_READY		;AND SET HIM READY AGAIN.
XLS1:
		MOV	BX,BP
		CALL	X_PUT_SMR		;DEALLOCATE THE SMR BLOCK.
		RET
XLS0:
		MOV	AX,DS:[BP+RCB_TCB]	;GET TARGET'S TCB ADDRESS,
		MOV	[SI+OCB_TCB],AX		;REGISTER NEW RESIDENT TASK.
		MOV	SI,BP			;RCB PTR TO CONVENIENT REGISTER
		MOV	AL,[SI+RCB_CORD]	;GET COORDINATION MODE.
		TEST	AL,AL			;NO COORDINATION?
		JE	XLS5			;CORRECT.
		CMP	AL,WTSTART		;NO: WAIT FOR START?
		MOV	AL,BLOCK+STA_STRT
		JE	XLS4			;YES.
		MOV	AL,BLOCK+STA_ENDT	;NO, MUST BE WAIT FOR END.
XLS4:
		CALL	X_TASK_STATE		;CHANGE REQUESTOR'S STATE
		JMP	SHORT XLS6
XLS5:
		CALL	X_SET_READY		;REQUESTOR MAY CONTINUE.
XLS6:
		MOV	BX,[SI+RCB_TCB]		;GET TARGET TCB ADDRESS
		MOV	AX,[BX+TCB_QRST]	;LINK THE RETURNING RCB
		MOV	[BX+TCB_QRST],SI	;TO THE HEAD OF THE TCB'S
		MOV	[SI],AX			;RCB WAIT QUEUE.
		CALL	X_NEXT_START		;STANDARD NEW RCB PROCEDURE.
XLD9:
		RET
X_LOADED	ENDP
MTOS_CS		ENDS
		END
