PAGE	60,132
INCLUDE B:EQUS.EQU
;
;		** MTOS-86 LEVEL II DYNAMIC  DEBUGGER **
;		** F. W. PORRETTO, NOVENBER 12, 1982  **
;		** MODIFIED 2/28/84 BY F. W. PORRETTO **
;		   FEATURES ADDED:
;			-BASE COMMAND,
;			-SOFT BREAKPOINTS IN THE GO COMMAND,
;			-TCD COMMAND.
;		** MODIFIED 4/18/84 BY F. W. PORRETTO **
;		   FEATURES ADDED:
;			-ABBREVIATED KEYWORDS,
;			-IMPROVED "FIRST-TIME-THROUGH" HANDLING.
;
;	THIS IS THE MTOS TASK DEBUGGER, INTENDED TO BE USED TO DEBUG TASKS IN
;	PROTOTYPE (RAM-BASED) SYSTEMS. IT IS NOT FOR USE ON DEVICE DRIVERS OR
;	ON CODE WHICH IS PART OF THE MTOS NUCLEUS.
;
		NAME	DYNAMIC_DEBUGGER
;
		PUBLIC	XDB_ENTRY,XDB_STACK
;
;	AS WITH ALL OTHER MTOS UTILITY TASKS, THE SVC INTERRUPT VECTOR NUMBER
;	IS DEFINED LOCALLY, TO ELIMINATE ANY DEPENDENCY ON EXTERNAL LINKAGE OR
;	INCLUDE FILES FOR THIS PIECE OF INFORMATION.
;
X_SVC_VECTOR	EQU	186
;
DEB_CODE	SEGMENT	PARA PUBLIC
		ASSUME	CS:DEB_CODE,DS:DEB_INIT
;
;	THE CODE SEGMENT IS USED BOTH FOR THE DEBUGGER'S EXECUTABLE CODE AND
;	FOR ITS INVARIANT TABLES AND MESSAGES.
;
PAGE
;
;		DEBUGGER INVARIANT TABLES AND MESSAGES:
;
;	THE KEYWORD TABLE, WHICH FOLLOWS, IS A TABLE OF VARIABLE-LENGTH ENTRIES
;	EACH OF WHICH DEFINES A DEBUGGER KEYWORD. THE FORMAT OF THE ENTRY IS:
;
;	BYTE 0:	     THE LENGTH OF THE KEYWORD IN BYTES: CALL IT K.
;	BYTE 1:	     THE MINIMUM LENGTH OF MATCHING INPUT TO MATCH THE KEYWORD.
;	BYTES 2-K+1: THE KEYWORD ITSELF.
;
KEYWRD_TABLE	EQU	THIS BYTE
		DB	2,2,'ES'
		DB	2,2,'DI'
		DB	2,2,'DX'
		DB	2,2,'CX'
		DB	2,2,'DS'
		DB	2,2,'SI'
		DB	2,2,'BP'
		DB	2,2,'BX'
		DB	2,2,'AX'
		DB	2,2,'IP'
		DB	2,2,'CS'
		DB	2,2,'FL'
		DB	2,2,'SS'
		DB	2,2,'SP'
		DB	3,2,'SET'
		DB	5,3,'QUERY'
		DB	5,4,'START'
		DB	4,2,'HALT'
		DB	4,2,'CONT'
		DB	7,3,'DISPLAY'
		DB	4,2,'BYTE'
		DB	4,2,'WORD'
		DB	4,2,'KILL'
		DB	2,2,'GO'
		DB	4,2,'EXIT'
		DB	5,5,'EXITK'
		DB	5,3,'GOCPU'
		DB	4,2,'REGS'
		DB	6,2,'BREAKS'
		DB	5,4,'STACK'
		DB	7,3,'QUALIFY'
		DB	4,2,'BASE'
		DB	3,2,'TCD'
		DB	0FFH
KEY_MAX		EQU	33
;
JUMP_TABLE	EQU	THIS WORD
		DW	R_REG
		DW	R_REG
		DW	R_REG
		DW	R_REG
		DW	R_REG
		DW	R_REG
		DW	R_REG
		DW	R_REG
		DW	R_REG
		DW	R_REG
		DW	R_REG
		DW	R_REG
		DW	R_SPSS
		DW	R_SPSS
		DW	R_SET
		DW	R_QUERY
		DW	R_START
		DW	R_HALT
		DW	R_CONT
		DW	R_DISPLAY
		DW	R_BYTE
		DW	R_WORD
		DW	R_KILL
		DW	R_GO
		DW	R_EXIT
		DW	R_EXITK
		DW	R_GOCPU
		DW	R_REGS
		DW	R_BREAKS
		DW	R_STACK
		DW	R_QUALIFY
		DW	R_BASE
		DW	R_TCD
;
HEX_DIGITS	DB	'0123456789ABCDEF'	;VALID HEXADECIMAL DIGITS.
TERM_TABLE	DB	' ,:;',CR,0		;TERMINATORS FOR GET_ATOM.
NR_TERMS	EQU	$-TERM_TABLE
;
ENDT_SDB	DB	03H			;TERMINATION SDB.
;
SEF_SDB		DB	08H,0			;LOCAL FLAG GROUP,
		DW	0001H			;SET FLAG 0.
;
START_MSG	DB	13,10,'** DYNAMIC DEBUGGER, V1.1 **',13,10,0
PROMPT_MSG	DB	13,10,'DEBUG>',0
;
STATE_MSGS	DW	DOR_MSG,RDY_MSG,BLK_MSG,ACT_MSG
DOR_MSG		DB	13,10,'TASK DORMANT',0
RDY_MSG		DB	13,10,'TASK READY',0
BLK_MSG		DB	13,10,'TASK BLOCKED',0
ACT_MSG		DB	13,10,'TASK ACTIVE',0
;
E_MSGS		DW	SYNERR,BKNOW,IDUSED,NOID,ALIVE,NOTASK,NOGO
SYNERR		DB	13,10,'SYNTAX ERROR!',0
BKNOW		DB	13,10,'ALREADY A BREAKPOINT!',0
IDUSED		DB	13,10,'BREAKPOINT ID IN USE!',0
NOID		DB	13,10,'NO SUCH BREAKPOINT!',0
ALIVE		DB	13,10,'TASK NOT HALTED!',0
NOTASK		DB	13,10,'NO SUCH TASK!',0
NOGO		DB	13,10,'ILLEGAL REQUEST!',0
;
ANY_STRING	DB	'ANY.'
DEB_CODE	ENDS				;FOR A MOMENT.
PAGE
;
;	INTERNALLY USEFUL SYMBOLS:
;
LOWER_A		EQU	061H
LOWER_Z		EQU	07AH
LOWER_TO_UPPER	EQU	061H-041H
;
ES_INDEX	EQU	1		;ES KEYWORD NUMBER.
DI_INDEX	EQU	ES_INDEX+1	;DI
DX_INDEX	EQU	DI_INDEX+1	;DX
CX_INDEX	EQU	DX_INDEX+1	;CX
DS_INDEX	EQU	CX_INDEX+1	;DS
SI_INDEX	EQU	DS_INDEX+1	;SI
BP_INDEX	EQU	SI_INDEX+1	;BP
BX_INDEX	EQU	BP_INDEX+1	;BX
AX_INDEX	EQU	BX_INDEX+1	;AX
IP_INDEX	EQU	AX_INDEX+1	;IP
CS_INDEX	EQU	IP_INDEX+1	;CS
FL_INDEX	EQU	CS_INDEX+1	;FL
SS_INDEX	EQU	FL_INDEX+1	;SS
SP_INDEX	EQU	SS_INDEX+1	;SP
;
;
BREAK_USE	EQU	0	;BREAKPOINT IN USE.
BREAK_CPU	EQU	1	;CPU NUMBER.
BREAK_OFF	EQU	2	;BREAKPOINT OFFSET.
BREAK_SEG	EQU	4	;BREAKPOINT SEGMENT.
BREAK_FLAGS	EQU	6	;BREAKPOINT FLAGS.
BREAK_TASK	EQU	8	;TASK TO WHICH BREAKPOINT IS RESTRICTED, OR -1.
BREAK_PASS	EQU	10	;NUMBER OF PASSES THROUGH BREAKPOINT BEFORE
				;TAKING DEBUGGER ACTION.
BREAK_SIZE	EQU	12	;SIZE OF BREAKPOINT TABLE ENTRY.
;
NUMBER		EQU	KEY_MAX+1	;NUMERIC INPUT.
ADDRESS		EQU	NUMBER+1	;ADDRESS INPUT.
HBRKPOINT	EQU	ADDRESS+1	;HARD BREAKPOINT ID INPUT.
SBRKPOINT	EQU	HBRKPOINT+1	;SOFT BREAKPOINT ID INPUT.
;
TRAP_ON		EQU	0100H		;T-BIT IN FLAGS REGISTER.
CR		EQU	0DH	;CARRIAGE RETURN.
HARD_POINT	EQU	0	;FLAGS FOR HARD BREAKPOINT.
SOFT_POINT	EQU	1	;  "    "  SOFT     "
ASCII_A		EQU	041H
ASCII_Z		EQU	05AH
ASCII_F		EQU	046H
ASCII_0		EQU	030H
ASCII_9		EQU	039H
ASCII_SPACE	EQU	020H
ASCII_COLON	EQU	03AH
SPC		EQU	ASCII_SPACE
AT_SIGN		EQU	040H
DOLLAR_SIGN	EQU	024H
;
E_TERM		EQU	0FFFFH	;TERMINATION CODE.
E_SYN		EQU	1	;SYNTAX ERROR.
E_BKNOW		EQU	2	;ALREADY A BREAKPOINT.
E_IDUSED	EQU	3	;BREAKPOINT ID IN USE.
E_NOID		EQU	4	;NO SUCH BREAKPOINT.
E_ALIVE		EQU	5	;TASK ALREADY RUNNING.
E_NOTASK	EQU	6	;NO SUCH TASK.
E_NOGO		EQU	7	;ILLEGAL ACTIVITY REQUESTED.
PAGE
;
;	INITIALIZATION IMAGE OF RAM DATA SEGMENT:
;
DEB_INIT	SEGMENT	PARA PUBLIC
INIT_BLOCK	EQU	THIS WORD
;
QCOPY_SDB	DB	18H		;SDB FOR COPYING START PARAMETERS:
		DW	QCOPY_RCB,0	;ADDRESS OF RCB BUFFER.
QCOPY_RCB	DB	16 DUP (?)	;RCB BUFFER: FIRST 16 BYTES NOT USED.
QBREAK_ID	DB	?		;BREAKPOINT ID OR 0.
QBREAK_FLAGS	DB	?		;FLAGS FOR BREAKPOINT.
CURR_CPU	DB	?		;CPU NUMBER.
		DB	?		;ALIGNMENT BYTE.
QBREAK_TASK	DW	?		;TASK NUMBER OF BREAKPOINTED TASK.
TCB_ADDR	DD	?		;TCB ADDRESS OF BREAKPOINTED TASK.
QBREAK_ADDR	DD	?		;ADDRESS OF BREAKPOINT IF ANY.
		DW	?		;UNUSED WORD.
;
TOUT_SDB	DB	015H,0C3H	;OUTPUT-TO-CONSOLE SDB:
OUT_DEV		DW	?		;OUTPUT DEVICE/UNIT.
		DB	0,0,0		;NO EVENT FLAG USAGE.
		DW	IO_STS,0	;ADDRESS OF STATUS DWORD,
		DW	OUT_PARMS,0	;ADDRESS OF PARAMETER BLOCK.
OUT_PARMS	EQU	THIS WORD	;PARAMETER BLOCK:
OUT_OFF		DW	?		;BUFFER OFFSET
OUT_SEG		DW	?		;BUFFER SEGMENT
OUT_LEN		DW	255		;BUFFER LENGTH.
;
TIN_SDB		DB	015H,0C2H	;INPUT-FROM-CONSOLE SDB:
IN_DEV		DW	?		;DEVICE/UNIT.
		DB	0,0,0		;NO EVENT FLAG USAGE.
		DW	IO_STS,0	;ADDRESS OF STATUS DWORD,
		DW	IN_PARMS,0	;ADDRESS OF PARAMETER BLOCK.
IN_PARMS	EQU	THIS WORD	;PARAMETER BLOCK:
IN_OFF		DW	IN_LINE+1	;BUFFER OFFSET,
IN_SEG		DW	?		;BUFFER SEGMENT,
IN_LEN		DW	?		;BUFFER CAPACITY.
IO_STS		EQU	THIS DWORD	;STATUS DOUBLEWORD.
		DW	?		;PRIMARY/SECONDARY STATUSES,
INPUT_CT	DW	?		;COUNT OF CHARACTERS FOR INPUT.
IN_LINE		DB	80,80 DUP (?)	;STANDARD LINE INPUT BUFFER.
;
SBREAK_SDB	DB	019H,0		;SET-BREAKPOINT SDB:
SBREAK_CPU	DB	?		;CPU NUMBER,
SBREAK_ID	DB	?		;BREAKPOINT ID REQUESTED,
SBREAK_FLAGS	DB	?		;BREAKPOINT FLAGS,
SBREAK_ADDR	EQU	THIS DWORD
SBREAK_OFF	DW	?		;BREAKPOINT OFFSET,
SBREAK_SEG	DW	?		;BREAKPOINT SEGMENT.
;
NUM_BREAKS	EQU	16		;ALLOW FOR 16 BREAKPOINTS.
BREAK_TABLE	DB	(NUM_BREAKS * BREAK_SIZE) DUP (0)
;
KBREAK_SDB	DB	019H,1		;KILL-BREAKPOINT SDB:
KBREAK_CPU	DB	?		;CPU NUMBER,
KBREAK_ID	DB	?		;BREAKPOINT ID TO BE KILLED,
;
START_SDB	DB	1,3		;START-TASK SDB:
START_ID	DW	?		;ID OF TARGET TASK.
		DB	?		;UNUSED PRIORITY VALUE BYTE.
START_P1	DW	0		;STARTUP PARAMETER 1 (AX)
START_P2	DW	0		;STARTUP PARAMETER 2 (BX)
;
QBREAK_BUF	DB	'@ , TASK:     ',13,10,0 ;BREAKPOINT EXECUTION REPORT.
OUTW_BUF	DB	13,10,'     ',0	;FOR NUMERIC WORD DISPLAY.
WORD_BUF	DB	5,'     ',0	;FOR WORD INPUT TO REGISTER.
RBREAK_BUF	DB	13,10,'@ : CPU '
RBREAK_CPU	DB	'  , ADDR '
RBREAK_SEG	DB	'    :'
RBREAK_OFF	DB	'     | PASS COUNT: '
RBREAK_PASS	DB	'    , TASK: '
RBREAK_TASK	DB	'    ',0
DISP_BUF	DB	13,10
ADDR_BUF	DB	'    :      '
SHOW_BUF	DB	48 DUP (' '),0
REGS_BUF	DB	13,10		;BUFFER FOR REGISTER DISPLAY:
		DB	'ES:     ',SPC,SPC,'DI:     ',SPC,SPC,'DX:     ',13,10
		DB	'CX:     ',SPC,SPC,'DS:     ',SPC,SPC,'SI:     ',13,10
		DB	'BP:     ',SPC,SPC,'BX:     ',SPC,SPC,'AX:     ',13,10
		DB	'IP:     ',SPC,SPC,'CS:     ',SPC,SPC,'FL:     ',13,10
		DB	'SS:     ',SPC,SPC,'SP:     ',13,10,0
;
SREP_BUF	DB	13,10,'STATE BYTE: '
STATE_BYTE	DB	'  , CURRENT PRIORITY: '
PRI_BYTE	DB	'  .',0
;
RTCD_BUF	DB	13,10,'TASK ENTRY POINT: '
RTCD_EPOINT	DB	'    :    , TCD ADDRESS= '
RTCD_ADDR	DB	'    :    ,',13,10,'DATA SEGMENT= '
RTCD_DS		DB	'    , EXTRA SEGMENT= '
RTCD_ES		DB	'    , STACK BASE= '
RTCD_SS		DB	'    :    ',0
;
HALT_SDB	DB	019H,3		;HALT-TASK SDB:
HALT_ID		DW	?
MIGRATE_SDB	DB	019H,2		;MIGRATE-CPU SDB:
MIGRATE_CPU	DB	?
GTCB_SDB	DB	019H,5		;GET-TCB-ADDRESS SDB:
GTCB_ID		DW	?
		DW	GTCB_ADDR,0
GTCB_ADDR	DD	?
CONT_SDB	DB	019H,4		;CONTINUE-TASK SDB:
CONT_ID		DW	?		;TASK ID.
BASE_IP		DW	0		;BASE FOR CURRENT MODULE.
;
INIT_LEN	EQU	$-INIT_BLOCK	;LENGTH OF INITIALIZATION PATTERN.
DEB_INIT	ENDS
;
;	RAM DATA SEGMENT:
;		OBVIOUSLY THIS WILL NEED TO BE REPLICATED IN THE APPROPRIATE
;	MANNER FOR MULTI-DEBUGGER SYSTEMS.
;
DEB_DATA	SEGMENT	PARA PUBLIC
		DB	INIT_LEN DUP (?)	;INITIALIZABLE REGION
		DW	100 DUP (?)		;STACK REGION
XDB_STACK	LABEL	WORD
DEB_DATA	ENDS
PAGE
;
;	** START OF EXECUTABLE CODE: **
;		ALL ENTRY TO THE DEBUGGER BEGINS HERE. THAT INCLUDES STARTS
;	OF THE DEBUGGER DUE TO EITHER ESCAPE KEY DEPRESSIONS OR BREAKPOINT
;	EXECUTIONS. AT EACH ENTRY TO THIS POINT, IT IS ASSUMED THAT THE DS
;	REGISTER CONTAINS THE PARAGRAPH ADDRESS OF THE APPROPRIATE SEGMENT
;	OF DATA; THAT VALUE, WHATEVER IT IS, IS TRUSTED IMPLICITLY AND NOT
;	CHANGED.
;
DEB_CODE	SEGMENT	PARA PUBLIC
;
XDB_ENTRY	PROC	FAR
		PUSH	AX			;SAVE DEVICE/UNIT #
		INT	LOW X_SVC_VECTOR+1	;ISSUE SEF SVC TO DETERMINE
		DD	SEF_SDB			;WHETHER WE'VE YET INITIALIZED:
		JNC	XDB0			;YES, WE HAVE.
		PUSH	DS			;NO:
		POP	ES			;MOVE DS VALUE TO ES,
		MOV	AX,DEB_INIT		;PREPARE TO MOVE IMAGE BLOCK
		MOV	DS,AX			;TO REAL DATA SEGMENT.
		MOV	SI,OFFSET INIT_BLOCK	;ADDRESS THE INIT. BLOCK.
		XOR	DI,DI			;DATA SEGMENT PARA-ALIGNED.
		MOV	CX,INIT_LEN
	REP	MOVSB				;INITIALIZE DATA SEGMENT.
		PUSH	ES
		POP	DS			;RESTORE DS.
;
;	INSERT THE VALUE OF THE DATA SEGMENT INTO ALL APPLICABLE LOCATIONS.
;
		MOV	WORD PTR QCOPY_SDB+3,DS
		MOV	WORD PTR TOUT_SDB+9,DS
		MOV	WORD PTR TOUT_SDB+13,DS
		MOV	IN_SEG,DS
		MOV	WORD PTR TIN_SDB+9,DS
		MOV	WORD PTR TIN_SDB+13,DS
		MOV	WORD PTR GTCB_SDB+6,DS
;
;	GET THE CONTENTS OF THE STARTUP BLOCK AND DETERMINE THE CAUSE OF THIS
;	STARTUP. IF DUE TO AN ESC DEPRESSION AT THE CONSOLE, PROCEED. IF DUE
;	TO A BREAKPOINT EXECUTION, CHECK THE BREAKPOINT'S PASS COUNT AND TASK
;	RESTRICTION. IF THE APPROPRIATE TASK EXECUTED THE BREAKPOINT, AND IF
;	THE PASS COUNT DECREMENTS TO 0, PROCEED. IF EITHER CONDITION IS NOT
;	MET, UNBLOCK THE AFFECTED TASK AND TERMINATE THE DEBUGGER.
;
XDB0:
		MOV	BP,OFFSET QCOPY_SDB
		CALL	DS_SVC			;COPY THE STARTUP PARAMETERS,
		CMP	QBREAK_ID,0		;BREAKPOINT?
		JNE	XDB1			;YES.
		POP	AX			;NO:
		MOV	OUT_DEV,AX		;PUT DEV/UNIT INTO THE
		MOV	IN_DEV,AX		;PLACES WHERE WE NEED IT.
		JMP	XDB2
XDB1:						;BREAKPOINT:
		POP	AX			;CLEAN THE STACK,
		MOV	AL,QBREAK_ID		;GET BREAKPOINT ID.
		CMP	AL,NUM_BREAKS		;ONE OF THE TWO SPECIALS?
		JAE	XDB11			;YES, NO MULTIPASS OR RESTRICT.
		MOV	BL,BREAK_SIZE		;FIND ENTRY IN BREAK_TABLE...
		MUL	BL
		MOV	BX,AX
		ADD	BX,OFFSET BREAK_TABLE	;...IN STANDARD FASHION.
		CMP	WORD PTR [BX+BREAK_TASK],-1	;RESTRICTED?
		JE	XDB9			;NO: BYPASS TASK MATCHUP.
		MOV	AX,QBREAK_TASK		;YES:
		CMP	AX,[BX+BREAK_TASK]	;TASKS MATCH?
		JE	XDB9			;YES: CHECK PASS COUNT.
		JMP	XDB6			;NO: UNBLOCK TASK AND DIE.
XDB9:
		DEC	WORD PTR [BX+BREAK_PASS]	;DECREMENT PASS COUNT
		JE	XDB10			;READY TO TRIGGER!
		JMP	XDB6			;NOT READY: SIMPLY EXIT.
XDB10:
		MOV	WORD PTR [BX+BREAK_PASS],1	;READY: SET PASS COUNT
						;TO 1 FOR BREAKPOINT REUSE.
;
;	KILL SOFT BREAKPOINTS $10 AND $11, JUST IN CASE THEY WERE SET BY THE
;	PREVIOUS USE OF THE GO COMMAND.
;
XDB11:
		MOV	KBREAK_ID,16		;FIRST KILL $10:
		MOV	AL,CURR_CPU
		MOV	KBREAK_CPU,AL
		MOV	BP,OFFSET KBREAK_SDB
		CALL	DS_SVC			;THUS.
		MOV	KBREAK_ID,17		;NOW KILL $11:
		CALL	DS_SVC			;THUS: PREPARE ANNOUNCEMENT.
;
;	THIS BREAKPOINT MAY BE TRIGGERED: FORMAT ANNOUNCEMENT AND PROCEED.
;
		MOV	AL,QBREAK_ID		;GET BREAKPOINT NUMBER,
		CMP	AL,NUM_BREAKS		;ONE OF THE TWO SPECIALS?
		JB	XDB12			;NO, PROCEED NORMALLY.
		MOV	QBREAK_BUF,DOLLAR_SIGN	;YES: ALWAYS SOFT,
		ADD	AL,048H			;$10 --> "$X", $11 --> "$Y"
		MOV	QBREAK_BUF+1,AL		;LAY DOWN INDICATOR, BYPASS
		JMP	XDB5			;CLEARING THE USE BYTE.
XDB12:
		MOV	DI,OFFSET QBREAK_BUF	;PUT ITS ASCII FORM INTO THE
		CALL	BYT_2_HEX		;ANNOUNCEMENT,
		CMP	QBREAK_FLAGS,HARD_POINT	;IS IT A HARD BREAKPOINT?
		MOV	QBREAK_BUF,AT_SIGN	;ASSUME SO...
		JE	XDB5			;YES.
		MOV	QBREAK_BUF,DOLLAR_SIGN	;NO: FORMAT SOFT INDICATOR,
		MOV	AL,QBREAK_ID		;AND CLEAR THE BREAKPOINT'S
		MOV	BL,BREAK_SIZE		;ENTRY IN THIS DEBUGGER'S
		MUL	BL			;LOCAL TABLE OF BREAKPOINTS.
		MOV	BX,AX			;TO DO THAT, WE MAKE AN INDEX
		ADD	BX,OFFSET BREAK_TABLE	;INTO THE BREAK TABLE AND ZAP
		MOV	BYTE PTR [BX+BREAK_USE],0	;THE "USE" BYTE.
XDB5:
		MOV	AX,QBREAK_TASK		;FOLLOW THAT WITH THE TASK'S
		MOV	DI,OFFSET QBREAK_BUF+10	;ID NUMBER,
		CALL	BIN_2_HEX
		MOV	SI,OFFSET QBREAK_BUF	;THEN ISSUE THE MESSAGE TO THE
		CALL	TOUT_DS			;CONSOLE.
		JMP	SHORT XDB3
XDB2:						;ESCAPE KEY STRUCK:
		MOV	SI,OFFSET START_MSG	;ISSUE THE STARTUP MESSAGE
		CALL	TOUT_CS			;TO THE CONSOLE.
XDB3:
		MOV	SI,OFFSET PROMPT_MSG	;ISSUE THE COMMAND PROMPT
		CALL	TOUT_CS			;TO THE CONSOLE.
		MOV	SI,OFFSET IN_LINE	;THEN TURN AROUND AND
		CALL	TIN			;READ RESPONSE...
		CMP	INPUT_CT,1		;ANY?
		JBE	XDB3			;NOT SO'S YOU'D NOTICE.
		MOV	SI,OFFSET IN_LINE+1	;YES:
		CALL	INTERP_LINE		;INTERPRET THE COMMAND.
		TEST	AX,AX			;ERROR??
		JE	XDB3			;NO, GO FOR ANOTHER.
		CMP	AX,E_TERM		;TERMINATION DIRECTED??
		JE	XDB4			;YES: TERMINATE THE RUN.
		DEC	AX			;NO: MAKE AN INDEX INTO THE
		SHL	AX,1			;TABLE OF ERROR MESSAGES,
		MOV	SI,AX
		MOV	SI,E_MSGS[SI]		;GET ADDRESS OF ERROR MESSAGE,
		CALL	TOUT_CS			;ISSUE IT,
		JMP	SHORT XDB3		;AND TRY AGAIN.
XDB6:
		MOV	AX,QBREAK_TASK		;GET AFFECTED TASK'S ID,
		MOV	CONT_ID,AX		;PUT INTO CONTINUE SDB,
		MOV	BP,OFFSET CONT_SDB
		CALL	DS_SVC			;UNBLOCK THE AFFECTED TASK.
XDB4:
		INT	LOW X_SVC_VECTOR+1
		DD	ENDT_SDB		;TERMINATE DEBUGGER.
XDB_ENTRY	ENDP
;
;	INTERPRET THE LEADING KEYWORD ON THE LATEST COMMAND LINE:
;
;		THIS ROUTINE PULLS THE FIRST ATOM OFF THE INPUT LINE AND 
;	ATTEMPTS TO MATCH IT AGAINST AN ENTRY IN THE KEYWORD TABLE. IF SUC-
;	CESSFUL, IT CALLS THE APPROPRIATE ROUTINE TO HANDLE THE COMMAND. IF
;	NOT, A SYNTAX ERROR IS DECLARED.
;
INTERP_LINE	PROC	NEAR
		CALL	GET_ATOM		;GET THE 1ST ATOM...
		CMP	AX,KEY_MAX		;REAL ATOM DETECTED?
		JA	IL0			;NO.
		TEST	AX,AX			;YES: NULL?
		JE	IL1			;YES, NO COMMAND ENTERED.
		DEC	AX			;NO: MAKE INDEX INTO THE
		SHL	AX,1			;CALL VECTOR,
		MOV	DI,AX
		CALL	WORD PTR CS:JUMP_TABLE[DI]	;GO TO HANDLER,
		JMP	SHORT IL1		;RETURN STATUS.
IL0:
		MOV	AX,E_SYN		;SYNTAX ERROR.
IL1:
		RET
INTERP_LINE	ENDP
PAGE
;******************************************************************************
;
;		** ACTION ROUTINES CALLED BY INTERP_LINE **
;
;	ALL OF THE FOLLOWING ROUTINES ARE RESPONSES TO THE FIRST ATOM ON
;	A DEBUG COMMAND LINE, WHICH MUST BE A KEYWORD IN THE KEYWRD_TABLE
;	TO BE VALID. THEY ALL POSSESS THE SAME ENTRY/EXIT PROTOCOL:
;
;	INPUT:	SI= POINTER TO BALANCE OF COMMAND LINE,
;		DI= 2*(KEYWORD INDEX-1), SINCE KEYWORD INDICES START AT 1.
;	OUTPUT:	AX= COMPLETION STATUS:
;		    0: SUCCESSFUL COMPLETION.
;		    ALL OTHER VALUES INDICATE A FAILURE OR TERMINATION.
;
;******************************************************************************
;
;	KEYWORD WAS A REGISTER NAME OTHER THAN SP OR SS...
;
;		RETRIEVE THE ASSOCIATED REGISTER VALUE AND PRINT IT.
;
R_REG		PROC	NEAR
		CMP	QBREAK_ID,0		;BREAKPOINT ENTERED US?
		JNZ	RAX0			;YES, OK.
		MOV	AX,E_NOGO		;NO: ILLEGAL REQUEST.
		RET
RAX0:
		LES	BP,TCB_ADDR		;GET ADDRESS OF TCB,
		LES	BP,DWORD PTR ES:[BP+TCB_SP] ;GET TASK'S STACK ADDRESS,
		MOV	AX,ES:[BP][DI]		;GET REGISTER VALUE.
		PUSH	DI			;SAVE REGISTER "INDEX" AND
		MOV	DI,OFFSET OUTW_BUF+2	;FORMAT THE VALUE OF THE
		MOV	SI,DI
		CALL	BIN_2_HEX		;REGISTER FOR PRINTING.
		MOV	SI,OFFSET OUTW_BUF
		CALL	TOUT_DS			;OUTPUT THE CURRENT VALUE.
		MOV	SI,OFFSET WORD_BUF
		CALL	TIN			;READ RESPONSE...
		INC	SI			;FORWARD TO INPUT BUFFER AND
		CALL	GET_ATOM		;GET ATOM AT START OF LINE,
		TEST	AX,AX			;ANY INPUT?
		JNE	RAX2			;YES.
		POP	DI			;NO:
		RET				;JUST FORGET THE WHOLE THING.
RAX2:
		CMP	AX,NUMBER		;HEXADECIMAL INPUT?
		JE	RAX3			;YES.
		POP	DI			;NO, CLEAN THE STACK
		MOV	AX,E_SYN		;ANOTHER SYNTAX ERROR.
		RET
RAX3:
		MOV	AX,DI			;PUT THE NUMBER IN AX REGISTER.
		LES	BP,TCB_ADDR		;ONCE AGAIN, WE'RE HEADED FOR
		LES	BP,DWORD PTR ES:[BP+TCB_SP]	;THE REGISTER FILE,
		POP	DI			;THIS TIME TO PUT THE NEW
		MOV	ES:[BP][DI],AX		;REGISTER VALUE THERE.
		XOR	AX,AX			;GOOD.
		RET
R_REG		ENDP
;
;	KEYWORD WAS SP OR SS...
;
;		THIS IS DISALLOWED, BECAUSE OF MTOS'S REQUIREMENT THAT A TASK'S
;	STACK HOLD ITS REGISTER FILE WHILE IT'S SUSPENDED.
;
R_SPSS		PROC	NEAR
		MOV	AX,E_NOGO
		RET
R_SPSS		ENDP
;
;	KEYWORD WAS 'STACK'...
;
;		THIS IS ALLOWED BECAUSE IT ONLY PRINTS OUT THE SS ASND SP 
;	REGISTER VALUES; IT DOES NOT PERMIT MODIFICATION.
;
R_STACK		PROC	NEAR
		CMP	QBREAK_ID,0		;BREAKPOINT EXECUTED?
		JNE	RS0			;YES.
		MOV	AX,E_NOGO		;NO: ILLEGAL REQUEST.
		RET
RS0:
		LES	BP,TCB_ADDR		;GET TCB ADDRESS,
		LES	BP,DWORD PTR ES:[BP+TCB_SP]	;GET STACK POINTER,
		ADD	BP,24			;ADVANCE PAST MTOS FRAME,
		MOV	AX,ES			;FORMAT IT IN ASCII INTO THE
		MOV	DI,OFFSET ADDR_BUF	;DISPLAY BUFFER FOR THESE
		CALL	BIN_2_HEX		;THINGS.
		MOV	BYTE PTR [DI],ASCII_COLON	;ADDRESS DIVIDER.
		INC	DI			;ADVANCE TO OFFSET REGION.
		MOV	AX,BP			;PUT OFFSET INTO DISPLAY
		CALL	BIN_2_HEX		;BUFFER AFTER SEGMENT.
		MOV	BYTE PTR [DI],0		;LAY DOWN A TERMINATOR.
		MOV	SI,OFFSET DISP_BUF	;NOW REPORT THE STACK ADDR
		CALL	TOUT_DS			;TO THE CONSOLE.
		XOR	AX,AX
		RET
R_STACK		ENDP
;
;	SET A BREAKPOINT:
;
;		VALIDATE THE OPERATION AND IF SUCCESSFUL, INSTALL THE BREAK-
;	POINT VIA THE SYSTEM SERVICE FOR IT. NOTE THAT, WHEN SET, ALL BREAK-
;	POINTS ARE TASK-UNRESTRICTED AND SET FOR A PASS COUNT OF 1.
;
R_SET		PROC	NEAR
		CALL	GET_ATOM		;GET BREAK ID.
		CMP	AX,HBRKPOINT		;HARD BREAKPOINT ID?
		MOV	SBREAK_FLAGS,HARD_POINT
		JE	SET0			;YES.
		CMP	AX,SBRKPOINT		;NO: SOFT BREAKPOINT ID?
		MOV	SBREAK_FLAGS,SOFT_POINT
		JE	SET0			;YES.
SET9:
		MOV	AX,E_SYN		;NO.
SET10:
		RET
SET0:
		MOV	AX,DI			;ID TO AL,
		MOV	SBREAK_ID,AL		;THENCE TO SDB,
		CALL	GET_ADDR		;GET ADDRESS...
		JC	SET9			;NO GOOD.
		MOV	SBREAK_OFF,DI		;PUT ADDRESS OF BREAKPOINT
		MOV	SBREAK_SEG,ES		;INTO SDB,
		MOV	AL,CURR_CPU		;THE CPU NUMBER TOO,
		MOV	SBREAK_CPU,AL
		MOV	BP,OFFSET SBREAK_SDB
		CALL	DS_SVC			;SET THE BREAKPOINT...
		JNZ	SET1			;DID IT!
		MOV	AX,E_IDUSED		;NO: ALREADY SET?
		JS	SET10			;IF SO, SIGN BIT WILL BE ON.
		MOV	AX,E_BKNOW		;NO, THE ADDRESS IS OCCUPIED BY
		RET				;A BREAKPOINT INSTRUCTION!
SET1:
		MOV	AL,SBREAK_ID		;GET BREAKPOINT ID,
		MOV	BL,BREAK_SIZE		;SIZE OF BREAK TABLE ENTRY,
		MUL	BL			;INDEX TO BREAK TABLE,
		MOV	BX,AX			;USE BX FOR BASE,
		ADD	BX,OFFSET BREAK_TABLE	;ADDRESS BREAK TABLE,
		MOV	BYTE PTR [BX+BREAK_USE],1	;SET ID IN USE.
		MOV	AL,SBREAK_CPU		;FILL OUT THE ENTRY:
		MOV	[BX+BREAK_CPU],AL	;CPU NUMBER,
		MOV	AL,SBREAK_FLAGS
		MOV	BYTE PTR [BX+BREAK_FLAGS],AL	;FLAGS,
		LES	DI,SBREAK_ADDR		;ADDRESS,
		MOV	[BX+BREAK_OFF],DI
		MOV	[BX+BREAK_SEG],ES
		MOV	WORD PTR [BX+BREAK_PASS],1	;ONE PASS,
		MOV	WORD PTR [BX+BREAK_TASK],-1	;UNRESTRICTED.
		XOR	AX,AX			;GOOD.
		RET
R_SET		ENDP
;
;	KILL A BREAKPOINT:
;
;		VALIDATE THE REQUEST, AND IF SUCCESSFUL ASK THE SYSTEM TO
;	KILL THE BREAKPOINT.
;
R_KILL		PROC	NEAR
		CALL	GET_ATOM		;GET BREAKPOINT ID:
		CMP	AX,HBRKPOINT		;HARD POINT ID?
		MOV	CL,HARD_POINT		;ASSUME SO...
		JE	KILL0			;YES.
		CMP	AX,SBRKPOINT		;NO: SOFT POINT ID?
		MOV	CL,SOFT_POINT		;ASSUME SO...
		JE	KILL0			;YES.
		MOV	AX,E_SYN		;NO:
		RET				;SYNTAX ERROR.
KILL0:
		MOV	AX,DI			;ID TO AX,
		MOV	KBREAK_ID,AL		;AND TO KILL SDB.
		MOV	BL,BREAK_SIZE		;FIND BREAK_TABLE ENTRY...
		MUL	BL
		MOV	BX,AX
		ADD	BX,OFFSET BREAK_TABLE	;...IN STANDARD FASHION.
		CMP	[BX+BREAK_FLAGS],CL	;FLAGS MATCH UP?
		JNE	KILL2			;NO.
		CMP	BYTE PTR [BX+BREAK_USE],0	;IN USE?
		JE	KILL2			;NO.
		MOV	BYTE PTR [BX+BREAK_USE],0	;YES: CLEAR USE FLAG.
		MOV	AL,CURR_CPU		;BRING THE CPU NUMBER TOO,
		MOV	KBREAK_CPU,AL
		MOV	BP,OFFSET KBREAK_SDB
		CALL	DS_SVC			;ATTEMPT THE KILL...
		JNZ	KILL1			;SUCCESS!
KILL2:
		MOV	AX,E_NOID		;FAILURE:
		RET				;RETURN ERROR.
KILL1:
		CMP	QBREAK_ID,0		;HERE FROM A BREAKPOINT?
		JE	KILL3			;NO.
		MOV	AL,KBREAK_ID		;YES:
		CMP	AL,QBREAK_ID		;THE NEWLY KILLED ONE?
		JNE	KILL3			;NO.
		LES	BP,TCB_ADDR		;YES: QUENCH THE T-BIT IN THE
		LES	BP,DWORD PTR ES:[BP+TCB_SP]	;STACK OF THE AFFECTED
		AND	WORD PTR ES:[BP+STK_FL],NOT TRAP_ON	;TASK.
KILL3:
		XOR	AX,AX
		RET
R_KILL		ENDP
;
;	FURTHER QUALIFY A STANDING HARD BREAKPOINT:
;
;		THIS FUNCTION ALLOWS THE USER TO ASSOCIATE A RESTRICTING TASK
;	AND A PASS COUNT WITH ANY HARD BREAKPOINT.
;
R_QUALIFY	PROC	NEAR
		CALL	GET_ATOM		;GET BREAKPOINT ID.
		CMP	AX,HBRKPOINT		;BETTER BE HARD...
		JE	RQL0			;IT IS.
RQL9:
		MOV	AX,E_SYN		;SYNTAX ERROR.
		RET
RQL0:
		MOV	AX,DI			;MAKE BREAKPOINT ID INTO...
		MOV	BL,BREAK_SIZE
		MUL	BL
		MOV	BX,AX
		ADD	BX,OFFSET BREAK_TABLE	;BREAK TABLE ENTR ADDRESS.
		CMP	BYTE PTR [BX+BREAK_USE],0	;IN USE?
		JE	RQL8			;NO.
		CMP	BYTE PTR [BX+BREAK_FLAGS],HARD_POINT	;HARD POINT?
		JE	RQL7			;YES, PROCEED.
RQL8:
		MOV	AX,E_NOGO		;ILLEGAL REQUEST.
		RET
RQL7:
		CALL	GET_ATOM		;GET PASS COUNT...
		CMP	AX,NUMBER		;?
		JNE	RQL9			;SYNTAX ERROR.
		MOV	[BX+BREAK_PASS],DI	;PUT IT INTO BREAK TABLE ENTRY
		CALL	GET_ATOM		;OPTIONAL TASK SPECIFIED?
		TEST	AX,AX			;?
		JE	RQL2			;NO, END OF INPUT.
		CMP	AX,NUMBER		;YES: NUMERIC INPUT?
		JNE	RQL9			;NO, SYNTAX ERROR.
		MOV	[BX+BREAK_TASK],DI	;YES: INTO BREAK TABLE ENTRY.
RQL2:
		XOR	AX,AX
		RET
R_QUALIFY	ENDP
;
;	DISPLAY CURRENT BREAKPOINTS:
;
;		WE USE THE TABLE KEPT LOCALLY, RATHER THAN REACHING INTO THE
;	LIST MAINTAINED BY MTOS, BECAUSE MTOS'S LIST MAY CONTAIN BREAKPOINTS
;	SET BY OTHER DEBUGGER TASKS!
;
R_BREAKS	PROC	NEAR
		MOV	AX,1
		MOV	BX,OFFSET BREAK_TABLE+BREAK_SIZE
RB0:
		CMP	BYTE PTR [BX+BREAK_USE],0	;IN USE?
		JE	RB1			;NO.
		PUSH	AX			;YES: SAVE THE ID,
		MOV	DI,OFFSET RBREAK_BUF+2	;AND FORMAT IT INTO THE
		CALL	BYT_2_HEX		;MESSAGE.
		MOV	BYTE PTR RBREAK_BUF+2,AT_SIGN	;ASSUME HARD POINT,
		CMP	BYTE PTR [BX+BREAK_FLAGS],HARD_POINT	;IS IT?
		JE	RB2			;YES.
		MOV	BYTE PTR RBREAK_BUF+2,DOLLAR_SIGN	;NO.
RB2:
		MOV	DI,OFFSET RBREAK_CPU	;ALSO PUT OUT THE CPU
		MOV	AL,[BX+BREAK_CPU]	;FOR THE BREAKPOINT,
		CALL	BYT_2_HEX
		MOV	AX,[BX+BREAK_SEG]	;AND THE ADDRESS AT WHICH IT
		MOV	DI,OFFSET RBREAK_SEG	;RESIDES.
		CALL	BIN_2_HEX
		MOV	AX,[BX+BREAK_OFF]
		MOV	DI,OFFSET RBREAK_OFF
		CALL	BIN_2_HEX		;SEGMENT:OFFSET.
		MOV	AX,[BX+BREAK_PASS]	;GET PASS COUNT,
		MOV	DI,OFFSET RBREAK_PASS
		CALL	BIN_2_HEX		;FORMAT IT.
		MOV	AX,[BX+BREAK_TASK]	;GET RESTRICTION TASK,
		MOV	DI,OFFSET RBREAK_TASK
		CMP	AX,0FFFFH		;RESTRICTED TO ONE TASK?
		JNE	RB3			;YES.
		MOV	AX,WORD PTR ANY_STRING	;NO: SAY "ANY."
		MOV	[DI],AX
		MOV	AX,WORD PTR ANY_STRING+2
		MOV	[DI+2],AX
		JMP	SHORT RB4
RB3:
		CALL	BIN_2_HEX		;FORMAT IT.
RB4:
		MOV	SI,OFFSET RBREAK_BUF
		CALL	TOUT_DS			;ISSUE 1 LINE OF REPORT.
		POP	AX			;RETRIEVE ID.
RB1:
		ADD	BX,BREAK_SIZE		;ADVANCE THRU TABLE,
		INC	AX			;INCREMENT BREAK ID,
		CMP	AX,NUM_BREAKS		;TOPPED OUT?
		JNE	RB0			;NOT YET.
		XOR	AX,AX			;YES.
		RET
R_BREAKS	ENDP
;
;	OPEN MEMORY FOR BYTE INSPECTION AND MODIFICATION.
;
R_BYTE		PROC	NEAR
		CALL	GET_ADDR		;GET MEMORY ADDRESS.
		JNC	RBY0			;GOT IT.
BERR0:						;NO:
		MOV	AX,E_SYN		;SYNTAX ERROR.
		RET
RBY0:
		MOV	BX,DI			;ADDRESS IN ES:BX.
RBY2:
		MOV	AX,ES			;FORMAT THE ADDRESS INTO
		MOV	DI,OFFSET ADDR_BUF	;THE DISPLAY BUFFER.
		CALL	BIN_2_HEX
		MOV	BYTE PTR [DI],ASCII_COLON
		INC	DI
		MOV	AX,BX			;ALSO FORMAT IN THE OFFSET
		CALL	BIN_2_HEX		;IN LIKE MANNER.
		MOV	BYTE PTR [DI],ASCII_SPACE
		INC	DI
		MOV	AL,ES:[BX]		;GET MEMORY BYTE,
		CALL	BYT_2_HEX		;CONVERT TO HEXADECIMAL,
		MOV	BYTE PTR [DI],ASCII_SPACE
		INC	DI
		MOV	BYTE PTR [DI],0		;DROP PADDING AFTERWARD,
		MOV	SI,OFFSET DISP_BUF	;AND DISPLAY THE BYTE.
		CALL	TOUT_DS
		MOV	SI,OFFSET WORD_BUF	;NOW ACCEPT POTENTIAL REPLACE-
		CALL	TIN			;MENT VALUE.
		CMP	INPUT_CT,3		;RIGHT SIZE?
		JA	BERR0			;NO.
		INC	SI			;YES:
		CALL	GET_ATOM		;CONVERT TO BINARY.
		JC	BERR1			;KILL THIS RUN.
		TEST	AX,AX			;NO INPUT?
		JE	RBY1			;CORRECT: FORWARD THRU MEMORY.
		CMP	AX,NUMBER		;NUMERIC INPUT?
		JNE	BERR1			;NO: KILL RUN.
		MOV	AX,DI			;YES:
		MOV	ES:[BX],AL		;PUT IT INTO MEMORY.
RBY1:
		INC	BX			;ADVANCE MEMORY POINTER,
		JMP	RBY2			;GO AROUND.
BERR1:
		XOR	AX,AX
		RET
R_BYTE		ENDP
;
;	OPEN MEMORY FOR WORD INSPECTION AND MODIFICATION.
;
R_WORD		PROC	NEAR
		CALL	GET_ADDR		;GET MEMORY ADDRESS.
		JNC	RW0			;GOT IT.
WERR0:						;NO:
		MOV	AX,E_SYN		;SYNTAX ERROR.
		RET
RW0:
		MOV	BX,DI			;ADDRESS IN ES:BX.
RW2:
		MOV	AX,ES			;FORMAT THE ADDRESS INTO
		MOV	DI,OFFSET ADDR_BUF	;THE DISPLAY BUFFER.
		CALL	BIN_2_HEX
		MOV	BYTE PTR [DI],ASCII_COLON
		INC	DI
		MOV	AX,BX			;ALSO FORMAT IN THE OFFSET
		CALL	BIN_2_HEX		;IN LIKE MANNER.
		MOV	BYTE PTR [DI],ASCII_SPACE
		INC	DI
		MOV	AX,ES:[BX]		;GET MEMORY WORD,
		CALL	BIN_2_HEX		;CONVERT TO HEXADECIMAL,
		MOV	BYTE PTR [DI],ASCII_SPACE
		INC	DI
		MOV	BYTE PTR [DI],0		;DROP PADDING AFTERWARD,
		MOV	SI,OFFSET DISP_BUF	;AND DISPLAY THE WORD.
		CALL	TOUT_DS
		MOV	SI,OFFSET WORD_BUF	;NOW ACCEPT POTENTIAL REPLACE-
		CALL	TIN			;MENT VALUE.
		INC	SI			;YES:
		CALL	GET_ATOM		;CONVERT TO BINARY.
		JC	WERR1			;KILL THIS RUN.
		TEST	AX,AX			;NO INPUT?
		JE	RW1			;CORRECT: FORWARD THRU MEMORY.
		CMP	AX,NUMBER		;NUMERIC INPUT?
		JNE	WERR1			;NO: KILL RUN.
		MOV	ES:[BX],DI		;PUT IT INTO MEMORY.
RW1:
		INC	BX			;ADVANCE MEMORY POINTER,
		INC	BX			;TWICE FOR WORDS.
		JMP	RW2			;GO AROUND.
WERR1:
		XOR	AX,AX
		RET
R_WORD		ENDP
;
;	DISPLAY THE CONTENTS OF MEMORY.
;
;		PRESUMED TO BE OPERATING ON THE NECESSARY CPU.
;
R_DISPLAY	PROC	NEAR
		CALL	GET_ADDR		;GET MEMORY ADDRESS.
		JNC	DS0			;GOT IT.
DSERR0:						;NO:
		MOV	AX,E_SYN		;SYNTAX ERROR.
		RET
DS0:
		MOV	BX,DI			;ADDRESS TO ES:BX.
		CALL	GET_ATOM		;GET LENGTH TO SHOW...
		JC	DSERR0			;ERROR.
		CMP	AX,NUMBER		;NUMERIC INPUT?
		JNE	DSERR0			;NO!
		MOV	DX,DI			;YES: KEEP IN DX.
DS1:
		CMP	DX,16			;16 BYTES OR MORE?
		MOV	CX,DX			;PREPARE FOR FEWER...
		JB	DS2			;FEWER.
		MOV	CX,16			;AT LEAST 16.
DS2:
		JCXZ	DSERR0			;ZERO IS INVALID.
		PUSH	BX			;SAVE DISPLAY ADDRESS.
		SUB	DX,CX			;REDUCE DISPLAY COUNT.
		MOV	DI,OFFSET SHOW_BUF	;ADDRESS DISPLAY BUFFER.
DS3:
		MOV	AL,ES:[BX]		;GET A BYTE,
		CALL	BYT_2_HEX		;FORMAT IT.
		MOV	BYTE PTR [DI],ASCII_SPACE
		INC	DI
		INC	BX			;ADVANCE MEMORY POINTER,
		LOOP	DS3			;AND GO AROUND 'TILL FINISHED.
		MOV	BYTE PTR [DI],0		;FINISHED: TERMINATOR.
		MOV	DI,OFFSET ADDR_BUF	;ADDRESS BUFFER POINTER,
		MOV	AX,ES			;FORMAT SEGMENT ADRESS INTO
		CALL	BIN_2_HEX		;IT,
		POP	AX			;RETRIEVE START BX,
		PUSH	BX			;SAVE NEW BX,
		MOV	BYTE PTR [DI],ASCII_COLON	;ADDRESS DIVIDER.
		INC	DI
		CALL	BIN_2_HEX		;FORMAT OFFSET ADDRESS INTO
		MOV	BYTE PTR [DI],ASCII_SPACE	;BUFFER,
		INC	DI			;AND DROP TWO SPACES FOR
		MOV	BYTE PTR [DI],ASCII_SPACE	;PADDING.
		MOV	SI,OFFSET DISP_BUF
		CALL	TOUT_DS			;ISSUE 1 LINE OF REPORT.
		POP	BX			;GET RESUMPTION BX,
		TEST	DX,DX			;ANY MORE??
		JNE	DS1			;YES.
		XOR	AX,AX			;NO.
		RET
R_DISPLAY	ENDP
;
;	QUERY THE STATE OF A TASK.
;
R_QUERY		PROC	NEAR
		CALL	GET_ATOM		;GET TASK ID.
		JC	QERR0			;ERROR.
		CMP	AX,NUMBER		;NUMERIC INPUT?
		JE	Q0			;YES.
QERR0:						;NO:
		MOV	AX,E_SYN		;SYNTAX ERROR.
		RET
QERR1:
		MOV	AX,E_NOTASK		;NO SUCH TASK.
		RET
Q0:
		MOV	GTCB_ID,DI		;TASK ID TO GET-TCB SDB.
		MOV	BP,OFFSET GTCB_SDB
		CALL	DS_SVC			;REQUEST THE TCB ADDRESS...
		JZ	QERR1			;NO SUCH TASK!
		LES	BP,GTCB_ADDR		;GET TCB ADDR TO ES:SI
		MOV	BL,ES:[BP+TCB_STATE]	;GET STATE BYTE,
		AND	BL,3			;REMOVE IRRELEVANCIES,
		XOR	BH,BH
		SHL	BX,1			;DOUBLE IT,
		MOV	SI,STATE_MSGS[BX]	;THEN ISSUE THE
		CALL	TOUT_CS			;STATE DESCRIPTION.
		MOV	AL,ES:[BP+TCB_STATE]	;GET STATE BYTE AGAIN,
		MOV	DI,OFFSET STATE_BYTE	;AND FORMAT IT INTO THE SECOND
		CALL	BYT_2_HEX		;LINE OF THE REPORT.
		MOV	AL,ES:[BP+TCB_PRI]	;GET CURRENT PRIORITY,
		MOV	DI,OFFSET PRI_BYTE	;AND DO LIKEWISE.
		CALL	BYT_2_HEX
		MOV	SI,OFFSET SREP_BUF	;NOW ISSUE THAT LINE.
		CALL	TOUT_DS
		XOR	AX,AX
		RET
R_QUERY		ENDP
;
;	MIGRATE TO SPECIFIED CPU.
;
R_GOCPU		PROC	NEAR
		CALL	GET_ATOM		;GET CPU NUMBR.
		JC	GCERR0			;ERROR.
		CMP	AX,NUMBER		;NUMERIC INPUT?
		JE	GC0			;YES.
GCERR0:						;NO,
		MOV	AX,E_SYN		;SYNTAX ERROR.
		RET
GC0:
		MOV	AX,DI			;CPU ID TO THE
		MOV	MIGRATE_CPU,AL		;MIGRATION SDB.
		MOV	BP,OFFSET MIGRATE_SDB
		CALL	DS_SVC			;REQUEST THE MIGRATION...
		JNZ	GCOUT			;WE'RE THERE.
		MOV	AX,E_NOGO		;NO SUCH CPU!
		RET
GCOUT:
		MOV	AL,MIGRATE_CPU		;INSTALL THE NEW CPU NUMBER
		MOV	CURR_CPU,AL		;WHERE WE EXPECT IT.
		XOR	AX,AX
		RET
R_GOCPU		ENDP
;
;	HALT SPECIFIED TASK:
;
;		THE TASK SPECIFIED MUST BE IN THE READY STATE.
;
R_HALT		PROC	NEAR
		CALL	GET_ATOM		;GET TASK NUMBER.
		JC	HLTER0			;ERROR.
		CMP	AX,NUMBER		;NUMERIC INPUT?
		JE	HLT0			;YES.
HLTER0:						;NO:
		MOV	AX,E_SYN		;SYNTAX ERROR.
		RET
HLTER1:
		MOV	AX,E_NOTASK		;NO SUCH TASK.
		RET
HLTER2:
		MOV	AX,E_NOGO		;ILLEGAL REQUEST.
		RET
HLT0:
		MOV	HALT_ID,DI		;TASK ID TO HALT SDB.
		MOV	BP,OFFSET HALT_SDB
		CALL	DS_SVC			;ISSUE THE REQUEST...
		JS	HLTER1			;NO SUCH TASK.
		JZ	HLTER2			;TASK NOT READY.
		XOR	AX,AX
		RET
R_HALT		ENDP
;
;	CONTINUE SPECIFIED TASK:
;
;		THE TASK SPECIFIED MUST BE IN THE DEBUGGER-HALT STATE.
;
R_CONT		PROC	NEAR
		CALL	GET_ATOM		;GET TASK NUMBER.
		JC	CNTER0			;ERROR.
		CMP	AX,NUMBER		;NUMERIC INPUT?
		JE	CNT0			;YES.
CNTER0:						;NO:
		MOV	AX,E_SYN		;SYNTAX ERROR.
		RET
CNTER1:
		MOV	AX,E_NOTASK		;NO SUCH TASK.
		RET
CNTER2:
		MOV	AX,E_NOGO		;ILLEGAL REQUEST.
		RET
CNT0:
		MOV	GTCB_ID,DI		;GET THE TCB ADDRESS OF THE
		MOV	BP,OFFSET GTCB_SDB	;SPECIFIED TASK.
		CALL	DS_SVC
		JZ	CNTER1			;NO SUCH TASK!
		LES	BP,GTCB_ADDR		;NOW CHECK IT FOR THE NECESSARY
		CMP	BYTE PTR ES:[BP+TCB_STATE],BLOCK+STA_HLT	;STATE:
		JNE	CNTER2			;WRONG STATE!
		MOV	CONT_ID,DI		;GOOD: PUT ID TO CONTINUE SDB.
		MOV	BP,OFFSET CONT_SDB
		CALL	DS_SVC			;ISSUE THE REQUEST...
		JS	CNTER1			;NO SUCH TASK.
		JZ	CNTER2			;TASK NOT HALTED.
		XOR	AX,AX
		RET
R_CONT		ENDP
;
;	START A SPECIFIED TASK:
;
;		VALIDATE THE ID NUMBER AND IF SUCCESSFUL, ASK THE SYSTEM TO
;	START THE TASK.
;
R_START		PROC	NEAR
		CALL	GET_ATOM		;GET TASK ID.
		CMP	AX,NUMBER		;NUMERIC INPUT?
		JE	ST0			;YES.
ST9:
		MOV	AX,E_SYN		;NO: ISSUE SYNTAX ERROR.
		RET
ST0:
		MOV	START_ID,DI		;PUT ID INTO SDB,
		PUSH	SI			;SAVE INPUT POINTER,
		CALL	GET_ADDR		;ATTEMPT TO GET OPTIONAL
						;ADDRESS PARAMETER.
		POP	SI			;RETRIEVE INPUT POINTER.
		JNC	ST1			;GOT THE ADDRESS.
		CALL	GET_ATOM		;FAILED: TRY NUMERIC.
		JC	ST9			;SYNTAX ERROR.
		CMP	AX,NUMBER		;GOT ONE: NUMERIC?
		JE	ST3			;YES, USE IT.
		TEST	AX,AX			;NO: END-OF-INPUT?
		JE	ST2			;YES.
		JMP	SHORT ST9		;NO: SYNTAX ERROR.
ST3:
		MOV	START_P1,DI		;YES: PUT INTO SDB.
		CALL	GET_ATOM		;TRY AGAIN FOR 2ND PARAMETER.
		JC	ST9			;SYNTAX ERROR.
		CMP	AX,NUMBER		;GOT ONE: NUMERIC?
		JE	ST4			;YES: USE IT FOR 2ND PARAM.
		TEST	AX,AX			;NO: END OF INPUT?
		JE	ST1			;YES.
		JMP	SHORT ST9		;NO: SYNTAX ERROR.
ST4:
		MOV	START_P2,DI		;YES: INTO THE SDB WITH IT.
		JMP	SHORT ST2		;PERFORM THE START.
ST1:
		MOV	START_P1,DI		;ADDRESS OFFSET TO P1,
		MOV	START_P2,ES		;PARAGRAPH TO P2.
ST2:
		MOV	BP,OFFSET START_SDB
		CALL	DS_SVC			;ISSUE THE REQUEST...
		JNZ	ST5			;GOOD.
		JS	ST6			;NO SUCH TASK ID.
		MOV	AX,E_ALIVE		;TASK NOT DORMANT.
		RET
ST6:
		MOV	AX,E_NOTASK		;NO SUCH TASK
		RET
ST5:
		XOR	AX,AX			;GOOD.
		RET
R_START		ENDP
;
;	EXIT AFTER KILLING ALL BREAKPOINTS:
;
;		WE GO THROUGH OUR LIST OF BREAKPOINTS AND REQUEST EACH ONE'S
;	DELETION BY THE SYSTEM, AND THEN TERMINATE THE DEBUGGER.
;
R_EXITK		PROC	NEAR
		MOV	AX,1			;FIRST ID= 1.
		MOV	BX,OFFSET BREAK_TABLE+BREAK_SIZE
EK0:
		CMP	BYTE PTR [BX+BREAK_USE],0	;BREAKPOINT IN USE?
		JE	EK1			;NO.
		MOV	BYTE PTR [BX+BREAK_USE],0	;YES: CLEAR IT.
		MOV	KBREAK_ID,AL		;PUT IT INTO KILL COMMAND
		MOV	CL,[BX+BREAK_CPU]	;GET CPU NUMBER,
		MOV	KBREAK_CPU,CL		;TO SDB,
		MOV	BP,OFFSET KBREAK_SDB
		CALL	DS_SVC			;ISSUE KILL REQUEST.
EK1:
		ADD	BX,BREAK_SIZE		;FORWARD THRU TABLE,
		INC	AX			;TO NEXT ID,
		CMP	AX,NUM_BREAKS		;IF ANY...
		JNE	EK0			;THERE IS.
		MOV	AX,E_TERM		;THERE ISN'T.
		RET
R_EXITK		ENDP
;
;	FOR TERMINATION WITH BREAKPOINTS IN PLACE:
;
R_EXIT		PROC	NEAR
		MOV	AX,E_TERM
		RET
R_EXIT		ENDP
;
;	DISPLAY ALL THE TASK'S REGISTERS:
;
;		NOT MUCH TO SAY ABOUT IT, REALLY.
;
R_REGS		PROC	NEAR
		CMP	QBREAK_ID,0		;BREAKPOINT EXECUTED?
		JNE	REGS1			;YES.
		MOV	AX,E_NOGO		;NO: ILLEGAL REQUEST.
		RET
REGS1:
		MOV	DI,OFFSET REGS_BUF+6	;INITIALIZE FOR CONVERSION,
		MOV	CX,14			;14 REGISTERS.
		MOV	SI,ES_INDEX		;START FROM 'ES REGISTER.'
REGS0:
		PUSH	DI			;SAVE FORMATTING POINTER,
		MOV	DI,SI			;REGISTER INDEX TO DI,
		CALL	GET_REG			;GET REGISTER VALUE.
		MOV	AX,DI			;INTO FORMATTER REGISTER,
		POP	DI			;RESTORE FORMATTING POINTER.
		PUSH	DI			;SAVE BUFFER POINTER,
		CALL	BIN_2_HEX		;DO CONVERSION,
		POP	DI			;RETRIEVE BUFFER POINTER,
		ADD	DI,10			;ADVANCE TO NEXT ENTRY,
		INC	SI			;ADVANCE TO NEXT REGISTER.
		LOOP	REGS0
		MOV	SI,OFFSET REGS_BUF	;WHEN DONE, PUT THE REPORT OUT
		CALL	TOUT_DS			;AS YOU MIGHT EXPECT.
		XOR	AX,AX
		RET
R_REGS		ENDP
;
;	GO FROM LAST BREAKPOINT OCCURRENCE:
;
;		THIS INVOLVES UN-HALTING THE BREAKPOINTED TASK, PLUS SETTING
;		0, 1 OR 2 SOFT BREAKPOINTS AS SPECIFIED BY THE COMMAND.
;
R_GO		PROC	NEAR
		CMP	QBREAK_ID,0		;BREAKPOINT CAUSED ENTRY?
		JNE	GG0			;YES.
		MOV	AX,E_NOGO		;NO: CAN'T DO IT!
		RET
GG0:
		CALL	GET_ADDR		;GET 1ST SOFT BREAKPOINT...
		JC	GG10			;NONE.
		MOV	SBREAK_ID,16		;GOT IT: BECOMES $10.
		MOV	SBREAK_OFF,DI		;OFFSET  ADDRESS,
		MOV	SBREAK_SEG,ES		;SEGMENT ADDRESS,
		MOV	SBREAK_FLAGS,SOFT_POINT	;THESE ARE ALWAYS SOFT.
		MOV	AL,CURR_CPU
		MOV	SBREAK_CPU,AL		;CPU NUMBER,
		MOV	BP,OFFSET SBREAK_SDB
		CALL	DS_SVC			;SET $10.
		CALL	GET_ADDR		;NOW TRY 2ND SOFT BREAKPOINT...
		JC	GG10			;NONE.
		MOV	SBREAK_ID,17		;GOT IT: BECOMES $11.
		MOV	SBREAK_OFF,DI		;OFFSET  ADDRESS,
		MOV	SBREAK_SEG,ES		;SEGMENT ADDRESS.
		CALL	DS_SVC			;SET $11.
GG10:
		MOV	AX,QBREAK_TASK		;GET TASK ID,
		MOV	CONT_ID,AX		;PUT TO SDB,
		MOV	BP,OFFSET CONT_SDB
		CALL	DS_SVC			;ISSUE CONTINUE REQUEST,
		MOV	AX,E_TERM		;AND KILL SELF.
		RET
R_GO		ENDP
;
;	SET INTRA_MODULE BASE FOR BREAKPOINT AND ADDRESS RELOCATION.
;
;		THE BASE SPECIFIED BY THIS COMMAND IS USED TO RELOCATE ADDRESS
;	REFERENCES WHICH ARE SPECIFIED AS ABSOLUTE NUMERIC VALUES. THIS RELOCA-
;	TION ALWAYS PRESUMES THE CS REGISTER TO BE IN EFFECT.
;
R_BASE		PROC	NEAR
		CALL	GET_ATOM		;GET BASE ADDRESS.
		JC	RBERR			;NONE!
		CMP	AX,NUMBER		;NUMERIC INPUT?
		JNE	RBERR			;NO!!
		MOV	BASE_IP,DI		;YES: STORE IT FOR LATER USE.
		XOR	AX,AX
		RET
RBERR:
		MOV	AX,E_SYN		;SYNTAX ERROR.
		RET
R_BASE		ENDP
;
;	GET TASK-LOCATING DATA FROM ITS TCD:
;
;		THIS COMMAND CAUSES THE DEBUGGER TO EXTRACT THE STARTING
;	POINT, STACK BASE, DS AND ES VALUES FROM THE SPECIFIED TASK'S TCD AND
;	DISPLAY THEM TO THE OPERATOR.
;
R_TCD		PROC	NEAR
		CALL	GET_ATOM		;GET TASK NUMBER.
		JC	RTERR			;NONE!
		CMP	AX,NUMBER		;NUMERIC INPUT?
		JNE	RTERR			;NO!!
		MOV	GTCB_ID,DI		;YES: USE IT TO FIND THE TCB
		MOV	BP,OFFSET GTCB_SDB	;OF THE SPECIFIED TASK:
		CALL	DS_SVC
		JZ	RTERR1			;NOT A VALID TASK NUMBER!
		LES	BP,GTCB_ADDR		;GOT IT: ADDRESS TCB,
		LES	BP,DWORD PTR ES:[BP+TCB_TCD]	;THEN TCD.
		MOV	AX,ES:[BP+TCD_CS]	;GET CODE SEGMENT,
		MOV	DI,OFFSET RTCD_EPOINT	;FORMAT IT INTO THE
		CALL	BIN_2_HEX		;REPORT,
		INC	DI			;BYPASS THE COLON,
		MOV	AX,ES:[BP+TCD_IP]	;GET OFFSET OF START POINT,
		CALL	BIN_2_HEX		;FORMAT THAT INTO REPORT,
		MOV	AX,ES			;NOW PUBLISH THE ADDRESS OF
		MOV	DI,OFFSET RTCD_ADDR	;THE TCD ITSELF: SEGMENT,
		CALL	BIN_2_HEX
		INC	DI			;BYPASS THE COLON,
		MOV	AX,BP			;THEN THE OFFSET.
		CALL	BIN_2_HEX
		MOV	AX,ES:[BP+TCD_DS]	;GET DATA SEGMENT,
		MOV	DI,OFFSET RTCD_DS	;FORMAT THAT INTO THE REPORT,
		CALL	BIN_2_HEX
		MOV	AX,ES:[BP+TCD_ES]	;GET EXTRA SEGMENT,
		MOV	DI,OFFSET RTCD_ES	;FORMAT IT INTO THE REPORT,
		CALL	BIN_2_HEX
		MOV	AX,ES:[BP+TCD_SS]	;GET STACK SEGMENT,
		MOV	DI,OFFSET RTCD_SS	;FORMAT IT INTO THE REPORT,
		CALL	BIN_2_HEX
		INC	DI			;BYPASS THE COLON,
		MOV	AX,ES:[BP+TCD_SP]	;GET STACK BASE OFFSET,
		CALL	BIN_2_HEX		;FORMAT THAT, FINALLY,
		MOV	SI,OFFSET RTCD_BUF	;AND PUT THE REPORT OUT TO
		CALL	TOUT_DS			;THE TERMINAL.
		XOR	AX,AX
		RET
RTERR:
		MOV	AX,E_SYN		;SYNTAX ERROR.
		RET
RTERR1:
		MOV	AX,E_NOTASK		;NOT A VALID TASK ID.
		RET
R_TCD		ENDP
PAGE
;******************************************************************************
;
;		UTILITY SUBROUTINES:
;
;	THE FOLLOWING ROUTINES ARE CALLED FREELY BY THE DEBUGGER'S ACTION
;	ROUTINES, AND SUPPORT THEIR OPERATIONS FROM BELOW. MOST OF THEM 
;	HAVE TO DO WITH PARSING AND FORMATTING; A FEW ARE CONVENIENCES FOR
;	EASY INPUT/OUTPUT.
;
;*****************************************************************************
;
;	CONVERT NEXT ATOM TO INTERNAL FORM:
;
;	INPUT:	SI= INPUT STRING PTR, RETURNED UPDATED;
;	OUTPUT:	AX= ATOM TYPE IF CARRY= 0,
;		IF AX=
;			1) 0, END OF INPUT ENCOUNTERED.
;			2) 1 TO KEY_MAX, KEYWORD DESIGNATOR.
;			3) NUMBER, HEX INPUT IN DI.
;			4) ADDRESS, DOUBLEWORD ADDRESS IN ES:DI.
;			5) HBRKPOINT, HARD BREAKPOINT, ID IN DI.
;			6) SBRKPOINT, SOFT BREAKPOINT, ID IN DI.
;
GET_ATOM	PROC	NEAR
		PUSH	BX
		PUSH	CX
		PUSH	DX
		PUSH	BP
GA0:
		CALL	GETCHR			;GET INPUT CHARACTER...
		JC	GA01			;END OF INPUT.
		CALL	CHK_TERM		;GOT IT: TERMINATOR?
		JNC	GA1			;NO.
		CMP	AL,CR			;YES: END OF INPUT??
		JNE	GA0			;NO.
GA01:
		XOR	AX,AX			;NO INPUT ATOM THIS TIME.
		JMP	GARET
GA1:
		CALL	CHK_ALPHA		;ALPHA CHARACTER?
		JNC	GA2			;NO.
		DEC	SI			;YES: BACK UP INPUT PTR.
		CALL	CHK_KEY			;KEYWORD?
		JC	GA11			;YES!
		XOR	DI,DI			;NO: CLEAR "ACCUMULATOR" AND
		JMP	SHORT GA31		;TRY NUMERIC CONVERSION.
GA11:
		MOV	DI,AX			;KEYWORD NR. TO DI,
		ADD	SI,CX			;ADVANCE INPUT PTR,
		INC	SI			;PLUS 1 FOR TERMINATOR,
		CLC				;INDICATE SUCCESS.
		JMP	GARET
GA2:
		CMP	AL,DOLLAR_SIGN		;SOFT BREAKPOINT INDICATOR?
		MOV	CX,SBRKPOINT		;ASSUME SO FOR NOW...
		JE	GA25			;YES.
		CMP	AL,AT_SIGN		;NO: HARD BREAKPOINT INDICATOR?
		MOV	CX,HBRKPOINT		;ASSUME SO FOR NOW...
		JNE	GA3			;NO, TRY NUMERICS.
GA25:
		CALL	GETCHR			;YES: GET CHARACTER.
		JC	GA99			;NO MORE!
		CALL	CHK_DIGIT		;VALID NUMERIC?
		JC	GA21			;YES.
GA98:
		CALL	PASS_TERM		;NO: HUNT/PASS A TERMINATOR,
GA99:
		STC				;INDICATE ERROR,
		JMP	GARET
GA21:
		CALL	H_TO_B			;CONVERT CHARACTER TO NIBBLE.
GA22:
		XOR	AH,AH
		MOV	DI,AX			;BREAKPOINT ID TO DI,
		CALL	GETCHR			;GET NEXT CHARACTER,
		JC	GA99			;END OF INPUT.
		CALL	CHK_TERM		;BETTER BE TERMINATOR.
		JNC	GA99			;IT ISN'T!
GA23:
		MOV	AX,CX			;IT IS: RETURN VARIETY OF
		CLC				;BREAKPOINT.
		JMP	GARET
GA3:
		XOR	DI,DI			;TRY NUMERIC CONVERSION:
		DEC	SI			;BACK UP TO FIRST DIGIT.
GA31:
		CALL	GETCHR			;GET A CHARACTER
		JC	GA99			;END OF INPUT.
		CALL	CHK_TERM		;CHECK FOR TERMINATOR.
		JC	GA32			;YUP.
		CALL	CHK_DIGIT		;NO: DIGIT?
		JNC	GA98			;NO: ERROR.
		CALL	H_TO_B			;YES: MAKE IT BINARY,
		XOR	AH,AH
		MOV	CL,4			;SHIFT CURRENT ACCUMULATED VAL-
		SHL	DI,CL			;UE 4 BITS LEFT,
		ADD	DI,AX			;ADD IN NEW NIBBLE.
		JMP	SHORT GA31		;GO FOR ANOTHER.
GA32:
		CMP	AL,ASCII_COLON		;COLON (ADDRESS DIVIDER)?
		JE	GA33			;YES.
		MOV	AX,NUMBER		;NO: STRAIGHT NUMBER.
		CLC
		JMP	GARET
GA33:
		MOV	ES,DI			;FRESH VALUE TO ES.
		XOR	DI,DI			;CLEAR ACCUMULATOR AGAIN.
GA34:
		CALL	GETCHR			;GET INPUT...
		JC	GA99			;NONE.
		CALL	CHK_TERM		;TERMINATOR?
		JC	GA35			;YES.
		CALL	CHK_DIGIT		;NO: DIGIT?
		JNC	GA99			;NO, ERROR!
		CALL	H_TO_B			;YES: CONVERT TO BINARY,
		XOR	AH,AH
		MOV	CL,4			;SHIFT ACCUMULATED VALUE LEFT
		SHL	DI,CL			;4 BITS FOR NEW NIBBLE,
		ADD	DI,AX			;ADD IN NEW NIBBLE.
		JMP	SHORT GA34		;TRY FOR ANOTHER.
GA35:
		MOV	AX,ADDRESS		;INDICATE ADDRESS.
		CLC
GARET:
		POP	BP
		POP	DX
		POP	CX
		POP	BX
		RET
GET_ATOM	ENDP
;
;	GET AN ADDRESS ATOM, OR REPORT FAILURE.
;
;	INPUT:	SI= POINTER TO INPUT STRING.
;	OUTPUT:	IF CARRY=0,
;			ES:DI= ADDRESS ATOM, SI UPDATED BEYOND IT.
;		IF CARRY=1, NO ADDRESS ATOM COULD BE CONSTRUCTED,
;			BUT SI HAS BEEN MOVED PAST THE NEXT ATOM ENCOUNTERED.
;		AX, DI AND ES ARE ALWAYS ALTERED.
;
GET_ADDR	PROC	NEAR
		PUSH	SI			;SAVE INPUT PTR.
		CALL	GET_ATOM		;GET INPUT ATOM.
		JC	GAD6			;FAULTY ATOM.
		CMP	AX,0			;END OF INPUT??
		JNE	GAD5			;NO, SOME OTHER CONDITION.
GAD6:
		POP	AX			;YES, CLEAN STACK,
		JMP	SHORT GAD2		;AND EXIT WITH CARRY.
GAD5:
		CMP	AX,ADDRESS		;ADDRESS??
		JNE	GAD0			;NO, RETRY.
		POP	AX			;YES: CLEAN STACK.
		CLC
		RET
GAD0:
		POP	SI			;SECOND TRY: RESTORE PTR,
		CALL	GET_ATOM		;GET LEADING ATOM:
		JC	GAD2			;END OF INPUT.
		CMP	AX,NUMBER		;NUMERIC INPUT?
		JE	GAD10			;YES.
		CMP	AX,14			;NO: REGISTER NAME?
		JA	GAD2			;NO, SYNTAX ERROR.
		MOV	DI,AX			;YES:
		CALL	GET_REG			;GET REGISTER VALUE.
		JC	GAD2			;ILLEGAL!
GAD1:
		MOV	AL,[SI-1]		;GET TERMINATOR BYTE...
		CMP	AL,ASCII_COLON		;COLON (ADDRESS DIVIDER)?
		JE	GAD3			;YES.
		PUSH	DI			;NO: SAVE 1ST ATOM'S VALUE.
		MOV	DI,CS_INDEX		;GET VALUE OF TASK'S
		CALL	GET_REG			;CS REGISTER AS SEGMENT
		MOV	ES,DI			;PUT IT INTO ES,
		POP	DI			;RESTORE OFFSET,
		JNC	GAD4			;ALL'S WELL.
GAD2:
		STC				;NO: INDICATE FAILURE.
		RET
GAD10:
		ADD	DI,BASE_IP		;FOR PURE NUMERICS, ADD THE
		JMP	GAD1			;CURRENT MODULE BASE.
GAD3:
		MOV	ES,DI			;PUT 1ST ATOM'S VALUE IN ES.
		CALL	GET_ATOM		;GET OFFSET ATOM.
		JC	GAD2			;NONE!!!
		CMP	AX,NUMBER		;GOT IT: NUMERIC?
		JE	GAD4			;YES.
		CMP	AX,14			;NO: REGISTER NAME?
		JA	GAD2			;NO.
		MOV	DI,AX			;YES: REGISTER INDEX TO DI,
		CALL	GET_REG			;GET VALUE OF REGISTER.
		JC	GAD2			;ILLEGAL!
GAD4:
		CLC				;SUCCESS!
		RET
GET_ADDR	ENDP
;
;	GET A REGISTER VALUE FROM THE TASK'S STACK OR TCB:
;
;	INPUT:	DI= REGISTER NAME INDEX INTO KEYWORD TABLE: 1= 'ES'
;	OUTPUT:	IF CARRY=0, DI= REGISTER VALUE.
;		IF CARRY=1, INVALID REQUEST FOR REGISTER VALUE.
;
GET_REG		PROC	NEAR
		CMP	QBREAK_ID,0		;BREAKPOINT GOT US HERE?
		JNE	GR2			;CORRECT.
		STC				;NO:
		RET				;INDICATE ILLEGALITY.
GR2:
		PUSH	ES
		PUSH	BP
		LES	BP,TCB_ADDR		;GET THE ADDRESS OF THE TOP OF
		LES	BP,DWORD PTR ES:[BP+TCB_SP]	;TASK'S STACK.
		CMP	DI,FL_INDEX		;"NORMAL" (STACKED) REGISTER?
		JA	GR0			;NO: SS OR SP.
		DEC	DI			;YES: PRODUCE INDEX FOR THE
		SHL	DI,1			;UPCOMING STACK ACCESS.
		MOV	DI,ES:[BP][DI]		;GET REGISTER VALUE.
GR1:
		POP	BP
		POP	ES
		CLC
		RET
GR0:
		CMP	DI,SS_INDEX		;SS REGISTER?
		MOV	DI,ES			;BE READY...
		JE	GR1			;YES.
		MOV	DI,BP			;NO, SP REGISTER.
		ADD	DI,24			;BYPASS THE MTOS FRAME.
		JMP	SHORT GR1
GET_REG		ENDP
;
;	CHECK WHETHER INPUT CHARACTER IS ALPHABETIC.
;
;	INPUT:	AL= INPUT CHARACTER.
;	OUTPUT:	IF CARRY=1, CHARACTER IS ALPHABETIC.
;		IF CARRY=0, NOT.
;
CHK_ALPHA	PROC	NEAR
		CMP	AL,ASCII_A		;LOW?
		JB	CA9			;YES.
		CMP	AL,ASCII_Z		;NO: HIGH?
		JA	CA9			;YES.
		STC				;NO: GOOD.
		RET
CA9:
		CLC
		RET
CHK_ALPHA	ENDP
;
;	CHECK WHETHER INPUT CHARACTER IS HEXADECIMAL-NUMERIC.
;
;	INPUT:	AL= INPUT CHARACTER.
;	OUTPUT:	IF CARRY=1, INPUT CHARACTER IS VALID HEX.
;		IF CARRY=0, NOT.
;
CHK_DIGIT	PROC	NEAR
		CMP	AL,ASCII_0		;LOW?
		JB	CD9			;YES.
		CMP	AL,ASCII_F		;NO: HIGH?
		JA	CD9			;YES.
		CMP	AL,ASCII_9		;NO: IN FORBIDDEN ZONE?
		JBE	CD0			;NO.
		CMP	AL,ASCII_A		;MAYBE...
		JB	CD9			;YES.
CD0:
		STC				;NO.
		RET
CD9:
		CLC
		RET
CHK_DIGIT	ENDP
;
;	ATTEMPT TO MATCH INPUT STRING WITH A KEYWORD.
;
;	INPUT:	SI= STRING POINTER.
;	OUTPUT:	IF CARRY=1,	AX= KEYWORD INDEX (1= 'ES').
;				CX= LENGTH OF ATOM EXCLUSIVE OF TERMINATOR.
;		IF CARRY=0, COULD NOT MATCH KEYWORD.
;
CHK_KEY		PROC	NEAR
		PUSH	BX
		PUSH	DX
		PUSH	DI
		MOV	DI,OFFSET KEYWRD_TABLE
		MOV	DL,1			;ID OF LOWEST KEYWORD.
;
;	DL=	IDENTIFIER OF NEXT KEYWORD TO CHECK.
;	CS:DI=	ADDRESS OF THAT KEYWORD TABLE ENTRY.
;	DS:SI=	ADDRESS OF INPUT STRING BEING TESTED.
;
CK0:
		MOV	CL,CS:[DI]		;GET KEYWORD LENGTH.
		CMP	CL,0FFH			;END OF TABLE?
		CLC				;BE READY FOR IT...
		JE	CK25			;YES.
		MOV	CH,CS:[DI+1]		;NO: GET MINIMUM-MATCH LENGTH.
		MOV	BX,SI			;SAVE INPUT POINTER IN BX,
		ADD	DI,2			;BYPASS KEYWORD LENGTH DATA
		XOR	DH,DH			;NO CHARS COMPARED YET.
;
;	DS:BX=	ADDRESS OF NEXT BYTE OF INPUT TO COMPARE TO KEYWORD,
;	CL=	LENGTH OF KEYWORD.
;	CH=	MINIMUM-LENGTH MATCHING STRING TO SATISFY THIS ENTRY.
;	DL=	KEYWORD IDENTIFIER NUMBER BEING CHECKED.
;	DH=	NUMBER OF CHARACTERS OF INPUT SUCCESSFULLY COMPARED.
;	DS:SI=	ADDRESS OF START OF INPUT STRING.
;	CS:DI=	ADDRESS OF NEXT BYTE OF KEYWORD TO COMPARE TO INPUT.
;
CK5:
		MOV	AL,[BX]			;GET A BYTE FROM THE INPUT.
		CALL	L_2_U			;CONVERT LOWER TO UPPER CASE.
		MOV	AH,CS:[DI]		;GET A BYTE FROM THE KEYWORD.
		CMP	AH,AL			;MATCH?
		JNE	CK1			;NO.
		INC	BX			;YES: BYPASS INPUT BYTE,
		INC	DI			;     BYPASS KEYWORD BYTE.
		INC	DH			;ONE MORE SUCCESSFUL COMPARISON
		DEC	CH			;ONE FEWER TO REACH MINIMUM,
		DEC	CL			;ONE FEWER IN KEYWORD ENTRY.
		JNE	CK5			;COMPARE NEXT TWO BYTES.
CK10:						;** END OF KEYWORD ENTRY: **
		MOV	AL,[BX]			;NEXT BYTE OF INPUT HAD BETTER
		CALL	CHK_TERM		;BE A VALID TERMINATOR...
		JNC	CK15			;NOT A TERMINATOR: FAIL!
CK20:
		MOV	AL,DL			;RETURN IDENTIFIER IN AX.
		MOV	CL,DH			;RETURN COMPARE LENGTH IN CX.
		XOR	AH,AH
		XOR	CH,CH
		STC				;INDICATE SUCCESS.
CK25:
		POP	DI
		POP	DX
		POP	BX
		RET
;
;	IF THE COMPARISON FAILS INSIDE THE KEYWORD, THEN WE CHECK FOR THE
;	POSSIBILITY OF AN ABBREVIATION.
;
;	DS:BX=	INPUT ADDRESS AT WHICH COMPARISON FAILED,
;	CH=	NUMBER OF CHARACTERS REMAINING BEFORE MINIMUM MET.
;	CL=	NUMBER OF CHARACTERS REMAINING IN KEYWORD.
;	DL=	KEYWORD IDENTIFIER,
;	DH=	NUMBER OF INPUT CHARACTERS SUCCESSFULLY COMPARED.
;	DS:SI=	ADDRESS OF START OF INPUT STRING,
;	CS:DI=	ADDRESS IN KEYWORD AT WHICH COMPARISON FAILED.
;
CK1:
		MOV	AL,[BX]			;GET LAST-CHECKED INPUT BYTE
		CALL	CHK_TERM		;IS IT A TERMINATOR?
		JNC	CK15			;NOT A TERMINATOR: FAIL!
		CMP	CH,0			;TERMINATOR: AT/PAST MINIMUM?
		JLE	CK20			;YES!
CK15:
		XOR	CH,CH			;FAILED ON THIS ENTRY:
		INC	DL			;ADVANCE IDENTIFIER NUMBER,
		ADD	DI,CX			;GO TO NEXT KEYWORD ENTRY
		JMP	CK0			;AND TRY AGAIN.
CHK_KEY		ENDP
;
;	GET AN INPUT CHARACTER.
;
;	INPUT:	SI= INPUT STRING POINTER.
;	OUTPUT: IF CARRY=0, AL= NEXT CHARACTER AND SI HAS BEEN INCREMENTED.
;		IF CARRY=1, NO FURTHER INPUT.
;
GETCHR		PROC	NEAR
		MOV	AL,[SI]			;GET BYTE FROM BUFFER.
		TEST	AL,AL			;ZERO?
		JNE	GCH0			;NO, GOOD.
		STC				;END OF INPUT.
		RET
GCH0:
		INC	SI			;ADVANCE BUFFER POINTER.
		CALL	L_2_U			;CONVERT LOWER TO UPPER CASE.
		CLC
		RET
GETCHR		ENDP
;
;	CHECK CHARACTER FOR MEMBERSHIP IN TERMINATOR SET.
;
;	INPUT:	AL= CHARACTER.
;	OUTPUT:	IF CARRY=1, CHARACTER IS A TERMINATOR.
;		IF CARRY=0, NOT.
;
CHK_TERM	PROC	NEAR
		PUSH	DI
		PUSH	CX
		PUSH	ES
		PUSH	CS			;SET CODE SEGMENT ADDRESS
		POP	ES			;INTO ES.
		MOV	DI,OFFSET TERM_TABLE	;ADDRESS TERMINATOR TBL.
		MOV	CX,NR_TERMS		;THIS MANY.
	REPNE	SCASB				;LOOK FOR IT.
		JNE	CT0			;NOT A TERMINATOR.
		POP	ES			;YES:
		POP	CX			;UNSAVE REGISTERS,
		POP	DI
		STC				;DECLARE A TERMINATOR.
		RET
CT0:
		POP	ES			;RESTORE ES AND CX, BUT
		POP	CX			;DECLARE NO TERMINATOR.
		POP	DI
		CLC
		RET
CHK_TERM	ENDP
;
;	PASS THE NEXT TERMINATOR IN THE INPUT STREAM.
;
;	INPUT:	SI= INPUT STRING POINTER.
;	OUTPUT:	SI IS UPDATED PAST THE NEXT ENCOUNTERED TERMINATOR, OR TO
;		END-OF-INPUT.
;
PASS_TERM	PROC	NEAR
		CMP	BYTE PTR [SI],0		;END OF INPUT?
		JNE	PT1			;NO.
		RET				;YES: RETURN.
PT1:
		LODSB				;GET A BYTE OF INPUT.
		CALL	CHK_TERM		;TERMINATOR?
		JNC	PT1			;NO, GET NEXT ONE.
		RET				;YES.
PASS_TERM	ENDP
;
;	CONVERT WORD BINARY TO HEX ASCII
;
;	INPUT:	AX= WORD TO CONVERT, DI= ADDRESS OF OUTPUT STRING.
;	OUTPUT:	DI HAS BEEN ADVANCED BY 4 BYTES.
;
BIN_2_HEX	PROC	NEAR
		PUSH	AX		;SAVE INPUT VALUE,
		MOV	AL,AH		;HIGH BYTE TO AL,
		CALL	BYT_2_HEX	;FORMAT IT.
		POP	AX		;RETRIEVE INPUT VALUE,
		CALL	BYT_2_HEX	;FORMAT LOW BYTE,
		RET			;DONE.
BIN_2_HEX	ENDP
;
;	HEX ASCII TO BYTE BINARY
;
;	INPUT:	SI= INPUT STRING ADDRESS.
;	OUTPUT:	IF CARRY=0, BL= CONVERTED VALUE.
;		IF CARRY=1, CONVERSION ERROR.
;
HEX_2_BYTE	PROC	NEAR
		PUSH	CX
		LODSB			;GET THE PRESUMED HEXADECIMAL DIGIT.
		CALL	H_TO_B		;CONVERT TO BINARY.
		JC	HERR1		;ERROR!
		MOV	CL,4		;GOOD:
		SHL	AL,CL		;SHIFT VALUE UP TO HIGH NIBBLE.
		MOV	CL,AL		;SAVE SHIFTED VALUE IN CL.
		LODSB			;GET NEXT DIGIT.
		CALL	H_TO_B		;CONVERT TO BINARY.
		JNC	HERR0		;GOOD.
		DEC	SI		;NO GOOD: BACK UP THE POINTER,
		MOV	AL,CL		;GET BACK "HIGH" NIBBLE,
		MOV	CL,4		;AND ARRANGE TO SHIFT IT BACK DOWN
		SHR	AL,CL		;TO LOW POSITION.
		POP	CX		;RESTORE CX,
		RET			;EXIT.
HERR0:
		ADD	AL,CL		;ADD LOW NIBBLE TO HIGH NIBBLE.
		POP	CX		;RESTORE CX,
		CLC			;INDICATE SUCCESS,
		RET			;EXIT.
HERR1:
		STC			;INDICATE FAILURE
		POP	CX		;RESTORE CX,
		RET			;EXIT.
HEX_2_BYTE	ENDP
;
;	DIGIT TO NIBBLE:
;
;	INPUT:	AL= CHARACTER TO BE CONVERTED.
;	OUTPUT:	IF CARRY=0, AL= BINARY VALUE OF INPUT HEX CHARACTER.
;		IF CARRY=1, CONVERSION ERROR.
;
H_TO_B		PROC	NEAR
		CMP	AL,ASCII_0		;>= '0' ?
		JB	HB0			;NO, ERROR.
		CMP	AL,ASCII_F		;YES: <= 'F' ?
		JA	HB0			;NO, ERROR.
		MOV	AH,ASCII_A-10		;YES: PREPARE FOR HIGH RANGE.
		CMP	AL,ASCII_A		;>= 'A' ?
		JAE	HB1			;YES, 'A' THROUGH 'F'
		CMP	AL,ASCII_9		;NO: <= '9' ?
		JA	HB0			;NO: ERROR!!!
		MOV	AH,ASCII_0		;YES: '0' THROUGH '9'
HB1:
		SUB	AL,AH			;REMOVE BIAS FROM CHARACTER.
		CLC
		RET
HB0:
		STC
		RET
H_TO_B		ENDP
;
;	BYTE BINARY TO HEX ASCII
;
;	INPUT:	DI= ADDRESS OF FORMATTING BUFFER, AL= BYTE TO FORMAT.
;	OUTPUT:	DI HAS BEEN ADVANCED BY 2 FROM THE INPUT FORMATTING POINTER.
;
BYT_2_HEX	PROC	NEAR
		PUSH	CX
		PUSH	SI
		MOV	AH,0
		PUSH	AX
		AND	AL,0F0H			;EXTRACT TOP NIBBLE
		MOV	CL,4
		SHR	AL,CL			;SHIFT IT INTO LOW NIBBLE.
		MOV	SI,AX			;AS ABOVE.
		MOV	AL,CS:HEX_DIGITS[SI]
		MOV	[DI],AL			;PUT ASCII TO OUTPUT STRING.
		INC	DI
		POP	AX
		AND	AL,0FH			;NOW BOTTOM NIBBLE.
		MOV	SI,AX
		MOV	AL,CS:HEX_DIGITS[SI]
		MOV	[DI],AL			;PUT ASCII TO OUTPUT STRING.
		INC	DI			;LEAVE DI AS PROMISED.
		POP	SI
		POP	CX
		RET
BYT_2_HEX	ENDP
;
;	CONVERT LOWER CASE LETTERS TO UPPER CASE.
;
;	INPUT:	AL=	CANDIDATE CHARACTER.
;	OUTPUT:	AL=	CHARACTER OR ITS UPPER CASE EQUIVALENT.
;
L_2_U		PROC	NEAR
		CMP	AL,LOWER_A		;LOWER CASE LETTER?
		JB	L_2_U_0			;NO.
		CMP	AL,LOWER_Z		;MAYBE...
		JA	L_2_U_0			;NO.
		SUB	AL,LOWER_TO_UPPER	;YES: CONVERT.
L_2_U_0:
		RET
L_2_U		ENDP
PAGE
;
;		** I/O TO/FROM CONSOLE ROUTINES **
;
;	OUTPUT TO CONSOLE FROM DATA SEGMENT
;
;		DS:SI PASSES OUTPUT STRING ADDRESS.
;
TOUT_DS		PROC	NEAR
		MOV	OUT_SEG,DS
		MOV	OUT_OFF,SI
		PUSH	BP		;SAVE VALUE OF BP,
		MOV	BP,OFFSET TOUT_SDB
		CALL	DS_SVC
		POP	BP
		RET
TOUT_DS		ENDP
;
;	OUTPUT TO CONSOLE FROM CODE SEGMENT
;
;	CALLED AS IS ABOVE.
;
TOUT_CS		PROC	NEAR
		MOV	OUT_SEG,CS
		MOV	OUT_OFF,SI
		PUSH	BP		;PRESERVE BP,
		MOV	BP,OFFSET TOUT_SDB
		CALL	DS_SVC
		POP	BP
		RET
TOUT_CS		ENDP
;
;	INPUT FROM CONSOLE
;
;		DS:SI CONTAINS ADDRESS OF INPUT CONTROL BLOCK.
;
TIN		PROC	NEAR
		PUSH	SI		;SAVE INPUT CONTROL BLOCK ADDRESS,
		PUSH	CX		;WHATEVER'S IN CX.
		LODSB			;GET LENGTH BYTE, POINT SI AT TEXT.
		MOV	CL,AL		;LENGTH BYTE TO CL.
		XOR	AX,AX
		XOR	CH,CH
		MOV	IN_LEN,CX		;STORE MAX LENGTH IN PARAMS.
		MOV	IN_OFF,SI		;PUT THAT IN WITH PARAMS.
TIN0:
		MOV	[SI],AL			;CLEAR A BYTE OF BUFFER,
		INC	SI			;ADVANCE POINTER,
		LOOP	TIN0			;AND GO AROUND TILL DONE.
		PUSH	BP
		MOV	BP,OFFSET TIN_SDB
		CALL	DS_SVC
		POP	BP
		POP	CX
		POP	SI
		RET
TIN		ENDP
;
;	CONVENIENCE FOR GENERATING TYPE 1 SVC CALLS:
;
;	INPUT:	BP= OFFSET OF SDB, ASSUMED TO BE IN SEGMENT ADDRESSED BY DS.
;
DS_SVC		PROC	NEAR
		PUSH	DS
		PUSH	BP
		INT	LOW X_SVC_VECTOR
		POP	BP
		POP	DS
		RET
DS_SVC		ENDP
DEB_CODE	ENDS
		END
