PAGE	60,132
INCLUDE B:BIG.INC
;
;	DIS.ASM:	DISCRETE SIGNALS CONTROL.
;	F. W. PORRETTO, 12/15/82
;
		NAME	DISCRETE_SERVICES
;
MTOS_CS		SEGMENT	PARA PUBLIC
		ASSUME	DS:MTOS_DS,CS:MTOS_CS,SS:MTOS_DS
		PUBLIC	X_DSCRTE,X_INPUT_DSCRTE,X_OUTPUT_DSCRTE
		EXTRN	X_SFLAGS	:NEAR
;
;	LOCAL EQUATES
;
DSIO  		EQU	01H		;SDB OFFSET OF OPERATION(INPUT/OUPUT)
DSTYPE		EQU	01H		;SDB OFFSET OF TYPE OF INPUT(OR,AND)
					; OR OUTPUT(SET,RESET)
DSGRP		EQU	02H		;SDB OFFSET OF DISCRETE GROUP ID
DSBITS		EQU	04H		;SDB OFFSET OF BIT PATTERN
IO_MASK		EQU	01H		;ISOLATE OPERATION
TYP_MASK	EQU	80H		;ISOLATE TYPE FIELD
DSGRP_MASK	EQU	01FFFH		;ISOLATE GROUP ID
PAGE
;*****************************************************************
;*         XFIND_DSCRTE:  FIND THE DISCRETE GROUP                *
;*****************************************************************
;
;	THIS PROCEDURE FINDS THE DISCRETE BYTE.
;
;	INPUT:	DX=	DISCRETE GROUP ID
;	OUTPUT:	ES:SI=	ADDRESS OF DISCRETE BYTE
;
XFIND_DSCRTE	PROC	NEAR
		MOV	AX,MTOS_GROM			;DISCRETE DATA IN
		MOV	ES,AX				; MTOS_GROM
		MOV	AX,DDD_SIZE			;MUL ID X SIZE OF
		MUL	DX				; DISCRETE DATA DEF'N
		MOV	SI,AX
		LES	SI,DWORD PTR ES:X_DDD_TABLE[SI]	;ADD TABLE ADDRESS
		              				; & GET DISCRETE ADDR
		RET
XFIND_DSCRTE	ENDP
PAGE
;*****************************************************************
;*         X_INPUT_DSCRTE:  INNER DISCRETE INPUT PROCEDURE       *
;*****************************************************************
;
;	THIS PROCEDURE INPUTS AND TESTS THE DISCRETE GROUP
;	AND PREPARES THE SVC STATUS.
;
;	INPUT:	DX=	DISCRETE GROUP ID
;		CH=	TYPE (INPUT/OUTPUT)
;		CL=	DISCRETE BIT PATTERN
;		BX=	TCB ADDRESS
;	OUTPUT:	AX=	SVC STATUS
;
X_INPUT_DSCRTE	PROC	NEAR
		PUSH	ES			;SAVE REGISTERS
		PUSH	SI
		PUSH	DX
		CALL	XFIND_DSCRTE		;FIND THE DISCRETE GROUP
		MOV	DL,ES:[SI]		;INPUT THE DISCRETE BYTE
		TEST	CH,CH			;AND TEST/OR TEST ?
		JE	DSCAND			;AND TEST.
		XOR	AX,AX			;OR TEST: CLEAR STATUS
		AND	DL,CL			;TEST GROUP
		TEST	DL,DL
		JE	ORDONE
		MOV	AX,FLAG_CF		;SET STATUS BIT FOR MATCH
ORDONE:		POP	DX			;RESTORE REGISTER
		POP	SI
		POP	ES
		RET
DSCAND:
		XOR	AX,AX			;AND TEST: CLEAR STATUS
		AND	DL,CL			;TEST GROUP
		CMP	DL,CL
		JNE	ANDDONE
		MOV	AX,FLAG_CF		;SET STATUS BIT FOR MATCH
ANDDONE:	POP	DX
		POP	SI
		POP	ES
		RET
X_INPUT_DSCRTE	ENDP
PAGE
;*****************************************************************
;*         X_OUTPUT_DSCRTE:   INNER DISCRETE OUTPUT PROCEDURE    *
;*****************************************************************
;
;	THIS PROCEDURE SETS OR RESETS THE SPECIFIED BITS
;	IN THE DISCRETE GROUP AND PREPARES THE SVC STATUS.
;
;	INPUT:	DX=	DISCRETE GROUP ID
;		CH=	TYPE(SET,RESET)
;		CL=	DISCRETE BIT PATTERN
;		BX=	TCB ADDRESS
;	OUTPUT:	AX=	SVC STATUS
;
X_OUTPUT_DSCRTE	PROC	NEAR
		PUSH	ES			;SAVE REGISTERS
		PUSH	SI
		PUSH	DX
		PUSH	DI
		CALL	XFIND_DSCRTE		;FIND DISCRETE GROUP
		MOV	DL,ES:[SI]		;INPUT DISCRETE BYTE
		TEST	CH,CH			;SET/RESET ?
		JNE	DSCSET
;
;	RESET DISCRETES
;
		NOT	CL			;INVERT THE MASK,
		AND	CL,DL			;AND IT ACROSS GROUP.
		JMP	SHORT DSCOUTPUT
;
;	SET DISCRETES
;
DSCSET:		OR	CL,DL			;OR MASK INTO GROUP.
;
;	OUTPUT NEW DISCRETES AND TEST FOR CHANGE
;
DSCOUTPUT:	MOV	ES:[SI],CL		;OUTPUT DISCRETE BYTE
		XOR	AX,AX			;CLEAR STATUS
		CMP	DL,CL			;ANY BITS CHANGE ?
		JE	DSCDONE
		MOV	AX,FLAG_CF
DSCDONE:	POP	DI		  	;RESTORE REGISTERS
		POP	DX
		POP	SI
		POP	ES
		RET
X_OUTPUT_DSCRTE	ENDP
PAGE
;************************************************
;*         X_DSCRTE - DISCRETE I/O SVC          *
;************************************************
;
;	THIS PROCEDURE VALIDATES THE SDB PARAMETERS,
;	INPUTS OR OUTPUTS THE DISCRETE GROUP AND
;	RETURNS STATUS TO THE TASK.
;
;	INPUT:	ES:SI=	SDB ADDRESS
;		BX=	TCB ADDRESS
;	OUTPUT:	NONE
;
X_DSCRTE	PROC	FAR
		MOV	CL,ES:[SI+DSBITS]	;GET DSBITS
		MOV	CH,ES:[SI+DSTYPE]		;GET TYPE
		AND	CH,TYP_MASK		;ISOLATE TYPE
		ROL	CH,1			;RIGHT JUSTIFY TYPE
		MOV	BL,ES:[SI+DSIO]		;GET OPERATION SELECTION
		AND	BL,IO_MASK		;ISOLATE IT
		MOV	DX,ES:[SI+DSGRP]	;GET GROUPID
		AND	DX,DSGRP_MASK		;ISOLATE GROUP ID
		MOV	AX,SVC_FAILED+PARM_ERR	;READY FOR FAILURE...
		CMP	DX,X_DISCRETE_NUM	;VALIDATE GROUP ID
		JGE	DSCERR			;NOT VALID.
		TEST	BL,BL			;VALID: INPUT OR OUTPUT ?
		JNE	DSCOUT			;OUTPUT.
		CALL	X_INPUT_DSCRTE		;INPUT.
		JMP	SHORT DSCERR
DSCOUT:
		CALL	X_OUTPUT_DSCRTE		;OUTPUT.
DSCERR:
		CALL	X_SFLAGS
		RET
X_DSCRTE	ENDP
MTOS_CS		ENDS
		END
