;
;			MTOS-86 LEVEL II
;			NOVEMBER 19, 1982
;			SYSTEM SYMBOLS
;
;	THE FOLLOWING SYMBOL DEFINITIONS ARE USED WIDELY THROUGH MTOS. THEY 
;DEFINE ALL THE SYSTEM'S INTERNAL DATA STRUCTURES. ANY CHANGE TO ANY ONE OF
;THEM SHOULD BE FOLLOWED BY THE REASSEMBLY OF THE ENTIRE SYSTEM, TO INSURE
;THAT THE NEW DEFINITIONS ARE OBSERVED IN ALL APPLICABLE PLACES.
;
;	TCD: TASK CONTROL DEFINITION BLOCK.
;
TCD_AX		EQU	0		;STARTUP PARAMETER 1
TCD_BX		EQU	TCD_AX+2	;STARTUP PARAMETER 2
TCD_DS		EQU	TCD_BX+2	;STARTUP DS
TCD_ES		EQU	TCD_DS+2	;STARTUP ES
TCD_SP		EQU	TCD_ES+2	;STARTUP SP
TCD_SS		EQU	TCD_SP+2	;STARTUP SS
TCD_IP		EQU	TCD_SS+2	;ENTRY POINT OFFSET
TCD_CS		EQU	TCD_IP+2	;ENTRY POINT SEGMENT
TCD_LOAD	EQU	TCD_CS+2	;ADDRESS OF LOAD DESCRIPTOR, IF ANY.
TCD_TIME	EQU	TCD_LOAD+4	;OVERRIDE TIME: UNIT, NUMBER OF UNITS.
TCD_OVPRI	EQU	TCD_TIME+2	;OVERRIDE PRIORITY INCREMENT.
TCD_BPRI	EQU	TCD_OVPRI+1	;BOUNDARY ON PRIORITY OVERRIDES.
TCD_INPRI	EQU	TCD_BPRI+1	;INHERENT PRIORITY
TCD_PID		EQU	TCD_INPRI+1	;CPU ID (0FFH = ALL CPUS)
TCD_FLAGS	EQU	TCD_PID+1	;TASK FLAG WORD: SEE TCB_FLAGS.
;
; 	NOTE THAT 4 BYTES ARE RESERVED FOR ALIGNMENT AND EXPANSION.
;
TCD_SIZE	EQU	TCD_FLAGS+6	;SIZE OF TCD IN BYTES.
;
;	TASK LOAD DESCRIPTOR IS PRIMARILY LOADER-TASK-DEPENDENT, BUT ONE
;	FIELD IS FIXED IN POSITIONAND SIZE:
;
TLD_OVID	EQU	0		;OVERLAY REGION NUMBER: 1 BYTE.
;
;	TCB: TASK (DYNAMIC) CONTROL BLOCK.
;
TCB_LINK	EQU	0		;FORWARD LINK OR 0.
TCB_PRI		EQU	TCB_LINK+2	;PRIORITY.
TCB_STATE	EQU	TCB_PRI+1	;CURRENT STATE: SEE BELOW.
TCB_SP		EQU	TCB_STATE+1	;CURRENT SP
TCB_SS		EQU	TCB_SP+2	;CURRENT SS
TCB_SCHDL	EQU	TCB_SS+2	;LAST START TIME, ABSOLUTE MSEC.
TCB_QRST	EQU	TCB_SCHDL+6	;RCB QUEUE HEAD.
TCB_CRCB	EQU	TCB_QRST+2	;CURRENT RCB.
TCB_TMB		EQU	TCB_CRCB+2	;TMB FOR PRIORITY OVERRIDE OR ENDR.
TCB_PID		EQU	TCB_TMB+2	;CPU ID (0FFH = ALL CPUS)
TCB_FLAGS	EQU	TCB_PID+1	;TASK FLAGS: SEE BELOW.
TCB_8087	EQU	TCB_FLAGS+1	;OFFSET ADDRESS OF 8087 SAVE AREA.
TCB_LOCK	EQU	TCB_8087+2	;LOCK STRUCTURE.
TCB_TCD		EQU	TCB_LOCK+4	;TCD DOUBLEWORD ADDRESS
TCB_SIZE	EQU	TCB_TCD+4	;SIZE OF TCB IN BYTES.
;
;	TASK FLAG BITS:
;
TFL_8087	EQU	00000001B	;TASK USES 8087 NDP
TFL_LOAD	EQU	00000010B	;TASK IS LOADABLE FROM MASS STORAGE.
TFL_NOHLT	EQU	00000100B	;TASK MAY NOT BE HALTED (A DEBUGGER)
TFL_OVPRI	EQU	00001000B	;OVERRIDE PRIORITY AFTER TIME EXPIRES
TFL_TSEM	EQU	00010000B	;SEMAPHORE BID CANCELLATION AFTER TIME
					;EXPIRES
TFL_TRES	EQU	00100000B	;PERIPHERAL UNIT RESERVE BID CANCELLA-
					;TION AFTER TIME EXPIRES
					;(NOTE: FOR THE ABOVE THREE TIMING OP-
					;TIONS, THE APPLICABLE TIME INTERVAL 
					;IS THE ONE IN TCD_TIME)
TFL_ALLOC	EQU	10000000B	;TCD/TCB COMBINATION WAS ALLOCATED FROM
					;THE SMR (DYNAMICALLY CREATED TASK)
;
;	THE FOLLOWING SYMBOLS DEFINE THE STATES A TASK MAY OCCUPY:
;
DORMANT		EQU	0		;TASK DORMANT
READY		EQU	1		;     READY
BLOCK		EQU	2		;     BLOCKED (ADDED TO SUBDESCRIPTOR)
ACTIVE		EQU	3		;     ACTIVE
;
;	SUBDESCRIPTORS ADDED TO "BLOCK" SYMBOL:
;
STA_EF		EQU	4		;EVENT FLAG WAIT
STA_SEMA	EQU	STA_EF+4	;SEMAPHORE WAIT
STA_LOAD	EQU	STA_SEMA+4	;WAIT FOR TARGET  TO BE LOADED
STA_STRT	EQU	STA_LOAD+4	;WAIT FOR TARGET  TO START
STA_ENDT	EQU	STA_STRT+4	;WAIT FOR TARGET  TO TERMINATE.
STA_MSG		EQU	STA_ENDT+4	;WAIT TO RECEIVE MESSAGE
STA_MSGRCVD	EQU	STA_MSG+4	;WAIT FOR OWN MESSAGE TO BE RECEIVED
STA_POOL	EQU	STA_MSGRCVD+4	;WAIT FOR ALLOCATABLE MEMORY
STA_HLT		EQU	STA_POOL+4	;DEBUGGER HALTED .
STA_BREAK	EQU	STA_HLT+4	;DEBUGGER BREAKPOINTED.
STA_PAUSE	EQU	STA_BREAK+4	; IN PAUSE CONDITION.
STA_CNTRL	EQU	STA_PAUSE+4	; AWAITING CONTROLLED PROGRAM ACCESS
STA_WAIT	EQU	STA_CNTRL+4	; AWAITING I/O COMPLETION
STA_ABORT	EQU	STA_WAIT+4	; ABORTED BY EXTERNAL REQUEST.
STA_SVC		EQU	STA_ABORT+4	; AWAITING SVC COMPLETION
STA_ENDR	EQU	STA_SVC+4	; AWAITING TIMED "RESTART"
STA_ERR0	EQU	STA_ENDR+4	;RUN TIME ERROR, TYPE 0.
STA_ERR1	EQU	STA_ERR0+4	; "   "     "    TYPE 1.
STA_ERR2	EQU	STA_ERR1+4	; "   "     "    TYPE 2.
STA_ERR3	EQU	STA_ERR2+4	; "   "     "    TYPE 3.
;
;	DCD: DEVICE CONTROL DEFINITION BLOCK.
;
DCD_UCD		EQU	0		;OFFSET OF 0TH UCD.
DCD_UNITS	EQU	DCD_UCD+2	;NUMBER OF UNITS.
DCD_SHR		EQU	DCD_UNITS+1	;DEVICE NR. OF SHARING DEVICE
					;(0FFH = NONE).
DCD_INIT	EQU	DCD_SHR+1	;ADDRESS OF INITIALIZATION ROUTINE.
DCD_SVC		EQU	DCD_INIT+4	;ADDRESS OF IRB INPUT ROUTINE
					;(SOMETIMES CALLED SVC REQUEST HANDLER)
DCD_FLAGS	EQU	DCD_SVC+4	;FLAGS: SEE BELOW.
DCD_FMAX	EQU	DCD_FLAGS+1	;MAXIMUM FUNCTION CODE FOR DRIVER.
;
;	NOTE THAT 2 BYTES ARE RESERVED FOR ALIGNMENT AND EXPANSION.
;
DCD_SIZE	EQU	DCD_FMAX+3	;SIZE OF DCD IN BYTES.
;
;	FLAGS FOR DCD_FLAGS FIELD OF DCD:
;
DFL_PARA	EQU	00000001B	;PARALLEL DRIVER.
DFL_TYPE2	EQU	00000010B	;TYPE 2 DRIVER.
;
;	UCD: UNIT CONTROL DEFINITION BLOCK.
;
UCD_PID		EQU	0		;CPU ID (NO GLOBAL OPTION HERE!)
UCD_INTNO	EQU	UCD_PID+1	;INTERRUPT NUMBER.
UCD_INTADR	EQU	UCD_INTNO+1	;INTERRUPT VECTOR ADDRESS.
UCD_UNS		EQU	UCD_INTADR+4	;UNSOLICITED INTERRUPT TASK NUMBER
UCD_PORT	EQU	UCD_UNS+2	;UNIT-DEPENDENT PARAMETERS:
					;16 BYTES ROOM ALLOWED.
UCD_SIZE	EQU	UCD_PORT+16	;SIZE OF UCD IN BYTES.
;
;	DCB: DEVICE CONTROL (DYNAMIC) BLOCK
;
DCB_LINK	EQU	0		;FORWARD LINK OR 0.
DCB_ID		EQU	DCB_LINK+2	;DEVICE NUMBER.
DCB_STS		EQU	DCB_ID+1	;DRIVER STATUS BYTE.
DCB_QIRB	EQU	DCB_STS+1	;DRIVER IRB QUEUE.
DCB_CIRB	EQU	DCB_QIRB+2	;DRIVER CURRENT IRB.
DCB_UCB		EQU	DCB_CIRB+2	;ADDRESS OF UNIT 0 UCB
DCB_DCD		EQU	DCB_UCB+2	;OFFSET IN MTOS_GROM OF DCD.
DCB_GPR		EQU	DCB_DCD+2	;DRIVER GENERAL STORAGE.
DCB_SIZE	EQU	DCB_GPR+8	;SIZE OF DCB IN BYTES.
;
;	UCB: UNIT CONTROL (DYNAMIC) BLOCK
;
UCB_LINK	EQU	0		;FORWARD LINK OR 0.
UCB_UNIT	EQU	UCB_LINK+2	;UNIT NUMBER
UCB_STS		EQU	UCB_UNIT+1	;UNIT STATUS BYTE.
UCB_QIRB	EQU	UCB_STS+1	;UNIT IRB QUEUE.
UCB_CIRB	EQU	UCB_QIRB+2	;UNIT CURRENT IRB.
UCB_SEQ		EQU	UCB_CIRB+2	;CURRENT IRB'S SEQUENCE NUMBER.
UCB_DCB		EQU	UCB_SEQ+2	;ADDRESS OF DRIVER DCB.
UCB_UCD		EQU	UCB_DCB+2	;OFFSET IN MTOS_GROM OF UCD.
UCB_TCB		EQU	UCB_UCD+2	;RESERVER'S TCB ADDRESS OR 0.
UCB_GPR		EQU	UCB_TCB+2	;GENERAL UNIT STORAGE.
UCB_SIZE	EQU	UCB_GPR+32	;SIZE OF UCB IN BYTES.
;
;	MPD:	MEMORY POOL DEFINITION BLOCK.
;
MPD_OFF		EQU	0		;OFFSET OF POOL (MUST BE 0)
MPD_DS		EQU	MPD_OFF+2	;SEGMENT OF POOL (MUST BE UNIQUE)
MPD_SZE		EQU	MPD_DS+2	;SIZE OF POOL (MUST BE >= 4)
MPD_SIZE	EQU	MPD_SZE+2	;SIZE OF MPD IN BYTES.
;
;	DDD:	DISCRETE DATA DEFINITION.
;
DDD_OFF		EQU	0		;OFFSET OF DISCRETE
DDD_SEG		EQU	DDD_OFF+2	;SEGMENT OF DISCRETE
DDD_SIZE	EQU	DDD_SEG+2	;SIZE OF DDD IN BYTES.
;
;	ORD:	OVERLAY REGION DEFINITION.
;
ORD_ID		EQU	0		;REGION NUMBER.
ORD_ADDR	EQU	ORD_ID+2	;REGION ADDRESS.
ORD_LEN		EQU	ORD_ADDR+4	;REGION LENGTH IN BYTES.
ORD_SIZE	EQU	ORD_LEN+2	;SIZE OF ORD IN BYTES.
;
;	OCB:	OVERLAY REGION CONTROL BLOCK.
;
OCB_LOCK	EQU	0		;LOCK.
OCB_ORD		EQU	OCB_LOCK+4	;ADDRESS OF ORD.
OCB_WQ		EQU	OCB_ORD+4	;HEAD OF WAIT QUEUE OF RCBS
OCB_TCB		EQU	OCB_WQ+2	;CURRENTLY-RESIDENT TASK'S TCB ADDRESS
OCB_PRI		EQU	OCB_TCB+2	;PRIORITY OF RESIDENT TASK
OCB_STS		EQU	OCB_PRI+1	;0 IF DORMANT, 1 IF RUNNING.
OCB_RCB		EQU	OCB_STS+1	;RCB CONTROLLING CURRENT LOAD OPERATION
OCB_SIZE	EQU	OCB_RCB+2	;SIZE OF OCB IN BYTES.
;
;	EFG:	EVENT FLAG GROUP.
;
EFG_FLAGS	EQU	0		;ACTUAL GROUP OF 16 FLAGS.
EFG_WAIT	EQU	EFG_FLAGS+2	;LIST OF WAIT BLOCKS.
EFG_SIZE	EQU	EFG_WAIT+2	;SIZE OF EFG IN BYTES.
;
;	SEM:	SEMAPHORE CONTROLLER.
;
SEM_LOCK	EQU	0		;LOCK.
SEM_TCB		EQU	SEM_LOCK+4	;TCB ADDRESS OF CURRENT OWNER.
SEM_LEVEL	EQU	SEM_TCB+2	;LEVEL OF SEMAPHORE NESTING.
SEM_WAIT	EQU	SEM_LEVEL+2	;LIST OF SWBS.
SEM_SIZE	EQU	SEM_WAIT+2	;SIZE OF SEMAPHORE IN BYTES.
;
;	MPB:	MEMORY POOL (DYNAMIC) BLOCK.
;
MPB_LOCK	EQU	0		;LOCK.
MPB_FREEQ	EQU	MPB_LOCK+4	;LIST OF FREE BLOCKS.
MPB_WAITQ	EQU	MPB_FREEQ+2	;LIST OF PWBS.
MPB_AVAIL	EQU	MPB_WAITQ+2	;NUMBER OF FREE BYTES CURRENTLY.
MPB_SIZE	EQU	MPB_AVAIL+2	;SIZE OF MPB IN BYTES.
;
;	MBX:	MAILBOX CONTROLLER.
;
MBX_LOCK	EQU	0		;LOCK.
MBX_WAITH	EQU	MBX_LOCK+4	;HEAD OF RMB LIST.
MBX_WAITT	EQU	MBX_WAITH+2	;TAIL OF RMB LIST.
MBX_MSGSH	EQU	MBX_WAITT+2	;HEAD OF MMB LIST.
MBX_MSGST	EQU	MBX_MSGSH+2	;TAIL OF MMB LIST.
MBX_SIZE	EQU	MBX_MSGST+2	;SIZE OF MAILBOX IN BYTES.
;
;	THE EWB, MMB, RMB, SWB, PWB, TMB, IRB AND RCB DATA STRUCTURES TO
;	BE DESCRIBED SHORTLY ARE ALL BUILT IN BLOCKS OF GLOBAL RAM WHICH
;	ARE ALLOCATED AS NEEDED FROM THE SYSTEM MEMORY RESOURCE (SMR).
;	ALL SMR BLOCKS ARE 32 BYTES LONG REGARDLESS OF APPLICATION; THE
;	"LENGTHS" GIVEN FOR THE ABOVEMENTIONED DATA STRUCTURES ONLY SPECI-
;	FY HOW MUCH OF THE BLOCK THEY USE.
;
;	AVAILABILITY OF SUFFICIENT SMR BLOCKS TO COPE WITH WORST-CASE SYSTEM
;	CONDITIONS IS A GATING CONDITION FOR THE DISPATCHING OF REQUESTS FOR
;	SERVICE. FOR FURTHER DETAILS, STUDY THE SOURCE CODE OF THE SVC IN-
;	TERRUPT HANDLER IN THE MODULE MISCIN.ASM.
;
;	SIZE OF SMR BLOCK:
;
SMR_SIZE	EQU	32		;SIZE IN BYTES.
;
;	EWB:	EVENT FLAG WAIT BLOCK.
;
EWB_LINK	EQU	0		;FORWARD LINK OR 0.
EWB_PRI		EQU	EWB_LINK+2	;WAIT BLOCK PRIORITY.
EWB_TYPE	EQU	EWB_PRI+1	;TEST TYPE: 0= AND, 1= OR.
EWB_MASK	EQU	EWB_TYPE+1	;TEST MASK
EWB_TCB		EQU	EWB_MASK+2	;TCB ADDRESS OF REQUESTOR.
EWB_SIZE	EQU	EWB_TCB+2	;SIZE OF EWB IN BYTES.
;
;	MMB:	MAILED MESSAGE BLOCK.
;
MMB_LINK	EQU	0		;FORWARD LINK OR 0.
MMB_PRI		EQU	MMB_LINK+2	;PRIORITY.
MMB_COOR	EQU	MMB_PRI+1	;COORDINATION: 
					;0= NONE, 1= WAIT TILL ACCEPTED.
MMB_P1		EQU	MMB_COOR+1	;WORD 1
MMB_P2		EQU	MMB_P1+2	;WORD 2
MMB_P3		EQU	MMB_P2+2	;WORD 3 OF MESSAGE.
MMB_TCB		EQU	MMB_P3+2	;TCB OF SENDER.
MMB_SIZE	EQU	MMB_TCB+2	;SIZE OF MMB IN BYTES.
;
;	RMB:	RECEIVE-MESSAGE BLOCK.
;
RMB_LINK	EQU	0		;FORWARD LINK OR 0.
RMB_TCB		EQU	RMB_LINK+2	;TCB OF REQUESTOR.
RMB_RADR	EQU	RMB_TCB+2	;ADDRESS OF 6 BYTE BUFFER.
RMB_SIZE	EQU	RMB_RADR+4	;SIZE OF RMB IN BYTES.
;
;	SWB:	SEMAPHORE (OR CONTROLLED PROGRAM) WAIT BLOCK.
;
SWB_LINK	EQU	0		;FORWARD LINK OR 0.
SWB_PRI		EQU	SWB_LINK+2	;PRIORITY.
SWB_TCB		EQU	SWB_PRI+2	;TCB ADDRESS OF REQUESTOR.
SWB_TMB		EQU	SWB_TCB+2	;TMB ADDRESS, IF ANY.
SWB_SIZE	EQU	SWB_TMB+2	;SIZE OF SWB IN BYTES.
;
;	PWB:	POOL WAIT BLOCKS.
;
PWB_LINK	EQU	0		;FORWARD LINK OR 0.
PWB_PRI		EQU	PWB_LINK+2	;PRIORITY.
PWB_TCB		EQU	PWB_PRI+2	;TCB ADDRESS OF REQUESTOR.
PWB_LEN		EQU	PWB_TCB+2	;BLOCK SIZE NEEDED.
PWB_BUFR	EQU	PWB_LEN+2	;ADDRESS OF POOL-BLOCK-ADDRESS BUFFER
PWB_SIZE	EQU	PWB_BUFR+4	;SIZE OF PWB IN BYTES.
;
;	IRB:	I/O REQUEST BLOCK.
;
IRB_LINK	EQU	0		;FORWARD LINK OR 0.
IRB_PRI		EQU	IRB_LINK+2	;PRIORITY.
IRB_TCB		EQU	IRB_PRI+2	;TCB ADDRESS OF REQUESTOR.
IRB_UCB		EQU	IRB_TCB+2	;UCB OF REQUESTED UNIT.
IRB_TMB		EQU	IRB_UCB+2	;ADDRESS OF TMB FOR THIS REQUEST
IRB_RIO		EQU	IRB_TMB+2	;START OF SDB IMAGE:
IRB_FUNCT	EQU	IRB_RIO+1	;	FUNCTION CODE,
IRB_DEVICE	EQU	IRB_FUNCT+1	;	DEVICE NUMBER,
IRB_UNIT	EQU	IRB_DEVICE+1	;	UNIT   NUMBER,
IRB_GRP		EQU	IRB_UNIT+1	;	FLAG GROUP (ALWAYS 0),
IRB_FLAGS	EQU	IRB_GRP+1	;	FLAG MASK,
IRB_STATUS	EQU	IRB_FLAGS+2	;	ADDRESS OF STATUS DOUBLEWORD,
IRB_BUFR	EQU	IRB_STATUS+4	;	ADDRESS OF DEVICE-DEPENDENT
					;	PARAMETER LIST.
IRB_SEQ		EQU	IRB_BUFR+4	;IRB'S SEQUENCE NUMBER.
IRB_SIZE	EQU	IRB_SEQ+2	;SIZE OF IRB IN BYTES.
;
;	TMB:	TIME MONITOR BLOCK.
;
TMB_LINK	EQU	0		;FORWARD LINK OR 0.
TMB_TIME	EQU	TMB_LINK+2	;WORD-WORD MILLISECOND COUNTDOWN.
TMB_MTIME	EQU	TMB_TIME	;MIDDLE WORD, ABSOLUTE EXPIRATION TIME
TMB_LTIME	EQU	TMB_TIME+2	;LOW WORD, ABSOLUTE EXPIRATION TIME.
TMB_TYPE	EQU	TMB_TIME+4	;TMB TYPE:
					;TYPE 1= DRIVER OR TOD TIMEOUT.
					;TYPE 2= TASK PAUSE EXPIRATION.
					;TYPE 3= EVENT FLAG TO BE SET.
					;TYPE 4= TASK RESTART TIMER.
					;TYPE 5= OVERRIDE-PRIORITY TIMER.
					;TYPE 6= SEMAPHORE TIMEOUT.
					;TYPE 7= RESERVE-UNIT TIMEOUT.
TMB_EFGRP	EQU	TMB_TYPE+1	;EVENT FLAG GROUP (TYPE 3)
TMB_UCB		EQU	TMB_TYPE+2	;UCB OF UNIT (TYPE 1)
TMB_SADR	EQU	TMB_UCB+2	;ADDRESS OF TIMEOUT ROUTINE (TYPE 1)
TMB_TCB		EQU	TMB_EFGRP+1	;TCB OF REQUESTOR (TYPES 2,3,4,5,6)
TMB_EFMSK	EQU	TMB_TCB+2	;EVENT FLAG MASK (TYPE 3)
TMB_OVPRI	EQU	TMB_TCB+2	;PRIORITY-OVERRIDE INCREMENT (TYPE 5).
TMB_BPRI	EQU	TMB_OVPRI+1	;OVERRIDE-PRIORITY BOUNDARY (TYPE 5).
TMB_OVTIME	EQU	TMB_BPRI+1	;OVERRIDE TIME INTERVAL, MSEC. (T. 5).
TMB_SWB		EQU	TMB_TCB+2	;SWB ADDRESS (TYPE 6)
TMB_SEM		EQU	TMB_SWB+2	;SEMAPHORE ADDRESS (TYPE 6)
TMB_IRB		EQU	TMB_TYPE+2	;IRB ADDRESS (TYPE 7)
TMB_HTIME	EQU	TMB_SADR+4	;HIGH WORD, ABSOLUTE EXPIRATION TIME.
TMB_SIZE	EQU	TMB_HTIME+2	;SIZE OF TMB IN BYTES.
;
;	SRB:	S-STATE REQUEST BLOCK.
;
SRB_LINK	EQU	0		;FORWARD LINK OR 0.
SRB_AX		EQU	SRB_LINK+2	;CALLER'S AX
SRB_BX		EQU	SRB_AX+2	;         BX
SRB_CX		EQU	SRB_BX+2	;         CX
SRB_DX		EQU	SRB_CX+2	;         DX
SRB_BP		EQU	SRB_DX+2	;         BP
SRB_SI		EQU	SRB_BP+2	;         SI
SRB_DI		EQU	SRB_SI+2	;         DI
SRB_ES		EQU	SRB_DI+2	;         ES
SRB_ADR		EQU	SRB_ES+2	;ADDRESS OF S-STATE ROUTINE
SRB_TCB		EQU	SRB_ADR+4	;X_ATCB AT TIME OF INTERRUPT.
SRB_SIZE	EQU	SRB_TCB+2	;SIZE OF SRB IN BYTES.
;
;	RCB:	RESTART CONTROL BLOCK.
;
RCB_LINK	EQU	0		;FORWARD LINK OR 0.
RCB_PRI		EQU	RCB_LINK+2	;PRIORITY OF REQUEST AND REQUESTED TASK
RCB_CORD	EQU	RCB_PRI+1	;COORDINATION:
					;0= NO COORDINATION TO BE ENFORCED,
					;1= REQUESTOR WAITS TILL TARGET STARTS,
					;2= REQUESTOR WAITS TILL TARGET ENDS.
RCB_WTCB	EQU	RCB_CORD+1	;REQUESTOR'S TCB.
RCB_AX		EQU	RCB_WTCB+2	;STARTUP AX
RCB_BX		EQU	RCB_AX+2	;        BX
RCB_TCB		EQU	RCB_BX+2	;TCB OF TASK TO BE STARTED.
RCB_PARAM	EQU	16		;START OF QCOPY PARAMETER REGION
RCB_SIZE	EQU	SMR_SIZE	;SIZE OF RCB IN BYTES.
;
;	SIZE OF AN 8087 SAVE AREA ALLOCATED FROM X_8087_TABLE.
;
NDP_SIZE	EQU	94		;SIZE IN BYTES.
;
;	EQUATES TO THE STACK OF AN INTERRUPTED OR SUSPENDED TASK:
;
STK_ES		EQU	0
STK_DI		EQU	2
STK_DX		EQU	4
STK_CX		EQU	6
STK_DS		EQU	8
STK_SI		EQU	10
STK_BP		EQU	12
STK_BX		EQU	14
STK_AX		EQU	16
STK_IP		EQU	18	;SAVED BY INTERRUPT.
STK_CS		EQU	20	;SAVED BY INTERRUPT.
STK_FL		EQU	22	;SAVED BY INTERRUPT.
STK_TAX1	EQU	24	;FIRST COPY: TCD_AX.
STK_TBX1	EQU	26	;            TCD_BX.
STK_TAX2	EQU	28	;SECOND COPY: TCD_AX.
STK_TBX2	EQU	30	;             TCD_BX.
STK_SVC_OFST	EQU	24	;SVC TYPE 1: SDB OFFSET.
STK_SVC_SEG	EQU	26	;            SDB SEGMENT.
;
;	MISCELLANEOUS USEFUL SYMBOLS:
;
PARM_ERR	EQU	0080H	;(SF) PARAMETER ERROR IN SDB.
SVC_FAILED	EQU	0040H	;(ZF) SVC PROCESSING ABORTED.
FLAG_CF		EQU	0001H	;(CF) GENERAL-PURPOSE INDICATOR.
FLAG_OF		EQU	0800H	;(OF) FOR CERTAIN SPECIFIC SVCS.
INIT_FLAGS	EQU	0200H	;(IF) INTERRUPTS ON.
LINK_END	EQU	0FFFFH	;END OF POOL FREE BLOCK LIST.
YES		EQU	0001H
NO		EQU	0000H
TRUE		EQU	YES
FALSE		EQU	NO
;
