PAGE	60,132
INCLUDE B:EQUS.EQU
;**************************************************************
;HWDEP.ASM:	HARDWARE DEPENDENT MODULE
;		THIS MODULE CONTAINS ALL OF THE HARDWARE DEPENDENCIES
;		OF MTOS-86.  THAT INCLUDES:
;
;		X_INIT_PIC:	INTERRUPT CONTROLLER INITIALIZATION
;		X_ISSUE_EOI:	END-OF-INTERRUPT CONTROL
;		X_PIT_INIT:	CLOCK TICK INITIALIZATION
;		X_SYSOUT:	SYSTEM ERROR DISPLAY
;		...AND THE ASSOCIATED EQUATES.
;
;		NOTE THAT THIS HWDEP MODULE IS CONFIGURED TO SUPPORT THE USE OF
;		THE INTEL iSBC-86/12A SINGLE-BOARD COMPUTER, WHICH POSSESSES
;		AN 8259A INTERRUPT CONTROLLER, AN 8253 CLOCK CHIP, AND AN 8251A
;		USART FOR CONNECTING A CONSOLE DEVICE.
;
;		THE USER WHO WISHES TO PORT MTOS-86 ONTO A DIFFERENT SYSTEM
;		MUST CHANGE EACH OF THESE ROUTINES ACCORDINGLY   
;**************************************************************
;	
		NAME	HWDEP
;
MTOS_GROM	SEGMENT	PARA PUBLIC
MTOS_GROM	ENDS
;
		EXTRN	X_SYS_DEV:ABS	;SYSTEM CONSOLE DEVICE NUMBER.
		EXTRN	X_SYS_UNIT:ABS	;SYSTEM CONSOLE UNIT NUMBER.
;
MTOS_CS		SEGMENT	PARA	PUBLIC
;
		ASSUME	CS:MTOS_CS
;********************************************************
;*              REQUIRED ROUTINES                       *
;********************************************************
		PUBLIC	X_PIT_INIT	;INITIALIZE MTOS CLOCK INTERRUPT.
		PUBLIC	X_ISSUE_EOI	;CLEAR THE MOST RECENT INTERRUPT.
		PUBLIC	X_INIT_PIC	;INITIALIZE THE INTERRUPT CONTROLLER.
		PUBLIC	X_SYSOUT	;EMERGENCY CONSOLE OUTPUT ROUTINE.
;********************************************************
;*              REQUIRED PUBLIC SYMBOLS                 *
;********************************************************
		PUBLIC	X_PIT_MS	;NR. OF MILLISECONDS PER CLOCK TICK.
;********************************************************
;*              LINKAGE TO MTOS SUPPORT                 *
;********************************************************
		EXTRN	X_DEVDATA:FAR	;FOR CALCULATING DCB AND UCB ADDRESSES
;******************
;8253 PIT EQUATES
;******************
;
X_PIT_MS	EQU	10	;NUMBER OF MILLISECONDS BETWEEN CLOCK TICKS.
PIT_RATE	EQU	1229	;NUMBER OF MINOR CLOCK CYCLES PER MILLISECOND.
				;(PIT IS DERIVED OFF OF 1.2288 MHZ CLOCK)
X_PIT_CMD	EQU	0D6H	;COMMAND PORT ADDRESS OF 8253 PIT.
X_PIT_DATA	EQU	0D0H	;COUNTER (DATA - TIMER 2) PORT OF 8253 PIT.
X_PIT_COUNT	EQU	PIT_RATE*X_PIT_MS	;INIT COUNT FOR THE CLOCK.
;
;******************
;8259 PIC EQUATES
;******************
;
X_PIC_BASE	EQU	20H	;LOWEST VECTOR NUMBER ASSIGNED TO PIC.
X_PIC_EOI	EQU	20H	;SOFT END-OF-INTERRUPT COMMAND TO PIC.
X_PIC		EQU	0C0H	;PIC PORT BASE.
;
;******************************************
;*     X_PIT_INIT:    PIT INITIALIZATION  *
;******************************************
;
;		THIS ROUTINE INITIALIZES AN INTEL 8253 PIT, COUNTER CHANNEL 0,
;	FOR THE PROGRAMMABLE SQUARE-WAVE MODE AT THE INTERRUPT FREQUENCY SPE-
;	CIFIED BY X_PIT_COUNT.
;
;	THIS ROUTINE NEED ONLY SAVE REGISTERS DS, ES AND SS. IT IS
;	CALLED ONLY ONCE, AT SYSTEM INITIALIZATION.
;
X_PIT_INIT	PROC	NEAR
		MOV	AL,034H			;PIT_0= INTERNAL RATE GENERATOR
		MOV	DX,X_PIT_CMD
		OUT	DX,AL		 	;FOR THE REAL-TIME INTERRUPT
		MOV	AX,X_PIT_COUNT		;GET THE TIME INTERVAL
		MOV	DX,X_PIT_DATA
		OUT	DX,AL			;AND LOAD THE CLOCK
		MOV	AL,AH			;MSB OF THE TIME INTERVAL
		OUT	DX,AL			;CLOCK IS READY NOW.
		RET
X_PIT_INIT	ENDP
;
;*****************************************************************
;*	X_INIT_PIC  -  INITIALIZE PROGRAMMABLE INTRP CONTROLLER  *
;*****************************************************************
;
;	THIS ROUTINE INITIALIZES AN INTEL 8259A PIC AS AN EDGE-TRIGGERED
;	MASTER WITH NO SLAVES, OPERATING WITH SOFTWARE-CONTROLLED END-OF-
;	INTERRUPT. IT ASSIGNS AN INTERRUPT VECTOR BASE NUMBER FROM THE
;	SYMBOL X_PIC_BASE, AND ENABLES ALL EIGHT INTERRUPT CHANNELS (THE
;	PIC "IRn" LINES). AFTER THE PIC HAS BEEN INITIALIZED IN THIS MAN-
;	NER, IR0 WILL PRODUCE AN INTERRUPT OF VECTOR X_PIC_BASE, IR1 WILL
;	PRODUCE AN INTERRUPT OF VECTOR X_PIC_BASE+1, AND SO ON.
;
;	THIS ROUTINE NEED ONLY SAVE REGISTERS DS, ES AND SS. IT IS CALLED ONLY
;	ONCE, AT SYSTEM INITIALIZATION TIME.
;
X_INIT_PIC	PROC	NEAR
		MOV	AL,013H			;SINGLE PIC, EDGE TRIGGERED
		MOV	DX,X_PIC
		OUT	DX,AL			;ICW1
		ADD	DX,2			;ADDR SECOND PIC PORT (PC)
		MOV	AX,X_PIC_BASE		;VECTOR BASE
		OUT	DX,AL			;ICW2
		MOV	AL,09H			;SLAVE (PC ONLY),NESTED,
						;NO AUTO-EOI
		OUT	DX,AL			;ICW4
		MOV	AL,000H			;ENABLE ALL LEVELS
		OUT	DX,AL			;MASK OUTPUT
		RET
X_INIT_PIC	ENDP
;
;***********************************************************************
;*X_ISSUE_EOI:	ISSUE AN EOI TO THE 8259 INTERRUPT CONTROLLER          *
;***********************************************************************
;
;	THIS ROUTINE MUST DO WHATEVER IS NECESSARY TO CLEAR THE MOST RECENT
;	INTERRUPT FROM THE INTERRUPT CONTROL CIRCUITRY. NORMALLY, THAT INVOLVES
;	ONLY SENDING A DISMISS-LAST-INTERRUPT COMMAND TO AN 8259A PIC, AS HERE.
;
;	THIS ROUTINE MUST SAVE ANY REGISTERS IT NEEDS BEFORE USING THEM. IT IS
;	CALLED AT EVERY INTERRUPT-DISMISSAL IN MTOS.
;
X_ISSUE_EOI	PROC	NEAR
		PUSH	AX
		PUSH	DX
 		MOV	DX,X_PIC
		MOV	AX,X_PIC_EOI	;ISSUE AN INTERRUPT CLEARANCE
		OUT	DX,AL		;TO THE 8259A.
		POP	DX
		POP	AX
		RET
X_ISSUE_EOI	ENDP
;
;*************************************************
;*    X_SYSOUT:      EMERGENCY CONSOLE OUTPUT    *
;*************************************************
;
;	INPUT:	CS:BX=	ADDRESS OF THE STRING TO PRINT
;		(NOTE: ALL CARRIAGE CONTROL MUST BE INCLUDED IN STRING.)
;		CX=	NUMBER OF CHARACTERS TO PRINT.
;		(NOTE: A NULL WILL ALSO TERMINATE OUTPUT.)
;
;	NOTE THAT THIS ROUTINE ASSUMES THAT THE CONSOLE'S UCD IS LAID OUT AS IS
;	THAT OF A UNIT FOR THE STANDARD MTOS-86 CONSOLE DRIVER.
;
;	THIS ROUTINE MUST SAVE ANY REGISTERS IT NEEDS BEFORE USING THEM, EXCEPT
;	FOR AX AND CX. IT IS ONLY CALLED IF:
;		-THE SYSTEM RUNS OUT OF SRBS AND ANOTHER INTERRUPT HANDLER
;		 NEEDS TO GO TO S STATE;
;		-THE SYSTEM RUNS OUT OF SMR BLOCKS IN THE MIDDLE OF PROCESSING
;		 A SERVICE CALL.
;
X_SYSOUT	PROC	NEAR
		PUSH	BX
		PUSH	DX
		PUSH	SI
		PUSH	ES
		MOV	AX,MTOS_GROM	;ADDRESS THE DATA
		MOV	ES,AX		;IN MTOS_GROM.
		PUSHF			;SAVE FLAGS REGISTER AND
		CLI			;DISABLE INTERRUPTS.
		MOV	AX,X_SYS_DEV	;GET DEVICE NUMBER OF CONSOLE
		CMP	AL,0FFH		;ANY CONSOLE DEVICE?
		JE	@3		;NO: DO NOTHING.
		MOV	DX,X_SYS_UNIT	;GET UNIT NUMBER OF CONSOLE
		CMP	DL,0FFH		;ANY CONSOLE UNIT?
		JE	@3		;NO: DO NOTHING.
		PUSH	BX		;WE NEED THIS REGISTER LATER
		CALL	X_DEVDATA	;CALCULATION OF THE UCB/DCB ADDRS
		MOV	SI,[BX+UCB_UCD]	;GET UCD ADDRESS FOR PORT INFO.
		POP	BX		;RESTORE ADDRESS OF STRING
		XOR	DH,DH		;ZERO TOP HALF.
@1:		MOV	DL,ES:[SI+UCD_PORT+4]	;CONTROL ADDRESS
@2:		IN	AL,DX		;GET THE PORT STATUS
		TEST	AL,01H		;XMTR READY?
		JZ	@2		;LOOP BACK WHEN IT ISN'T
		MOV	DL,ES:[SI+UCD_PORT+7]	;DATA ADDRESS
		MOV	AL,CS:[BX]	;GET THE CHARACTER
		INC	BX
		TEST	AL,AL		;END-OF-STRING?
		JE	@3		;SKIP OUT WHEN SO TERMINATED.
		OUT	DX,AL		;PRINT ONE CHARACTER
		LOOP	@1		;CONTINUE TILL AL OUT.
@3:		POPF			;TURN THE LIGHTS BACK ON.
		POP	ES
		POP	SI
		POP	DX
		POP	BX
		RET
X_SYSOUT	ENDP
MTOS_CS		ENDS
		END
