PAGE	60,132
INCLUDE B:BIG.INC
;
;	INIT.ASM:	INITIALIZATION PROCEDURE FOR MTOS-86 LEVEL II.
;			WITH ALTERATIONS FOR THE PC
;
;	F. W. PORRETTO, 6/15/82
;	J.S.WHITE 12/1/83
;
;	THIS MODULE CONTAINS THE INITIALIZATION MAINLINE, WHICH THE CONFIGURER
;	MUST ARRANGE TO BE EXECUTED AT ANY SYSTEM RESET OR POWER TRANSITION.
;	THE MAINLINE CALLS UPON OTHER ROUTINES, SOME OF WHICH ARE IN INIT.ASM
;	AND SOME OF WHICH ARE DEFINED EXTERNALLY, TO INITIALIZE ALL THE GLOBAL
;	MTOS DATA STRUCTURES, THE PIC AND PIT, AND ANY OTHER INITIALIZABLE
;	DEVICES. AT THE CONCLUSION OF THE INITIALIZATION PROCEDURE, THE 
;	CPU INITIATES THE INITIALIZATION TASK, AFTER WHICH IT JUMPS TO ITS
;	SCHEDULER ENTRY POINT. NEVER AGAIN AFTER SYSTEM INITIALIZATION IS THIS
;	MODULE'S CODE EXECUTED.
;
		NAME	MTOS_INITIALIZATION
;
MTOS_CS		SEGMENT	PARA PUBLIC
		ASSUME	CS:MTOS_CS,DS:MTOS_DS,SS:MTOS_DS
		PUBLIC	XMTOS86			;INITIALIZATION ENTRY POINT.
;
		EXTRN	X_INIT_SRB:NEAR		;INITIALIZE THE SRB FREE LIST.
		EXTRN	X_INIT_PIC:NEAR		;INITIALIZE PIC(S).
		EXTRN	X_PIT_INIT:NEAR		;INITIALIZE THE SYSTEM CLOCK.
		EXTRN	XDEVDATA:FAR		;SETUP LOCAL DRIVER VARIABLES.
		EXTRN	X_INIT_TASK:NEAR	;INITIALIZE TASK STACK, ETC.
		EXTRN	X_ALLOC_8087:NEAR	;ALLOCATE 8087 SAVE AREA.
		EXTRN	X_UNEXPECTED:NEAR	;UNEXPECTED INTERRUPT HDLR.
		EXTRN	X_GET_TCB_ADR:NEAR	;GET TCB ADDRESS
		EXTRN	X_START_TASK:NEAR	;START TASK
		EXTRN	X_SCHDLR:NEAR		;SCHEDULER ENTRY POINT
		EXTRN	X_GET_SMR:NEAR		;ALLOCATE SMR MEMORY BLOCK.
PAGE
;**********************************************
;*	INTERRUPT VECTOR INITIALIZATION.      *
;**********************************************
;
INIT_VECTORS	PROC	NEAR
		PUSH	DS		;WE'LL BE USING THIS.
		XOR	DI,DI		;AND WE'LL BE WORKING IN
		MOV	DS,DI		;PARAGRAPH 0.
		MOV	CX,256		;THIS MANY ENTRIES IN VECTOR REGION.
		MOV	BX,OFFSET X_UNEXPECTED	;PREPARE!
INITV00:
		MOV	[DI],BX		;X_UNEXPECTED GOES INTO EVERY VECTOR,
		MOV	[DI+2],CS	;TO COVER THE ONES TO WHICH NEITHER A
		ADD	DI,4		;DEVICE UNIT NOR A NONDEVICE VECTOR
		LOOP	INITV00		;ASSIGNS A USE.
;
;	NOW PROCESS THE VECTOR TABLE.
;
INITV0A:
		MOV	SI,OFFSET X_VECTOR_TABLE
INITV01:
		MOV	DI,ES:[SI]	;GET VECTOR NUMBER OF NEXT ENTRY.
		CMP	DI,0FFFFH	;END OF TABLE?
		JE	INITV02		;YES.
		ADD	DI,DI		;NO:
		ADD	DI,DI		;MAKE DI= OFFSET OF VECTOR POINTER
		MOV	AX,ES:[SI+2]	;GET OFFSET OF HANDLER ADDRESS
		MOV	[DI],AX		;LAY IT DOWN...
		MOV	AX,ES:[SI+4]	;GET SEGMENT OF HANDLER ADDRESS
		MOV	[DI+2],AX	;LIKEWISE.
		ADD	SI,6		;ONWARD.
		JMP	INITV01
;
;	FINALLY, GO THROUGH ALL THE UNITS OF ALL THE DEVICES, AND BUILD THE
;	VECTOR POINTERS FOR THEM.
;
INITV02:
		MOV	CX,X_UNIT_NUM
		JCXZ	INITV03			;IT'S LEGAL TO HAVE NO DRIVERS.
		MOV	SI,OFFSET X_UCD_TABLE	;START FROM TOP OF TABLE.
INITV02L:
		MOV	AL,ES:[SI+UCD_INTNO]	;GET VECTOR NUMBER
		XOR	AH,AH
		MOV	DI,AX			;IN 16-BIT FORM TO DI.
		TEST	DI,DI			;ZERO?
		JE	INITV02LB		;IF SO, NO INTERRUPT FOR UNIT.
		ADD	DI,DI			;IF NOT, MAKE VECTOR NUMBER
		ADD	DI,DI			;INTO OFFSET INTO PARAGRAPH 0.
		MOV	AX,ES:[SI+UCD_INTADR]	;GET HANDLER OFFSET
		MOV	[DI],AX			;LAY IT DOWN...
		MOV	AX,ES:[SI+UCD_INTADR+2]	;THEN HANDLER SEGMENT,
		MOV	[DI+2],AX		;LIKEWISE.
INITV02LB:
		ADD	SI,UCD_SIZE		;ADVANCE UCD POINTER
		LOOP	INITV02L		;AND GO AROUND AGAIN, MAYBE.
INITV03:
		POP	DS			;ALL DONE.
		RET
;
INIT_VECTORS	ENDP
;
;***********************************************
;*	INITIALIZE THE 8087 FREE STACK.        *
;***********************************************
;
INIT_8087	PROC	NEAR
		MOV	DI,OFFSET X_8087_TABLE	;ADDRESS OF 8087 REGION.
		MOV	X_8087_STACK,DI		;INITIALIZE QUEUE HEAD.
		MOV	CX,X_8087_NUM		;NUMBER OF FREE AREAS.
		JCXZ	IN872			;** NONE **
		DEC	CX			;ONE OR MORE.
IN870:
		JCXZ	IN871			;DONE WITH 8087 FREE AREAS.
		LEA	SI,[DI+NDP_SIZE]	;DETERMINE FORWARD LINK
		MOV	[DI],SI			;LINK NEXT ENTRY TO CURRENT
		MOV	DI,SI			;FORWARD ONE MORE ENTRY
		LOOP	IN870			;AND DO IT AGAIN.
IN871:
IN872:
		RET
INIT_8087	ENDP
;
;*******************************************
;*	INITIALIZE ALL STATIC TCBS.        *
;*******************************************
;
INIT_TCBS	PROC	NEAR
		MOV	CX,X_STATIC_TASKS
		MOV	BX,OFFSET X_TCB_TABLE
		MOV	DI,OFFSET X_TCD_TABLE
INITT0:
		MOV	[BX+TCB_TCD],DI		;HOOK EACH TCB TO ITS TCD
		MOV	[BX+TCB_TCD+2],ES	;BY DOUBLEWORD ADDRESS
		MOV	AL,ES:[DI+TCD_FLAGS]	;GET TASK FLAGS
		AND	AL,NOT TFL_ALLOC	;INSURE STATIC CHARACTERISTIC
		MOV	[BX+TCB_FLAGS],AL
		TEST	AL,TFL_8087		;IS TASK AN 8087 USER?
		JE	INITT3			;NO.
		CALL	X_ALLOC_8087		;GET AN 8087 SAVE AREA
		MOV	[BX+TCB_8087],BP	;AND LINK TCB TO IT.
INITT3:
		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_OVPRI	;PRIORITY OVER-
						;RIDE SPECIFIED?
		JE	INITT2			;NO, NO OVERRIDE.
INITT1:
		MOV	SI,BX
		CALL	X_GET_SMR		;YES: ALLOCATE AN SMR BLOCK
		XCHG	BX,SI			;FOR OVERRIDE TMB.
		MOV	[BX+TCB_TMB],SI
INITT2:
		ADD	BX,TCB_SIZE		;GO TO NEXT TASK'S
		ADD	DI,TCD_SIZE		;DATA STRUCTURES.
		LOOP	INITT0
		RET
;
INIT_TCBS	ENDP
;
;*****************************************************
;*	INITIALIZE ALL OVERLAY CONTROL BLOCKS.       *
;*****************************************************
;
INIT_OCBS	PROC	NEAR
		MOV	SI,OFFSET X_ORD_TABLE
		MOV	BX,OFFSET X_OCB_TABLE
		MOV	CX,X_OVERLAYS_NUM	;ARE THERE ANY OVERLAYS?
		JCXZ	IOCB9			;NO.
IOCB1:
		MOV	[BX+OCB_ORD],SI		;INSERT DWORD ADDRESS OF THE
		MOV	[BX+OCB_ORD+2],ES	;ORD.
		ADD	BX,OCB_SIZE
		ADD	SI,ORD_SIZE
		LOOP	IOCB1
IOCB9:
		RET
INIT_OCBS	ENDP
;
; ***********************************************************
; *            INIT_POOLS - MEMORY POOLS INITIALIZATION     *
; ***********************************************************
;
INIT_POOLS	PROC	NEAR
		MOV	CX,X_MEM_POOL_NUM		;NUMBER OF MEMORY POOLS
		JCXZ	NO_POOLS			;** NONE IN SYSTEM **
		MOV	DI,OFFSET X_MPD_TABLE		;GET ADDRESSES OF THE
		MOV	BX,OFFSET X_MPB_TABLE		;POOL DATA STRUCTURES
NEXT_POOL:
		PUSH	ES
		MOV	DX,ES:[DI+MPD_SZE]		;GET POOL SIZE, BYTES.
		LES	SI,DWORD PTR ES:[DI+MPD_OFF]	;ADR OF THE POOL START
		MOV	WORD PTR [BX+MPB_FREEQ],SI	;START ADR TO FREE Q
		MOV	WORD PTR ES:[SI],LINK_END	;SET END OF LINK IN BLK
		MOV	WORD PTR ES:[SI+2],DX		;COUNT INTO BLOCK
		MOV	WORD PTR [BX+MPB_AVAIL],DX	;AVAILABLE COUNT TO MAX
		ADD	BX,MPB_SIZE			;NEXT ENTRY
		ADD	DI,MPD_SIZE			;NEXT ENTRY
		POP	ES
		LOOP	NEXT_POOL			;DO UNTIL POOL COUNT 0
NO_POOLS:
		RET
INIT_POOLS	ENDP
;
; **********************************************************************
; *             INIT_DCBS - DCB/UCB INITIALIZATION                     *
; **********************************************************************
;
INIT_DCBS	PROC	NEAR
		MOV	DX,X_DRIVER_NUM			;NUMBER OF DCB'S
		MOV	DH,DL
		TEST	DH,DH				;ANY DCB'S ?
		JE	DONE_DCBS			; --NO--
		XOR	DL,DL				;DL=0
		MOV	BX,OFFSET X_DCD_TABLE		;TABLE OF DCD'S
		MOV	DI,OFFSET X_DCB_TABLE		;TABLE OF DCB'S
		MOV	SI,OFFSET X_UCB_TABLE		;TABLE OF UCB'S
NEXT_DEV:	
		MOV	CH,BYTE PTR ES:[BX+DCD_UNITS]	;NO. OF DEVICE UNITS
		XOR	CL,CL				;CL=0 FOR UNIT 0
;
;	REGISTERS: BX=DCD  CL=UNIT #  CH=UNIT COUNT  DI=DCB  SI=UCB
;		   AX=UCD  DL=DEV  #  DH=DEV  COUNT
;
		MOV	BYTE PTR [DI+DCB_ID],DL		;DEV # IN DCB
		MOV	WORD PTR [DI+DCB_DCD],BX	; PUT DCD INTO DCB
		MOV	WORD PTR [DI+DCB_UCB],SI	; PUT UCB INTO DCB
		MOV	AX,ES:[BX+DCD_UCD]		;UCD ADDR - FIRST
NEXT_UCB:
		MOV	BYTE PTR [SI+UCB_UNIT],CL	;UNIT # IN UCB
		MOV	WORD PTR [SI+UCB_DCB],DI	;DCB INTO UCB
		MOV	WORD PTR [SI+UCB_UCD],AX	;UCD INTO UCB
		ADD	AX,UCD_SIZE			;NEXT UCD
		INC	CL				;NEXT UNIT #
		CMP	CL,CH				;DONE WITH UNITS
		JE	NEXT_DCB			; --YES--
		PUSH	AX				;SAVE AX 
		MOV	AX,SI				;SI = CURRENT UCB
		ADD	AX,UCB_SIZE			;AX = NEXT UCB
		MOV	[SI+UCB_LINK],AX		;LINK TO NEXT UCB
		MOV	SI,AX				;NEXT UCB NOW CURRENT
		POP	AX				;RESTORE AX
		JMP	SHORT NEXT_UCB			;PROCESS THE UCB
;
;	NEXT DEVICE BLOCK
;
NEXT_DCB:
		ADD	SI,UCB_SIZE			;NEXT UCB
		ADD	BX,DCD_SIZE			;NEXT DCD
		INC	DL				;NEXT DEVICE #
		CMP	DL,DH				;MORE DEVICES ?
		JE	DONE_DCBS			; --NO--
		MOV	CX,DI				;YES: DI = CURRENT DCB
		ADD	CX,DCB_SIZE			;CX = NEXT DCB
		MOV	[DI+DCB_LINK],CX		;SET CURRENT LINK= NEXT
		MOV	DI,CX				;SET NEXT DCB= CURRENT
		JMP	SHORT NEXT_DEV			;PROCESS NEXT DCB
DONE_DCBS:	RET
INIT_DCBS	ENDP
;
;*****************************************************
;*	INITIALIZE THE SYSTEM MEMORY RESOURCE        *
;*****************************************************
;
;	THIS ROUTINE LINKS THE BLOCKS ALLOCATED BY THE CONFIGURER INTO THE 
;	SMR FREE BLOCK LIST, WHOSE HEAD IS X_SMR_LIST.
;
INIT_SMR	PROC	NEAR
;
		MOV	SI,OFFSET X_SMR		;GET START OF REGION
		MOV	DI,OFFSET X_SMR_LIST	;GET ADDRESS OF HEAD
		MOV	CX,X_SMR_NUM 		;GET NUMBER OF BLOCKS .
SMR_LOOP:
		MOV	[DI],SI			;LINK CURRENT TO PREVIOUS
		MOV	DI,SI			;CURRENT BECOMES PREVIOUS
		ADD	SI,SMR_SIZE		;ADVANCE TO NEXT
		LOOP	SMR_LOOP		;DO IT AGAIN.
		MOV	X_SMR_AVAIL,X_SMR_NUM	;FINALLY, DECLARE THE NR. FREE.
		RET
INIT_SMR	ENDP
PAGE
;***************************************
;*	INITIALIZATION MAINLINE.       *
;***************************************
;
;		THIS SECTION OF CODE IS EXECUTED ONCE PER PROCESSOR IN THE
;	LIFE (I.E., FROM RESET TO RESET) OF AN MTOS-86 LEVEL II SYSTEM, AND
;	THEN NEVER AGAIN. IT ATTENDS TO ALL INITIALIZATION NECESSITIES WITH-
;	IN THE SYSTEM, INCLUDING THOSE OF DEVICE DRIVERS.
;
XMTOS86		PROC	FAR
		CLI			;JUST IN CASE.
		MOV	AX,MTOS_DS
		MOV	SS,AX		;STACK ALWAYS IN MTOS_DS.
		MOV	SP,X_MTOS_SP
		MOV	DS,AX		;NORMALLY DS= GLOBAL RAM.
		MOV	AX,MTOS_GROM
		MOV	ES,AX		;WE NEED ES= ROM TO START OFF.
INIT01:
		XOR	AL,AL		;WE'LL START BY ZEROING
		MOV	CX,X_GINIT_SIZE	;THE GLOBAL RAM REGION
		XOR	DI,DI		;ALL SEGMENTS ARE PARAGRAPH-ALIGNED.
INIT01L:
		MOV	[DI],AL
		INC	DI
		LOOP	INIT01L		;PROCEED UNTIL FINISHED.
		MOV	X_ROUND_ROBIN,X_RR_TICKS
					;START ROUND ROBIN COUNTER.
		CALL	INIT_VECTORS	;INITIALIZE CPU'S INTERRUPT VECTORS
		CALL	X_INIT_SRB	;INITIALIZE SRB FREE LIST.
		CALL	INIT_SMR	;INITIALIZE SMR SYSTEM.
		CALL	INIT_POOLS	;INITIALIZE ALL POOLS.
		CALL	INIT_8087	;MAKE THE 8087 FREE STACK.
		CALL	INIT_TCBS	;INITIALIZE ALL TCBS.
		CALL	INIT_OCBS	;INITIALIZE ALL OVERLAY CONTROLS.
		CALL	INIT_DCBS	;INITIALIZE ALL DCBS AND UCBS.
		CALL	X_INIT_PIC	;INITIALIZE PIC(S)
		CALL	X_PIT_INIT	;INITIALIZE PITS.
;
;
;	NOW THE DRIVER SYSTEM:
;
		MOV	CX,X_DRIVER_NUM
		JCXZ	INIT20		;NO DRIVERS: RARE BUT LEGAL.
		XOR	CX,CX
INIT12:
		MOV	XDEVNUM,CH		;DRIVER NUMBER
		MOV	XDEVUNIT,CL		;UNIT NUMBER
		CALL	FAR PTR XDEVDATA	;SET UP XDCB, XUCB.
		MOV	SI,XUCB			;GET UCB ADDRESS
		PUSH	ES			;YES:
		PUSH	DS
		PUSH	CX
		PUSH	SI
		MOV	SI,XDCB			;GO THROUGH THE DCB
		MOV	DI,[SI+DCB_DCD]		;TO GET THE DCD
		PUSH	DI			;SAVE THAT.
		CALL	DWORD PTR ES:[DI+DCD_INIT]	;CALL INIT ROUTINE.
		POP	DI
		POP	SI
		POP	CX
		POP	DS
		POP	ES
INIT13:
		INC	CL			;ADVANCE UNIT NUMBER
		CMP	CL,ES:[DI+DCD_UNITS]	;TOPPED OUT ON THIS DEVICE?
		JNE	INIT12			;NO, KEEP GOING.
		XOR	CL,CL			;YES: GO TO UNIT 0 OF
		INC	CH			;NEXT DEVICE,
		MOV	AX,X_DRIVER_NUM
		CMP	CH,AL			;IF ANY...
		JNE	INIT12			;THERE IS.
;
;	DRIVER SYSTEM INITIALIZED: ONCE AGAIN, EVERYBODY BUT THE MASTER 
;	BACKS OFF FOR A FEW CYCLES, WHILE THE MASTER INITIALIZES THE STARTUP
;	TASK, IF ANY.
;
INIT20:
		MOV	AX,X_INIT_ID	;GET TASK ID...
		CMP	AX,X_STATIC_TASKS	;WAS ONE DESIGNATED?
		JAE	INIT30		;NO.
		CALL	X_GET_TCB_ADR	;GET TCB ADDRESS
		MOV	DI,BX		;OF TARGET TASK.
		XOR	SI,SI		;NO REQUESTOR.
		LES	BX,DWORD PTR [BX+TCB_TCD]	;GET TCD ADDRESS.
		MOV	CL,ES:[BX+TCD_INPRI]	;GET INHERENT PRIORITY
		XOR	CH,CH		;NO COORDINATION POSSIBLE!
		XOR	BP,BP		;AUTO ALLOCATE.
		XOR	AX,AX		;CLEAR OUT THE TWO WORD
		XOR	BX,BX		;STARTUP ARGUMENT.
		CALL	X_START_TASK	;START THE TASK.
INIT30:
		JMP	X_SCHDLR	;GO!!
XMTOS86		ENDP
MTOS_CS		ENDS
		END	XMTOS86
