PAGE	60,132
INCLUDE B:BIG.INC
;
;	MAIL.ASM:	MAILBOX CONTROL ROUTINES.
;	F. W. PORRETTO, 12/15/82.
;
;	INCLUDED IN THIS MODULE ARE THE STANDARD PROCESSES FOR LOCAL MESSAGE
;	MAILING AND DELIVERY, AND THE EXTENSIONS NECESSARY TO SUPPORT A
;	MAILBOX-BASED MTOS-86 NETWORKING SCHEME.
;
		NAME	MAILBOX_SERVICES
;
		PUBLIC	X_SEND,X_RECV,X_SEND_MSG,X_RECV_MSG
;
MTOS_CS		SEGMENT PARA PUBLIC
		ASSUME	DS:MTOS_DS,CS:MTOS_CS,SS:MTOS_DS
		EXTRN	X_QUEUE		:NEAR
		EXTRN	X_DQUEUE	:NEAR
		EXTRN	X_SET_READY	:NEAR
		EXTRN	X_TASK_STATE	:NEAR
		EXTRN	X_GET_TCB_ADR	:NEAR
		EXTRN	X_START_TASK	:NEAR
		EXTRN	X_SFLAGS	:NEAR
		EXTRN	X_GET_SMR	:NEAR	;GET SMR BLOCKS
		EXTRN	X_PUT_SMR	:NEAR	;FREE SMR BLOCKS
		EXTRN	X_GET_INHPRI	:NEAR	;GET INHERENT TASK PRIORITY
;
;	LOCAL EQUATES
;
WT		EQU	01H		;SDB OFFSET OF WAIT OPTION
PRIS		EQU	01H		;              PRIORITY OPTION
PRIV		EQU	02H		;              PRIV PRIORITY
NODE		EQU	03H		; 	       NODE ID
BOX		EQU	04H		;	       MAILBOX ID
BUFR		EQU	05H		;              MESSAGE BUFFER ADDRESS
WT_MASK		EQU	80H		;ISOLATE THE WT OPTION
PBLOCK_MASK	EQU	40H		;ISOLATE THE PBLOCK OPTION
PRIS_MASK	EQU	03H		;ISOLATE THE PRIORITY OPTION
;
;	FOR USE IN THE MMB_COOR FIELD OF THE MMB:
;
SENDER_BLOCKED	EQU	00000010B	;SENDER BLOCKED UNTIL DELIVERY.
OVFLO_ON	EQU	00000001B	;SET RECEIVER'S OVERFLOW FLAG.
PAGE
;*********************************************************************
;*        XFIND_MB:    FIND MAILBOX CONTROL BLOCK.                   *
;*********************************************************************
;
; 	THIS PROCEDURE FINDS THE MAILBOX CONTROL BLOCK IN THE TABLE.
;
;	INPUT:	DL=	MAILBOX ID
;	OUTPUT:	DI=	ADDRESS OF MAILBOX DATA BLOCK
;
XFIND_MB	PROC	NEAR
		PUSH	DX
		MOV	AX,MBX_SIZE
		MUL	DL
		ADD	AX,OFFSET X_MBX_TABLE
		MOV	DI,AX
		POP	DX
		RET
XFIND_MB	ENDP
PAGE
;*********************************************************************
;*           XBUILD_MMB:    BUILD A MAILED MESSAGE BLOCK.            *
;*********************************************************************
;
;	THIS PROCEDURE ALLOCATES AND BUILDS AN MMB.
;
;	INPUT:	SI=	TCB ADDRESS
;		CH=	PRIV
;		CL=	PRIS
;		DH=	WAIT PARAMETER AND OVERFLOW FLAG.
;		ES:DI=	POINTER TO MESSAGE
;
XBUILD_MMB	PROC	NEAR
		CALL	X_GET_SMR		;ALLOCATE SMR BLOCK.
		CMP	CL,02			;HOW IS PRIORITY SELECTED?
		JG	INHPRI			;CALLER'S INHERENT.
		JL	PRIVPRI			;NO, ABSOLUTE SDB VALUE.
		MOV	AL,[SI+TCB_PRI]		;NO, CALLER'S CURRENT!
		JMP	MMBNEXT
INHPRI:
		XCHG	BX,SI			;BX = TCB
		CALL	X_GET_INHPRI		;AL = INHER PRI
		XCHG	BX,SI
		JMP	MMBNEXT
PRIVPRI:
		MOV	AL,CH
MMBNEXT:
		MOV	[BX+MMB_PRI],AL		;INSERT SELECTED PRIORITY.
		MOV	[BX+MMB_COOR],DH	;SET COORD, OVFLO FLAG.
		MOV	AX,ES:[DI]		;MESSAGE MUST BE MOVED
		MOV	[BX+MMB_P1],AX		;LITERALLY INTO THE MMB.
		MOV	AX,ES:[DI+2]		;SINCE IT'S SIX BYTES UNDER
		MOV	[BX+MMB_P2],AX		;ALL CASES, THIS IS NOT TOO
		MOV	AX,ES:[DI+4]		;DIFFICULT TO ARRANGE.
		MOV	[BX+MMB_P3],AX		;THAT'S THAT.
		MOV	[BX+MMB_TCB],SI		;INSERT SENDER'S TCB ADDR
		RET
XBUILD_MMB	ENDP
PAGE
;*********************************************************************
;*            XBUILD_RMB:       BUILD A RECEIVE-MESSAGE BLOCK.       *
;*********************************************************************
;
;	THIS PROCEDURE ALLOCATES AND BUILDS AN RMB.
;
;	INPUT:	SI=	TCB ADDRESS
;		ES:DI=	ADDRESS OF MESSAGE BUFFER
;	OUTPUT:	BX=	RMB
;
XBUILD_RMB	PROC	NEAR
		CALL	X_GET_SMR		;ALLOCATE AN SMR BLOCK.
		MOV	[BX+RMB_TCB],SI		;TCB ADDR
		MOV	[BX+RMB_RADR],DI	;OFFSET MSG BUFR
		MOV	[BX+RMB_RADR+2],ES	;SEGMENT OF MSG BUFR
		RET
XBUILD_RMB	ENDP
PAGE
;*********************************************************************
;*      XMOVE_MSG:      TRANSFER A MESSAGE FROM SENDER TO RECEIVER.  *
;*********************************************************************
;
;	THIS PROCEDURE TRANSFERS THE MESSAGE FROM THE MMB
;	TO THE BUFFER WHOSE ADDRESS IS IN THE RMB. ALSO, IF THE MMB SPECIFIES
;	THAT THE RECEIVER'S OVERFLOW FLAG BE SET UPON DELIVERY, THAT IS DONE
;	AT THIS POINT.
;
;	INPUT:	SI=	RMB ADDRESS
;		DI=	MMB ADDRESS
;	OUTPUT:	NONE
;
XMOVE_MSG	PROC	NEAR
		PUSH	ES
		PUSH	BX
		PUSH	CX
		PUSH	DI
		PUSH	SI
		XCHG	SI,DI			;SI=MMB, DI=RMB!
		LES	DI,DWORD PTR [DI+RMB_RADR]	;GET BUFFER ADDRESS
		ADD	SI,MMB_P1		;GET MESSAGE ADDRESS
		MOV	CX,3			;WE'LL MOVE THREE WORDS
	REP	MOVSW				;FROM SENDER TO RECEIVER.
		POP	SI
		POP	DI			;RESTORE RMB AND MMB ADDRS.
		TEST	BYTE PTR [DI+MMB_COOR],OVFLO_ON	;SET RCVR'S OVFLO?
		JE	XMM0			;NO.
		PUSH	X_CTCB			;YES:
		MOV	AX,[SI+RMB_TCB]		;GET RECEIVER TCB,
		MOV	X_CTCB,AX		;PUT CONVENIENTLY TO X_CTCB,
		MOV	AX,FLAG_OF		;SET THE OVERFLOW BIT ON
		CALL	X_SFLAGS		;IN HIS FLAGS REGISTER.
		POP	X_CTCB
XMM0:
		MOV	BX,SI			;DEALLOCATE BOTH THE
		CALL	X_PUT_SMR		;RMB AND MMB BLOCKS BACK
		MOV	BX,DI			;TO THE SMR SYSTEM NOW THAT
		CALL	X_PUT_SMR		;THE MESSAGE HAS BEEN MOVED.
		POP	CX
		POP	BX
		POP	ES
		RET
XMOVE_MSG	ENDP
PAGE
;*********************************************************************
;*      XDQ_MMB:    DEQUEUE A MAILED MESSAGE BLOCK.                  *
;*********************************************************************
;
;	THIS PROCEDURE DEQUEUES THE FIRST (HIGHEST PRIORITY)
;	ELEMENT ON THE MAILBOX MESSAGE QUEUE.
;
;	INPUT:	DI=	ADDRESS OF MAILBOX DATA
;	OUTPUT:	SI=	MMB, IF MESSAGE WAITING
;			0 , IF NO MESSAGE WAITING.
;
XDQ_MMB 	PROC	NEAR
		MOV	SI,[DI+MBX_MSGSH]	;GET CURRENT HEAD POINTER.
		TEST	SI,SI			;NULL MESSAGE QUEUE?
		JE	XDQM0			;YES.
		MOV	AX,[SI]			;NO: GET HEAD FORWARD LINK.
		MOV	[DI+MBX_MSGSH],AX	;FLINK(OLD HEAD)= NEW HEAD.
		TEST	AX,AX			;QUEUE NOW NULL?
		JNE	XDQM0			;NO, STILL A MESSAGE.
		MOV	[DI+MBX_MSGST],AX	;YES: ZAP TAIL POINTER.
XDQM0:
		RET
XDQ_MMB		ENDP
PAGE
;*********************************************************************
;*      XDQ_RMB:      DEQUEUE A RECEIVE-MESSAGE BLOCK.               *
;*********************************************************************
;
;	THIS PROCEDURE DEQUEUES THE FIRST ELEMENT
;	ON THE MAILBOX WAIT-FOR-MESSAGES QUEUE.
;
;	INPUT:	DI=	ADDRESS OF MAILBOX DATA
;
;	OUTPUT:	SI=	RMB, IF RECEIVER WAITING
;			0, IF NO RECEIVER WAITING.
;
XDQ_RMB		PROC	NEAR
		MOV	SI,[DI+MBX_WAITH]	;GET HEAD OF RMB QUEUE.
		TEST	SI,SI			;QUEUE NULL?
		JE	XDQR0			;YES.
		MOV	AX,[SI]			;NO: GET HEAD FORWARD LINK.
		MOV	[DI+MBX_WAITH],AX	;FLINK(OLD HEAD)= NEW HEAD.
		TEST	AX,AX			;QUEUE NOW NULL??
		JNE	XDQR0			;NO, AT LEAST 1 RMB LEFT.
		MOV	[DI+MBX_WAITT],AX	;YES: ZAP TAIL PTR.
XDQR0:
		RET
XDQ_RMB		ENDP
PAGE
;*********************************************************************
;*       X_SEND_MSG:   "INNER" SEND-MESSAGE FACILITY.                *
;*********************************************************************
;
;	THIS PROCEDURE SENDS THE MESSAGE.  IT TRANSFERS THE
;	MESSAGE TO THE HIGHEST PRIORITY RECEIVER, IF ONE IS
;	WAITING.  IF NOT, IT QUEUES THE MESSAGE TO THE MAILBOX
;	AND IF THE SENDER WISHES TO WAIT FOR A RECIEVER, IT
;	BLOCKS THE SENDER.  THIS PROCEDURE ALSO PREPARES THE
;	SVC STATUS FOR THE SENDER.
;
;	INPUT:	DH=	WAIT OPTION (WT)
;		DL=	MAILBOX ID (BOX)
;		CH=	PRIV PRIORITY (PRIV)
;		CL=	PRIORITY OPTION (PRIS)
;		BX=	TCB ADDRESS
;		ES:DI=	POINTER TO 3 WORD MESSAGE
;	OUTPUT:	AX=	SVC STATUS
;
X_SEND_MSG	PROC	NEAR
		PUSH	ES			;SAVE REGISTERS
		PUSH	DI
		PUSH	SI
		PUSH	BX
		MOV	SI,BX			;SENDER'S TCB ADDRESS MUST BE
						;IN SI AS A PARAMETER TO THE
						;XBUILD_MMB ROUTINE.
		CALL	XBUILD_MMB		;BUILD AN MMB IN SMR BLOCK.
		CALL	XFIND_MB		;FIND THE MBX.
		CALL	XDQ_RMB			;GET RECEIVER, IF ANY.
		TEST	SI,SI			;IS THERE A RECEIVER?
		JE	NORECVR			;NO.
		MOV	DI,BX			;DI=MMB, SI=RMB,
		MOV	BX,[SI+RMB_TCB]		;BX= RECEIVER'S TCB.
		CALL	XMOVE_MSG		;TRANSPORT MESSAGE,
		CALL	X_SET_READY		;MAKE RECEIVER READY AGAIN,
		JMP	SHORT SENDDONE		;FINISHED.
NORECVR:					;NO RECEIVER AT PRESENT:
						;BX= MMB, DI= MBX.
		PUSH	WORD PTR [BX+MMB_TCB]	;SAVE SENDER'S TCB ADDRESS.
		MOV	AL,[BX+MMB_PRI]		;GET MESSAGE PRIORITY.
		TEST	AL,AL			;ZERO?
		JE	NOR1			;YES: FIFO QUEUEING!
		MOV	SI,[DI+MBX_MSGST]	;NO: SAVE CURRENT TAIL PTR.
		PUSH	DI			;SAVE MBX ADDR.
		ADD	DI,MBX_MSGSH		;LOCATE MMB QUEUE
		CALL	X_QUEUE			;AND PUT MMB ON IT.
		POP	DI			;RESTORE MBX ADDR.
		TEST	SI,SI			;WAS QUEUE NULL?
		JE	NOR3			;YES: CHANGE TAIL TOO!
		CMP	[SI],BX			;DOES OLD TAIL NOW POINT TO
						;THE NEW MMB?
		JNE	MQDONE			;NO, TAIL PTR STILL VALID.
		JMP	SHORT NOR3		;YES: NEW ENTRY= NEW TAIL.
NOR1:
		CMP	WORD PTR [DI+MBX_MSGSH],0	;MMB QUEUE NULL?
		JNE	NOR2			;NO, ONLY ALTER TAIL PTR.
		MOV	[DI+MBX_MSGSH],BX	;YES: NEW ENTRY= NEW HEAD.
		JMP	SHORT NOR3
NOR2:
		MOV	SI,[DI+MBX_MSGST]	;GET OLD TAIL ADDRESS,
		MOV	[SI],BX			;HOOK IT TO NEW ENTRY,
NOR3:
		MOV	[DI+MBX_MSGST],BX	;NEW ENTRY= NEW TAIL.
MQDONE:
		POP	BX			;RETRIEVE SENDER'S TCB,
		TEST	DH,SENDER_BLOCKED	;DID SENDER SAY HE'D WAIT?
		JE	SENDDONE		;NO.
		MOV	AX,BLOCK+STA_MSGRCVD	;YES:
		CALL	X_TASK_STATE		;PUT SENDER IN COORD. STATE.
SENDDONE:
		XOR	AX,AX				;CLEAR STATUS
		POP	BX				;RESTORE REGISTERS
		POP	SI
		POP	DI
		POP	ES
		RET
X_SEND_MSG	ENDP
PAGE
;*********************************************************************
;*      X_RECV_MSG:   INNER RECEIVE-MESSAGE FACILITY.                *
;*********************************************************************
;
;	THIS PROCEDURE RECEIVES A MAILBOX MESSAGE.  IF A
;	MESSAGE IS WAITITNG, IT IS RECIEVED AND IF THE SENDER
;	WAS WAITING  IT IS ACTIVATED. IF THERE IS NO MESSAGE
;	AND THE RECEIVER WISHES TO WAIT, IT IS BLOCKED AND 
;	QUEUED. THE SVC STATUS IS PREPARED FOR THE RECEIVER.
;
;	INPUT:	DH=	WAIT OPTION (WT)
;		DL=	MAILBOX ID (BOX)
;		ES:DI=	POINTER TO 3 WORD MESSAGE BUFFER
;	OUTPUT:	AX=	SVC STATUS
;
X_RECV_MSG	PROC	NEAR
		PUSH	ES			;SAVE REGISTERS
		PUSH	DI
		PUSH	SI
		PUSH	BX
		MOV	SI,BX			;SI MUST PASS REQUESTOR'S TCB
						;ADDRESS AS A PARAMETER TO THE
						;XBUILD_RMB ROUTINE.
		CALL	XBUILD_RMB		;BUILD RMB IN SMR BLOCK.
		CALL	XFIND_MB		;FIND MAILBOX CONTROL DATA
		CALL	XDQ_MMB			;GET TOP MMB, IF ANY...
		TEST	SI,SI			;ANY?
		JE	NOMSG			;NOT AT THIS TIME.
		MOV	DI,SI			;MAKE DI= MMB, AND
		MOV	SI,BX			;SI= RMB.
		PUSH	WORD PTR [DI+MMB_TCB]	;SAVE SENDER'S TCB ADDRESS.
		MOV	BL,[DI+MMB_COOR]	;SAVE COORDINATION IN BL.
		CALL	XMOVE_MSG		;TRANSPORT THE MESSAGE.
		TEST	BL,SENDER_BLOCKED	;MUST WE UNBLOCK SENDER?
		POP	BX			;RETRIEVE HIS TCB ADDRESS...
		JE	NOWTSEND		;NO, NOT SPECIFIED.
		CALL	X_SET_READY		;YES.
NOWTSEND:
		XOR	AX,AX			;CLEAR STATUS
		JMP	SHORT RMDONE		;AND EXIT.
NOMSG:						;NO MESSAGE AWAITING PICKUP:
						;BX=RMB, DI=MBX.
		TEST	DH,DH			;DID RECEIVER SAY HE'D WAIT?
		JE	NORECWT			;NO.
		PUSH	WORD PTR [BX+RMB_TCB]	;YES: SAVE REQUESTOR TCB,
		CMP	WORD PTR [DI+MBX_WAITH],0	;ANY WAITING RMBS?
		JNE	NOM1			;YES, NORMAL APPEND TO TAIL.
		MOV	[DI+MBX_WAITH],BX	;NO, NEW RMB= NEW HEAD.
		JMP	SHORT NOM2
NOM1:
		MOV	SI,[DI+MBX_WAITT]	;GET CURRENT TAIL POINTER,
		MOV	[SI],BX			;ATTACH IT TO NEW RMB,
NOM2:
		MOV	[DI+MBX_WAITT],BX	;SET NEW RMB= NEW TAIL.
		POP	BX			;BLOCK RECEIVER PENDING
		MOV	AX,BLOCK+STA_MSG	;MESSAGE AVAILABILITY.
		CALL	X_TASK_STATE
		XOR	AX,AX			;CLEAR STATUS
		JMP	SHORT RMDONE		;AND EXIT.
NORECWT:					;RECEIVER WANTS FAILURE STATUS:
		CALL	X_PUT_SMR		;FREE RMB
		MOV	AX,SVC_FAILED		;SET STATUS
RMDONE:
		POP	BX			;RESTORE REG
		POP	SI
		POP	DI
		POP	ES
		RET
X_RECV_MSG	ENDP
PAGE
;*********************************************************************
;*      X_SEND:  SEND MAILBOX MESSAGE SERVICE                        *
;*********************************************************************
;
;	THIS IS THE FRONT END FOR ALL MESSAGE TRANSMISSIONS. BEFORE ENTER-
;	ING MAIN PROCESSING, IT CHECKS THE NODE FIELD OF THE SDB FOR A NON-
;	ZERO VALUE. IF THE NODE NUMBER IS NONZERO, THE NETWORK MANAGER IS
;	QSTRTED WITH THE ADDRESS OF THE SDB AS HIS STARTUP PARAMETERS. ELSE,
;	THE ROUTINE SENDS THE MESSAGE AND PASSES THE SVC STATUS	TO THE TASK.
;
;	INPUT:	ES:SI=	SDB ADDRESS
;		BX=	TCB ADDRESS
;	OUTPUT:	NONE
;
X_SEND		PROC	FAR
		CMP	BYTE PTR ES:[SI+NODE],0	;LOCAL MAILBOX ADDRESSED?
		JNE	SEND1			;NO! INVOKE NET MGR.
		MOV	CH,ES:[SI+PRIV]		;YES: PARMS INTO REGISTERS
		MOV	CL,ES:[SI+PRIS]
		AND	CL,PRIS_MASK
		MOV	DH,ES:[SI+WT]
		AND	DH,WT_MASK+PBLOCK_MASK	;GET WT, PBLOCK PARAMS.
		ROL	DH,1			;MOVE THEM TO BITS 0 AND
		ROL	DH,1			;1 OF DH.
		MOV	DL,ES:[SI+BOX]
		LES	DI,DWORD PTR ES:[SI+BUFR]
;
;	VALIDATE THE PARAMETERS
;
		MOV	AX,SVC_FAILED+PARM_ERR	;PREPARE FOR FAILURE.
		TEST	CL,CL			;VALID PRIS=1,2 OR 3
		JE	SEND0			;ZERO NOT VALID.
		PUSH	AX
		MOV	AX,X_MAILBOX_NUM
		CMP	DL,AL			;VALID MAILBOX NUMBER?
		POP	AX
		JAE	SEND0			;NO.
		CALL	X_SEND_MSG		;YES: CALL INNER ROUTINE.
SEND0:
		CALL	X_SFLAGS		;RETURN SVC STATUS.
		RET
;
;	A NETWORK MAIL REQUEST HAS BEEN DETECTED:
;	THESE ARE HANDLED OUTSIDE THE NUCLEUS, BY THE NETWORK MANAGER. IT
;	IS THEREFORE NECESSARY TO BLOCK THE SENDER AS IS THE DEBUGGER HAD
;	DONE SO (SO THAT THE NET MGR CAN UNDO IT), AND TO QSTRT THE NET
;	MGR WITH THE ADDRESS OF THE SDB AS HIS STARTUP PARAMETER WORDS.
;
SEND1:
		PUSH	BX			;SAVE REQUESTOR'S TCB.
		MOV	AX,X_NETMGR_TASK	;COMPUTE TCB ADDRESS OF
		CALL	X_GET_TCB_ADR		;NETWORK MANAGER...
		JC	SEND2			;NO SUCH TASK.
		MOV	DI,BX			;GOT IT: TO DI WITH IT.
		POP	BX			;RETRIEVE REQUESTOR TCB,
		MOV	AX,BLOCK+STA_HLT	;AND BLOCK HIM SO THAT THE
		CALL	X_TASK_STATE		;NET MGR CAN UNBLOCK HIM WHEN
						;THE MESSAGE IS DELIVERED.
		MOV	AX,SI			;SDB OFFSET TO AX,
		MOV	SI,BX			;REQUESTOR TCB TO SI,
		MOV	BX,ES			;SDB SEGMENT TO BX.
		MOV	CX,00FFH		;HIGH PRI, NO COORD.
		XOR	BP,BP			;AUTO ALLOCATE.
		CALL	X_START_TASK		;START NET MANAGER.
		RET
SEND2:
		POP	BX			;CLEAN THE STACK,
		MOV	AX,SVC_FAILED+PARM_ERR	;RETURN SVC FAILURE STATUS
		JMP	SHORT SEND0		;TO REQUESTOR.
X_SEND		ENDP
PAGE
;************************************************************************
;*         X_RECV:    RECEIVE MAILBOX MESSAGE SERVICE                   *
;************************************************************************
;
;	THIS PROCEDURE VALIDATES THE SDB PARAMETERS,
;	RECEIVES A MAILBOX MESSAGE AND PASSES STATUS
;	TO THE TASK.
;
;	INPUT:	ES:SI=	SDB ADDRESS
;		BX=	TCB ADDRESS
;	OUTPUT:	NONE
;
X_RECV		PROC	FAR
		MOV	DH,ES:[SI+WT]		;GET WT PARAMETER TO DH,
		AND	DH,WT_MASK		;ISOLATE IT AND MOVE IT INTO
		ROL	DH,1			;BIT 0 OF DH.
		MOV	DL,ES:[SI+BOX]		;MAILBOX NR. TO DL.
		LES	DI,DWORD PTR ES:[SI+BUFR]	;BUFR. ADDR. TO ES:SI
;
;	VALIDATE PARAMETERS
;
		MOV	AX,X_MAILBOX_NUM
		CMP	DL,AL			;GOOD MAILBOX NR?
		MOV	AX,SVC_FAILED+PARM_ERR	;PREPARE FOR FAILURE...
		JAE	RECV0			;NO.
;
;	VALID PARAMETERS
;
		CALL	X_RECV_MSG		;PERFORM RECEIVE PROCESSING,
RECV0:
		CALL	X_SFLAGS		;RETURN OPERATION STATUS.
		RET
X_RECV		ENDP
MTOS_CS		ENDS
		END
