PAGE	60,132
INCLUDE B:BIG.INC
;
;	MISCIN.ASM:	MISCELLANEOUS INTERRUPT HANDLERS FOR LEVEL II.
;	F. W. PORRETTO, 12/15/82
;
;	HERE ARE COLLECTED THE INTERRUPT HANDLERS FOR:
;		-BOTH TYPES OF SVC,
;		-THE 8086 NATIVE INTERRUPT CONDITIONS,
;		-ILLEGAL BREAKPOINT AND SINGLESTEP INTERRUPTS.
;
		NAME	MISC_INTERRUPTS
;
		EXTRN	X_MAX_SVC:ABS
;
MTOS_CS		SEGMENT	PARA PUBLIC
		ASSUME	CS:MTOS_CS,DS:MTOS_DS,SS:MTOS_DS
		PUBLIC	X_SVC_INT,X_SVC_INT1
		PUBLIC	X_UNEXPECTED,X_DUMMY
		PUBLIC	X_ZERO,X_NMI,X_INTO
;
;	UN-COMMENT THESE IF NOT USING THE DEBUGGER: ALSO SEE BUGS.ASM.
;
;		PUBLIC	X_BPT,X_SS
;
		EXTRN	X_SFLAGS:NEAR
		EXTRN	X_SYSOUT:NEAR
		EXTRN	X_TASK_STATE:NEAR
		EXTRN	X_SET_READY:NEAR
		EXTRN	X_WAIT_SMR:NEAR
		EXTRN	X_SCHDLR:NEAR
		EXTRN	X_ELIPSE:FAR
		EXTRN	X_SYSTATE:FAR
		EXTRN	X_XEL:FAR
		EXTRN	XERRLOG:FAR
		EXTRN	X_JT:DWORD
		EXTRN	X_MEM_RQRD:BYTE
		EXTRN	X_UNX_MSG:WORD
		EXTRN	X_NMI_MSG:WORD
		EXTRN	X_MSG_ADR:WORD
;
PAGE
;**************************************************
;*      SVC INTERRUPT HANDLERS, TYPES 1 & 2       *
;**************************************************
;
;	THESE DIFFER ONLY IN THE PUTATIVE LOCATION FOR THE SDB ADDRESS:
;		-TYPE 1: ON THE STACK BELOW THE RETURN ADDRESS;
;		-TYPE 2: AFTER THE SVC INSTRUCTION, IN-STREAM.
;
X_SVC_INT	PROC	FAR
		CALL	X_ELIPSE		;I-STATE, PLEASE.
		MOV	BX,X_ATCB		;GET REQUESTOR'S TCB
		LES	SI,DWORD PTR [BX+TCB_SP]	;GET STACK PTR,
		MOV	WORD PTR ES:[SI+STK_FL],INIT_FLAGS	;INIT FLAGS,
		LES	SI,DWORD PTR ES:[SI+STK_SVC_OFST] ;GET SDB ADDRESS,
		JMP	SHORT SVC_COMN		;ENTER COMMON THREAD.
X_SVC_INT1	LABEL	FAR
		CALL	X_ELIPSE
		MOV	BX,X_ATCB		;AS ABOVE.
		PUSH	DS			;NEED WORK REGS.
		LES	SI,DWORD PTR [BX+TCB_SP]	;GET STACK PTR,
		MOV	WORD PTR ES:[SI+STK_FL],INIT_FLAGS	;INIT FLAGS,
		LDS	DI,DWORD PTR ES:[SI+STK_IP]	;GET CURRENT IP/CS.
		ADD	WORD PTR ES:[SI+STK_IP],4	;BYPASS SDB POINTER,
		LES	SI,DWORD PTR [DI]	;GET ADDRESS TO ES:SI.
		POP	DS
SVC_COMN:
		MOV	X_ATCB,0		;LIE TO THE SCHEDULER,
;
;	THE FOLLOWING FOUR INSTRUCTIONS SUPPORT THE USE OF THE 8087 NUMERIC
;	DATA COPROCESSOR. IF THAT DEVICE IS NOT USED, THEY SHOULD BE LEFT
;	COMMENTED OUT. ELSE, REMOVE THE LEADING SEMICOLONS.
;
;		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_8087	;8087 USER?
;		JE	SVC_COM0		;NO.
;		MOV	BP,[BX+TCB_8087]	;YES: GET SAVE AREA ADDRESS
;		FSAVE	DS:[BP]			;AND SAVE NDP'S REGISTERS.
;
SVC_COM0:
		CALL	X_SYSTATE		;AND GO TO S STATE.
;
;	AT ARRIVAL HERE, ES:SI= SDB ADDRESS, BX= REQUESTOR'S TCB.
;
		MOV	X_CTCB,BX		;PUT REQUESTOR'S TCB ADDR INTO
		MOV	DI,XCSRB		;ALL THE PLACES WHERE IT'S
		MOV	[DI+SRB_TCB],BX		;EXPECTED TO BE.
		MOV	AL,ES:[SI]		;GET SDB SERVICE CODE.
		MOV	DX,X_MAX_SVC
		CMP	AL,DL			;TOO HIGH?
		JAE	SVC01			;YES, ERROR.
		TEST	AL,AL			;NO: TOO LOW?
		JE	SVC01			;YES, ERROR.
;
;	THE SERVICE CODE IS FINE: INSURE AVAILABILITY OF SUFFICIENT SMR.
;
		XOR	AH,AH
		MOV	DI,AX			;SAVE 16-BIT SERVICE CODE.
		MOV	AX,BLOCK+STA_SVC	;MOVE THE REQUESTOR TO THE
		CALL	X_TASK_STATE		;SAFETY STATE.
		CMP	CS:X_MEM_RQRD[DI],0	;SERVICE NEEDS SMR?
		JE	SVC02			;NO, SAFE TO DISPATCH IT.
	LOCK	CMP	X_SMR_AVAIL,X_MIN_MEM	;YES: SAFE AMOUNT AVAILABLE?
		JAE	SVC02			;YES, DISPATCH SERVICE.
		MOV	BX,XCSRB		;NO:
		CALL	X_WAIT_SMR		;SIDELINE THE REQUEST.
		JMP	X_SCHDLR
SVC01:
		MOV	AX,SVC_FAILED+PARM_ERR
		CALL	X_SFLAGS		;BAD SERVICE CODE.
		CALL	X_SET_READY		;READY THE REQUESTOR.
		RET
SVC02:
		SHL	DI,1			;MULTIPLY THE SERVICE CODE
		SHL	DI,1			;BY 4 TO INDEX JUMP TABLE.
		CALL	CS:X_JT[DI]		;CALL SERVICE ROUTINE.
		MOV	BX,X_CTCB		;GET REQUESTOR'S TCB AGAIN,
		CMP	BYTE PTR [BX+TCB_STATE],BLOCK+STA_SVC	;SAFETY?
		JNE	SVC03			;NO: DON'T TOUCH IT.
		MOV	AX,BLOCK		;YES: SEND IT BACK
		CALL	X_TASK_STATE		;FROM SAFETY STATE
		CALL	X_SET_READY		;TO READY STATE!
SVC03:
		RET
X_SVC_INT	ENDP
;
;*********************************************
;*     DUMMY SERVICE CALL HANDLER            *
;*********************************************
;
;	THIS INTERCEPTS SERVICE CALLS TO UNCONFIGURED SERVICES.
;
X_DUMMY		PROC	FAR
		MOV	AX,SVC_FAILED+PARM_ERR
		CALL	X_SFLAGS
		RET
X_DUMMY		ENDP
PAGE
;************************************************
;*     UNEXPECTED INTERRUPT HANDLER             *
;************************************************
;
;	THIS HANDLER MERELY ACTIVATES THE ERROR LOGGER TO ISSUE A MESSAGE.
;
X_UNEXPECTED	PROC	FAR
		CALL	X_ELIPSE		;GO DIRECTLY TO
		CALL	X_SYSTATE		;S STATE.
		MOV	XERRMSG,OFFSET X_UNX_MSG	;SET UP ADDRESS OF
		MOV	XERRMSG+2,CS		;ERROR MESSAGE,
		CALL	XERRLOG			;START ERROR LOGGER!
		RET				;ALL WE CAN DO.
X_UNEXPECTED	ENDP
;
;************************************************
;*     NONMASKABLE INTERRUPT HANDLER            *
;************************************************
;
;	THIS HANDLER PROTECTS THE SYSTEM AGAINST STATE TRANSITION VIOLATION:
;	SINCE THE NMI CAN OCCUR IN DI-STATE, WE SENSE FOR THAT CONDITION 
;	HERE AND BUFFER IT OFF.
;
X_NMI		PROC	FAR
		PUSH	BP
		MOV	BP,SP			;WE'LL BE LOOKING AT STACK.
		TEST	WORD PTR [BP+6],0200H	;INTERRUPTS ON?
		POP	BP
		JE	NMIRET			;NO: DISCARD THIS INTERRUPT!
		CALL	X_ELIPSE		;YES: GO DIRECTLY TO
		CALL	X_SYSTATE		;S STATE.
		MOV	XERRMSG,OFFSET X_NMI_MSG	;SET UP THE NMI
		MOV	XERRMSG+2,CS		;ERROR MESSAGE,
		CALL	XERRLOG			;START ERROR LOGGER,
		RET				;THAT'S ALL!
NMIRET:
		IRET
X_NMI		ENDP
PAGE
;**************************************************
;*       8086 NATIVE INTERRUPT HANDLERS           *
;**************************************************
;
;	WHICHEVER OF THESE IS ACTIVATED BY AN INTERRUPT CAUSES THE
;	ERROR LOGGER TO BE ACTIVATED WITH A MATCHING MESSAGE.
;
X_ZERO		PROC	FAR
		CALL	X_ELIPSE
		MOV	AX,BLOCK+STA_ERR0
		JMP	SHORT INTRPCOM
;
;	INCLUDED HERE ARE HANDLERS TO REPLACE THE DEBUGGER'S SUPPORT HANDLERS
;	(IN BUGS.ASM) WHEN THE DEBUGGER IS NOT DESIRED.
;
;X_SS		LABEL	FAR
;		CALL	X_ELIPSE
;		MOV	AX,BLOCK+STA_ERR1
;		JMP	SHORT INTRPCOM
;X_BPT		LABEL	FAR
;		CALL	X_ELIPSE
;		MOV	AX,BLOCK+STA_ERR2
;		JMP	SHORT INTRPCOM
;
X_INTO		LABEL	FAR
		CALL	X_ELIPSE
		MOV	AX,BLOCK+STA_ERR3
INTRPCOM:
		MOV	BX,X_ATCB		;GET TCB ADDRESS,
		MOV	X_ATCB,0		;FOOL THE SCHEDULER.
;
;	THE FOLLOWING FOUR INSTRUCTIONS SUPPORT THE USE OF THE 8087 NUMERIC
;	DATA COPROCESSOR. IF THAT DEVICE IS NOT USED, THEY SHOULD BE LEFT
;	COMMENTED OUT. ELSE, REMOVE THE LEADING SEMICOLONS.
;
;		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_8087	;8087 USER?
;		JE	INTRP0			;NO.
;		MOV	BP,[BX+TCB_8087]	;YES: GET SAVE AREA ADDRESS
;		FSAVE	DS:[BP]			;AND SAVE NDP'S REGISTERS.
;
INTRP0:
 		CALL	X_SYSTATE		;TO S STATE.
		MOV	X_CTCB,BX		;PLUG X_CTCB WITH TCB.
		TEST	BX,BX			;ANY TASK INVOLVED??
		JE	INTRP15			;NO.
		PUSH	AX			;YES:
		CALL	X_TASK_STATE		;BLOCK IT.
		POP	AX
INTRP15:
		SUB	AX,BLOCK+STA_ERR0	;RENORMALIZE ERROR CODE,
		MOV	SI,AX			;USE IT AS INDEX INTO TABLE
		LES	DI,DWORD PTR CS:X_MSG_ADR[SI]	;OF MESSAGES,
		MOV	XERRMSG,DI		;PUT CORRECT ADDRESS UP FOR
		MOV	XERRMSG+2,ES		;ERROR LOGGER,
		CALL	XERRLOG			;ACTIVATE ERROR LOGGER,
		RET				;DONE!
X_ZERO		ENDP
;
MTOS_CS		ENDS
		END
