PAGE	60,132
INCLUDE B:BIG.INC
;
;	PAUSE.ASM:	TASK PAUSE/RESUME SERVICES.
;	F. W. PORRETTO, 12/15/82.
;
;		HERE ARE COLLECTED THE TWO SERVICES DESCRIBED ABOVE, AND THEIR
;	SUPPORT ROUTINES.
;
		NAME	PAUSE_RESUME_SERVICES
;
MTOS_CS		SEGMENT PARA PUBLIC
		ASSUME	DS:MTOS_DS,CS:MTOS_CS,SS:MTOS_DS
		PUBLIC	X_PAUSE,X_RESUME
		PUBLIC	X_PAUSE_TASK,X_RESUME_TASK
		EXTRN	X_DQUEUE_TMB	  :NEAR
		EXTRN	X_QUEUE_TMB	  :NEAR
		EXTRN	X_SFLAGS	  :NEAR
		EXTRN	X_SET_READY	  :NEAR
		EXTRN	X_TASK_STATE	  :NEAR
		EXTRN	X_GET_TCB_ADR	  :NEAR
		EXTRN	X_GET_SMR	  :NEAR
		EXTRN	X_PUT_SMR         :NEAR
		EXTRN	X_GET_INTERVAL	  :NEAR
;
;	LOCAL EQUATES
;
TME		EQU	01H		;PAUSE SDB OFFSET OF TIME CODE
LEN		EQU	02H		;          OFFSET OF LEN
TME_MASK	EQU	018H		;          MASK TO ISOLATE TME
;
ID		EQU	01H		;RESUME SDB OFFSET OF TASK ID
PAGE
;********************************************
;*     XBUILD_PTMB:  BUILD A PAUSE TMB.     *
;********************************************
;
;	THIS PROCEDURE BUILDS A TMB FOR A TASK PAUSE.  THE
;	TMB IS BUILT IN THE SMR PROVIDED.
;
;	INPUT:	BX=	SMR
;		SI=	TCB
;		DL=	TME (SHIFTED LEFT TWO BITS)
;		DH=	LEN
;	OUTPUT:	BX=	PAUSE TMB
;
XBUILD_PTMB	PROC	NEAR
		PUSH	DX			;STORE REGISTERS
		PUSH	CX
		MOV	AX,DX			;CONVERT TME AND LEN TO
						;INTERVAL
		CALL	X_GET_INTERVAL
		MOV	[BX+TMB_TIME],CX	;MOVE INTERVAL TO TMB
		MOV	[BX+TMB_TIME+2],DX
		MOV	BYTE PTR [BX+TMB_TYPE],02H	;PUT TYPE IN TMB
		MOV	[BX+TMB_TCB],SI		;PUT TCB ADDR IN TM
		POP	CX			;RESTORE REGISTERS
		POP	DX
		RET
XBUILD_PTMB	ENDP
PAGE
;*************************************************
;*     XDQ_PTMB:   DEQUEUE A TASK-PAUSE TMB.     *
;*************************************************
;
;	THIS PROCEDURE SEARCHES THE TMB QUEUE FOR THE SPECIFIED
;	PAUSE TMB AND DEQUEUES AND DEALLOCATES IT.
;
;	INPUT:	BX=	TCB ADDRESS OF PAUSED TASK
;	OUTPUT:	NONE.
;
XDQ_PTMB	PROC	NEAR
		PUSH	DI
		PUSH	SI
		MOV	DI,OFFSET X_TMB_Q
PT0:
		MOV	SI,[DI]			;GET LINK WORD
		TEST	SI,SI			;END OF QUEUE ?
		JE	PTRET			;YES, EXIT.
		CMP	BYTE PTR [SI+TMB_TYPE],2	;PAUSE TMB ?
		JNE	PTNEXT			;NO.
		CMP	[SI+TMB_TCB],BX		;YES: FOR RIGHT TASK ?
		JNE	PTNEXT			;NO.
		MOV	AX,[SI]			;YES: DEQUEUE THE TMB
		MOV	[DI],AX			;FROM TIMER QUEUE.
		XCHG	BX,SI			;THEN RETURN IT TO THE
		CALL	X_PUT_SMR		;SMR FREE LIST.
		XCHG	BX,SI
PTRET:
		POP	SI			;RESTORE REGISTERS
		POP	DI
		RET
PTNEXT:
		MOV	DI,SI			;CURRENT BECOMES PREVIOUS
		JMP	SHORT PT0		;GO AROUND AGAIN.
XDQ_PTMB	ENDP
PAGE
;*******************************************************
;*     X_PAUSE_TASK:  "INNER" TASK PAUSE HANDLER.      *
;*******************************************************
;
;	THIS PROCEDURE BLOCKS THE TASK AND QUEUES THE PAUSE TMB.
;	IT ALSO PREPARES THE SVC STATUS. ALTHOUGH THE PARAMETERS
;	ARE ALL PASSED IN REGISTERS, THIS PROCEDURE CAN NOT BE
;	USED BY SYSTEM FUNCTIONS BECAUSE THE WAIT BLOCK(TMB) MUST
;	POINT TO A TASK.
;
;	INPUT:	DL=	TME (SHIFTED LEFT TWO BITS)
;		DH=	LEN
;		BX=	TCB ADDRESS
;	OUTPUT:	AX=	SVC STATUS
;
X_PAUSE_TASK	PROC	NEAR
		MOV	SI,BX			;SI=TCB ADDRESS
		CALL	X_GET_SMR		;BX=SMR
		CALL	XBUILD_PTMB		;BX=PAUSE TMB
		CALL	X_QUEUE_TMB		;ACTIVATE PAUSE TIMER.
		MOV	BX,SI			;TCB BACK TO BX,
		MOV	AX,BLOCK+STA_PAUSE	;BLOCK TASK AWAITING
		CALL	X_TASK_STATE		;PAUSE EXPIRATION.
		XOR	AX,AX			;SET STATUS
		RET
X_PAUSE_TASK	ENDP
PAGE
;********************************************************
;*     X_RESUME_TASK:  "INNER" RESUME-TASK HANDLER.     *
;********************************************************
;
;	THIS PROCEDURE DEQUEUES THE PAUSE TMB, ACTIVATES THE
;	PAUSED TASK (TARGET TASK) AND PREPARES THE STATUS.
;
;	INPUT:	AX=	ID OF TASK TO RESUME (TARGET)
;		BX=	TCB ADDRESS OF REQUESTING TASK
;	OUTPUT:	AX=	SVC STATUS
;
X_RESUME_TASK	PROC	NEAR
		PUSH	SI			;STORE REGISTERS
		PUSH	DI
		MOV	SI,BX			;SI=REQUESTER'S TCB
		CALL	X_GET_TCB_ADR		;GET TARGET'S TCB
		JC	RSERR0			;NOT A VALID TASK ID.
		CMP	BYTE PTR [BX+TCB_STATE],BLOCK+STA_PAUSE	;CHECK STATE
		JNE	RSERR1			;TARGET NOT PAUSED!
		PUSH	BX			;STORE TARGET TCB ADDR
		CALL	XDQ_PTMB		;REMOVE PAUSE TMB.
		POP	BX			;BX = TARGET'S TCB ADDR
		PUSH	X_CTCB			;STORE CURRENT TCB
		MOV	X_CTCB,BX		;PLACE TARGET'S TCB IN X_CTCB
		MOV	AX,FLAG_CF		;CARRY MEANS TASK WAS RESUMED.
		CALL	X_SFLAGS		;SEND STATUS
		POP	X_CTCB			;RESTORE CURRENT TCB
		CALL	X_SET_READY		;READY TARGET
		MOV	BX,SI			;BX=REQUESTOR'S TCB
		XOR	AX,AX			;SET STATUS
		POP	DI
		POP	SI
		RET
RSERR1:						;TASK NOT PAUSED:
		MOV	AX,SVC_FAILED+FLAG_CF	;INDICATE DIFFERENT STATE.
		JMP	SHORT RSERR2
RSERR0:
		MOV	AX,SVC_FAILED+PARM_ERR	;INDICATE INVALID TASK.
RSERR2:
		MOV	BX,SI			;REQUESTOR TCB BACK TO BX.
		POP	DI
		POP	SI
		RET
X_RESUME_TASK	ENDP
PAGE
;************************************************
;*       X_PAUSE: HANDLE TASK-PAUSE SVC.        *
;************************************************
;
;	THIS PROCEDURE VALIDATES THE SDB PARAMETERS, BLOCKS
;	THE TASK, QUEUES THE TMB AND RETURNS STATUS TO THE TASK.
;
;	INPUT:	ES:SI=	SDB ADDRESS
;		BX=	TCB ADDRESS
;	OUTPUT:	NONE
;
X_PAUSE		PROC	FAR
		MOV	DH,ES:[SI+LEN]		;GET LEN FROM SDB
		MOV	DL,ES:[SI+TME]		;GET TME FROM SDB
		MOV	AX,SVC_FAILED+PARM_ERR	;READY FOR FAILURE.
		CMP	DL,TME_MASK		;VALID TME ?
		JGE	PAUSERR			;NO.
		CALL	X_PAUSE_TASK		;YES: PAUSE TASK
PAUSERR:
		CALL	X_SFLAGS		;RETURN STATUS
		RET
X_PAUSE		ENDP
PAGE
;**********************************************
;*     X_RESUME:  HANDLE RESUME-TASK SVC.     *
;**********************************************
;
;	THIS PROCEDURE VALIDATES THE SDB PARAMETERS, DEQUEUES
; 	THE PAUSE TMB, ACTIVATES THE PAUSED TASK AND RETURNS
;	STATUS TO THE REQUESTING TASK.
;
;	INPUT:	ES:SI=	SDB ADDRESS
;		BX=	TCB ADDRESS
;	OUTPUT:	NONE
;
X_RESUME	PROC	FAR
		MOV	AX,ES:[SI+ID]		;GET ID OF TASK TO RESUME
		CALL	X_RESUME_TASK		;RESUME TASK
		CALL	X_SFLAGS		;RETURN STATUS
		RET
X_RESUME	ENDP
;
MTOS_CS		ENDS
		END
