PAGE	60,132
INCLUDE B:BIG.INC
;
;	PIT.ASM:	CLOCK INTERRUPT HANDLER AND TIMEOUT PROCESSORS.
;	F. W. PORRETTO, 12/15/82
;
;		HERE ARE COLLECTED ALL THE ABOVEMENTIONED PROCEDURES, PLUS THE
;	TWO ROUTINES (X_QUEUE_TMB AND X_DQUEUE_TMB) USED TO MANIPULATE THE
;	SYSTEM TIMER QUEUE.
;
		NAME	PIT_DRIVER
;
MTOS_CS		SEGMENT PARA PUBLIC
;
		ASSUME	CS:MTOS_CS,DS:MTOS_DS,SS:MTOS_DS
;
		PUBLIC	X_PIT_INT	;PIT INTERRUPT HANDLER
		PUBLIC	X_QUEUE_TMB	;ADD TIME BLOCK TO QUEUE
		PUBLIC	X_DQUEUE_TMB	;REMOVE A TMB FROM QUEUE
		PUBLIC	X_TYPE_0,X_TYPE_1,X_TYPE_2,X_TYPE_3
		PUBLIC	X_TYPE_4,X_TYPE_5,X_TYPE_6,X_TYPE_7
;
		EXTRN	X_ELIPSE:FAR,X_SYSTATE:FAR,X_XEL:FAR
		EXTRN	X_ISSUE_EOI	:NEAR	;ISSUE EOI TO INTRPT CNTRLLR
		EXTRN	X_PUT_SMR	:NEAR	;FREE SMR BLOCK.
		EXTRN	X_INTIME	:NEAR	;CURRENT INTERNAL TIME
		EXTRN	X_TASK_STATE	:NEAR	;MOVE TASK TO SPECIFIED STATE
		EXTRN	X_TAB		:WORD	;TIMER TYPE JUMP TABLE.
		EXTRN	X_SCHDLR	:NEAR	;SCHEDULER ENTRY POINT.
		EXTRN	X_SET_READY	:NEAR	;TASK TO THE READY STATE
		EXTRN	X_QUEUE		:NEAR	;GENERAL QUEUE ROUTINE
		EXTRN	X_DQUEUE	:NEAR	;GENERAL DE-QUEUE ROUTINE
		EXTRN	X_SET_EF	:NEAR	;SET EVENT FLAG(S)
		EXTRN	XOVER		:FAR	;TYPE 2 DRIVER COMPLETION.
		EXTRN	X_GET_INTERVAL	:NEAR	;COMPUTE COUNTDOWNS.
;
;	HANDY SYMBOLS:
;		1. FOR TMBS WHOSE ADDRESS IS IN SI.
;
TM_HTIME	EQU	WORD PTR [SI+TMB_HTIME]	;HIGH WORD OF EXPIRATION TIME
TM_MTIME	EQU	WORD PTR [SI+TMB_TIME]	;MIDDLE WORD OF EXPIRATION
TM_LTIME	EQU	WORD PTR [SI+TMB_TIME+2]	;LOWER WORD.
TM_TYPE		EQU	BYTE PTR [SI+TMB_TYPE]	;TIMER TYPE (0-6)
TM_UCB		EQU	WORD PTR [SI+TMB_UCB]	;UCB ADDRESS FOR TYPE 1 TMB
TM_SADR		EQU	DWORD PTR [SI+TMB_SADR]	;SERVICE ADDRESS FOR TYPE 1 TMB
TM_EFGRP	EQU	BYTE PTR [SI+TMB_EFGRP]	;EVENT FLAG GROUP, TYPE 3 TMB
TM_EFMSK	EQU	WORD PTR [SI+TMB_EFMSK]	;EVENT FLAG MASK, TYPE 3 TMB
TM_OVPRI	EQU	WORD PTR [SI+TMB_OVPRI]	;OVERRIDE AND BOUNDARY PRIORITY
TM_BPRI		EQU	BYTE PTR [SI+TMB_BPRI]	;BOUNDARY PRIORITY (TYPE 5)
TM_SWB		EQU	WORD PTR [SI+TMB_SWB]	;SEMAPHORE WAIT BLOCK ADDR.
TM_SEM		EQU	WORD PTR [SI+TMB_SEM]	;SEMAPHORE CONTROL BLOCK ADDR.
TM_IRB		EQU	WORD PTR [SI+TMB_IRB]	;RESERVE-UNIT IRB (TYPE 7).
TM_LINK		EQU	WORD PTR [SI+TMB_LINK]	;LINK TO NEXT TMB OR 0.
TM_TCB		EQU	WORD PTR [SI+TMB_TCB]	;TCB ADDRESS, TYPE 2,3,4,5,6,7
;
;	2: A CONVENIENCE REDEFINITION FOR XSTATUS:
;
XPRIMARY	EQU	BYTE PTR XSTATUS
;
;	3: TIME OUT STATUS FOR I/O REQUESTS:
;
TIME_OUT	EQU	0FCH
PAGE
;
;	CONVENIENCE ROUTINE TO CHECK FOR TMB EXPIRATION:
;	THE TMB ADDRESS IS IN SI, AND THE CONDITION CODES REGISTER RETURNS
;	THE RESULT OF THE CHECK:
;		LESS OR EQUAL: THE TMB HAS EXPIRED.
;		GREATER:       THE TMB IS STILL RUNNING.
;
X_CHK_EXPIRED	PROC	NEAR
		MOV	AX,X_ABS_TIME	;GET HIGH WORD OF ABSOLUTE TIME.
		CMP	TM_HTIME,AX	;COMPARE TO TMB HIGH WORD.
		JNE	@3		;COMPARISON OVER.
		MOV	AX,X_ABS_TIME+2	;NOT OVER: GET ABSOLUTE MIDDLE WORD
		CMP	TM_MTIME,AX	;COMPARE TO TMB MIDDLE WORD
		JNE	@3		;COMPARISON OVER.
		MOV	AX,X_ABS_TIME+4	;STILL NOT DONE: LOW WORD
		CMP	TM_LTIME,AX	;COMPARED AGAINST TMB LOW WORD
@3:
		RET			;WE CAN CERTAINLY RETURN NOW.
X_CHK_EXPIRED	ENDP
PAGE
;*******************************************
;*	TIMER INTERRUPT HANDLER		   *
;*******************************************
;
;	THE CLOCK INTERRUPT HANDLER MAINTAINS CONTROL OF ALL CURRENTLY ACTIVE
;	TIME MONITOR BLOCKS (TMBS). AT EACH CLOCK TICK, THE FOLLOWING ACTIONS
;	ARE PERFORMED:
;
;	1) THE ABSOLUTE TIME SINCE INITIALIZATION AND THE CURRENT TIME STATIS-
;	   TICS COUNTER ARE INCREMENTED;
;	2) THE HEAD TMB ON THE QUEUE (REMEMBER, THEY'RE LINKED IN ORDER
;	   OF ASCENDING EXPIRATION TIME) IS CHECKED FOR HAVING EXPIRED.
;
;	IN THE CASE WHERE THE HEAD TMB HAS YET TO EXPIRE, THE INTERRUPT HANDLER
;	EXITS AT ONCE. OTHERWISE:
;
;	3) THE CLOCK HANDLER GOES TO S-STATE ;AN INDEFINITELY LONG INTERVAL
;	   MAY ELAPSE WHILE THIS OCCURS.
;	4) THE S-STATE PORTION OF THE CLOCK HANDLER PROCESSES ALL EXPIRED
;	   TMBS ACCORDING TO THEIR TYPES.
;
;	NOTE THAT, UNTIL IT HAS BEEN ESTABLISHED TO BE ABSOLUTELY NECESSARY,
;	THE INTERRUPT HANDLER DOES NOT EXIT DI-STATE. THIS FACILITATES THE
;	HANDLING OF CLOCK INTERRUPTS AT THE HIGHEST POSSIBLE SPEEDS WITH THE
;	LOWEST POSSIBLE TOTAL CPU OVERHEAD.
;
;	NOTE ALSO THAT ONLY THE INITIALIZATION MASTER PERFORMS CLOCK PROCESSING
;	AS DESCRIBED ABOVE. THE OTHER PROCESSORS ONLY USE THEIR CLOCK TICKS
;	FOR LOAD BALANCING.
;
X_PIT_INT	PROC	FAR
		PUSH	AX
		PUSH	SI
		PUSH	DS
;
;	THIS IS WHERE TRUE TIME HANDLING TAKES PLACE.
;
@0:
		MOV	AX,MTOS_DS	;SET UP DS TO 
		MOV	DS,AX		;GLOBAL RAM SEGMENT.
		XOR	AX,AX
		MOV	SI,OFFSET X_ABS_TIME	;UPDATE THE
		ADD	WORD PTR [SI+4],X_PIT_MS	;SYSTEM ABSOLUTE TIME.
		JNC	@6
		ADC	[SI+2],AX
		ADC	[SI],AX
@6:
		MOV	SI,X_TMB_Q	;GET HEAD OF TMB QUEUE
		TEST	SI,SI		;ZERO?
		JE	@1		;YES, NO TIME PROCESSING TO DO.
		CALL	X_CHK_EXPIRED	;NO: CHECK HEAD TMB FOR EXPIRATION:
		JBE	@2		;IT HAS EXPIRED.
@1:
		DEC	X_ROUND_ROBIN	;NO: DECREMENT ROUND ROBIN CT.
		JA	@00		;MORE TO GO.
		MOV	X_ROUND_ROBIN,X_RR_TICKS	;DONE: RELOAD CTR.
@000:
		POP	DS
		POP	SI
		POP	AX
		CALL	X_ELIPSE	;GO THROUGH THE SCHEDULER
		CALL	X_SYSTATE	;FOR THE HELL OF IT.
		RET			;THIS FORCES ROUND-ROBIN RESCHEDULING.
@00:
		CMP	X_SRB_Q,0	;ANY PENDING SRBS?
		JNE	@000		;YES!
		CALL	X_ISSUE_EOI	;NO, ISSUE EOI AND WALK AWAY
		POP	DS
		POP	SI
		POP	AX
		IRET			;RETURN TO INTERRUPTED PROGRAM.
;
;	IF A TMB HAS EXPIRED, THE SYSTEM TRANSITS TO S STATE, REDOES THE
;	EXPIRATION CHECK, AND PROCESSES *ALL* EXPIRED TMBS AT ONCE, WITH
;	ONE EXCEPTION: IF A TYPE 1 TMB IS DISCOVERED TO HAVE EXPIRED, IT
;	IS PROCESSED AND TIME HANDLING IS SHUT DOWN UNTIL THE NEXT TICK.
;
@2:
		POP	DS
		POP	SI
		POP	AX
		CALL	X_ELIPSE
		CALL	X_SYSTATE	;GO TO S STATE.
@5:
		MOV	SI,X_TMB_Q	;SET UP TO SCAN THE TMBS:
		TEST	SI,SI		;ANY TMB?
		JE	@4		;NO, WE'RE DONE.
		CLI			;YES:
		CALL	X_CHK_EXPIRED	;HAS IT EXPIRED?
		STI
		JA	@4		;NO, WE'RE FINISHED.
		MOV	AX,[SI]		;YES: GET FORWARD LINK
		MOV	X_TMB_Q,AX	;AND REMOVE EXPIRED TMB FROM QUEUE.
		MOV	AL,TM_TYPE	;GET THE TMB TYPE
		CBW			;MAKE A WORD FROM IT
		SHL	AX,1		;DOUBLE IT
		MOV	DI,AX		;PUT IT INTO AN INDEX REGISTER
		CALL	WORD PTR CS:X_TAB[DI]	;AND CALL TIME PROCESSOR.
		JMP	@5		;ON RETURN, TRY FOR ANOTHER TMB.
@4:
		RET			;AND EXIT.
X_PIT_INT	ENDP
;
;DUMMY TYPE SERVICE ROUTINE
;
X_TYPE_0	PROC	NEAR
		RET
X_TYPE_0	ENDP
PAGE
;************************************************
;*	X_QUEUE_TMB - ADD A TMB TO TIME QUEUE   *
;************************************************
;
;	TMBS ARE ADDED TO THE TIME QUEUE IN ASCENDING ORDER
;	OF TIME TO EXPIRATION. THE ABSOLUTE TIME OF A TMB'S EXPIRATION IS
;	DETERMINED BY ADDING THE SYSTEM ABSOLUTE TIME AT THE INVOCATION OF
;	X_QUEUE_TMB TO THE COUNTDOWN SPECIFIED IN THE TMB.
;
;	INPUT		- BX= TMB BLOCK ADDR TO ADD
;	OUTPUT		- NONE
;
X_QUEUE_TMB	PROC	NEAR
		PUSH	BX
		PUSH	CX
		PUSH	DX
		PUSH	DI
		PUSH	SI
		MOV	AX,[BX+TMB_TIME+2]	;GET THE MILLISECOND COUNTDOWN
		MOV	CX,[BX+TMB_TIME]	;IN ORDER TO CONVERT IT INTO
		XOR	DX,DX			;A MILLISECOND WALL TIME.
		CLI
		ADD	AX,X_ABS_TIME+4	;THUS...
		ADC	CX,X_ABS_TIME+2
		ADC	DX,X_ABS_TIME	;...SEE?
		STI
		MOV	[BX+TMB_LTIME],AX	;NOW PUT THE ABSOLUTE EXPI-
		MOV	[BX+TMB_MTIME],CX	;RATION TIME INTO THE
		MOV	[BX+TMB_HTIME],DX	;TMB FOR LATER USE.
		MOV	DI,OFFSET X_TMB_Q	;START FROM HEAD OF QUEUE,
		MOV	SI,[DI]			;GET ADDR OF HEAD TMB.
;
;	WE WILL NOW LOOK THROUGH THE TMB QUEUE TO DETERMINE WHERE THE NEW
;	TMB SHOULD BE LINKED IN:
;
;	SI= ADDRESS OF CURRENT-COMPARISON TMB, DI= ADDRESS OF PREVIOUS TMB.
;	BX= ADDRESS OF NEW TMB.
;
TMB_LOOP:
		TEST	SI,SI		;END OF QUEUE?
		JE	TMB_INSERT	;YES, HOOK ON AT TAIL.
		CMP	TM_HTIME,DX	;NO: COMPARE THE TIMES OF EXPIRATION
		JB	GO_DOWN		;OF THE TWO TMBS
		JA	TMB_INSERT	;TO DETERMINE
		CMP	TM_MTIME,CX	;WHETHER TO INSERT
		JB	GO_DOWN		;THE NEW TMB
		JA	TMB_INSERT	;IN FRONT OF
		CMP	TM_LTIME,AX	;THE COMPARISON TMB.
		JAE	TMB_INSERT
GO_DOWN:				;MUST DESCEND FURTHER DOWN TMB LIST.
		MOV	DI,SI		;CURRENT BECOMES PREVIOUS,
		MOV	SI,[SI]		;AND NEXT BECOMES CURRENT.
		JMP	TMB_LOOP	;DO ANOTHER COMPARISON.
TMB_INSERT:				;WE CAN INSERT HERE:
		MOV	[DI],BX		;FLINK(PREVIOUS)=NEW,
		MOV	[BX],SI		;FLINK(NEW)=CURRENT.
QDONE:
		POP	SI
		POP	DI
		POP	DX
		POP	CX
		POP	BX
		RET
X_QUEUE_TMB	ENDP
PAGE
;*****************************************************
;*	X_DQUEUE_TMB - REMOVE A TMB FROM QUEUE       *
;*****************************************************
;
;	IF THE SPECIFIED TMB IS ON THE TIMER QUEUE, IT IS REMOVED.
;
;	INPUT		- BX= TMB ADR TO REMOVE
;	OUTPUT		- CARRY= 0 IF THE BLOCK WAS REMOVED,
;			         1 IF THE BLOCK WAS NOT FOUND.
;
X_DQUEUE_TMB	PROC	NEAR
		PUSH	DI
		MOV	DI,OFFSET X_TMB_Q	;WHILE WE USE IT TO 
		CALL	X_DQUEUE		;CALL GENERAL DEQUEUE ROUTINE
		POP	DI
		RET
X_DQUEUE_TMB	ENDP
PAGE
;	*****************************************
;	*	TIME OUT PROCESSING		*
;	*****************************************
;
;	TIMEOUTS COME IN THE FOLLOWING TYPES:
;			DRIVER - DRIVER SET TIME OUT OCCURS
;			TASK PAUSE - TASK PAUSE IS SATISFIED
;			EVENT FLAG - SET FLAG AFTER TIME IS SATISFIED
;			TASK RESTART - A TERMINATION WITH AUTO RESTART
;					IS READY FOR THE RESTART.
;			TASK PRIORITY - THE TASK HAS TIMED OUT AND PRIORITY
;					MUST BE CHANGED.
;			SEMAPHORE BID TIMEOUT - A TASK'S BID FOR A SEMAPHORE
;			                        OR CONTROLLED PROGRAM IS CAN-
;			                        CELLED, AND THE TASK RETURNED
;			                        TO READY STATE.
;			RESERVE-UNIT TIMEOUT - A TASK'S BID TO RESERVE A UNIT
;			                       OF SOME DEVICE IS CANCELLED, 
;			                       AND THE TASK RETURNED TO READY
;			                       STATE.
;
;	INPUT	- SI= TMB JUST TIMED OUT
;
;	AT ENTRY TO ANY ONE OF THESE ROUTINES, THE TMB WHOSE ADDRESS IS IN
;	SI HAS BEEN DETECTED TO HAVE EXPIRED. IT HAS BEEN DEQUEUED FROM THE
;	TMB QUEUE, AND IS READY FOR WHATEVER KIND OF PROCESSING IT WAS SET
;	UP TO TRIGGER. WHILE AN X_TYPE_n ROUTINE IS RUNNING, THE TMB QUEUE
;	IS UNLOCKED; THE CPU IS IN S STATE.
;
;
;	SUBROUTINE TO FREE A TMB
;
FREETMB		PROC	NEAR
		XCHG	BX,SI		;TIMER BLOCK TO DEALLOCATE
		CALL	X_PUT_SMR	;RETURN TMB TO SMR FREE LIST.
		XCHG	BX,SI
		RET
FREETMB		ENDP
;****************************************************
;* 	TYPE 1 - DRIVER AND TOD CLOCK TIME OUT      *
;****************************************************
;
;	EITHER A DRIVER TMB OR A TIME-OF-DAY TMB HAS EXPIRED. IN EITHER CASE
;	THE ROUTINE WHOSE ADDRESS IS IN TMB_SADR IS INVOKED. DRIVERS HAVE THE
;	OPTION OF DEALLOCATING THE TMB, IF IT IS DESIRABLE.
;
X_TYPE_1	PROC	NEAR
		MOV	BX,TM_UCB		;GET UCB FROM TMB
		MOV	XUCB,BX
		MOV	XUCB+2,DS		;SET UP XUCB
		CALL	DWORD PTR TM_SADR	;CALL DRIVER TIMEOUT ROUTINE.
		RET				;RETURN TO SCHEDULER.
X_TYPE_1	ENDP
;***************************************************
;*	TYPE 2 - TASK PAUSE SATISFIED              *
;***************************************************
;
;	THE TMB CONTAINS THE TCB OF THE TASK TO BE RESUMED SINCE
;	THE PAUSE IT REQUESTED HAS BEEN TIMED OUT.
;	THE TASK IS SET READY AND THE BLOCK RETURNED TO THE MTOS
;	MEMORY RESOURCE.
;
X_TYPE_2	PROC	NEAR
		MOV	BX,TM_TCB		;BX= TCB ADR
		CMP	BYTE PTR [BX+TCB_STATE],BLOCK+STA_PAUSE
		JNE	XT21			;IF NOT STILL PAUSED, ABORT.
		CALL	X_SET_READY		;PUT IT ON READY QUEUE
XT21:
		CALL	FREETMB			;FREE THE TIMER BLOCK
		RET
X_TYPE_2	ENDP
;***************************************************
;*	TYPE 3 - EVENT FLAG SETTING                *
;***************************************************
;
;	THIS ROUTINE IS ENTERED IF A REQUEST TO SET AN EVENT FLAG
;	AFTER A SPECIFIED TIME HAS BEEN SATISFIED. THE EVENT FLAG
;	IS SET AND ANY TASK(S) WAITING FOR IT ARE RETURNED TO READY STATE.
;	THE TMB IS DELETED.
;
X_TYPE_3	PROC	NEAR
		MOV	DL,TM_EFGRP	;GET FLAG GROUP NUMBER
		MOV	CX,TM_EFMSK	;    FLAG MASK WORD
		MOV	BX,TM_TCB	;TCB IN CASE FLAG GROUP=0.
		CALL	X_SET_EF	;SET THE GROUP.
		CALL	FREETMB		;DEALLOCATE THE TMB
		RET
X_TYPE_3	ENDP
;*****************************************************
;*	TYPE 4 - TASK TO RESTART AUTOMATICALLY       *
;*****************************************************
;
;	TIME-OUT RESTART PROCESSING MOVES THE TASK SPECIFIED IN THE TMB FROM
;	A "BLOCKED AWAITING RESTART EXPIRATION" STATE TO READY STATE.
;	THE ASSUMPTION IS MADE THAT THE CALL TO THE ENDR SERVICE HAS RE-
;	SULTED IN THE TASK'S STARTUP CONFIGURATION BEING ENFORCED (I. E.,
;	THE REGISTER AND STACK CONDITION PRODUCED BY A CALL TO X_INIT_TASK).
;	BECAUSE THE TRANSITION IS FROM BLOCKED TO READY AND NOT DORMANT TO
;	READY AS IS USUAL FOR THE START OPERATION, X_SET_READY WILL NOT SET
;	THE TCB_SCHDL LAST-START TIME, WHICH MUST THEREFORE BE DONE EXPLICIT-
;	LY BY THIS ROUTINE. IF THE TMB WAS ASSIGNED FOR THE ENDR ONLY, IT
;	IS DEALLOCATED. IF IT WAS ASSIGNED AT INITIALIZATION TIME FOR DY-
;	NAMIC PRIORITY UPDATE, IT IS RETAINED, RECONFIGURED AND PUT ONTO
;	THE TIMER QUEUE AGAIN.
;
X_TYPE_4	PROC	NEAR
		MOV	BX,TM_TCB		;BX= TCB ADR
		PUSH	BX			;SAVE TCB ADDRESS
		ADD	BX,TCB_SCHDL		;INVOKE THE ROUTINE THAT
		CALL	X_INTIME		;SETS THE LAST-START TIME.
		POP	BX			;RESTORE TCB ADDRESS
		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_OVPRI	;PRIORITY OVER-
						;RIDE SPECIFIED FOR TASK?
		JNE	TYPE45			;YES: RETAIN TMB.
		MOV	WORD PTR [BX+TCB_TMB],0 ;NO: SET TMB ADR TO ZERO
		CALL	FREETMB			;NOT NEEDED - FREE IT
		JMP	SHORT TYPE4RET
TYPE45:
		LES	DI,DWORD PTR [BX+TCB_TCD]	;GET TCD ADDRESS,
		MOV	AX,ES:[DI+TCD_TIME]	;GET THE TASK'S OVERRIDE-
		CALL	X_GET_INTERVAL		;TIMEOUT VALUE FROM
		MOV	TM_MTIME,CX		;THE TCD TO THE TMB.
		MOV	TM_LTIME,DX
		MOV	[SI+TMB_OVTIME],CX	;ALSO SAVE THEM IN THE TWO
		MOV	[SI+TMB_OVTIME+2],DX	;REFRESHER WORDS FOR TYPE 5'S
		MOV	AX,ES:[DI+TCD_OVPRI]	;OVERRIDE AND BOUNDARY PRIORITY
		MOV	TM_OVPRI,AX		;ALSO GO INTO TYPE 5 TMB.
		MOV	TM_TYPE,5		;TYPE= PRIORITY OVERRIDE
		XCHG	BX,SI
		CALL	X_QUEUE_TMB		;START THE TMB
		XCHG	BX,SI
TYPE4RET:
		CALL	X_SET_READY		;READY THE TASK,
		RET
X_TYPE_4	ENDP
;*********************************************************
;*	TYPE 5 - PRIORITY OVERRIDE TIMER PROCESSING      *
;*********************************************************
;
;	THIS ROUTINE UPDATES THE PRIORITY OF THE ASSOCIATED TASK FROM WHAT-
;	EVER IT MIGHT CURRENTLY BE TO THAT PLUS ITS PRIORITY-OVERRIDE INCRE-
;	MENT, UNLESS THE STATED BOUNDARY VALUE HAS ALREADY BEEN ATTAINED. IF
;	THAT BOUNDARY HAS BEEN REACHED, THE TIMER IS DISMISSED BUT NOT DEAL-
;	LOCATED. ELSE, AFTER THE PRIORITY OF THE TASK HAS BEEN ADJUSTED, THE
;	TIMER IS REQUEUED.
;
X_TYPE_5	PROC	NEAR
		MOV	BX,TM_TCB		;GET TCB ADDRESS OF TASK.
		MOV	AX,TM_OVPRI		;GET INCREMENT AND BOUNDARY.
		CMP	AH,[BX+TCB_PRI]		;BOUNDARY MET?
		JNE	T5A			;NO, DO THE ADD.
		RET				;YES: NO FURTHER WORK.
T5A:
		ADD	[BX+TCB_PRI],AL		;ADD THE INCREMENT.
		CMP	BYTE PTR [BX+TCB_STATE],READY	;TASK READY?
		JNE	T5B			;NO.
		MOV	AX,BLOCK		;YES: REMOVE IT FROM
		CALL	X_TASK_STATE		;READY QUEUE AND
		CALL	X_SET_READY		;PUT IT BACK IN NEW PLACE.
T5B:
		MOV	AX,[SI+TMB_OVTIME]	;REFRESH THE COUNTDOWN FROM
		MOV	[SI+TMB_TIME],AX	;THE OVERRIDE TIME STORED
		MOV	AX,[SI+TMB_OVTIME+2]	;FURTHER DOWN IN THE TMB.
		MOV	[SI+TMB_TIME+2],AX
		MOV	BX,SI
		CALL	X_QUEUE_TMB		;REACTIVATE THE TIMER.
		RET
X_TYPE_5	ENDP
;************************************************************
;*         TYPE 6: SEMAPHORE BID TIMEOUT.                   *
;************************************************************
;
;		IF THE ASSOCIATED SEMAPHORE BID, REPRESENTED BY A SWB ON A
;	SEMAPHORE WAIT QUEUE, HAS NOT SUCCEEDED YET, THE BID IS CANCELLED:
;	THE SWB IS DISCARDED, AND THE BIDDING TASK IS RETURNED TO READY
;	STATE WITH A FAILURE STATUS.
;
X_TYPE_6	PROC	NEAR
		MOV	DI,TM_SEM		;GET SEMAPHORE ADDRESS
		MOV	BX,TM_SWB		;GET SWB ADDRESS
		MOV	BP,TM_TCB		;GET TCB ADDRESS
		CALL	FREETMB			;FREE THE TMB.
		ADD	DI,SEM_WAIT		;AND REMOVE SWB
		CALL	X_DQUEUE		;FROM SEMAPHORE WAIT QUEUE.
		JC	XT60			;NOT FOUND: OPERATION FINISHED!
		CALL	X_PUT_SMR		;DEALLOCATE THE SWB.
		MOV	BX,BP			;GET ADDRESS OF TCB TO BX.
		LES	BP,DWORD PTR [BX+TCB_SP]	;GO TO TASK STACK,
		OR	WORD PTR ES:[BP+STK_FL],SVC_FAILED+FLAG_OF
						;DELIVER FAILURE STATUS.
		CALL	X_SET_READY		;RETURN TASK TO READY STATE.
XT60:
		RET
X_TYPE_6	ENDP
;;**********************************************************************
;*        TYPE 7: RESERVE-UNIT BID CANCELLATION.                       *
;***********************************************************************
;
;	THE REQUEST TO RESERVE A PERIPHERAL UNIT IS CANCELLED, AND THE RE-
;	QUESTING TASK RECEIVES A TIMEOUT STATUS.
;
X_TYPE_7	PROC	NEAR
		MOV	DI,TM_IRB		;GET IRB ADDRESS
		CALL	FREETMB			;TMB NOW NOT NEEDED.
		MOV	XCIRB,DI		;SET INTERFACE VARIABLE.
		MOV	XPRIMARY,TIME_OUT	;SET COMPLETION STATUS.
		POP	AX			;DESTROY "NEAR" RETURN ADDR AND
		JMP	XOVER			;COMPLETE THE REQUEST THUS.
X_TYPE_7	ENDP
MTOS_CS		ENDS
		END
