PAGE	60,132
;
;	PLM.ASM:	PL/M-86 -TO- MTOS-86 INTERFACE
;	F. W. PORRETTO,	8/10/82
;
;	THIS MODULE CONTAINS ALL THE CODE REQUIRED TO PROVIDE A SERVICE
;	INTERFACE USABLE BY INTEL PL/M-86 PROGRAMS. ALL THE BASIC SERVICES
;	ARE SUPPORTED. ANY HIGH-LEVEL LANGUAGE WHOSE PARAMETER-PASSING CON-
;	VENTIONS ARE A CLOSE MATCH TO THOSE OF PL/M-86 MAY BE FITTED TO THIS
;	INTERFACE, THOUGH SOME CUSTOMIZATION OF THESE ROUTINES MAY BE REQUIRED.
;
;	THE ROUTINES WHICH PROVIDE THE INTERFACE ARE MULTIPROCESSOR-CO-ENTRANT,
;	A REQUIREMENT FOR THIS PARTICULAR APPLICATION. THE REQUIRED SDBS ARE
;	BUILT ON THE REQUESTING TASK'S STACK, AND SUBMITTED TO MTOS VIA THE
;	TYPE 1 SVC (SDB ADDRESS PUSHED BEFORE INTERRUPT INSTRUCTION). THERE IS
;	A REQUIREMENT FOR A RATHER UNUSUAL KIND OF STACK MAINTENANCE, OWING
;	TO THE ORDER OF EVENTS AND THE WAY IN WHICH VARIOUS REQUIRED ITEMS ARE
;	SAVED:
;
;	1)AT THE "BOTTOM" OF THE STACK WILL BE THE USER'S PARAMETERS. PL/M-86
;	  MAINTAINS A CALL-BY-VALUE CONVENTION, AND PUSHES ITS ARGUMENTS ONTO
;	  THE STACK IN INVERSE ORDER OF POSITION IN THE CALLING SEQUENCE.
;
;	2)ABOVE THAT IS THE RETURN ADDRESS TO THE TASK, WHICH IS ASSUMED TO BE
;	  A PL/M PROCEDURE DECLARED LARGE.
;
;	3)ABOVE THAT, THE INTERFACE ROUTINES ALLOCATE SPACE IN WHICH TO BUILD
;	  THE SDB FOR THE REQUESTED FUNCTION. IT SHOULD ALSO BE OBSERVED HERE
;	  THAT THE INTERFACE DOES ALL NECESSARY SHIFTING AND PARAMETER COMBI-
;	  NING, BUT NO VALIDATION.
;
;	4)ABOVE THAT, AT THE TOP, ARE THE REGISTERS OF THE CALLING TASK, SAVED
;	  AS NECESSARY, AND FOR A SHORT TIME THE ADDRESS OF THE SDB AS IT IS
;	  SUBMITTED TO MTOS.
;
;	THE READER WILL NOTE THAT THE FUNDAMENTAL OPERATIONS OF ANY SELECTED
;	ROUTINE ARE MUCH THE SAME AS THOSE OF ANY OTHER. TO CONSERVE MEMORY,
;	SEVERAL ROUTINES SHARE PROCESSING WITH ONE ANOTHER. THE ACTUAL GOINGS-
;	ON SHOULD BE FAIRLY OBVIOUS, GIVEN A KNOWLEDGE OF THE MTOS-86 SDB FOR-
;	MATS AND BEARING IN MIND THE STACK CONTROL SCHEME DESCRIBED ABOVE.
;
		NAME	PLM_TO_MTOS
;
PLM_CS		SEGMENT PARA PUBLIC
		ASSUME	CS:PLM_CS
		PUBLIC	ALOC,CNTRL,CPRI,DALOC,DSCRTE,ENDT,ENDR,PAUSE
		PUBLIC	QSTRT,STRT,RECV,REF,RESUME,RIO,RSF,SEF
		PUBLIC	SEFET,SEND,STIME,TEF,TIME,TSF,WEF,QCOPY
		PUBLIC	CREATE,DELETE,DELTSK,LOADED,HSFS
                PUBLIC	GETPARM,SVCOK
;
;	SPECIAL NOTE!!
;
;		IN THIS, THE IBM PC MACROASSEMBLER VERSION OF MTOS-86UP V3.0,
;	IT HAS BEEN NECESSARY TO MAKE A SLIGHT CHANGE TO ACCOMMODATE A FAULT
;	IN THE ASSEMBLER: THE VECTOR NUMBERS FOR THE TWO SVC INTERRUPT IN-
;	STRUCTIONS ARE DECLARED LOCALLY TO THIS MODULE, THUS:
;
X_SVC_VECTOR	EQU	186
;
PAGE
;
;	TYPE 1 SVC GENERATOR:
;
;	THIS ROUTINE PRESUMES THAT AT SS:BP IS THE SDB, AND THAT AT 
;	SS:[BP][DI] IS THE PL/M-86 CALLING PROCEDURE'S STATUS VARIABLE,
;	INTO WHICH THE FLAGS REGISTER IS FORCED ON RETURN FROM MTOS.
;
MTOS		PROC	NEAR
		PUSH	SS
		PUSH	BP
		INT	LOW X_SVC_VECTOR
		POP	BP
		POP	SS
		PUSHF
		LDS	DI,DWORD PTR [BP][DI]	;GET ADDRESS OF STATUS WORD
		POP	WORD PTR [DI]		;RETURN STATUS WORD
		RET
MTOS		ENDP
PAGE
;
;	SYMBOLS FOR ALOC:
;
ALOCS_WTID	EQU	1
ALOCS_SZE	EQU	2
ALOCS_BUFR	EQU	4
ALOCS_SIZE	EQU	8	;ALLOCATED FOR SDB.
ALOC_STS	EQU	12
ALOC_BUFR	EQU	16
ALOC_SZE	EQU	20
ALOC_ID		EQU	22
ALOC_WT		EQU	24
ALOC_SIZE	EQU	14	;USED BY PL/M-86.
;
;
ALOC		PROC	FAR
		SUB	SP,ALOCS_SIZE		;ALLOW FOR SDB
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],0FH	;ALOC SERVICE CODE.
		MOV	AL,[BP+ALOC_WT]
		ROR	AL,1
		OR	AL,[BP+ALOC_ID]
		MOV	[BP+ALOCS_WTID],AL	;WAIT PARAM AND POOL ID.
		MOV	AX,[BP+ALOC_SZE]
		MOV	[BP+ALOCS_SZE],AX	;SIZE OF BLOCK NEEDED.
		LDS	DI,DWORD PTR [BP+ALOC_BUFR]
		MOV	[BP+ALOCS_BUFR],DI	;BUFFER OFFSET
		MOV	[BP+ALOCS_BUFR+2],DS	;       SEGMENT
		MOV	DI,ALOC_STS
		CALL	MTOS			;TYPE 1 SVC
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,ALOCS_SIZE		;DESTROY SDB
		RET	ALOC_SIZE		;DONE.
ALOC		ENDP
PAGE
;
;	SYMBOLS FOR CNTRL:
;
CNTRLS_SE	EQU	1
CNTRLS_CID	EQU	2
CNTRLS_SIZE	EQU	4	;ALLOCATED FOR SDB.
CNTRL_STS	EQU	8
CNTRL_CID	EQU	12
CNTRL_SE	EQU	14
CNTRL_SIZE	EQU	8	;USED BY PL/M-86
;
;
CNTRL		PROC	FAR
		SUB	SP,CNTRLS_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	AL,[BP+CNTRL_SE]
		ROR	AL,1
		MOV	[BP+CNTRLS_SE],AL	;ENTER/EXIT FLAG.
		MOV	AL,[BP+CNTRL_CID]
		MOV	[BP+CNTRLS_CID],AL	;CONTROLLED PROGRAM ID.
		MOV	BYTE PTR [BP],011H	;CNTRL CODE.
		MOV	DI,CNTRL_STS
		CALL	MTOS			;TYPE 1 SVC.
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,CNTRLS_SIZE		;DESTROY SDB
		RET	CNTRL_SIZE
CNTRL		ENDP
PAGE
;
;	SYMBOLS FOR CPRI:
;
CPRIS_NPS	EQU	1
CPRIS_PRIV	EQU	2
CPRIS_SIZE	EQU	4	;ALLOCATED FOR SDB.
CPRI_STS	EQU	8
CPRI_PRIV	EQU	12
CPRI_NPS	EQU	14
CPRI_SIZE	EQU	8	;USED BY PL/M-86.
;
;
CPRI		PROC	FAR
		SUB	SP,CPRIS_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	AL,[BP+CPRI_NPS]
		MOV	[BP+CPRIS_NPS],AL	;PRIORITY SELECTOR.
		MOV	AL,[BP+CPRI_PRIV]
		MOV	[BP+CPRIS_PRIV],AL	;(MAYBE) PRIORITY VALUE.
		MOV	BYTE PTR [BP],05H	;CPRI CODE.
		MOV	DI,CPRI_STS
		CALL	MTOS			;TYPE 1 SVC.
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,CPRIS_SIZE		;DESTROY SDB.
		RET	CPRI_SIZE
CPRI		ENDP
PAGE
;
;	SYMBOLS FOR DALOC:
;
DALOCS_SZE	EQU	2
DALOCS_BUFR	EQU	4
DALOCS_SIZE	EQU	8	;ALLOCATED FOR SDB.
DALOC_STS	EQU	12
DALOC_BUFR	EQU	16
DALOC_SZE	EQU	20
DALOC_SIZE	EQU	10	;USED BY PL/M-86.
;
;
DALOC		PROC	FAR
		SUB	SP,DALOCS_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	AX,[BP+DALOC_SZE]
		MOV	[BP+DALOCS_SZE],AX	;SIZE OF BLOCK.
		LDS	DI,DWORD PTR [BP+DALOC_BUFR]
		MOV	[BP+DALOCS_BUFR],DI
		MOV	[BP+DALOCS_BUFR+2],DS	;BUFFER POINTER
		MOV	BYTE PTR [BP],010H	;DALOC CODE.
		MOV	DI,DALOC_STS
		CALL	MTOS			;TYPE 1 SVC.
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,DALOCS_SIZE		;DESTROY SDB.
		RET	DALOC_SIZE
DALOC		ENDP
PAGE
;
;	SYMBOLS FOR DSCRTE:
;
DSCRTES_IOTYP	EQU	1
DSCRTES_GRP	EQU	2
DSCRTES_BITS	EQU	4
DSCRTES_SIZE	EQU	6	;ALLOCATED FOR SDB.
DSCRTE_STS	EQU	10
DSCRTE_BITS	EQU	14
DSCRTE_GRP	EQU	16
DSCRTE_TYP	EQU	18
DSCRTE_IO	EQU	20
DSCRTE_SIZE	EQU	12	;USED BY PL/M-86.
;
;
DSCRTE		PROC	FAR
		SUB	SP,DSCRTES_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],014H	;DSCRTE CODE.
		MOV	AL,[BP+DSCRTE_TYP]
		ROR	AL,1
		OR	AL,[BP+DSCRTE_IO]
		MOV	[BP+DSCRTES_IOTYP],AL	;IO AND TYPE COMBINED.
		MOV	AX,[BP+DSCRTE_GRP]
		MOV	[BP+DSCRTES_GRP],AX	;GROUP NUMBER
		MOV	AL,[BP+DSCRTE_BITS]
		MOV	[BP+DSCRTES_BITS],AX	;BIT PATTERN.
		MOV	DI,DSCRTE_STS
		CALL	MTOS			;TYPE 1 SVC
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,DSCRTES_SIZE		;DESTROY SDB.
		RET	DSCRTE_SIZE
DSCRTE		ENDP
PAGE
;
;	SYMBOLS FOR ENDR:
;
ENDRS_RSBTME	EQU	1
ENDRS_LEN	EQU	2
ENDRS_SIZE	EQU	4	;ALLOCATED FOR SDB.
ENDR_LEN	EQU	8
ENDR_TME	EQU	10
ENDR_RSB	EQU	12
;
;
ENDR		PROC	FAR
		SUB	SP,ENDRS_SIZE		;ALLOW FOR SDB.
		MOV	BP,SP			;MARK THE SDB.
		MOV	BYTE PTR [BP],04H	;ENDR CODE.
		MOV	AL,[BP+ENDR_RSB]
		ROR	AL,1
		MOV	AH,[BP+ENDR_TME]
		SHL	AH,1
		SHL	AH,1
		OR	AL,AH
		MOV	[BP+ENDRS_RSBTME],AL	;RESTART BASE AND UNIT COMBINED
		MOV	AL,[BP+ENDR_LEN]
		MOV	[BP+ENDRS_LEN],AL	;NR OF TIME UNITS
		CALL	MTOS			;NO RETURN FROM ENDR.
ENDR		ENDP
;
ENDT		PROC	FAR
		INT	LOW X_SVC_VECTOR+1
		DD	ENDT_SDB
ENDT		ENDP
ENDT_SDB	DB	03H		;ENDT CODE.
PAGE
;
;	SYMBOLS FOR PAUSE:
;
PAUSES_TME	EQU	1
PAUSES_LEN	EQU	2
PAUSES_SIZE	EQU	4
PAUSE_STS	EQU	8
PAUSE_LEN	EQU	12
PAUSE_TME	EQU	14
PAUSE_SIZE	EQU	8
;
;
PAUSE		PROC	FAR
		SUB	SP,PAUSES_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],06H	;PAUSE CODE.
		MOV	AL,[BP+PAUSE_TME]
		SHL	AL,1
		SHL	AL,1
		MOV	[BP+PAUSES_TME],AL	;TIME UNIT
		MOV	AL,[BP+PAUSE_LEN]
		MOV	[BP+PAUSES_LEN],AL	;NUMBER OF UNITS.
		MOV	DI,PAUSE_STS
		CALL	MTOS			;TYPE 1 SVC
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,PAUSES_SIZE		;DESTROY SDB
		RET	PAUSE_SIZE
PAUSE		ENDP
PAGE
;
;	SYMBOLS FOR STRT AND QSTRT:
;
STRTS_TP	EQU	1
STRTS_TASK	EQU	2
STRTS_PRIV	EQU	4
STRTS_P1	EQU	5
STRTS_P2	EQU	7
STRTS_SIZE	EQU	10	;ALLOCATED FOR SDB.
STRT_STS	EQU	14
STRT_P2		EQU	18
STRT_P1		EQU	20
STRT_PRIV	EQU	22
STRT_PRIS	EQU	24
STRT_TASK	EQU	26
STRT_TCORD	EQU	28
STRT_SIZE	EQU	16	;USED BY PL/M-86.
;
;
QSTRT		PROC	FAR
		SUB	SP,STRTS_SIZE		;ALLOW FOR SDB.
		PUSH	CX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],02H	;QSTRT CODE.
		JMP	STRT_IN			;TRANSFER TO STRT ROUTINE.
QSTRT		ENDP
;
STRT		PROC	FAR
		SUB	SP,STRTS_SIZE
		PUSH	CX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB AS ABOVE.
		MOV	BYTE PTR [BP],01H	;STRT CODE.
STRT_IN		LABEL	NEAR
		MOV	AL,[BP+STRT_TCORD]
		MOV	CL,5
		SHL	AL,CL
		OR	AL,[BP+STRT_PRIS]
		MOV	[BP+STRTS_TP],AL	;COORDINATION AND PRIORITY SEL.
		MOV	AX,[BP+STRT_TASK]
		MOV	[BP+STRTS_TASK],AX	;TASK NUMBER.
		MOV	AL,[BP+STRT_PRIV]
		MOV	[BP+STRTS_PRIV],AL	;(MAYBE) PRIORITY VALUE.
		MOV	AX,[BP+STRT_P1]
		MOV	[BP+STRTS_P1],AX	;PARAMETER 1
		MOV	AX,[BP+STRT_P2]
		MOV	[BP+STRTS_P2],AX	;PARAMETER 2
		MOV	DI,STRT_STS
		CALL	MTOS			;TYPE 1 SVC
		POP	DS
		POP	DI
		POP	BP
		POP	CX
		ADD	SP,STRTS_SIZE		;DESTROY SDB.
		RET	STRT_SIZE
STRT		ENDP
PAGE
;
;	SYMBOLS FOR RECV:
;
RECVS_WT	EQU	1
RECVS_0	EQU	2
RECVS_BOX	EQU	4
RECVS_BUFR	EQU	5
RECVS_SIZE	EQU	10	;ALLOCATED FOR SDB.
RECV_STS	EQU	14
RECV_BUFR	EQU	18
RECV_BOX	EQU	22
RECV_WT	EQU	24
RECV_SIZE	EQU	12	;USED BY PL/M-86
;
;
RECV		PROC	FAR
		SUB	SP,RECVS_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	AL,[BP+RECV_WT]
		ROR	AL,1
		MOV	[BP+RECVS_WT],AL	;WAIT PARAMETER.
		MOV	WORD PTR [BP+RECVS_0],0	;ZAP THE RESERVED FIELDS.
		MOV	AL,[BP+RECV_BOX]
		MOV	[BP+RECVS_BOX],AL	;BOX NUMBER
		LDS	DI,DWORD PTR [BP+RECV_BUFR]
		MOV	[BP+RECVS_BUFR],DI	;BUFFER OFFSET,
		MOV	[BP+RECVS_BUFR+2],DS	;SEGMENT.
		MOV	BYTE PTR [BP],0EH	;RECV CODE.
		MOV	DI,RECV_STS
		CALL	MTOS			;TYPE 1 SVC.
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,RECVS_SIZE		;DESTROY SDB.
		RET	RECV_SIZE
RECV		ENDP
PAGE
;
;	SYMBOLS FOR REF, SEF
;
REFS_GRP	EQU	1
REFS_FLAGS	EQU	2
REFS_SIZE	EQU	4	;ALLOCATED FOR SDB.
REF_STS		EQU	8
REF_FLAGS	EQU	12
REF_GRP		EQU	14
REF_SIZE	EQU	8	;USED BY PL/M-86.
;
;
REF		PROC	FAR
		SUB	SP,REFS_SIZE
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8		;MARK THE SDB.
		MOV	BYTE PTR [BP],09H	;REF CODE.
REF_IN		LABEL	NEAR
		MOV	AL,[BP+REF_GRP]
		MOV	[BP+REFS_GRP],AL	;GROUP NUMBER.
		MOV	AX,[BP+REF_FLAGS]
		MOV	[BP+REFS_FLAGS],AX	;FLAG PATTERN.
		MOV	DI,REF_STS
		CALL	MTOS			;TYPE 1 SVC
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,REFS_SIZE		;DESTROY THE SDB
		RET	REF_SIZE
REF		ENDP
;
SEF		PROC	FAR
		SUB	SP,REFS_SIZE
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],08H	;SEF CODE.
		JMP	REF_IN
SEF		ENDP
PAGE
;
;	SYMBOLS FOR RESUME:
;
RESUMES_TASK	EQU	1
RESUMES_SIZE	EQU	4	;ALLOCATED FOR SDB.
RESUME_STS	EQU	8
RESUME_TASK	EQU	12
RESUME_SIZE	EQU	6	;USED BY PL/M-86
;
;
RESUME		PROC	FAR
		SUB	SP,RESUMES_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],07H	;RESUME CODE.
		MOV	AX,[BP+RESUME_TASK]
		MOV	[BP+RESUMES_TASK],AX	;TASK NUMBER.
		MOV	DI,RESUME_STS
		CALL	MTOS			;TYPE 1 SVC.
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,RESUMES_SIZE		;DESTROY THE SDB.
		RET	RESUME_SIZE
RESUME		ENDP
PAGE
;
;	SYMBOLS FOR RIO:
;
RIOS_FUNC	EQU	1
RIOS_DEV	EQU	2
RIOS_UNIT	EQU	3
RIOS_GRP	EQU	4
RIOS_FLAGS	EQU	5
RIOS_STA	EQU	7
RIOS_PARAM	EQU	11
RIOS_SIZE	EQU	16	;ALLOCATED FOR SDB.
RIO_STS		EQU	20
RIO_PARAM	EQU	24
RIO_STA		EQU	28
RIO_FLAGS	EQU	32
RIO_GRP		EQU	34
RIO_UNIT	EQU	36
RIO_DEV		EQU	38
RIO_FUNC	EQU	40
RIO_SIZE	EQU	22	;USED BY PL/M-86.
;
;
RIO		PROC	FAR
		SUB	SP,RIOS_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],015H	;RIO CODE.
		MOV	AL,[BP+RIO_FUNC]
		MOV	[BP+RIOS_FUNC],AL	;I/O FUNCTION CODE.
		MOV	AL,[BP+RIO_DEV]
		MOV	[BP+RIOS_DEV],AL	;DEVICE
		MOV	AL,[BP+RIO_UNIT]
		MOV	[BP+RIOS_UNIT],AL	;UNIT
		MOV	AL,[BP+RIO_GRP]
		MOV	[BP+RIOS_GRP],AL	;FLAG GROUP
		MOV	AX,[BP+RIO_FLAGS]
		MOV	[BP+RIOS_FLAGS],AX	;FLAG PATTERN
		LDS	DI,DWORD PTR [BP+RIO_STA]
		MOV	[BP+RIOS_STA],DI
		MOV	[BP+RIOS_STA+2],DS	;STATUS BUFFER POINTER
		LDS	DI,DWORD PTR [BP+RIO_PARAM]
		MOV	[BP+RIOS_PARAM],DI
		MOV	[BP+RIOS_PARAM+2],DS	;PARAMETER STRING POINTER.
		MOV	DI,RIO_STS
		CALL	MTOS			;TYPE 1 SVC
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,RIOS_SIZE		;DESTROY THE SDB
		RET	RIO_SIZE
RIO		ENDP
PAGE
;
;	SYMBOLS FOR RSF:
;
RSFS_SID	EQU	1
RSFS_SIZE	EQU	4	;ALLOCATED FOR SDB.
RSF_STS		EQU	8
RSF_SID		EQU	12
RSF_SIZE	EQU	6	;USED BY PL/M-86.
;
;
RSF		PROC	FAR
		SUB	SP,RSFS_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],16H	;RSF CODE.
RSF_IN		LABEL	NEAR
		MOV	AX,[BP+RSF_SID]
		MOV	[BP+RSFS_SID],AX	;SEMAPHORE NUMBER.
		MOV	DI,RSF_STS
		CALL	MTOS			;TYPE 1 SVC.
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,RSFS_SIZE		;DESTROY THE SDB.
		RET	RSF_SIZE
RSF		ENDP
;
TSF		PROC	FAR
		SUB	SP,RSFS_SIZE
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],017H	;TSF CODE.
		JMP	RSF_IN
TSF		ENDP
PAGE
;
;	SYMBOLS FOR SEFET:
;
SEFETS_GRP	EQU	1
SEFETS_FLAGS	EQU	2
SEFETS_TME	EQU	4
SEFETS_LEN	EQU	5
SEFETS_SIZE	EQU	6	;ALLOCATED FOR SDB.
SEFET_STS	EQU	10
SEFET_LEN	EQU	14
SEFET_TME	EQU	16
SEFET_FLAGS	EQU	18
SEFET_GRP	EQU	20
SEFET_SIZE	EQU	12	;USED BY PL/M-86.
;
;
SEFET		PROC	FAR
		SUB	SP,SEFETS_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],0BH	;SEFET CODE.
		MOV	AL,[BP+SEFET_LEN]
		MOV	[BP+SEFETS_LEN],AL	;NUMBER OF TIME UNITS.
		MOV	AL,[BP+SEFET_TME]
		SHL	AL,1
		SHL	AL,1
		MOV	[BP+SEFETS_TME],AL	;TIME UNIT CODE.
		MOV	AX,[BP+SEFET_FLAGS]
		MOV	[BP+SEFETS_FLAGS],AX	;FLAG PATTERN
		MOV	AL,[BP+SEFET_GRP]
		MOV	[BP+SEFETS_GRP],AL	;FLAG GROUP NUMBER.
		MOV	DI,SEFET_STS
		CALL	MTOS			;TYPE 1 SVC.
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,SEFETS_SIZE		;DESTROY SDB.
		RET	SEFET_SIZE
SEFET		ENDP
PAGE
;
;	SYMBOLS FOR SEND:
;
SENDS_WPS	EQU	1
SENDS_PRIV	EQU	2
SENDS_NODE	EQU	3
SENDS_BOX	EQU	4
SENDS_BUFR	EQU	5
SENDS_SIZE	EQU	10	;ALLOCATED FOR SDB.
SEND_STS	EQU	14
SEND_BUFR	EQU	18
SEND_BOX	EQU	22
SEND_PRIV	EQU	24
SEND_PRIS	EQU	26
SEND_WT		EQU	28
SEND_SIZE	EQU	16	;USED BY PL/M-86.
;
;
SEND		PROC	FAR
		SUB	SP,SENDS_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],0DH	;SEND CODE.
		MOV	AL,[BP+SEND_WT]
		ROR	AL,1
		OR	AL,[BP+SEND_PRIS]
		MOV	[BP+SENDS_WPS],AL	;WAIT MODE AND PRIORITY SELECT.
		MOV	AL,[BP+SEND_PRIV]
		MOV	[BP+SENDS_PRIV],AL	;(MAYBE) PRIORITY VALUE.
		MOV	BYTE PTR [BP+SENDS_NODE],0	;ZAP NODE NUMBER.
		MOV	AL,[BP+SEND_BOX]
		MOV	[BP+SENDS_BOX],AL	;MAILBOX NUMBER.
		LDS	DI,DWORD PTR [BP+SEND_BUFR]
		MOV	[BP+SENDS_BUFR],DI	;BUFFER OFFSET
		MOV	[BP+SENDS_BUFR+2],DS	;       SEGMENT.
		MOV	DI,SEND_STS
		CALL	MTOS			;TYPE 1 SVC
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,SENDS_SIZE		;DESTROY THE SDB.
		RET 	SEND_SIZE
SEND		ENDP
PAGE
;
;	SYMBOLS FOR TIME, STIME:
;
TIMES_FMT	EQU	1
TIMES_BUFR	EQU	2
TIMES_SIZE	EQU	6	;ALLOCATED FOR SDB.
TIME_STS	EQU	10
TIME_BUFR	EQU	14
TIME_FMT	EQU	18
TIME_SIZE	EQU	10	;USED BY PL/M-86.
;
;
STIME		PROC	FAR
		SUB	SP,TIMES_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],013H	;STIME CODE.
		JMP	TIME_IN
STIME		ENDP
;
TIME		PROC	FAR
		SUB	SP,TIMES_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],012H	;TIME CODE.
TIME_IN		LABEL	NEAR
		MOV	BYTE PTR [BP+TIMES_FMT],0	;ZAP FORMAT CODE.
		LDS	DI,DWORD PTR [BP+TIME_BUFR]
		MOV	[BP+TIMES_BUFR],DI	;BUFFER OFFSET,
		MOV	[BP+TIMES_BUFR+2],DS	;       SEGMENT.
		MOV	DI,TIME_STS
		CALL	MTOS			;TYPE 1 SVC
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,TIMES_SIZE		;DESTROY THE SDB.
		RET	TIME_SIZE
TIME		ENDP
PAGE
;
;	SYMBOLS FOR TEF, WEF:
;
TEFS_TGRP	EQU	1
TEFS_FLAGS	EQU	2
TEFS_SIZE	EQU	4	;ALLOCATED FOR SDB.
TEF_STS		EQU	8
TEF_FLAGS	EQU	12
TEF_GRP		EQU	14
TEF_TYP		EQU	16
TEF_SIZE	EQU	10	;USED BY PL/M-86.
;
;
TEF		PROC	FAR
		SUB	SP,TEFS_SIZE
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],0AH	;TEF CODE.
TEF_IN		LABEL	NEAR
		MOV	AL,[BP+TEF_TYP]
		ROR	AL,1
		OR	AL,[BP+TEF_GRP]
		MOV	[BP+TEFS_TGRP],AL	;TEST TYPE AND GROUP NUMBER
		MOV	AX,[BP+TEF_FLAGS]
		MOV	[BP+TEFS_FLAGS],AX	;FLAG PATTERN.
		MOV	DI,TEF_STS
		CALL	MTOS			;TYPE 1 SVC
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,TEFS_SIZE		;DESTROY THE SDB.
		RET	TEF_SIZE
TEF		ENDP
;
WEF		PROC	FAR
		SUB	SP,TEFS_SIZE
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],0CH	;WEF CODE.
		JMP	TEF_IN
WEF		ENDP
PAGE
;
;	SYMBOLS FOR QCOPY:
;
QCOPYS_BUFR	EQU	1
QCOPYS_SIZE	EQU	6	;ALLOCATED FOR SDB
QCOPY_STS	EQU	10
QCOPY_BUFR	EQU	14
QCOPY_SIZE	EQU	8	;USED BY PL/M-86.
;
;
QCOPY		PROC	FAR
		SUB	SP,QCOPYS_SIZE
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],018H	;QCOPY CODE.
		LDS	DI,DWORD PTR [BP+QCOPY_BUFR]
		MOV	[BP+QCOPYS_BUFR],DI	;BUFFER OFFSET,
		MOV	[BP+QCOPYS_BUFR+2],DS	;       SEGMENT.
		MOV	DI,QCOPY_STS
		CALL	MTOS
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,QCOPYS_SIZE		;DESTROY THE SDB.
		RET	QCOPY_SIZE
QCOPY		ENDP
PAGE
;
;	SYMBOLS FOR CREATE:
;
CREATES_TCD	EQU	1
CREATES_BUFR	EQU	5
CREATES_SIZE	EQU	10	;ALLOCATED FOR SDB.
CREATE_STS	EQU	14
CREATE_BUFR	EQU	18
CREATE_TCD	EQU	22
CREATE_SIZE	EQU	12	;USED BY PL/M-86.
CREATE		PROC	FAR
		SUB	SP,CREATES_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],1AH	;CREATE CODE.
		LDS	DI,DWORD PTR [BP+CREATE_TCD]	;ADDRESS OF TCD IMAGE
		MOV	[BP+CREATES_TCD],DI
		MOV	[BP+CREATES_TCD+2],DS	;...TO SDB.
		LDS	DI,DWORD PTR [BP+CREATE_BUFR]	;TASK ID BUFFER ADDR.
		MOV	[BP+CREATES_BUFR],DI
		MOV	[BP+CREATES_BUFR+2],DS		;... TO SDB.
		MOV	DI,CREATE_STS
		CALL	MTOS			;TYPE 1 SVC.
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,CREATES_SIZE		;DESTROY SDB.
		RET	CREATE_SIZE
CREATE		ENDP
PAGE
;
;	DELETE SERVICE NEEDS NO SYMBOLS, SINCE THE SDB IS ONLY A FUNCTION CODE
;
DELETE		PROC	FAR
		INT	LOW X_SVC_VECTOR+1	;TYPE 2 SVC
		DD	DELETE_SDB
DELETE		ENDP
DELETE_SDB	DB	01BH			;DELETE CODE.
;
;	SYMBOLS FOR DELTSK:
;
DELTSKS_ID	EQU	1
DELTSKS_SIZE	EQU	4		;ALLOCATED FOR SDB.
DELTSK_STS	EQU	8
DELTSK_ID	EQU	12
DELTSK_SIZE	EQU	6		;USED BY PL/M-86.
;
;
DELTSK		PROC	FAR
		SUB	SP,DELTSKS_SIZE		;ALLOW FOR SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],01CH	;DELTSK CODE.
		MOV	AX,[BP+DELTSK_ID]	;TASK ID...
		MOV	[BP+DELTSKS_ID],AX	;...TO SDB.
		MOV	DI,DELTSK_STS
		CALL	MTOS			;TYPE 1 SVC
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,DELTSKS_SIZE		;DESTROY SDB.
		RET	DELTSK_SIZE
DELTSK		ENDP
PAGE
;
;	SYMBOLS FOR LOADED:
;
LOADEDS_STATUS	EQU	1
LOADEDS_ORD	EQU	2
LOADEDS_SIZE	EQU	6	;ALLOCATED FOR THE SDB.
LOADED_STS	EQU	10
LOADED_ORD	EQU	14
LOADED_STATUS	EQU	18
LOADED_SIZE	EQU	10	;USED BY PL/M-86
;
;
LOADED		PROC	FAR
		SUB	SP,LOADEDS_SIZE		;ALLOW FOR THE SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP
		ADD	BP,8			;MARK THE SDB.
		MOV	BYTE PTR [BP],01DH	;LOADED SERVICE CODE.
		MOV	AL,[BP+LOADED_STATUS]	;GET LOAD COMPLETION STATUS
		MOV	[BP+LOADEDS_STATUS],AL	;TO SDB.
		LDS	DI,DWORD PTR [BP+LOADED_ORD]	;ADDR OF ORD
		MOV	[BP+LOADEDS_ORD],DI	;TO THE SDB.
		MOV	[BP+LOADEDS_ORD+2],DS	;(BOTH PARTS.)
		MOV	DI,LOADED_STS		;ADDRESS THE STATUS VARIABLE.
		CALL	MTOS			;TYPE 1 SVC.
		POP	DS
		POP	DI
		POP	BP
		POP	AX			;RESTORE REGISTERS,
		ADD	SP,LOADEDS_SIZE		;DESTROY STACKED SDB,
		RET	LOADED_SIZE		;THAT'S IT.
LOADED		ENDP
;
;	SYMBOLS FOR HSFS SERVICE:
;
HSFSS_OPCODE	EQU	1	;SDB:	HSFS OPCODE.
HSFSS_FMASK	EQU	2	;	EVENT FLAG MASK WORD.
HSFSS_STS	EQU	4	;	ADDRESS OF STATUS WORD.
HSFSS_RPB	EQU	8	;	ADDRESS OF REQUEST RPB.
HSFSS_SIZE	EQU	12	;BYTES NEEDED TO CONSTRUCT SDB.
HSFS_RPB	EQU	16	;PLM:	ADDRESS OF REQUEST RPB.
HSFS_STS	EQU	20	;	ADDRESS OF STATUS WORD.
HSFS_FMASK	EQU	24	;	EVENT FLAG MASK WORD.
HSFS_OPCODE	EQU	26	;	HSFS OPCODE.
HSFS_SIZE	EQU	12	;BYTES USED BY PL/M-86 TO PASS PARAMETERS.
;
;	THE INTERFACE PROCEDURE.
;
HSFS		PROC	FAR
		SUB	SP,HSFSS_SIZE		;MAKE ROOM FOR GENERATED SDB.
		PUSH	AX
		PUSH	BP
		PUSH	DI
		PUSH	DS
		MOV	BP,SP			;MARK THE SDB FRAME,
		ADD	BP,8			;BELOW THE SAVE AREA.
		MOV	BYTE PTR [BP],01EH	;SDB SERVICE CODE.
		MOV	AX,[BP+HSFS_OPCODE]	;HSFS OPCODE.
		MOV	[BP+HSFSS_OPCODE],AL	;(BECOMES A BYTE.)
		MOV	AX,[BP+HSFS_FMASK]	;EVENT FLAG MASK WORD
		MOV	[BP+HSFSS_FMASK],AX
		LDS	DI,DWORD PTR [BP+HSFS_STS]	;ADDRESS OF STATUS
		MOV	[BP+HSFSS_STS],DI	;WORD BUFFER.
		MOV	[BP+HSFSS_STS+2],DS
		LDS	DI,DWORD PTR [BP+HSFS_RPB]	;ADDRESS OF REQUEST'S
		MOV	[BP+HSFSS_RPB],DI	;RPB.
		MOV	[BP+HSFSS_RPB+2],DS
		PUSH	SS			;ADDRESS OF THE SDB ONTO
		PUSH	BP			;THE TASK'S STACK.
		INT	LOW X_SVC_VECTOR	;TYPE 1 SERVICE CALL.
		ADD	SP,4			;DESTROY SDB ADDRESS.
		POP	DS
		POP	DI
		POP	BP
		POP	AX
		ADD	SP,HSFSS_SIZE		;DESTROY SDB CONSTRUCTION.
		RET	HSFS_SIZE		;RETURN TO CALLER.
HSFS		ENDP
PAGE
;
;	UTILITY PROCEDURES SVCOK AND GETPARM:
;
SVCOK		PROC	FAR
		PUSH	BP
		MOV	BP,SP
		XOR	AL,AL		;ASSUME FALSITY.
		TEST	WORD PTR [BP+6],0040H	;SVC FAILED?
		JNE	@1		;YES.
		NOT	AL		;NO.
@1:		POP	BP
		RET	2
SVCOK		ENDP
;
;
GETPARM		PROC	FAR
		MOV	ES,BX		;BX PARAMETER BECOMES POINTER SEGMENT
		MOV	BX,AX		;AX PARAMETER BECOMES POINTER OFFSET.
		RET
GETPARM		ENDP
;
PLM_CS		ENDS
;
		END
