PAGE	60,132
INCLUDE B:BIG.INC
;
;	POOL.ASM:	LOW-LEVEL MEMORY POOL HANDLERS.
;	F. W. PORRETTO, 12/15/82
;
;		HERE ARE COLLECTED THE THREE ROUTINES WHICH MANAGE MTOS'S
;	APPLICATION-ACCESSIBLE MEMORY POOLS DIRECTLY.
;
		NAME	MEMORY_POOL_SERVICES
;
MTOS_CS		SEGMENT PARA PUBLIC
		ASSUME	CS:MTOS_CS,DS:MTOS_DS,SS:MTOS_DS
		EXTRN	X_SCHDLR	:NEAR	;MTOS SCHEDULER ENTRY POINT
		EXTRN	X_QUEUE		:NEAR	;GENERAL BLOCK QUEUE ROUTINE
		EXTRN	X_DQUEUE	:NEAR	;GENERAL BLK DE-QUEUE ROUTINE
		EXTRN	X_GET_SMR	:NEAR	;ALLOCATE AN SMR BLOCK
		EXTRN	X_PUT_SMR	:NEAR	;FREE AN SMR BLOCK.
		EXTRN	X_SET_READY	:NEAR	;SET TASK TO READY STATE
		EXTRN	X_TASK_STATE	:NEAR	;MOVE TASK TO NONREADY STATE
		EXTRN	X_MPD_SIZE_VAR	:WORD	;FOR MULTIPLIES.
		EXTRN	X_MPB_SIZE_VAR	:WORD	;DITTO.
;
		PUBLIC	X_GET_BLOCK,X_FREE_BLOCK,X_WAIT_FOR_BLOCK
		PUBLIC	X_POOL_DATA
;
BLOCK_SIZE	EQU	02H		;DISPLACEMENT IN BLOCK TO BYTE COUNT
PAGE
;***********************************************************
;*     X_POOL_DATA: LOCATE POOL'S MPD AND MPB BLOCKS.      *
;***********************************************************
;
;	INPUT:	DX=	MEMORY POOL ID
;	OUTPUT:	DI=	MEMORY POOL DESCRIPTOR TABLE (MPD)
;	        BX=	MEMORY POOL BLOCK            (MPB)
;
;		THE ADDRESSES OF THE MPD AND MPB FOR THE SPECIFIED POOL ARE
;	COMPUTED AND RETURNED TO THE CALLER.
;
X_POOL_DATA	PROC	NEAR	
		MOV	AX,DX		;POOL ID TO AX.
		PUSH	DX		;SAVE POOL ID.
		MUL	CS:X_MPB_SIZE_VAR	;MULTIPLY IT BY MPB_SIZE
		MOV	BX,AX		;THEN FINISH CALCULATION BY ADDING
		ADD	BX,OFFSET X_MPB_TABLE	;BASE ADDRESS OF MPB TABLE.
		POP	AX		;REFRESH POOL ID,
		PUSH	AX		;SAVE IT AGAIN,
		MUL	CS:X_MPD_SIZE_VAR	;MULTIPLY IT BY MPD_SIZE,
		MOV	DI,AX		;AND ADD TO THAT
		ADD	DI,OFFSET X_MPD_TABLE	;THE BASE OF MPD TABLE.
		POP	DX		;DONE.
		RET
X_POOL_DATA	ENDP
PAGE
;*************************************************************************
;*     X_WAIT_FOR_BLOCK: BLOCK TASK PENDING POOL MEMORY AVAILABILITY.    *
;*************************************************************************
;
;	INPUT:	DX=	POOL ID
;		ES:DI=	BUFFER ADDRESS FOR ALLOCATED BLOCK
;		CX=	COUNT OF BYTES REQUIRED
;	OUTPUT:	NONE.
;
;	THIS ROUTINE CAUSES TO BE BUILT A POOL WAIT BLOCK (PWB) WHICH IS AT-
;	TACHED TO THE WAIT QUEUE OF THE SPECIFIED POOL. THE REQUESTING TASK IS
;	BLOCKED UNTIL THE CONDITIONS SPECIFIED IN THE PWB CAN BE MET.
;
X_WAIT_FOR_BLOCK	PROC	NEAR
		PUSH	BX
		PUSH	SI
		PUSH	DI
		CALL	X_GET_SMR		;ALLOCATE A BLOCK FOR THE PWB
		MOV	SI,X_CTCB		;MOVE THE TCB ADDRESS
		MOV	[BX+PWB_TCB],SI		;INTO IT.
		MOV	AL,[SI+TCB_PRI]		;SET UP ITS PRIORITY FROM
		MOV	[BX+PWB_PRI],AL		;THE TCB TOO.
		MOV	[BX+PWB_BUFR],DI	;INSERT BUFFER ADDRESS
		MOV	[BX+PWB_BUFR+2],ES
		MOV	[BX+PWB_LEN],CX		;INSERT LENGTH.
		PUSH	BX			;SAVE PWB ADDRESS
		CALL	X_POOL_DATA		;GET MPB ADDRESS
		ADD	BX,MPB_WAITQ		;ADDRESS THE WAIT QUEUE
		MOV	DI,BX
		POP	BX			;GET BACK THE PWB AND
		CALL	X_QUEUE			;ATTACH IT TO THE WAIT QUEUE.
		MOV	BX,X_CTCB		;FINALLY,
		MOV	AX,BLOCK+STA_POOL	;MOVE REQUESTING TASK TO
		CALL	X_TASK_STATE		;BLOCKED STATE.
		POP	DI
		POP	SI
		POP	BX
		RET
X_WAIT_FOR_BLOCK ENDP
PAGE
;
;	UTILITY ROUTINE TO CHECK BLOCK SUITABILITY FOR ALLOCATION:
;
;	INPUT: ES:SI= BLOCK TO CHECK,
;	       CX=    NUUMBER OF BYTES REQUIRED.
;
;	OUTPUT: FLAGS REGISTER RETURNS THE DECISION:
;	        CARRY=1:	BLOCK UNSUITABLE
;	        CARRY=0:
;			ZERO=1:	BLOCK FITS ALLOCATION EXACTLY
;			ZERO=0:	BLOCK IS LARGE BY AT LEAST 4 BYTES.
;
CHK_BLK		PROC	NEAR
		CMP	ES:[SI+BLOCK_SIZE],CX	;IS BLOCK LARGE ENOUGH?
		JBE	CHK_OUT			;EITHER EXACTLY OR NOT AT ALL.
		MOV	AX,ES:[SI+BLOCK_SIZE]	;MAYBE...
		SUB	AX,CX			;COMPUTE FRAGMENT SIZE
		CMP	AX,3			;LEGAL FRAGMENT CREATED?
		JA	CHK_OUT			;YES.
		STC				;NO.
CHK_OUT:	RET
CHK_BLK		ENDP
PAGE
;***********************************************************
;*     X_GET_BLOCK:    ALLOCATE BLOCK FROM MEMORY POOL.    *
;***********************************************************
;
;	INPUT  - DX = MEMORY POOL ID
;	       - CX = NUMBER OF BYTES TO ALLOCATE
;	OUTPUT - CARRY=0: BLOCK ALLOCATED, ADDRESS IN ES:BX,
;	         CARRY=1: NO SUITABLE BLOCK AVAILABLE AT THIS TIME.
;	REGISTERS ES AND BX ARE DESTROYED EVEN IF ALLOCATION FAILS.
;
;		THE POOL CONTROL ALGORITHM IS A RESTRICTED FIRST FIT SEARCH
;	AT ALLOCATION TIME. CERTAIN SINGULARITIES IN THE USE OF AVAILABLE
;	MEMORY MUST BE OBSERVED, OWING TO THE UNCONTROLLABILITY OF A BLOCK
;	LESS THAN FOUR BYTES IN LENGTH. TO FACILITATE GARBAGE COLLECTION AT
;	DEALLOCATION TIME, THE FREE BLOCK LIST IS KEPT IN ASCENDING ADDRESS
;	ORDER. IT IS REQUIRED THAT THE POOL'S OFFSET FROM ITS BASE PARAGRAPH
;	BE 0, NO MATTER HOW LARGE IT IS OR WHERE LOCATED. IT IS ALSO REQUIRED
;	THAT EACH POOL'S PARAGRAPH ADDRESS BE UNIQUE.
;
X_GET_BLOCK	PROC	NEAR
		PUSH	SI
		PUSH	DI
		CALL	X_POOL_DATA		;GET MPB AND MPD ADDRESSES
		PUSH	BX			;SAVE MPB ADDRESS.
		PUSH	DS			;SAVE DS AND ARRANGE TO REACH
		MOV	AX,MTOS_GROM		;THE GLOBAL ROM SEGMENT.
		MOV	DS,AX
		LES	DI,DWORD PTR [DI+MPD_OFF]	;SET ES TO POOL SEGMENT
		POP	DS			;RESTORE MTOS_DS.]
		ADD	BX,MPB_FREEQ		;POINT BX AT FREE LIST HEAD.
		MOV	SI,[BX]			;GET ADDR OF FIRST FREE BLOCK
		CMP	SI,LINK_END		;IS THERE ONE?
		JE	GET_FAIL		;NO.
;
;	THE FIRST BLOCK REQUIRES SPECIAL HANDLING, BECAUSE IT IS LINKED TO THE
;	MPB ITSELF, WHICH RESIDES IN THE MTOS_DS SEGMENT AND NOT IN THE POOL'S
;	OWN SEGMENT.
;
GET001:
		CALL	CHK_BLK			;IS THE FIRST BLOCK SUITABLE?
		JC	GET002			;CARRY MEANS NO.
		JNE	GET011			;YES, BUT INEXACT FIT.
		MOV	AX,ES:[SI]		;EXACT FIT: UNLINK THE BLOCK
		MOV	[BX],AX			;FROM THE FREE LIST.
		MOV	BX,SI			;BX RETURNS BLOCK ADDRESS.
		JMP	SHORT REDUCE_POOL	;REDUCE AVAILABLE BYTES.
GET011:
		SUB	ES:[SI+BLOCK_SIZE],CX	;REDUCE BLOCK SIZE BY ALLOCATION
		MOV	BX,ES:[SI+BLOCK_SIZE]	;GET FRAGMENT SIZE AND
		ADD	BX,SI			;COMPUTE ALLOCATED BLOCK ADDRESS
		JMP	SHORT REDUCE_POOL	;REDUCE AVAILABLE BYTES.
GET002:
		MOV	BX,SI			;LINK TO NEXT FREE BLOCK.
		MOV	SI,ES:[SI]
		CMP	SI,LINK_END		;END OF FREE LIST?
		JE	GET_FAIL		;YES.
		CALL	CHK_BLK			;NO.
		JC	GET002			;HOWEVER, BLOCK ISN'T SUITABLE
		JNE	GET011			;GOOD, BUT INEXACT FIT.
		MOV	AX,ES:[SI]		;EXACT FIT: REMOVE BLOCK EN-
		MOV	ES:[BX],AX		;TIRELY FROM THE FREE LIST.
		MOV	BX,SI			;BX RETURNS BLOCK ADDRESS.
REDUCE_POOL:
		POP	DI			;RETRIEVE MPB ADDRESS
		SUB	[DI+MPB_AVAIL],CX	;THIS MUCH LESS IS FREE.
		CLC
		JMP	SHORT GET_OVER
GET_FAIL:
		POP	BX			;UNLOAD MPB FROM STACK.
		STC				;ALLOCATION FAILED.
GET_OVER:
		POP	DI
		POP	SI
		RET
X_GET_BLOCK	ENDP
PAGE
;*********************************************************************
;*     X_FREE_BLOCK: DEALLOCATE BLOCK TO MEMORY POOL AND PERFORM     *
;*                   ALL CONTINGENT PROCESSING.                      *
;*********************************************************************
;
;	INPUT:	DX=	MEMORY POOL ID
;		ES:BX=	MEMORY BLOCK ADDRESS TO BE DEALLOCATED
;		CX=	NUMBER OF BYTES TO DEALLOCATE
;	OUTPUT:	NONE
;
;		AS MENTIONED IN THE PREAMBLE TO X_GET_BLOCK, THE FREE LIST
;	IS KEPT IN ASCENDING ADDRESS ORDER. THE NEWLY FREED BLOCK MUST BE
;	INSERTED INTO THE FREE LIST AT THE POINT APPROPRIATE TO ITS ADDRESS,
;	AND THEN GARBAGE COLLECTION BEGINS. ATTEMPTS ARE MADE TO CONCATENATE 
;	THE NEWLY FREED BLOCK WITH BOTH THE PREVIOUS FREE BLOCK AND THE NEXT
;	FREE BLOCK; THUS, EITHER TWO OR THREE SEPARATE BLOCKS MAY BECOME ONE
;	BLOCK EQUAL TO THE SUM OF THEIR SIZES. AFTER THE GARBAGE COLLECTION
;	PHASE, ALL THE PWBS QUEUED TO THE POOL'S WAIT QUEUE ARE EXAMINED TO
;	SEE IF ANY OF THEM CAN NOW BE SATISFIED. FOR ANY WHICH ARE NOW SATIS-
;	FIABLE, THE PWB IS DEALLOCATED AND THE REQUESTING TASK IS UNBLOCKED.
;
X_FREE_BLOCK	PROC	NEAR
		PUSH	BP
		PUSH	SI
		PUSH	DI
		MOV	SI,BX			;SI= BLOCK ADDRESS TO FREE.
		MOV	ES:[SI+BLOCK_SIZE],CX	;INSERT ITS SIZE INTO IT.
		CALL	X_POOL_DATA		;GET MPB, MPD ADDRESSES.
		ADD	[BX+MPB_AVAIL],CX	;INCREASE FREE SPACE.
		PUSH	BX			;SAVE MPB ADDRESS.
		ADD	BX,MPB_FREEQ		;ADDRESS FREE LIST HEAD.
		MOV	DI,[BX]			;GET FIRST FREE BLOCK.
;
;	THE FIRST POSITION ON THE FREE LIST, AS WITH THE ALLOCATION PROCEDURE,
;	IS A SPECIAL CASE AND IS CHECKED SEPARATELY.
;
		CMP	SI,DI			;NEW BLOCK GOES AT FRONT?
		JA	BEGIN_SORT		;NO, FARTHER DOWN.
		MOV	ES:[SI],DI		;YES: LINK IT IN AS THE
		MOV	[BX],SI			;FREE LIST HEAD BLOCK.
		JMP	SHORT CAT_2		;ATTEMPT DORSAL CONCATENATION
BEGIN_SORT:
		MOV	BX,DI			;PREVIOUS= CURRENT,
		MOV	DI,ES:[DI]		;CURRENT= FLINK(CURRENT).
		CMP	SI,DI			;INSERTION HERE?
		JA	BEGIN_SORT		;NO, KEEP LOOKING.
		MOV	ES:[BX],SI		;YES: LINK THE BLOCK IN
		MOV	ES:[SI],DI		;'TWIXT PREVIOUS AND CURRENT
;
;	WITH ALL CASES OTHER THAN INSERTION AT THE FRONT OF THE FREE LIST,
;	AN ATTEMPT MUST BE MADE TO CONCATENATE THE PREVIOUS BLOCK WITH THE
;	NEWLY INSERTED BLOCK (VENTRAL CONCATENATION).
;
		MOV	AX,ES:[BX+BLOCK_SIZE]	;IS CONCATENATION
		ADD	AX,BX			;POSSIBLE?
		CMP	AX,SI			;THIS WILL TELL...
		JNE	CAT_2			;NO, TRY DORSAL SURFACES.
		ADD	ES:[BX+BLOCK_SIZE],CX	;YES: EXPAND PREV. BLOCK
		MOV	AX,ES:[SI]		;TO INCLUDE INSERTED BLOCK
		MOV	ES:[BX],AX		;AND ELIMINATE NEW ONE.
		MOV	SI,BX			;NOW PREV. BECOMES "NEW"
CAT_2:
		CMP	DI,LINK_END		;IS THERE A BLOCK AT THE
						;DORSAL EDGE?
		JE	DONE_FREE		;NO.
		MOV	AX,ES:[SI+BLOCK_SIZE]	;YES: GET NEW BLOCK LENGTH
		ADD	AX,SI			;FIND DORSAL EDGE
		CMP	AX,DI			;CONCATENATION POSSIBLE?
		JNE	DONE_FREE		;NO.
		MOV	AX,ES:[DI]		;YES: DELETE THE NEXT BLOCK
		MOV	ES:[SI],AX		;FROM THE LIST, EXPANDING THE
		MOV	AX,ES:[DI+BLOCK_SIZE]	;NEW BLOCK TO COVER ITS
		ADD	ES:[SI+BLOCK_SIZE],AX	;MEMORY.
DONE_FREE:
;
;	HAVING PERFORMED ALL GARBAGE COLLECTION, WE NOW HAVE A NEW BLOCK ON
;	THE POOL FREE LIST WHOSE SIZE WE KNOW. IF IT IS POSSIBLE TO SATISFY
;	ANY OF THE WAITING REQUESTS IN PWBS AT THIS POINT, IT WILL BE BECAUSE
;	OF THE NEW BLOCK. THEREFORE, THE NEW BLOCK'S SIZE IS USED TO WEED OUT
;	ANY PWBS WHICH CONTINUE TO BE UNSATISFIABLE, OBVIATING THEIR RESUBMIT-
;	TAL TO X_GET_BLOCK. ALL OTHER PWBS ON THE POOL'S WAIT QUEUE ARE RESUB-
;	MITTED, UNTIL THE NEW BLOCK BECOMES TOO SMALL TO SATISFY ANY MORE RE-
;	QUESTS.
;
		MOV	BP,ES:[SI+BLOCK_SIZE]	;CARRY SIZE OF NEW BLOCK IN BP.
		POP	DI			;RETRIEVE MPB ADDR
		PUSH	DI			;SAVE IT AGAIN.
		LEA	DI,[DI+MPB_WAITQ]	;ADDRESS THE WAIT QUEUE
		MOV	SI,[DI]			;GET HEAD ENTRY
NEXT_PWB:
		TEST	BP,BP			;BLOCK DOWN TO 0 BYTES?
		JE	SCAN_OVER		;YES, SCAN COMPLETE.
		TEST	SI,SI			;NO: ANY MORE PWBS?
		JE	SCAN_OVER		;NO.
		PUSH	WORD PTR [SI]		;YES: SAVE FORWARD LINK.
		MOV	AX,BP			;NEW BLOCK'S SIZE TO AX.
		SUB	AX,[SI+PWB_LEN]		;SUBTRACT SIZE REQUESTED BY PWB
		JB	PWB_FAIL		;NEW BLOCK TOO SMALL.
		JE	NEXTP0			;NEW BLOCK JUST RIGHT.
		CMP	AX,4			;LARGER THAN REQUESTED, BUT...
		JB	PWB_FAIL		;NOT LARGE ENOUGH.
NEXTP0:
		MOV	BP,AX			;MOVE FRAGMENT SIZE BACK TO BP
		MOV	CX,[SI+PWB_LEN]		;GET NEEDED LENGTH.
		CALL	X_GET_BLOCK		;ATTEMPT TO ALLOCATE IT.
		JC	PWB_FAIL		;ALLOCATION FAILED.
		PUSH	BP			;SAVE NEW FRAGMENT LENGTH.
		PUSH	ES			;ALLOCATION SUCCEEDED:
		LES	BP,DWORD PTR [SI+PWB_BUFR]	;GET BUFFER ADDRESS.
		POP	WORD PTR ES:[BP+2]	;INSERT THE SEGMENT,
		MOV	ES:[BP],BX		;AND THE OFFSET OF THE BLOCK.
		POP	BP			;RETRIEVE NEW FRAGMENT LENGTH.
		POP	WORD PTR [DI]		;UNLINK THE PWB
		MOV	BX,[SI+PWB_TCB]
		CALL	X_SET_READY		;UNBLOCK WAITING TASK
		MOV	BX,SI			;FINALLY,
		CALL	X_PUT_SMR		;DEALLOCATE THE PWB.
		MOV	SI,[DI]			;LINK FORWARD,
		JMP	SHORT NEXT_PWB		;AND TRY AGAIN.
PWB_FAIL:
		MOV	DI,SI			;CURRENT BECOMES PREVIOUS AND
		POP	SI			;NEXT BECOMES CURRENT
		JMP	SHORT NEXT_PWB		;TRY AGAIN.
SCAN_OVER:
		POP	BX			;RETRIEVE MPB ADDRESS
		POP	DI
		POP	SI
		POP	BP
		RET
X_FREE_BLOCK	ENDP
MTOS_CS		ENDS
		END
