PAGE	60,132
INCLUDE B:BIG.INC
;
;	RIO.ASM:	UNIVERSAL DEVICE DRIVER INTERFACE AND SUPPORT
;			ROUTINES FOR DRIVERS AND SIMILAR PROCEDURES.
;	F. W. PORRETTO, 12/15/82
;
;		HERE ARE COLLECTED ALL THE ROUTINES INVOLVED WITH DEVICE 
;	DRIVER ACTIVATION OR SUPPORT.
;
		NAME	DEVICE_SERVICES
;
MTOS_CS		SEGMENT PARA PUBLIC
		ASSUME	CS:MTOS_CS,DS:MTOS_DS,SS:MTOS_DS
		EXTRN	X_SCHDLR	:NEAR	;ENTRY POINT OF SCHEDULER.
		EXTRN	X_SFLAGS	:NEAR	;SET SVC CONDITION CODES
		EXTRN	X_GET_INTERVAL	:NEAR	;COMPUTE COUNTDOWN.
		EXTRN	X_QUEUE_TMB	:NEAR	;TIMER BLOCK QUEUE ROUTINE
		EXTRN	X_DQUEUE_TMB	:NEAR	;TIMER BLOCK DE-QUEUE ROUTINE
		EXTRN	X_TASK_STATE	:NEAR	;MOVE TASK TO STATE.
		EXTRN	X_SET_READY	:NEAR	;PUT TASK ON READY QUEUE
		EXTRN	X_START_TASK	:NEAR	;EFFECT A QSTRT.
		EXTRN	X_GET_TCB_ADR	:NEAR	;TASK ID TO TCB ADDRESS
		EXTRN	X_DQUEUE	:NEAR	;GENERAL DE-QUEUE ROUTINE
		EXTRN	X_QUEUE		:NEAR	;GENERAL QUEUE ROUTINE
		EXTRN	X_SET_EF	:NEAR	;SET FLAG GROUP
		EXTRN	X_RESET_EF	:NEAR	;RESET FLAG GROUP.
		EXTRN	X_GET_BLOCK	:NEAR	;ALLOCATE FROM POOL.
		EXTRN	X_FREE_BLOCK	:NEAR	;DEALLOCATE TO POOL.
		EXTRN	X_GET_SMR	:NEAR	;GET SMR BLOCK.
		EXTRN	X_PUT_SMR	:NEAR	;FREE SMR BLOCK.
		EXTRN	X_DCB_SIZE_VAR  :BYTE	;SIZE OF DCB FOR MULTIPLYING
		EXTRN	X_UCB_SIZE_VAR  :BYTE	;      UCB
;
		PUBLIC	X_RIO,XIODONE,XOVER,XDEVDATA
		PUBLIC	XQSIRB,XQPIRB,XQTMB,XDQTMB,X_DEVDATA
		PUBLIC	XSETUNS,XERRLOG,XDELTMB,XDWORD,XTIMEOUT
		PUBLIC	XQSETUNS,XGETSMR,XPUTSMR,XALOC,XDALOC
;
;	USEFUL SYMBOLS: FOR DCB_STS AND UCB_STS:
;
UNIT_BUSY	EQU	00000001B	;DRIVER OR UNIT BUSY.
;
;	USEFUL SYMBOLS: FOR THE FUNCTION CODE BYTE OF THE REQUEST.
;
FUNCT_MASK	EQU	00001111B	;FOR MASKING REQUESTED FUNCTION.
PRE_EMPT	EQU	10000000B	;FUNCTION PREEMPTS PRIORITY/RESERVE.
BLOCK_TASK	EQU	01000000B	;BLOCK TASK FOR I/O COMPLETION.
;
;	FOR THE EVENT FLAG GROUP NUMBER IN THE RIO REQUEST:
;
EFGRP_MASK	EQU	00011111B
;
;	FINALLY, THE STATUSES FOR CERTAIN KINDS OF COMPLETION:
;
SUCCESS		EQU	2		;SUCCESS.
TIME_OUT	EQU	0FCH		;TIMEOUT STATUS.
BAD_RSRV	EQU	0FEH		;BAD RESERVE OR RELEASE REQUEST.
BAD_PARAMS	EQU	0FFH		;BAD REQUEST PARAMETERS
;
XPRIMARY	EQU	BYTE PTR XSTATUS	;PRIMARY STATUS BYTE.
;
PAGE
;******************************************************************
;*      X_IOINIT:    INITIALIZE FOR NEXT I/O REQUEST              *
;******************************************************************
;
;	INPUT:	DI=	ADDRESS OF THE IRB
;	OUTPUT:	NONE
;
;		THIS ROUTINE SUBSUMES THE FOLLOWING PROCESSING:
;	1) IF THERE ARE EVENT FLAGS TO BE SET WHEN THE I/O REQUEST COMPLETES,
;	   THIS ROUTINE CLEARS THEM.
;	2) THE STATUS DOUBLEWORD POINTED TO BY THE TASK'S SDB IS CLEARED TO
;	   FOUR BYTES OF ZERO.
;	3) IF THE TASK'S SDB SPECIFIED WAIT-FOR-REQUEST-COMPLETION, THIS ROU-
;	   TINE BLOCKS THE REQUESTOR TASK.
;
;	THE READER WILL NOTE THAT THIS ROUTINE DOES SOMETHING VERY UNUSUAL FOR
;	AN MTOS SVC PROCESSOR OR SATELLITE ROUTINE THEREOF: IT MAY RETURN THE
;	REQUESTOR TASK TO READY STATE, A CHORE NORMALLY HANDLED ONLY BY THE 
;	SVC INTERRUPT HANDLER. THE REASON FOR THIS IS THAT X_IOINIT IS ONLY
;	EXECUTED AFTER AN INTERPROCESSOR MIGRATION, AFTER WHICH ALL CONNECTION
;	WITH THE SVC INTERRUPT HANDLER HAS BEEN SEVERED. SINCE THE SVC INTER-
;	RUPT HANDLER WILL NOT GET THE CHANCE TO RETURN THE REQUESTOR TO READY
;	STATE... WE MUST DO IT!
;
X_IOINIT	PROC	NEAR
		PUSH	DS
		PUSH	ES
		PUSH	DI
		PUSH	BX
		PUSH	DX
		MOV	BX,[DI+IRB_TCB]		;TCB ADDR OF REQUESTER
		TEST	BX,BX			;ANY TCB MENTIONED?
		JE	TASK_WAIT		;NO.
		MOV	DL,[DI+IRB_GRP]		;EVENT FLAG GROUP
		MOV	CX,[DI+IRB_FLAGS]	;GET FLAG MASK WORD
		CALL	X_RESET_EF		;PRECONDITION THE FLAG GROUP
		MOV	AL,[DI+IRB_FUNCT]	;GET FUNCTION BYTE TO AL.
		LES	DI,DWORD PTR [DI+IRB_STATUS]
		MOV	WORD PTR ES:[DI],0	;SET STATUS TO 0,0
		MOV	WORD PTR ES:[DI+2],0
		TEST	AL,BLOCK_TASK		;DID FUNCTION BYTE SPECIFY THAT
						;TASK WAS TO WAIT FOR I/O?
		JNE	TASK_WAIT		;YES, LEAVE TASK BLOCKED.
		CALL	X_SET_READY		;NO, MOVE TASK TO READY STATE.
TASK_WAIT:
		POP	DX
		POP	BX
		POP	DI
		POP	ES
		POP	DS
		RET
X_IOINIT	ENDP
PAGE
;**************************************************************
;*     X_RIO: UNIVERSAL DEVICE DRIVER REQUEST PROCESSOR.      *
;**************************************************************
;
;	INPUT:	ES:SI=	SDB ADDRESS,
;		BX=	REQUESTOR'S TCB ADDRESS
;
;	THIS IS THE INTERFACE TO ALL DEVICE DRIVERS. IT VERIFIES THE
;	REQUEST'S DEVICE NUMBER, UNIT NUMBER AND EF GROUP NUMBER, VERI-
;	FIABLE DEVICE-INDEPENDENT PARAMETERS. IF THEY PASS VALIDATION,
;	FURTHER PROCESSING IS BIFURCATED:
;--TYPE 1 DRIVERS:
;	THE REQUEST DATA IS PASSED TO THE ADDRESS GIVEN IN DCD_SVC IN
;	AN IRB, AND ALL HANDLING OF THE REQUEST FROM THAT POINT FOR-
;	WARD IS UP TO THE DRIVER. THIS DRIVER TYPE IS AVAILABLE FOR
;	DEVICES REQUIRING MORE COMPLEX CONTROL THAN THAT POSSIBLE TO
;	A TYPE 2 DRIVER.
;--TYPE 2 DRIVERS:
;	THE REQUEST DATA IS PUT INTO AN IRB, AS ABOVE, AND THE DRIVER
;	STATUS(ES) IS/ARE CHECKED TO DETERMINE WHETHER THE REQUEST MAY
;	BE ACTIVATED IMMEDIATELY. IF SO, THE REQUEST IS PASSED TO THE 
;	APPROPRIATE DRIVER ENTRY POINT THROUGH THE VECTOR TABLE WHOSE
;	ADDRESS IS PRESUMED TO BE IN DCD_SVC. IF NOT, THE REQUEST IS
;	AUTOMATICALLY ENTERED ONTO THE CORRECT WAIT QUEUE (THE ONE IN
;	THE DCB FOR SERIAL DRIVERS; THE ONE IN THE RELEVANT UCB FOR
;	PARALLEL DRIVERS) IN PRIORITY ORDER, AND CONTROL IS RETURNED
;	TO THE SCHEDULER. UPON CHANGE IN STATUS AT THE DRIVER DUE TO
;	OTHER EVENTS (COMPLETION OF THE CURRENT REQUEST OR CHANGE IN
;	RESERVATION STATUS), THE APPROPRIATE WAIT QUEUE WILL BE SCANNED
;	FOR THE NEXT REQUEST TO ACTIVATE, AND IF AN ACTIVATABLE RE-
;	QUEST IS FOUND, IT IS SUBMITTED TO THE DRIVER THROUGH THE VEC-
;	TOR TABLE AS ABOVE.
;
RIO_FUNCT	EQU	01H		;DRIVER FUNCTION CODE.
RIO_DEVICE	EQU	02H		;DEVICE #
RIO_UNIT	EQU	03H		;UNIT #
RIO_GRP		EQU	04H		;EVENT FLAG GROUP
RIO_SIZE	EQU	0FH		;SIZE OF THE RIO SVC REQUEST (SDB)
;
X_RIO		PROC	FAR
		MOV	AL,ES:[SI+RIO_GRP]	;GET EF GROUP NUMBER,
		AND	AL,EFGRP_MASK		;MASK OUT BITS 5,6,7,
		MOV	CX,X_GEF_NUM
		CMP	AL,CL			;COMPARE TO MAX VALUE...
		JA	RIOERR			;TOO HIGH, INVALID.
		MOV	AL,ES:[SI+RIO_DEVICE]	;WITHIN RANGE: GET DEVICE AND
		MOV	DL,ES:[SI+RIO_UNIT]	;UNIT NUMBERS,
		MOV	CH,ES:[SI+RIO_FUNCT]	;ALSO GET THE FUNCTION BYTE AND
		AND	CH,FUNCT_MASK		;ISOLATE FUNCTION CODE.
		MOV	XDEVNUM,AL		;SET THE DEVICE AND UNIT
		MOV	XDEVUNIT,DL		;NUMBER VARIABLES.
		CALL	FAR PTR XDEVDATA	;GOOD DEVICE AND UNIT?
		JC	RIOERR			;NO.
		PUSH	ES			;YES: UCB AND DCB NOW KNOWN
		MOV	DI,XDCB			;GET DCB ADDRESS
		MOV	AX,MTOS_GROM		;GO TO THE
		MOV	ES,AX			;GLOBAL ROM SEGMENT
		MOV	BP,[DI+DCB_DCD]		;GET DCD ADDRESS
		MOV	AX,ES:[BP+DCD_FLAGS]	;GET FLAGS AND MAX. FUNC. NR.
		POP	ES			;CAN NOW RESTORE ES.
		TEST	AL,DFL_TYPE2		;TYPE 2 DRIVER??
		JE	RIO00			;NO, BYPASS FUNC. CHECK.
		CMP	CH,AH			;YES: VALIDATE FUNCTION CODE:
		JBE	RIO00			;VALID (LOW OR = MAXIMUM).
RIOERR:
		MOV	AX,SVC_FAILED+PARM_ERR
		CALL	X_SFLAGS
		RET
RIO00:
		MOV	BX,X_CTCB		;RESTORE REQUESTOR'S TCB LONG
		MOV	AX,BLOCK+STA_WAIT	;ENOUGH TO PUT HIM IN BLOCKED
		CALL	X_TASK_STATE		;FOR I/O STATE.
RIO01:
		CALL	X_GET_SMR		;GET BLOCK FOR TMB.
		PUSH	BX			;SAVE ITS ADDRESS.
		CALL	X_GET_SMR		;GET BLOCK FOR IRB.
		MOV	XCIRB,BX		;PUT ITS ADDRESS
		MOV	XCIRB+2,DS		;INTO CONTROL VARIABLE.
		POP	WORD PTR [BX+IRB_TMB]	;LINK TMB TO IT.
		MOV	DI,X_CTCB		;GET REQUESTOR'S TCB ADDRESS
		MOV	AL,[DI+TCB_PRI]		;HIS CURRENT PRIORITY
		MOV	[BX+IRB_PRI],AL		;BECOMES IRB'S PRIORITY.
		MOV	[BX+IRB_TCB],DI		;INSERT TCB ADDRESS TOO.
		MOV	AX,XUCB			;GET UCB ADDRESS
		MOV	[BX+IRB_UCB],AX		;PUT THAT IN IRB, TOO.
		MOV	CX,RIO_SIZE		;PREPARE TO MOVE SDB DATA:
RIO02:
		MOV	AL,ES:[SI]
		MOV	[BX+IRB_RIO],AL
		INC	SI
		INC	BX
		LOOP	RIO02
;
;	IS THIS A TYPE 1 OR TYPE 2 DRIVER?
;
;	TYPE 1 DRIVERS ARE THOSE COMPATIBLE WITH MTOS-86 V1.5.
;	TYPE 2 DRIVERS USE THE SIMPLIFIED DRIVER SUPPORT SCHEME INTRODUCED
;	IN MTOS-86MP V2.2.
;
		MOV	BX,XDCB			;GET DCB ADDRESS
		MOV	AX,MTOS_GROM		;ONCE AGAIN WE NEED TO GO TO
		MOV	ES,AX			;GLOBAL ROM SEGMENT.
		MOV	SI,[BX+DCB_DCD]		;GET DCD ADDRESS
		MOV	DI,XCIRB		;INITIALIZE THE REQUEST
		CALL	X_IOINIT		;ENVIRONMENT (TASK + FLAGS)
		MOV	AL,ES:[SI+DCD_FLAGS]	;GET DRIVER FLAGS.
		TEST	AL,DFL_TYPE2		;TYPE 2 DRIVER?
		JNE	RIO03			;YES!
		CALL	DWORD PTR ES:[SI+DCD_SVC]	;NO: ENTER SVC REQUEST
						;PROCESSOR OF DRIVER DIRECTLY.
		RET
;
;	THE DRIVER IS DESIGNATED AS TYPE 2:
;
;	1)QUEUE THE IRB TO THE APPROPRIATE I/O WAIT QUEUE,
;	2)CHECK FOR THE CURRENT ACTIVITY CONDITIONS ON THE DRIVER,
;	3)IF A REQUEST CAN BE STARTED AT THIS TIME, DO SO!
;
RIO03:
		MOV	DI,XUCB			;GET ADDRESS OF PARALLEL
		ADD	DI,UCB_QIRB		;IRB WAIT QUEUE TO DI.
		TEST	AL,DFL_PARA		;PARALLEL DRIVER?
		JNZ	RIO04			;YES.
		LEA	DI,[BX+DCB_QIRB]	;NO: ADDRESS SERIAL WAIT QUEUE.
RIO04:
		MOV	BX,XCIRB		;GET ADDRESS OF IRB.
		TEST	BYTE PTR [BX+IRB_FUNCT],PRE_EMPT	;PREEMPTIVE?
		JE	RIO04A			;NO: QUEUE IT NORMALLY.
		MOV	BYTE PTR [BX+IRB_PRI],255	;YES: OVERRIDE PRIORITY
		MOV	AX,[DI]			;AND FORCE IT ONTO WAIT QUEUE
		MOV	[DI],BX			;AT THE HEAD, SO THAT IT WILL
		MOV	[BX],AX			;BE CONSIDERED FIRST.
		JMP	SHORT RIO05
RIO04A:
		CALL	X_QUEUE			;ATTACH IRB TO WAIT QUEUE IN
						;POSITION BY ITS PRIORITY.
RIO05:
		MOV	DI,BX			;IRB ADDRESS TO DI.
		MOV	AX,XSEQ			;GET NEXT ASSIGNABLE SEQUENCE
		MOV	[DI+IRB_SEQ],AX		;NUMBER TO THE IRB AND ADVANCE
		INC	XSEQ			;THE SEQUENCE NUMBER CONTROLLER
		TEST	BYTE PTR [DI+IRB_FUNCT],FUNCT_MASK	;RESERVE?
		JNE	RIOOUT			;NO.
		MOV	BX,[DI+IRB_TCB]		;YES: GET RQSTR'S TCB ADDRESS
		TEST	BX,BX			;ANY TCB?
		JE	RIOOUT			;NO.
		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_TRES	;YES: RESERVES
						;UNDER TIMEOUT?
		JE	RIOOUT			;NO, DONE.
		LES	BP,DWORD PTR [BX+TCB_TCD]	;YES: GO TO TCD
		MOV	AX,ES:[BP+TCD_TIME]	;FOR OVERRIDE TIME
		CALL	X_GET_INTERVAL		;CONVERT TO COUNTDOWN
		MOV	BX,[DI+IRB_TMB]		;GET RQST TMB ADDRESS
		MOV	[BX+TMB_TIME],CX	;PUT COUNTDOWN
		MOV	[BX+TMB_TIME+2],DX	;INTO TMB.
		MOV	BYTE PTR [BX+TMB_TYPE],7	;TYPE 7 TMB, OF COURSE
		MOV	[BX+TMB_IRB],DI		;LOG THE IRB IN THE TMB
		CALL	X_QUEUE_TMB		;START THE TIMER.
RIOOUT:
		JMP	FAR PTR TYPE2_SCHDLR	;ENTER THE TYPE 2 SCHEDULER.
X_RIO		ENDP
PAGE
;*********************************************************************
;*        TYPE2_SCHDLR: ATTEMPT TO START A TYPE 2 DRIVER REQUEST     *
;*********************************************************************
;
;	INPUT:	XUCB=	UCB ADDRESS OF RELEVANT DEVICE UNIT.
;	OUTPUT:	XUCB= 	UCB ADDRESS OF NEWLY RELEVANT UNIT,
;			ALL REGISTERS DESTROYED.
;
;	THIS ROUTINE SCHEDULES TYPE 2 DRIVERS' REQUESTS: HERE THE SCAN FOR
;	POSSIBLE ACTIVATION OF THE NEXT REQUEST BEGINS. IT IS A CALL TARGET
;	FROM THE ROUTINE XOVER, THE REQUEST COMPLETION ROUTINE FOR TYPE 2
;	DRIVERS, AND ALSO FROM THE RIO SERVICE INTERFACE.
;
TYPE2_SCHDLR	PROC	FAR
		MOV	DI,XUCB			;PREPARE BY GETTING UCB
		MOV	AX,MTOS_GROM		;SETUP TO REACH THE
		MOV	ES,AX			;GLOBAL ROM SEGMENT.
		MOV	BX,[DI+UCB_DCB]		;GET ADDRESS OF THE
		MOV	XDCB,BX			;DCB: PUT INTO INTERFACE WORD,
		MOV	BX,[BX+DCB_DCD]		;THEN GET THE DCD FOR DRIVER.
		MOV	AL,ES:[BX+DCD_FLAGS]	;GET DRIVER FLAGS TO AL.
		LEA	BP,[DI+UCB_CIRB]	;CURRENT IRB ADDRESS CELL,
		LEA	BX,[DI+UCB_STS]		;ADDR OF UNIT STATUS BYTE
		ADD	DI,UCB_QIRB		;WAIT QUEUE HEAD.
		TEST	AL,DFL_PARA		;PARALLEL DRIVER?
		JNE	TYP26			;YES.
		MOV	DI,XDCB			;NO:
		LEA	BP,[DI+DCB_CIRB]	;GET SAME STUFF FOR THE
		LEA	BX,[DI+DCB_STS]		;SERIAL DRIVER CASE.
		ADD	DI,DCB_QIRB
;
;	AT THIS POINT, FOR BOTH SERIAL AND PARALLEL DRIVERS:
;		BP= ADDRESS OF "CURRENT IRB" CELL IN DCB OR UCB,
;		BX= ADDRESS OF STATUS BYTE CELL IN DCB OR UCB,
;		DI= HEAD CELL ADDRESS FOR DRIVER OR UNIT IRB WAIT QUEUE.
;
TYP26:
		TEST	BYTE PTR [BX],UNIT_BUSY	;IS "UNIT" BUSY?
		JNE	TY210			;IF SO, WE'RE DONE.
TYP27:
		MOV	SI,[DI]			;CARRY IRB ADDRESS IN SI.
		TEST	SI,SI			;ANY MORE?
		JE	TY210			;NO, NONE TO START JUST NOW.
		PUSH	BX			;SAVE ADDRESS OF STATUS CELL.
		MOV	BX,[SI+IRB_UCB]		;YES: GET UNIT UCB ADDRESS
		MOV	XUCB,BX			;SET CONTROL VARIABLE.
		MOV	AX,[BX+UCB_TCB]		;GET RESERVER'S TCB ADDRESS
		POP	BX			;RESTORE STATUS CELL ADDRESS
		TEST	AX,AX			;HAS ANYONE RESERVED UNIT?
		JE	TYP28			;NOT RESERVED.
		CMP	AX,[SI+IRB_TCB]		;YES: RESERVER = REQUESTOR?
		JE	TYP28			;YES!
		TEST	BYTE PTR [SI+IRB_FUNCT],PRE_EMPT	;NO: PREEMPT?
		JNZ	TYP28			;YES!
		MOV	DI,SI			;NO:
		JMP	SHORT TYP27		;TRY NEXT IRB.
;
;	WE HAVE FOUND A STARTABLE IRB:
;
;	1)REMOVE IT FROM THE WAIT QUEUE,
;	2)FINISH CONFIGURING THE PRIVATE VARIABLES,
;	3)SET DEVICE/UNIT BUSY AND DISPATCH THE FUNCTION INDICATED.
;
;	AT THIS POINT IN THE SCAN,
;		BP AND BX CARRY THE ADDRESSES DESCRIBED ABOVE,
;		SI= ADDRESS OF THE IRB CHOSEN FOR ACTIVATION,
;		DI= ADDRESS OF PREVIOUS IRB OR HEAD OF WAIT QUEUE.
;
TYP28:
		MOV	AX,[SI]			;REMOVE THE IRB FROM WHATEVER
		MOV	[DI],AX			;QUEUE IT'S ON.
		OR	BYTE PTR [BX],UNIT_BUSY	;SET DEVICE OR UNIT BUSY.
		MOV	XCIRB,SI		;IRB ADDRESS TO CTL. VARIABLE.
		MOV	DS:[BP],SI		;ALSO IN CURRENT IRB CELL OF
						;DCB OR UCB, WHICHEVER.
		MOV	AX,[SI+IRB_SEQ]		;GET IRB SEQUENCE NUMBER FROM
		MOV	BX,[SI+IRB_UCB]		;IRB AND REGISTER IT IN THE
		MOV	[BX+UCB_SEQ],AX		;UCB BEFORE ACTIVATION.
		MOV	AL,[SI+IRB_FUNCT]	;GET FUNCTION CODE BYTE:
		AND	AL,FUNCT_MASK		;REMOVE UPPER BITS:
		JE	TYP29R			;RESERVE = FUNCTION CODE 0.
		DEC	AL			;NO:
		JE	TYP29S			;RELEASE = FUNCTION CODE 1.
		DEC	AL			;NO: NORMALIZE TO BOTTOM USER
						;FUNCTION CODE OF 2.
		CBW
		SHL	AX,1			;QUADRUPLE THE FUNCTION CODE
		SHL	AX,1			;AS INDEX INTO VECTOR POINTED
		MOV	DI,AX			;TO BY DCD_SVC:
		MOV	BX,XDCB			;GET THE DCB ADDRESS AND THEN
		MOV	BX,[BX+DCB_DCD]		;THE DCD OFFSET.
		MOV	AX,MTOS_GROM		;ONCE MORE WE ENTER THE GLOBAL
		MOV	ES,AX			;ROM SEGMENT:
		LES	BX,DWORD PTR ES:[BX+DCD_SVC]	;GET TABLE BASE ADDRESS
		CALL	DWORD PTR ES:[BX][DI]	;ENTER CORRECT ROUTINE.
TY210:
		RET				;ALL DONE.
;
;	BUILTIN RESERVE FUNCTION:
;
;		IF THE UNIT IS CURRENTLY UNRESERVED, RESERVE IT IN FAVOR
;	OF THE REQUESTOR. IF HE HAS ALREADY RESERVED IT, RETURN AN ERROR.
;
TYP29R:
		MOV	BX,XUCB			;RESERVE: GET UCB MARKER
		CMP	WORD PTR [BX+UCB_TCB],0	;IS IT 0?
		JE	TYP29R1			;YES: WE MAY RESERVE.
		MOV	XPRIMARY,BAD_RSRV	;NO: BAD RESERVATION ATTEMPT
		JMP	FAR PTR XOVER		;COMPLETE THE REQUEST.
TYP29R1:
		MOV	AX,[SI+IRB_TCB]		;GET TCB OF REQUESTOR
		MOV	[BX+UCB_TCB],AX		;SET INTO UCB MARKER.
		MOV	XPRIMARY,SUCCESS
		JMP	FAR PTR XOVER		;COMPLETE THE REQUEST.
;
;	BUILTIN RELEASE FUNCTION:
;
;		IF THE UNIT IS RESERVED TO THE REQUESTOR, RELEASE IT TO 
;	UNRESERVED STATUS. IF IT IS ALREADY UNRESERVED, RETURN AN ERROR.
;
TYP29S:
		MOV	BX,XUCB			;GET UCB ADDRESS
		CMP	WORD PTR [BX+UCB_TCB],0	;IS MARKER 0?
		JNE	TYP29S1			;NO, PROCEED.
		MOV	XPRIMARY,BAD_RSRV	;YES: INVALID RELEASE ATTEMPT
		JMP	FAR PTR XOVER		;COMPLETE THE REQUEST.
TYP29S1:
		MOV	WORD PTR [BX+UCB_TCB],0	;CLEAR THE MARKER
		MOV	XPRIMARY,SUCCESS
		JMP	FAR PTR XOVER		;COMPLETE THE REQUEST.
TYPE2_SCHDLR	ENDP
PAGE
;************************************************************************
;*	XOVER:		TYPE 2 DRIVER REQUEST COMPLETION ROUTINE.       *
;************************************************************************
;
;	ENTERED BY JMP INSTRUCTION!
;
;	INPUT:	XCIRB=	ADDRESS OF IRB TO BE COMPLETED.
;	OUTPUT:	NONE, FORMALLY.
;
;	THIS ROUTINE IS ENTERED AT S STATE, FROM A DRIVER OR TYPE2_SCHDLR. ITS
;	INVOCATION CAUSES MTOS TO DECLARE THE REQUEST WHOSE ADDRESS IS IN 
;	XCIRB COMPLETE, AND TO ENTER THE RESCHEDULING SCAN FOR THE DEVICE UNIT
;	WHOSE DEVICE AND UNIT NUMBERS ARE IN THE IRB.
;
;	NOTE!!
;		IF ENTERED FROM A DRIVER, IT IS ASSUMED THAT THE DOUBLEWORD
;	ATOP THE STACK IS THE RETURN ADDRESS TO THE SCHEDULER. THUS, WHEN THIS
;	ROUTINE FINISHES ITS PROCESSING, IT JMPS TO TYPE2_SCHDLR TO SEEK OUT
;	A FRESH REQUEST TO SCHEDULE FOR THE DRIVER.
;
XOVER		PROC	FAR
		MOV	AX,MTOS_DS
		MOV	DS,AX			;NECESSARY PRECONDITION.
		MOV	SI,XCIRB		;GET IRB ADDRESS
		MOV	AL,[SI+IRB_DEVICE]	;GET DEVICE AND
		MOV	AH,[SI+IRB_UNIT]	;UNIT NUMBERS
		MOV	XDEVNUM,AL		;PUT THEM INTO THE
		MOV	XDEVUNIT,AH		;CONTROL VARIABLES.
		CALL	FAR PTR XDEVDATA	;SET UP XDCB, XUCB.
		CALL	FAR PTR XIODONE		;COMPLETE THE REQUEST.
		MOV	BX,XDCB			;GET DCB ADDRESS,
		AND	BYTE PTR [BX+DCB_STS],NOT UNIT_BUSY ;CLEAR SERIAL BUSY,
		MOV	BX,XUCB			;FOR PARALLEL DRIVERS THERE ARE
		AND	BYTE PTR [BX+UCB_STS],NOT UNIT_BUSY	;INDEPENDENT
						;BUSY BITS IN THE UCBS.
		JMP	FAR PTR TYPE2_SCHDLR	;CROSS OVER TO RESCHEDULER.
XOVER		ENDP
PAGE
;*************************************************************************
;*	XTIMEOUT:	CREATE A TIME MONITOR BLOCK FOR TYPE 2 DRIVER.   *
;*************************************************************************
;
;	THIS ROUTINE CREATES A TYPE 2 DRIVER'S TIME MONITOR BLOCK, AND SEES TO
;	IT THAT THE CORRECT PROCESSING WILL TAKE PLACE UPON ITS EXPIRATION.
;
;	INPUT:	XCIRB=	CURRENT IRB ADDRESS,
;		XTUNIT= TIME UNIT CODE, 0 TO 5
;		XTNUM=  NUMBER OF UNITS.
;	OUTPUT:	NONE.
;
XTIMEOUT	PROC	FAR
		PUSH	SI
		PUSH	DS
		PUSH	BX
		PUSH	AX
		MOV	AX,MTOS_DS
		MOV	DS,AX
		MOV	SI,XCIRB		;GET IRB ADDRESS
		MOV	SI,[SI+IRB_TMB]		;GET TMB ADDRESS
		TEST	SI,SI			;ANY?
		JE	XT0			;NO.
		MOV	AL,XTUNIT		;YES: GET TIME UNIT CODE.
		SHL	AL,1			;SHIFT IT UP
		SHL	AL,1			;TWO BITS.
		MOV	AH,XTNUM		;GET NUMBER OF UNITS.
		CALL	X_GET_INTERVAL		;COMPUTE MILLISECOND INTERVAL.
		MOV	[SI+TMB_TIME],CX	;ENTER THAT INTERVAL INTO
		MOV	[SI+TMB_TIME+2],DX	;THE TMB.
		MOV	WORD PTR [SI+TMB_SADR],OFFSET TMB_EXPIRED ;SET ADDRESS OF
		MOV	[SI+TMB_SADR+2],CS	;TIMEOUT HANDLER ROUTINE.
		MOV	BYTE PTR [SI+TMB_TYPE],1
		MOV	BX,XCIRB		;GO THROUGH IRB
		MOV	AX,[BX+IRB_UCB]		;FOR UCB ADDRESS
		MOV	[SI+TMB_UCB],AX		;THAT GOES IN TMB, TOO.
		MOV	BX,SI
		CALL	X_QUEUE_TMB		;ACTIVATE THE TMB
XT0:
		POP	AX
		POP	BX
		POP	DS
		POP	SI
		RET
XTIMEOUT	ENDP
PAGE
;********************************************************
;*	HANDLER FOR TIMEOUTS CREATED BY XTIMEOUT:       *
;********************************************************
;
TMB_EXPIRED	PROC	FAR
		MOV	BX,XUCB			;GET UCB ADDRESS
		MOV	DI,[BX+UCB_DCB]		;GET DCB ADDRESS
		MOV	SI,[DI+DCB_DCD]		;GET DCD ADDRESS
		MOV	AX,MTOS_GROM
		MOV	ES,AX
		TEST	BYTE PTR ES:[SI+DCD_FLAGS],DFL_PARA ;PARALLEL DRIVER?
		JNE	TE1			;YES.
		MOV	SI,[DI+DCB_CIRB]	;NO.
		TEST	BYTE PTR [DI+DCB_STS],UNIT_BUSY	;DRIVER BUSY?
		JE	TE10			;NO.
TE2:
		MOV	XCIRB,SI		;IRB ADDRESS TO XCIRB,
		MOV	XPRIMARY,TIME_OUT	;TIMEOUT STATUS TO XSTATUS,
		JMP	FAR PTR XOVER			;COMPLETE THE REQUEST.
TE10:
		RET
TE1:
		MOV	SI,[BX+UCB_CIRB]	;GET CURRENT UNIT IRB.
		TEST	BYTE PTR [BX+UCB_STS],UNIT_BUSY	;UNIT BUSY?
		JE	TE10			;NO.
		JMP	SHORT TE2		;YES.
TMB_EXPIRED	ENDP
PAGE
;******************************************************************
;*	XDEVDATA    -   GET DCB AND UCB ADDRESSES FOR DEV/UNIT    *
;*	X_DEVDATA   -   (ASSEMBLER ROUTINE)                       *
;******************************************************************
;
;	THE DCB AND UCB ADDRESSES ARE COMPUTED USING THE SPECIFIED
;	DEVICE (DRIVER) NUMBER AND UNIT NUMBER.
;
;					XDEVDATA		X_DEVDATA
;					-------			---------
;	INPUT:	DEVICE # (BYTE)		XDEVNUM			    AL
;		UNIT # (BYTE)		XDEVUNIT		    DL
;	OUTPUT:	DCB ADR (DWORD)		XDCB			    DI
;		UCB ADR (DWORD)		XUCB			    BX
;	
;	IF X_DEVDATA RETURNS CARRY=1, IT INDICATES THAT THE DEVICE/UNIT
;	NUMBER COMBINATION WAS INVALID.
;
;		FOR X_DEVDATA, DS IS ASSUMED TO CONTAIN MTOS_DS.
;
XDEVDATA	PROC	FAR
		PUSH	AX
		PUSH	DS
		PUSH	DX
		PUSH	BX
		PUSH	DI
		MOV	AX,MTOS_DS		;SET DS FOR MTOS DATA
		MOV	DS,AX
		MOV	XDCB+2,AX
		MOV	XUCB+2,AX
		MOV	AL,XDEVNUM		;DEVICE (DRIVER) #
		MOV	DL,XDEVUNIT		;UNIT #
		CALL	FAR PTR X_DEVDATA	;USE THE ASSEMBLER ROUTINE
		MOV	XUCB,BX			;SAVE UCB
		MOV	XDCB,DI			;   DCB
		POP	DI
		POP	BX
		POP	DX
		POP	DS
		POP	AX
		RET
XDEVDATA ENDP
;
;	X_DEVDATA
;
X_DEVDATA	PROC	FAR
		PUSH	CX
		MOV	CX,X_DRIVER_NUM
		CMP	AL,CL			;IS DRIVER NUMBER LEGAL?
		POP	CX
		JB	XDEV0			;YES.
		STC				;NO
		RET
XDEV0:
		PUSH	AX
		PUSH	DX			;SAVE DX FOR LATER
		XOR	AH,AH			;INSURE 0
		MUL	CS:X_DCB_SIZE_VAR	;SIZE * DEVICE # = OFFSET
		ADD	AX,OFFSET X_DCB_TABLE   ;ADD OFFSET TO TABLE TO GET ADR
		MOV	DI,AX			;DCB IN DI
		MOV	AL,DL			;UNIT NUMBER TO AL.
		PUSH	ES			;WE MUST GO INTO THE
		MOV	BX,MTOS_GROM
		MOV	ES,BX			;GLOBAL ROM SEGMENT
		MOV	BX,[DI+DCB_DCD]		;GET DCD ADDRESS
		CMP	AL,ES:[BX+DCD_UNITS]	;CHECK UNIT COUNT FOR DRIVER
		POP	ES
		JB	XDEV1			;UNIT NUMBER IS LEGAL.
		STC				;UNIT NUMBER IS ILLEGAL.
		POP	DX
		POP	AX
		RET
XDEV1:
		XOR	AH,AH
		MUL	CS:X_UCB_SIZE_VAR	;UCB OFFSET
		ADD	AX,WORD PTR [DI+DCB_UCB] ;OFFSET + START ADR = UCB ADR
		MOV	BX,AX
		POP	DX			;RESTORE DX
		POP	AX
		CLC				;ALL'S WELL: TELL USER.
		RET
X_DEVDATA	ENDP
PAGE
;********************************************************************
;*      XDELTMB:    DELETE DRIVER TIMER BLOCK WHERE FOUND           *
;********************************************************************
;
;	INPUT:	CURRENT IRB IS USED TO GET TMB (IRB_TMB)
;	OUTPUT:	NONE
;
;		THIS ROUTINE USES THE TMB ADDRESS DECLARED IN THE CURRENT IRB
;	TO DRIVE ITS PROCESSING. IF THE ADDRESS IS ZERO, THE TMB HAS ALREADY 
;	BEEN DEALLOCATED. IF NOT, IT IS SOUGHT ON THE TIMER QUEUE AND REMOVED
;	IF FOUND THERE, AND THEN DEALLOCATED.
;
XDELTMB 	PROC	FAR
		PUSH	DS
		PUSH	BX
		PUSH	AX
		MOV	AX,MTOS_DS		;SET MTOS DS
		MOV	DS,AX
		CALL	FAR PTR XDQTMB		;CHECK TIME QUEUE
		MOV	BX,XCIRB		;GET IRB ADDRESS
		TEST	BX,BX			;ZERO?
		JE	DELTMB_DONE		;YES, NO WORK.
		XOR	AX,AX			;NO:
		XCHG	AX,[BX+IRB_TMB]		;GET TMB ADDRESS, ZAP FIELD.
		TEST	AX,AX			;ANY TMB?
		JE	DELTMB_DONE		;NO.
		MOV	BX,AX			;YES:
		CALL	X_PUT_SMR		;DEALLOCATE THE TMB.
DELTMB_DONE: 
		POP	AX
		POP	BX
		POP	DS
		RET
XDELTMB		ENDP
PAGE
;******************************************************************
;*      XERRLOG:  LOG AN ERROR TO SYSTEM CONSOLE                  *
;******************************************************************
;
;	INPUT:	XERRMSG HAS THE ADDRESS OF THE MESSAGE. THE MESSAGE MAY BE
;		TERMINATED BY A 0 (ZERO) OR A CARRIAGE RETURN CHARACTER;	;	OUTPUT:	NONE
;
;	THE ADDRESS SPECIFIED IN XERRMSG IS SENT TO THE ERROR LOG TASK VIA
;	ITS AX/BX STARTUP VALUES. THE TASK IS QSTRTED BY THIS ACTION. IT PRINTS
;	THE SPECIFIED MESSAGE, UP TO 255 CHARACTERS, AND ISSUES AN ENDT.
;
XERRLOG		PROC	FAR
		PUSH	AX
		PUSH	BX
		PUSH	CX
		PUSH	DS
		PUSH	SI
		PUSH	DI
		PUSH	BP
		MOV	AX,MTOS_DS
		MOV	DS,AX			;SET UP MTOS_DS.
		MOV	AX,X_ERRLOG_TASK	;GET TASK ID.
		CALL	X_GET_TCB_ADR		;YES: GET TCB ADDRESS.
		JC	NOERRT			;** NO ERROR LOGGER **
		MOV	DI,BX
		XOR	SI,SI			;NO REQUESTOR,
		XOR	CH,CH			;NO COORDINATION,
		MOV	CL,255			;HIGH PRIORITY.
		MOV	AX,XERRMSG		;GET THE ADDRESS OF THE
		MOV	BX,XERRMSG+2		;MESSAGE TO AX,BX
		XOR	BP,BP			;AUTOMATIC ALLOCATE.
		CALL	X_START_TASK		;DO IT!
NOERRT:
		POP	BP
		POP	DI
		POP	SI
		POP	DS
		POP	CX
		POP	BX
		POP	AX
		RET
XERRLOG		ENDP
PAGE
;********************************************************************
;*	XIODONE  -  COMPLETE I/O REQUEST PROCESSING                 *
;********************************************************************
;
;	INPUT:	XCIRB=	IRB TO BE COMPLETED.
;		XSTATUS CONTAINS THE 4 BYTES OF STATUS TO RETURN
;	OUTPUT:	NONE
;
;		THIS ROUTINE POST-PROCESSES A COMPLETED I/O REQUEST, SENDING
;	FLAGS AND STATUS BACK TO THE REQUESTOR, KILLING ANY UNEXPIRED TIMER,
;	REMOVING THE ASSOCIATED IRB FROM THE CORRECT QUEUE IF IT'S ON ONE,
;	AND DEALLOCATING BOTH THE IRB AND ITS TMB.
;
XIODONE		PROC	FAR
		PUSH	AX
		PUSH	DS
		MOV	AX,MTOS_DS		;SET UP THE MTOS DS
		MOV	DS,AX
		PUSH	BX
		PUSH	CX
		PUSH	DX
		PUSH	DI
		PUSH	SI
		PUSH	ES
		PUSH	BP
		MOV	DI,XCIRB		;GET CURRENT IRB ADR
		TEST	DI,DI			;IRB DISPOSED OF?
		JE	CONT_3			;YES.
		MOV	BX,[DI+IRB_UCB]		;NO: GET UCB ADDRESS FROM IRB
		MOV	AX,[BX+UCB_SEQ]		;GET REGISTERED SEQUENCE NR.
		CMP	AX,[DI+IRB_SEQ]		;DOES IT MATCH IRB SEQUENCE NR?
		JNE	CONT_3			;IF NOT, DO NOTHING.
		MOV	BX,[DI+IRB_TMB]		;YES: TMB ADDRESS FROM IRB
		TEST	BX,BX			;DEALLOCATED YET?
		JE	SET_STATUS		;YES.
		CALL	X_DQUEUE_TMB		;NO: REMOVE FROM TIME QUEUE
		CALL	X_PUT_SMR		;RETURN SMR BLOCK.
SET_STATUS:
		LES	SI,DWORD PTR [DI+IRB_STATUS]	;SEND BACK COMPLETION
		MOV	AX,XSTATUS		;STATUS: A DOUBLEWORD.
		MOV	WORD PTR ES:[SI],AX
		MOV	AX,XSTATUS+2
		MOV	WORD PTR ES:[SI+2],AX
		MOV	BX,[DI+IRB_TCB]		;GET REQUESTOR'S TCB ADDRESS
		TEST	BX,BX			;ANY TCB?
		JE	CONT_1			;NO, BYPASS EF LOGIC, BLOCK TOO
		MOV	DL,[DI+IRB_GRP]		;YES: GET FLAG GROUP
		MOV	CX,[DI+IRB_FLAGS]	;     GET FLAG SET MASK.
		CALL	X_SET_EF		;SET SPECIFIED EVENT FLAGS.
		TEST	BYTE PTR [DI+IRB_FUNCT],BLOCK_TASK	;TASK BLOCKED?
		JE	CONT_1			;NO.
		CALL	X_SET_READY		;YES: READY THE TASK.
;
;	BEFORE DEALLOCATING THE IRB (THE FINAL STAGE OF REQUEST COMPLETION),
;	IT MUST BE REMOVED FROM ANY QUEUE IT MIGHT BE ON. THIS REQUIRES THAT
;	WE CHECK THE APPROPRIATE WAIT QUEUE BEFORE PROCEEDING.
;
CONT_1:
		MOV	BX,DI
		MOV	SI,[DI+IRB_UCB]			;UCB ADR
		MOV	WORD PTR [SI+UCB_CIRB],0	;DESTROY IRB ADDRESS.
		MOV	DI,[SI+UCB_DCB]			;DCB ADR
		MOV	WORD PTR [DI+DCB_CIRB],0	;DESTROY IRB ADDRESS.
		MOV	AX,MTOS_GROM		;POINT ES AT THE GLOBAL ROM
		MOV	ES,AX			;SEGMENT AGAIN.
		MOV	BP,[DI+DCB_DCD]		;ADDRESS THE DCD FOR THE DRIVER
		ADD	DI,DCB_QIRB		;POINT DI AT SERIAL WAIT QUEUE
		TEST	BYTE PTR ES:[BP+DCD_FLAGS],DFL_PARA	;PARALLEL DRVR?
		JZ	CONT_2			;NO, SERIAL.
		LEA	DI,[SI+UCB_QIRB]	;YES: CHECK PARALLEL WAIT QUEUE
CONT_2:
		CALL	X_DQUEUE		;REMOVE IRB FROM WAIT QUEUE.
		CALL	X_PUT_SMR		;RETURN SMR BLOCK.
		MOV	XCIRB,0			;REMOVE LAST TRACES.
CONT_3:
		POP	BP
		POP	ES
		POP	SI
		POP	DI
		POP	DX
		POP	CX
		POP	BX
		POP	DS
		POP	AX
		RET
XIODONE		ENDP
PAGE
;************************************************************
;*      XQSIRB:    QUEUE IRB TO WAIT Q OF SERIAL DRIVER     *
;*      XQPIRB:    QUEUE IRB TO WAIT Q OF PARALLEL DRIVER   *
;************************************************************
;
;	INPUT:	XCIRB=	ADDRESS OF IRB TO QUEUE.
;	OUTPUT:	NONE
;
XQSIRB		PROC	FAR
		PUSH	AX
		PUSH	DS
		MOV	AX,MTOS_DS		;SETUP MTOS DATA SEG
		MOV	DS,AX
		PUSH	BX
		PUSH	DI
		MOV	BX,XCIRB		;GET CURRENT IRB
		TEST	BX,BX			;IS THERE ONE?
		JE	XQS0			;NO.
		MOV	DI,[BX+IRB_UCB]		;UCB ADR
		MOV	DI,[DI+UCB_DCB]		;DCB ADR
		ADD	DI,DCB_QIRB		;ADR OF QUEUE
		CALL	X_QUEUE			;QUEUE IT
XQS0:
		POP	DI
		POP	BX
		POP	DS
		POP	AX
		RET
XQSIRB  	ENDP
;
XQPIRB		PROC	FAR
		PUSH	AX
		PUSH	DS
		MOV	AX,MTOS_DS
		MOV	DS,AX
		PUSH	BX
		PUSH	DI
		MOV	BX,XCIRB		;CURRENT IRB
		TEST	BX,BX			;IS THERE ONE?
		JE	XQP0			;NO.
		MOV	DI,[BX+IRB_UCB]		;UCB ADR
		ADD	DI,UCB_QIRB		;CALCULATE QUEUE ADR
		CALL	X_QUEUE
XQP0:
		POP	DI
		POP	BX
		POP	DS
		POP	AX
		RET
XQPIRB  	ENDP
PAGE
;***************************************************************
;*      XQTMB:    QUEUE A TIME MONITOR BLOCK TO TIME QUEUE     *
;***************************************************************
;
;	INPUT:	THE IRB_TMB FIELD OF THE IRB WHOSE ADDRESS IS IN XCIRB.
;	OUTPUT:	NONE
;
;		THIS ROUTINE USES THE IRB_TMB FIELD OF THE CURRENT IRB (XCIRB)
;	FOR THE TMB ADDRESS, AND CALLS X_QUEUE_TMB TO ATTACH THAT TMB TO THE
;	QUEUE OF ACTIVE TMBS.
;
XQTMB		PROC	FAR
		PUSH	AX
		PUSH	DS
		MOV	AX,MTOS_DS		;SETUP MTOS DATA SEG.
		MOV	DS,AX
		PUSH	BX
		MOV	BX,XCIRB		;CURRENT IRB
		TEST	BX,BX			;IS THERE ONE?
		JE	XQT0			;NO.
		MOV	AX,[BX+IRB_UCB]		;GET UCB ADDRESS
		MOV	BX,[BX+IRB_TMB]		;CURRENT TMB
		TEST	BX,BX			;IS THERE ONE?
		JE	XQT0			;NO.
		MOV	[BX+TMB_UCB],AX		;YES: PUT IN THE UCB ADDRESS
		MOV	BYTE PTR [BX+TMB_TYPE],1	;AND MARK IT TYPE 1.
		CALL	X_QUEUE_TMB		;QUEUE IT TO THE TIME QUEUE
XQT0:
		POP	BX
		POP	DS
		POP	AX
		RET	
XQTMB		ENDP
PAGE
;************************************************
;*	XDQTMB: DEACTIVATE TMB WITHOUT          *
;*              DEALLOCATING IT.                *
;************************************************
;
;	IF THE TMB HAS NOT PREVIOUSLY BEEN DEALLOCATED, IT IS REMOVED FROM THE
;	TIMER QUEUE, AND LEFT AVAILABLE TO THE DRIVER.
;
;	INPUT:	XCIRB PASSES CURRENT IRB, IN WHICH IRB_TMB IS TMB ADDRESS.
;	OUTPUT:	NONE.
;
XDQTMB		PROC	FAR
		PUSH	AX
		PUSH	DS
		PUSH	BX
		MOV	AX,MTOS_DS
		MOV	DS,AX			;MTOS_DS ADDRESSING.
		MOV	BX,XCIRB		;GET IRB ADDRESS.
		TEST	BX,BX			;ANY?
		JE	DQT_DONE		;NO.
		MOV	BX,[BX+IRB_TMB]		;YES: GET TMB ADDRESS.
		TEST	BX,BX			;ANY?
		JE	DQT_DONE		;NO, PREVIOUSLY DEALLOCATED.
		CALL	X_DQUEUE_TMB		;YES: REMOVE FROM TIMER QUEUE.
DQT_DONE:
		POP	BX
		POP	DS
		POP	AX
		RET
XDQTMB		ENDP
PAGE
;*****************************************************************
;*      XQSETUNS:    QSTRT UNSOLICITED INTERRUPT TASK            *
;*****************************************************************
;
;	THIS ROUTINE STARTS THE SPECIFIED TASK IN A QSTRT-LIKE FASHION.
;
;	INPUT:	THE CURRENT UCB (XUCB) IS USED TO GET THE UCB WHICH
;		HAS THE UCD WHICH HAS THE TASK ID TO BE STARTED.
;	        THE STARTUP AX,BX VALUES FOR THE TASK WILL BE TAKEN FROM
;		THE XSTATUS DOUBLEWORD.
;	OUTPUT:	NONE
;
XQSETUNS 	PROC	FAR
		PUSH	ES
		PUSH	DS
		PUSH	BP
		PUSH	DI
		PUSH	SI
		PUSH	CX
		PUSH	BX
		PUSH	AX
		MOV	AX,MTOS_DS
		MOV	DS,AX		;SET UP MTOS_DS.
		MOV	AX,MTOS_GROM	;WE ALSO NEED TO BE ABLE TO REACH
		MOV	ES,AX		;THE GLOBAL ROM SEGMENT.
		MOV	BX,XUCB		;GET UCB ADDRESS.
		MOV	DI,[BX+UCB_UCD]	;    UCD ADDRESS.
		MOV	AX,ES:[DI+UCD_UNS]	;UI TASK ID.
		CALL	X_GET_TCB_ADR	;GET UI TASK TCB ADDRESS.
		JC	XSU0		;NOT A VALID TASK ID!
		MOV	DI,BX		;YES:
		XOR	SI,SI		;NO REQUESTOR,
		XOR	CH,CH		;NO COORDINATION,
		LES	BX,DWORD PTR [BX+TCB_TCD]	;GO TO THE TCD FOR THE
		MOV	CL,ES:[BX+TCD_INPRI]	;INHERENT PRIORITY,
		XOR	BP,BP		;AUTOMATIC ALLOCATION,
		MOV	AX,XSTATUS	;THE AX/BX STARTUP VALUES ARE THE TWO
		MOV	BX,XSTATUS+2	;WORDS IN XSTATUS.
		CALL	X_START_TASK	;GO!
XSU0:
		POP	AX
		POP	BX
		POP	CX
		POP	SI
		POP	DI
		POP	BP
		POP	DS
		POP	ES
		RET
XQSETUNS 	ENDP
PAGE
;***************************************************************************
;*        XSETUNS:	XQSETUNS FOR DORMANT TASKS ONLY.                   *
;***************************************************************************
;
;	THIS ROUTINE STARTS THE UNIT'S UNSOLICITED INPUT TASK ONLY IF THE
;	TASK IS CURRENTLY DORMANT (I.E., IN A STRT-LIKE MANNER). IN ALL 
;	OTHER RESPECTS IT BEHAVES THE SAME WAY AS XQSETUNS, WHICH IT CALLS.
;
XSETUNS		PROC	FAR
		PUSH	AX
		PUSH	BX
		PUSH	DS
		PUSH	ES
		MOV	AX,MTOS_GROM		;WE'LL NEED TO REACH INTO THE
		MOV	ES,AX			;GLOBAL ROM SEGMENT.
		MOV	AX,MTOS_DS
		MOV	DS,AX			;THIS SHOULD NEED NO COMMENT.
		MOV	BX,XUCB			;GET UNIT ADDRESS.
		MOV	BX,[BX+UCB_UCD]		;GO THROUGH UCB TO UCD.
		MOV	AX,ES:[BX+UCD_UNS]	;GET UI TASK ID FROM UCD.
		CALL	X_GET_TCB_ADR		;GET TCB ADDRESS...
		JC	XST0			;INVALID TASK ID!
		CMP	BYTE PTR [BX+TCB_STATE],DORMANT	;TASK DORMANT?
		JNE	XST1			;NO, DO NOTHING.
		CALL	XQSETUNS		;YES: START THE TASK.
XST1:
XST0:
		POP	ES
		POP	DS
		POP	BX
		POP	AX
		RET
XSETUNS		ENDP
PAGE
;**************************************************************************
;*         XALOC:	ALLOCATE BLOCK FROM SPECIFIED MEMORY POOL         *
;**************************************************************************
;
;	INPUT:	XPOOLNUM CARRIES MEMORY POOL NUMBER,
;		XBLKLEN CARRIES LENGTH OF DESIRED BLOCK IN BYTES.
;	OUTPUT:	IF ALLOCATION SUCCEEDED, THE BLOCK ADDRESS IS LEFT IN THE
;		XPOINTER POINTER VARIABLE. IF IT FAILED, XPOINTER IS SET TO
;		THE DISTINCT VALUE 0:0.
;
XALOC		PROC	FAR
		PUSH	AX
		PUSH	BX
		PUSH	CX
		PUSH	DX
		PUSH	DS
		PUSH	ES
		MOV	AX,MTOS_DS
		MOV	DS,AX
		MOV	DL,XPOOLNUM
		XOR	DH,DH
		MOV	CX,XBLKLEN		;DX= POOL#, CX= BYTES NEEDED,
		CALL	X_GET_BLOCK		;TRY TO ALLOCATE...
		JNC	XA0			;GOT THE REQUESTED BLOCK.
		XOR	BX,BX			;NO,
		MOV	ES,BX			;SET BLOCK ADDR TO 0:0.
XA0:
		MOV	XOFFSET,BX		;DELIVER BLOCK ADDRESS
		MOV	XSEGMENT,ES		;IN XPOINTER VARIABLE.
		POP	ES
		POP	DS
		POP	DX
		POP	CX
		POP	BX
		POP	AX
		RET
XALOC		ENDP
PAGE
;***********************************************************************
;*       XDALOC:	DEALLOCATE BLOCK TO SPECIFIED POOL.            *
;***********************************************************************
;
;	INPUT:	XPOINTER CARRIES DOUBLEWORD ADDRESS OF BLOCK TO BE
;		DEALLOCATED,
;		XPOOLNUM CARRIES POOL ID NUMBER,
;		XBLKLEN CARRIES LENGTH OF BLOCK IN BYTES.
;	OUTPUT:	NONE.
;
XDALOC		PROC	FAR
		PUSH	AX
		PUSH	BX
		PUSH	CX
		PUSH	DX
		PUSH	DS
		PUSH	ES
		MOV	AX,MTOS_DS
		MOV	DS,AX
		LES	BX,XPOINTER	;GET ADDRESS OF BLOCK TO BE FREED.
		MOV	DL,XPOOLNUM	;GET POOL NUMBER.
		XOR	DH,DH
		MOV	CX,XBLKLEN	;GET LENGTH OF BLOCK.
		CALL	X_FREE_BLOCK	;DEALLOCATE THE BLOCK.
		POP	ES
		POP	DS
		POP	DX
		POP	CX
		POP	BX
		POP	AX
		RET
XDALOC		ENDP
PAGE
;********************************************************************
;*      XGETSMR:	ALLOCATE SMR BLOCK FROM SMR FREE LIST       *
;*      XPUTSMR:	DEALLOCATE SMR BLOCK TO SMR FREE LIST       *
;********************************************************************
;
;	INPUT:	FOR XGETSMR, NONE; FOR XPUTSMR, OFFSET ADDRESS OF THE 
;		BLOCK IS EXPECTED IN XOFFSET.
;	OUTPUT:	FOR XGETSMR, ADDRESS OF THE BLOCK IS RETURNED IN XOFFSET,
;		FOR XPUTSMR, NONE.
;
XGETSMR		PROC	FAR
		PUSH	AX
		PUSH	BX
		PUSH	DS
		MOV	AX,MTOS_DS
		MOV	DS,AX
		CALL	X_GET_SMR		;ALLOCATE A BLOCK,
		MOV	XOFFSET,BX		;PUT ADDRESS IN XOFFSET,
		POP	DS
		POP	BX
		POP	AX
		RET				;COMPLEX, EH?
XGETSMR		ENDP
;
XPUTSMR		PROC	FAR
		PUSH	AX
		PUSH	BX
		PUSH	DS
		MOV	AX,MTOS_DS
		MOV	DS,AX
		MOV	BX,XOFFSET		;GET ADDRESS OF BLOCK TO FREE
		CALL	X_PUT_SMR		;AND FREE IT.
		POP	DS
		POP	BX
		POP	AX
		RET
XPUTSMR		ENDP
PAGE
;********************************************************************
;*	XDWORD  -  CONVERT OFFSET TO OFFSET POINTER                 *
;********************************************************************
;
;	ROUTINE MAKES MTOS DATA SEGMENT VARIABLES INTO DOUBLE WORD
;	ADDRESSES FOR PL/M PROGRAMS
;
;	INPUT:   XOFFSET=	VALUE TO CONVERT
;	OUTPUT:  XPOINTER=	DOUBLE WORD ADDRESS
;
XDWORD		PROC	FAR
		PUSH	AX
		PUSH	DS
		MOV	AX,MTOS_DS
		MOV	DS,AX
		MOV	XOFFSET+2,AX
		POP	DS
		POP	AX
		RET
XDWORD		ENDP
MTOS_CS 	ENDS
		END
