PAGE	60,132
INCLUDE B:BIG.INC
;
;	SEMACP.ASM:	SEMAPHORE AND CONTROLLED-PROGRAM PROCESSING.
;	F. W. PORRETTO, 12/15/82
;
;		THESE ROUTINES IMPLEMENT ALL CONTROLS FOR THE LEVEL II
;	SEMAPHORES AND CONTROLLED PROGRAMS FACILITIES, WHICH ARE ACTUALLY 
;	ONE AND THE SAME.
;
		NAME	SEMA_CNTRL_PROGRAM_SERVICES
;
		PUBLIC	X_TSF,X_RSF,X_SET_CP_SEM,X_RESET_CP_SEM
		PUBLIC	X_CNTRL
;
MTOS_CS		SEGMENT	PARA PUBLIC
		ASSUME	CS:MTOS_CS,DS:MTOS_DS,SS:MTOS_DS
		EXTRN	X_SET_READY	:NEAR
		EXTRN	X_TASK_STATE	:NEAR
		EXTRN	X_SFLAGS	:NEAR
		EXTRN	X_QUEUE		:NEAR
		EXTRN	X_DQUEUE	:NEAR
		EXTRN	X_GET_SMR	:NEAR
		EXTRN	X_PUT_SMR	:NEAR
		EXTRN	X_QUEUE_TMB	:NEAR
		EXTRN	X_DQUEUE_TMB	:NEAR
		EXTRN	X_GET_INTERVAL	:NEAR
;
;	LOCAL EQUATES
;
;SEMAPHORE EQUATES
;
SFLAG		EQU	01		;SDB OFFSET TO SEMA ID
SFLAG_MASK	EQU	0FFFH		;ISOLATE SEMA ID
;
;CONTROLLED PROGRAM EQUATES
;
SE		EQU	01H		;SDB OFFSET OF START/END
CID		EQU	02H		;OFFSET OF CONTROLLED PROGRAM ID
SE_MASK		EQU	080H		;ISOLATE START/END BIT
PAGE
;*********************************************************
;*          FIND_SEM:  FIND THE SEMAPHORE DATA           *
;*********************************************************
;
;	THIS PROCEDURE FINDS THE SEMAPHORE DATA IN THE
;	SEMAPHORE TABLE.
;
;	INPUT:	DX=	SEMAPHORE OR CONTROLLED PROGRAM ID
;		CARRY=	0 FOR SEMAPHORE,
;			1 FOR CONTROLLED PROGRAM.
;	OUTPUT:	DI=	ADDRESS OF SEMAPHORE DATA
;
FIND_SEM	PROC	NEAR
		PUSH	DX			;STORE REGISTERS
		MOV	DI,OFFSET X_SCB_TABLE	;PREPARE FOR SEMAPHORE.
		JNC	XF0			;IT IS A SEMAPHORE.
		MOV	DI,OFFSET X_CP_TABLE	;NO, IT'S A C-P.
XF0:
		MOV	AX,SEM_SIZE
		MUL	DX			;MULTIPLY ID X
						;SIZE OF SEMA DATA
						;BLOCK
		ADD	DI,AX			;ADD STARTING ADDRESS
						;OF SEMA DATA TABLE
		POP	DX
		RET
FIND_SEM	ENDP
PAGE
;****************************************************************
;*         X_SET_CP_SEM:  SET A SEMAPHORE/CONTROLLED PROGRAM    *
;****************************************************************
;
;	BOTH THE SEMAPHORE AND CONTROLLED PROGRAM "INNER"
;	ROUTINES CALL THIS PROCEDURE AFTER LOCATING THE
;	SEMAPHORE OR CONTROLLED PROGRAM DATA,RESPECTIVELY.
;
;	SINCE BOTH FACILITIES USE THE SEMAPHORE STRUCTURES
;	AND FUNCTION THE SAME WAY, THIS PROCEDURE HANDLES
;	BOTH AS SEMAPHORES.
;
;	THIS PROCEDURE TESTS THE OWNERSHIP.  IF THE REQUESTING
;	TASK ALREADY OWNS THE SEMAPHORE, THE LEVEL WORD IS
;	INCREMENTED. IF NO ONE OWNS THE SEMAPHORE, THE
;	REQUESTING TASK BECOMES THE OWNER.  IF ANOTHER TASK
;	OWNS THE SEMAPHORE, THIS TASK IS BLOCKED AND A 
;	SEMAPHORE WAIT BLOCK IS ADDED TO THE QUEUE. THE
;	SVC STATUS IS ALSO PREPARED.
;
;	INPUT:	DX=	SEMAPHORE OR C-P NUMBER,
;		CARRY=	0 FOR SEMA,
;			1 FOR C-P.
;		BX=	TCB ADDRESS
;	OUTPUT:	AX=	SVC STATUS.
;
X_SET_CP_SEM	PROC	NEAR
		PUSH	BX
		PUSH	SI
		PUSH	DI
		PUSH	BP
		CALL	FIND_SEM		;LOCATE THE SEMA DATA BLOCK.
		MOV	SI,BX			;SI=TCB ADDRESS
;
;	TEST SEMAPHORE OWNERSHIP
;
		CMP	WORD PTR [DI+SEM_LEVEL],0	;UNOWNED ?
		JE	SETSEM
		CMP	[DI+SEM_TCB],SI		;THIS TASK ?
		JE	INCSEM
;
;	ANOTHER TASK OWNS SEMAPHORE SO THIS TASK MUST WAIT
;
		CALL	X_GET_SMR		;ALLOCATE A BLOCK FOR SWB.
		MOV	AL,[SI+TCB_PRI]		;GET TASK PRIORITY.
		MOV	[BX+SWB_PRI],AL		;PUT PRIORITY TO SWB,
		MOV	[BX+SWB_TCB],SI		;ALSO INSERT TCB ADDRESS.
		TEST	BYTE PTR [SI+TCB_FLAGS],TFL_TSEM	;CANCELLATION
						;AFTER TIME ENABLED?
		JE	XSC0			;NO.
		PUSH	ES
		LES	BP,DWORD PTR [SI+TCB_TCD]	;GET REQUESTOR'S TCD
		MOV	AX,ES:[BP+TCD_TIME]	;GET OVERRIDE TIME
		POP	ES
		PUSH	DX
		PUSH	CX
		PUSH	BX
		CALL	X_GET_INTERVAL		;COMPUTE COUNTDOWN.
		CALL	X_GET_SMR		;ALLOCATE A TMB
		MOV	[BX+TMB_TIME],CX	;PUT COUNTDOWN
		MOV	[BX+TMB_TIME+2],DX	;INTO THE TMB
		POP	BP			;SWB ADDRESS TO BP.
		MOV	DS:[BP+SWB_TMB],BX	;TMB ADDRESS INTO SWB.
		MOV	WORD PTR [BX+TMB_SWB],BP	;AND VICE VERSA.
		MOV	[BX+TMB_TCB],SI		;TCB ADDR INTO TMB,
		MOV	[BX+TMB_SEM],DI		;SEMAPHORE ADDR INTO TMB TOO.
		MOV	BYTE PTR [BX+TMB_TYPE],6	;SEMAPHORE TIMEOUT.
		POP	CX
		POP	DX
		CALL	X_QUEUE_TMB		;START THE TIMER.
		MOV	BX,BP			;SWB ADDRESS TO BX,
XSC0:
		ADD	DI,SEM_WAIT		;ADDRESS WAIT QUEUE,
		CALL	X_QUEUE			;QUEUE THE SWB TO WAIT QUEUE,
		MOV	BX,SI			;BX= TCB
		MOV	AX,BLOCK+STA_SEMA	;BLOCK TASK
		CALL	X_TASK_STATE
		MOV	AX,FLAG_CF		;SET WAIT BIT IN STATUS
		POP	BP
		POP	DI
		POP	SI			;RESTORE REGISTERS
		POP	BX
		RET
;
;	UNOWNED OR OWNED BY THIS TASK
;
SETSEM:
		MOV	[DI+SEM_TCB],SI		;SET OWNERSHIP
INCSEM:
		INC	WORD PTR [DI+SEM_LEVEL]		;INC LEVEL
		XOR	AX,AX			;SET STATUS
		POP	BP
		POP	DI
		POP	SI			;RESTORE REGISTERS
		POP	BX
		RET
X_SET_CP_SEM	ENDP
PAGE
;*********************************************************************
;*         X_RESET_CP_SEM:  RESET SEMAPHORE/CONTROLLED PROGRAM       *
;*********************************************************************
;
;	BOTH THE SEMAPHORE AND CONTROLLED PROGRAM INNER
;	ROUTINES CALL THIS PROCEDURE AFTER LOCATING THE
;	SEMAPHORE OR CONTROLLED PROGRAM DATA,RESPECTIVELY.
;
;	SINCE BOTH FACILITIES USE THE SEMAPHORE STRUCTURES,
;	AND FUNCTION THE SAME WAY THIS PROCEDURE HANDLES
;	THEM BOTH AS SEMAPHORES.
;
;	THIS PROCEDURE TESTS THE OWNERSHIP.  IF THE
;	REQUESTING TASK OWNS THE SEMAPHORE, THE LEVEL
;	WORD IS DECREMENTED AND TESTED. IF THE LEVEL
;	WORD IS  NOW ZERO, THE HIGHEST PRIORITY SWB
;	IS PROCESSED.  IF THE SEMAPHORE IS UNOWNED
;	OR OWNED BY ANOTHER TASK, THE SEMAPHORE IS NOT
;	RESET AND AN ERROR STATUS IS PREPARED.
;
;	INPUT:	DX=	SEMAPHORE OR C-P NUMBER
;		CARRY=	0 FOR SEMA,
;			1 FOR C-P.
;		BX=	TCB ADDRESS
;	OUTPUT:	AX=	SVC STATUS
;
X_RESET_CP_SEM	PROC	NEAR
		PUSH	DX				;SAVE REGISTERS
		PUSH	SI
		PUSH	BX
		CALL	FIND_SEM		;LOCATE THE SEMA DATA BLOCK.
;
;	TEST THE SEMAPHORE OWNERSHIP
;
		CMP	[DI+SEM_TCB],BX		;THIS TASK OWNER ?
		JE	RESETSEM		;YES.
;
;	SEMA UNOWNED OR OWNED BY ANOTHER TASK
;
		MOV	AX,SVC_FAILED+FLAG_CF	;ERROR STATUS
		POP	BX			;RESTORE REGISTERS
		POP	SI
		POP	DX
		RET
;
;	SEMA OWNED BY THIS TASK
;
RESETSEM:
		DEC	WORD PTR [DI+SEM_LEVEL]		;DEC LEVEL WORD
		JNE	RSEMDONE			;STILL NONZERO.
;
;	THIS TASK FINISHED WITH SEMAPHORE
;
		MOV	WORD PTR [DI+SEM_TCB],0		;CLEAR OWNERSHIP
		CMP	WORD PTR [DI+SEM_WAIT],0	;ANYONE WAITING?
		JE	RSEMDONE			;NO.
		MOV	SI,[DI+SEM_WAIT]		;YES: GET TOP SWB.
		MOV	AX,[SI]				;UNLINK IT FROM THE
		MOV	[DI+SEM_WAIT],AX		;SEMA WAIT QUEUE.
		MOV	DX,[SI+SWB_TCB]			;SET OWNERSHIP
		MOV	[DI+SEM_TCB],DX			;TO NEW TCB ADDR.
		INC	WORD PTR [DI+SEM_LEVEL]		;INC LEVEL WORD
		MOV	BX,[SI+SWB_TMB]		;IS THERE A TMB CONTROLLING
		TEST	BX,BX			;THIS SWB'S LIFESPAN?
		JE	RSC0			;NO.
		CALL	X_DQUEUE_TMB		;YES: REMOVE IT FROM THE Q.
		JC	RSC0			;NOT FOUND!
		CALL	X_PUT_SMR		;FOUND: DEALLOCATE IT.
RSC0:
		MOV	BX,DX			;READY THE TASK WHICH JUST
		CALL	X_SET_READY		;GAINED OWNERSHIP
		MOV	BX,SI			;BX=PROCESSED SWB
		CALL	X_PUT_SMR		;FREE SWB(SMR)
		XOR	AX,AX			;SET STATUS
		POP	BX			;RESTORE REGISTERS
		POP	SI
		POP	DX
		RET
;
;	FINISH UP HERE IF THIS TASK KEEPS SEMAPHORE OR IF THIS TASK
;	FREES SEMAPHORE AND NO ONE IS WAITING.
;
RSEMDONE:
		XOR	AX,AX				;SET STATUS
		POP	BX				;RESTORE REGISTERS
		POP	SI
		POP	DX
		RET
X_RESET_CP_SEM	ENDP
PAGE
;*************************************************
;*   X_TSF:    TEST AND SET SEMAPHORE SVC        *
;*************************************************
;
;	THIS PROCEDURE VALIDATES THE SDB PARAMETERS,
;	SETS THE SEMAPHORE OR BLOCKS THE TASK, AND RETURNS
;	STATUS TO THE TASK.
;
;	INPUT:	ES:SI=	SDB
;		BX=	TCB
;	OUTPUT:	NONE
;
X_TSF		PROC	FAR
		MOV	DX,ES:[SI+SFLAG]	;SEMAPHORE ID FROM SDB
		AND	DX,SFLAG_MASK		;ISOLATE ID #
		CMP	DX,X_SEMA_NUM		;VALID ID ?
		MOV	AX,SVC_FAILED+PARM_ERR	;BE READY...
		JAE	TSFERR			;--NO--
		CLC
		CALL	X_SET_CP_SEM		;YES: TEST AND SET SEMA
TSFERR:
		CALL	X_SFLAGS		;RETURN STATUS
		RET
X_TSF		ENDP
PAGE
;*******************************************
;*  X_RSF:    RESET SEMAPHORE SVC          *
;*******************************************
;
;	THIS PROCEDURE VALIDATES THE SDB PARAMETERS,
;	RESETS THE SEMAPHORE, PROCESSES THE WAIT QUEUE,
;	AND RETURNS STATUS TO THE TASK.
;
;	INPUT:	ES:SI=	SDB
;		BX=	TCB
;	OUTPUT:	NONE
;
X_RSF		PROC	FAR
		MOV	DX,ES:[SI+SFLAG]		;GET ID
		AND	DX,SFLAG_MASK			;ISOLATE ID
		CMP	DX,X_SEMA_NUM			;VALID ID ?
		MOV	AX,SVC_FAILED+PARM_ERR		;BE READY...
		JAE	RSFERR				;NO.
		CLC					;YES:
		CALL	X_RESET_CP_SEM			;RESET SEMAPHORE
RSFERR:
		CALL	X_SFLAGS			;RETURN STATUS
		RET
X_RSF		ENDP
PAGE
;*********************************************************************
;*      X_CNTRL:  CONTROLLED PROGRAMS SERVICE                        *
;*********************************************************************
;
;	THIS PROCEDURE VALIDATES THE SDB PARAMETERS AND
;	ENTERS OR EXITS THE CONTROLLED PROGRAM.  THEN IT
;	RETURNS STATUS TO THE TASK.
;
;	INPUT:	ES:SI=	SDB
;		BX=	TCB
;	OUTPUT:	NONE
;
X_CNTRL		PROC	FAR
		MOV	CL,BYTE PTR ES:[SI+SE]	;GET START/END
		AND	CL,SE_MASK		;ISOLATE SE
		XOR	DH,DH			;CLEAR DH
		MOV	DL,ES:[SI+CID]		;GET CP ID
		CMP	DX,X_CONTROL_NUM	;VALID ID ?
		MOV	AX,SVC_FAILED+PARM_ERR	;BE READY FOR BAD NEWS...
		JAE	CNTRLERR		;NO.
		TEST	CL,CL			;YES: ENTER OR EXIT CP ?
		JNE	CNTEXIT			;-- EXIT --
		STC				;-- ENTER --
		CALL	X_SET_CP_SEM
		JMP	SHORT CNTRLERR		;RETURN STATUS.
CNTEXIT:
		STC
		CALL	X_RESET_CP_SEM
CNTRLERR:
		CALL	X_SFLAGS		;RETURN STATUS
		RET
X_CNTRL		ENDP
MTOS_CS		ENDS
		END
