PAGE	60,132
INCLUDE B:BIG.INC
;
;	SRB.ASM:	SCHEDULER AND SRB-MANAGEMENT ROUTINES.
;	F. W. PORRETTO, 12/15/82
;
;		THESE ROUTINES PROVIDE FOR THE MANIPULATION OF SRBS AND THE
;	SCHEDULING OF ALL S-STATE AND A-STATE ACTIVITY.
;
		NAME	SCHEDULER_SERVICES
;
MTOS_CS		SEGMENT PARA PUBLIC
		ASSUME	CS:MTOS_CS,DS:MTOS_DS,SS:MTOS_DS
		EXTRN	X_TASK_STATE:NEAR
		EXTRN	X_SET_READY:NEAR
		EXTRN	X_SYSOUT:NEAR
		EXTRN	X_MSG_SRB_OUT:WORD
;
		PUBLIC	X_SCHDLR
		PUBLIC	X_QUEUE_SRB
		PUBLIC	X_INIT_SRB,X_GET_SRB,X_PUT_SRB
PAGE
;************************************************
;*   X_INIT_SRB: INITIALIZE SRB FREE LIST.      *
;************************************************
;
X_INIT_SRB	PROC	NEAR
		MOV	X_SRB_AVAIL,X_SRB_NUM
		MOV	AX,OFFSET X_SRB_START
		MOV	X_SRB_FREEQ,AX		;HEAD OF FREE QUEUE
		MOV	XCSRB+2,DS			;PUT IN DS
		MOV	CX,X_SRB_NUM - 1	;SRB COUNT - 1
		XOR	DI,DI
LP:
		ADD	AX,SRB_SIZE		;NEXT BLOCK
		MOV	X_SRB_START[DI],AX	;FILL LINK
		ADD	DI,SRB_SIZE		;NEXT BLOCK
		LOOP	LP
		RET
X_INIT_SRB	ENDP
;**************************************************************
;*     X_GET_SRB:      ALLOCATE AN SRB FROM THE FREE LIST     *
;**************************************************************
;
;	THIS ROUTINE NOT ONLY ALLOCATES A FREE SRB (IF ANY), IT ALSO FILLS IT
;	IN FROM THE CURRENT REGISTER FILE, PROPAGATING THE CURRENT AX, BX, BP,
;	SI, DI, CX, DX, AND ES REGISTER VALUES INTO THE EMBRYONIC S-STATE PRO-
;	CESS AT ITS ENTRY POINT.
;
;	NOTE: UNAVAILABILITY OF A FREE SRB AT THIS POINT WILL CRASH MTOS.
;
X_GET_SRB	PROC	NEAR
		PUSH	DI		;DI.
		PUSHF			;SAVE STATE OF INTERRUPT FLAG.
		PUSH	DI		;THIS IS TO BE EXPLAINED LATER.
		CLI			;INTERRUPTS OFF.
		CMP	X_SRB_AVAIL,0		;AVAILABLE ?
		JE	NONE_LEFT		; NO ---
		MOV	DI,X_SRB_FREEQ		;YES: TAKE THE TOP ONE.
		MOV	[DI+SRB_AX],AX		;FILL IN THE REG'S
		MOV	[DI+SRB_BX],BX
		MOV	[DI+SRB_CX],CX
		MOV	[DI+SRB_DX],DX
		MOV	[DI+SRB_ES],ES
		MOV	[DI+SRB_BP],BP
		MOV	[DI+SRB_SI],SI
		POP	WORD PTR [DI+SRB_DI]	;SEE WHY THE EXTRA PUSH NOW?
		MOV	AX,X_ATCB
		MOV	[DI+SRB_TCB],AX		;CURRENT ACTIVE TCB
		MOV	AX,[DI]
		MOV	X_SRB_FREEQ,AX		;ESTABLISH NEW HEAD OF Q
		DEC	X_SRB_AVAIL		;LESS ONE SRB
		MOV	BX,DI			;PUT ADR INTO BX
		POPF
		POP	DI
		RET
NONE_LEFT:					;** NO SRB FREE: CRASH! **
		MOV	CX,32			;COUNT OF CHARACTERS
		MOV	BX,OFFSET X_MSG_SRB_OUT	;ADDRESS OF MESSAGE
		CALL	X_SYSOUT
HALT:		HLT
		JMP	SHORT HALT
X_GET_SRB	ENDP
;***********************************************************
;*     X_PUT_SRB:       RETURN SRB TO THE SRB FREE LIST    *
;***********************************************************
;
;	 THE SRB IS PLACED AT THE HEAD OF THE FREE LIST, BUT THIS IS OF NO
;	SPECIAL SIGNIFICANCE TO ANY OTHER MTOS INTERNAL OPERATION.
;
;	INPUT:	BX= ADDRESS OF FREED SRB.
;	OUTPUT:	NONE.
;
X_PUT_SRB	PROC	NEAR
		PUSH	DI
		PUSHF
		CLI
		MOV	AX,X_SRB_FREEQ		;HEAD OF Q INTO NEW BLOCK
		MOV	[BX],AX
		MOV	X_SRB_FREEQ,BX		;NEW BLOCK AS HEAD
		INC	X_SRB_AVAIL		;ADD IN NEW BLOCK TO TOTAL
		POPF
		POP	DI
		RET
X_PUT_SRB	ENDP
PAGE
;******************************************************************************
;*	ROUTINES TO QUEUE AND DEQUEUE SRBS TO SPECIFIED SCHEDULER QUEUES:     *
;******************************************************************************
;
;	INTERFACE:
;	INPUT TO X_QUEUE_SRB:		BX CARRIES SRB ADDRESS.
;	OUTPUT FROM X_DQUEUE_SRB:	BX CARRIES SRB ADDRESS OR 0.
;
X_QUEUE_SRB	PROC	NEAR
		PUSH	SI
		PUSH	DI
		PUSHF
		MOV	WORD PTR [BX],0		;CLEAR NEW SRB'S LINK WORD.
		CLI
		CMP	X_SRB_Q,0		;ANY EXIST YET?
		JNE	XQMP0			;YES.
		MOV	X_SRB_QEND,BX		;NO, DEGENERATE CASE.
		MOV	X_SRB_Q,BX		;NEW ONE IS BOTH HEAD AND TAIL.
		JMP	XQMP1
XQMP0:
		MOV	SI,X_SRB_QEND		;GET ADDRESS OF CURRENT TAIL
		MOV	[SI],BX			;LINK FORWARD TO NEW ENTRY
		MOV	X_SRB_QEND,BX		;NEW SRB BECOMES TAIL ENTRY.
XQMP1:
		POPF
		POP	DI
		POP	SI
		RET
X_QUEUE_SRB	ENDP
;
X_DQUEUE_SRB	PROC	NEAR
		PUSH	DI
		PUSHF
		MOV	BX,X_SRB_Q		;GET VALUE OF QUEUE HEAD:
		TEST	BX,BX			;ZERO?
		JE	XDQMP0			;YES.
		MOV	AX,[BX]			;NO: GET ENTRY'S FORWARD LINK
		MOV	X_SRB_Q,AX		;AND ADJUST HEAD.
		TEST	AX,AX			;QUEUE EMPTY?
		JNE	XDQMP0			;NO.
		MOV	X_SRB_QEND,AX		;YES, CLEAR TAIL WORD TOO.
XDQMP0:
		POPF
		POP	DI
		RET
X_DQUEUE_SRB	ENDP
PAGE
;************************************************
;*          S C H E D U L E R                   *
;************************************************
;
;	THE SCHEDULER MAY BE ENTERED IN ANY OF THE FOLLOWING WAYS:
;		1) DIRECT JUMP FROM THE INITIALIZATION THREAD.
;		2) DIRECT JUMP FROM X_XEL.
;		3) RETURN FROM ANY TERMINATING S-STATE PROCEDURE.
;
;		THE SCHEDULER MAKES ALL DECISIONS ABOUT WHAT ACTIVITY THE CPU
;	WILL BE TURNED TO NEXT, EXCEPT FOR INTERRUPT NESTING AND UNNESTING.
;
;		WHEN ENTERED, THE SCHEDULER:
;	1)CHECKS WHETHER THERE ARE ANY SRBS ON THE CPU'S SRB QUEUE. IF
;	  THERE ARE, THE FIRST ONE IS DEQUEUED, THE S-STATE ENVIRONMENT IS
;	  SET UP (AX, BX, BP, SI, CX, DX, DI, ES AND DS=MTOS_DS), AND THE 
;	  SPECIFIED ROUTINE IS CALLED.
;	2)IF THERE ARE NO WAITING SRBS ON THE QUEUE, THE READY QUEUE IS
;	  CHECKED:
;		2.1)IF THERE ARE ONE OR MORE READY TASKS SCHEDULABLE ON THIS
;		    PROCESSOR, THE SCHEDULER SELECTS THE ONE HIGHEST IN PRI-
;		    ORITY AND GIVES IT CPU CONTROL.
;		2.2)IF THERE ARE NO SCHEDULABLE READY TASKS, THE SCHEDULER 
;		    HALTS WITH INTERRUPTS ENABLED, AND REEXAMINES THE SRB
;		    QUEUE AFTER THE NEXT INTERRUPT.
;
X_SCHDLR	PROC	NEAR
		CLI			;INTERRUPTS OFF WHILE STATE VARIABLES
					;ARE IN FLUX: MIGHT CHANGE STATE.
		MOV	SP,X_MTOS_SP	;FLATTEN THE STACK.
		CALL	X_DQUEUE_SRB	;ARE ANY SRBS PENDING
		MOV	XCSRB,BX	;MY EXAMINATION?
		STI			;WE'RE EITHER IN S STATE OR H STATE NOW
					;SO INTERRUPTS CAN BE PERMITTED AGAIN.
		TEST	BX,BX		;WELL??
		JE	SCHA00		;NO: LOOK FOR A TASK TO SCHEDULE.
		MOV	AX,[BX+SRB_TCB]	;YES: GET TCB AT TIME OF INTERRUPT
		MOV	X_CTCB,AX	;INTO X_CTCB
		LES	SI,DWORD PTR [BX+SRB_ADR]	;GET CALL ADDRESS
		MOV	X_S_STATE,SI	;AND FILL IT INTO
		MOV	X_S_STATE+2,ES	;THE JUMP WORD.
		MOV	ES,[BX+SRB_ES]	;NOW ESTABLISH THE S STATE CONTEXT:
		MOV	DI,[BX+SRB_DI]
		MOV	SI,[BX+SRB_SI]
		MOV	BP,[BX+SRB_BP]
		MOV	DX,[BX+SRB_DX]
		MOV	CX,[BX+SRB_CX]
		MOV	AX,[BX+SRB_AX]
		MOV	BX,[BX+SRB_BX]
		CALL	DWORD PTR X_S_STATE	;AND ENTER THE S STATE ROUTINE.
		MOV	BX,XCSRB	;NOW GET THE SRB USED
		CALL	X_PUT_SRB	;AND DEALLOCATE IT.
		JMP	X_SCHDLR	;RENEW THE SCHEDULER'S INNER CYCLE.
;
;	MAJOR APPLICATION STATE QUEST:
;
SCHA00:
		MOV	XCSRB,-1	;WE MUST FAKE S STATE FOR THE
					;READY QUEUE PERUSAL, BECAUSE
					;THE QUEUES MUST BE HELD LOCKED.
		MOV	DI,OFFSET X_LREADY_Q	;ADDRESS OURS.
		MOV	BX,[DI]		;GET FIRST ENTRY'S ADDRESS.
		TEST	BX,BX		;ARE THERE ANY TASKS READY AT ALL??
		JE	SCHA20		;NO: GO TO IDLE STATE.
;
;	WE HAVE SELECTED A TASK TO ACTIVATE:
;		1)UNLOCK READY QUEUES,
;		2)SET TCB ADDRESS INTO X_ATCB,
;		3)INSERT TCB_SS, TCB_SP INTO SS, SP,
;		4)RECORD TASK'S RESUMPTION IP/CS,
;		5)UNSAVE TASK'S REGISTER FILE, ENTERING TASK!
;
SCHA16:
		CLI			;SINCE A TASK IS GOING TO BE SCHEDULED
					;WE MUST DISABLE INTERRUPTS WHILE THE
					;STATE VARIABLES ARE CHANGING.
		MOV	AX,[BX]		;REMOVE SELECTED TASK FROM ITS
		MOV	[DI],AX		;READY QUEUE.
		MOV	BYTE PTR [BX+TCB_STATE],ACTIVE	;TASK NOW ACTIVE.
		MOV	XCSRB,0		;CLEAR S-STATE-SRB VARIABLE AND
		MOV	X_ATCB,BX	;ENTER A STATE.
		LES	DI,DWORD PTR [BX+TCB_SP]	;GET TASK STACK POINTER
		LES	DI,DWORD PTR ES:[DI+STK_IP]	;AND RESUMPTION ADDRESS
		MOV	WORD PTR X_TASK_IPCS,DI		;RESUMPTION ADDRESS TO
		MOV	WORD PTR X_TASK_IPCS+2,ES	;TRACE CONTROL WORD.
		MOV	SS,[BX+TCB_SS]	;IMPLEMENT THE TASK'S
		MOV	SP,[BX+TCB_SP]	;STACK POINTER.
;
;	THE FOLLOWING FOUR INSTRUCTIONS PROVIDE FOR THE USE OF THE 8087 NUMERIC
;	DATA PROCESSOR AS A COPROCESSOR TO THE 8086. IF THE 8087 IS NOT BEING
;	EMPLOYED, THEY SHOULD BE LEFT COMMENTED OUT. ELSE, REMOVE THE LEADING
;	SEMICOLONS. SEE ALSO THE METAROUTINE X_XEL (STATES.ASM).
;
;		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_8087	;USING NDP?
;		JE	SCHA19		;NO.
;		MOV	BP,[BX+TCB_8087]	;YES: GET SAVE AREA ADDRESS,
;		FRSTOR	DS:[BP]		;AND RESTORE 8087 REGISTERS.
;
SCHA19:
		POP	ES		;UNSAVE REGISTERS...
		POP	DI
		POP	DX
		POP	CX
		POP	DS
		POP	SI
		POP	BP
		POP	BX
		POP	AX
		IRET			;...AND ENTER THE TASK!
;
;	HERE'S WHERE IDLE STATE (H STATE) IS ENFORCED:
;
;	THE CPU HAS NO TASKS TO RUN AND NO SRBS TO DISPATCH. THEREFORE, IT MUST
;	HALT WITH INTERRUPTS ENABLED AND LOOP THROUGH THE SCHEDULER AGAIN AFTER
;	THE NEXT INTERRUPT.
;
SCHA20:
		MOV	XCSRB,0		;IN H STATE,
					;X_INTLEVEL=XCSRB=X_ATCB= 0!
		HLT			;TWIDDLE THUMBS,
		JMP	X_SCHDLR	;AND TRY AGAIN AFTER NEXT INTERRUPT.
X_SCHDLR	ENDP
MTOS_CS		ENDS
		END
