PAGE	60,132
INCLUDE B:BIG.INC
;
;	START.ASM:	TASK START/TERMINATE SERVICES.
;	F. W. PORRETTO, 12/15/82
;
;		HERE ARE COLLECTED ALL THE ROUTINES HAVING TO DO WITH THE 
;	FOUR MTOS OPERATIONS WHICH START AND TERMINATE TASKS, PLUS THE SMALL
;	ROUTINE WHICH PROVIDES THE QCOPY SERVICE.
;
		NAME	START_END_TASK_SERVICES
;
MTOS_CS		SEGMENT PARA PUBLIC
		ASSUME	CS:MTOS_CS,DS:MTOS_DS,SS:MTOS_DS
		EXTRN	X_GET_TCB_ADR	:NEAR		;TASK ID TO TCB ADR
		EXTRN	X_GET_SMR	:NEAR		;GET MEMORY
		EXTRN	X_PUT_SMR	:NEAR		;FREE MEMORY
		EXTRN	X_SFLAGS	:NEAR		;SET SVC CONDITION FLAGS
		EXTRN	X_SET_READY	:NEAR		;TASK TO READY QUEUE
		EXTRN	X_INIT_TASK	:NEAR		;SETUP TASKS STACK
		EXTRN	X_TASK_STATE	:NEAR		;MOVE TASK TO STATE
		EXTRN	X_QUEUE_TMB	:NEAR		;TIMER BLOCK QUEUED
		EXTRN	X_DQUEUE_TMB	:NEAR		;TIME OUT BLOCK DEQUEUED
		EXTRN	X_QUEUE		:NEAR		;GENERAL QUEUE ROUTINE
		EXTRN	X_DQUEUE	:NEAR		;GENERAL DE-QUEUE
		EXTRN	X_GET_INTERVAL	:NEAR		;COMPUTE TIME INTERVAL
		EXTRN	X_GET_INHPRI	:NEAR		;GET INHERENT PRIORITY
		EXTRN	X_OVIN		:NEAR		;OVERLAY-IN REGISTRY
		EXTRN	X_OVOUT		:NEAR		;OVERLAY-OUT REGISTRY.
;
		PUBLIC	X_STRT,X_QSTRT
		PUBLIC	X_ENDT,X_ENDR
		PUBLIC	X_QCOPY
		PUBLIC	X_START_TASK
		PUBLIC	X_NEXT_START
;
;	LOCAL EQUATES
;
;	   STRT AND QSTRT SDB EQUATES
;
STRT_TYPE	EQU	00H				;SVC TYPE     (BYTE)
STRT_TASK	EQU	02H				;TASK ID      (WORD)
STRT_CORD	EQU	01H				;COORDINATION (BYTE,60H)
STRT_PRI	EQU	01H				;PRIORITY OPTION (BYTE,03H)
STRT_PRIV	EQU	04H				;PRIORITY      (BYTE)
STRT_P1		EQU	05H				;PARAMETER #1  
STRT_P2		EQU	07H				;PARAMETER #2  
QSTART		EQU	02H				;SVC TYPE FOR QSTRT
CORD		EQU	060H				;COORDINATION MASK
PRI		EQU	03H				;PRIORITY MASK
WTSTART		EQU	1	;WAIT-FOR-TARGET-START
WTEND		EQU	2	;WAIT-FOR-TARGET-TERMINATION
;
PAGE
;*****************************************************************************
;*	X_NEXT_START:	IF TASK HAS BECOME DORMANT, PROCESS ITS NEXT         *
;*			RCB, IF ANY.                                         *
;*****************************************************************************
;
;	THIS ROUTINE ENFORCES THE RULE OF ALL CONTINGENT PROCESSING AS IT
;	APPLIES TO TASK TERMINATION AND SUBSEQUENT STARTUP.
;	THIS ROUTINE IS CALLED WHENEVER:
;		-A TASK TERMINATES,
;		-A TASK ACQUIRES A NEW RCB WHILE DORMANT,
;		-A TASK'S PENDING LOAD OPERATION COMPLETES, SO THAT THE 
;		 OVERLAY MANAGER CAN GIVE IT BACK THE RCB USED TO CONTROL
;		 THE LOAD.
;
;	INPUT:	BX=	TCB ADDRESS.
;	OUTPUT:	NONE.
;
X_NEXT_START	PROC	NEAR
		CMP	WORD PTR [BX+TCB_QRST],0	;ANY RESTARTS WAITING?
		JNE	NS0			;YES.
		RET				;NO.
NS0:
		PUSH	ES
		PUSH	SI
		PUSH	DI
		PUSH	DX
		PUSH	CX
		MOV	SI,[BX+TCB_QRST]	;GET THE RESTART QUEUE HEAD:
		MOV	AX,[SI]			;UNLINK THE HEAD RCB FROM
		MOV	[BX+TCB_QRST],AX	;THE RESTART QUEUE.
		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_LOAD	;LOADABLE?
		JE	NS4			;NO.
		MOV	BP,SI			;YES:
		CALL	X_OVIN			;ASK OVLY. MGR. FOR PERMISSION
		JNC	NS4			;GOT IT!
		MOV	AX,BLOCK+STA_LOAD	;NO: BLOCK THE TARGET TASK
		CALL	X_TASK_STATE		;PENDING LOAD COMPLETION.
		JMP	SHORT NS1
NS4:
		MOV	[BX+TCB_CRCB],SI	;THIS RCB IS NOW CURRENT.
		CALL	X_INIT_TASK		;INITIALIZE TARGET'S STACK.
		LES	DI,DWORD PTR [BX+TCB_SP]	;NOW GO TO STACK TO
		MOV	AX,[SI+RCB_AX]		;IMPLANT THE TWO STARTUP
		MOV	ES:[DI+STK_AX],AX	;PARAMETERS CARRIED IN THE
		MOV	AX,[SI+RCB_BX]		;RCB, IN THE TARGET'S AX AND
		MOV	ES:[DI+STK_BX],AX	;BX REGISTERS.
		CMP	BYTE PTR [SI+RCB_CORD],1	;WAIT FOR STARTUP?
		JNE	NS2			;NO.
		MOV	DI,[SI+RCB_WTCB]	;YES: GET REQUESTOR'S TCB
		XCHG	BX,DI			;AND MAKE THE REQUESTOR
		CALL	X_SET_READY		;READY AGAIN.
		XCHG	BX,DI
NS2:
		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_OVPRI	;PRIORITY-OVER-
						;RIDE TMB ASSIGNED HERE?
		JE	NS5			;NONE.
		LES	DI,DWORD PTR [BX+TCB_TCD]	;GO TO THE TCD FOR THE
		MOV	AX,ES:[DI+TCD_TIME]	;OVERRIDE TIMEOUT, WHICH IS A
		CALL	X_GET_INTERVAL		;STANDARD TIME SPECIFIER.
		MOV	AX,ES:[DI+TCD_OVPRI]	;GET  PRIORITY INCREMENT AND
						;BOUNDARY FROM TCD.
		MOV	DI,[BX+TCB_TMB]		;GET TMB ADDRESS FROM TCB
		MOV	[DI+TMB_OVPRI],AX	;INSTALL INCREMENT, BOUNDARY.
		MOV	[DI+TMB_OVTIME],CX	;INSTALL REPEATING OVERRIDE
		MOV	[DI+TMB_OVTIME+2],DX	;INTERVAL IN MILLISECONDS.
		MOV	[DI+TMB_TIME],CX	;AND INSERT THE COUNTDOWN
		MOV	[DI+TMB_TIME+2],DX	;INTO IT.
		MOV	BYTE PTR [DI+TMB_TYPE],5	;TYPE=PRIORITY OVERRIDE
		MOV	[DI+TMB_TCB],BX		;INSERT TCB ADDRESS, OF COURSE
		XCHG	BX,DI
		CALL	X_QUEUE_TMB		;START THE TMB
		XCHG	BX,DI
NS5:
		MOV	AL,[SI+RCB_PRI]		;FINALLY, SET THE TASK'S
		MOV	[BX+TCB_PRI],AL		;PRIORITY FROM THE RCB.
		CALL	X_SET_READY		;MAKE TARGET TASK READY AGAIN.
NS1:
		POP	CX
		POP	DX
		POP	DI
		POP	SI
		POP	ES
		RET
X_NEXT_START	ENDP
PAGE
;*****************************************************************************
;*	X_START_TASK:	CONSTRUCT AND ENQUEUE AN RCB FROM THE PARAMETERS     *
;*			SPECIFIED IN THE INPUT REGISTERS.                    *
;*****************************************************************************
;
;	THIS IS THE "INNER" START-TASK FACILITY; IT REQUIRES NO SDB AND THERE-
;	FORE IS SUITABLE FOR USE WITHIN THE SYSTEM, OR FROM A DEVICE DRIVER.
;	IT PERFORMS THE BASIC PROCESSING NECESSARY FOR BOTH THE STRT AND THE
;	QSTRT SVCS.
;
;	INPUT:
;		AX,BX:	STARTUP PARAMETERS FOR TARGET'S AX AND BX REGISTERS,
;		CH:	COORDINATION MODE: 0=NONE, 1=WAIT FOR START,
;					   2=WAIT FOR END.
;		CL:	TARGET'S STARTUP PRIORITY.
;		SI:	REQUESTING TASK'S TCB ADDRESS
;		DI:	TARGET TASK'S TCB ADDRESS
;		BP:	ADDRESS OF SMR BLOCK TO BE USED AS RCB IF NONZERO,
;			OR ZERO TO TELL X_START_TASK TO ALLOCATE FROM SMR.
;	OUTPUT:
;		NONE.
;
X_START_TASK	PROC	NEAR
		PUSH	AX
		PUSH	BX
		TEST	BP,BP			;SMR BLOCK ADDRESS INPUT?
		JNE	XST02			;YES, DON'T ALLOCATE ONE.
		CALL	X_GET_SMR		;GET A BLOCK FOR THE RCB
		MOV	BP,BX			;ADDRESS TO BP FOR CONSISTENCY.
XST02:
		MOV	BX,BP			;GET ADDRESS OF RCB TO BX.
		POP	WORD PTR [BX+RCB_BX]	;FILL IT IN WITH THE
		POP	WORD PTR [BX+RCB_AX]	;STARTUP PARAMETERS.
		MOV	[BX+RCB_PRI],CX		;INSERT COORDINATION AND
						;PRIORITY.
		MOV	[BX+RCB_WTCB],SI	;INSERT REQUESTOR'S TCB.
		MOV	[BX+RCB_TCB],DI		;INSERT TARGET TCB.
		PUSH	DI			;SAVE TARGET TCB ADDRESS
		ADD	DI,TCB_QRST		;GET HIS RESTART QUEUE
		CALL	X_QUEUE			;AND ATTACH NEW RCB.
		POP	BX			;RETRIEVE TCB ADDRESS
;
;	WE MUST DETERMINE THE COORDINATION CONDITIONS FOR THE REQUESTOR:
;	IF HE ASKED FOR SOME SPECIFIC (NON-ZERO) COORDINATION MODE, HE
;	GETS IT. ELSE, IF HE MUST WAIT FOR A LOAD OPERATION, HE IS TRANS-
;	PARENTLY BLOCKED; IF NOT, HE IS PERMITTED TO RUN.
;
		CMP	BYTE PTR [BX+TCB_STATE],BLOCK+STA_LOAD	;IS TARGET
						;LOADING JUST NOW?
		JE	XST04			;YES.
		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_LOAD ;NO: LOADABLE?
		JE	XST05			;NO, USE NATIVE COORDINATION
		CMP	BYTE PTR [BX+TCB_STATE],DORMANT	;YES: TARGET DORMANT?
		JNE	XST05			;NO, MUST BE IN MEMORY.
XST04:
		TEST	CH,CH			;IF COORD= 0,
		JNE	XST05			;OVERRIDE IT WITH WTSTART.
		MOV	CH,WTSTART		;LIKE THAT.
XST05:
		TEST	CH,CH			;COORD= 0?
		JE	XST00			;YES.
		MOV	AX,BLOCK+STA_STRT	;NO:
		DEC	CH			;COORD= WAIT FOR START?
		JE	XST01			;YES.
		MOV	AX,BLOCK+STA_ENDT	;NO, WAIT FOR END.
XST01:
		XCHG	BX,SI			;BLOCK THE REQUESTOR
		CALL	X_TASK_STATE		;APPROPRIATELY.
		XCHG	BX,SI
;
;	THE TARGET TASK MAY BE DORMANT, IN WHICH CASE THE RCB JUST CREATED
;	AND ADDED TO ITS QUEUE SHOLUD BE PROCESSED AT ONCE.
;
XST00:
		CMP	BYTE PTR [BX+TCB_STATE],DORMANT	;TARGET TASK DORMANT?
		JNE	XST03			;NO.
		CALL	X_NEXT_START		;YES: PROCESS NEXT RCB IF
						;THERE IS ONE, WHICH WE KNOW
						;THERE IS!
XST03:
		RET
X_START_TASK	ENDP
PAGE
;********************************************************************
;*	X_ENDR:    TERMINATE TASK WITH TIMED RESTART                *
;********************************************************************
;
;	THE REQUESTING TASK IS MOVED TO THE SPECIAL "BLOCKED PENDING
;	TIMED RESTART" STATE, AND A RESTART TIMER IS BUILT FOR IT.
;
;	INPUT:	ES:SI=	SDB ADDRESS.
;		BX=	REQUESTOR'S TCB.
;	OUTPUT:	NONE
;
;		THIS ROUTINE PUTS THE CALLER INTO THE SPECIAL TASK STATE
;	"BLOCKED AWAITING RESTART EXPIRATION." IF SOME OTHER TASK IS AWAIT-
;	ING THIS ONE'S TERMINATION, IT IS UNBLOCKED. A TMB OF TYPE 4 IS 
;	GENERATED AND ACTIVATED FROM THE SDB PARAMETERS, AT THE EXPIRATION
;	OF WHICH THE TASK WILL BE PUT BACK INTO READY STATE AS IT HAD BEEN
;	WHEN LAST STARTED.
;
;	LOCAL EQUATES
;
ENDR_RSB	EQU	01			;TYPE OF BASE (MSK = 80H)
ENDR_TIME	EQU	01			;PARAMETERS FOR TIME INTERVAL
TUNITS_MASK	EQU	01CH			;MASK FOR TIME-UNITS.
;
X_ENDR		PROC	FAR
		MOV	AX,BLOCK+STA_ENDR	;SET STATE TO BLOCKED
						;FOR TIMED RESTART.
		CALL	X_TASK_STATE
;
;	IF A TASK IS AWAITING THIS ONE'S TERMINATION, IT MUST BE RETURNED
;	TO READY STATE. HOWEVER, ANY TASK NOT IN DORMANT STATE IS REQUIRED
;	TO HAVE A CURRENT RCB, SO THE RCB IS KEPT RATHER THAN BEING DEALLO-
;	CATED. DEALLOCATION WILL ONLY TAKE PLACE AFTER AN ENDT.
;
		PUSH	BX
		PUSH	SI			;SAVE THE INPUT
		PUSH	ES			;WHILE WE DEFLECT TO THAT OTHER
		MOV	SI,[BX+TCB_CRCB]	;CONCERN: GET RCB ADDRESS.
		CMP	BYTE PTR [SI+RCB_CORD],WTEND	;WAIT FOR END?
		JNE	ENDR21			;NO, PROCEED.
		MOV	BYTE PTR [SI+RCB_CORD],0	;YES: CLEAR TRIGGER
		LES	DI,DWORD PTR [BX+TCB_SP] ;GET TERMINATING TASK'S STACK
		MOV	AX,ES:[DI+STK_AX]	;GET HIS LAST AX.
		MOV	DI,[SI+RCB_WTCB]	;GET WAITER'S TCB,
		LES	DI,DWORD PTR [DI+TCB_SP]	;GO TO HIS STACK AND
		MOV	ES:[DI+STK_AX],AX	;DELIVER AX.
		MOV	BX,[SI+RCB_WTCB]	;FINALLY, RETURN REQUESTOR TO
		CALL	X_SET_READY		;READY STATE.
ENDR21:
		POP	ES
		POP	SI
		POP	BX
ENDR30:
		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_OVPRI	;PRIORITY OVER-
						;RIDE TMB ASSIGNED HERE?
		JE	ENDR20			;-- NO --
		PUSH	BX
		MOV	BX,[BX+TCB_TMB]		;YES: REMOVE IT FROM TIME Q
						;AND SAVE IT FOR USE LATER.
		CALL	X_DQUEUE_TMB
		POP	BX
ENDR20:
		MOV	AX,ES:[SI+ENDR_TIME]	;MAKE COUNTDOWN FROM
		AND	AL,TUNITS_MASK		;THE SDB-SPECIFIED
		CALL	X_GET_INTERVAL		;TIME PARAMETERS.
		MOV	AL,ES:[SI+ENDR_RSB]	;OPTION OF CURRENT OR LAST TIME
		PUSH	AX
		CALL	X_INIT_TASK		;PREPARE THE TASK FOR RESTART
		LES	DI,DWORD PTR [BX+TCB_SP]	;GO TO TASK STACK TO
		MOV	SI,[BX+TCB_CRCB]	;REPLACE THE "RUNTIME" PARAME-
		MOV	AX,[SI+RCB_AX]		;TERS SUBMITTED BY REQUESTOR
		MOV	ES:[DI+STK_AX],AX	;TASK IN HIS STRT OR QSTRT
		MOV	AX,[SI+RCB_BX]		;REQUEST, WHICH ARE SAVED IN
		MOV	ES:[DI+STK_BX],AX	;THE "CURRENT RCB."
		POP	AX			;RESTORE COORDINATION SELECT
		TEST	AL,080H			;BASE ON LAST START TIME?
		JNE	ENDR01			;NO, CURRENT TIME.
		XOR	AX,AX			;YES:
		ADD	DX,[BX+TCB_SCHDL+4]	;ADD LAST START TIME TO
		ADC	CX,[BX+TCB_SCHDL+2]	;THE INTERVAL JUST CONVERTED
		ADC	AX,[BX+TCB_SCHDL]	;TO A MILLISECOND COUNT.
		CLI				;;; WE NEED A MOMENT'S PEACE
		SUB	DX,X_ABS_TIME+4		;;; ADJUST THE COUNTDOWN
		SBB	CX,X_ABS_TIME+2		;;; BY THE RUN TIME OF THE
		SBB	AX,X_ABS_TIME		;;; TASK TO-DATE.
		STI				;;; TIME MAY BE COUNTED AGAIN.
		JE	ENDR01			;IF AX <> 0, TROUBLE.
		XOR	DX,DX			;WHICH WE FIX BY SETTING THE
		XOR	CX,CX			;COUNTDOWN TO 0.
ENDR01:
		CMP	WORD PTR [BX+TCB_TMB],0	;TMB ALREADY AVAILABLE
		JNE	ENDR75			;-- YES --
		XCHG	BX,SI			;NO: TCB ADDR TO SI.
		CALL	X_GET_SMR		;GET TMB FROM SMR.
		MOV	[SI+TCB_TMB],BX		;PUT TMB INTO TCB (=SI) 
		XCHG	BX,SI			;TCB ADDR BACK TO BX.
ENDR75:
		MOV	SI,[BX+TCB_TMB]			;SI = TMB ADDRESS
		MOV	[SI+TMB_TCB],BX			;PUT IN TCB
		MOV	BYTE PTR [SI+TMB_TYPE],4	;SET TYPE OF TMB TO
		MOV	[SI+TMB_TIME],CX		;MOVE IN INTERVAL
		MOV	[SI+TMB_TIME+2],DX
		MOV	BX,SI
		CALL	X_QUEUE_TMB			;START IT GOING
		RET
X_ENDR		ENDP
PAGE
;***************************************************************************
;*	X_ENDT:		TERMINATE CALLING TASK UNCONDITIONALLY.            *
;***************************************************************************
;
;	THIS ROUTINE HANDLES SVC 03, RETURNING THE CALLER TO DORMANT STATE
;	AND PROCESSING HIS NEXT RCB, IF ANY.
;
;	INPUT:	ES:SI= SDB ADDRESS (SUCH AS IT IS),
;		BX=	CALLER'S TCB ADDRESS.
;
X_ENDT		PROC	FAR
		MOV	AX,DORMANT
		CALL	X_TASK_STATE		;MOVE CALLER TO DORMANT STATE.
		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_OVPRI	;PRIORITY-OVER-
						;RIDE TMB ASSIGNED HERE?
		JE	XET0			;NO.
		PUSH	BX			;YES:
		MOV	BX,[BX+TCB_TMB]		;GET ADDRESS OF TMB AND
		CALL	X_DQUEUE_TMB		;MAKE SURE IT'S OFF TMB QUEUE.
		POP	BX
XET0:
		PUSH	BX			;SAVE TCB ADDRESS
		MOV	SI,[BX+TCB_CRCB]	;GET CURRENT RCB ADDRESS
		MOV	AL,[SI+RCB_CORD]	;GET COORDINATION BYTE.
		CMP	AL,WTEND			;WAIT FOR TERMINATION?
		JNE	XET2				;NO.
		LES	DI,DWORD PTR [BX+TCB_SP]	;YES: GET STACK PTR.
		MOV	AX,ES:[DI+STK_AX]		;GET LAST AX VALUE.
		MOV	DI,[SI+RCB_WTCB]		;GET REQUESTOR'S TCB
		LES	DI,DWORD PTR [DI+TCB_SP]	;GET HIS STACK PTR.
		MOV	ES:[DI+STK_AX],AX		;INSTALL NEW AX VALUE.
		MOV	BX,[SI+RCB_WTCB]		;SEND THE REQUESTOR
		CALL	X_SET_READY			;BACK TO READY STATE.
XET2:
		MOV	BX,SI			;DEALLOCATE THE SUPERANNUATED
		CALL	X_PUT_SMR		;RCB TO THE SMR.
		POP	BX			;REFRESH BX WITH TCB ADDRESS
		CALL	X_NEXT_START		;PROCESS NEXT RCB, IF ANY.
		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_LOAD	;LOADABLE?
		JE	XET1			;NO.
		CMP	BYTE PTR [BX+TCB_STATE],DORMANT	;YES: DORMANT NOW??
		JNE	XET1			;NO.
		CALL	X_OVOUT			;YES! RELEASE OVERLAY REGION.
XET1:
		RET
X_ENDT		ENDP
PAGE
;******************************************************************************
;*	X_STRT:		SVC ROUTINE TO START TASK IF DORMANT.                 *
;******************************************************************************
;
;	THIS ROUTINE ACCEPTS SVC 01 AND INVOKES START-TASK PROCESSING IF
;	THE DESIGNATED TARGET TASK IS CURRENTLY DORMANT.
;
;	INPUT:	ES:SI= SDB ADDRESS, BX= REQUESTOR'S TCB ADDRESS.
;
X_STRT		PROC	FAR
		PUSH	BX			;SAVE REQUESTOR'S TCB ADDRESS.
		MOV	AX,ES:[SI+STRT_TASK]	;GET TARGET'S TASK ID
		CALL	X_GET_TCB_ADR		;VALIDATE AND MAKE TCB ADDR.
		JC	ST00			;INVALID TASK ID.
		CMP	BYTE PTR [BX+TCB_STATE],DORMANT	;TARGET DORMANT?
		JE	ST01			;YES, PROCEED.
		POP	BX			;RETRIEVE REQUESTOR'S TCB.
		MOV	AX,SVC_FAILED+FLAG_CF	;TELL REQUESTOR THAT HIS SVC
ST02:
		CALL	X_SFLAGS		;FAILED.
		RET
ST00:
		POP	BX			;GET REQUESTOR'S TCB ADDRESS
		MOV	AX,SVC_FAILED+PARM_ERR	;INVALID TASK NUMBER
		JMP	SHORT ST02		;CAUSED FAILURE.
ST01:
		POP	DI		;RETRIEVE REQUESTOR'S TCB TO DI
		MOV	BX,DI		;REQUESTOR INTO BX FOR CALL TO...
		CALL	FAR PTR X_QSTRT	;INVOKE QSTRT PROCESSING
		RET
X_STRT		ENDP
PAGE
;******************************************************************************
;*	X_QSTRT:	CREATE AN RCB FOR THE SPECIFIED TASK AND START IT     *
;*			IF IT IS DORMANT.                                     *
;******************************************************************************
;
;	THIS ROUTINE HANDLES SVC 02, BUILDING AN RCB (RESTART CONTROL BLOCK)
;	FOR THE SPECIFIED TASK AND ATTACHING THE RCB TO THE TCB'S RESTART
;	QUEUE. IF THE TASK IS CURRENTLY DORMANT, THE FRESH RCB IS USED TO
;	START THE TASK AT ONCE.
;
;	INPUT:	ES:SI= SDB ADDRESS, BX= TCB ADDRESS OF REQUESTOR.
;
X_QSTRT		PROC	FAR
		MOV	AX,ES:[SI+STRT_TASK]	;GET TARGET TASK ID.
		MOV	BP,BX			;SAVE REQUESTOR'S TCB ADDR.
		CALL	X_GET_TCB_ADR		;VALIDATE AND MAKE TCB ADDR:
		JNC	QST01			;OKAY.
QST00:
		MOV	AX,SVC_FAILED+PARM_ERR	;NOT OKAY:
		CALL	X_SFLAGS		;TELL CALLER.
		RET
QST01:
		CMP	BYTE PTR ES:[SI+STRT_CORD],CORD	;VALID COORDINATION?
		JAE	QST00			;NO.
		MOV	AL,ES:[SI+STRT_CORD]	;YES, GET BYTE AGAIN
		PUSH	AX			;SAVE IT.
		AND	AL,PRI			;EXTRACT PRIORITY FIELD
		XOR	AH,AH
		SHL	AX,1			;DOUBLE IT,
		MOV	DI,AX			;AND INDEX INTO THE TABLE
		CALL	WORD PTR CS:QST_PRI[DI]	;OF PRIORITY GENERATORS.
		POP	AX			;GET COORD. BYTE BACK.
		AND	AL,CORD			;MASK OUT ALL BUT COORD.
		SHR	AL,1			;SHIFT THE COORD FIELD
		SHR	AL,1			;TEDIOUSLY DOWN
		SHR	AL,1			;FROM BITS 5 AND 6
		SHR	AL,1			;TO BITS 0 AND 1
		SHR	AL,1			;WHEW!
		CMP	AL,3			;DISALLOWED VALUE?
		JE	QST00			;YES.
		MOV	CH,AL			;PUT IT INTO CH,
		MOV	DI,BX			;TARGET TCB TO DI,
		MOV	AX,ES:[SI+STRT_P1]	;STARTUP PARAMETERS TO AX AND
		MOV	BX,ES:[SI+STRT_P2]	;BX, OF COURSE,
		MOV	SI,BP			;REQUESTOR'S TCB TO SI,
		XOR	BP,BP			;X_START_TASK GETS SMR BLK
		CALL	X_START_TASK		;GO!
		RET
X_QSTRT		ENDP
;
QST_PRI		DW	PRI_0,PRI_1,PRI_2,PRI_3
;
;	THE FOLLOWING FOUR PRIORITY GENERATORS EXPECT:
;		BP= REQUESTOR'S TCB,
;		BX= TARGET'S TCB.
;
PRI_0		PROC	NEAR		;GREATER OF TARGET'S INHERENT OR 
		CALL	X_GET_INHPRI		;GET TARGET'S INHERENT PRI.
		MOV	CL,AL			;TO CL, PLEASE.
		CMP	CL,DS:[BP+TCB_PRI]	;COMPARE TO CALLER'S PRIORITY
		JAE	PRIRT0		;TARGET'S PRIORITY TO BE USED.
		MOV	CL,DS:[BP+TCB_PRI]	;CALLER'S PRIORITY IS HIGHER.
PRIRT0:
		RET
PRI_0		ENDP
;
PRI_1		PROC	NEAR
		MOV	CL,ES:[SI+STRT_PRIV]	;PRIORITY COMES STRAIGHT FROM
		RET				;THE SDB.
PRI_1		ENDP
;
PRI_2		PROC	NEAR
		MOV	CL,DS:[BP+TCB_PRI]	;PRIORITY COMES FROM CALLER'S
		RET				;CURRENT PRIORITY.
PRI_2		ENDP
;
PRI_3		PROC	NEAR
		CALL	X_GET_INHPRI		;USE TARGET'S INHERENT PRI.
		MOV	CL,AL
		RET
PRI_3		ENDP
PAGE
;********************************************************************
;*	SVC HANDLER FOR SPECIAL PARAMETER-COPYING SERVICE:          *
;********************************************************************
;
;	THIS FUNCTION COPIES THE DATA FROM THE CURRENT RCB TO
;	THE ADDRESS SPECIFIED BY THE CALLER'S SDB. THE COPY IS ALWAYS 32
;	BYTES LONG, REGARDLESS OF THE USE OF THE PARAMETERS.
;
;	INPUT:	ES:SI=	SDB ADDRESS, BX= REQUESTOR'S TCB.
;
X_QCOPY		PROC	FAR
		LES	DI,DWORD PTR ES:[SI+1]	;GET BUFFER ADDRESS
		MOV	SI,[BX+TCB_CRCB]	;GET ADDRESS OF PARAMETERS
		CLD
		MOV	CX,RCB_SIZE/2		;LENGTH OF MOVE IN WORDS.
	REP	MOVSW				;DO THE MOVE.
		RET
X_QCOPY		ENDP
;
MTOS_CS		ENDS
		END
