PAGE	60,132
INCLUDE B:BIG.INC
;
;	STATES.ASM:	SYSTEM METAROUTINES (STATE TRANSITION ROUTINES)
;	F. W. PORRETTO, 12/15/82
;
;		HERE ARE COLLECTED THE VARIOUS FOUNDATION ROUTINES FOR ALL
;	MTOS STATE TRANSITION PROCESSING, PLUS THE INITIALIZATION OF THE
;	8259A PROGRAMMABLE INTERRUPT CONTROLLER.
;
		NAME	STATE_SERVICE
MTOS_CS		SEGMENT PARA PUBLIC
;
		ASSUME	CS:MTOS_CS,DS:MTOS_DS,SS:MTOS_DS
;
		PUBLIC	X_ELIPSE,X_XEL,X_SYSTATE
		PUBLIC	XSYSTEM,XEXSYS

		EXTRN	X_INIT_PIC:NEAR		;INITIALIZE THE INTRPT CNTRLER
		EXTRN	X_ISSUE_EOI:NEAR	;ISSUE EOI TO INT CONTROLLER
		EXTRN	X_GET_SRB:NEAR		;GET A FREE SYSTEM REQ BLOCK
		EXTRN	X_QUEUE_SRB:NEAR	;ADD TO THE END OF SRB QUEUE
		EXTRN	X_SET_READY:NEAR	;MAKE TASK READY
		EXTRN	X_SCHDLR:NEAR		;SCHEDULER ENTRY POINT
PAGE
;	******************************************
;	*  X_ELIPSE: GO FROM DI TO I STATE.      *
;	******************************************
;
;	X_ELIPSE IS THE ROUTINE WHICH FIRST INFORMS MTOS THAT AN INTERRUPT
;	EVENT IS IN PROGRESS. PRIOR TO THE INVOCATION OF X_ELIPSE, THE INTER-
;	RUPT'S PROCESSING HAS IMPLICITLY USED THE CURRENT STACK, AND HAS 
;	OCCURRED IN THE UNBOUNDED CONTEXT OF WHATEVER WAS GOING ON AT THE TIME
;	OF THE INTERRUPT. AFTER CALLING X_ELIPSE,
;
;	1)THE INTERRUPT NESTING COUNTER HAS BEEN INCREMENTED,
;	2)THE STACK IN USE IS THE MTOS STACK,
;	3)THE DS REGISTER HAS BEEN SET TO MTOS_DS,
;	4)IF THE SYSTEM WAS IN A-STATE WHEN THE INTERRUPT OCCURRED, THE TASK'S
;	  REGISTER FILE WILL HAVE BEEN SAVED ON ITS STACK AND ITS STACK POINTER
;	  AND SEGMENT WILL HAVE BEEN SAVED IN ITS TCB.
;
;	X_ELIPSE MUST BE INVOKED WITH A FAR CALL.
;
X_ELIPSE	PROC	FAR
		PUSH	BP
		MOV	BP,SP		;MARK CURRENT STACK FRAME
		PUSH	SI
		PUSH	DS
		PUSH	CX
		PUSH	DX
		PUSH	DI		;EXCEPT FOR AX AND BX,
		PUSH	ES		;CURRENT CONTEXT NOW SAVED.
		MOV	DX,MTOS_DS
		MOV	DS,DX		;MTOS_DS NOW IN FORCE AS DS.
		MOV	CX,[BP+4]	;EXTRACT RETURN ADDRESS FROM STACK
		MOV	DI,[BP+2]	;USED TO RETURN TO THE CALLER
		MOV	[BP+4],AX	;AX AND BX MUST BE SAVED WHERE THAT
		MOV	[BP+2],BX	;RETURN ADDRESS WAS PUT.
;
;	WE WILL INSTALL THE SYSTEM STACK IF:
;	1)X_INTLEVEL IS CURRENTLY 0 (NO INTERRUPT NESTING YET), AND
;	2)X_ATCB IS NOT CURRENTLY 0 (A TASK IS ACTIVE).
;
		CMP	X_INTLEVEL,0	;INTERRUPTS NESTED YET?
		JNE	X_EL_RET	;YES, RETAIN CURRENT STACK.
		MOV	BX,X_ATCB	;NO: GET CURRENT TASK'S TCB
		TEST	BX,BX		;IS SYSTEM IN A-STATE?
		JE	X_EL_RET	;NO, MTOS STACK ALREADY INSTALLED.
		MOV	[BX+TCB_SP],SP	;YES: SAVE THE CURRENT STACK POINTER
		MOV	[BX+TCB_SS],SS	;IN THE TASK'S TCB.
		MOV	SS,DX		;NOW INSTALL THE MTOS STACK.
		MOV	SP,X_MTOS_SP
X_EL_RET:
		PUSH	CX		;PUSH RETURN CS,
		PUSH	DI		;THEN RETURN IP.
		INC	X_INTLEVEL	;NEST INTERRUPTS ONE LEVEL DEEPER.
		STI			;REENABLE THEM.
		RET			;RETURN TO CALLING PROGRAM.
X_ELIPSE	ENDP
PAGE
;	*******************************************
;	*    X_SYSTATE: GO FROM I TO S STATE      *
;	*******************************************
;
;	X_SYSTATE CREATES A SYSTEM STATE REQUEST. THIS INVOLVES THE ALLOCATION
;	OF AN SRB, FILLING IT IN FROM THE REGISTERS, STATE VARIABLES AND STACK,
;	AND QUEUEING IT TO THE CORRECT SRB QUEUE.
;	THE INTERRUPT STATE IS THEN EXITED VIA THE TRANSITION ROUTINE X_XEL.
;
;	X_SYSTATE MUST BE INVOKED BY A FAR CALL.
;
X_SYSTATE	PROC	FAR
		PUSH	AX		;;MUST SAVE CONTENTS OF AX
		MOV	AX,MTOS_DS	;;SET UP THE MTOS DATA SEGMENT
		MOV	DS,AX
		POP	AX		;;SAVED BY SUBROUTINE
		CLI			;;;INTERRUPTS OFF!
		CALL	X_GET_SRB	;;GET AN SRB AND FILL IT IN
		POP	WORD PTR [BX+SRB_ADR]	;RETRIEVE THE RETURN ADDRESS
		POP	WORD PTR [BX+SRB_ADR+2]	;DIRECTLY INTO THE SRB.
		CALL	X_QUEUE_SRB	;;QUEUE SRB TO OUR SRB QUEUE
		JMP	SHORT X_XEL_0	;;COMPLETE THE TRANSITION VIA X_XEL.
X_SYSTATE	ENDP
PAGE
;	***************************************************************
;	*     X_XEL: DISMISS CURRENT INTERRUPT THREAD AND OPTIONALLY  *
;	*            ENTER THE SCHEDULER.                             *
;	***************************************************************
;
;	THIS SUPREMELY IMPORTANT METAROUTINE CHOOSES THE RETURN PATH OUT OF
;	THE INTERRUPT THREAD:
;
;	1)IF INTERRUPTS ARE NESTED MORE THAN ONE LEVEL DEEP, THE PREVIOUS
;	  INTERRUPT THREAD IS RESUMED.
;	2)IF THE DISMISSAL OF THE CURRENT INTERRUPT THREAD BRINGS THE NESTING
;	  LEVEL DOWN TO 0:
;		2.1)IF S-STATE IS IN FORCE, THE INTERRUPTED S-STATE PROGRAM IS
;			RESUMED.
;		2.2)IF S-STATE IS NOT IN FORCE:
;			2.2.1)IF ANY SRBS ARE WAITING ON THE SRB QUEUE, THE
;				SCHEDULER IS ENTERED.
;			2.2.2)IF THERE ARE NO WAITING SRBS:
;				2.2.2.1)IF A TASK WAS ACTIVE, ITS CONTEXT IS
;					UNSAVED INTO THE REGISTERS AND IT
;					REGAINS CPU CONTROL,
;				2.2.2.2)ELSE, THE SCHEDULER IS ENTERED.
;
;	NOTE THAT X_XEL IS ** NOT CALLED! ** IT IS ENTERED VIA A JMP.
;
X_XEL		PROC	FAR
		CLI				;COMING IN FROM ANYWHERE...
		MOV	AX,MTOS_DS		;FIRST ESTABLISH
		MOV	DS,AX			;THE MTOS_DS DATA SEGMENT.
X_XEL_0		LABEL	NEAR
		CALL	X_ISSUE_EOI	;ISSUE EOI TO INTRPT CONTROLLER 
		DEC	X_INTLEVEL	;REDUCE INTERRUPT NESTING.
		JNZ	UNNEST1		;ANOTHER INTERRUPT UNDER THIS ONE!
		CMP	XCSRB,0		;NO, ** CHANGING STATE: ** WAS S STATE
					;IN FORCE AT THE TIME OF THE INTERRUPT?
		JNZ	UNNEST1		;YES: RESUME INTERRUPTED PROCEDURE.
;
;	THE INTERRUPT OCCURRED IN A OR H STATE:
;
;	IF SRBS ARE PENDING, WE MUST ATTEND TO THEM; THEREFORE, THE TASK WHOSE
;	ADDRESS IS IN X_ATCB, IF ANY, IS REQUEUED TO THE READY QUEUE AND THE
;	SCHEDULER IS ENTERED. ELSE THE INTERRUPTED THREAD IS RESUMED AT THE
;	POINT OF THE INTERRUPT.
;
		MOV	BX,X_ATCB	;GET ACTIVE TCB NOW TO SAVE SOME CODE.
		CMP	X_SRB_Q,0	;TO THE SRB QUEUE: ANY SRBS?
		JZ	UNNEST		;NO: RESUME INTERRUPTED ACTIVITY.
		MOV	X_ATCB,0	;YES, ** CHANGING STATE: ** CLEAR THE
					;NOW IRRELEVANT A-STATE-TCB VARIABLE.
		TEST	BX,BX		;WAS A TASK ACTIVE?
		JZ	RESCHDL		;NO: ENTER SCHEDULER DIRECTLY.
		MOV	XCSRB,-1	;YES: FAKE S STATE,
		STI			;ALLOW INTERRUPTS.
;
;	THE FOLLOWING FOUR INSTRUCTIONS SUPPORT THE USE OF THE 8087 NUMERIC
;	DATA PROCESSOR AS A COPROCESSOR TO THE 8086. IF NOT USING THE 8087,
;	LEAVE THESE INSTRUCTIONS COMMENTED OUT. ELSE, REMOVE THE LEADING
;	SEMICOLONS. SEE ALSO THE SCHEDULER (SRB.ASM).
;
;		TEST	BYTE PTR [BX+TCB_FLAGS],TFL_8087	;USING NDP?
;		JE	NONDP		;NO.
;		MOV	BP,[BX+TCB_8087]	;YES: GET SAVE AREA ADDRESS,
;		FSAVE	DS:[BP]		;AND SAVE THE NDP'S REGISTERS.
;
NONDP:
		CALL	X_SET_READY	;RETURN ACTIVE TASK TO READY QUEUE
RESCHDL:
		JMP	X_SCHDLR	;AND ENTER THE SCHEDULER.
UNNEST:
		TEST	BX,BX		;WAS A TASK ACTIVE AT INTERRUPT TIME?
		JZ	UNNEST1		;NO, UNNEST TO H STATE.
		MOV	SS,[BX+TCB_SS]	;YES: SET UP ITS CORRECT
		MOV	SP,[BX+TCB_SP]	;STACK CONTROLS.
UNNEST1:
		POP	ES
		POP	DI
		POP	DX
		POP	CX
		POP	DS
		POP	SI
		POP	BP
		POP	BX
		POP	AX
		IRET			;RETURN TO INTERRUPTED ACTIVITY.
X_XEL		ENDP
PAGE
;****************************************************************
;*     XSYSTEM: GO FROM DI-TO-S STATE: PL/M-86 DRIVERS ONLY!    *
;****************************************************************
;
;	XSYSTEM PROVIDES THE TRANSISITION FROM DI-STATE DIRECTLY
;	TO S-STATE FOR DEVICE DRIVERS CODED IN PL/M-86. XSYSTEM
;	MUST BE CALLED FROM THE UNNESTED LEVEL OF AN INTERRUPT
;	PROCEDURE WHICH HAS DECLARED NO VARIABLES TO FUNCTION CORRECTLY.
;	THE STATE OF THE STACK IS IMPOSSIBLE TO DETERMINE ACCURATELY
;	UNLESS THESE RULES ARE FOLLOWED.
;
RET_OFST	EQU	WORD PTR [BX+SRB_ADR+0]
RET_SEG		EQU	WORD PTR [BX+SRB_ADR+2]
;
XSYSTEM		PROC	FAR
		MOV	AX,MTOS_DS	;DATA SEGMENT LOCATION
		MOV	DS,AX		;FOR GOOD OLD MTOS.
		CALL	X_GET_SRB	;ALLOCATE AN AVAILABLE ONE
		POP	RET_OFST	;STORE CONTINUATION ADDRESS
		POP	RET_SEG		;(OFFSET/SEGMENT)
		CALL	X_QUEUE_SRB	;OK, READY TO RUN.
;
;RESTORE PL/M-86'S REGISTER SAVE
;
		MOV	SP,BP		;RESET STACK POINTER
		POP	BP		;AND GET THE BP
		POP	DI		;DISCARD DS REG PUSHED BY PLM
		POP	DI		;DISCARD LONG CALL TO PROCEDURE PROPER
		POP	DI		
		POP	DI		;RESTORE ACTUAL REGISTERS
		POP	SI
		POP	BX
		POP	DX
		POP	CX
		POP	AX
		POP	DS
		POP	ES
		PUSH	AX		;STORE REGISTERS IN X_ELIPSE ORDER
		PUSH	BX
		PUSH	BP
		PUSH	SI
		PUSH	DS
		PUSH	CX
		PUSH	DX
		PUSH	DI
		PUSH	ES		;COMPLETE CONTEXT NOW SAVED.
		MOV	CX,MTOS_DS	;RESTORE IT AGAIN.
		MOV	DS,CX
		CMP	X_INTLEVEL,0	;ANY INTERRUPTS NESTED JUST NOW?
		JNZ	XSYS_1		;IF SO, LEAVE THE STACK ALONE.
		MOV	BX,X_ATCB	;NO: GET THE ACTIVE TCB ADDRESS.
		TEST	BX,BX		;WAS A-STATE INTERRUPTED?
		JZ	XSYS_1		;NO: LEAVE THE STACK ALONE.
		MOV	[BX+TCB_SP],SP	;YES: SAVE THE TASK'S STACK CONTROLS
		MOV	[BX+TCB_SS],SS	;IN ITS TCB, COMPLETING THE PRESERVA-
					;TION OF ITS INTERRUPTED CONTEXT.
		MOV	SS,CX		;INSTALL AS THE STACK FOR FURTHER PRO-
		MOV	SP,X_MTOS_SP		;CESSING THE SYSTEM STACK.
XSYS_1:
		INC	X_INTLEVEL	;X_XEL EXPECTED IT COUNTED UP.
		JMP	X_XEL		;LET MTOS TAKE IT FROM HERE.
XSYSTEM		ENDP
;
;**************************************************************************
;*     XEXSYS: PL/M-86 S-STATE EXIT ROUTINE: COMPLEMENTS XSYSTEM.         *
;**************************************************************************
;
;	XEXSYS PROVIDES THE RETURN LINK FROM THE DRIVER WRITTEN IN
;	PL/M-86 TO THE MTOS-86 SCHEDULER. THIS ROUTINE FUNCTIONS HAND-IN-HAND
;	WITH XSYSTEM. XEXSYS MUST BE THE LAST EXECUTABLE STATEMENT OF THE
;	INTERRUPT PROCEDURE.
;
XEXSYS		PROC	FAR
		POP	AX		;WASTE THE RETURN TO INTERRUPT
		POP	AX		;PROCEDURE ADDRESS
		RET			;AND INSTEAD RETURN (INTER-SEGMENT)
					;TO MTOS.
XEXSYS		ENDP
MTOS_CS		ENDS
		END
