PAGE	60,132
INCLUDE B:BIG.INC
;
;	SUBR1.ASM:	MTOS SUBROUTINES POOL.
;	F. W. PORRETTO, 12/15/82
;
		NAME	MTOS_GENERAL_SUBROUTINES
;
MTOS_CS		SEGMENT PARA PUBLIC
		ASSUME	CS:MTOS_CS,DS:MTOS_DS,SS:MTOS_DS
		EXTRN	X_SYSOUT	:NEAR	;EMERGENCY MSG TO CONSOLE
		EXTRN	X_DEVDATA	:FAR
		EXTRN	X_QUEUE_SRB	:NEAR	;ATTACH SRBS TO QUEUE
		EXTRN	XMTOS86		:NEAR	;IN CASE OF SYSTEM MEMORY CRASH
		EXTRN	X_TIMUNS	:WORD
		EXTRN	X_TCB_SIZE_VAR	:WORD
		EXTRN	X_NO_SMR_MSG	:WORD
;
		PUBLIC	X_SFLAGS,X_QUEUE,X_DQUEUE,X_GET_TCB_ADR
		PUBLIC	X_GET_TASK_ID
		PUBLIC	X_SET_READY,X_TASK_STATE,X_INIT_TASK
		PUBLIC	X_INTIME,X_GET_INHPRI,X_GET_INTERVAL
		PUBLIC	X_GET_SMR,X_PUT_SMR,X_WAIT_SMR
;
;
;
PAGE
;****************************************************************
;*        X_INTIME:    GET INTERNAL TIME                        *
;****************************************************************
;
;	INPUT:	BX= BUFER ADDRESS WITHIN MTOS_DS.
;	OUTPUT:	INTERNAL TIME (48 BITS) IN BUFFER
;
;		THE INTERNAL TIME, X_ABS_TIME, IS MAINTAINED BY MTOS IN MILLI-
;	SECONDS SINCE SYSTEM INITIALIZATION. THIS TIME IS THE BASIS FOR ALL 
;	ENDR COMPUTATIONS AND TRANSFORMATIONS OF TIMER COUNTDOWNS INTO EXPI-
;	RATION TIMES. IT IS NOT AVAILABLE TO THE USER;THE USER MUST NOT BE
;	ALLOWED TO CHANGE IT.
;
X_INTIME	PROC	NEAR
		PUSHF			;SAVE CURRENT INTERRUPT FLAG
		CLI			;AND DISABLE INTERRUPTS.
		MOV	AX,X_ABS_TIME
		MOV	[BX],AX
		MOV	AX,X_ABS_TIME+2
		MOV	[BX+2],AX
		MOV	AX,X_ABS_TIME+4
		MOV	[BX+4],AX
		POPF			;RETRIEVE OLD INTERRUPT FLAG.
		RET
X_INTIME	ENDP
PAGE
;*******************************************************************
;*      X_SFLAGS:    SET CONDITION CODES FOR SVC CALL              *
;*******************************************************************
;
;	THE SVC CALL CONDITION CODES ARE SET FOR THE CURRENT SVC
;	BEING PROCESSED.
;
;	INPUT:	AX=	CONDITION CODES TO SET
;	OUTPUT:	NONE
;
;	THE CURRENT TCB (X_CTCB) IS USED TO GET THE TCB TO SET
;	THE CONDITION CODES FOR.
;
X_SFLAGS	PROC	NEAR
		PUSH	BX
		PUSH	ES
		MOV	BX,X_CTCB		;CURRENT TCB ADR
		LES	BX,DWORD PTR [BX+TCB_SP]	;CURRENT TASK STACK.
	LOCK	OR	ES:[BX+STK_FL],AX	;PUT IN THE FLAGS
		POP	ES
		POP	BX
		RET	
X_SFLAGS	ENDP
PAGE
;**********************************************************************
;*      X_QUEUE:    ADD BLOCK TO PRIORITY-ORDERED QUEUE.              *
;**********************************************************************
;
;	INPUT:	BX=	ADDRESS OF BLOCK TO ADD TO QUEUE 
;		DI=	ADDRESS OF THE QUEUE ORIGIN WORD
;	OUTPUT:	NONE
;
;	THE SPECIFIED BLOCK IS PUT ON THE SPECIFIED QUEUE IN
;	PRIORITY ORDER, THE PRIORITY BEING BYTE 2 IN ALL THE BLOCKS
;	QUEUED. WITHIN A PRIORITY LEVEL, ORDERING IS FIRST-IN, FIRST OUT.
;	NOTE THAT ALL MTOS LINKED LISTS ARE FORWARD-GOING ONLY, AND THAT
;	LINKAGE IS BY OFFSET ADDRESS ONLY, PRESUMING THAT THE DATA STRUCTURE
;	IS LIMITED TO ONE SEGMENT.
;
X_QUEUE		PROC	NEAR
		PUSH	BX
		PUSH	DI
		PUSH	SI
		MOV	AL,[BX+2]		;PRIORITY OF NEW BLOCK
		MOV	SI,[DI]			;FIRST BLOCK ON QUEUE
NEXT:
		TEST	SI,SI				;BLOCK EXIST ?
		JE	RELINK				;--NO--
		CMP	AL,[SI+2]		;PRIORITY OF NEW BLOCK
						;GREATER THAN CURRENT ?
		JA	RELINK			;--YES--
		MOV	DI,SI			;SET CURRENT AS PREVIOUS
		MOV	SI,[SI]			;NEXT IS NOW CURRENT
		JMP	NEXT				;TRY NEXT BLOCK
RELINK:
		MOV	[BX],SI			;SET NEW BLOCK POINTING TO 
						;	CURRENT BLOCK.
		MOV	[DI],BX			;PREVIOUS POINTS TO NEW BLOCK
		POP	SI
		POP	DI
		POP	BX
		RET
X_QUEUE 	ENDP
PAGE
;************************************************************************
;*      X_DQUEUE:    REMOVE A BLOCK FROM A QUEUE                        *
;************************************************************************
;
;	INPUT:	BX=	ADDR OF BLOCK TO REMOVE FROM QUEUE
;		DI=	ADDR OF THE QUEUE ORIGIN - HEAD OF QUEUE
;	OUTPUT: CARRY=	0 IF BLOCK WAS REMOVED
;			1 IF BLOCK WAS NOT FOUND
;
X_DQUEUE	PROC	NEAR
		PUSH	DI	
		PUSH	SI
		MOV	SI,[DI]		;FIRST BLOCK ON QUEUE
NEXT_ONE:
		TEST	SI,SI			;END OF QUEUE ?
		JE	NOT_FOUND		;--YES--
		CMP	SI,BX			;MATCH ?
		JE	FOUND			;--YES--
		MOV	DI,SI			;TRY NEXT ONE
		MOV	SI,[SI]
		JMP	NEXT_ONE
;
;  FOUND A MATCH
;
FOUND:
		MOV	AX,[SI]			;NEXT BLOCK
		MOV	[DI],AX			;SET PREVIOUS TO NEXT
		CLC				;CLEAR CARRY FOR GOOD RETURN
		JMP	SHORT DQ_RET
;
;  DID NOT FIND THE BLOCK
;
NOT_FOUND:
		STC				;SET CARRY FOR NOT REMOVED
DQ_RET:
		POP	SI
		POP	DI
		RET	
X_DQUEUE	ENDP
PAGE
;**********************************************************************
;*      X_GET_TCB_ADR:    COMPUTE TCB ADDRESS FROM TASK NUMBER        *
;*      X_GET_TASK_ID:    COMPUTE TASK NUMBER FROM TCB ADDRESS        *
;**********************************************************************
;
;	X_GET_TCB_ADR:
;	INPUT:	AX=	TASK ID
;	OUTPUT:	BX=	TCB ADDRESS
;		CARRY=	0 IF VALID TASK,
;			1 IF NOT
;
;	X_GET_TASK_ID:
;	INPUT:	BX=	TCB ADDRESS
;	OUTPUT:	AX=	TASK NUMBER IF CF=0.
;			(IF CF= 1, DIVIDE LEFT A REMAINDER OR THE
;			RESULT WAS > X_TASK_NUM.)
;
X_GET_TCB_ADR	PROC	NEAR
		CMP	AX,X_TASK_NUM
		JB	VALID_TCB
		STC
		RET
VALID_TCB:	PUSH	DX
		MUL	CS:X_TCB_SIZE_VAR 	;TCB ENTRY SIZE * TASK ID = OFFSET
		ADD	AX,OFFSET X_TCB_TABLE  ;TABLE ADR + OFFSET = TCB ADR
		MOV	BX,AX		
		CMP	WORD PTR [BX+TCB_TCD+2],0	;TCB IN USE?
		JNE	XGT1			;YES.
		POP	DX			;NO:
		STC				;INDICATE FAILURE.
		RET
XGT1:		POP	DX
		CLC
		RET	
X_GET_TCB_ADR   ENDP
;
X_GET_TASK_ID	PROC	NEAR
		PUSH	DX
		MOV	AX,BX		;MOVE TCB ADDRESS TO AX
		SUB	AX,OFFSET X_TCB_TABLE	;REMOVE BASE OF TABLE
		XOR	DX,DX		;PREPARE TO DIVIDE...
		DIV	CS:X_TCB_SIZE_VAR	;PERFORM DIVISION.
		TEST	DX,DX		;REMAINDER?
		POP	DX
		JNE	XGTID0		;YES, BAD INPUT.
		CMP	AX,X_TASK_NUM	;NO: NUMBER WITHIN RANGE?
		JAE	XGTID0		;NO.
		CMP	WORD PTR [BX+TCB_TCD+2],0	;YES: TCD DEFINED?
		JE	XGTID0		;NO, NOT A TASK.
		CLC			;YES.
		RET
XGTID0:		STC
		RET
X_GET_TASK_ID	ENDP
PAGE
;***********************************************************************
;*      X_SET_READY:    MOVE TASK TO THE READY STATE                   *
;***********************************************************************
;
;	INPUT:	BX=	TCB ADDRESS
;	OUTPUT:	CARRY=	1 IF TASK WAS IN SAFETY STATE (BLOCK+STA_SVC)
;			OR ALREADY READY,
;			0 IF A STATE-TRANSITION TOOK PLACE.
;
;		THE TASK'S STATE IS CHECKED: IF IT IS CURRENTLY DORMANT, THE 
;	CURRENT SYSTEM TIME IS FILLED INTO TCB_SCHDL. THE TCB IS THEN SET
;	READY AND LINKED TO THE CORRECT READY QUEUE IN PRIORITY ORDER.
;
X_SET_READY	PROC	NEAR
		CMP	BYTE PTR [BX+TCB_STATE],READY	;IS TASK READY?
		JE	XSR03			;YES, DO NOTHING.
		CMP	BYTE PTR [BX+TCB_STATE],BLOCK+STA_SVC	;SAFETY STATE?
		JNE	XSR00			;NO, WE MAY READY IT.
XSR03:
		STC				;IN CASE ANYONE CARES
		RET
XSR00:		PUSH	DI
		MOV	AL,[BX+TCB_STATE]
		PUSH	AX			;SAVE INPUT STATE.
		LEA	DI,X_LREADY_Q		;ADDRESS THE READY QUEUE.
XSR01:		CALL	X_QUEUE			;ATTACH TASK TO QUEUE.
		MOV	BYTE PTR [BX+TCB_STATE],READY	;SET STATE.
		POP	AX			;RETRIEVE INPUT STATE,
		CMP	AL,DORMANT		;WAS IT DORMANT?
		JNE	XSR02			;NO.
		PUSH	BX
		ADD	BX,TCB_SCHDL
		CALL	X_INTIME		;INSERT START TIME.
		POP	BX
XSR02:
		CLC
		POP	DI
		RET
X_SET_READY	ENDP
PAGE
;***********************************************************************
;*      X_TASK_STATE:    MOVE THE TASK TO THE SPECIFIED STATE          *
;***********************************************************************
;
;	THE SPECIFIED TASK IS SET TO THE SPECIFIED NON-READY STATE.
;
;	INPUT:	AX=	FULLY QUALIFIED TASK STATE WORD,
;		BX=	TCB ADDRESS.
;	OUTPUT:	NONE
;
;	THE TASK IS REMOVED FROM THE READY QUEUE AND MOVED TO WHATEVER STATE
;	IS DESCRIBED BY THE WORD IN AX. NOTE THAT, SINCE THE DEQUEUEING ROU-
;	TINES ARE FAILSAFE, THE TASK NEED NOT BE ON THE READY QUEUE TO MAKE USE
;	OF THIS ROUTINE. NOTE ALSO THAT IF THE TRANSITION IS TO ACTIVE STATE,
;	THE TCB ADDRESS OF THE TASK MUST STILL BE SET INTO THE X_ATCB WORD BY
;	THE CALLER (PRESUMABLY THE SCHEDULER).
;
X_TASK_STATE	PROC	NEAR
		PUSH	DI
		PUSH	AX		;SAVE DESTINATION STATE
		CMP	BYTE PTR [BX+TCB_STATE],READY	;TASK READY?
		JNE	XTS00		;NO.
		LEA	DI,X_LREADY_Q	;ADDRESS THE READY QUEUE.
XTS01:
		CALL	X_DQUEUE	;REMOVE TASK FROM READY QUEUE
XTS00:
		POP	AX			;RETRIEVE AND
		MOV	[BX+TCB_STATE],AL	;INSERT NEW STATE
		POP	DI
		RET
X_TASK_STATE	ENDP
PAGE
;******************************************************************
;*      X_INIT_TASK:    INITIALIZE TCB AND TASK STACK             *
;******************************************************************
;
;	INPUT:	BX=	TCB ADDRESS
;	OUTPUT:	NONE
;
;	THIS ROUTINE INITIALIZES A DORMANT TASK'S STACK. IT MUST NOT BE
;	USED ON A TASK NOT IN THE DORMANT STATE OR THE FUNCTIONAL EQUIVALENT.
;
X_INIT_TASK	PROC	NEAR
		PUSH	ES
		PUSH	DX
		PUSH	DI
		PUSH	SI
		PUSH	DS
		LES	DI,DWORD PTR [BX+TCB_TCD]	;GET TCD ADDRESS
		PUSH	ES			;THIS WILL BECOME CLEAR LATER.
		LES	SI,DWORD PTR ES:[DI+TCD_SP]
		SUB	SI,32			;TOP OF STARTUP STACK.
		POP	DS			;NOW DS= TCD SEGMENT ADDRESS.
		MOV	AX,[DI+TCD_AX]		;GET TCD AX AND
		MOV	DX,[DI+TCD_BX]		;BX PARAMETERS.
		MOV	ES:[SI+STK_TAX1],AX	;SET THOSE PARAMETERS
		MOV	ES:[SI+STK_TBX1],DX	;TWICE EACH
		MOV	ES:[SI+STK_TAX2],AX	;INTO THE STACK
		MOV	ES:[SI+STK_TBX2],DX	;OF THE TASK.
		MOV	WORD PTR ES:[SI+STK_FL],INIT_FLAGS	;STARTUP FLAGS.
		MOV	AX,[DI+TCD_CS]		;SET THE
		MOV	ES:[SI+STK_CS],AX	;CODE SEGMENT.
		MOV	AX,[DI+TCD_IP]		;SET THE
		MOV	ES:[SI+STK_IP],AX	;INSTRUCTION POINTER.
		MOV	AX,[DI+TCD_DS]		;SET THE
		MOV	ES:[SI+STK_DS],AX		;DATA SEGMENT.
		MOV	AX,[DI+TCD_ES]		;SET THE
		MOV	ES:[SI+STK_ES],AX	;EXTRA SEGMENT.
		POP	DS			;DONE WITH TCD, SET DS=MTOS_DS
		MOV	[BX+TCB_SP],SI		;SAVE THE STACK POINTER
		MOV	[BX+TCB_SS],ES		;IN THE TCB.
		POP	SI
		POP	DI
		POP	DX
		POP	ES
		RET
X_INIT_TASK	ENDP
PAGE
;****************************************************************************
;*      X_GET_INHPRI:   GET THE INHERENT PRIORITY OF THE SPECIFIED TASK.    *
;****************************************************************************
;
;	INPUT:	BX= TCB ADDRESS,
;	OUTPUT:	AL= INHERENT PRIORITY.
;
X_GET_INHPRI	PROC	NEAR
		PUSH	ES
		PUSH	DI
		LES	DI,DWORD PTR [BX+TCB_TCD]	;GO TO THE TCD
		MOV	AL,ES:[DI+TCD_INPRI]		;GET THE INHERENT PRI.
		POP	DI
		POP	ES
		RET
X_GET_INHPRI	ENDP
;
PAGE
;**********************************************************************
;*              X_GET_INTERVAL:    COMPUTE TIME COUNTDOWN             *
;**********************************************************************
;
;	INPUT:	AL=	UNIT CODE, MULTIPLIED BY FOUR.
;		AH=	NUMBER OF UNITS.
;	OUTPUT:	CX,DX=	TIME INTERVAL (HIGH,LOW)
;
X_GET_INTERVAL	PROC	NEAR
		PUSH	DI
		PUSH	BX
		MOV	BL,AL		;TYPE OF UNIT TO BL,
		XOR	BH,BH		;CLEAR BH.
		MOV	AL,AH		;NUMBER OF TIME UNITS.
		TEST	AL,AL				;NONE ?
		JNE	INT15				;-- NO --
		XOR	CX,CX				;INTERVAL TO ZERO
		XOR	DX,DX
		JMP	INTRET
INT15:
		XOR	AH,AH		;CLEAR AH FOR MULTIPLY.
		MOV	DI,AX				;SAVE NUMBER OF UNITS
;
;	BECAUSE THE TIME UNITS CODES RUN AT INTERVALS OF 4 (REMEMBER THE 
;	SDB 1CH SHIFT MASK), THE UNIT CODE AS SUBMITTED IS A PERFECT INDEX
;	INTO THE X_TIMUNS TABLE OF MILLISECONDS PER UNIT.
;
		MUL	CS:X_TIMUNS[BX+2]		;LOW ORDER FIRST
		PUSH	DX
		PUSH	AX
		MOV	AX,DI				;HIGH ORDER
		MUL	CS:X_TIMUNS[BX]
		POP	CX				;LOW ORDER
		POP	DX				;HIGH ORDER
		ADD	DX,AX
		XCHG	CX,DX
INTRET:
		POP	BX
		POP	DI
		RET
X_GET_INTERVAL	ENDP
PAGE
;*****************************************************************************
;*              SYSTEM MEMORY RESOURCE ROUTINES (SMR)                        *
;*****************************************************************************
;
;	X_GET_SMR:	GET AN SMR BLOCK.
;	X_PUT_SMR:	FREE AN SMR BLOCK.
;	X_WAIT_SMR:	SIDELINE A SERVICE REQUEST TO AWAIT SMR BLOCKS.
;
;		THESE ROUTINES MANIPULATE THE SYSTEM MEMORY RESOURCE AND ITS
;	CONTROL STRUCTURES:
;
;	X_SMR_LIST:	THE HEAD OF THE FREE SMR BLOCK QUEUE.
;	X_SMR_Q:	THE QUEUE OF SRBS AWAITING SUFFICIENT SMR BLOCKS
;	X_SMR_AVAIL:	THE NUMBER OF SMR BLOCKS CURRENTLY AVAILABLE
;	
;		1:	X_WAIT_SMR.
;
;	INPUT:	BX= ADDRESS OF SRB FOR SIDELINING S-STATE REQUEST.
;	OUTPUT:	NONE.
;
X_WAIT_SMR	PROC	NEAR
		PUSH	DI
		MOV	DI,OFFSET X_SMR_Q	;GET HEAD OF WAIT QUEUE
						;COMPOSED OF SRBS.
@6:
		CMP	WORD PTR [DI],0		;AT END OF QUEUE YET?
		JE	@7			;YES, HOOK ON.
		MOV	DI,[DI]			;NO, LINK FORWARD
		JMP	@6			;TO EXAMINE NEXT ONE.
@7:
		MOV	[DI],BX			;LINK SRB TO TAIL OF QUEUE
		MOV	WORD PTR [BX],0		;CLEAR NEW TAIL'S FWD. LINK.
		POP	DI
		RET
X_WAIT_SMR	ENDP
;
;	X_GET_SMR:	ALLOCATE AN SMR BLOCK.
;
;	INPUT:	NONE.
;	OUTPUT:	BX= ADDRESS OF SMR BLOCK.
;
X_GET_SMR	PROC	NEAR
		PUSH	DI
		CMP	X_SMR_AVAIL,0		;ANY FREE??
		JAE	XGNS0			;YES, DO IT.
		MOV	BX,OFFSET X_NO_SMR_MSG	;NO:
		MOV	CX,50			;OUTPUT A CATASTROPHE MSG
		CALL	X_SYSOUT		;TO CONSOLE
		CLI
		HLT				;SYSTEM FIRMLY DOWN.
XGNS0:
		DEC	X_SMR_AVAIL		;REDUCE FREE COUNT.
		MOV	BX,X_SMR_LIST		;GET ADDRESS OF FIRST ONE.
		MOV	AX,[BX]			;GET FORWARD LINK,
		MOV	X_SMR_LIST,AX		;SET LIST HEAD TO FLINK.
		MOV	WORD PTR [BX],0		;CLEAR FLINK OF ALLOCATED BLK.
		POP	DI
		RET
X_GET_SMR	ENDP
;
;	X_PUT_SMR:	DEALLOCATE AN SMR BLOCK.
;
;	INPUT:	BX= ADDRESS OF BLOCK.
;	OUTPUT:	NONE.
;
;	THIS ROUTINE ENFORCES THE RULE OF ALL CONTINGENT PROCESSING AS IT AP-
;	PLIES TO SMR BLOCKS: ANY S-STATE REQUESTS WAITING FOR SMR ARE RETURNED
;	TO THE SRB QUEUE TO BE RETRIED.
;
X_PUT_SMR	PROC	NEAR
		PUSH	SI
		PUSH	DI
		INC	X_SMR_AVAIL		;INCREASE FREE COUNT.
		MOV	AX,X_SMR_LIST		;GET CURRENT LIST HEAD
		MOV	X_SMR_LIST,BX		;AND SET FREED CHAIN=NEW HEAD
		MOV	[BX],AX			;HOOK LIST TO NEW ENTRY.
		MOV	SI,SMR_SIZE-2	;LENGTH OF SMR IN WORDS, MINUS 1.
		XOR	AX,AX
XPNS1:
		MOV	[BX][SI],AX	;ZAP A WORD OF THE FREED BLOCK,
		SUB	SI,2
		JNE	XPNS1		;AND GO AROUND FOR ANOTHER...
		MOV	BX,X_SMR_Q	;DONE ZAPPING: GET HEAD OF WAIT QUEUE
		MOV	X_SMR_Q,0	;AND ZAP IT.
		TEST	BX,BX		;ANY SRBS WAITING?
		JE	XPNS3		;NO.
XPNS4:
		PUSH	WORD PTR [BX]	;SAVE LINK TO NEXT SRB
		CALL	X_QUEUE_SRB	;RETURN THIS ONE TO THE SRB QUEUE.
		POP	BX		;RETRIEVE FORWARD LINK
		TEST	BX,BX		;DONE?
		JNE	XPNS4		;NOT YET.
XPNS3:
		POP	DI
		POP	SI
		RET
X_PUT_SMR	ENDP
;
MTOS_CS		ENDS
		END
