PAGE	60,132
INCLUDE B:BIG.INC
;
;	TIME.ASM:	ASCII TIME AND DATE SERVICES.
;	F. W. PORRETTO, 12/15/82
;
		NAME	TIME_SVC_SERVICE
;
		PUBLIC	X_STIME,X_TIME,X_TIME_UPDATE
		PUBLIC	X_SET_TIME,X_READ_TIME
;
MTOS_CS		SEGMENT PARA PUBLIC
		ASSUME	CS:MTOS_CS,DS:MTOS_DS,SS:MTOS_DS
		EXTRN	X_SFLAGS	:NEAR	;
		EXTRN	X_INTIME	:NEAR	;GET INTERNAL TIME
		EXTRN	X_DQUEUE_TMB	:NEAR	;REMOVE TMB FROM TIMER QUEUE
		EXTRN	X_QUEUE_TMB	:NEAR	;PLACE TMB ON TIMER QUEUE
;
;	LOCAL EQUATES
;
FORMT		EQU	01		;SDB OFFSET TO OPTION FIELD
BUFR		EQU	02		;             BUFFER ADDRESS
FORMT_MASK	EQU	80H		;ISOLATE FORMT FIELD
;
;	EQUATES FOR SAVING TOD IN SYSTEM TABLE - X_ASCTOD
;
TOD_DAY		EQU	00
TOD_MON		EQU	01
TOD_YEAR	EQU	02
TOD_HOUR	EQU	04
TOD_MIN		EQU	05
;
;	AT ONE POINT IN THE CONVERSION/VALIDATION STAGE, IT IS NECESSARY TO
;	SAVE THE TIME SUBMITTED IN A TEMPORARY REGION BUILT ON THE STACK.
;	THESE VARIABLES ARE USED TO REFER TO THAT REGION.
;
SAV_DAY		EQU	BYTE PTR SS:[BP]
SAV_MON		EQU	BYTE PTR SS:[BP+1]
SAV_YEAR	EQU	WORD PTR SS:[BP+2]
SAV_HOUR	EQU	BYTE PTR SS:[BP+4]
SAV_MIN		EQU	BYTE PTR SS:[BP+5]
;
ASC0		EQU	30H		;ASCII 0
ASC9		EQU	39H		;      9
ASCDASH		EQU	2DH		;      - (DASH)
ASCSPACE	EQU	20H		;        (SPACE)
ASCCOLON	EQU	3AH		;      :
ASCZEROS	EQU	3030H		;TWO ZERO'S
;
;	DATA STRUCTURES - FIXED
;
MONTH_TABLE	EQU	(THIS BYTE) - 3 
		DB	'JANFEBMARAPRMAYJUNJULAUGSEPOCTNOVDEC'
MONTH_DAY_TABLE EQU	(THIS BYTE) -1
		DB	31,28,31,30,31,30,31,31,30,31,30,31
PAGE
;************************************************
;*  XASCBIN:   CONVERT ASCII TO BINARY          *
;************************************************
;
;	THIS PROCEDURE CONVERTS TWO ASCII DIGITS TO ONE BYTE.
;
;	INPUT:	ES:BX=	ADDRESS OF INPUT STRING
;	OUTPUT:	AL=	NUMBER FROM 0 TO 99
;		CARRY=	1 IF CONVERSION ERROR (NON-ASCII NUMBER)
;		BX=	UPDATED BY TWO IF VALID RETURN
;
XASCBIN		PROC	NEAR
		PUSH	DX
		MOV	AL,ES:[BX]			;HI ORDER ASCII NUMBER
		CMP	AL,ASC9				;TOO HIGH ?
		JA	ASCERR				;--YES--
		CMP	AL,ASC0				;TOO LOW ?
		JB	ASCERR				;-- YES --
		CBW
		SUB	AX,ASC0				;NORMALIZE
		MOV	DX,10
		MUL	DX				;MULTIPLY BY 10
		XCHG	AX,DX
		MOV	AL,ES:[BX+1]			;LOW ORDER BYTE
		CMP	AL,ASC0				;TOO LOW ?
		JB	ASCERR				;--YES
		CMP	AL,ASC9				;TOO HIGH ?
		JA	ASCERR				;--YES--
		CBW
		SUB	AX,ASC0				;NORMALIZE
		ADD	AX,DX				;TENS + UNITS
		ADD	BX,2				;UPDATE POINTER
		CLC					;CARRY = 0
		JMP	SHORT ASCRET
ASCERR:
		STC					;CARRY = 1
ASCRET:
		POP	DX
		RET
XASCBIN		ENDP
PAGE
;**************************************************
;*      XBINASC:    CONVERT BINARY TO ASCII       *
;**************************************************
;
;	THIS PROCEDURE CONVERTS A NUMBER FROM 0 TO 99
;	TO TWO ASCII CHARACTERS
;
;	INPUT:	ES:BX=	ADDRESS OF OUTPUT BUFFER
;		AL=	NUMBER TO CONVERT
;	OUTPUT:	ES:BX=	WITH CHARACTERS AND BX IS UPDATED BY 2
;
XBINASC		PROC	NEAR
		MOV	WORD PTR ES:[BX],ASCZEROS	;TWO ZERO'S
		TEST	AL,AL				;NUMBER = 0 ?
		JE	BINRET				;--YES--
		PUSH	DX
		CBW
		MOV	DX,10				;DIVIDE BY TEN
		DIV	DL
		ADD	AL,ASC0				;CREATE ASCII
		ADD	AH,ASC0
		MOV	ES:[BX],AL
		MOV	ES:[BX+1],AH
		POP	DX
BINRET:
		ADD	BX,2				;UPDATE OFFSET
		RET
XBINASC		ENDP
PAGE
;*********************************************************************
;*      XQ_TOD_TMB:  HANDLE THE TOD TMB WHEN THE TIME IS SET         *
;*********************************************************************
;
;	THE X_SET_TIME PROCEDURE CALLS THIS PROCEDURE TO
;	HANDLE THE TIME/DATE TIME MONITOR BLOCK.  THIS
;	PROCEDURE SEARCHES THE TMB FOR THE TOD TMB.  IF
;	IT FINDS IT, IT DEQUEUES IT,  SETS THE TIMEOUT TO
;	ONE MINUTE AND REQUEUES IT.  IF THE TMB IS NOT
;	ON THE QUEUE, IT TESTS IF IT WAS EVER ON THE QUEUE.
;	IF IT WAS, IT MUST HAVE TIMED OUT, SO NO ADDITIONAL
;	PROCESSING IS REQUIRED.  IF THE TMB WAS NEVER ON
;	THE QUEUE, THIS PROCEDURE FILLS ALL THE REQUIRED
;	FIELDS AND QUEUES THE TMB.
;
XQ_TOD_TMB	PROC	NEAR
		PUSH	BX				;STORE REGISTERS
;
;	CHECK IF TOD TIMER BLOCK ON TIMER QUEUE
;
		MOV	BX,OFFSET X_TOD_TMB	;ADR OF DEDICATED TMB
		CALL	X_DQUEUE_TMB		;REMOVE IT FROM THE 
						; TIMER QUEUE (IF ON IT)
		JNC	TMBON			;C=0 IF BLOCK FOUND
;
;	TMB WAS NOT ON THE QUEUE, SO TEST IF IT WAS EVER ON THE
;	QUEUE.  IF NOT, FILL AND QUEUE IT.
;
		CMP	BYTE PTR [BX+TMB_TYPE],0
		JNE	XQDONE
;
;	TMB WAS NEVER ON QUEUE, SO FILL IT AND QUEUE
;
		MOV	[BX+TMB_SADR+2],CS	;SET THE INTERRUPT ADDRESS.
		MOV	WORD PTR [BX+TMB_SADR],OFFSET X_TIME_UPDATE
		MOV	BYTE PTR [BX+TMB_TYPE],1 ;USE DRIVER TYPE
TMBON:
		MOV	WORD PTR [BX+TMB_TIME],0	;RESET MSEC COUNTDOWN
		MOV	WORD PTR [BX+TMB_TIME+2],60000
		CALL	X_QUEUE_TMB		 ;START IT GOING
XQDONE:
		POP	BX
        	RET
XQ_TOD_TMB	ENDP
PAGE
;******************************************************************
;*      XPUTASC:   PLACE ASCII CHARACTER IN BUFFER                *
;******************************************************************
;
;	THIS PROCEDURE MOVES A CHARACTER INTO THE
;	SPECIFIED BUFFER AND UPDATES THE OFFSET BY ONE
;
;	INPUT:	ES:BX = POINTER TO DESTINATION FOR CHARACTER.
;		AL=	CHARACTER
;	OUTPUT:	BX IS UPDATED BY ONE
;
XPUTASC		PROC	NEAR
		MOV	ES:[BX],AL		;PUT IN CHARACTER
		INC	BX
		RET
XPUTASC		ENDP
PAGE
;****************************************************************
;*         X_READ_TIME:  INNER READ ASCII DATE/TIME ROUTINE     *
;****************************************************************
;
;	THIS PROCEDURE READS THE ASCII DATE/TIME AND PREPARES
;	THE SVC STATUS.
;
;	INPUT:	ES:BX=	BUFFER ADDRESS TO RETURN ASCII DATE/TIME
;	OUTPUT:	AX=	SVC STATUS
;
X_READ_TIME	PROC	NEAR
		PUSH	DI			;SAVE REGISTERS
		PUSH	SI
		PUSH	CX
		PUSH	DX
		PUSH	BP
;
;	MOVE CURRENT DATE/TIME TO A SAVE AREA ON THE STACK.
;
		PUSH	X_ASCTOD+4		;STORE IT ON STACK: HOUR/MIN,
		PUSH	X_ASCTOD+2		;YEAR,
		PUSH	X_ASCTOD		;MONTH/DAY.
;
;	CONVERT TO ASCII, INSERTING DASHES, SPACES, COLONS, ETC.
;
		MOV	BP,SP			;ACCESS WITH BP REGISTER
		MOV	AL,SAV_DAY		;AL=DAY,AH=MONTH
		CALL	XBINASC			;CONVERT & STORE DAY
		MOV	AL,ASCDASH
		CALL	XPUTASC
		MOV	AL,SAV_MON		;GET MONTH.
		CBW				;EXTEND
		MOV	SI,AX			;MUL * 3
		SAL	AX,1
		ADD	SI,AX
		MOV	CX,3			;GET 3 LETTER ASCII MONTH
;
TNEXT:		MOV	AL,CS:MONTH_TABLE[SI]
		CALL	XPUTASC
		INC	SI
		LOOP	TNEXT
;
		MOV	AL,ASCDASH
		CALL	XPUTASC
		MOV	AX,SAV_YEAR		;GET YEAR.
		MOV	DX,100			;DIVIDE TO GET TWO PARTS
		DIV	DL			;AL=QUOTIENT,AH=REMAINDER
DIV_DONE:
		PUSH	AX			;SAVE REMAINDER WHILE WE
		CALL	XBINASC			;CONVERT HUNDRED'S PART.
		POP	AX			;RETRIEVE UNIT'S PART.
		MOV 	AL,AH			;TO AL FOR CONVERSION.
		CALL	XBINASC			;CONVERT IT.
		MOV	AL,ASCSPACE		;PAD WITH SPACE.
		CALL	XPUTASC
		MOV	AL,SAV_HOUR		;CONVERT & STORE HOUR
		CALL	XBINASC
		MOV	AL,ASCCOLON
		CALL	XPUTASC
		MOV	AL,SAV_MIN		;GET/CONVERT MINUTE.
		CALL	XBINASC
		ADD	SP,6			;DESTROY SAVE AREA.
		XOR	AX,AX			;CLEAR STATUS
		POP	BP			;RESTORE REGISTERS
		POP	DX
		POP	CX
		POP	SI
		POP	DI
		RET
X_READ_TIME	ENDP
PAGE
;****************************************************************
;*         X_SET_TIME:  INNER SET ASCII DATE/TIME PROCEDURE     *
;****************************************************************
;
;	THIS PROCEDURE SETS THE ASCII DATE AND TIME AND
;	PREPARES THE SVC STATUS.
;
;	INPUT:	ES:BX=	ADDRESS OF ASCII STRING SPECIFYING TIME.
;	OUTPUT:	AX=	SVC STATUS
;
X_SET_TIME	PROC	NEAR
		PUSH	DX			;SAVE REGISTERS
		PUSH	CX
		PUSH	BX
		PUSH	DI
		PUSH	SI
		PUSH	BP
		SUB	SP,6			;CREATE THE TEMPORARY SAVE
		MOV	BP,SP			;REGION ON THE STACK.
;
;	CONVERT DATE/TIME TO BINARY (SAVE ON STACK) AND VALIDATE
;
		CALL	XASCBIN			;CONVERT DAY
		JB	STIME_ERR5		;CONVERSION ERROR ? -- YES --
		TEST	AL,AL			;DAY = 0 THEN ERROR ?
		JE	STIME_ERR5		;--YES--
		MOV	SAV_DAY,AL		;SAVE IT.
		INC	BX			;PASS DASH
		MOV	SI,OFFSET MONTH_TABLE	;GET MONTH - CONVERT TO 1-12
		MOV	CX,12
NEXTMON:
		MOV	AX,ES:[BX]		;FIRST TWO LETTERS OF MONTH
		CMP	WORD PTR CS:[SI+3],AX	;MATCH TABLE ?
		JNE	WRONG_MONTH		;--NO--
		MOV	AL,ES:[BX+2]		;THIRD LETTER 
		CMP	BYTE PTR CS:[SI+5],AL	;MATCH ?
		JE	VALID_MONTH		;--YES--
WRONG_MONTH:
		ADD	SI,3			;TRY NEXT MONTH
		LOOP	NEXTMON
		JMP	STIME_ERR5		;ERROR - COULD NOT MATCH MONTH
VALID_MONTH:
		NEG	CL			;CONVERT LOOP COUNTER INTO THE
		ADD	CL,13			;MONTH NUMBER DESIRED.
		MOV	SAV_MON,CL		;SAVE MONTH.
		ADD	BX,4			;BYPASS MONTH AND DASH
		CALL	XASCBIN			;YEAR: FIRST TWO CHARACTERS
		JNB	STIME15			;--NO ERROR--
STIME_ERR5:
		JMP	STIME_ERR		;CRUTCH TO REACH STIME_ERR
STIME15:
		MOV	DL,100			;FIRST TWO CHARACTERS ARE
		MUL	DL			;HUNDREDS PART OF YEAR
		MOV	SAV_YEAR,AX		;SAVE THAT FOR A MOMENT...
		CALL	XASCBIN			;CONVERT UNITS PART OF YEAR
		JB	STIME_ERR		;CONVERSION ERROR.
		ADD	SAV_YEAR,AX		;GOOD: ADD TO 100'S PART.
;
;	  HOUR AND MINUTE
;
		INC	BX			;BYPASS SPACE
		CALL	XASCBIN			;CONVERT HOUR.
		JB	STIME_ERR		;CONVERSION ERROR.
		CMP	AX,23			;HOUR = 0 TO 23 ?
		JA	STIME_ERR		;--NO--
		MOV	SAV_HOUR,AL		;SAVE THE HOUR.
		INC	BX			;BYPASS THE COLON.
		CALL	XASCBIN			;CONVERT MINUTE.
		JB	STIME_ERR		;CONVERSION ERROR.
		CMP	AX,59			;MINUTE = 0 TO 59 ?
		JA	STIME_ERR		;--NO--
		MOV	SAV_MIN,AL		;SAVE THE MINUTE.
;
;	  COMPLETE THE VALIDATION - CHECK DAY WITH LEAP YEAR STUFF
;
		XOR	DX,DX			;DX = 0 FOR NOT A LEAP YEAR
		CMP	SAV_MON,2		;IS IT FEBRUARY??
		JNE	DAY_CHECK		;--NO--
		MOV	AX,SAV_YEAR		;YES: LEAP YEAR CHECK.
		AND	AX,0003H		;DIVISBLE BY FOUR ?
		JNE	DAY_CHECK		;--NO-- (LOW TWO BITS NOT 00)
		INC	DX			;DX=1 FOR LEAP YEAR
DAY_CHECK:
		MOV	AL,SAV_MON		;GET THE MONTH AGAIN.
		CBW
		MOV	SI,AX
		MOV	AL,CS:MONTH_DAY_TABLE [SI]  ;DAYS IN MONTH
		ADD	AL,DL			;ADD IN POSSIBLE LEAP YEAR
		CMP	AL,SAV_DAY		;DAY WITHIN MONTH'S RANGE?
		JL	STIME_ERR		;--YES--
PAGE
;
;	MOVE BINARY VALUES TO ASCTOD AND HANDLE TIMER BLOCK
;
		MOV	BX,OFFSET X_ASCTOD	;ADR OF ASCII DATE/TIME.
		MOV	AX,SS:[BP]			;MOVE THEM.
		MOV	X_ASCTOD,AX
		MOV	AX,SS:[BP+2]
		MOV	X_ASCTOD+2,AX
		MOV	AX,SS:[BP+4]
		MOV	X_ASCTOD+4,AX
		CALL	XQ_TOD_TMB		;HANDLE THE TIMER BLOCK
;
;	SUCCESSFUL WRAP-UP
;
		XOR	AX,AX			;CLEAR STATUS
STIME_OVER:
		ADD	SP,6			;DESTROY SAVE AREA.
		POP	BP			;RESTORE REGISTERS
		POP	SI
		POP	DI
		POP	BX
		POP	CX
		POP	DX
		RET
;
;	ERROR  WRAP-UP
;
STIME_ERR:
		MOV	AX,SVC_FAILED+PARM_ERR	;ERROR STATUS
		JMP	SHORT STIME_OVER
X_SET_TIME	ENDP
PAGE
;***************************************************
;*  X _TIME_UPDATE - PROCESS 1 MINUTE UPDATE       *
;***************************************************
;
;	THIS IS THE PROCEDURE WHICH UPDATES THE TIME
;	WHEN THE ONE MINUTE TIME MONITOR BLOCK EXPIRES.
;
X_TIME_UPDATE	PROC	FAR
		MOV	BX,OFFSET X_TOD_TMB		;TIMER BLOCK ADR
		MOV	WORD PTR [BX+TMB_TIME+2],60000	;ONE MINUTE TIME OUT
		MOV	WORD PTR [BX+TMB_TIME],0
		CALL	X_QUEUE_TMB			;GET IT GOING
;
;	UPDATE THE ASCII EQUIVALENT VALUES IN SYSTEM TABLE
;
		MOV	BX,OFFSET X_ASCTOD
		INC	BYTE PTR [BX+TOD_MIN]		;UPDATE MINUTE
		CMP	BYTE PTR [BX+TOD_MIN],60	;REACHED 60 ?
		JNE	UPTIME_RET			;--NO--
;
;	UPDATE HOUR SINCE MINUTE OVERFLOWED
;
		MOV	BYTE PTR [BX+TOD_MIN],0		;MINUTE = 0
		INC	BYTE PTR [BX+TOD_HOUR]		;UPDATE HOUR
		CMP	BYTE PTR [BX+TOD_HOUR],24	;OVERFLOW ?
		JNE	UPTIME_RET			;--NO--
;
;	UPDATE DAY WHEN HOUR OVERFLOWS.
;
		MOV	BYTE PTR [BX+TOD_HOUR],0
		INC	BYTE PTR [BX+TOD_DAY]		;UPDATE DAY
		MOV	AL,[BX+TOD_MON]			;NEED MONTH/DAY TO 
							; FIND IF LEAP YEAR.
		CBW
                MOV     SI,AX
		MOV	DL,CS:MONTH_DAY_TABLE [SI]	;DAYS IN MONTH
		CMP	SI,2				;MONTH = FEBRUARY ?
		JNE	MONTH_UP			;--NO--
		MOV	AX,[BX+TOD_YEAR]		;GET YEAR
		AND	AX,0003H
		JNE	MONTH_UP			;--NOT A LEAP YEAR--
		INC	DL
MONTH_UP:
		CMP	BYTE PTR [BX+TOD_DAY],DL	;PASSED LAST DAY ?
		JLE	UPTIME_RET			;--YES--
;
;	UPDATE MONTH WHEN DAYS-IN-MONTH EXCEEDED.
;
		MOV	BYTE PTR [BX+TOD_DAY],1
		INC	BYTE PTR [BX+TOD_MON]		;UPDATE MONTH
		CMP	BYTE PTR [BX+TOD_MON],13	;MONTHS UP ?
		JNE	UPTIME_RET
;
;	UPDATE YEAR AT END OF DECEMBER.
;
		MOV	BYTE PTR [BX+TOD_MON],1		;START WITH JANUARY
		INC	WORD PTR [BX+TOD_YEAR]
		CMP	WORD PTR [BX+TOD_YEAR],10000	;OVERFLOW ?
		JNE	UPTIME_RET			;--NO--
		MOV	WORD PTR [BX+TOD_YEAR],0	;SET IT TO ZERO !!!!
UPTIME_RET:
		RET
X_TIME_UPDATE	ENDP
PAGE
;****************************************************************
;*         X_STIME:  SET ASCII DATE/TIME SVC                    *
;****************************************************************
;
;	THIS PROCEDURE VALIDATES THE SDB PARAMETERS,
;	SETS THE ASCII DATE/TIME AND RETURNS STATUS TO
;	THE TASK.
;
;	INPUT:	ES:SI=	SDB ADDRESS
;		BX=	TCB ADDRESS
;
X_STIME		PROC	FAR
		MOV	AX,SVC_FAILED+PARM_ERR	;READY TO FAIL...
		CMP	BYTE PTR ES:[SI+FORMT],0	;FORMAT BYTE 0?
		JNE	STIMERR			;NO.
		LES	BX,DWORD PTR ES:[SI+BUFR]	;YES: ADDRESS TIME,
		CALL	X_SET_TIME		;SET IT.
STIMERR:
		CALL	X_SFLAGS
		RET
X_STIME		ENDP
PAGE
;****************************************************************
;*         X_TIME:  READ ASCII DATE/TIME SVC                    *
;****************************************************************
;
;	THIS PROCEDURE VALIDATES THE SDB PARAMETERS,
;	READS THE ASCII DATE/TIME AND RETURNS STATUS
;	TO THE TASK.
;
;	INPUT:	ES:SI=	SDB ADDRESS
;		BX=	TCB ADDRESS
;	OUTPUT:	NONE
;
X_TIME		PROC	FAR
		MOV	AX,SVC_FAILED+PARM_ERR	;READY TO FAIL...
		CMP	BYTE PTR ES:[SI+FORMT],0	;FORMAT BYTE 0?
		JNE	TIMERR			;NO.
		LES	BX,DWORD PTR ES:[SI+BUFR]	;YES: ADDRESS BUFFER,
		CALL	X_READ_TIME		;READ TIME INTO IT.
TIMERR:
		CALL	X_SFLAGS		;RETURN STATUS
		RET
X_TIME		ENDP
MTOS_CS		ENDS
		END
