! File:    EX.TIMER.PLZ.S !
! Updated: 821202 / PH !
! Created: 821201 / PH !
 
!=======
; PROCESS TIMER
; This process, if enabled, decrements a stop time value
; every second until that value reaches zero. It also
; displays a string every second to show it's working.
; The timing is achieved by a message signaled to SEC_SM
; every second.
; The process TIMER initializes itself as disabled and
; must explicitly be enabled by receiving a message at
; TMR_SM. The enabling message is returned to a message-
; pool (POOLSM). Disabling is handled by poling the shared
; variable TIMEEN.
;=======!
 
EX_TIMER module
 
!=======!
! FILE:    RTZ.DECLS.PLZ.S !
! UPDATED: 820120 / PH     !
! REV:     2.00            !
!=======!
 
type
	W_PTR ^word
 
	SEMAPHORE record [
		SEMMGQ W_PTR
		SEMPRQ ^PCB
		SEMSEM ^SEMAPHORE ]
 
	IPCB record [
		IPCNAM array [6 byte]
		IPCENT word
		IPCSTZ word
		IPCPTY byte ]
 
	PCB record [
		PCBLNK ^PCB
		PCBDLY word
		PCBDLK ^PCB
		PCBSEM ^SEMAPHORE
		PCBSP  word
		PCBPTY byte
		PCBSTT byte
		PCBNAM ^IPCB
		PCBRND word
		PCBPCB ^PCB ]
 
external
	P_CPRO procedure (IPCB_PTR ^IPCB)
	P_CSEM procedure (SEMAPHORE_PTR ^SEMAPHORE)
	P_EWAI procedure (SEMAPHORE_PTR ^SEMAPHORE)
		returns (MESSAGE_PTR W_PTR)
	P_LDCW procedure (CONTROL W_PTR)
	P_SEVC procedure (LEVEL word VECTOR W_PTR)
	P_SGNL procedure (SEMAPHORE_PTR ^SEMAPHORE MESSAGE_PTR W_PTR)
	P_SPTY procedure (PCB_PTR ^PCB PRIORITY byte)
	P_WAIT procedure (SEMAPHORE_PTR ^SEMAPHORE TIME_LIMIT word)
		returns (MESSAGE_PTR W_PTR)
 
	Z_RUNP ^PCB
	Z_SLCF,Z_SLCT,Z_SLCP,Z_EOMF byte
 
	Z_00SM,Z_01SM,Z_02SM,Z_03SM SEMAPHORE
	Z_04SM,Z_05SM,Z_06SM,Z_07SM SEMAPHORE
	Z_08SM,Z_09SM,Z_10SM,Z_11SM,Z_12SM,Z_13SM SEMAPHORE
	Z_14SM,Z_15SM,Z_16SM,Z_17SM,Z_18SM,Z_19SM SEMAPHORE
 
constant
	ZRCINT := 0     ! INTERRUPT MESSAGE !
	ZRCMIS := %44   ! MISSED INTERRUPT !
	ZRCTO  := %80   ! TIME-OUT MESSAGE !
 
!-------!
! END OF RTZ DECLARATIONS !
!-------!
 
!=======!
! FILE:    RTZ.DECLS.UTILITY.PLZ.S !
! UPDATED: 820120 / PH             !
! REV:     2.00                    !
!=======!
 
type
	RTZ_CON_MESSAGE record [
		MSGLNK ^word
		MSGRQC byte
		MSGRC  byte
		MSGLEN word
		MSGRSM ^SEMAPHORE
		MSGBAD ^byte
		MSGBSZ word
		MSGBCN word ]
 
	RTZ_CON_MESSAGE_PTR ^RTZ_CON_MESSAGE
 
	RTZ_RTC_MESSAGE record [
		MSGLNK ^word
		MSGRQC byte
		MSGRC  byte
		MSGLEN word
		MSGRSM ^SEMAPHORE
		MSGSEC byte
		MSGMIN byte
		MSGHRS byte ]
 
	RTZ_RTC_MESSAGE_PTR ^RTZ_RTC_MESSAGE
 
	RTZ_MGR_MESSAGE record [
		MSGLNK ^word
		MSGRQC byte
		MSGRC  byte
		MSGLEN word
		MSGRSM ^SEMAPHORE
		MSGWL  word
		MSGSPC W_PTR ]
 
	RTZ_MGR_MESSAGE_PTR ^RTZ_MGR_MESSAGE
 
constant
	ZQCRDC := %61	! GET TIME !
	ZQCWRC := %62	! SET TIME !
	ZQCWAT := %63	! WAKE ME AT ... !
	ZQCWAF := %64	! WAKE ME AFTER ... !
 
	ZQCMCF := %68	! CONDITIONAL, FAST !
	ZQCMCB := %69	! CONDITIONAL, BEST !
	ZQCMUF := %6A	! INCONDITIONAL, FAST !
	ZQCMUB := %6B	! INCONDITIONAL, BEST !
 
	ZQCRDA := 1	! READ ASCII !
	ZQCRDB := 5	! READ BINARY !
	ZQCWRA := 2	! WRITE ASCII !
	ZQCWRB := 6	! WRITE BINARY !
	ZQCRDS := 9	! READ STATUS !
	ZQCWRS := %A	! WRITE STATUS !
 
	ZRCOK  := 0	! NORMAL TERMINATION !
	ZRCUNK := %41	! UNKNOWN REQUEST CODE !
	ZRCERR := %42	! UNSPECIFIED ERROR !
	ZRCBSY := %43	! DEVICE BUSY !
	ZRCOVF := %45	! DATA OVERFLOW !
	ZRCEOF := %46	! END OF FILE !
	ZRCDWN := %47	! DEVICE OFF LINE !
	ZRCEOM := %48	! END OF MEDIA !
 
!-------!
! END OF RTZ-UTILITY DECLARATIONS !
!-------!
!^L!
 
constant
	ON := %FF
	OFF := 0
 
external
	POOLSM,SEC_SM,TMR_SM SEMAPHORE
	TIMEEN byte
	DISPLAY procedure (PTR ^byte)
 
global
	TIMER procedure
	local
		PTR RTZ_RTC_MESSAGE_PTR
		TIMER_COUNT word
	entry
BEGIN: do
		TIMEEN := OFF
	WAIT: do
			PTR := RTZ_RTC_MESSAGE_PTR P_EWAI (#TMR_SM)
			TIMEEN := ON
			TIMER_COUNT := word (PTR^.MSGSEC and %F
				+ (PTR^.MSGSEC/%10) * 10
				+ 60 * (PTR^.MSGMIN and %F + (PTR^.MSGMIN/%10) * 10)
				+ 3600 * (PTR^.MSGHRS and %F + (PTR^.MSGHRS/%10) *  10))
			P_SGNL (#POOLSM,W_PTR PTR)
			do
				if TIMER_COUNT = 0 then repeat from BEGIN fi
				if TIMEEN = OFF then repeat from WAIT fi
				PTR := RTZ_RTC_MESSAGE_PTR P_EWAI (#SEC_SM)
				TIMER_COUNT -= 1
				DISPLAY (#'Timer%R')
			od
		od ! WAIT !
	od ! BEGIN !
	end TIMER
 
end EX_TIMER
