*H RTZ.DBG.BUF
;UPDATED: 821028 / PH
;REV:     2.01
 
;=======
; GETVAL - GET VALUE
;
; ACCEPTS: HL = RETURN ADDRESS IF 1ST CHARACTER IS SPACE
;
; RETURNS: DE = BINARY VALUE
;          EXITS IF UNVALID OR NO ENTRY
;=======
 
	EXTERNAL	Z$IBUF
 
GETVAL
	CALL	INCHAR		;GET 1ST CHAR SEPARATE
	EX	(SP),HL		;ASSUME ' ' OR '^'
	CP	' '
	RET	Z
	CP	'^'
	RET	Z
	CP	0DH
	JP	Z,CMDWAI		;EXIT
	EX	(SP),HL		;RESTORE NORMAL RETURN
	LD	HL,Z$IBUF-1	;SAVE 1ST CHARACTER
	LD	(HL),A
	CALL	INLIN		;GET REST OF INPUT
	CALL	GET16
	JP	NZ,DATERR		;UNVALID ENTRY
	RET
 
;=======
; GET1P - GET EXACTLY ONE PARAMETER
; GET1PC - GET ONE PARAMETER CONDITIONAL
;
; ACCEPTS: HL = ^STRING
;
; RETURNS: DE = BINARY VALUE
;          HL = ^1ST NON-DIGIT
;          Z$P1 = BINARY VALUE
;          TC IF NO PARAMETER
;          EXITS IF UNVALID STRING
;=======
 
	EXTERNAL	Z$P1
 
GET1P
	CALL	GET1PC
	JP	C,PRMERR
	RET
GET1PC
	CALL	PEEL			;PEEL OFF BLANKS
	RET	C
	CALL	GETNUM
	JP	NZ,PRMERR		;INCORRECT TERMINATOR
	LD	(Z$P1),DE
	RET
 
;=======
; GET2P - GET EXACTLY TWO PARAMETERS
; GET2PC - GET TWO PARAMETERS CONDITIONAL
;
; ACCEPTS: HL = ^STRING
;
; RETURNS: DE = 2ND PARAMETER
;          HL = ^1ST NON-DIGIT AFTER 2ND PRM
;          Z$P1 = 1ST PARAMETER
;          Z$P2 = 2ND PARAMETER
;          TC IF LESS THAN 2 PARAMETERS
;          A  = # OF FOUND PARAMETERS
;          EXITS IF UNVALID ENTRY
;=======
 
	EXTERNAL	Z$P2,Z$P1
 
GET2P
	CALL	GET2PC
	JP	C,PRMERR
	RET
GET2PC
	CALL	PEEL			;PEEL OFF BLANKS
	LD	A,0			;NO PARAMETER FOUND
	RET	C
	CALL	GETNUM
	LD	A,0			;NO PARAMETER FOUND
	RET	C
	LD	(Z$P1),DE
	LD	A,(HL)		;DELIMITER MUST BE ','
	CP	0DH			;ONE PARAMETER ?
	JR	Z,GET2PB		;YES
	CP	','
	JP	NZ,PRMERR
	INC	HL
	CALL	PEEL
	CALL	GETNUM
	JP	NZ,PRMERR		;INCORRECT TERMINATOR
	LD	A,2
	JR	NC,GET2PA		;VALUE PRESENT
GET2PB
	LD	A,1			;ONE PARAMETER FOUND
	LD	DE,1			;NO VALUE DEFAULTS TO 1
GET2PA
	LD	(Z$P2),DE
	RET
 
;=======
; GETNUM - SCAN AN ASCII-STRING CONSIDERING SPECIAL
;          CHARACTERS
;
; ACCEPTS: HL = ^STRING
;
; RETURNS: DE = BINARY VALUE
;          HL = ^FIRST NON-HEXDIGIT
;          TZ IF 1ST NON-DIGIT IS <CR>
;          TC IF NO DIGIT IS ENTERED
;
; AFFECTS: A
;=======
 
GETNUM
	LD	A,(HL)
	CP	'@'
	JR	NZ,GETN10
 
; THIS IS AN INDIRECT VALUE
 
	INC	HL
	CALL	GETNUM		;RECURS
	EX	DE,HL		;HL:=^VALUE
	LD	A,(HL)		;HL:=VALUE
	INC	HL
	LD	H,(HL)
	LD	L,A
	EX	DE,HL
	RET
 
GETN10
	CP	'$'
	JR	NZ,GET16		;normal digits
 
; Register content
 
	INC	HL			;A:=first letter
	LD	A,(HL)
	INC	HL			;HL:=^second letter
	PUSH	HL			;save ^string
	CP	'S'			;special treatment for SP
	JR	Z,GETNSP
	CP	'I'			;take second letter if index
	JR	NZ,GETN15
	LD	A,(HL)
GETN15
	LD	HL,REGTBL		;look for register
	LD	BC,REGTSZ
	CPIR
	JP	NZ,PRMERR		;not a register
	CALL	CHKTP
	PUSH	DE
	POP	IX
	LD	L,(IX+PCBSP)
	LD	H,(IX+PCBSP+1)
	ADD	HL,BC
	ADD	HL,BC
	LD	E,(HL)
	INC	HL
	LD	D,(HL)
GETN18
	POP	HL			;HL:=^terminator ?
	INC	HL
	LD	A,(HL)
	XOR	0DH			;CY:=0
	RET
GETNSP
	CALL	CHKTP
	PUSH	DE
	POP	IX
	LD	E,(IX+PCBSP)
	LD	D,(IX+PCBSP+1)
	LD	HL,2*7		;reach user part of stack
	ADD	HL,DE
	EX	DE,HL
	JR	GETN18
 
REGTBL
	DEFM	'PH BDXY'
REGTSZ	equ	$-REGTBL
 
;=======
; GET16 - SCAN AN ASCII-STRING FOR A 4-DIGIT HEX-VALUE
;
; ACCEPTS: HL = ^STRING
;
; RETURNS: DE = BINARY VALUE
;          HL = ^FIRST NON-HEXDIGIT
;          TZ IF 1ST NON-DIGIT IS <CR>
;          TC IF NO DIGIT IS ENTERED
;
; AFFECTS: A
;=======
 
GET16
	PUSH	BC
	EX	DE,HL
	LD	HL,0			;ZERO SO FAR
	LD	B,-1			;NO DIGITS ENTERED
GET16A
	LD	A,(DE)		;A:=NEXT CHARACTER
	CALL	CNVHEX		;HEX ?
	INC	DE			;INC BUFFER POINTER
	INC	B			;KEEP TRACK OF ENTERED DIGITS
	JR	NC,GET16A		;GO ON AS LONG AS DIGITS
	EX	DE,HL
	DEC	HL
	DEC	B
	LD	A,(HL)		;A:=NON-DIGIT
	CP	0DH			;TZ IF <CR>
	LD	A,B
	RLA				;TC IF NO DIGIT
	POP	BC
	RET
 
;=======
; CNVHEX - CONVERT HEX-DIGIT
;
; ACCEPTS: HL = BINARY VALUE
;          A  = CHARACTER TO BE CONVERTED
;
; RETURNS: HL = HL*16+bin(A) IF HEXDIGIT
;          TC IF NON-HEXDIGIT
;
; AFFECTS: A
;=======
 
CNVHEX
	SUB	'0'
	RET	C			;< '0'
	ADD	A,0E9H
	RET	C			;> 'F'
	ADD	A,6
	JP	P,CNVH50		;>= 'A'
	ADD	A,7
	RET	C			;> '9'
CNVH50
	ADD	A,10
	ADD	HL,HL		;HL:=HL*16
	ADD	HL,HL
	ADD	HL,HL
	ADD	HL,HL
	OR	L			;CLEAR CY
	LD	L,A
	RET
 
;=======
; PEEL - PEEL OFF BLANKS
;
; ACCEPTS: HL = ^STRING
;
; RETURNS: HL = ^FIRST NON-BLANK
;          TC IF NO BLANK AT ALL
;
; AFFECTS: A
;=======
 
PEEL
	LD	A,(HL)
	CP	' '
	SCF
	RET	NZ			;NO BLANK AT ALL
PEEL50
	INC	HL
	LD	A,(HL)
	CP	' '
	JR	Z,PEEL50
	AND	A			;CLEAR CY
	RET
 
;=======
; PUT16 - PUT A 16-BIT BINARY VALUE INTO AN ASCII-STRING
;
; ACCEPTS: DE = BINARY VALUE
;          HL = ^STRING
;
; RETURNS: HL = HL+4
;
; AFFECTS: A,F
;=======
 
PUT16
	LD	A,D			;START WITH HIGH-BYTE
	CALL	PUT8
	LD	A,E			;THEN TAKE LOW-BYTE
 
;=======
; PUT8 - PUT A 8-BIT BINARY VALUE INTO AN ASCII-STRING
;
; ACCEPTS: A  = BINARY VALUE
;          HL = ^STRING
;
; RETURNS: HL = HL+2
;
; AFFECTS: A,F
;=======
 
PUT8
	PUSH	AF			;START WITH UPPER NIBBLE
	RRA
	RRA
	RRA
	RRA
	CALL	PUT4
	POP	AF
 
;=======
; PUT4 - PUT A 4-BIT BINARY VALUE INTO AN ASCII-STRING
;
; ACCEPTS: A (LOW) = BINARY VALUE
;          HL      = ^STRING
;
; RETURNS: HL      = HL+1
;
; AFFECTS: A,F
;=======
 
PUT4
	AND	0FH			;CLEAR UPPER NIBBLE
	ADD	A,90H		;convert to ASCII
	DAA
	ADC	A,40H
	DAA
	LD	(HL),A
	INC	HL
	RET
 
;=======
; PUTEOS - PUT <CR>,0FFH INTO A STRING
;
; ACCEPTS: HL = ^STRING
;
; RETURNS: HL = HL+1 (i.e. HL = ^EOS-MARK)
;=======
 
PUTEOS
	LD	(HL),0DH
	INC	HL
	LD	(HL),0FFH
	RET
 
;=======
; PUTSTR - CONCATENATE STRINGS
;
; ACCEPTS: DE = ^<STRING TO BE APPENDED>,<0FFH>
;          HL = ^STRING
;
; RETURNS: DE = ^POSITION AFTER APPENDED STRING (SRC)
;          HL = ^POSITION AFTER APPENDED STRING (DST)
;
; AFFECTS: A,F
;=======
 
PUTSTR
	LD	A,(DE)		;MOVE ONE CHARACTER
	LD	(HL),A
	INC	A			;END OF STRING ?
	RET	Z			;YES, WORK IS DONE
	INC	DE
	INC	HL
	JR	PUTSTR
 
;=======
; OTOBUF - PRINT Z$OBUF
;
; AFFECTS: HL
;=======
 
	EXTERNAL	Z$OBUF
 
OTOBUF
	LD	HL,Z$OBUF
	JP	OUTSTR
 
;=======
; PUT8_ - PUT8 + ' '
; PUT16_ - PUT16 + ' '
;=======
 
PUT8_
	CALL	PUT8
	JR	PUT8_5
PUT16_
	CALL	PUT16
PUT8_5
	LD	(HL),' '
	INC	HL
	RET
 
;=======
; PUTINS - PUT INSTRUCTION (DISSASSEMBLED)
;
; Accepts: DE = ^INSTRUCTION
;          HL = ^STRING
;
; Returns: HL = ^NEXT POS IN STRING
;
; Affects: A,F
;=======
 
PUTINS
	PUSH	BC
	PUSH	DE
	PUSH	IX
	PUSH	IY
	PUSH	DE			;IX:=^INPUT LOCATION
	POP	IX
	PUSH	HL			;IY:=^STRING
	POP	IY
	CALL	DISM
	PUSH	IY			;HL:=^NEXT POS IN STRING
	POP	HL
	LD	(HL),0FFH		;END MARK
	POP	IY
	POP	IX
	POP	DE
	POP	BC
	RET
 
;=======
; PUTNAM - PUT PROCESS NAME AND PCB ADDRESS
;
; ACCEPTS: IX = ^PCB
;          HL = ^STRING
;
; RETURNS: HL = ^POSITION AFTER APPENDED STRING (DST)
;
; AFFECTS: A,F,DE
;=======
 
PUTNAM
	PUSH	BC			;SAVE BC
	LD	E,(IX+PCBNAM)	;DE:=^PROCESS NAME
	LD	D,(IX+PCBNAM+1)
	LD	BC,6			;MOVE NAME TO OUTBUF
	EX	DE,HL
	LDIR
	EX	DE,HL		;MOVE ADDRESS TO OUTBUF
	POP	BC
	LD	(HL),','
	INC	HL
	PUSH	IX
	POP	DE
	CALL	PUT16_
	LD	(HL),0FFH
	RET
