*H RTZ.DBG.SBR
;UPDATED: 821028 / PH
;REV:     2.01
 
;=======
; CHKTP - CHECK IF ANY TP PRESENT
;
; returns: DE = ^TPCB if present
;
; affects: A,F
;=======
 
CHKTP
	LD	DE,(Z$TPCB)
	LD	A,E
	OR	D
	JP	Z,TPERR
	RET
 
;=======
; CHKSEM - CHECK IF SEMAPHORE
; CHKPCB - CHECK IF PROCESS
; CHKQUE - CHECK IF IN QUEUE
;
; ACCEPTS: HL = ^QUEUE-HEAD (CHKQUE ONLY)
;          DE = ^SEARCHED ITEM
;
; RETURNS: TC IF NOT FOUND
;
; AFFECTS: AF,BC,HL
;=======
 
	EXTERNAL	Z$FSEM,Z$FPCB
 
CHKSEM
	LD	BC,4			;OFFSET TO STATIC SEM-LINK
	LD	HL,Z$FSEM		;SEMAPHORE QUEUE-HEAD
	JR	CHKQ10
CHKPCB
	LD	BC,16		;OFFSET TO STATIC PCB-LINK
	LD	HL,Z$FPCB		;PCB QUEUE-HEAD
	JR	CHKQ10
CHKQUE
	LD	BC,0			;NO OFFSET TO LINK
CHKQ10
	LD	A,(HL)		;HL:=^NEXT ITEM
	INC	HL
	LD	H,(HL)
	LD	L,A
	OR	H
	SCF
	RET	Z			;END OF QUEUE, NOT FOUND
	AND	A
	SBC	HL,DE		;THIS IS IT ?
	RET	Z			;YES, READY
	ADD	HL,DE		;RESTORE HL
	ADD	HL,BC		;ADJUST FOR OFFSET TO LINK
	JR	CHKQ10
 
;=======
; RESPIX - Send message back to RESPONSE-SEMAPHORE
;
; ACCEPTS: IX = ^MESSAGE
;
; RETURNS: --
;
; AFFECTS: DE,HL
;=======
 
RESPIX
	PUSH	IX			;HL:=^MESSAGE
	POP	HL
	LD	E,(IX+MSGRSM)
	LD	D,(IX+MSGRSM+1)
	JP	Z_SGNL
*H COMMAND DECODERS
;UPDATED: 811111 / PH
 
;=======
; G_XCMD - SHORTHAND COMMAND DECODER
;
; ACCEPTS: DE = ^TABLE OF COMMANDS
;          HL = ^COMMAND STRING
;
; RETURNS: HL = ^DELIMITER IN GIVEN COMMAND IF MATCH
;          DE = RETURN PARAMETER IF MATCH
;               0 IF NOT FOUND
;               1 IF AMBIGOUS COMMAND
;          FZ IF OK
;
; AFFECTS: A
;
; Table organisation:
;
; LENGTH byte STRING array [* byte] PARAMETER word
; 0 = end of table
;
; example:      DEFB 5
;               DEFM 'FIRST'
;               DEFW 0101H
;               DEFB 6
;               DEFM 'SECOND'
;               DEFW 0202H
;               DEFB 0
;=======
 
G_XCMD
	PUSH	BC
	PUSH	IX
	PUSH	IY
	LD	IX,0			;CLEAR RETURN PARAMETER
DSH000
	LD	A,(DE)		;B:=COMMAND.LEN
	LD	B,A
	INC	DE			;DE:=^COMMAND.STRING
	OR	A			;END OF TABLE ?
	JR	Z,DSH900		;YES
	PUSH	HL			;SAVE ^STRING
DSH050
	LD	A,(DE)		;MATCH ?
	CP	(HL)
	JR	NZ,DSH200		;NO
	INC	DE			;YES, NEXT CHARACTER
	INC	HL
	LD	A,(HL)		;NEXT CHARACTER IS DELIMITER ?
	CALL	DELIM?
	JR	Z,DSH100		;YES, COMPLETE MATCH
	DJNZ	DSH050		;GO ON
	INC	DE			;NO MATCH, SKIP TO NEXT COMMAND
	INC	DE			;  SKIP THIS RETURN PARAMETER
	POP	HL			;RESTORE ^STRING TO BE DECODED
	JR	DSH000
DSH100
	PUSH	HL			;IY:=^DELIMITER
	POP	IY
	PUSH	IX			;HL:=RETURN PARAMETER
	POP	HL
	LD	A,L			;RETURN PARAMETER ALREADY SET ?
	OR	H
	LD	IX,1
	JR	NZ,DSH905		;YES, TERMINATE (AMBIGOUS COMMAND)
	LD	L,B			;SET RETURN PARAMETER
	LD	H,0
	DEC	L
	ADD	HL,DE
	LD	E,(HL)
	INC	HL
	LD	D,(HL)
	INC	HL
	PUSH	DE			;IX:=RETURN PARAMETER
	POP	IX
	JR	DSH250
DSH200
	LD	L,B			;SKIP THIS COMMAND AND TRY NEXT
	LD	H,0			;HL:=CHARACTERS LEFT IN COMMAND
	INC	L			;SKIP THIS RETURN PARAMETER
	INC	L
	ADD	HL,DE		;DE:=^NEXT COMMAND
DSH250
	EX	DE,HL
	POP	HL			;RESTORE ^STRING TO BE DECODED
	JR	DSH000
DSH905
	POP	HL
DSH900
	PUSH	IX			;DE:=RETURN PARAMETER
	POP	DE
	PUSH	IY			;HL:=^DELIMITER
	POP	HL
	POP	IY			;RESTORE REGISTERS
	POP	IX
	POP	BC
	LD	A,E			;UPDATE ZERO-FLAG
	AND	0FEH
	OR	D
	RET
 
;=======
; G_CCMD - COMPLETE COMMAND DECODER
;
; ACCEPTS: DE = ^TABLE OF COMMANDS
;          HL = ^COMMAND STRING
;
; RETURNS: HL = ^DELIMITER IN GIVEN COMMAND IF MATCH
;          DE = RETURN PARAMETER IF MATCH
;               0 IF NOT FOUND
;          FZ IF OK
;
; AFFECTS: A
;
; Table organisation:
;
; LENGTH byte STRING array [* byte] PARAMETER word
; 0 = end of table
;
; example:      DEFB 5
;               DEFM 'FIRST'
;               DEFW 0101H
;               DEFB 6
;               DEFM 'SECOND'
;               DEFW 0202H
;               DEFB 0
;=======
 
G_CCMD
	PUSH	BC
DSC000
	LD	A,(DE)		;GET COMMAND.LEN
	INC	DE			;DE:=^COMMAND.STRING
	LD	BC,0			;ASSUME 0 RETURN
	OR	A			;END OF COMMAND TABLE ?
	JR	Z,DSC900		;YES
	PUSH	HL			;SAVE ^STRING
	LD	B,A
DSC050
	LD	A,(DE)		;MATCH SO FAR ?
	CP	(HL)
	JR	NZ,DSC200		;NO, EXAMINE NEXT COMMAND
	INC	DE			;YES, NEXT CHARACTER
	INC	HL
	DJNZ	DSC050		;GO ON IF CHARACTERS LEFT
	LD	A,(HL)		;COMPLETE MATCH, CHECK FOR --
	CALL	DELIM?		;  DELIMITER
	EX	DE,HL		;GET RETURN PARAMETER
	LD	C,(HL)
	INC	HL
	LD	B,(HL)
	INC	HL
	EX	DE,HL
	JR	Z,DSC150		;DELIMITER OK
	POP	HL
	JR	DSC000		;TOO LONG STRING, TEST NEXT
DSC150
	POP	DE
DSC900
	LD	E,C			;DE:=RETURN PARAMETER
	LD	D,B
	POP	BC
	LD	A,E			;SET FLAGS
	OR	D
	RET
DSC200
	LD	L,B			;SKIP THIS COMMAND AND TRY NEXT
	LD	H,0			;HL:=CHARACTERS LEFT IN COMMAND
	INC	L			;COMPENSATE FOR RETURN PARAMETER
	INC	L
	ADD	HL,DE		;DE:=^NEXT COMMAND
	EX	DE,HL
	POP	HL
	JR	DSC000
 
;=======
; DELIM? - CHECK IF DELIMITER
;
; ACCEPTS: HL = ^CHARACTER
;
; RETURNS: TZ IF <CR>,'/' OR ' '
;=======
 
DELIM?
	CP	0DH
	RET	Z
	CP	'/'
	RET	Z
	CP	' '
	RET
*H GEN.TABLE
;UPDATED: 800507 / PH
 
;=======
;
;       TABLE HANDLING
;       ==============
;
; A TABLE in this module holds 16-bits items
; proceeded by one byte giving the number of items
; currently in the table.
; The first byte should be initialized to zero !
;
; example:
;
; ATABLE: DEFS	1
;         DEFS	14*2
;
; This table can hold 14 items.
;=======
 
;=======
; CAPTBL - CONDITIONAL APPEND TO TABLE
;
; ACCEPTS: HL = ^TABLE
;          DE = ITEM TO APPEND
;          A  = MAXIMUM # OF ITEMS ALLOWED
;
; RETURNS: FC IF SUCCESSFUL APPEND
;          TC IF TABLE FULL
;
; AFFECTS: A,B,C,HL
;=======
 
CAPTBL
	DEC	A
	CP	(HL)			;FULL ?
	RET	C			;YES
 
;=======
; APTBL - APPEND TO TABLE
;
; ACCEPTS: HL = ^TABLE
;          DE = ITEM TO APPEND
;
; AFFECTS: B,C,HL,F
;=======
 
APTBL
	LD	C,(HL)		;C:=# OF ITEMS IN TABLE
	INC	(HL)			;ONE ITEM MORE
	LD	B,0			;BC:=# OF ITEMS IN TABLE
	INC	HL			;HL:=^FIRST ITEM
	ADD	HL,BC		;HL:=^FIRST EMPTY POSITION
	ADD	HL,BC
	LD	(HL),E		;APPEND NEW ITEM (LOW)
	INC	HL
	LD	(HL),D		;APPEND NEW ITEM (HIGHBYTE)
	RET
 
;=======
; RMTBL - REMOVE FROM TABLE
;
; ACCEPTS: HL = ^TABLE
;          DE = ITEM TO BE REMOVED
;
; RETURNS: FC IF REMOVAL
;          TC IF NO REMOVAL
;
; AFFECTS: A,B,C,DE,HL
;=======
 
RMTBL
	PUSH	HL			;SAVE ^TABLE-HEAD
	CALL	FDTBL		;IF "DE" NOT EXISTS IN TBL
	POP	DE			;DE:=^TABLE-HEAD
	RET	C			;  THEN NO REMOVAL
	EX	DE,HL		;DE:=^ITEM TO BE REMOVED
	DEC	(HL)			;ONE ITEM LESS
	LD	L,E
	LD	H,D
	INC	HL			;HL:=^NEXT ITEM IN TBL
	INC	HL
	AND	A			;TRUE REMOVAL
	DEC	B			;B:=# OF ITEMS LEFT IN TABLE
	RET	Z			;RET IF LAST ITEM
	LD	C,B			;BC:=# OF BYTES LEFT IN TABLE
	LD	B,0
	SLA	C
	RL	B
	LDIR				;COMPRESS TBL
	RET
 
;=======
; FDTBL - FIND IN TABLE
;
; ACCEPTS: HL = ^TABLE
;          DE = ITEM LOOKED FOR
;
; RETURNS: B  = # OF ITEMS LEFT (INCLUDING HL^)
;          FC IF MATCH
;            HL = ^MATCHING ITEM
;          TC IF NO MATCH
;            HL = ^FIRST EMPTY POSITION
;
; AFFECTS: A,HL
;=======
 
FDTBL
	LD	B,(HL)		;B:=# OF ITEMS IN TABLE
	INC	B
FDTBL1
	INC	HL			;HL:=^CURRENT ITEM (LOWBYTE)
	DEC	B			;END OF TABLE ?
	SCF
	RET	Z			;YES, RETURN (CY=1)
	LD	A,(HL)		;A:=CURRENT LOWBYTE
	CP	E			;PARTIAL MATCH ?
	INC	HL			;HL:=^CURRENT HIGHBYTE
	JR	NZ,FDTBL1		;NO
	LD	A,(HL)		;YES, TEST HIGHBYTE
	CP	D			;MATCH ?
	JR	NZ,FDTBL1		;NO
	DEC	HL			;HL:=^MATCHING LOWBYTE
	AND	A			;YES, --
	RET				;  RETURN WITH CY=0
*H GEN.QUEUE
;UPDATED: 810901  / PH
 
;=======
; G_ENQ - APPEND TO QUEUE
;
; ACCEPTS: HL = ^QUEUE-HEAD
;          DE = ^ITEM TO BE APPENDED
;
; AFFECTS: AF,HL
;=======
 
G_ENQ
	LD	A,(HL)
	INC	HL
	OR	(HL)			;LAST ITEM ?
	JR	Z,ENQ50		;YES, HL -> 0
	LD	A,(HL)		;NO, HL:=^NEXT ITEM
	DEC	HL
	LD	L,(HL)
	LD	H,A
	JR	G_ENQ
ENQ50
	LD	(HL),D		;FIX HL -> DE
	DEC	HL
	LD	(HL),E
	LD	(DE),A		;FIX DE -> 0
	INC	DE
	LD	(DE),A
	DEC	DE
	RET
 
;=======
; G_DEQ - REMOVE FIRST ITEM OF QUEUE
;
; ACCEPTS: HL = ^QUEUE-HEAD
;
; RETURNS: HL = ^REMOVED ITEM IF NOT EMPTY
;          TC IF QUEUE WAS EMPTY
;
; AFFECTS: AF,HL
;=======
 
G_DEQ
	LD	A,(HL)		;CHECK IF EMPTY
	INC	HL
	OR	(HL)
	SCF				;ASSUME EMPTY
	RET	Z
	PUSH	DE
	LD	D,(HL)		;DE:=^1ST ITEM
	DEC	HL
	LD	E,(HL)
	LD	A,(DE)		;FIX HL -> 2ND
	LD	(HL),A
	INC	DE
	INC	HL
	LD	A,(DE)
	LD	(HL),A
	DEC	DE
	EX	DE,HL
	POP	DE
	AND	A			;CLEAR CY
	RET
 
;=======
; G_REMQ - REMOVE OUT OF QUEUE
;
; ACCEPTS: HL = ^QUEUE-HEAD
;          DE = ^ITEM TO BE REMOVED
;
; RETURNS: TC IF NOT FOUND
;
; AFFECTS: AF,HL
;=======
 
G_REMQ
	PUSH	BC
	LD	C,E			;BC:=^SEARCHED ITEM
	LD	B,D
REMQ10
	LD	E,(HL)		;SET UP "HL -> DE ->"
	INC	HL
	LD	D,(HL)
	EX	DE,HL		;DE -> HL ->
	LD	A,L
	OR	H
	SCF
	JR	Z,REMQ50		;END OF QUEUE, NOT FOUND
	AND	A
	SBC	HL,BC		;ITEM FOUND ?
	ADD	HL,BC		;HL:=^NEXT ITEM TO BE CHECKED
	JR	NZ,REMQ10
	INC	BC			;DE -> BC -> xx
	LD	A,(BC)		;FIX DE -> xx
	LD	(DE),A
	DEC	BC
	DEC	DE
	LD	A,(BC)
	LD	(DE),A
REMQ50
	LD	E,C
	LD	D,B
	POP	BC
	RET
*H GEN.SWITCH
;UPDATED: 810824 / PH
 
;=======
; SWITCH - DECODE COMMAND SWITCHES
;
; ACCEPTS: HL = ^'/' IN COMMAND STRING
;
; RETURNS: BCDE = BITPATTERN ACCORDING TO SWITCHES
;          HL = ^FIRST NON-SWITCH (' ' OR CR) IF '/'
;               IS FOUND, ELSE UNAFFECTED
;
; AFFECTS: A,F
;=======
 
SWITCH
	XOR	A
	LD	E,A			;BCDE:=0
	LD	D,A
	LD	C,A
	LD	B,A
	LD	A,(HL)		;MUST START WITH A '/'
	CP	'/'
	RET	NZ
	PUSH	HL
SWIT03
	POP	HL
SWIT05
	INC	HL			;NEXT CHARACTER
	LD	A,(HL)
	CP	' '			;READY IF ' ' OR CR
	RET	Z
	CP	0DH
	RET	Z
	SUB	'A'			;< 'A'
	JR	C,SWIT05		;YES, NEXT CHARACTER
	CP	'Z'-'A'+1		;> 'Z'
	JR	NC,SWIT05
	PUSH	HL
	LD	L,33			;32 BITS TO ROTATE
SWIT10
	RR	B
	RR	C
	RR	D
	RR	E
	DEC	L			;KEEP TRACK OF ROTATIONS
	JR	Z,SWIT03		;READY
	SUB	1			;affect CY
	JR	SWIT10
