	title	'RTZ.CON.HW.0.MCZ'
;Updated: 820429 / PH
 
;=======
; HARDWARE MODULE FOR RTZ CONSOLE # 0
; -----------------------------------
;
; Device: 8251 on MCB-card
;
; Receiver: Z_02SM, CTC2 (port 0D6H) on MCB-card
;           9600 Baud
;
; Transmitter: Z_03SM, CTC2 (port 0D6H) on MCB-card
;              9600 Baud
;
; Note: Transmitter works with time-outs, receiver
;       with a polling timer-interrupt.
;       Z_01SM used as ^C-semaphore.
;=======
 
;FILE:    EQU.CTC
	unlist
;UPDATED: 800513 / PH
 
;=======
; CTC EQUATES
;
; SOME USEFUL EQUATES CONCERNING CTC-INITIALIZATION
; USAGE:  LD  A,C_CTRL+C_IE+C_TIM+C_P16+C_TC
;         OUT (CTC2),A
;=======
 
C_IE		EQU	80H		;INTERRUPTS ENABLED
C_ID  	EQU	0		;INTERRUPTS DISABLED
C_CNT	EQU	40H		;COUNTER-MODE
C_TIM	EQU	0		;TIMER-MODE
C_P256	EQU	20H		;PRESCALER 256  (TIMER MODE ONLY)
C_P16	EQU	0		;PRESCALER  16  (TIMER MODE ONLY)
C_POS	EQU	10H		;POSITIVE EDGE
C_NEG	EQU	0		;NEGATIVE EDGE
C_XTRG	EQU	8		;EXTERNAL TRIGGER (TIMER)
C_TC		EQU	4		;TIME-CONSTANT WILL FOLLOW
C_NOTC	EQU	0		;NO TIME-CONSTANT WILL FOLLOW
C_STOP	EQU	2		;STOP COUNTING
C_CTRL	EQU	1		;CONTROL-BYTE
 
	list
 
;-------
; DEFINITIONS
;-------
 
	GLOBAL	Z_C0RZ,Z_C0GS
	EXTERNAL	Z_02SM
 
Z_C0RZ	DEFW	30		;RINGBUFFER SIZE
Z_C0GS	DEFW	Z_02SM	;RECEIVER INTERRUPT SEMAPHORE
TTYDAT	EQU	0DEH
TTYSTT	EQU	0DFH
	title	'INIT'
 
;=======
; Z_INI0 - USER INITIALIZATION
;=======
 
	GLOBAL	Z_INI0
	EXTERNAL	Z_LDCW,Z_SEVC,Z_01SM
 
Z_INI0
	LD	A,10B		;FORCE 8251 TO COMMAND
	OUT	(TTYSTT),A
	LD	A,40H		;INTERNAL RESET
	OUT	(TTYSTT),A
	LD	A,0CEH		;MODE INSTRUCTION
	OUT	(TTYSTT),A
	LD	A,37H		;COMMAND INSTRUCTION
	OUT	(TTYSTT),A
	IN	A,(TTYDAT)	;CLEAR BUFFER
 
	LD	DE,2			;SET UP DIRECT INTERRUPT
	LD	HL,URTINT
	CALL	Z_SEVC
	LD	DE,CTC2CW
	CALL	Z_LDCW
	RET
 
CTC2CW
	DEFB	0D6H
	DEFB	2
	DEFB	C_CTRL+C_IE+C_TIM+C_TC
	DEFB	0FFH
	title	'TRANSMITTER / RECEIVER'
 
;=======
; Z_SCH0 - SET CHARACTER
;
; ACCEPTS: A  = ASCII CHARACTER
;=======
 
	GLOBAL	Z_SCH0
	EXTERNAL	Z_WAIT,Z_03SM
 
Z_SCH0
	OUT	(TTYDAT),A
	LD	DE,Z_03SM		;WAIT FOR TX-BUFFER EMPTY
	LD	HL,1
	CALL	Z_WAIT
	RET	
	
;=======
; URTINT - UART INTERRUPT
;=======
 
	EXTERNAL	Z_ISGL,Z_GCI0
	GLOBAL	Z_DWAS
 
URTINT
	PUSH	AF
	IN	A,(TTYSTT)	;READ 8251 STATUS
	BIT	1,A			;RxRDY ?
	JR	Z,URTTX		;NO, DO NOTHING
URTREC
	IN	A,(TTYDAT)	;YES, RECEIVE
	AND	7FH
	CP	3
	JR	Z,CTRC		;HANDLE CONTROL-C
	CALL	Z_GCI0
	POP	AF
	CALL	Z_ISGL
	DEFW	Z_02SM
	DEFB	2
URTTX
	POP	AF
	EI
	RETI
CTRC
	POP	AF
	CALL	Z_ISGL
Z_DWAS
	DEFW	Z_01SM
	DEFB	1
 
	end
