*H RTZ.CON.HW.0.SIO
;UPDATED:  820115 / PH
;REV:      2.00
 
;=======
; HARDWARE MODULE FOR RTZ CONSOLE # 0
; -----------------------------------
;
; Device: SIO channel A (Metric Card)
;
; Receiver: Z_10SM (I-level 10), 4800 Baud
;
; Transmitter: Z_08SM (I-level 8), 4800 Baud
;
; Note: Z_01SM used as ^C-semaphore.
;=======
 
*I EQU.CTC
 
;-------
; DEFINITIONS
;-------
 
	GLOBAL	Z_C0RZ,Z_C0GS
	EXTERNAL	Z_10SM
 
Z_C0RZ	DEFW	30		;SIZE OF TYPE-AHEAD BUFFER
Z_C0GS	DEFW	Z_10SM	;RECEIVE INTERRUPT SEMAPHORE
SIOAD 	EQU	0C0H
SIOAC 	EQU	0C1H
SIOBC 	EQU	0C3H
BAUDGN	EQU	0C4H
*H INIT
 
;=======
; Z_INI0 - USER INITIALIZATION
;=======
 
	GLOBAL	Z_INI0
	EXTERNAL	Z_LDCW,Z_SEVC,Z_00VC
 
Z_INI0
	LD	HL,SIORXR
	LD	(Z_00VC+18),HL
	LD	(Z_00VC+22),HL
 
	LD	A,C_ID+C_TIM+C_P16+C_TC+C_CTRL
	OUT	(BAUDGN),A
	LD	A,1
	OUT	(BAUDGN),A
	LD	DE,8
	LD	HL,SIOTXR
	CALL	Z_SEVC
	LD	DE,10
	LD	HL,SIORXR
	CALL	Z_SEVC
	LD	DE,SIOACW
	CALL	Z_LDCW
	LD	DE,SIOBCW
	CALL	Z_LDCW
	RET
 
SIOACW
	DEFB	SIOAC
	DEFB	9
	DEFB	00011000B		;RESET A
	DEFB	4			;^4
	DEFB	01001100B		;NO PC, 2 STOP
	DEFB	3			;^3
	DEFB	01000001B		;7 BITS, B0 = RX
	DEFB	5			;^5
	DEFB	10101010B		;RTS=1, TX
	DEFB	1			;^1
	DEFB	01011110B		;
 
SIOBCW
	DEFB	SIOBC
	DEFB	4
	DEFB	2			;^2
	DEFB	16			;INTERRUPT VECTOR
	DEFB	1			;^1
	DEFB	100B			;STATUS AFFECTS VECTOR
*H TRANSMITTER
 
;=======
; Z_SCH0 - SET CHARACTER
;
; ACCEPTS: A  = ASCII CHARACTER
;=======
 
	GLOBAL	Z_SCH0
	EXTERNAL	Z_WAIT,Z_08SM
 
Z_SCH0
	PUSH	AF
	LD	A,1			;^1
	DI
	OUT	(SIOAC),A
	LD	A,01011010B	;ENABLE TX-RUPT
	EI
	OUT	(SIOAC),A
	POP	AF
	OUT	(SIOAD),A
	LD	DE,Z_08SM		;WAIT FOR TX-BUFFER EMPTY
	LD	HL,4
	CALL	Z_WAIT
	RET	
 
;=======
; SIOTXR
;=======
 
	EXTERNAL	Z_ISGL
 
SIOTXR
	PUSH	AF
	LD	A,1			;^1
	OUT	(SIOAC),A
	LD	A,01011000B	;DISABLE TX-RUPT
	OUT	(SIOAC),A
	POP	AF
	CALL	Z_ISGL
	DEFW	Z_08SM
	DEFB	8
*H RECEIVER
 
;=======
; SIORXR
;=======
 
	EXTERNAL	Z_GCI0,Z_ISGL,Z_01SM
	GLOBAL	Z_DWAS
 
SIORXR
	PUSH	AF
	IN	A,(SIOAD)
	AND	7FH			;TAKE CARE OF ^C
	CP	3
	JR	Z,CTRC
	CALL	Z_GCI0
	POP	AF
	CALL	Z_ISGL
	DEFW	Z_10SM
	DEFB	10
CTRC
	POP	AF
	CALL	Z_ISGL
Z_DWAS
	DEFW	Z_01SM
	DEFB	1
 
	END
