RTZSIZ	equ	32
	title	'RTZ.INIT'
;Updated: 840207 / PH
;Created: <820120 / Peter Holmgren, DIGITAILOR AB
 
;=======
; RTZ - A REAL-TIME EXECUTIVE FOR Z-80 APPLICATIONS
;
; COPYRIGHT: DIGITAILOR AB, SWEDEN
;
; VER. 2.00: Released 8111xx
;      2.10: Released 8402xx
;=======
 
;FILE:    RTZ.EQU.S
;UPDATED: 820120 / PH
 
;=======
; ALL INTERNAL EQUATES FOR RTZ/2
;=======
 
;-------
; Initial Process Control Block
;-------
 
IPCNAM	EQU	0		;PROCESS NAME
IPCENT	EQU	6		;PROCESS ENTRY
IPCSTZ	EQU	8		;STACK SIZE
IPCPTY	EQU	10		;PROCESS PRIORITY
 
;-------
; Initial Semaphore
;-------
 
ISMADR	EQU	0		;SEMAPHORE ADDRESS
 
;-------
; Configuration Table - Z$CRTB
;-------
 
CNFIPT	EQU	0		;^INITIAL PROCESS TABLE
CNFNOP	EQU	2		;# OF IPCB:S IN IPT
CNFIST	EQU	3		;^INITIAL  SEMAPHORE TABLE
CNFNOS	EQU	5		;# OF ISEM:S IN IST
 
;-------
; Process Control Block
;-------
 
PCBLNK	EQU	0		;LINK FIELD
PCBDLY	EQU	2		;DELAY TIME
PCBDLK	EQU	4		;DELAY LINK FIELD
PCBSEM	EQU	6		;SEMAPHORE
PCBSP 	EQU	8		;STACK POINTER
PCBPTY	EQU	10		;PROCESS PRIORITY
PCBSTT	EQU	11		;STATUS BITS
PCBNAM	EQU	12		;^NAME
PCBRND	EQU	14		;RANDOM FIELD
PCBPCB	EQU	16		;STATIC PCB-LINK
 
;-------
; Semaphore
;-------
 
SEMMGQ	EQU	0		;MESSAGE QUEUE
SEMPRQ	EQU	2		;PROCESS (WAITING) QUEUE
SEMSEM	EQU	4		;STATIC SEMAPHORE-LINK
 
SEMIMG	EQU	6		;INTERRUPT MESSAGE
 
;-------
; Minimum Message
;-------
 
MSGLNK	EQU	0		;QUEUE LINK
MSGRQC	EQU	2		;REQUEST CODE
MSGRC 	EQU	3		;RETURN CODE
MSGLEN	EQU	4		;LENGTH
MSGRSM	EQU	6		;RESPONSE SEMAPHORE
 
;-------
; Device Request Message
;-------
 
MSGBAD	EQU	8		;BUFFER ADDRESS
MSGBSZ	EQU	10		;BUFFER SIZE
MSGBCN	EQU	12		;BYTE COUNT
 
MSGSEC	EQU	8		;SECONDS
MSGMIN	EQU	9		;MINUTES
MSGHRS	EQU	10		;HOURS
 
MSGWL 	EQU	8		;WANTED LENGTH
MSGSPC	EQU	10		;MAX SPACE OR ^FREE SPACE
 
;-------
; Device Request Bits
;-------
 
ZQBRD 	EQU	1		;00.. ..01 READ
;ZQbRD 	EQU	0
ZQBWRT	EQU	2		;00.. ..10 WRITE
;ZQbWRT	EQU	1
ZQBASC	EQU	0		;00.. .0.. ASCII
ZQBBIN	EQU	4		;00.. .1.. BINARY
;ZQbBIN	EQU	2
ZQBSTT	EQU	8		;00.. 1... STATUS
;ZQbSTT	EQU	4
 
;-------
; Device Request Codes
;-------
 
      						;00xx xxxx COMMON CODES
ZQCRDA	EQU	ZQBRD+ZQBASC		;READ ASCII
ZQCRDB	EQU	ZQBRD+ZQBBIN		;READ BINARY
ZQCWRA	EQU	ZQBWRT+ZQBASC		;WRITE ASCII
ZQCWRB	EQU	ZQBWRT+ZQBBIN		;WRITE BINARY
ZQCRDS	EQU	ZQBRD+ZQBSTT		;READ STATUS
ZQCWRS	EQU	ZQBWRT+ZQBSTT		;WRITE STATUS
 
      				;01xx xxxx COMMAND CODES
ZQCRDC	EQU	61H		;READ CLOCK
ZQCWRC	EQU	62H		;WRITE CLOCK
ZQCWAT	EQU	63H		;WAKE AT
ZQCWAF	EQU	64H		;WAKE AFTER
 
ZQCMCF	EQU	68H		;CONDITIONAL, FAST
ZQCMCB	EQU	69H		;CONDITIONAL, BEST
ZQCMUF	EQU	6AH		;UNCONDITIONAL, FAST
ZQCMUB	EQU	6BH		;UNCONDITIONAL, BEST
 
;-------
; System Return Codes
;-------
 
ZRCINT	EQU	0		;INTERRUPT MESSAGE
ZRCMIS	EQU	44H		;MISSED INTERRUPT
ZRCTO 	EQU	80H		;1... .... TIME-OUT
 
;-------
; Device Return Codes
;-------
 
ZRCOK 	EQU	0		;NORMAL TERMINATION
ZRCUNK	EQU	41H		;UNKNOWN REQUEST CODE
ZRCERR	EQU	42H		;UNSPECIFIED ERROR
ZRCBSY	EQU	43H		;DEVICE BUSY
ZRCOVF	EQU	45H		;DATA OVERFLOW
ZRCEOF	EQU	46H		;END OF FILE
ZRCDWN	EQU	47H		;DEVICE OFF LINE
ZRCEOM	EQU	48H		;END OF MEDIA
 
;-------
; HELP EQUATES
;-------
 
STTDLY	EQU	3		;BIT 3 IN PCBSTT
STCKAF	EQU	8		;OFFSET TO AF IN STACK
STCKHL	EQU	10		;OFFSET TO HL IN STACK
STCKPC	EQU	12		;OFFSET TO PC IN STACK
SZISTK	EQU	50		;SIZE OF IDLE STACK
 
;FILE:    EQU.CTC
;UPDATED: 800513 / PH
 
;=======
; CTC EQUATES
;
; SOME USEFUL EQUATES CONCERNING CTC-INITIALIZATION
; USAGE:  LD  A,C_CTRL+C_IE+C_TIM+C_P16+C_TC
;         OUT (CTC2),A
;=======
 
C_IE		EQU	80H		;INTERRUPTS ENABLED
C_ID  	EQU	0		;INTERRUPTS DISABLED
C_CNT	EQU	40H		;COUNTER-MODE
C_TIM	EQU	0		;TIMER-MODE
C_P256	EQU	20H		;PRESCALER 256  (TIMER MODE ONLY)
C_P16	EQU	0		;PRESCALER  16  (TIMER MODE ONLY)
C_POS	EQU	10H		;POSITIVE EDGE
C_NEG	EQU	0		;NEGATIVE EDGE
C_XTRG	EQU	8		;EXTERNAL TRIGGER (TIMER)
C_TC		EQU	4		;TIME-CONSTANT WILL FOLLOW
C_NOTC	EQU	0		;NO TIME-CONSTANT WILL FOLLOW
C_STOP	EQU	2		;STOP COUNTING
C_CTRL	EQU	1		;CONTROL-BYTE
 
 
	GLOBAL	Z_RTZB
	EXTERNAL	Z_CRTB,Z_CTC0,Z_TICK,Z_SYLV
	eject
 
Z_RTZB
	DI				;ENTRY-POINT FOR RTZ
	JR	INIT03
	text	'RTZ 2.10'
 
; CLEAR RTZ VARIABLES
 
INIT03
	LD	HL,Z_00VC
	LD	DE,Z_00VC+1
	LD	BC,71H+2*RTZSIZ;SIZE OF RAM !!
	LD	(HL),0
	LDIR
 
; BUILD EXECUTION-QUEUE
 
	LD	HL,Z$FPCB-PCBPCB
	LD	(Z$LPCB),HL	;INITIATE PCB-LIST
	LD	HL,Z$IRAM		;LINK IDLE PROC. TO Z$XQUE
	LD	(Z_HEAP),HL
	LD	IX,IDLIPC		;IX:=^IPCB IDLE
	LD	SP,Z_HEAP+14
	CALL	INIPCB
	LD	(Z$XQUE),BC	;IDLE TO XQUE
 
	LD	HL,Z_HEAP+2	;INITIATE FREE SPACE POINTER
	LD	(Z_HEAP),HL
 
; SET UP PROCESS CONTROL BLOCK
 
	LD	SP,Z$IRAM+18+14
	LD	IX,(Z_CRTB+CNFIPT);IX:=^IPT.1ST (=^FIRST IPCB)
	LD	A,(Z_CRTB+CNFNOP);A:=# OF PROCESSES
INIT05
	PUSH	AF
	PUSH	IX
	CALL	INIPCB
	CALL	Z$TOXQ		;LINK BC^ TO XQUE
	POP	IX
	POP	AF
	LD	DE,11		;NO
	ADD	IX,DE		;IX:=^NEW ITEM IN IPT
	DEC	A
	JR	NZ,INIT05
 
; SET UP SEMAPHORE LINK
 
	LD	HL,Z$FSEM-SEMSEM
	LD	(Z$LSEM),HL	;INITIATE SEM-LIST
	LD	IX,(Z_CRTB+CNFIST);IX:=^IST.1ST
	LD	A,(Z_CRTB+CNFNOS);C:=# OF SEMAPHORES
	LD	C,A
INIT10
	LD	L,(IX+0)		;HL:=^SEM
	LD	H,(IX+1)
	CALL	INISEM
	INC	IX
	INC	IX
	DEC	C
	JR	NZ,INIT10
 
;INITIATE RTZ VARIABLES
 
	LD	A,8
	LD	(Z_SLCT),A	;TIME
	LD	A,200
	LD	(Z_SLCP),A	;PROCESS PRIORITY FOR SLICING
 
; FIX INTERRUPT SEMAPHORES
 
	LD	HL,ISEMTB		;HL:=^TABLE OF I-SEMAPHORES
	LD	A,ISEMCT		;A:=# OF I-SEMAPHORES
	LD	BC,SEMIMG		;BC:=OFFSET TO I-MESSAGE
INIT40
	LD	E,(HL)		;DE:=^SEMAPHORE
	INC	HL
	LD	D,(HL)
	INC	HL
	EX	DE,HL		;HL:=^SEMAPHORE
	ADD	HL,BC		;HL:=^SEM.IMG
	LD	(HL),0		;CLEAR SEM.IMG.LNK
	INC	HL
	LD	(HL),0
	INC	HL
	INC	HL
	LD	(HL),ZRCINT	;SEM.IMG.RC
	INC	HL
	LD	(HL),6		;SEM.IMG.LEN
	INC	HL
	LD	(HL),0
	EX	DE,HL
	DEC	A
	JR	NZ,INIT40
 
; FIX INTERRUPT VECTORS
 
	IM	2
	LD	HL,R00RPT		;SET UP SYSTEM INTERRUPTS
	LD	(Z_00VC),HL
	LD	BC,RPTSIZ		;SIZE OF SYSTEM I-HANDLER
	LD	DE,R01RPT
	LD	HL,Z_00VC+2
	LD	A,RTZSIZ-1
INIT50
	LD	(HL),E
	INC	HL
	LD	(HL),D
	INC	HL
	EX	DE,HL
	ADD	HL,BC
	EX	DE,HL
	DEC	A
	JR	NZ,INIT50
	LD	HL,Z_00VC		;INITIALIZE ^RUPT VECTORS
	LD	A,H			;  A:=HIGHBYTE OF Z_00VC
	LD	I,A
	LD	A,(Z_CTC0)	;C:=ADDRESS TO CTC0
	LD	C,A
	OUT	(C),L
	LD	A,(Z_SYLV)	;GET SYS-CLOCK INTERRUPT-LEVEL
	LD	B,A
	OR	A
	JR	Z,INIT25
INIT20
	INC	C
	INC	HL
	INC	HL
	DJNZ	INIT20
INIT25
	LD	DE,SYSRPT
	LD	(HL),E		;LOAD SYSRPT VECTOR
	INC	HL
	LD	(HL),D
	LD	A,C_CTRL+C_IE+C_TIM+C_P256+C_TC
	OUT	(C),A
	LD	A,(Z_TICK)	;SYTEM TICK:=TC*256*Z_TICK
	OUT	(C),A		;  AT 2 MHZ
 
	JP	Z$SCED		;BEGIN EXECUTION
 
;-------
; SET UP PCB AND LINK IT TO PCB-LIST
;
; ACCEPTS: IX = ^IPCB
;
; RETURNS: BC = ^PCB
;
; AFFECTS: A,BC,DE,HL,IX,IY,SP
;-------
 
INIPCB
	LD	HL,18		;ALLOCATE PCB
	CALL	Z_GETM
	PUSH	HL			;IY:=^PCB
	POP	IY
	XOR	A			;A:=0
	LD	(IY+PCBLNK),A	;=== PCBLNK ===
	LD	(IY+PCBLNK+1),A
	LD	(IY+PCBDLK),A	;=== PCBDLK ===
	LD	(IY+PCBDLK+1),A
	LD	(IY+PCBSTT),A	;=== STATUS ===
	LD	A,(IX+IPCPTY)	;=== PRIORITY ===
	LD	(IY+PCBPTY),A
	LD	L,(IX+IPCSTZ)	;=== STACK ===
	LD	H,(IX+IPCSTZ+1);HL:=STACK SIZE
	LD	C,L			;BC:=STACK SIZE
	LD	B,H
	CALL	Z_GETM		;HL:=STACK TOP
	DEC	BC			;SAVE TWO BYTES FOR PLZ
	DEC	BC
	LD	E,L			;DE:=STACK TOP + 1
	LD	D,H
	INC	DE			;FILL STACK WITH C7H
	LD	(HL),0AAH
	LDIR
	PUSH	HL			;SAVE SP
	LD	(HL),0BBH		;MARK EXTRA TWO BYTES OF --
	LDI				;STACK (FOR PLZ)
	LD	E,(IX+IPCENT)	;=== PC ===
	LD	D,(IX+IPCENT+1);DE:=PC (START VALUE)
	POP	HL
	DEC	HL			;INITIAL PC TO STACK
	LD	(HL),D
	DEC	HL
	LD	(HL),E
	LD	DE,-12		;LOCATE SPACE FOR REGISTERS
	ADD	HL,DE
	LD	(IY+PCBSP),L
	LD	(IY+PCBSP+1),H
	PUSH	IX			;=== FIX ^IPCB ===
	POP	HL			;HL:=^IPCB
	LD	(IY+PCBNAM),L
	LD	(IY+PCBNAM+1),H
	LD	(IY+PCBPCB),0	;=== PCB-LINK ===
	LD	(IY+PCBPCB+1),0;LAST PCB IN PCB-LIST
	PUSH	IY			;BC:=^PCB
	POP	BC
	LD	IY,(Z$LPCB)	;LINK BC^ TO PCB-LIST
	LD	(IY+PCBPCB),C
	LD	(IY+PCBPCB+1),B
	LD	(Z$LPCB),BC	;^LAST PCB IN PCB-LIST
	RET
 
 
;-------
; INITIATE SEMAPHORE POINTERS
;
; ACCEPTS: HL = ^SEMAPHORE
;
; AFFECTS: A,B,IY
;-------
 
INISEM
	PUSH	HL
	XOR	A			;A:=0
	LD	B,6
CSEM1
	LD	(HL),A		;CLEAR ALL POINTERS
	INC	HL
	DJNZ	CSEM1
	POP	HL
	LD	IY,(Z$LSEM)	;IY:=^LAST SEM
	LD	(IY+SEMSEM),L	;INSERT NEW SEM
	LD	(IY+SEMSEM+1),H
	LD	(Z$LSEM),HL
	RET
	title	'RTZ.SBR'
;UPDATED: 820107 / PH
;REV:     2.00
 
;=======
; Z$SCED - MAIN SCHEDULER
;=======
 
	GLOBAL	Z$SCED
 
Z$SCED
	LD	IX,(Z$XQUE)	;IX:=^PROC TO BE RUNNING
	LD	HL,Z$ICNT
	DI
	LD	A,(HL)
	OR	A			;ANY PENDING RUPTS
	JR	NZ,SCED10		;YES
	LD	(Z$STAT),A	;NO, NOT IN RTZ!
	LD	L,(IX+PCBSP)	;HL:=SP (RUNNING PROC)
	LD	H,(IX+PCBSP+1)
	LD	SP,HL
	POP	IY
	POP	IX
	POP	DE
	POP	BC
	POP	AF
	POP	HL
	EI
	RETI				;AND DISPATCH TO USER PROC.
SCED10
	DEC	A			;ONE PENDING RUPT LESS
	LD	(HL),A
	LD	HL,Z$ISTK		;HL:=^STACK OF PENDING RUPTS
	ADD	A,L
	LD	L,A
	JR	NC,SCED12
	INC	H
SCED12
	LD	E,(HL)
	EI
	LD	D,0
	INC	E			;SYSTEM CLOCK ?
	JP	Z,SYSCLK		;YES
	LD	HL,ISEMTB-2	;-2 FOR COMPENSATION OF "INC E"
	ADD	HL,DE		;HL:=^POS. # "DE" IN ISEMTB
	ADD	HL,DE
	LD	E,(HL)		;DE:=Z_xxSM
	INC	HL
	LD	D,(HL)
	PUSH	DE			;IX:=Z_RXSM
	POP	IX
	JP	R00R05
 
;=======
; Z$SREG - SAVE ALL REGISTERS
;
; RETURNS: IX = ^PCB for running process
;
; AFFECTS: A,F
;=======
 
	GLOBAL	Z$SREG
 
Z$SREG
	EX	(SP),HL		;PUSH HL, HL:=RETURN ADDRESS
	PUSH	AF
	LD	A,0FFH		;IN RTZ !!
	LD	(Z$STAT),A
	EI
	PUSH	BC
SREGDE
	PUSH	DE
	PUSH	IX
	PUSH	IY
	PUSH	HL			;PUSH RETURN ADDRESS
	LD	HL,2			;HL:=SP (RUNNING)
	ADD	HL,SP
	LD	IX,(Z$XQUE)	;IX:=^RUNNING PROCESS
	LD	(IX+PCBSP),L	;SAVE SP IN PCB
	LD	(IX+PCBSP+1),H
	LD	HL,STCKHL+2	;RESTORE HL
	ADD	HL,SP
	LD	A,(HL)
	INC	HL
	LD	H,(HL)
	LD	L,A
	RET
 
;=======
; Z$IDLE - IDLE PROCESS
;=======
 
Z$IDLE
	HALT
	JR	Z$IDLE
 
;-------
; IDLIPC - IPCB OF IDLE PROCESS
;-------
 
IDLIPC
	text	'Z$IDLE'
	DEFW	Z$IDLE
	DEFW	SZISTK		;STACK SIZE
	DEFB	255			;PTY
 
;=======
; Z$TODQ - LINK TO TIME-QUEUE
;
; ACCEPTS: BC = ^PROC TO BE LINKED
;          DE = TIME-LIMIT
;
; WARNING! CY-flag assumed to be cleared after label
;          LINK.
;=======
 
Z$TODQ
	LD	IY,Z$DQUE-PCBDLK
	LD	HL,(Z$DQUE)	;CHECK IF ANY PROC IN Z$DQUE
	PUSH	BC
	LD	BC,0			;INIT ACCUMULATED DELAY
TODQ10
	PUSH	HL			;IX:=^Z$DQUE.CURRENT
	POP	IX
	LD	A,L
	OR	H
	JR	Z,TODQ50		;NO PROC LEFT
	LD	L,(IX+PCBDLY)	;HL:=TIME-LIMIT (CURRENT)
	LD	H,(IX+PCBDLY+1)
	ADD	HL,BC		;HL:=TIME-LIMIT (ACCUM.)
	SBC	HL,DE		;HL:=TIME-LIMIT DIFFERENCE
	JR	NC,TODQ20
	ADD	HL,DE		;RESTORE HL
	LD	C,L			;UPDATE ACCUMULATED DELAY
	LD	B,H
	LD	L,(IX+PCBDLK)	;HL:=^NEXT ITEM IN Z$DQUE
	LD	H,(IX+PCBDLK+1)
	PUSH	IX			;IY:=^PREDECESSOR
	POP	IY
	JR	TODQ10
TODQ20
	LD	(IX+PCBDLY),L	;ADJUST DELAYTIME FOR ITEM
	LD	(IX+PCBDLY+1),H;  AFTER INSERTED
TODQ50
	EX	DE,HL		;HL:=TIME-LIMIT FOR INSERTED
	SBC	HL,BC		;CARRY IS ZERO !!
	POP	BC			;GET ^PROC TO BE LINKED
	LD	(IY+PCBDLK),C	;INSERT NEW ITEM (BC^)
	LD	(IY+PCBDLK+1),B
	PUSH	BC			;SWAP BC,IX
	PUSH	IX
	POP	BC
	POP	IX
	LD	(IX+PCBDLK),C	;^SUCCESSOR TO INSERTED
	LD	(IX+PCBDLK+1),B
	LD	(IX+PCBDLY),L	;SET DELAY FOR INSERTED
	LD	(IX+PCBDLY+1),H
	SET	STTDLY,(IX+PCBSTT)
	RET
 
;=======
; Z$FRDQ - REMOVE (BC^) FROM DELAY-QUEUE
;
; ACCEPTS: BC = ^PROC TO BE REMOVED
;=======
 
Z$FRDQ
	LD	IX,Z$DQUE-PCBDLK
FRDQ10
	LD	L,(IX+PCBDLK)	;HL:=^PROC IN Z$DQUE
	LD	H,(IX+PCBDLK+1)
	AND	A			;CLEAR CY
	SBC	HL,BC		;HL = ^PROC TO BE REM. ?
	JR	Z,FRDQ20		;YES, REMOVE
	ADD	HL,BC		;RESTORE HL
	PUSH	HL			;IX := HL
	POP	IX
	JR	FRDQ10
FRDQ20
	PUSH	BC			;IY:=^PROC TO BE REMOVED
	POP	IY
	RES	STTDLY,(IY+PCBSTT)
	LD	L,(IY+PCBDLK)	;HL:=^PROC AFTER (BC^)
	LD	H,(IY+PCBDLK+1)
	LD	(IX+PCBDLK),L	;PROC.PCBDLK (BEFORE BC^)
	LD	(IX+PCBDLK+1),H;  := ^PROC AFTER (BC^)
	LD	A,L			;(BC^) LAST PROC IN Z$DQUE ?
	OR	H
	RET	Z			;YES
	INC	HL			;NO, HL:=^PROC (BC^).TIME
	INC	HL
	LD	A,(HL)		;FIX PCBDLY FOR (HL^)
	ADD	A,(IY+PCBDLY)
	LD	(HL),A
	INC	HL
	LD	A,(HL)
	ADC	A,(IY+PCBDLY+1)
	LD	(HL),A
	RET
 
;=======
; Z$ENQ - APPEND TO QUEUE
;
; ACCEPTS: HL = ^QUEUE-HEAD
;          DE = ^ITEM TO BE APPENDED
;
; AFFECTS: AF,HL
;=======
 
	GLOBAL	Z$ENQ
 
Z$ENQ
	LD	A,(HL)
	INC	HL
	OR	(HL)			;LAST ITEM ?
	JR	Z,ENQ50		;YES, HL -> BC = 0
	LD	A,(HL)		;NO, HL:=^NEXT ITEM
	DEC	HL
	LD	L,(HL)
	LD	H,A
	JR	Z$ENQ
ENQ50
	LD	(HL),D		;FIX HL -> DE
	DEC	HL
	LD	(HL),E
	LD	(DE),A		;FIX DE -> 0
	INC	DE
	LD	(DE),A
	DEC	DE
	RET
 
;=======
; Z$TOXQ - LINK PROCESS TO EXECUTION QUEUE
;
; ACCEPTS: BC = ^PCB TO BE LINKED
;
; AFFECTS: A,BC,DE,HL,IX
;=======
 
	GLOBAL	Z$TOXQ
 
Z$TOXQ
	LD	HL,PCBPTY		;HL:=^PROC.PTY (TO BE LINKED)
	ADD	HL,BC
	LD	A,(HL)		;A:=PROC.PTY (TO BE LINKED)
	LD	HL,Z$XQUE
	LD	IX,(Z$XQUE)	;IX:=^XQUE.1ST
TOXQ50
	CP	(IX+PCBPTY)	;PROC.PTY ">" XQUE.CUR.PTY ?
	JR	C,TOXQ70		;YES
	LD	E,(IX+PCBLNK)	;DE:=^XQUE.NXT
	LD	D,(IX+PCBLNK+1)
	PUSH	IX			;HL:=^XQUE.CUR
	POP	HL
	PUSH	DE			;IX:=^XQUE.NXT
	POP	IX
	JR	TOXQ50
TOXQ70
	LD	(HL),C		;INSERT BC^ BEFORE IX^
	INC	HL
	LD	(HL),B
	LD	L,C			;HL:=^PROC (INSERTED)
	LD	H,B
	PUSH	IX			;BC:=^PROC AFTER INSERTED
	POP	BC
	LD	(HL),C
	INC	HL
	LD	(HL),B
	RET
 
;=======
; Z$FRXQ - REMOVE RUNNING PROCESS FROM EXECUTION QUEUE
;
; AFFECTS: A,HL
;=======
 
	GLOBAL	Z$FRXQ
 
Z$FRXQ
	LD	HL,(Z$XQUE)	;HL:=^XQUE.1ST
	LD	A,(HL)		;A:=^XQUE.2ND(LOWBYTE)
	LD	(Z$XQUE),A	;OLD 2ND:=NEW 1ST (LOWBYTE)
	INC	HL
	LD	A,(HL)		;A:=^XQUE.2ND(HIGHBYTE)
	LD	(Z$XQUE+1),A	;OLD 2ND:=NEW 1ST (HIGHBYTE)
	RET
 
;=======
; Z_GETM - ALLOCATE MEMORY
;
; ACCEPTS: HL = # OF REQUESTED BYTES
;
; RETURNS: HL     =  ^ALLOCATED MEMORY (LOWEST ADDRESS)
;          Z_EOMF =  0 IF OK
;                 <> 0 IF END OF MEMORY
;=======
 
	GLOBAL	Z_GETM
	EXTERNAL	Z_HEAP
 
Z_GETM
	PUSH	DE			;SAVE DE
	DI
	LD	DE,(Z_HEAP)	;DE:=^FIRST FREE SPACE
	ADD	HL,DE		;ALLOCATE !
	LD	(Z_HEAP),HL
	EI
	PUSH	AF			;CHECK IF END OF MEMORY
	LD	(HL),0AAH
	LD	A,(HL)
	SUB	0AAH
	LD	(Z_EOMF),A	;SET IF EOM
	POP	AF
	EX	DE,HL		;HL:=^ALLOCATED MEMEORY
	POP	DE			;RESTORE DE
	RET
 
;=======
; WAIBRK - CHECK IF DEBUGGER IS PRESENT
; SIGBRK
;
; AFFECTS: BC,AF
;=======
 
WAIBRK
	LD	BC,(Z$WABP)
	JR	SIGB10
SIGBRK
	LD	BC,(Z$SIBP)
SIGB10
	LD	A,C
	OR	B
	RET	Z
	PUSH	BC
	RET
	title	'RTZ.CALLS'
;UPDATED: 811129 / PH
;REV:     2.00
 
;=======
; Z_POLL
;
; ACCEPTS: DE = ^SEMAPHORE
; RETURNS: HL = ^MESSAGE IF ANY
;          HL = 0 IF NO MESSAGE AT SEMAPHORE
;=======
 
Z_POLL
	LD	C,E			;BC:=^SEMAPHORE
	LD	B,D
	EX	DE,HL		;DE:=^TIME-OUT
 
; CHECK IF THERE IS ANY MESSAGE AT SEM (BC^) BEFORE
 
	LD	A,(BC)		;HL:=^MSGQUE.1ST
	LD	L,A
	INC	BC
	LD	A,(BC)
	LD	H,A
	OR	L			;ANY MESSAGE AT SEM ?
	JR	Z,RTZ250		;YES, RETURN WITH HL=0
	JR	RTZ200
 
;=======
; Z_EWAI - RTZ-CALL "EWAI"
;
; ACCEPTS: DE = ^SEMAPHORE
; RETURNS: HL = ^MESSAGE
;=======
 
	GLOBAL	Z_EWAI
 
Z_EWAI
	CALL	Z$SREG		;SAVE REGISTERS
	CALL	WAIBRK
	LD	(IX+PCBSEM),E	;UPDATE DEFAULT SEMAPHORE
	LD	(IX+PCBSEM+1),D
	LD	HL,0			;WAIT FOR EVER IF NO MESSAGE
	JR	WAIT05
 
;=======
; Z_WAIT - RTZ-CALL "WAIT"
;
; ACCEPTS: DE = ^SEMAPHORE
;          HL = TIME-LIMIT
; RETURNS: HL = ^MESSAGE
;=======
 
	GLOBAL	Z_WAIT
 
Z_WAIT
	CALL	Z$SREG		;SAVE REGISTERS
	CALL	WAIBRK
	LD	(IX+PCBSEM),E	;UPDATE DEFAULT SEMAPHORE
	LD	(IX+PCBSEM+1),D
	LD	A,L			;CHECK IF A POLL REQUEST
	OR	H
	JR	Z,Z_POLL
WAIT05
	LD	C,E			;BC:=^SEMAPHORE
	LD	B,D
	EX	DE,HL		;DE:=^TIME-OUT
 
; CHECK IF THERE IS ANY MESSAGE AT SEM (BC^) BEFORE
 
	LD	A,(BC)		;HL:=^MGQ.1ST
	LD	L,A
	INC	BC
	LD	A,(BC)
	LD	H,A
	OR	L			;ANY MESSAGE AT SEM ?
	JR	NZ,RTZ200		;YES
 
; LINK PROCESS TO SEMAPHORE
 
	INC	BC			;BC:=^SEM.PRQ
	PUSH	BC
	LD	BC,(Z$XQUE)	;BC:=^PROC(RUNNING)
	CALL	Z$FRXQ		;REMOVE PROC(RUNNING)
	POP	HL			;HL:=^SEM.PRQ
	PUSH	DE			;SAVE TIME-LIMIT
	LD	E,C			;DE:=PROC(RUNNING)
	LD	D,B
	CALL	Z$ENQ
	POP	DE
	LD	A,E			;CHECK IF TIMED WAIT
	OR	D
	CALL	NZ,Z$TODQ		;YES. LINK TO DELAY-QUEUE
	JP	Z$SCED
 
; REMOVE MESSAGE FROM SEMAPHORE AND GIVE IT TO
; RUNNING PROCESS
; BC = ^SEM.MGQ+1
; HL = ^MGQ.1ST
 
RTZ200
	INC	HL			;HL:=^MGQ.1ST+1
	LD	A,(HL)		;SEM.MGQ:=^MGQ.2ND
	LD	(BC),A
	DEC	HL
	DEC	BC
	LD	A,(HL)
	LD	(BC),A
RTZ250
	LD	C,(IX+PCBSP)	;BC:=SP(RUNNING)
	LD	B,(IX+PCBSP+1)
	PUSH	BC			;IX:=SP(RUNNING)
	POP	IX
	LD	(IX+STCKHL),L	;HL IN STACK:=^MESSAGE
	LD	(IX+STCKHL+1),H
	INC	HL			;HL:=^OPCODE-FIELD
	INC	HL
	INC	HL
	LD	A,(HL)		;SET A AND FLAGS
	OR	A
	PUSH	AF
	POP	HL
	LD	(IX+STCKAF),L
	LD	(IX+STCKAF+1),H
	JP	Z$SCED
 
;=======
; Z_SGNL - RTZ-CALL "SIGNAL"
;
; ACCEPTS: DE = ^SEM
;          HL = ^MSG
;=======
 
	GLOBAL	Z_SGNL
 
Z_SGNL
	CALL	Z$SREG		;SAVE REGISTERS
	CALL	SIGBRK
 
; CHECK IF THERE IS ANY PROCESS AT SEM (DE^) BEFORE
 
	PUSH	DE			;PUSH ^SEM.MGQ
	INC	DE			;HL:=^SEM.PRQ
	INC	DE
	EX	DE,HL		;DE:=^MESSAGE
	LD	A,(HL)		;HL:=^PRQ.1ST
	INC	HL
	LD	H,(HL)
	LD	L,A
	OR	H			;ANY PROCESS AT SEM ?
	JR	NZ,RTZ400		;YES
 
; LINK MESSAGE TO SEMAPHORE
 
	POP	HL			;HL:=^SEM.MGQ
	CALL	Z$ENQ
	JP	Z$SCED		;  AND DISPATCH
 
; REMOVE PROCESS FROM SEMAPHORE WITH MESSAGE
; DE = ^MESSAGE
; HL = ^PRQ.1ST (^PROCESS TO BE REMOVED)
; TS = ^SEMAPHORE
 
RTZ400
	POP	BC			;BC:=^SEM.PRQ
	INC	BC
	INC	BC
	LD	A,(HL)		;SEM.PRQ:=^WQUE.2ND
	LD	(BC),A
	INC	HL
	INC	BC
	LD	A,(HL)
	LD	(BC),A
	DEC	HL
	LD	C,L			;BC:=^PROC(REMOVED)
	LD	B,H
	PUSH	HL			;IX:=^PROC(REMOVED)
	POP	IX
	LD	L,(IX+PCBSP)	;HL:=SP(REMOVED)
	LD	H,(IX+PCBSP+1)
	PUSH	HL			;IY:=SP(REMOVED)
	POP	IY
	LD	(IY+STCKHL),E	;HL IN STACK:=^MESSAGE
	LD	(IY+STCKHL+1),D
	INC	DE			;DE:=^OPCODE-FIELD
	INC	DE
	INC	DE
	LD	A,(DE)		;SET A AND FLAGS
	OR	A
	PUSH	AF
	POP	HL
	LD	(IY+STCKAF),L
	LD	(IY+STCKAF+1),H
	BIT	STTDLY,(IX+PCBSTT);IS REM. PROC IN DELAYQUE ?
	CALL	NZ,Z$FRDQ		;YES, REMOVE IT
	CALL	Z$TOXQ		;LINK IT TO Z$XQUE
	JP	Z$SCED		;  AND DISPATCH
 
;=======
; Z_LDCW - RTZ-CALL "LOAD CONTROL WORD FOR Z-80 --
;                    DEVICE"
;
; ACCEPTS: DE = ^CONTROL VECTOR
;=======
 
	GLOBAL	Z_LDCW
 
Z_LDCW
	PUSH	AF
	PUSH	BC
	PUSH	HL
	EX	DE,HL
	LD	C,(HL)		;C:=PORT ADDRESS
	INC	HL
	LD	B,(HL)		;B:=# OF CONTROL BYTES
	INC	HL
	DI
	OTIR
	EI
	EX	DE,HL
	POP	HL
	POP	BC
	POP	AF
	RET
 
;=======
; Z_SEVC - SET VECTOR FOR DIRECT INTERRUPT
;
; ACCEPTS: DE = INTERRUPT LEVEL
;          HL = INTERRUPT VECTOR
;=======
 
	GLOBAL	Z_SEVC
 
Z_SEVC
	PUSH	BC
	PUSH	HL
	LD	C,L			;BC:=^SERVICE ROUTINE
	LD	B,H
	LD	HL,Z_00VC
	ADD	HL,DE		;HL:=^LOCATION OF I-VECTOR
	ADD	HL,DE
	DI
	LD	(HL),C		;LOAD I-VECTOR
	INC	HL
	LD	(HL),B
	EI
	POP	HL
	POP	BC
	RET
 
;=======
; Z_SPTY - SET PROCESS PRIORITY
;
; ACCEPTS: DE = ^PCB
;           L = NEW PRIORITY
;=======
 
	GLOBAL	Z_SPTY
 
Z_SPTY
	CALL	Z$SREG
	LD	C,E			;BC:=^PCB
	LD	B,D
	PUSH	HL
	CALL	Z$FRXQ		;REMOVE FROM XQUE
	POP	HL
	LD	(IX+PCBPTY),L	;LOAD NEW PRIORITY
	CALL	Z$TOXQ		;INSERT IN XQUE AGAIN
	JP	Z$SCED
 
;=======
; Z_CPRO - CREATE PROCESS
;
; ACCEPTS: DE = ^INITIAL PROCESS CONTROL BLOCK
;=======
 
	GLOBAL	Z_CPRO
 
Z_CPRO
	CALL	Z$SREG
	PUSH	DE			;IX:=^IPCB
	POP	IX
	CALL	INIPCB
	CALL	Z$TOXQ		;LINK BC^ TO XQUE
	JP	Z$SCED
 
;=======
; Z_CSEM - CREATE SEMAPHORE
;
; ACCEPTS: DE = ^SEMAPHORE
;=======
 
	GLOBAL	Z_CSEM
 
Z_CSEM
	CALL	Z$SREG
	EX	DE,HL		;HL:=^SEMAPHORE
	CALL	INISEM
	JP	Z$SCED
	title	'RTZ.RUPT'
;UPDATED: 820120 / PH
;REV:     2.00
 
;=======
; SYSRPT - SYSTEM CLOCK INTERRUPT
;=======
 
SYSRPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,-1			;A:=INTERRUPT #SYS
	JP	NZ,R00R55		;YES
	POP	AF			;NO, JUST GO ON
	CALL	Z$SREG
 
; HANDLE TIME-SLICING IF ANY
 
SYSCLK
	LD	A,(Z_SLCF)
	OR	A			;SLICE ENABLED ?
	JR	Z,NOSLIC		;NO
	LD	A,(Z_SLCP)	;A:=SLICE PRIORITY
	CP	(IX+PCBPTY)	;RUNNING PROC.ON SLICE PTY ?
	JR	NZ,NOSLIC		;NO
	LD	HL,Z$SLCC		;YES, HL:=^SLICE-COUNT
	INC	(HL)
	LD	A,(Z_SLCT)	;A:=SLICE TIME
	SUB	(HL)			;TIME FOR SLICE ?
	JR	NZ,NOSLIC		;NO
	LD	(HL),A		;YES, RESET SLICE-COUNTER
	CALL	Z$FRXQ		;YES, REMOVE AND
	PUSH	IX			;  INSERT
	POP	BC			;BC:=IX
	CALL	Z$TOXQ
NOSLIC
 
; HANDLE SYSTEM TICK
 
	LD	HL,(Z$DQUE)	;HL:=^DQUE.1ST
	LD	A,L			;DELAY-QUEUE EMPTY ?
	OR	H
	JP	Z,Z$SCED		;YES, DISPATCH
	INC	HL			;NO
	INC	HL			;HL:=^TIME-LIMIT OF 1ST
	LD	E,(HL)		;DE:=TIME-LIMIT
	INC	HL
	LD	D,(HL)
	DEC	DE			;ONE TICK !!
	LD	A,E			;TIME OUT ?
	OR	D
	LD	(HL),D		;STORE DECREASED TIME-LIMIT
	DEC	HL
	LD	(HL),E
	JP	NZ,Z$SCED		;NO TIME-OUT, DISPATCH
	INC	HL
 
; REMOVE PROCESS FROM DELAY QUEUE (TIME-OUT)
 
SYSR10
	INC	HL			;HL:=^DLINK OF 1ST
	LD	E,(HL)		;DE:=^DQUE.2ND
	INC	HL
	LD	D,(HL)
	LD	BC,(Z$DQUE)	;BC:=^PROC(REM.FROM Z_DQUE)
	LD	(Z$DQUE),DE
	PUSH	BC			;IX:=^PROC
	POP	IX
	RES	STTDLY,(IX+PCBSTT)
 
; REMOVE PROCESS (BC^ = IX^) FROM SEMAPHORE TOO
 
	LD	L,(IX+PCBSEM)	;HL:=^SEM
	LD	H,(IX+PCBSEM+1)
	INC	HL			;HL:=^SEMPRQ.1ST
	INC	HL
SYSR20  ;FIND PROCESS (TO BE REMOVED) IN SEMPRQ
	LD	E,(HL)		;DE:=^PROC.NEXT
	INC	HL
	LD	D,(HL)
	EX	DE,HL		;HL:=^PROC.NEXT,DE:=^SEMPRQ.1ST+1
	AND	A			;CLEAR CY
	SBC	HL,BC		;WANTED PROCESS ?
	JR	Z,SYSR50		;YES
	ADD	HL,BC		;NO, RESTORE HL
	JR	SYSR20
SYSR50				;(DE) -> (BC) -> (HL)
	INC	BC			;BC:=^PROC+1
	LD	A,(BC)		;(DE) -> (HL)  HIGHBYTE
	LD	(DE),A
	DEC	BC
	DEC	DE
	LD	A,(BC)		;(DE) -> (HL) LOWBYTE
	LD	(DE),A
 
; LINK TIME-OUT MESSAGE TO PROCESS (BC^ = IX^)
 
	LD	L,(IX+PCBSP)	;HL:=SP (PROC TO BE REMOVED)
	LD	H,(IX+PCBSP+1)
	PUSH	HL			;IX:=SP (PROC TO BE REMOVED)
	POP	IX
	LD	HL,TOUTMG		;HL:=^TIME-OUT MESSAGE
	LD	(IX+STCKHL),L	;HL IN STACK:=^MESSAGE
	LD	(IX+STCKHL+1),H
	LD	A,ZRCTO		;A:=TIME-OUT RETURN CODE
	OR	A
	PUSH	AF
	POP	HL
	LD	(IX+STCKAF),L
	LD	(IX+STCKAF+1),H
 
	CALL	Z$TOXQ		;LINK TO EXECUTION QUEUE
	LD	HL,(Z$DQUE)	;HL:=^DQUE.1ST
	LD	A,L			;DELAY-QUEUE EMPTY ?
	OR	H
	JP	Z,Z$SCED		;YES, DISPATCH
 
	INC	HL			;NO
	INC	HL			;HL:=^TIME-LIMIT OF 1ST
	LD	A,(HL)		;CHECK IF ANOTHER TIME-OUT
	INC	HL
	OR	(HL)
	JP	NZ,Z$SCED		;NO, DISPATCH
	JR	SYSR10		;YES
 
;-------
; TOUTMG - TIME-OUT MESSAGE
;-------
 
TOUTMG
	DEFW	0			;NO LINK
	DEFB	0			;NO REQUEST CODE
	DEFB	ZRCTO		;TIME-OUT
	DEFW	6			;6 BYTES
 
;=======
; R00RPT - SYSTEM CTC0 INTERRUPT
;=======
 
	EXTERNAL	Z_00SM
 
R00RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	JR	NZ,R00R50		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_00SM		;IX:=^CURRENT I-SEMAPHORE
R00R05
	LD	(IX+SEMIMG+MSGRC),ZRCINT;ASSUME NO MISSED RUPT
	LD	L,(IX+SEMPRQ)	;HL:=^SEMPRQ.1ST
	LD	H,(IX+SEMPRQ+1)
	LD	A,L
	OR	H			;ANY PROCESS AT SEM ?
	JR	NZ,R00R20		;YES
 
; LINK INTERRUPT-MESSAGE TO SEMAPHORE
 
	LD	A,(IX+SEMMGQ)	;ANY MSG HERE BEFORE ?
	OR	(IX+SEMMGQ+1)
	JR	Z,R00R10		;NO
	LD	(IX+SEMIMG+MSGRC),ZRCMIS;YES, MISSED RUPT
	JP	Z$SCED
R00R10
	PUSH	IX			;HL:=^INTERRUPT SEMAPHORE
	POP	HL
	LD	A,SEMIMG		;MESSAGE TO SEMAPHORE
	ADD	A,L
	LD	(HL),A
	LD	A,0
	ADC	A,H
	INC	HL
	LD	(HL),A
	JP	Z$SCED
 
; REMOVE PROCESS FROM SEMAPHORE "IX^"
 
R00R20
	PUSH	IX			;TS:=^SEM
	LD	BC,SEMIMG		;PREP FOR RTZ400
	ADD	IX,BC
	PUSH	IX
	POP	DE			;DE:=IX+SEMIMG
	JP	RTZ400
 
; ALREADY IN RTZ !!
 
R00R50
	XOR	A			;INTERRUPT LEVEL #0
R00R55
	PUSH	HL			;SAVE HL ALSO
	PUSH	AF			;SAVE LEVEL #
	LD	HL,Z$ICNT
	LD	A,(HL)
	CP	21			;MORE THAN 20 PENDING RUPTS ?
	JR	NC,R00R57		;YES, SKIP THIS ONE
	INC	(HL)			;NO, ONE MORE RUPT PENDING
	LD	HL,Z$ISTK		;HL:=^STACK OF PENDING RUPTS
	ADD	A,L
	LD	L,A
	JR	NC,R00R56
	INC	H
R00R56
	POP	AF
	LD	(HL),A
	JR	R00R58
R00R57
	POP	AF
R00R58
	POP	HL
	POP	AF
	EI
	RETI
 
;=======
; R01RPT - SYSTEM CTC1 INTERRUPT
;=======
 
	EXTERNAL	Z_01SM
 
R01RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,1			;  INTERRUPT LEVEL #1
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_01SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
RPTSIZ	EQU	$-R01RPT
 
;=======
; R02RPT - SYSTEM CTC2 INTERRUPT
;=======
 
	EXTERNAL	Z_02SM
 
R02RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,2			;  INTERRUPT LEVEL #2
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_02SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R03RPT - SYSTEM CTC3 INTERRUPT
;=======
 
	EXTERNAL	Z_03SM
 
R03RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,3			;  INTERRUPT LEVEL #3
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_03SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
	if	RTZSIZ>4
 
;=======
; R04RPT - SYSTEM LEVEL 4 INTERRUPT
;=======
 
	EXTERNAL	Z_04SM
 
R04RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,4			;  A:=INTERRUPT #4
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_04SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R05RPT - SYSTEM LEVEL 5 INTERRUPT
;=======
 
	EXTERNAL	Z_05SM
 
R05RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,5			;  INTERRUPT LEVEL #5
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_05SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R06RPT - SYSTEM LEVEL 6 INTERRUPT
;=======
 
	EXTERNAL	Z_06SM
 
R06RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,6			;  INTERRUPT LEVEL #6
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_06SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R07RPT - SYSTEM LEVEL 7 INTERRUPT
;=======
 
	EXTERNAL	Z_07SM
 
R07RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,7			;  INTERRUPT LEVEL #7
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_07SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
	if	RTZSIZ>8
 
;=======
; R08RPT - SYSTEM LEVEL 8 INTERRUPT
;=======
 
	EXTERNAL	Z_08SM
 
R08RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,8			;  INTERRUPT LEVEL #8
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_08SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R09RPT - SYSTEM LEVEL 9 INTERRUPT
;=======
 
	EXTERNAL	Z_09SM
 
R09RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,9			;  INTERRUPT LEVEL #9
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_09SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R10RPT - SYSTEM LEVEL 10 INTERRUPT
;=======
 
	EXTERNAL	Z_10SM
 
R10RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,10			;  INTERRUPT LEVEL #10
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_10SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R11RPT - SYSTEM LEVEL 11 INTERRUPT
;=======
 
	EXTERNAL	Z_11SM
 
R11RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,11			;  INTERRUPT LEVEL #11
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_11SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R12RPT - SYSTEM LEVEL 12 INTERRUPT
;=======
 
	EXTERNAL	Z_12SM
 
R12RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,12			;  INTERRUPT LEVEL #12
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_12SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R13RPT - SYSTEM LEVEL 13 INTERRUPT
;=======
 
	EXTERNAL	Z_13SM
 
R13RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,13			;  INTERRUPT LEVEL #13
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_13SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R14RPT - SYSTEM LEVEL 14 INTERRUPT
;=======
 
	EXTERNAL	Z_14SM
 
R14RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,14			;  INTERRUPT LEVEL #14
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_14SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R15RPT - SYSTEM LEVEL 15 INTERRUPT
;=======
 
	EXTERNAL	Z_15SM
 
R15RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,15			;  INTERRUPT LEVEL #15
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_15SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
	if	RTZSIZ>16
 
;=======
; R16RPT - SYSTEM LEVEL 16 INTERRUPT
;=======
 
	EXTERNAL	Z_16SM
 
R16RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,16			;  INTERRUPT LEVEL #16
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_16SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R17RPT - SYSTEM LEVEL 17 INTERRUPT
;=======
 
	EXTERNAL	Z_17SM
 
R17RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,17			;  INTERRUPT LEVEL #17
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_17SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R18RPT - SYSTEM LEVEL 18 INTERRUPT
;=======
 
	EXTERNAL	Z_18SM
 
R18RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,18			;  INTERRUPT LEVEL #18
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_18SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R19RPT - SYSTEM LEVEL 19 INTERRUPT
;=======
 
	EXTERNAL	Z_19SM
 
R19RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,19			;  INTERRUPT LEVEL #19
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_19SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
	if	RTZSIZ>20
 
;=======
; R20RPT - SYSTEM LEVEL 20 INTERRUPT
;=======
 
	EXTERNAL	Z_20SM
 
R20RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,20			;  INTERRUPT LEVEL #20
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_20SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R21RPT - SYSTEM LEVEL 21 INTERRUPT
;=======
 
	EXTERNAL	Z_21SM
 
R21RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,21			;  INTERRUPT LEVEL #21
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_21SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R22RPT - SYSTEM LEVEL 22 INTERRUPT
;=======
 
	EXTERNAL	Z_22SM
 
R22RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,22			;  INTERRUPT LEVEL #22
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_22SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R23RPT - SYSTEM LEVEL 23 INTERRUPT
;=======
 
	EXTERNAL	Z_23SM
 
R23RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,23			;  INTERRUPT LEVEL #23
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_23SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R24RPT - SYSTEM LEVEL 24 INTERRUPT
;=======
 
	EXTERNAL	Z_24SM
 
R24RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,24			;  INTERRUPT LEVEL #24
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_24SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R25RPT - SYSTEM LEVEL 25 INTERRUPT
;=======
 
	EXTERNAL	Z_25SM
 
R25RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,25			;  INTERRUPT LEVEL #25
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_25SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R26RPT - SYSTEM LEVEL 26 INTERRUPT
;=======
 
	EXTERNAL	Z_26SM
 
R26RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,26			;  INTERRUPT LEVEL #26
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_26SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R27RPT - SYSTEM LEVEL 27 INTERRUPT
;=======
 
	EXTERNAL	Z_27SM
 
R27RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,27			;  INTERRUPT LEVEL #27
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_27SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R28RPT - SYSTEM LEVEL 28 INTERRUPT
;=======
 
	EXTERNAL	Z_28SM
 
R28RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,28			;  INTERRUPT LEVEL #28
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_28SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R29RPT - SYSTEM LEVEL 29 INTERRUPT
;=======
 
	EXTERNAL	Z_29SM
 
R29RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,29			;  INTERRUPT LEVEL #29
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_29SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R30RPT - SYSTEM LEVEL 30 INTERRUPT
;=======
 
	EXTERNAL	Z_30SM
 
R30RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,30			;  INTERRUPT LEVEL #30
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_30SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
 
;=======
; R31RPT - SYSTEM LEVEL 31 INTERRUPT
;=======
 
	EXTERNAL	Z_31SM
 
R31RPT
	PUSH	AF			;SAVE AF
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	LD	A,31			;  INTERRUPT LEVEL #31
	JP	NZ,R00R55		;YES
	POP	AF			;NO, RESTORE AF
	CALL	Z$SREG		;  AND SAVE ALL REGISTERS
	LD	IX,Z_31SM		;IX:=^CURRENT I-SEMAPHORE
	JP	R00R05
	endif
	endif
	endif
	endif
;=======
; Z_ISGL - SIGNAL FROM DIRECT INTERRUPT
;
; ACCEPTS: WORD AFTER CALL = INTERRUPT SEMAPHORE
;=======
 
	GLOBAL	Z_ISGL
 
Z_ISGL
	EX	(SP),HL		;PUSH HL, HL:=^PARAMETERS
	DI
	PUSH	AF
	PUSH	BC			;SAVE USER BC
	LD	A,(Z$STAT)
	OR	A			;IN RTZ ?
	JR	NZ,ISGL10		;YES
 
; NOT IN RTZ
 
	INC	A			;NOW WE'RE IN RTZ
	LD	(Z$STAT),A
	EI
	LD	C,(HL)		;BC:=SEMAPHORE ADDRESS
	INC	HL
	LD	B,(HL)
	LD	HL,ISGL20		;FIX FOR Z$SREG
	JP	SREGDE		;COMPLETE Z$SREG
ISGL20
	PUSH	BC			;IX:=^INTERRUPT SEMAPHORE
	POP	IX
	JP	R00R05		;GO TO SYSTEM INTERRUPT
 
; ALREADY IN RTZ
 
ISGL10
	INC	HL			;HL:=^INTERRUPT #
	INC	HL
	LD	B,(HL)		;B:=INTERRUPT #
	LD	HL,Z$ICNT
	LD	A,(HL)
	CP	21			;MORE THAN 20 PENDING RUPTS ?
	JR	NC,ISGL15		;YES, SKIP THIS ONE
	INC	(HL)			;NO, ONE MORE RUPT PENDING
	LD	HL,Z$ISTK		;HL:=^STACK OF PENDING RUPTS
	ADD	A,L
	LD	L,A
	JR	NC,ISGL18
	INC	H
ISGL18
	LD	(HL),B
ISGL15
	POP	BC
	POP	AF
	POP	HL
	EI
	RETI
 
;-------
; ISEMTB - TABLE OF ALL INTERRUPT SEMAPHORES
;-------
 
ISEMTB
	DEFW	Z_00SM
	DEFW	Z_01SM
	DEFW	Z_02SM
	DEFW	Z_03SM
	if	RTZSIZ>4
	DEFW	Z_04SM
	DEFW	Z_05SM
	DEFW	Z_06SM
	DEFW	Z_07SM
	if	RTZSIZ>8
	DEFW	Z_08SM
	DEFW	Z_09SM
	DEFW	Z_10SM
	DEFW	Z_11SM
	DEFW	Z_12SM
	DEFW	Z_13SM
	DEFW	Z_14SM
	DEFW	Z_15SM
	if	RTZSIZ>16
	DEFW	Z_16SM
	DEFW	Z_17SM
	DEFW	Z_18SM
	DEFW	Z_19SM
	if	RTZSIZ>20
	DEFW	Z_20SM
	DEFW	Z_21SM
	DEFW	Z_22SM
	DEFW	Z_23SM
	DEFW	Z_24SM
	DEFW	Z_25SM
	DEFW	Z_26SM
	DEFW	Z_27SM
	DEFW	Z_28SM
	DEFW	Z_29SM
	DEFW	Z_30SM
	DEFW	Z_31SM
	endif
	endif
	endif
	endif
ISEMCT	EQU	RTZSIZ
	title	'RTZ.RAM'
	begin	1
;UPDATED: 820120 / PH
 
;=======
; RTZ SYSTEM RAM
;
; 2.00: 8111xx
;=======
 
	GLOBAL	Z_00VC
	GLOBAL	Z_RUNP,Z$XQUE,Z$DQUE
 
Z_00VC	DEFS	RTZSIZ*2	;INTERRUPT VECTORS
Z_RUNP
Z$XQUE	DEFS	2		;HEAD OF EXECUTION QUEUE
      	DEFS	2		;=0 DUMMY TIMELIMIT
Z$DQUE	DEFS	2		;HEAD OF DELAY QUEUE
Z$IRAM	DEFS	18+SZISTK	;PCB AND STACK FOR Z$IDLE
 
	GLOBAL	Z$STAT,Z_EOMF
	GLOBAL	Z$FPCB,Z$LPCB,Z$FSEM,Z$LSEM
 
Z$STAT	DEFS	1		;=FF IF IN RTZ
Z$ICNT	DEFS	1		;# OF PENDING RUPTS
Z$ISTK	DEFS	20		;"INTERRUPT STACK"
Z$FPCB	DEFS	2		;^FIRST ITEM IN PCB-LIST
Z$LPCB	DEFS	2		;^LAST ITEM IN PCB-LIST
Z$FSEM	DEFS	2		;^FIRST ITEM IN SEM-LIST
Z$LSEM	DEFS	2		;^LAST ITEM IN SEM-LIST
Z_EOMF	DEFS	1		;END OF MEMORY FLAG
 
;-------
; TIME-SLICE VARIABLES
;-------
 
	GLOBAL	Z_SLCF,Z_SLCT,Z_SLCP
 
Z_SLCF	DEFS	1		;0=NOSLICE (DEFAULT)
Z$SLCC	DEFS	1		;SLICE COUNTER
Z_SLCT	DEFS	1		;TIME (*TICK) (DEFAULT=8)
Z_SLCP	DEFS	1		;PRIORITY (DEFAULT=200)
 
;-------
; DEBUGGER DEPENDENT VARIABLES
;-------
 
	GLOBAL	Z$SIBP,Z$WABP
 
Z$SIBP	DEFS	2		;^ROUTINE IN RTZ.DBG IF <> 0
Z$WABP	DEFS	2		;^ROUTINE IN RTZ.DBG IF <> 0

	end
