	title	'RTZ.RTC'
;Updated: 831012 / OL
;Created: <810624 / Peter Holmgren, DIGITAILOR AB
 
;=======
; Z_RTCK - REAL-TIME CLOCK
;
; RTC 2.00: Released 820125
;     2.01: Released 821212
;     2.02: Released 830921
;           Adjustable for different system ticks
;           Lots of globals replaced by equates
;           Better handling of multiple time-outs
;     2.03: Each 'ZQCWAF' generated 1 sek more delay!
;           Shorter SUBTIM
;=======
 
;FILE:    RTZ.EQU.S
;UPDATED: 820120 / PH
 
;=======
; ALL INTERNAL EQUATES FOR RTZ/2
;=======
 
;-------
; Initial Process Control Block
;-------
 
IPCNAM	EQU	0		;PROCESS NAME
IPCENT	EQU	6		;PROCESS ENTRY
IPCSTZ	EQU	8		;STACK SIZE
IPCPTY	EQU	10		;PROCESS PRIORITY
 
;-------
; Initial Semaphore
;-------
 
ISMADR	EQU	0		;SEMAPHORE ADDRESS
 
;-------
; Configuration Table - Z$CRTB
;-------
 
CNFIPT	EQU	0		;^INITIAL PROCESS TABLE
CNFNOP	EQU	2		;# OF IPCB:S IN IPT
CNFIST	EQU	3		;^INITIAL  SEMAPHORE TABLE
CNFNOS	EQU	5		;# OF ISEM:S IN IST
 
;-------
; Process Control Block
;-------
 
PCBLNK	EQU	0		;LINK FIELD
PCBDLY	EQU	2		;DELAY TIME
PCBDLK	EQU	4		;DELAY LINK FIELD
PCBSEM	EQU	6		;SEMAPHORE
PCBSP 	EQU	8		;STACK POINTER
PCBPTY	EQU	10		;PROCESS PRIORITY
PCBSTT	EQU	11		;STATUS BITS
PCBNAM	EQU	12		;^NAME
PCBRND	EQU	14		;RANDOM FIELD
PCBPCB	EQU	16		;STATIC PCB-LINK
 
;-------
; Semaphore
;-------
 
SEMMGQ	EQU	0		;MESSAGE QUEUE
SEMPRQ	EQU	2		;PROCESS (WAITING) QUEUE
SEMSEM	EQU	4		;STATIC SEMAPHORE-LINK
 
SEMIMG	EQU	6		;INTERRUPT MESSAGE
 
;-------
; Minimum Message
;-------
 
MSGLNK	EQU	0		;QUEUE LINK
MSGRQC	EQU	2		;REQUEST CODE
MSGRC 	EQU	3		;RETURN CODE
MSGLEN	EQU	4		;LENGTH
MSGRSM	EQU	6		;RESPONSE SEMAPHORE
 
;-------
; Device Request Message
;-------
 
MSGBAD	EQU	8		;BUFFER ADDRESS
MSGBSZ	EQU	10		;BUFFER SIZE
MSGBCN	EQU	12		;BYTE COUNT
 
MSGSEC	EQU	8		;SECONDS
MSGMIN	EQU	9		;MINUTES
MSGHRS	EQU	10		;HOURS
 
MSGWL 	EQU	8		;WANTED LENGTH
MSGSPC	EQU	10		;MAX SPACE OR ^FREE SPACE
 
;-------
; Device Request Bits
;-------
 
ZQBRD 	EQU	1		;00.. ..01 READ
;ZQbRD 	EQU	0
ZQBWRT	EQU	2		;00.. ..10 WRITE
;ZQbWRT	EQU	1
ZQBASC	EQU	0		;00.. .0.. ASCII
ZQBBIN	EQU	4		;00.. .1.. BINARY
;ZQbBIN	EQU	2
ZQBSTT	EQU	8		;00.. 1... STATUS
;ZQbSTT	EQU	3
 
;-------
; Device Request Codes
;-------
 
      						;00xx xxxx COMMON CODES
ZQCRDA	EQU	ZQBRD+ZQBASC		;READ ASCII
ZQCRDB	EQU	ZQBRD+ZQBBIN		;READ BINARY
ZQCWRA	EQU	ZQBWRT+ZQBASC		;WRITE ASCII
ZQCWRB	EQU	ZQBWRT+ZQBBIN		;WRITE BINARY
ZQCRDS	EQU	ZQBRD+ZQBSTT		;READ STATUS
ZQCWRS	EQU	ZQBWRT+ZQBSTT		;WRITE STATUS
 
      				;01xx xxxx COMMAND CODES
ZQCRDC	EQU	61H		;READ CLOCK
ZQCWRC	EQU	62H		;WRITE CLOCK
ZQCWAT	EQU	63H		;WAKE AT
ZQCWAF	EQU	64H		;WAKE AFTER
 
ZQCMCF	EQU	68H		;CONDITIONAL, FAST
ZQCMCB	EQU	69H		;CONDITIONAL, BEST
ZQCMUF	EQU	6AH		;UNCONDITIONAL, FAST
ZQCMUB	EQU	6BH		;UNCONDITIONAL, BEST
 
;-------
; System Return Codes
;-------
 
ZRCINT	EQU	0		;INTERRUPT MESSAGE
ZRCMIS	EQU	44H		;MISSED INTERRUPT
ZRCTO 	EQU	80H		;1... .... TIME-OUT
 
;-------
; Device Return Codes
;-------
 
ZRCOK 	EQU	0		;NORMAL TERMINATION
ZRCUNK	EQU	41H		;UNKNOWN REQUEST CODE
ZRCERR	EQU	42H		;UNSPECIFIED ERROR
ZRCBSY	EQU	43H		;DEVICE BUSY
ZRCOVF	EQU	45H		;DATA OVERFLOW
ZRCEOF	EQU	46H		;END OF FILE
ZRCDWN	EQU	47H		;DEVICE OFF LINE
ZRCEOM	EQU	48H		;END OF MEDIA
 
;-------
; HELP EQUATES
;-------
 
STTDLY	EQU	3		;BIT 3 IN PCBSTT
STCKAF	EQU	8		;OFFSET TO AF IN STACK
STCKHL	EQU	10		;OFFSET TO HL IN STACK
STCKPC	EQU	12		;OFFSET TO PC IN STACK
SZISTK	EQU	50		;SIZE OF IDLE STACK
 
 
Z$PRSC	equ	-1			;prescaler (0.2 s)
Z$RTQU	equ	3			;head of "wake at" queue
Z$RTDQ	equ	Z$RTQU+2		;head of "wake after" queue
 
; Up to here is cleared before start
 
Z$TKMG	equ	Z$RTDQ+2		;tick (0.2 s) message
Z$TISM	equ	Z$TKMG+2		;pretick semaphore
Z$CBSM	equ	Z$TISM+6		;tick (0.2 s) semaphore
 
Z$ACTM	equ	Z$CBSM+6		;help for "tortdq"
Z$ACTT	equ	Z$ACTM+3		;      - " -
 
RTCSIZ	equ	Z$ACTT+3
 
	global	Z_RTCK
	external	Z_SGNL,Z_WAIT,Z_EWAI,Z_CSEM,Z_CPRO
	external	Z_CKSM,Z_RTCP
 
	text	'RTZ.RTC 2.03'
	title	'Z_RTCK
Z_RTCK
	LD	DE,Z_SECS+Z$TISM;CREATE NECESSARY SEMAPHORES
	CALL	Z_CSEM
	LD	DE,Z_SECS+Z$CBSM
	CALL	Z_CSEM
	LD	DE,IPCBCB		;CREATE BUFFER-PROCESS
	CALL	Z_CPRO
 
	XOR	A			;CLEAR CLOCK
	LD	HL,Z_CKUP		;  ^DISPLAY ROUTINES
	LD	B,Z$TKMG+3	;  QUEUE HEADS
CLOC05
	LD	(HL),A
	INC	HL
	DJNZ	CLOC05
 
;-------
; MAIN LOOP
;-------
 
CLOC10
	LD	DE,Z_SECS+Z$CBSM;WAIT FOR TICK
	CALL	Z_EWAI
	LD	HL,Z_SECS+Z$PRSC;COUNT 200 MS:S
	INC	(HL)
	LD	A,(HL)
	CP	5			;NEW SECOND ?
	JP	C,REQEST		;NO - RETURN
	CALL	CLOCUP
	JR	C,UPDATE
	CALL	CLOCUP
	JR	C,UPDATE
	CALL	CLOCUP
	CP	24H			;NEW DAY ?
	JR	C,UPDATE
	LD	(HL),0		;CLEAR HOURS
	LD	IX,(Z_SECS+Z$RTQU);IX:=^RTQU.1ST
CLOC20
	DEFB	0DDH			;RTQU EMPTY ?
	LD	A,L
	DEFB	0DDH
	OR	H
	JR	Z,UPDATE		;YES
	LD	A,(IX+MSGHRS)	;NO, SET TIME TO TODAY
	SUB	24H
	LD	(IX+MSGHRS),A
	LD	L,(IX+0)		;IX:=^RTQU.NEXT
	LD	H,(IX+1)
	PUSH	HL
	POP	IX
	JR	CLOC20
 
;-------
; UPDATE DISPLAY
;-------
 
UPDATE:
	LD	HL,(Z_CKUP)	;HL:=^DISPLAY ROUTINE IF ANY
	LD	A,L
	OR	H			;DISPLAY ?
	CALL	NZ,JPHL		;YES, CALL ROUTINE
 
;-------
; CHECK IF TIME-OUT
;-------
 
CLOC25
	LD	IX,(Z_SECS+Z$RTQU);IX:=^RTQU.1ST
	DEFB	0DDH			;RTQU EMPTY ?
	LD	A,L
	DEFB	0DDH
	OR	H
	JR	Z,CLOC27
	CALL	CPTIME		;TIME-OUT ?
	JR	NZ,CLOC27		;NO, GO ON
	LD	HL,Z_SECS+Z$RTQU
	CALL	FRQUEU		;YES, REMOVE ITEM FROM RTQU
	JR	CLOC25		;CHECK IF MORE TIME-OUTS
 
CLOC27
	LD	HL,(Z_SECS+Z$RTDQ);HL:=^RTDQ.1ST
	LD	A,L			;RTDQ EMPTY ?
	OR	H
	JR	Z,REQEST		;YES
	PUSH	HL			;NO, TICK ONE SECOND
	POP	IX			;IX:=^RTDQ.1ST
	LD	DE,MSGSEC		;DE:=^RTDQ.1ST.TIME
	ADD	HL,DE
	EX	DE,HL
	LD	HL,ONESEC
	CALL	SUBTIM
	JR	C,CLOC29		;Timeout
 
CLOC28:
	LD	A,(IX+MSGSEC)
	OR	(IX+MSGMIN)
	OR	(IX+MSGHRS)
	JR	NZ,REQEST	;No timeout
 
CLOC29:
	LD	HL,Z_SECS+Z$RTDQ
	CALL	FRQUEU		;REMOVE ITEM FROM RTDQ
	LD	HL,(Z_SECS+Z$RTDQ);check if more time-outs
	LD	A,L
	OR	H
	PUSH	HL
	POP	IX
	JR	NZ,CLOC28		;Queue is not empty
 
;-------
; CHECK IF ANY REQUEST
;-------
 
REQEST
	LD	DE,Z_CKSM		;ANY REQUEST ?
	LD	HL,0			;POLL
	CALL	Z_WAIT
	LD	A,L
	OR	H
	JP	Z,CLOC10		;NO
	PUSH	HL			;SAVE ^REQUEST-MESSAGE
	INC	HL			;HL:=^CLCKMG.MSGRQC
	INC	HL
	LD	A,(HL)		;A:=CLCKMG.MSGRQC
	INC	HL
	LD	(HL),ZRCOK	;ASSUME GOOD RQC
	INC	HL			;HL:=^CLKMSG.RSM
	INC	HL
	INC	HL
	LD	C,(HL)		;BC:=^RESPONSE SEMAPHORE
	INC	HL
	LD	B,(HL)
	INC	HL			;HL:=^CLCKMG.SS
	CP	ZQCRDC
	JR	NZ,CLOC60
 
;-------
; WHAT TIME IS IT ?
;-------
 
	LD	DE,Z_SECS		;DE:=^SECOND
	EX	DE,HL
CLOC55
	PUSH	BC
	LD	BC,3
	LDIR
	POP	DE
	POP	HL			;HL:=^CLCKMG
	CALL	Z_SGNL
	JP	CLOC10
 
CLOC60
	CP	ZQCWRC
	JR	NZ,CLOC70
 
;-------
; SET TIME
;-------
 
	LD	DE,Z_SECS+Z$PRSC;DE:=^PRESCALER
	XOR	A			;CLEAR PRESCALER
	LD	(DE),A
	INC	DE			;DE:=^SECOND
	JR	CLOC55
 
CLOC70
	CP	ZQCWAT
	JR	NZ,CLOC80
 
;-------
; WAKE ME AT "HH.MM.SS"
;-------
 
	POP	IX			;IX:=^REQUEST MESSAGE
	CALL	CPTIME		;TODAY ?
	JR	C,CLOC75		;YES
	LD	A,(IX+MSGHRS)	;NO, MAKE IT TOMORROW
	ADD	A,24H
	LD	(IX+MSGHRS),A
CLOC75
	PUSH	IX			;LINK IX^ TO RTQU
	POP	BC
	CALL	TORTQU
	JP	CLOC10
 
CLOC80
	CP	ZQCWAF
	JR	NZ,CLOC90
 
;-------
; WAKE ME AFTER "HH.MM.SS"
;-------
 
	POP	BC			;BC:=^REQUEST  MESSAGE
	CALL	TORTDQ
	JP	CLOC10
 
;-------
; UNKNOWN OPCODE
;-------
 
CLOC90
	POP	HL			;HL:=^REQUEST MESSAGE
	LD	E,L			;DE:=^REQUEST MESSAGE.MSGRC
	LD	D,H
	INC	DE
	INC	DE
	INC	DE
	LD	A,ZRCUNK
	LD	(DE),A
	LD	E,C
	LD	D,B
	CALL	Z_SGNL
	JP	CLOC10
	title	'FRQUEU
;=======
; FRQUEU - REMOVE MESSAGE FROM Z$RTQU OR Z$RTDQ
;
; ACCEPTS: HL = ^QUEUE-HEAD
;          IX = ^ITEM TO BE REMOVED (1ST IN QUEUE)
;=======
 
FRQUEU
	LD	E,(IX+0)		;DE:=^QUEUE.2ND
	LD	D,(IX+1)
	LD	(HL),E		;MAKE IT 1ST
	INC	HL
	LD	(HL),D
	LD	E,(IX+MSGRSM)	;DE:=^RESPONSE SEMAPHORE
	LD	D,(IX+MSGRSM+1)
	PUSH	IX			;HL:=^MESSAGE
	POP	HL
	JP	Z_SGNL
 
;=======
; TORTQU - LINK TO Z$RTQU
;
; ACCEPTS: BC = ^ITEM TO BE LINKED
;=======
 
TORTQU
	PUSH	BC			;IY:=^ITEM TO BE LINKED
	POP	IY
	LD	HL,Z_SECS+Z$RTQU;HL:=^RTQU.HEAD
	LD	DE,(Z_SECS+Z$RTQU);DE:=^RTQU.1ST
TORT10
	PUSH	DE			;IX:=^RTQU.NEXT
	POP	IX
	LD	A,E			;RTQU ENDED ?
	OR	D
	JR	Z,INSERT		;YES
	CALL	CPTIMS		;NEW.HOUR < CUR.HOUR ?
	JR	C,INSERT		;YES, INSERT
	LD	E,(IX+0)		;DE:=^RTQU.NEXT
	LD	D,(IX+1)
	PUSH	IX			;HL:=^RTQU.OLD
	POP	HL
	JR	TORT10
INSERT
	LD	(HL),C		;(HL) --> (BC)
	INC	HL
	LD	(HL),B
	LD	L,C			;(BC) --> (IX)
	LD	H,B
	PUSH	IX
	POP	BC
	LD	(HL),C
	INC	HL
	LD	(HL),B
	RET
	title	'TORTDQ
;=======
; TORTDQ - LINK TO Z$RTDQ
;
; ACCEPTS: BC = ^REQUEST-MESSAGE (TO BE LINKED)
;=======
 
TORTDQ
	LD	HL,0			;CLEAR Z$ACTT
	LD	(Z_SECS+Z$ACTT),HL
	LD	(Z_SECS+Z$ACTT+1),HL
	LD	IX,Z_SECS+Z$RTDQ;IX := ^HEAD OF DELAY-QUEUE
	JR	TODQ15
TODQ10
	PUSH	HL			;IX:=^DQUE.CUR
	POP	IX
	LD	A,L			;CHECK IF ANY ITEM LEFT
	OR	H
	JR	Z,TODQ50		;NO ITEM LEFT, LINK !
	LD	HL,Z_SECS+Z$ACTT;Z$ACTT := Z$ACTT + IX^.TIME
	LD	A,(IX+MSGSEC)
	AND	A
	CALL	ADDDIG
	LD	A,(IX+MSGMIN)
	CALL	ADDDIG
	LD	A,(IX+MSGHRS)
	CALL	ADDDIG
	PUSH	IY			;SAVE IX & IY
	PUSH	IX
	LD	IY,Z_SECS+Z$ACTT-MSGSEC
	PUSH	BC			; Compare Z$ACTT - ITEM.TIME
	POP	IX
	CALL	CPTIMS
	POP	IX
	POP	IY
	JR	NC,TODQ30		;INSERT IF Z$ACTM >= ITEM.TIME
TODQ15:
	LD	DE,Z_SECS+Z$ACTM;UPDATE ACCUMULATED DELAY
	LD	HL,Z_SECS+Z$ACTT
	PUSH	BC
	LD	BC,3
	LDIR
	POP	BC
	PUSH	IX			;IY:=^PREDECESSOR TO ITEM TO --
	POP	IY			;  BE LINKED
	LD	L,(IX+0)		;HL:=^NEXT ITEM IN Z$RTDQ
	LD	H,(IX+1)
	JR	TODQ10
TODQ30
	LD	DE,Z_SECS+Z$ACTT;SET UP FOR  SUBTIM
	LD	HL,MSGSEC
	ADD	HL,BC
	CALL	SUBTIM		;IX^.TIME:=Z$ACTM-ITEM.TIME
TODQ50
	LD	(IY+0),C		;FIX ^INSERTED ITEM
	LD	(IY+1),B
	LD	L,C			;FIX ^SUCCESSOR
	LD	H,B
	PUSH	IX
	POP	DE
	LD	(HL),E
	INC	HL
	LD	(HL),D
	LD	HL,MSGSEC		;SET UP FOR SUBTIM
	ADD	HL,BC
	EX	DE,HL
	LD	HL,Z_SECS+Z$ACTM
	PUSH	BC
	POP	IX
	title	'SUBTIM
;=======
; SUBTIM - SUBTRACT TIMES
;
; Accepts: DE = ^SUBTRACTOR
;          HL = ^SUBTRAHEND
;          IX = ^DIFFERENCE-SEC (SEC=8)
;
; Affects: A,F,DE,HL
;=======
 
SUBTIM
	LD	A,(DE)
	SUB	(HL)
	DAA
	JR	NC,SUBT10
	ADD	A,-40H
SUBT10
	LD	(IX+MSGSEC),A
	INC	DE
	INC	HL
	LD	A,(DE)
	SBC	A,(HL)
	DAA
	JR	NC,SUBT20
	ADD	A,-40H
SUBT20:
	LD	(IX+MSGMIN),A
	INC	DE
	INC	HL
	LD	A,(DE)
	SBC	A,(HL)
	DAA
	JR	NC,SUBT30
	ADD	A,-76H
SUBT30
	LD	(IX+MSGHRS),A
	RET
 
;-------
; ADDDIG - add part of time
;-------
 
ADDDIG:
	ADC	A,(HL)
	DAA
	JR	C,ADDD10
	CP	60H
	JR	NC,ADDD10
	AND	A
	JR	ADDD20
ADDD10:
	ADD	A,40H
	DAA
	SCF
ADDD20:
	LD	(HL),A
	INC	HL
	RET
 
;=======
; CPTIME - COMPARE WITH TIME
; CPTIMS - COMPARE TIMES
;
; ACCEPTS: IX = ^RTCK-MESSAGE
;          IY = ^RTCK-MESSAGE (CPTIMS ONLY)
;=======
 
CPTIME
	LD	IY,Z_SECS-MSGSEC
CPTIMS
	LD	A,(IY+MSGHRS)	;A:=HOUR
	CP	(IX+MSGHRS)	;DIFFERENCE IN HOURS ?
	RET	NZ			;YES
	LD	A,(IY+MSGMIN)	;A:=MINUTE
	CP	(IX+MSGMIN)	;DIFFERENCE IN MINUTES ?
	RET	NZ			;YES
	LD	A,(IY+MSGSEC)	;A:=SECOND
	CP	(IX+MSGSEC)
	RET
 
;=======
; CLOCUP - UPDATE HL^ OF CLOCK
;=======
 
CLOCUP
	LD	(HL),0		;CLEAR HL^
	INC	HL			;HL:=^NEXT TIME-UNIT
	LD	A,(HL)
	ADD	A,1			;ONE MORE TICK
	DAA
	LD	(HL),A
	CP	60H
	RET
 
;-------
; JPHL - CALL (HL)
;-------
 
JPHL
	JP	(HL)
 
;-------
; CONSTANT
;-------
 
ONESEC
	DEFB	1
	DEFW	0
 
 
;=======
; Z_RCBF - BUFFER-PROCESS FOR KEEPING PACE WITH TIME
;=======
 
Z_RCBF
	LD	DE,Z_SECS+Z$TISM;WAIT FOR TICK
	LD	HL,(Z_RTCP)	;get RTC prescaler
	CALL	Z_WAIT
	LD	DE,Z_SECS+Z$CBSM;TRIGG RT-CLOCK
	LD	HL,Z_SECS+Z$TKMG
	CALL	Z_SGNL
	JR	Z_RCBF
 
IPCBCB
	text	'Z_RCBF'
	DEFW	Z_RCBF
	DEFW	40
	DEFB	1
 
	title	'RTZ.RTC.RAM
;Updated: 830921 / PH
;Created: <810624 / Peter Holmgren, DIGITAILOR AB
 
;=======
; RAM-area for RTZ-utilty RTZ.RTC
;
; RTC 2.02: Released 830921
;           Lots of globals replaced by equates
;=======
 
	begin	1
	global	Z_CKUP,Z_SECS
 
Z_CKUP	DEFS	2		;ENTRY FOR CLOCK DISPLAY
PRSC	DEFS	1		;RT-CLOCK
Z_SECS	DEFS	3
RTQU	DEFS	2		;HEAD OF TIME-QUEUE
RTDQ	DEFS	2		;HEAD OF DELAY-QUEUE
 
; Up to here is cleared before start
 
TKMG	DEFS	2		;tick (0.2 s) message
TISM	DEFS	6		;pretick semaphore
CBSM	DEFS	6		;tick (0.2 s) semaphore
 
ACTM	DEFS	3		;help for "tortdq"
ACTT	DEFS	3
 
	end
